@@ -1,25 +1,31 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #include <aws/appmesh/model/UpdateVirtualNodeRequest.h>
 #include <aws/core/utils/json/JsonSerializer.h>
-#include <aws/core/http/URI.h>
-#include <aws/core/utils/memory/stl/AWSStringStream.h>
 
 #include <utility>
 
 using namespace Aws::AppMesh::Model;
 using namespace Aws::Utils::Json;
 using namespace Aws::Utils;
-using namespace Aws::Http;
 
 UpdateVirtualNodeRequest::UpdateVirtualNodeRequest() : 
     m_clientToken(Aws::Utils::UUID::RandomUUID()),
     m_clientTokenHasBeenSet(true),
     m_meshNameHasBeenSet(false),
-    m_meshOwnerHasBeenSet(false),
     m_specHasBeenSet(false),
     m_virtualNodeNameHasBeenSet(false)
 {
@@ -44,17 +50,6 @@ Aws::String UpdateVirtualNodeRequest::SerializePayload() const
   return payload.View().WriteReadable();
 }
 
-void UpdateVirtualNodeRequest::AddQueryStringParameters(URI& uri) const
-{
-    Aws::StringStream ss;
-    if(m_meshOwnerHasBeenSet)
-    {
-      ss << m_meshOwner;
-      uri.AddQueryStringParameter("meshOwner", ss.str());
-      ss.str("");
-    }
-
-}
 
 
 
