@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/cloudfront/CloudFront_EXPORTS.h>
@@ -9,8 +19,6 @@
 #include <aws/core/utils/DateTime.h>
 #include <aws/cloudfront/model/ActiveTrustedSigners.h>
 #include <aws/cloudfront/model/DistributionConfig.h>
-#include <aws/core/utils/memory/stl/AWSVector.h>
-#include <aws/cloudfront/model/AliasICPRecordal.h>
 #include <utility>
 
 namespace Aws
@@ -28,10 +36,8 @@ namespace Model
 {
 
   /**
-   * <p>A distribution tells CloudFront where you want content to be delivered from,
-   * and the details about how to track and manage content delivery.</p><p><h3>See
-   * Also:</h3>   <a
-   * href="http://docs.aws.amazon.com/goto/WebAPI/cloudfront-2020-05-31/Distribution">AWS
+   * <p>The distribution's information.</p><p><h3>See Also:</h3>   <a
+   * href="http://docs.aws.amazon.com/goto/WebAPI/cloudfront-2018-11-05/Distribution">AWS
    * API Reference</a></p>
    */
   class AWS_CLOUDFRONT_API Distribution
@@ -429,103 +435,6 @@ namespace Model
      */
     inline Distribution& WithDistributionConfig(DistributionConfig&& value) { SetDistributionConfig(std::move(value)); return *this;}
 
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline const Aws::Vector<AliasICPRecordal>& GetAliasICPRecordals() const{ return m_aliasICPRecordals; }
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline bool AliasICPRecordalsHasBeenSet() const { return m_aliasICPRecordalsHasBeenSet; }
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline void SetAliasICPRecordals(const Aws::Vector<AliasICPRecordal>& value) { m_aliasICPRecordalsHasBeenSet = true; m_aliasICPRecordals = value; }
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline void SetAliasICPRecordals(Aws::Vector<AliasICPRecordal>&& value) { m_aliasICPRecordalsHasBeenSet = true; m_aliasICPRecordals = std::move(value); }
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline Distribution& WithAliasICPRecordals(const Aws::Vector<AliasICPRecordal>& value) { SetAliasICPRecordals(value); return *this;}
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline Distribution& WithAliasICPRecordals(Aws::Vector<AliasICPRecordal>&& value) { SetAliasICPRecordals(std::move(value)); return *this;}
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline Distribution& AddAliasICPRecordals(const AliasICPRecordal& value) { m_aliasICPRecordalsHasBeenSet = true; m_aliasICPRecordals.push_back(value); return *this; }
-
-    /**
-     * <p>AWS services in China customers must file for an Internet Content Provider
-     * (ICP) recordal if they want to serve content publicly on an alternate domain
-     * name, also known as a CNAME, that they've added to CloudFront. AliasICPRecordal
-     * provides the ICP recordal status for CNAMEs associated with distributions.</p>
-     * <p>For more information about ICP recordals, see <a
-     * href="https://docs.amazonaws.cn/en_us/aws/latest/userguide/accounts-and-credentials.html">
-     * Signup, Accounts, and Credentials</a> in <i>Getting Started with AWS services in
-     * China</i>.</p>
-     */
-    inline Distribution& AddAliasICPRecordals(AliasICPRecordal&& value) { m_aliasICPRecordalsHasBeenSet = true; m_aliasICPRecordals.push_back(std::move(value)); return *this; }
-
   private:
 
     Aws::String m_id;
@@ -551,9 +460,6 @@ namespace Model
 
     DistributionConfig m_distributionConfig;
     bool m_distributionConfigHasBeenSet;
-
-    Aws::Vector<AliasICPRecordal> m_aliasICPRecordals;
-    bool m_aliasICPRecordalsHasBeenSet;
   };
 
 } // namespace Model
