@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/cognito-identity/CognitoIdentity_EXPORTS.h>
@@ -29,7 +39,7 @@ namespace Model
   {
   public:
     UpdateIdentityPoolRequest();
-
+    
     // Service request name is the Operation name which will send this request out,
     // each operation should has unique request name, so that we can get operation's name from this request.
     // Note: this is not true for response, multiple operations may have the same response name,
@@ -144,43 +154,6 @@ namespace Model
     inline UpdateIdentityPoolRequest& WithAllowUnauthenticatedIdentities(bool value) { SetAllowUnauthenticatedIdentities(value); return *this;}
 
 
-    /**
-     * <p>Enables or disables the Basic (Classic) authentication flow. For more
-     * information, see <a
-     * href="https://docs.aws.amazon.com/cognito/latest/developerguide/authentication-flow.html">Identity
-     * Pools (Federated Identities) Authentication Flow</a> in the <i>Amazon Cognito
-     * Developer Guide</i>.</p>
-     */
-    inline bool GetAllowClassicFlow() const{ return m_allowClassicFlow; }
-
-    /**
-     * <p>Enables or disables the Basic (Classic) authentication flow. For more
-     * information, see <a
-     * href="https://docs.aws.amazon.com/cognito/latest/developerguide/authentication-flow.html">Identity
-     * Pools (Federated Identities) Authentication Flow</a> in the <i>Amazon Cognito
-     * Developer Guide</i>.</p>
-     */
-    inline bool AllowClassicFlowHasBeenSet() const { return m_allowClassicFlowHasBeenSet; }
-
-    /**
-     * <p>Enables or disables the Basic (Classic) authentication flow. For more
-     * information, see <a
-     * href="https://docs.aws.amazon.com/cognito/latest/developerguide/authentication-flow.html">Identity
-     * Pools (Federated Identities) Authentication Flow</a> in the <i>Amazon Cognito
-     * Developer Guide</i>.</p>
-     */
-    inline void SetAllowClassicFlow(bool value) { m_allowClassicFlowHasBeenSet = true; m_allowClassicFlow = value; }
-
-    /**
-     * <p>Enables or disables the Basic (Classic) authentication flow. For more
-     * information, see <a
-     * href="https://docs.aws.amazon.com/cognito/latest/developerguide/authentication-flow.html">Identity
-     * Pools (Federated Identities) Authentication Flow</a> in the <i>Amazon Cognito
-     * Developer Guide</i>.</p>
-     */
-    inline UpdateIdentityPoolRequest& WithAllowClassicFlow(bool value) { SetAllowClassicFlow(value); return *this;}
-
-
     /**
      * <p>Optional key:value pairs mapping provider names to provider app IDs.</p>
      */
@@ -532,9 +505,6 @@ namespace Model
     bool m_allowUnauthenticatedIdentities;
     bool m_allowUnauthenticatedIdentitiesHasBeenSet;
 
-    bool m_allowClassicFlow;
-    bool m_allowClassicFlowHasBeenSet;
-
     Aws::Map<Aws::String, Aws::String> m_supportedLoginProviders;
     bool m_supportedLoginProvidersHasBeenSet;
 
