@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/ecs/ECS_EXPORTS.h>
@@ -164,145 +174,6 @@ namespace Model
     inline Container& WithName(const char* value) { SetName(value); return *this;}
 
 
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline const Aws::String& GetImage() const{ return m_image; }
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline bool ImageHasBeenSet() const { return m_imageHasBeenSet; }
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline void SetImage(const Aws::String& value) { m_imageHasBeenSet = true; m_image = value; }
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline void SetImage(Aws::String&& value) { m_imageHasBeenSet = true; m_image = std::move(value); }
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline void SetImage(const char* value) { m_imageHasBeenSet = true; m_image.assign(value); }
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline Container& WithImage(const Aws::String& value) { SetImage(value); return *this;}
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline Container& WithImage(Aws::String&& value) { SetImage(std::move(value)); return *this;}
-
-    /**
-     * <p>The image used for the container.</p>
-     */
-    inline Container& WithImage(const char* value) { SetImage(value); return *this;}
-
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline const Aws::String& GetImageDigest() const{ return m_imageDigest; }
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline bool ImageDigestHasBeenSet() const { return m_imageDigestHasBeenSet; }
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline void SetImageDigest(const Aws::String& value) { m_imageDigestHasBeenSet = true; m_imageDigest = value; }
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline void SetImageDigest(Aws::String&& value) { m_imageDigestHasBeenSet = true; m_imageDigest = std::move(value); }
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline void SetImageDigest(const char* value) { m_imageDigestHasBeenSet = true; m_imageDigest.assign(value); }
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline Container& WithImageDigest(const Aws::String& value) { SetImageDigest(value); return *this;}
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline Container& WithImageDigest(Aws::String&& value) { SetImageDigest(std::move(value)); return *this;}
-
-    /**
-     * <p>The container image manifest digest.</p>  <p>The
-     * <code>imageDigest</code> is only returned if the container is using an image
-     * hosted in Amazon ECR, otherwise it is omitted.</p> 
-     */
-    inline Container& WithImageDigest(const char* value) { SetImageDigest(value); return *this;}
-
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline const Aws::String& GetRuntimeId() const{ return m_runtimeId; }
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline bool RuntimeIdHasBeenSet() const { return m_runtimeIdHasBeenSet; }
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline void SetRuntimeId(const Aws::String& value) { m_runtimeIdHasBeenSet = true; m_runtimeId = value; }
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline void SetRuntimeId(Aws::String&& value) { m_runtimeIdHasBeenSet = true; m_runtimeId = std::move(value); }
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline void SetRuntimeId(const char* value) { m_runtimeIdHasBeenSet = true; m_runtimeId.assign(value); }
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline Container& WithRuntimeId(const Aws::String& value) { SetRuntimeId(value); return *this;}
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline Container& WithRuntimeId(Aws::String&& value) { SetRuntimeId(std::move(value)); return *this;}
-
-    /**
-     * <p>The ID of the Docker container.</p>
-     */
-    inline Container& WithRuntimeId(const char* value) { SetRuntimeId(value); return *this;}
-
-
     /**
      * <p>The last known status of the container.</p>
      */
@@ -734,15 +605,6 @@ namespace Model
     Aws::String m_name;
     bool m_nameHasBeenSet;
 
-    Aws::String m_image;
-    bool m_imageHasBeenSet;
-
-    Aws::String m_imageDigest;
-    bool m_imageDigestHasBeenSet;
-
-    Aws::String m_runtimeId;
-    bool m_runtimeIdHasBeenSet;
-
     Aws::String m_lastStatus;
     bool m_lastStatusHasBeenSet;
 
