@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #include <aws/events/model/PutPermissionRequest.h>
 #include <aws/core/utils/json/JsonSerializer.h>
@@ -13,7 +23,6 @@ using namespace Aws::Utils::Json;
 using namespace Aws::Utils;
 
 PutPermissionRequest::PutPermissionRequest() : 
-    m_eventBusNameHasBeenSet(false),
     m_actionHasBeenSet(false),
     m_principalHasBeenSet(false),
     m_statementIdHasBeenSet(false),
@@ -25,12 +34,6 @@ Aws::String PutPermissionRequest::SerializePayload() const
 {
   JsonValue payload;
 
-  if(m_eventBusNameHasBeenSet)
-  {
-   payload.WithString("EventBusName", m_eventBusName);
-
-  }
-
   if(m_actionHasBeenSet)
   {
    payload.WithString("Action", m_action);
