@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #include <aws/iot/model/ListSecurityProfilesRequest.h>
 #include <aws/core/utils/json/JsonSerializer.h>
@@ -18,8 +28,7 @@ using namespace Aws::Http;
 ListSecurityProfilesRequest::ListSecurityProfilesRequest() : 
     m_nextTokenHasBeenSet(false),
     m_maxResults(0),
-    m_maxResultsHasBeenSet(false),
-    m_dimensionNameHasBeenSet(false)
+    m_maxResultsHasBeenSet(false)
 {
 }
 
@@ -45,13 +54,6 @@ void ListSecurityProfilesRequest::AddQueryStringParameters(URI& uri) const
       ss.str("");
     }
 
-    if(m_dimensionNameHasBeenSet)
-    {
-      ss << m_dimensionName;
-      uri.AddQueryStringParameter("dimensionName", ss.str());
-      ss.str("");
-    }
-
 }
 
 
