@@ -1,14 +1,23 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/neptune/Neptune_EXPORTS.h>
 #include <aws/neptune/NeptuneRequest.h>
 #include <aws/core/utils/memory/stl/AWSString.h>
 #include <aws/core/utils/memory/stl/AWSVector.h>
-#include <aws/neptune/model/CloudwatchLogsExportConfiguration.h>
 #include <utility>
 
 namespace Aws
@@ -19,12 +28,15 @@ namespace Model
 {
 
   /**
+   * <p/><p><h3>See Also:</h3>   <a
+   * href="http://docs.aws.amazon.com/goto/WebAPI/neptune-2014-10-31/ModifyDBClusterMessage">AWS
+   * API Reference</a></p>
    */
   class AWS_NEPTUNE_API ModifyDBClusterRequest : public NeptuneRequest
   {
   public:
     ModifyDBClusterRequest();
-
+    
     // Service request name is the Operation name which will send this request out,
     // each operation should has unique request name, so that we can get operation's name from this request.
     // Note: this is not true for response, multiple operations may have the same response name,
@@ -448,139 +460,211 @@ namespace Model
 
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline const Aws::String& GetOptionGroupName() const{ return m_optionGroupName; }
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline bool OptionGroupNameHasBeenSet() const { return m_optionGroupNameHasBeenSet; }
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline void SetOptionGroupName(const Aws::String& value) { m_optionGroupNameHasBeenSet = true; m_optionGroupName = value; }
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline void SetOptionGroupName(Aws::String&& value) { m_optionGroupNameHasBeenSet = true; m_optionGroupName = std::move(value); }
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline void SetOptionGroupName(const char* value) { m_optionGroupNameHasBeenSet = true; m_optionGroupName.assign(value); }
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline ModifyDBClusterRequest& WithOptionGroupName(const Aws::String& value) { SetOptionGroupName(value); return *this;}
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline ModifyDBClusterRequest& WithOptionGroupName(Aws::String&& value) { SetOptionGroupName(std::move(value)); return *this;}
 
     /**
-     * <p> <i>(Not supported by Neptune)</i> </p>
+     * <p>A value that indicates that the DB cluster should be associated with the
+     * specified option group. Changing this parameter doesn't result in an outage
+     * except in the following case, and the change is applied during the next
+     * maintenance window unless the <code>ApplyImmediately</code> parameter is set to
+     * <code>true</code> for this request. If the parameter change results in an option
+     * group that enables OEM, this change can cause a brief (sub-second) period during
+     * which new connections are rejected but existing connections are not interrupted.
+     * </p> <p>Permanent options can't be removed from an option group. The option
+     * group can't be removed from a DB cluster once it is associated with a DB
+     * cluster.</p>
      */
     inline ModifyDBClusterRequest& WithOptionGroupName(const char* value) { SetOptionGroupName(value); return *this;}
 
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline const Aws::String& GetPreferredBackupWindow() const{ return m_preferredBackupWindow; }
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline bool PreferredBackupWindowHasBeenSet() const { return m_preferredBackupWindowHasBeenSet; }
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline void SetPreferredBackupWindow(const Aws::String& value) { m_preferredBackupWindowHasBeenSet = true; m_preferredBackupWindow = value; }
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline void SetPreferredBackupWindow(Aws::String&& value) { m_preferredBackupWindowHasBeenSet = true; m_preferredBackupWindow = std::move(value); }
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline void SetPreferredBackupWindow(const char* value) { m_preferredBackupWindowHasBeenSet = true; m_preferredBackupWindow.assign(value); }
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline ModifyDBClusterRequest& WithPreferredBackupWindow(const Aws::String& value) { SetPreferredBackupWindow(value); return *this;}
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline ModifyDBClusterRequest& WithPreferredBackupWindow(Aws::String&& value) { SetPreferredBackupWindow(std::move(value)); return *this;}
 
     /**
      * <p>The daily time range during which automated backups are created if automated
-     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.</p>
-     * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region.</p> <p>Constraints:</p> <ul> <li> <p>Must be in the
-     * format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in Universal
-     * Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the preferred
-     * maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p> </li>
-     * </ul>
+     * backups are enabled, using the <code>BackupRetentionPeriod</code> parameter.
+     * </p> <p>The default is a 30-minute window selected at random from an 8-hour
+     * block of time for each AWS Region. </p> <p>Constraints:</p> <ul> <li> <p>Must be
+     * in the format <code>hh24:mi-hh24:mi</code>.</p> </li> <li> <p>Must be in
+     * Universal Coordinated Time (UTC).</p> </li> <li> <p>Must not conflict with the
+     * preferred maintenance window.</p> </li> <li> <p>Must be at least 30 minutes.</p>
+     * </li> </ul>
      */
     inline ModifyDBClusterRequest& WithPreferredBackupWindow(const char* value) { SetPreferredBackupWindow(value); return *this;}
 
@@ -589,7 +673,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -599,7 +683,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -609,7 +693,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -619,7 +703,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -629,7 +713,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -639,7 +723,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -649,7 +733,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -659,7 +743,7 @@ namespace Model
      * <p>The weekly time range during which system maintenance can occur, in Universal
      * Coordinated Time (UTC).</p> <p>Format: <code>ddd:hh24:mi-ddd:hh24:mi</code> </p>
      * <p>The default is a 30-minute window selected at random from an 8-hour block of
-     * time for each AWS Region, occurring on a random day of the week.</p> <p>Valid
+     * time for each AWS Region, occurring on a random day of the week. </p> <p>Valid
      * Days: Mon, Tue, Wed, Thu, Fri, Sat, Sun.</p> <p>Constraints: Minimum 30-minute
      * window.</p>
      */
@@ -696,143 +780,77 @@ namespace Model
 
 
     /**
-     * <p>The configuration setting for the log types to be enabled for export to
-     * CloudWatch Logs for a specific DB cluster.</p>
-     */
-    inline const CloudwatchLogsExportConfiguration& GetCloudwatchLogsExportConfiguration() const{ return m_cloudwatchLogsExportConfiguration; }
-
-    /**
-     * <p>The configuration setting for the log types to be enabled for export to
-     * CloudWatch Logs for a specific DB cluster.</p>
-     */
-    inline bool CloudwatchLogsExportConfigurationHasBeenSet() const { return m_cloudwatchLogsExportConfigurationHasBeenSet; }
-
-    /**
-     * <p>The configuration setting for the log types to be enabled for export to
-     * CloudWatch Logs for a specific DB cluster.</p>
-     */
-    inline void SetCloudwatchLogsExportConfiguration(const CloudwatchLogsExportConfiguration& value) { m_cloudwatchLogsExportConfigurationHasBeenSet = true; m_cloudwatchLogsExportConfiguration = value; }
-
-    /**
-     * <p>The configuration setting for the log types to be enabled for export to
-     * CloudWatch Logs for a specific DB cluster.</p>
-     */
-    inline void SetCloudwatchLogsExportConfiguration(CloudwatchLogsExportConfiguration&& value) { m_cloudwatchLogsExportConfigurationHasBeenSet = true; m_cloudwatchLogsExportConfiguration = std::move(value); }
-
-    /**
-     * <p>The configuration setting for the log types to be enabled for export to
-     * CloudWatch Logs for a specific DB cluster.</p>
-     */
-    inline ModifyDBClusterRequest& WithCloudwatchLogsExportConfiguration(const CloudwatchLogsExportConfiguration& value) { SetCloudwatchLogsExportConfiguration(value); return *this;}
-
-    /**
-     * <p>The configuration setting for the log types to be enabled for export to
-     * CloudWatch Logs for a specific DB cluster.</p>
-     */
-    inline ModifyDBClusterRequest& WithCloudwatchLogsExportConfiguration(CloudwatchLogsExportConfiguration&& value) { SetCloudwatchLogsExportConfiguration(std::move(value)); return *this;}
-
-
-    /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline const Aws::String& GetEngineVersion() const{ return m_engineVersion; }
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline bool EngineVersionHasBeenSet() const { return m_engineVersionHasBeenSet; }
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline void SetEngineVersion(const Aws::String& value) { m_engineVersionHasBeenSet = true; m_engineVersion = value; }
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline void SetEngineVersion(Aws::String&& value) { m_engineVersionHasBeenSet = true; m_engineVersion = std::move(value); }
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline void SetEngineVersion(const char* value) { m_engineVersionHasBeenSet = true; m_engineVersion.assign(value); }
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline ModifyDBClusterRequest& WithEngineVersion(const Aws::String& value) { SetEngineVersion(value); return *this;}
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline ModifyDBClusterRequest& WithEngineVersion(Aws::String&& value) { SetEngineVersion(std::move(value)); return *this;}
 
     /**
-     * <p>The version number of the database engine. Currently, setting this parameter
-     * has no effect. To upgrade your database engine to the most recent release, use
-     * the <a>ApplyPendingMaintenanceAction</a> API.</p> <p>For a list of valid engine
-     * versions, see <a>CreateDBInstance</a>, or call
-     * <a>DescribeDBEngineVersions</a>.</p>
+     * <p>The version number of the database engine to which you want to upgrade.
+     * Changing this parameter results in an outage. The change is applied during the
+     * next maintenance window unless the ApplyImmediately parameter is set to
+     * true.</p> <p>For a list of valid engine versions, see <a>CreateDBInstance</a>,
+     * or call <a>DescribeDBEngineVersions</a>.</p>
      */
     inline ModifyDBClusterRequest& WithEngineVersion(const char* value) { SetEngineVersion(value); return *this;}
 
-
-    /**
-     * <p>A value that indicates whether the DB cluster has deletion protection
-     * enabled. The database can't be deleted when deletion protection is enabled. By
-     * default, deletion protection is disabled.</p>
-     */
-    inline bool GetDeletionProtection() const{ return m_deletionProtection; }
-
-    /**
-     * <p>A value that indicates whether the DB cluster has deletion protection
-     * enabled. The database can't be deleted when deletion protection is enabled. By
-     * default, deletion protection is disabled.</p>
-     */
-    inline bool DeletionProtectionHasBeenSet() const { return m_deletionProtectionHasBeenSet; }
-
-    /**
-     * <p>A value that indicates whether the DB cluster has deletion protection
-     * enabled. The database can't be deleted when deletion protection is enabled. By
-     * default, deletion protection is disabled.</p>
-     */
-    inline void SetDeletionProtection(bool value) { m_deletionProtectionHasBeenSet = true; m_deletionProtection = value; }
-
-    /**
-     * <p>A value that indicates whether the DB cluster has deletion protection
-     * enabled. The database can't be deleted when deletion protection is enabled. By
-     * default, deletion protection is disabled.</p>
-     */
-    inline ModifyDBClusterRequest& WithDeletionProtection(bool value) { SetDeletionProtection(value); return *this;}
-
   private:
 
     Aws::String m_dBClusterIdentifier;
@@ -871,14 +889,8 @@ namespace Model
     bool m_enableIAMDatabaseAuthentication;
     bool m_enableIAMDatabaseAuthenticationHasBeenSet;
 
-    CloudwatchLogsExportConfiguration m_cloudwatchLogsExportConfiguration;
-    bool m_cloudwatchLogsExportConfigurationHasBeenSet;
-
     Aws::String m_engineVersion;
     bool m_engineVersionHasBeenSet;
-
-    bool m_deletionProtection;
-    bool m_deletionProtectionHasBeenSet;
   };
 
 } // namespace Model
