@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/rds/RDS_EXPORTS.h>
@@ -24,7 +34,7 @@ namespace Model
   {
   public:
     DescribeDBEngineVersionsRequest();
-
+    
     // Service request name is the Operation name which will send this request out,
     // each operation should has unique request name, so that we can get operation's name from this request.
     // Note: this is not true for response, multiple operations may have the same response name,
@@ -186,42 +196,42 @@ namespace Model
 
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline const Aws::Vector<Filter>& GetFilters() const{ return m_filters; }
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline bool FiltersHasBeenSet() const { return m_filtersHasBeenSet; }
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline void SetFilters(const Aws::Vector<Filter>& value) { m_filtersHasBeenSet = true; m_filters = value; }
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline void SetFilters(Aws::Vector<Filter>&& value) { m_filtersHasBeenSet = true; m_filters = std::move(value); }
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline DescribeDBEngineVersionsRequest& WithFilters(const Aws::Vector<Filter>& value) { SetFilters(value); return *this;}
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline DescribeDBEngineVersionsRequest& WithFilters(Aws::Vector<Filter>&& value) { SetFilters(std::move(value)); return *this;}
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline DescribeDBEngineVersionsRequest& AddFilters(const Filter& value) { m_filtersHasBeenSet = true; m_filters.push_back(value); return *this; }
 
     /**
-     * <p>This parameter isn't currently supported.</p>
+     * <p>This parameter is not currently supported.</p>
      */
     inline DescribeDBEngineVersionsRequest& AddFilters(Filter&& value) { m_filtersHasBeenSet = true; m_filters.push_back(std::move(value)); return *this; }
 
@@ -229,7 +239,7 @@ namespace Model
     /**
      * <p> The maximum number of records to include in the response. If more than the
      * <code>MaxRecords</code> value is available, a pagination token called a marker
-     * is included in the response so you can retrieve the remaining results. </p>
+     * is included in the response so that the following results can be retrieved. </p>
      * <p>Default: 100</p> <p>Constraints: Minimum 20, maximum 100.</p>
      */
     inline int GetMaxRecords() const{ return m_maxRecords; }
@@ -237,7 +247,7 @@ namespace Model
     /**
      * <p> The maximum number of records to include in the response. If more than the
      * <code>MaxRecords</code> value is available, a pagination token called a marker
-     * is included in the response so you can retrieve the remaining results. </p>
+     * is included in the response so that the following results can be retrieved. </p>
      * <p>Default: 100</p> <p>Constraints: Minimum 20, maximum 100.</p>
      */
     inline bool MaxRecordsHasBeenSet() const { return m_maxRecordsHasBeenSet; }
@@ -245,7 +255,7 @@ namespace Model
     /**
      * <p> The maximum number of records to include in the response. If more than the
      * <code>MaxRecords</code> value is available, a pagination token called a marker
-     * is included in the response so you can retrieve the remaining results. </p>
+     * is included in the response so that the following results can be retrieved. </p>
      * <p>Default: 100</p> <p>Constraints: Minimum 20, maximum 100.</p>
      */
     inline void SetMaxRecords(int value) { m_maxRecordsHasBeenSet = true; m_maxRecords = value; }
@@ -253,7 +263,7 @@ namespace Model
     /**
      * <p> The maximum number of records to include in the response. If more than the
      * <code>MaxRecords</code> value is available, a pagination token called a marker
-     * is included in the response so you can retrieve the remaining results. </p>
+     * is included in the response so that the following results can be retrieved. </p>
      * <p>Default: 100</p> <p>Constraints: Minimum 20, maximum 100.</p>
      */
     inline DescribeDBEngineVersionsRequest& WithMaxRecords(int value) { SetMaxRecords(value); return *this;}
@@ -317,132 +327,91 @@ namespace Model
 
 
     /**
-     * <p>A value that indicates whether only the default version of the specified
-     * engine or engine and major version combination is returned.</p>
+     * <p>Indicates that only the default version of the specified engine or engine and
+     * major version combination is returned.</p>
      */
     inline bool GetDefaultOnly() const{ return m_defaultOnly; }
 
     /**
-     * <p>A value that indicates whether only the default version of the specified
-     * engine or engine and major version combination is returned.</p>
+     * <p>Indicates that only the default version of the specified engine or engine and
+     * major version combination is returned.</p>
      */
     inline bool DefaultOnlyHasBeenSet() const { return m_defaultOnlyHasBeenSet; }
 
     /**
-     * <p>A value that indicates whether only the default version of the specified
-     * engine or engine and major version combination is returned.</p>
+     * <p>Indicates that only the default version of the specified engine or engine and
+     * major version combination is returned.</p>
      */
     inline void SetDefaultOnly(bool value) { m_defaultOnlyHasBeenSet = true; m_defaultOnly = value; }
 
     /**
-     * <p>A value that indicates whether only the default version of the specified
-     * engine or engine and major version combination is returned.</p>
+     * <p>Indicates that only the default version of the specified engine or engine and
+     * major version combination is returned.</p>
      */
     inline DescribeDBEngineVersionsRequest& WithDefaultOnly(bool value) { SetDefaultOnly(value); return *this;}
 
 
     /**
-     * <p>A value that indicates whether to list the supported character sets for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>CharacterSetName</code> parameter for
-     * <code>CreateDBInstance</code>, the response includes a list of supported
-     * character sets for each engine version. </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the
+     * response includes a list of supported character sets for each engine version.
+     * </p>
      */
     inline bool GetListSupportedCharacterSets() const{ return m_listSupportedCharacterSets; }
 
     /**
-     * <p>A value that indicates whether to list the supported character sets for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>CharacterSetName</code> parameter for
-     * <code>CreateDBInstance</code>, the response includes a list of supported
-     * character sets for each engine version. </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the
+     * response includes a list of supported character sets for each engine version.
+     * </p>
      */
     inline bool ListSupportedCharacterSetsHasBeenSet() const { return m_listSupportedCharacterSetsHasBeenSet; }
 
     /**
-     * <p>A value that indicates whether to list the supported character sets for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>CharacterSetName</code> parameter for
-     * <code>CreateDBInstance</code>, the response includes a list of supported
-     * character sets for each engine version. </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the
+     * response includes a list of supported character sets for each engine version.
+     * </p>
      */
     inline void SetListSupportedCharacterSets(bool value) { m_listSupportedCharacterSetsHasBeenSet = true; m_listSupportedCharacterSets = value; }
 
     /**
-     * <p>A value that indicates whether to list the supported character sets for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>CharacterSetName</code> parameter for
-     * <code>CreateDBInstance</code>, the response includes a list of supported
-     * character sets for each engine version. </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>CharacterSetName</code> parameter for <code>CreateDBInstance</code>, the
+     * response includes a list of supported character sets for each engine version.
+     * </p>
      */
     inline DescribeDBEngineVersionsRequest& WithListSupportedCharacterSets(bool value) { SetListSupportedCharacterSets(value); return *this;}
 
 
     /**
-     * <p>A value that indicates whether to list the supported time zones for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>,
-     * the response includes a list of supported time zones for each engine version.
-     * </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response
+     * includes a list of supported time zones for each engine version. </p>
      */
     inline bool GetListSupportedTimezones() const{ return m_listSupportedTimezones; }
 
     /**
-     * <p>A value that indicates whether to list the supported time zones for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>,
-     * the response includes a list of supported time zones for each engine version.
-     * </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response
+     * includes a list of supported time zones for each engine version. </p>
      */
     inline bool ListSupportedTimezonesHasBeenSet() const { return m_listSupportedTimezonesHasBeenSet; }
 
     /**
-     * <p>A value that indicates whether to list the supported time zones for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>,
-     * the response includes a list of supported time zones for each engine version.
-     * </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response
+     * includes a list of supported time zones for each engine version. </p>
      */
     inline void SetListSupportedTimezones(bool value) { m_listSupportedTimezonesHasBeenSet = true; m_listSupportedTimezones = value; }
 
     /**
-     * <p>A value that indicates whether to list the supported time zones for each
-     * engine version.</p> <p>If this parameter is enabled and the requested engine
-     * supports the <code>TimeZone</code> parameter for <code>CreateDBInstance</code>,
-     * the response includes a list of supported time zones for each engine version.
-     * </p>
+     * <p>If this parameter is specified and the requested engine supports the
+     * <code>TimeZone</code> parameter for <code>CreateDBInstance</code>, the response
+     * includes a list of supported time zones for each engine version. </p>
      */
     inline DescribeDBEngineVersionsRequest& WithListSupportedTimezones(bool value) { SetListSupportedTimezones(value); return *this;}
 
-
-    /**
-     * <p>A value that indicates whether to include engine versions that aren't
-     * available in the list. The default is to list only available engine
-     * versions.</p>
-     */
-    inline bool GetIncludeAll() const{ return m_includeAll; }
-
-    /**
-     * <p>A value that indicates whether to include engine versions that aren't
-     * available in the list. The default is to list only available engine
-     * versions.</p>
-     */
-    inline bool IncludeAllHasBeenSet() const { return m_includeAllHasBeenSet; }
-
-    /**
-     * <p>A value that indicates whether to include engine versions that aren't
-     * available in the list. The default is to list only available engine
-     * versions.</p>
-     */
-    inline void SetIncludeAll(bool value) { m_includeAllHasBeenSet = true; m_includeAll = value; }
-
-    /**
-     * <p>A value that indicates whether to include engine versions that aren't
-     * available in the list. The default is to list only available engine
-     * versions.</p>
-     */
-    inline DescribeDBEngineVersionsRequest& WithIncludeAll(bool value) { SetIncludeAll(value); return *this;}
-
   private:
 
     Aws::String m_engine;
@@ -471,9 +440,6 @@ namespace Model
 
     bool m_listSupportedTimezones;
     bool m_listSupportedTimezonesHasBeenSet;
-
-    bool m_includeAll;
-    bool m_includeAllHasBeenSet;
   };
 
 } // namespace Model
