@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/sagemaker/SageMaker_EXPORTS.h>
@@ -18,10 +28,7 @@ namespace Model
     NOT_SET,
     ml_eia1_medium,
     ml_eia1_large,
-    ml_eia1_xlarge,
-    ml_eia2_medium,
-    ml_eia2_large,
-    ml_eia2_xlarge
+    ml_eia1_xlarge
   };
 
 namespace NotebookInstanceAcceleratorTypeMapper
