@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #include <aws/core/utils/Outcome.h>
 #include <aws/core/auth/AWSAuthSigner.h>
@@ -32,7 +42,6 @@
 #include <aws/serverlessrepo/model/ListApplicationVersionsRequest.h>
 #include <aws/serverlessrepo/model/ListApplicationsRequest.h>
 #include <aws/serverlessrepo/model/PutApplicationPolicyRequest.h>
-#include <aws/serverlessrepo/model/UnshareApplicationRequest.h>
 #include <aws/serverlessrepo/model/UpdateApplicationRequest.h>
 
 using namespace Aws;
@@ -50,7 +59,7 @@ static const char* ALLOCATION_TAG = "ServerlessApplicationRepositoryClient";
 ServerlessApplicationRepositoryClient::ServerlessApplicationRepositoryClient(const Client::ClientConfiguration& clientConfiguration) :
   BASECLASS(clientConfiguration,
     Aws::MakeShared<AWSAuthV4Signer>(ALLOCATION_TAG, Aws::MakeShared<DefaultAWSCredentialsProviderChain>(ALLOCATION_TAG),
-        SERVICE_NAME, Aws::Region::ComputeSignerRegion(clientConfiguration.region)),
+        SERVICE_NAME, clientConfiguration.region),
     Aws::MakeShared<ServerlessApplicationRepositoryErrorMarshaller>(ALLOCATION_TAG)),
     m_executor(clientConfiguration.executor)
 {
@@ -60,7 +69,7 @@ ServerlessApplicationRepositoryClient::ServerlessApplicationRepositoryClient(con
 ServerlessApplicationRepositoryClient::ServerlessApplicationRepositoryClient(const AWSCredentials& credentials, const Client::ClientConfiguration& clientConfiguration) :
   BASECLASS(clientConfiguration,
     Aws::MakeShared<AWSAuthV4Signer>(ALLOCATION_TAG, Aws::MakeShared<SimpleAWSCredentialsProvider>(ALLOCATION_TAG, credentials),
-         SERVICE_NAME, Aws::Region::ComputeSignerRegion(clientConfiguration.region)),
+         SERVICE_NAME, clientConfiguration.region),
     Aws::MakeShared<ServerlessApplicationRepositoryErrorMarshaller>(ALLOCATION_TAG)),
     m_executor(clientConfiguration.executor)
 {
@@ -71,7 +80,7 @@ ServerlessApplicationRepositoryClient::ServerlessApplicationRepositoryClient(con
   const Client::ClientConfiguration& clientConfiguration) :
   BASECLASS(clientConfiguration,
     Aws::MakeShared<AWSAuthV4Signer>(ALLOCATION_TAG, credentialsProvider,
-         SERVICE_NAME, Aws::Region::ComputeSignerRegion(clientConfiguration.region)),
+         SERVICE_NAME, clientConfiguration.region),
     Aws::MakeShared<ServerlessApplicationRepositoryErrorMarshaller>(ALLOCATION_TAG)),
     m_executor(clientConfiguration.executor)
 {
@@ -84,7 +93,6 @@ ServerlessApplicationRepositoryClient::~ServerlessApplicationRepositoryClient()
 
 void ServerlessApplicationRepositoryClient::init(const ClientConfiguration& config)
 {
-  SetServiceClientName("ServerlessApplicationRepository");
   m_configScheme = SchemeMapper::ToString(config.scheme);
   if (config.endpointOverride.empty())
   {
@@ -107,14 +115,21 @@ void ServerlessApplicationRepositoryClient::OverrideEndpoint(const Aws::String&
       m_uri = m_configScheme + "://" + endpoint;
   }
 }
-
 CreateApplicationOutcome ServerlessApplicationRepositoryClient::CreateApplication(const CreateApplicationRequest& request) const
 {
   Aws::Http::URI uri = m_uri;
   Aws::StringStream ss;
   ss << "/applications";
   uri.SetPath(uri.GetPath() + ss.str());
-  return CreateApplicationOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return CreateApplicationOutcome(CreateApplicationResult(outcome.GetResult()));
+  }
+  else
+  {
+    return CreateApplicationOutcome(outcome.GetError());
+  }
 }
 
 CreateApplicationOutcomeCallable ServerlessApplicationRepositoryClient::CreateApplicationCallable(const CreateApplicationRequest& request) const
@@ -154,7 +169,15 @@ CreateApplicationVersionOutcome ServerlessApplicationRepositoryClient::CreateApp
   ss << "/versions/";
   ss << request.GetSemanticVersion();
   uri.SetPath(uri.GetPath() + ss.str());
-  return CreateApplicationVersionOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_PUT, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_PUT, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return CreateApplicationVersionOutcome(CreateApplicationVersionResult(outcome.GetResult()));
+  }
+  else
+  {
+    return CreateApplicationVersionOutcome(outcome.GetError());
+  }
 }
 
 CreateApplicationVersionOutcomeCallable ServerlessApplicationRepositoryClient::CreateApplicationVersionCallable(const CreateApplicationVersionRequest& request) const
@@ -188,7 +211,15 @@ CreateCloudFormationChangeSetOutcome ServerlessApplicationRepositoryClient::Crea
   ss << request.GetApplicationId();
   ss << "/changesets";
   uri.SetPath(uri.GetPath() + ss.str());
-  return CreateCloudFormationChangeSetOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return CreateCloudFormationChangeSetOutcome(CreateCloudFormationChangeSetResult(outcome.GetResult()));
+  }
+  else
+  {
+    return CreateCloudFormationChangeSetOutcome(outcome.GetError());
+  }
 }
 
 CreateCloudFormationChangeSetOutcomeCallable ServerlessApplicationRepositoryClient::CreateCloudFormationChangeSetCallable(const CreateCloudFormationChangeSetRequest& request) const
@@ -222,7 +253,15 @@ CreateCloudFormationTemplateOutcome ServerlessApplicationRepositoryClient::Creat
   ss << request.GetApplicationId();
   ss << "/templates";
   uri.SetPath(uri.GetPath() + ss.str());
-  return CreateCloudFormationTemplateOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return CreateCloudFormationTemplateOutcome(CreateCloudFormationTemplateResult(outcome.GetResult()));
+  }
+  else
+  {
+    return CreateCloudFormationTemplateOutcome(outcome.GetError());
+  }
 }
 
 CreateCloudFormationTemplateOutcomeCallable ServerlessApplicationRepositoryClient::CreateCloudFormationTemplateCallable(const CreateCloudFormationTemplateRequest& request) const
@@ -255,7 +294,15 @@ DeleteApplicationOutcome ServerlessApplicationRepositoryClient::DeleteApplicatio
   ss << "/applications/";
   ss << request.GetApplicationId();
   uri.SetPath(uri.GetPath() + ss.str());
-  return DeleteApplicationOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_DELETE, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_DELETE, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return DeleteApplicationOutcome(NoResult());
+  }
+  else
+  {
+    return DeleteApplicationOutcome(outcome.GetError());
+  }
 }
 
 DeleteApplicationOutcomeCallable ServerlessApplicationRepositoryClient::DeleteApplicationCallable(const DeleteApplicationRequest& request) const
@@ -288,7 +335,15 @@ GetApplicationOutcome ServerlessApplicationRepositoryClient::GetApplication(cons
   ss << "/applications/";
   ss << request.GetApplicationId();
   uri.SetPath(uri.GetPath() + ss.str());
-  return GetApplicationOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return GetApplicationOutcome(GetApplicationResult(outcome.GetResult()));
+  }
+  else
+  {
+    return GetApplicationOutcome(outcome.GetError());
+  }
 }
 
 GetApplicationOutcomeCallable ServerlessApplicationRepositoryClient::GetApplicationCallable(const GetApplicationRequest& request) const
@@ -322,7 +377,15 @@ GetApplicationPolicyOutcome ServerlessApplicationRepositoryClient::GetApplicatio
   ss << request.GetApplicationId();
   ss << "/policy";
   uri.SetPath(uri.GetPath() + ss.str());
-  return GetApplicationPolicyOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return GetApplicationPolicyOutcome(GetApplicationPolicyResult(outcome.GetResult()));
+  }
+  else
+  {
+    return GetApplicationPolicyOutcome(outcome.GetError());
+  }
 }
 
 GetApplicationPolicyOutcomeCallable ServerlessApplicationRepositoryClient::GetApplicationPolicyCallable(const GetApplicationPolicyRequest& request) const
@@ -362,7 +425,15 @@ GetCloudFormationTemplateOutcome ServerlessApplicationRepositoryClient::GetCloud
   ss << "/templates/";
   ss << request.GetTemplateId();
   uri.SetPath(uri.GetPath() + ss.str());
-  return GetCloudFormationTemplateOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return GetCloudFormationTemplateOutcome(GetCloudFormationTemplateResult(outcome.GetResult()));
+  }
+  else
+  {
+    return GetCloudFormationTemplateOutcome(outcome.GetError());
+  }
 }
 
 GetCloudFormationTemplateOutcomeCallable ServerlessApplicationRepositoryClient::GetCloudFormationTemplateCallable(const GetCloudFormationTemplateRequest& request) const
@@ -396,7 +467,15 @@ ListApplicationDependenciesOutcome ServerlessApplicationRepositoryClient::ListAp
   ss << request.GetApplicationId();
   ss << "/dependencies";
   uri.SetPath(uri.GetPath() + ss.str());
-  return ListApplicationDependenciesOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return ListApplicationDependenciesOutcome(ListApplicationDependenciesResult(outcome.GetResult()));
+  }
+  else
+  {
+    return ListApplicationDependenciesOutcome(outcome.GetError());
+  }
 }
 
 ListApplicationDependenciesOutcomeCallable ServerlessApplicationRepositoryClient::ListApplicationDependenciesCallable(const ListApplicationDependenciesRequest& request) const
@@ -430,7 +509,15 @@ ListApplicationVersionsOutcome ServerlessApplicationRepositoryClient::ListApplic
   ss << request.GetApplicationId();
   ss << "/versions";
   uri.SetPath(uri.GetPath() + ss.str());
-  return ListApplicationVersionsOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return ListApplicationVersionsOutcome(ListApplicationVersionsResult(outcome.GetResult()));
+  }
+  else
+  {
+    return ListApplicationVersionsOutcome(outcome.GetError());
+  }
 }
 
 ListApplicationVersionsOutcomeCallable ServerlessApplicationRepositoryClient::ListApplicationVersionsCallable(const ListApplicationVersionsRequest& request) const
@@ -457,7 +544,15 @@ ListApplicationsOutcome ServerlessApplicationRepositoryClient::ListApplications(
   Aws::StringStream ss;
   ss << "/applications";
   uri.SetPath(uri.GetPath() + ss.str());
-  return ListApplicationsOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_GET, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return ListApplicationsOutcome(ListApplicationsResult(outcome.GetResult()));
+  }
+  else
+  {
+    return ListApplicationsOutcome(outcome.GetError());
+  }
 }
 
 ListApplicationsOutcomeCallable ServerlessApplicationRepositoryClient::ListApplicationsCallable(const ListApplicationsRequest& request) const
@@ -491,7 +586,15 @@ PutApplicationPolicyOutcome ServerlessApplicationRepositoryClient::PutApplicatio
   ss << request.GetApplicationId();
   ss << "/policy";
   uri.SetPath(uri.GetPath() + ss.str());
-  return PutApplicationPolicyOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_PUT, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_PUT, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return PutApplicationPolicyOutcome(PutApplicationPolicyResult(outcome.GetResult()));
+  }
+  else
+  {
+    return PutApplicationPolicyOutcome(outcome.GetError());
+  }
 }
 
 PutApplicationPolicyOutcomeCallable ServerlessApplicationRepositoryClient::PutApplicationPolicyCallable(const PutApplicationPolicyRequest& request) const
@@ -512,40 +615,6 @@ void ServerlessApplicationRepositoryClient::PutApplicationPolicyAsyncHelper(cons
   handler(this, request, PutApplicationPolicy(request), context);
 }
 
-UnshareApplicationOutcome ServerlessApplicationRepositoryClient::UnshareApplication(const UnshareApplicationRequest& request) const
-{
-  if (!request.ApplicationIdHasBeenSet())
-  {
-    AWS_LOGSTREAM_ERROR("UnshareApplication", "Required field: ApplicationId, is not set");
-    return UnshareApplicationOutcome(Aws::Client::AWSError<ServerlessApplicationRepositoryErrors>(ServerlessApplicationRepositoryErrors::MISSING_PARAMETER, "MISSING_PARAMETER", "Missing required field [ApplicationId]", false));
-  }
-  Aws::Http::URI uri = m_uri;
-  Aws::StringStream ss;
-  ss << "/applications/";
-  ss << request.GetApplicationId();
-  ss << "/unshare";
-  uri.SetPath(uri.GetPath() + ss.str());
-  return UnshareApplicationOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_POST, Aws::Auth::SIGV4_SIGNER));
-}
-
-UnshareApplicationOutcomeCallable ServerlessApplicationRepositoryClient::UnshareApplicationCallable(const UnshareApplicationRequest& request) const
-{
-  auto task = Aws::MakeShared< std::packaged_task< UnshareApplicationOutcome() > >(ALLOCATION_TAG, [this, request](){ return this->UnshareApplication(request); } );
-  auto packagedFunction = [task]() { (*task)(); };
-  m_executor->Submit(packagedFunction);
-  return task->get_future();
-}
-
-void ServerlessApplicationRepositoryClient::UnshareApplicationAsync(const UnshareApplicationRequest& request, const UnshareApplicationResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context) const
-{
-  m_executor->Submit( [this, request, handler, context](){ this->UnshareApplicationAsyncHelper( request, handler, context ); } );
-}
-
-void ServerlessApplicationRepositoryClient::UnshareApplicationAsyncHelper(const UnshareApplicationRequest& request, const UnshareApplicationResponseReceivedHandler& handler, const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context) const
-{
-  handler(this, request, UnshareApplication(request), context);
-}
-
 UpdateApplicationOutcome ServerlessApplicationRepositoryClient::UpdateApplication(const UpdateApplicationRequest& request) const
 {
   if (!request.ApplicationIdHasBeenSet())
@@ -558,7 +627,15 @@ UpdateApplicationOutcome ServerlessApplicationRepositoryClient::UpdateApplicatio
   ss << "/applications/";
   ss << request.GetApplicationId();
   uri.SetPath(uri.GetPath() + ss.str());
-  return UpdateApplicationOutcome(MakeRequest(uri, request, Aws::Http::HttpMethod::HTTP_PATCH, Aws::Auth::SIGV4_SIGNER));
+  JsonOutcome outcome = MakeRequest(uri, request, HttpMethod::HTTP_PATCH, Aws::Auth::SIGV4_SIGNER);
+  if(outcome.IsSuccess())
+  {
+    return UpdateApplicationOutcome(UpdateApplicationResult(outcome.GetResult()));
+  }
+  else
+  {
+    return UpdateApplicationOutcome(outcome.GetError());
+  }
 }
 
 UpdateApplicationOutcomeCallable ServerlessApplicationRepositoryClient::UpdateApplicationCallable(const UpdateApplicationRequest& request) const
