@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #pragma once
 #include <aws/sqs/SQS_EXPORTS.h>
@@ -252,79 +262,6 @@ namespace Model
     inline SendMessageBatchResultEntry& WithMD5OfMessageAttributes(const char* value) { SetMD5OfMessageAttributes(value); return *this;}
 
 
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline const Aws::String& GetMD5OfMessageSystemAttributes() const{ return m_mD5OfMessageSystemAttributes; }
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline bool MD5OfMessageSystemAttributesHasBeenSet() const { return m_mD5OfMessageSystemAttributesHasBeenSet; }
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline void SetMD5OfMessageSystemAttributes(const Aws::String& value) { m_mD5OfMessageSystemAttributesHasBeenSet = true; m_mD5OfMessageSystemAttributes = value; }
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline void SetMD5OfMessageSystemAttributes(Aws::String&& value) { m_mD5OfMessageSystemAttributesHasBeenSet = true; m_mD5OfMessageSystemAttributes = std::move(value); }
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline void SetMD5OfMessageSystemAttributes(const char* value) { m_mD5OfMessageSystemAttributesHasBeenSet = true; m_mD5OfMessageSystemAttributes.assign(value); }
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline SendMessageBatchResultEntry& WithMD5OfMessageSystemAttributes(const Aws::String& value) { SetMD5OfMessageSystemAttributes(value); return *this;}
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline SendMessageBatchResultEntry& WithMD5OfMessageSystemAttributes(Aws::String&& value) { SetMD5OfMessageSystemAttributes(std::move(value)); return *this;}
-
-    /**
-     * <p>An MD5 digest of the non-URL-encoded message system attribute string. You can
-     * use this attribute to verify that Amazon SQS received the message correctly.
-     * Amazon SQS URL-decodes the message before creating the MD5 digest. For
-     * information about MD5, see <a
-     * href="https://www.ietf.org/rfc/rfc1321.txt">RFC1321</a>.</p>
-     */
-    inline SendMessageBatchResultEntry& WithMD5OfMessageSystemAttributes(const char* value) { SetMD5OfMessageSystemAttributes(value); return *this;}
-
-
     /**
      * <p>This parameter applies only to FIFO (first-in-first-out) queues.</p> <p>The
      * large, non-consecutive number that Amazon SQS assigns to each message.</p>
@@ -411,9 +348,6 @@ namespace Model
     Aws::String m_mD5OfMessageAttributes;
     bool m_mD5OfMessageAttributesHasBeenSet;
 
-    Aws::String m_mD5OfMessageSystemAttributes;
-    bool m_mD5OfMessageSystemAttributesHasBeenSet;
-
     Aws::String m_sequenceNumber;
     bool m_sequenceNumberHasBeenSet;
   };
