@@ -1,7 +1,17 @@
-﻿/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+﻿/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #include <aws/storagegateway/model/DeleteTapeRequest.h>
 #include <aws/core/utils/json/JsonSerializer.h>
@@ -14,9 +24,7 @@ using namespace Aws::Utils;
 
 DeleteTapeRequest::DeleteTapeRequest() : 
     m_gatewayARNHasBeenSet(false),
-    m_tapeARNHasBeenSet(false),
-    m_bypassGovernanceRetention(false),
-    m_bypassGovernanceRetentionHasBeenSet(false)
+    m_tapeARNHasBeenSet(false)
 {
 }
 
@@ -36,12 +44,6 @@ Aws::String DeleteTapeRequest::SerializePayload() const
 
   }
 
-  if(m_bypassGovernanceRetentionHasBeenSet)
-  {
-   payload.WithBool("BypassGovernanceRetention", m_bypassGovernanceRetention);
-
-  }
-
   return payload.View().WriteReadable();
 }
 
