@@ -1,7 +1,17 @@
-/**
- * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
- * SPDX-License-Identifier: Apache-2.0.
- */
+/*
+* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+*
+* Licensed under the Apache License, Version 2.0 (the "License").
+* You may not use this file except in compliance with the License.
+* A copy of the License is located at
+*
+*  http://aws.amazon.com/apache2.0
+*
+* or in the "license" file accompanying this file. This file is distributed
+* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
+* express or implied. See the License for the specific language governing
+* permissions and limitations under the License.
+*/
 
 #include <aws/transfer/TransferManager.h>
 #include <aws/core/utils/memory/stl/AWSStreamFwd.h>
@@ -46,7 +56,7 @@ namespace Aws
         std::shared_ptr<TransferManager> TransferManager::Create(const TransferManagerConfiguration& config)
         {
             // Because TransferManager's ctor is private (to ensure it's always constructed as a shared_ptr)
-            // Aws::MakeShared does not have access to that private constructor. This workaround essentially
+            // Aws::MakeShared does not have access to that private constructor. This workaround essentially 
             // enables Aws::MakeShared to construct TransferManager.
             struct MakeSharedEnabler : public TransferManager {
                 MakeSharedEnabler(const TransferManagerConfiguration& config) : TransferManager(config) {}
@@ -61,7 +71,7 @@ namespace Aws
             assert(m_transferConfig.transferExecutor);
             for (uint64_t i = 0; i < m_transferConfig.transferBufferMaxHeapSize; i += m_transferConfig.bufferSize)
             {
-                m_bufferManager.PutResource(Aws::NewArray<unsigned char>(static_cast<size_t>(m_transferConfig.bufferSize), CLASS_TAG));
+                m_bufferManager.PutResource(Aws::NewArray<unsigned char>((m_transferConfig.bufferSize), CLASS_TAG));
             }
         }
 
@@ -92,9 +102,9 @@ namespace Aws
             return this->DoUploadFile(fileStream, bucketName, keyName, contentType, metadata, context);
         }
 
-        std::shared_ptr<TransferHandle> TransferManager::DownloadFile(const Aws::String& bucketName,
-                                                                      const Aws::String& keyName,
-                                                                      CreateDownloadStreamCallback writeToStreamfn,
+        std::shared_ptr<TransferHandle> TransferManager::DownloadFile(const Aws::String& bucketName, 
+                                                                      const Aws::String& keyName, 
+                                                                      CreateDownloadStreamCallback writeToStreamfn, 
                                                                       const DownloadConfiguration& downloadConfig,
                                                                       const Aws::String& writeToFile,
                                                                       const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context)
@@ -108,27 +118,9 @@ namespace Aws
             return handle;
         }
 
-        std::shared_ptr<TransferHandle> TransferManager::DownloadFile(const Aws::String& bucketName,
-                                                                      const Aws::String& keyName,
-                                                                      uint64_t fileOffset,
-                                                                      uint64_t downloadBytes,
-                                                                      CreateDownloadStreamCallback writeToStreamfn,
-                                                                      const DownloadConfiguration& downloadConfig,
-                                                                      const Aws::String& writeToFile,
-                                                                      const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context)
-        {
-            auto handle = Aws::MakeShared<TransferHandle>(CLASS_TAG, bucketName, keyName, fileOffset, downloadBytes, writeToStreamfn, writeToFile);
-            handle->ApplyDownloadConfiguration(downloadConfig);
-            handle->SetContext(context);
-
-            auto self = shared_from_this();
-            m_transferConfig.transferExecutor->Submit([self, handle] { self->DoDownload(handle); });
-            return handle;
-        }
-
-        std::shared_ptr<TransferHandle> TransferManager::DownloadFile(const Aws::String& bucketName,
-                                                                      const Aws::String& keyName,
-                                                                      const Aws::String& writeToFile,
+        std::shared_ptr<TransferHandle> TransferManager::DownloadFile(const Aws::String& bucketName, 
+                                                                      const Aws::String& keyName, 
+                                                                      const Aws::String& writeToFile, 
                                                                       const DownloadConfiguration& downloadConfig,
                                                                       const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context)
         {
@@ -240,7 +232,7 @@ namespace Aws
         {
             assert(m_transferConfig.transferInitiatedCallback);
             Aws::FileSystem::CreateDirectoryIfNotExists(directory.c_str());
-
+         
             auto self = shared_from_this(); // keep transfer manager alive until all callbacks are finished.
             auto handler = [self](const Aws::S3::S3Client* client, const Aws::S3::Model::ListObjectsV2Request& request,
                 const Aws::S3::Model::ListObjectsV2Outcome& outcome,
@@ -301,7 +293,7 @@ namespace Aws
                     {
                         uint64_t partSize = (std::min)(totalSize - i * m_transferConfig.bufferSize, m_transferConfig.bufferSize);
                         bool lastPart = (i == partCount - 1) ? true : false;
-                        handle->AddQueuedPart(Aws::MakeShared<PartState>(CLASS_TAG, static_cast<int>(i + 1), 0, partSize, lastPart));
+                        handle->AddQueuedPart(Aws::MakeShared<PartState>(CLASS_TAG, static_cast<int>(i + 1), 0, static_cast<size_t>(partSize), lastPart));
                     }
                 }
                 else
@@ -319,7 +311,7 @@ namespace Aws
             }
             else
             {
-                uint64_t bytesLeft = 0;
+                size_t bytesLeft = 0;
                 //at this point we've been going synchronously so this is consistent
                 const auto failedPartsSize = handle->GetFailedParts().size();
                 for (auto failedParts : handle->GetFailedParts())
@@ -373,9 +365,6 @@ namespace Aws
 
                     uploadPartRequest.SetBody(preallocatedStreamReader);
                     uploadPartRequest.SetContentType(handle->GetContentType());
-                    if (m_transferConfig.computeContentMD5) {
-                        uploadPartRequest.SetContentMD5(Aws::Utils::HashingUtils::Base64Encode(Aws::Utils::HashingUtils::CalculateMD5(*preallocatedStreamReader)));
-                    }
                     auto asyncContext = Aws::MakeShared<TransferHandleAsyncContext>(CLASS_TAG);
                     asyncContext->handle = handle;
                     asyncContext->partState = partsIter->second;
@@ -424,7 +413,7 @@ namespace Aws
 
         void TransferManager::DoSinglePartUpload(const std::shared_ptr<Aws::IOStream>& streamToPut, const std::shared_ptr<TransferHandle>& handle)
         {
-            auto partState = Aws::MakeShared<PartState>(CLASS_TAG, 1, 0, handle->GetBytesTotalSize(), true);
+            auto partState = Aws::MakeShared<PartState>(CLASS_TAG, 1, 0, static_cast<size_t>(handle->GetBytesTotalSize()), true);
 
             handle->UpdateStatus(TransferStatus::IN_PROGRESS);
             handle->SetIsMultipart(false);
@@ -449,9 +438,6 @@ namespace Aws
             auto preallocatedStreamReader = Aws::MakeShared<Aws::IOStream>(CLASS_TAG, streamBuf);
 
             putObjectRequest.SetBody(preallocatedStreamReader);
-            if (m_transferConfig.computeContentMD5) {
-                putObjectRequest.SetContentMD5(Aws::Utils::HashingUtils::Base64Encode(Aws::Utils::HashingUtils::CalculateMD5(*preallocatedStreamReader)));
-            }
 
             auto self = shared_from_this(); // keep transfer manager alive until all callbacks are finished.
             auto uploadProgressCallback = [self, partState, handle](const Aws::Http::HttpRequest*, long long progress)
@@ -630,7 +616,7 @@ namespace Aws
         {
             assert(retryHandle->GetStatus() != TransferStatus::IN_PROGRESS);
             assert(retryHandle->GetStatus() != TransferStatus::COMPLETED);
-            assert(retryHandle->GetStatus() != TransferStatus::NOT_STARTED);
+            assert(retryHandle->GetStatus() != TransferStatus::NOT_STARTED);           
 
             if (retryHandle->GetStatus() == TransferStatus::ABORTED)
             {
@@ -642,34 +628,23 @@ namespace Aws
             retryHandle->UpdateStatus(TransferStatus::NOT_STARTED);
             retryHandle->Restart();
             TriggerTransferStatusUpdatedCallback(retryHandle);
-
+            
             auto self = shared_from_this();
-            m_transferConfig.transferExecutor->Submit([self, retryHandle]
+            m_transferConfig.transferExecutor->Submit([self, retryHandle] 
                                                       { self->DoDownload(retryHandle); });
 
             return retryHandle;
         }
 
-        static Aws::String FormatRangeSpecifier(uint64_t rangeStart, uint64_t rangeEnd)
-        {
-            Aws::StringStream rangeStream;
-            rangeStream << "bytes=" << rangeStart << "-" << rangeEnd;
-            return rangeStream.str();
-        }
-
         void TransferManager::DoSinglePartDownload(const std::shared_ptr<TransferHandle>& handle)
         {
             auto queuedParts = handle->GetQueuedParts();
             assert(queuedParts.size() == 1);
 
             auto partState = queuedParts.begin()->second;
-            auto request = m_transferConfig.getObjectTemplate;
+            Aws::S3::Model::GetObjectRequest request;
             request.SetCustomizedAccessLogTag(m_transferConfig.customizedAccessLogTag);
             request.SetContinueRequestHandler([handle](const Aws::Http::HttpRequest*) { return handle->ShouldContinue(); });
-            request.SetRange(
-                FormatRangeSpecifier(
-                    handle->GetBytesOffset(),
-                    handle->GetBytesOffset() + handle->GetBytesTotalSize() - 1));
             request.WithBucket(handle->GetBucketName())
                    .WithKey(handle->GetKey());
 
@@ -715,10 +690,17 @@ namespace Aws
             TriggerTransferStatusUpdatedCallback(handle);
         }
 
+        static Aws::String FormatRangeSpecifier(std::size_t rangeStart, std::size_t rangeEnd)
+        {
+            Aws::StringStream rangeStream;
+            rangeStream << "bytes=" << rangeStart << "-" << rangeEnd;
+            return rangeStream.str();
+        }
+
         bool TransferManager::InitializePartsForDownload(const std::shared_ptr<TransferHandle>& handle)
         {
             bool isRetry = handle->HasParts();
-            uint64_t bufferSize = m_transferConfig.bufferSize;
+            size_t bufferSize = static_cast<size_t>(m_transferConfig.bufferSize);
             if (!isRetry)
             {
                 Aws::S3::Model::HeadObjectRequest headObjectRequest;
@@ -730,16 +712,7 @@ namespace Aws
                 {
                     headObjectRequest.SetVersionId(handle->GetVersionId());
                 }
-
-                if (handle->GetBytesTotalSize() != 0)
-                {
-                    // if non zero, then offset and numbytes were passed, so using part download
-                    headObjectRequest.SetRange(
-                        FormatRangeSpecifier(
-                            handle->GetBytesOffset(),
-                            handle->GetBytesOffset() + handle->GetBytesTotalSize() - 1));
-                }
-
+            
                 auto headObjectOutcome = m_transferConfig.s3Client->HeadObject(headObjectRequest);
 
                 if (!headObjectOutcome.IsSuccess())
@@ -755,21 +728,17 @@ namespace Aws
                     return false;
                 }
 
-                auto downloadSize = static_cast<uint64_t>(headObjectOutcome.GetResult().GetContentLength());
+                std::size_t downloadSize = static_cast<size_t>(headObjectOutcome.GetResult().GetContentLength());
                 handle->SetBytesTotalSize(downloadSize);
                 handle->SetContentType(headObjectOutcome.GetResult().GetContentType());
                 handle->SetMetadata(headObjectOutcome.GetResult().GetMetadata());
-                /* When bucket versioning is susppended, head object will return "null" for unversioned object.
-                 * Send following GetObject with "null" as versionId will result in 403 access denied error if your IAM role or policy
-                 * doesn't have GetObjectVersion permission.
-                 */
-                if(handle->GetVersionId().empty() && headObjectOutcome.GetResult().GetVersionId() != "null")
+                if(handle->GetVersionId().empty())
                 {
                     handle->SetVersionId(headObjectOutcome.GetResult().GetVersionId());
                 }
 
                 // For empty file, we create 1 part here to make downloading behaviors consistent for files with different size.
-                std::size_t partCount = (std::max)((downloadSize + bufferSize - 1) / bufferSize, static_cast<uint64_t>(1));
+                std::size_t partCount = (std::max)((downloadSize + bufferSize - 1) / bufferSize, static_cast<std::size_t>(1));
                 handle->SetIsMultipart(partCount > 1);    // doesn't make a difference but let's be accurate
 
                 for(std::size_t i = 0; i < partCount; ++i)
@@ -802,7 +771,7 @@ namespace Aws
             TriggerTransferStatusUpdatedCallback(handle);
 
             bool isMultipart = handle->IsMultipart();
-            uint64_t bufferSize = m_transferConfig.bufferSize;
+            size_t bufferSize = static_cast<size_t>(m_transferConfig.bufferSize);
 
             if(!isMultipart)
             {
@@ -816,14 +785,14 @@ namespace Aws
             while(queuedPartIter != queuedParts.end() && handle->ShouldContinue())
             {
                 const auto& partState = queuedPartIter->second;
-                uint64_t rangeStart = handle->GetBytesOffset() + ( partState->GetPartId() - 1 ) * bufferSize;
-                uint64_t rangeEnd = rangeStart + partState->GetSizeInBytes() - 1;
+                std::size_t rangeStart = ( partState->GetPartId() - 1 ) * bufferSize;
+                std::size_t rangeEnd = rangeStart + partState->GetSizeInBytes() - 1;
                 auto buffer = m_bufferManager.Acquire();
                 partState->SetDownloadBuffer(buffer);
 
-                CreateDownloadStreamCallback responseStreamFunction = [partState, buffer, rangeEnd, rangeStart]()
+                CreateDownloadStreamCallback responseStreamFunction = [partState, buffer, rangeEnd, rangeStart]() 
                 {
-                    auto bufferStream = Aws::New<Aws::Utils::Stream::DefaultUnderlyingStream>(CLASS_TAG,
+                    auto bufferStream = Aws::New<Aws::Utils::Stream::DefaultUnderlyingStream>(CLASS_TAG, 
                             Aws::MakeUnique<Aws::Utils::Stream::PreallocatedStreamBuf>(CLASS_TAG, buffer, rangeEnd - rangeStart + 1));
                     partState->SetDownloadPartStream(bufferStream);
                     return bufferStream;
@@ -833,7 +802,7 @@ namespace Aws
                 {
                     partState->SetDownloadBuffer(buffer);
 
-                    auto getObjectRangeRequest = m_transferConfig.getObjectTemplate;
+                    Aws::S3::Model::GetObjectRequest getObjectRangeRequest;
                     getObjectRangeRequest.SetCustomizedAccessLogTag(m_transferConfig.customizedAccessLogTag);
                     getObjectRangeRequest.SetContinueRequestHandler([handle](const Aws::Http::HttpRequest*) { return handle->ShouldContinue(); });
                     getObjectRangeRequest.SetBucket(handle->GetBucketName());
@@ -891,13 +860,13 @@ namespace Aws
             if (handle->HasFailedParts())
             {
                 handle->UpdateStatus(DetermineIfFailedOrCanceled(*handle));
-                TriggerTransferStatusUpdatedCallback(handle);
+                TriggerTransferStatusUpdatedCallback(handle);      
             }
         }
 
-        void TransferManager::HandleGetObjectResponse(const Aws::S3::S3Client* client,
+        void TransferManager::HandleGetObjectResponse(const Aws::S3::S3Client* client, 
                                                       const Aws::S3::Model::GetObjectRequest& request,
-                                                      const Aws::S3::Model::GetObjectOutcome& outcome,
+                                                      const Aws::S3::Model::GetObjectOutcome& outcome, 
                                                       const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context)
         {
             AWS_UNREFERENCED_PARAM(client);
@@ -1129,15 +1098,15 @@ namespace Aws
 
         bool TransferManager::MultipartUploadSupported(uint64_t length) const
         {
-            return length > m_transferConfig.bufferSize &&
-                   m_transferConfig.s3Client            &&
+            return length > m_transferConfig.bufferSize && 
+                   m_transferConfig.s3Client            && 
                    m_transferConfig.s3Client->MultipartUploadSupported();
         }
 
         std::shared_ptr<TransferHandle> TransferManager::CreateUploadFileHandle(Aws::IOStream* fileStream,
                                                                                 const Aws::String& bucketName,
                                                                                 const Aws::String& keyName,
-                                                                                const Aws::String& contentType,
+                                                                                const Aws::String& contentType, 
                                                                                 const Aws::Map<Aws::String, Aws::String>& metadata,
                                                                                 const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context,
                                                                                 const Aws::String& fileName)
@@ -1160,7 +1129,7 @@ namespace Aws
             AWS_LOGSTREAM_TRACE(CLASS_TAG, "Seeking input stream to determine content-length to upload file to bucket: "
                     << bucketName << " with key: " << keyName);
             fileStream->seekg(0, std::ios_base::end);
-            auto length = static_cast<uint64_t>(fileStream->tellg());
+            size_t length = static_cast<size_t>(fileStream->tellg());
             fileStream->seekg(0, std::ios_base::beg);
             AWS_LOGSTREAM_TRACE(CLASS_TAG, "Setting content-length to " << length << " bytes. To upload file to bucket: "
                     << bucketName << " with key: " << keyName);
@@ -1220,7 +1189,7 @@ namespace Aws
 
         std::shared_ptr<TransferHandle> TransferManager::DoUploadFile(const Aws::String& fileName,
                                                                       const Aws::String& bucketName,
-                                                                      const Aws::String& keyName,
+                                                                      const Aws::String& keyName, 
                                                                       const Aws::String& contentType,
                                                                       const Aws::Map<Aws::String, Aws::String>& metadata,
                                                                       const std::shared_ptr<const Aws::Client::AsyncCallerContext>& context)
