@@ -1,41 +1,42 @@
 #!/usr/bin/env python3
 """Command line utility for determining what jstests have been added or modified."""
-
 import copy
-import json
+import datetime
 import logging
 import os.path
 import shlex
 import subprocess
 import sys
-
-from math import ceil
 from collections import defaultdict
-from typing import Optional, Set, Tuple, List, Dict
+from math import ceil
+from typing import Optional, Set, Tuple, List, Dict, Iterable
 
 import click
+import requests
+import yaml
+from evergreen.api import RetryingEvergreenApi, EvergreenApi
+from git import Repo
 import structlog
 from structlog.stdlib import LoggerFactory
-import yaml
 
-from git import Repo
-from shrub.config import Configuration
+from shrub.v2 import Task, TaskDependency, BuildVariant, ShrubProject, ExistingTask
 
 # Get relative imports to work when the package is not installed on the PYTHONPATH.
 if __name__ == "__main__" and __package__ is None:
     sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
 
 # pylint: disable=wrong-import-position
-from buildscripts.patch_builds.change_data import find_changed_files
+from buildscripts.patch_builds.change_data import find_changed_files_in_repos
+import buildscripts.resmokelib.parser
 from buildscripts.resmokelib.suitesconfig import create_test_membership_map, get_suites
-
 from buildscripts.resmokelib.utils import default_if_none, globstar
 from buildscripts.ciconfig.evergreen import parse_evergreen_file, ResmokeArgs, \
     EvergreenProjectConfig, VariantTask
-from buildscripts.util.teststats import TestStats, get_stats_from_s3
+from buildscripts.util.fileops import write_file
+from buildscripts.util.teststats import TestStats
 from buildscripts.util.taskname import name_generated_task
-from buildscripts.patch_builds.task_generation import resmoke_commands, TimeoutInfo, TaskList
-
+from buildscripts.patch_builds.task_generation import (resmoke_commands, TimeoutInfo,
+                                                       validate_task_generation_limit)
 # pylint: enable=wrong-import-position
 
 structlog.configure(logger_factory=LoggerFactory())
@@ -46,13 +47,14 @@ EXTERNAL_LOGGERS = {
     "urllib3",
 }
 
+AVG_TEST_RUNTIME_ANALYSIS_DAYS = 14
 AVG_TEST_SETUP_SEC = 4 * 60
 AVG_TEST_TIME_MULTIPLIER = 3
 CONFIG_FILE = ".evergreen.yml"
 DEFAULT_PROJECT = "mongodb-mongo-master"
+DEFAULT_REPO_LOCATIONS = [".", "./src/mongo/db/modules/enterprise"]
 REPEAT_SUITES = 2
 EVERGREEN_FILE = "etc/evergreen.yml"
-MAX_TASKS_TO_CREATE = 1000
 MIN_AVG_TEST_OVERFLOW_SEC = float(60)
 MIN_AVG_TEST_TIME_SEC = 5 * 60
 # The executor_file and suite_files defaults are required to make the suite resolver work
@@ -135,7 +137,7 @@ class GenerateConfig(object):
     """Configuration for how to generate tasks."""
 
     def __init__(self, build_variant: str, project: str, run_build_variant: Optional[str] = None,
-                 distro: Optional[str] = None):
+                 distro: Optional[str] = None, task_id: Optional[str] = None):
         # pylint: disable=too-many-arguments,too-many-locals
         """
         Create a GenerateConfig.
@@ -144,11 +146,13 @@ class GenerateConfig(object):
         :param project: Project to run tasks on.
         :param run_build_variant: Build variant to run new tasks on.
         :param distro: Distro to run tasks on.
+        :param task_id: Evergreen task being run under.
         """
         self.build_variant = build_variant
         self._run_build_variant = run_build_variant
         self.distro = distro
         self.project = project
+        self.task_id = task_id
 
     @property
     def run_build_variant(self):
@@ -179,7 +183,7 @@ class GenerateConfig(object):
             raise ValueError(f"Build variant '{build_variant}' not found in Evergreen file")
 
 
-def _is_file_a_test_file(file_path: str) -> bool:
+def is_file_a_test_file(file_path: str) -> bool:
     """
     Check if the given path points to a test file.
 
@@ -196,21 +200,18 @@ def _is_file_a_test_file(file_path: str) -> bool:
     return True
 
 
-def find_changed_tests(repo: Repo) -> Set[str]:
+def find_changed_tests(repos: Iterable[Repo]) -> Set[str]:
     """
     Find the changed tests.
 
-    Use git to find which files have changed in this patch.
-    TODO: This should be expanded to search for enterprise modules.
+    Use git to find which test files have changed in this patch.
     The returned file paths are in normalized form (see os.path.normpath(path)).
 
+    :param repos: List of repos containing changed files.
     :returns: Set of changed tests.
     """
-    changed_files = find_changed_files(repo)
-    LOGGER.debug("Found changed files", files=changed_files)
-    changed_tests = {os.path.normpath(path) for path in changed_files if _is_file_a_test_file(path)}
-    LOGGER.debug("Found changed tests", files=changed_tests)
-    return changed_tests
+    changed_files = find_changed_files_in_repos(repos)
+    return {os.path.normpath(path) for path in changed_files if is_file_a_test_file(path)}
 
 
 def find_excludes(selector_file: str) -> Tuple[List, List, List]:
@@ -225,9 +226,8 @@ def find_excludes(selector_file: str) -> Tuple[List, List, List]:
 
     try:
         js_test = yml["selector"]["js_test"]
-    except KeyError as exc:
-        raise Exception(
-            f"The selector file {selector_file} is missing the 'selector.js_test' key") from exc
+    except KeyError:
+        raise Exception(f"The selector file {selector_file} is missing the 'selector.js_test' key")
 
     return (default_if_none(js_test.get("exclude_suites"), []),
             default_if_none(js_test.get("exclude_tasks"), []),
@@ -295,12 +295,12 @@ def _set_resmoke_args(task):
     resmoke_args.
     """
 
-    resmoke_args = task.resmoke_args
+    resmoke_args = task.combined_resmoke_args
     suite_name = ResmokeArgs.get_arg(resmoke_args, "suites")
     if task.is_generate_resmoke_task:
         suite_name = task.get_vars_suite_name(task.generate_resmoke_tasks_command["vars"])
 
-    return ResmokeArgs.get_updated_arg(resmoke_args, "suites", suite_name)
+    return ResmokeArgs.set_updated_arg(resmoke_args, "suites", suite_name)
 
 
 def _distro_to_run_task_on(task: VariantTask, evg_proj_config: EvergreenProjectConfig,
@@ -343,6 +343,7 @@ def _gather_task_info(task: VariantTask, tests_by_suite: Dict,
     :return: Dictionary of information needed to run task.
     """
     return {
+        "display_task_name": _get_task_name(task),
         "resmoke_args": _set_resmoke_args(task),
         "tests": tests_by_suite[task.resmoke_suite],
         "use_multiversion": task.multiversion_path,
@@ -379,7 +380,7 @@ def create_task_list(evergreen_conf: EvergreenProjectConfig, build_variant: str,
     # Find all the build variant tasks.
     exclude_tasks_set = set(exclude_tasks)
     all_variant_tasks = {
-        _get_task_name(task): task
+        task.name: task
         for task in evg_build_variant.tasks
         if task.name not in exclude_tasks_set and task.combined_resmoke_args
     }
@@ -394,13 +395,6 @@ def create_task_list(evergreen_conf: EvergreenProjectConfig, build_variant: str,
     return task_list
 
 
-def _write_json_file(json_data, pathname):
-    """Write out a JSON file."""
-
-    with open(pathname, "w") as fstream:
-        json.dump(json_data, fstream, indent=4)
-
-
 def _set_resmoke_cmd(repeat_config: RepeatConfig, resmoke_args: [str]) -> [str]:
     """Build the resmoke command, if a resmoke.py command wasn't passed in."""
     new_args = [sys.executable, "buildscripts/resmoke.py", "run"]
@@ -487,65 +481,125 @@ def _generate_timeouts(repeat_config: RepeatConfig, test: str,
     return TimeoutInfo.default_timeout()
 
 
-def _get_task_runtime_history(project: str, task: str, variant: str):
+def _get_task_runtime_history(evg_api: Optional[EvergreenApi], project: str, task: str,
+                              variant: str):
     """
-    Fetch historical average runtime for all tests in a task from S3.
+    Fetch historical average runtime for all tests in a task from Evergreen API.
 
+    :param evg_api: Evergreen API.
     :param project: Project name.
     :param task: Task name.
     :param variant: Variant name.
     :return: Test historical runtimes, parsed into teststat objects.
     """
-    data = get_stats_from_s3(project, task, variant)
-    test_runtimes = TestStats(data).get_tests_runtimes()
-    return test_runtimes
+    if not evg_api:
+        return []
+
+    try:
+        end_date = datetime.datetime.utcnow().replace(microsecond=0)
+        start_date = end_date - datetime.timedelta(days=AVG_TEST_RUNTIME_ANALYSIS_DAYS)
+        data = evg_api.test_stats_by_project(project, after_date=start_date.strftime("%Y-%m-%d"),
+                                             before_date=end_date.strftime("%Y-%m-%d"),
+                                             tasks=[task], variants=[variant], group_by="test",
+                                             group_num_days=AVG_TEST_RUNTIME_ANALYSIS_DAYS)
+        test_runtimes = TestStats(data).get_tests_runtimes()
+        return test_runtimes
+    except requests.HTTPError as err:
+        if err.response.status_code == requests.codes.SERVICE_UNAVAILABLE:
+            # Evergreen may return a 503 when the service is degraded.
+            # We fall back to returning no test history
+            return []
+        else:
+            raise
 
 
-def create_generate_tasks_config(evg_config: Configuration, tests_by_task: Dict,
+def _create_task(index: int, test_count: int, test: str, task_data: Dict,
+                 task_runtime_stats: List[TestStats], generate_config: GenerateConfig,
+                 repeat_config: RepeatConfig, task_prefix: str) -> Task:
+    # pylint: disable=too-many-arguments,too-many-locals
+    """
+    Create the described shrub sub task.
+
+    :param index: Index of task being created.
+    :param test_count: Total number of testing being created.
+    :param test: Test task is being generated for.
+    :param task_data: Data about task to create.
+    :param task_runtime_stats: Historical runtime of test.
+    :param generate_config: Configuration of how to generate the task.
+    :param repeat_config: Configuration of how the task should be repeated.
+    :param task_prefix: String to prefix generated task with.
+    :return: Shrub task for given configuration.
+    """
+    # TODO: Extract multiversion related code into separate tooling - SERVER-47137
+    multiversion_path = task_data.get("use_multiversion")
+    display_task_name = task_data["display_task_name"]
+    resmoke_args = task_data["resmoke_args"]
+    sub_task_name = name_generated_task(f"{task_prefix}:{display_task_name}", index, test_count,
+                                        generate_config.run_build_variant)
+    LOGGER.debug("Generating sub-task", sub_task=sub_task_name)
+
+    test_unix_style = test.replace('\\', '/')
+    run_tests_vars = {
+        "resmoke_args":
+            f"{resmoke_args} {repeat_config.generate_resmoke_options()} {test_unix_style}"
+    }
+    if multiversion_path:
+        run_tests_vars["task_path_suffix"] = multiversion_path
+    timeout = _generate_timeouts(repeat_config, test, task_runtime_stats)
+    commands = resmoke_commands("run tests", run_tests_vars, timeout, multiversion_path)
+    dependencies = {TaskDependency("compile")}
+
+    return Task(sub_task_name, commands, dependencies)
+
+
+def create_generated_tasks(tests_by_task: Dict, task_prefix: str, generate_config: GenerateConfig,
+                           repeat_config: RepeatConfig, evg_api: EvergreenApi) -> Set[Task]:
+    """
+    Create the set of tasks to run the given tests_by_task.
+
+    :param tests_by_task: Dictionary of tests to generate tasks for.
+    :param task_prefix: Prefix all task names with this.
+    :param generate_config: Configuration of what to generate.
+    :param repeat_config: Configuration of how to repeat tests.
+    :param evg_api: Evergreen API.
+    :return: Set of shrub tasks to run tests_by_task.
+    """
+    tasks: Set[Task] = set()
+    for task in sorted(tests_by_task):
+        task_info = tests_by_task[task]
+        test_list = task_info["tests"]
+        task_runtime_stats = _get_task_runtime_history(evg_api, generate_config.project,
+                                                       task_info["display_task_name"],
+                                                       generate_config.build_variant)
+        test_count = len(test_list)
+        for index, test in enumerate(test_list):
+            tasks.add(
+                _create_task(index, test_count, test, task_info, task_runtime_stats,
+                             generate_config, repeat_config, task_prefix))
+
+    return tasks
+
+
+def create_generate_tasks_config(build_variant: BuildVariant, tests_by_task: Dict,
                                  generate_config: GenerateConfig, repeat_config: RepeatConfig,
-                                 include_gen_task: bool = True,
-                                 task_prefix: str = "burn_in") -> Configuration:
+                                 evg_api: Optional[EvergreenApi], include_gen_task: bool = True,
+                                 task_prefix: str = "burn_in") -> None:
     # pylint: disable=too-many-arguments,too-many-locals
     """
     Create the config for the Evergreen generate.tasks file.
 
-    :param evg_config: Shrub configuration to add to.
+    :param build_variant: Shrub configuration to add to.
     :param tests_by_task: Dictionary of tests to generate tasks for.
     :param generate_config: Configuration of what to generate.
     :param repeat_config: Configuration of how to repeat tests.
+    :param evg_api: Evergreen API.
     :param include_gen_task: Should generating task be include in display task.
     :param task_prefix: Prefix all task names with this.
-    :return: Shrub configuration with added tasks.
     """
-    task_list = TaskList(evg_config)
-    resmoke_options = repeat_config.generate_resmoke_options()
-    for task in sorted(tests_by_task):
-        multiversion_path = tests_by_task[task].get("use_multiversion")
-        task_runtime_stats = _get_task_runtime_history(generate_config.project, task,
-                                                       generate_config.build_variant)
-        resmoke_args = tests_by_task[task]["resmoke_args"]
-        test_list = tests_by_task[task]["tests"]
-        distro = tests_by_task[task].get("distro", generate_config.distro)
-        for index, test in enumerate(test_list):
-            # Evergreen always uses a unix shell, even on Windows, so instead of using os.path.join
-            # here, just use the forward slash; otherwise the path separator will be treated as
-            # the escape character on Windows.
-            sub_task_name = name_generated_task(f"{task_prefix}:{task}", index, len(test_list),
-                                                generate_config.run_build_variant)
-            LOGGER.debug("Generating sub-task", sub_task=sub_task_name)
-
-            test_unix_style = test.replace('\\', '/')
-            run_tests_vars = {"resmoke_args": f"{resmoke_args} {resmoke_options} {test_unix_style}"}
-            if multiversion_path:
-                run_tests_vars["task_path_suffix"] = multiversion_path
-            timeout = _generate_timeouts(repeat_config, test, task_runtime_stats)
-            commands = resmoke_commands("run tests", run_tests_vars, timeout, multiversion_path)
-
-            task_list.add_task(sub_task_name, commands, ["compile"], distro)
-
-    existing_tasks = [BURN_IN_TESTS_GEN_TASK] if include_gen_task else None
-    task_list.add_to_variant(generate_config.run_build_variant, BURN_IN_TESTS_TASK, existing_tasks)
-    return evg_config
+    tasks = create_generated_tasks(tests_by_task, task_prefix, generate_config, repeat_config,
+                                   evg_api)
+    existing_tasks = {ExistingTask(BURN_IN_TESTS_GEN_TASK)} if include_gen_task else None
+    build_variant.display_task(BURN_IN_TESTS_TASK, tasks, execution_existing_tasks=existing_tasks)
 
 
 def create_task_list_for_tests(
@@ -575,19 +629,22 @@ def create_task_list_for_tests(
     return create_task_list(evg_conf, build_variant, tests_by_executor, exclude_tasks)
 
 
-def create_tests_by_task(build_variant: str, repo: Repo, evg_conf: EvergreenProjectConfig) -> Dict:
+def create_tests_by_task(build_variant: str, repos: Iterable[Repo],
+                         evg_conf: EvergreenProjectConfig) -> Dict:
     """
     Create a list of tests by task.
 
     :param build_variant: Build variant to collect tasks from.
-    :param repo: Git repo being tracked.
+    :param repos: Git repositories being tracked.
     :param evg_conf: Evergreen configuration.
     :return: Tests by task.
     """
-    changed_tests = find_changed_tests(repo)
+    changed_tests = find_changed_tests(repos)
+    LOGGER.debug("Found changed tests", files=changed_tests)
     exclude_suites, exclude_tasks, exclude_tests = find_excludes(SELECTOR_FILE)
     changed_tests = filter_tests(changed_tests, exclude_tests)
 
+    buildscripts.resmokelib.parser.set_run_options()
     if changed_tests:
         return create_task_list_for_tests(changed_tests, build_variant, evg_conf, exclude_suites,
                                           exclude_tasks)
@@ -598,30 +655,31 @@ def create_tests_by_task(build_variant: str, repo: Repo, evg_conf: EvergreenProj
 
 # pylint: disable=too-many-arguments
 def create_generate_tasks_file(tests_by_task: Dict, generate_config: GenerateConfig,
-                               repeat_config: RepeatConfig, task_prefix: str = 'burn_in',
-                               include_gen_task: bool = True) -> Dict:
+                               repeat_config: RepeatConfig, evg_api: Optional[EvergreenApi],
+                               task_prefix: str = 'burn_in', include_gen_task: bool = True) -> str:
     """
     Create an Evergreen generate.tasks file to run the given tasks and tests.
 
     :param tests_by_task: Dictionary of tests and tasks to run.
     :param generate_config: Information about how burn_in should generate tasks.
     :param repeat_config: Information about how burn_in should repeat tests.
+    :param evg_api: Evergreen api.
     :param task_prefix: Prefix to start generated task's name with.
     :param include_gen_task: Should the generating task be included in the display task.
     :returns: Configuration to pass to 'generate.tasks'.
     """
-    evg_config = Configuration()
-    evg_config = create_generate_tasks_config(evg_config, tests_by_task, generate_config,
-                                              repeat_config, include_gen_task=include_gen_task,
-                                              task_prefix=task_prefix)
+    build_variant = BuildVariant(generate_config.run_build_variant)
+    create_generate_tasks_config(build_variant, tests_by_task, generate_config, repeat_config,
+                                 evg_api, include_gen_task=include_gen_task,
+                                 task_prefix=task_prefix)
 
-    json_config = evg_config.to_map()
-    tasks_to_create = len(json_config.get('tasks', []))
-    if tasks_to_create > MAX_TASKS_TO_CREATE:
-        LOGGER.warning("Attempting to create more tasks than max, aborting", tasks=tasks_to_create,
-                       max=MAX_TASKS_TO_CREATE)
+    shrub_project = ShrubProject.empty()
+    shrub_project.add_build_variant(build_variant)
+
+    if not validate_task_generation_limit(shrub_project):
         sys.exit(1)
-    return json_config
+
+    return shrub_project.json()
 
 
 def run_tests(tests_by_task: Dict, resmoke_cmd: [str]):
@@ -662,8 +720,22 @@ def _configure_logging(verbose: bool):
         logging.getLogger(log_name).setLevel(logging.WARNING)
 
 
+def _get_evg_api(evg_api_config: str, local_mode: bool) -> Optional[EvergreenApi]:
+    """
+    Get an instance of the Evergreen Api.
+
+    :param evg_api_config: Config file with evg auth information.
+    :param local_mode: If true, do not connect to Evergreen API.
+    :return: Evergreen Api instance.
+    """
+    if not local_mode:
+        return RetryingEvergreenApi.get_api(config_file=evg_api_config)
+    return None
+
+
 def burn_in(repeat_config: RepeatConfig, generate_config: GenerateConfig, resmoke_args: str,
-            generate_tasks_file: str, no_exec: bool, evg_conf: EvergreenProjectConfig, repo: Repo):
+            generate_tasks_file: str, no_exec: bool, evg_conf: EvergreenProjectConfig,
+            repos: Iterable[Repo], evg_api: EvergreenApi) -> None:
     """
     Run burn_in_tests with the given configuration.
 
@@ -673,17 +745,19 @@ def burn_in(repeat_config: RepeatConfig, generate_config: GenerateConfig, resmok
     :param generate_tasks_file: File to write generated config to.
     :param no_exec: Do not execute tests, just discover tests to run.
     :param evg_conf: Evergreen configuration.
-    :param repo: Git repository.
+    :param repos: Git repositories to check.
+    :param evg_api: Evergreen API client.
     """
     # Populate the config values in order to use the helpers from resmokelib.suitesconfig.
     resmoke_cmd = _set_resmoke_cmd(repeat_config, list(resmoke_args))
 
-    tests_by_task = create_tests_by_task(generate_config.build_variant, repo, evg_conf)
+    tests_by_task = create_tests_by_task(generate_config.build_variant, repos, evg_conf)
     LOGGER.debug("tests and tasks found", tests_by_task=tests_by_task)
 
     if generate_tasks_file:
-        json_config = create_generate_tasks_file(tests_by_task, generate_config, repeat_config)
-        _write_json_file(json_config, generate_tasks_file)
+        json_text = create_generate_tasks_file(tests_by_task, generate_config, repeat_config,
+                                               evg_api)
+        write_file(generate_tasks_file, json_text)
     elif not no_exec:
         run_tests(tests_by_task, resmoke_cmd)
     else:
@@ -711,12 +785,18 @@ def burn_in(repeat_config: RepeatConfig, generate_config: GenerateConfig, resmok
               help="The maximum number of times to repeat tests if time option is specified.")
 @click.option("--repeat-tests-secs", "repeat_tests_secs", default=None, type=int, metavar="SECONDS",
               help="Repeat tests for the given time (in secs).")
+@click.option("--evg-api-config", "evg_api_config", default=CONFIG_FILE, metavar="FILE",
+              help="Configuration file with connection info for Evergreen API.")
+@click.option("--local", "local_mode", default=False, is_flag=True,
+              help="Local mode. Do not call out to evergreen api.")
 @click.option("--verbose", "verbose", default=False, is_flag=True, help="Enable extra logging.")
+@click.option("--task_id", "task_id", default=None, metavar='TASK_ID',
+              help="The evergreen task id.")
 @click.argument("resmoke_args", nargs=-1, type=click.UNPROCESSED)
 # pylint: disable=too-many-arguments,too-many-locals
 def main(build_variant, run_build_variant, distro, project, generate_tasks_file, no_exec,
          repeat_tests_num, repeat_tests_min, repeat_tests_max, repeat_tests_secs, resmoke_args,
-         verbose):
+         local_mode, evg_api_config, verbose, task_id):
     """
     Run new or changed tests in repeated mode to validate their stability.
 
@@ -754,6 +834,8 @@ def main(build_variant, run_build_variant, distro, project, generate_tasks_file,
     :param repeat_tests_max: Once this number of repetitions has been reached, stop repeating.
     :param repeat_tests_secs: Continue repeating tests for this number of seconds.
     :param resmoke_args: Arguments to pass through to resmoke.
+    :param local_mode: Don't call out to the evergreen API (used for testing).
+    :param evg_api_config: Location of configuration file to connect to evergreen.
     :param verbose: Log extra debug information.
     """
     _configure_logging(verbose)
@@ -763,17 +845,20 @@ def main(build_variant, run_build_variant, distro, project, generate_tasks_file,
                                  repeat_tests_min=repeat_tests_min,
                                  repeat_tests_max=repeat_tests_max,
                                  repeat_tests_num=repeat_tests_num)  # yapf: disable
-    repeat_config.validate()
     generate_config = GenerateConfig(build_variant=build_variant,
                                      run_build_variant=run_build_variant,
                                      distro=distro,
-                                     project=project)  # yapf: disable
-    generate_config.validate(evg_conf)
+                                     project=project,
+                                     task_id=task_id)  # yapf: disable
+    if generate_tasks_file:
+        generate_config.validate(evg_conf)
 
-    repo = Repo(".")
+    evg_api = _get_evg_api(evg_api_config, local_mode)
+
+    repos = [Repo(x) for x in DEFAULT_REPO_LOCATIONS if os.path.isdir(x)]
 
     burn_in(repeat_config, generate_config, resmoke_args, generate_tasks_file, no_exec, evg_conf,
-            repo)
+            repos, evg_api)
 
 
 if __name__ == "__main__":
