@@ -1,7 +1,5 @@
 ############################################################
-# This section verifies start, stop, and restart after
-# installation within a new EC2 instance spun up by Kitchen.
-#
+# This section verifies start, stop, and restart.
 # - stop mongod so that we begin testing from a stopped state
 # - verify start, stop, and restart
 ############################################################
@@ -57,34 +55,15 @@ else
   end
 end
 
-if os[:arch] == 'x86_64'
-  # install_compass does not run Amazon Linux but *does* run on Amazon Linux 2,
-  # but the 'redhat' family includes both, apparently. We need to specifically
-  # exclude Amazon Linux from the set of allowed distributions here because the
-  # version strings would otherwise pass it.
-  if ((os[:family] == 'redhat' and os[:name] != "amazon" and os[:release].split('.')[0].to_i >= 7) or
-      (os[:name] == 'ubuntu' and os[:release].split('.')[0].to_i >= 16) or
-      (os[:name] == 'debian' and os[:release].split('.')[0].to_i >= 9) or
-      (os[:name] == 'amazon' and os[:release].split('.')[0].to_i == 2))
-    describe command("install_compass") do
-      its('exit_status') { should eq 0 }
-      its('stderr') { should eq '' }
-    end
-  elsif os[:name] == 'suse'
-    describe command("install_compass") do
-      its('exit_status') { should eq 1 }
-      its('stderr') { should match /You are using an unsupported platform/ }
-    end
-  else
-    describe command("install_compass") do
-      its('exit_status') { should eq 1 }
-      its('stderr') { should match /You are using an unsupported Linux distribution/ }
-    end
+if os[:arch] == 'x86_64' and os[:name] != 'amazon' and
+  ((os[:name] == 'ubuntu' and os[:release].split('.')[0].to_i > 12) or 
+    (os[:family] == 'redhat' and os[:release].split('.')[0].to_i >= 7))
+  describe command("install_compass") do
+    its('exit_status') { should eq 0 }
   end
 else
   describe command("install_compass") do
-    its('exit_status') { should eq 1 }
-    its('stderr') { should match /Sorry, MongoDB Compass is only supported on 64-bit Intel platforms/ }
+    its('exit_status') { should_not eq 0 }
   end
 end
 
@@ -137,14 +116,7 @@ if sysvinit
 end
 
 if systemd
-  unit_file_prefix = ''
-  if os[:name] == 'suse'
-    # Putting systemd unit files in /usr, which may be a separate partition
-    # and therefore not available during isolated startups, is bad practice.
-    # But it's what SUSE has chosen to do, so we have to deal with it.
-    unit_file_prefix = '/usr'
-  end
-  describe file("#{unit_file_prefix}/lib/systemd/system/mongod.service") do
+  describe file('/lib/systemd/system/mongod.service') do
     it { should be_file }
   end
 end
@@ -171,6 +143,7 @@ if deb
   describe file('/var/lib/mongodb') do
     it { should be_directory }
   end
+
   describe user('mongodb') do
     it { should exist }
     its('groups') { should include 'mongodb' }
@@ -178,7 +151,8 @@ if deb
     # account shells
     its('shell') {
       if ((os[:name] == 'debian' and os[:release].split('.')[0] == '10') or
-          (os[:name] == 'ubuntu' and os[:release] == '18.04'))
+          (os[:name] == 'ubuntu' and os[:release] == '18.04') or
+          (os[:name] == 'ubuntu' and os[:release] == '20.04'))
         should eq '/usr/sbin/nologin'
       else
         should eq '/bin/false'
