@@ -14,6 +14,7 @@ from ... import core
 from ... import errors
 from ... import utils
 from ...utils import registry
+from ...multiversionconstants import LAST_STABLE_MONGOS_BINARY
 
 
 class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-instance-attributes
@@ -23,11 +24,10 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
     _SHARD_REPLSET_NAME_PREFIX = "shard-rs"
 
     def __init__(  # pylint: disable=too-many-arguments,too-many-locals
-            self, logger, job_num, mongos_executable=None, mongos_options=None,
-            mongod_executable=None, mongod_options=None, dbpath_prefix=None, preserve_dbpath=False,
-            num_shards=1, num_rs_nodes_per_shard=None, num_mongos=1, enable_sharding=None,
-            enable_balancer=True, enable_autosplit=True, auth_options=None, configsvr_options=None,
-            shard_options=None):
+            self, logger, job_num, mongos_executable=None, mongos_options=None, mongod_options=None,
+            dbpath_prefix=None, preserve_dbpath=False, num_shards=1, num_rs_nodes_per_shard=1,
+            num_mongos=1, enable_sharding=None, enable_balancer=True, enable_autosplit=True,
+            auth_options=None, configsvr_options=None, shard_options=None, mixed_bin_versions=None):
         """Initialize ShardedClusterFixture with different options for the cluster processes."""
 
         interface.Fixture.__init__(self, logger, job_num, dbpath_prefix=dbpath_prefix)
@@ -37,14 +37,16 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
 
         self.mongos_executable = mongos_executable
         self.mongos_options = utils.default_if_none(mongos_options, {})
-        self.mongod_executable = mongod_executable
         self.mongod_options = utils.default_if_none(mongod_options, {})
         self.mongod_options["set_parameters"] = mongod_options.get("set_parameters", {}).copy()
         self.mongod_options["set_parameters"]["migrationLockAcquisitionMaxWaitMS"] = \
                 mongod_options["set_parameters"].get("migrationLockAcquisitionMaxWaitMS", 30000)
         self.preserve_dbpath = preserve_dbpath
-        self.num_shards = num_shards
-        self.num_rs_nodes_per_shard = num_rs_nodes_per_shard
+        # Use 'num_shards' and 'num_rs_nodes_per_shard' values from the command line if they exist.
+        num_shards_option = config.NUM_SHARDS
+        self.num_shards = num_shards if not num_shards_option else num_shards_option
+        num_rs_nodes_per_shard_option = config.NUM_REPLSET_NODES
+        self.num_rs_nodes_per_shard = num_rs_nodes_per_shard if not num_rs_nodes_per_shard_option else num_rs_nodes_per_shard_option
         self.num_mongos = num_mongos
         self.enable_sharding = utils.default_if_none(enable_sharding, [])
         self.enable_balancer = enable_balancer
@@ -52,6 +54,14 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         self.auth_options = auth_options
         self.configsvr_options = utils.default_if_none(configsvr_options, {})
         self.shard_options = utils.default_if_none(shard_options, {})
+        self.mixed_bin_versions = utils.default_if_none(mixed_bin_versions,
+                                                        config.MIXED_BIN_VERSIONS)
+        if self.mixed_bin_versions is not None and num_rs_nodes_per_shard is not None:
+            num_mongods = self.num_shards * self.num_rs_nodes_per_shard
+            if len(self.mixed_bin_versions) != num_mongods:
+                msg = (("The number of binary versions specified: {} do not match the number of"\
+                        " nodes in the sharded cluster: {}.")).format(len(self.mixed_bin_versions), num_mongods)
+                raise errors.ServerFailure(msg)
 
         self._dbpath_prefix = os.path.join(self._dbpath_prefix, config.FIXTURE_SUBDIR)
 
@@ -59,6 +69,21 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         self.mongos = []
         self.shards = []
 
+    def pids(self):
+        """:return: pids owned by this fixture if any."""
+        out = []
+        if self.configsvr is not None:
+            out.extend(self.configsvr.pids())
+        else:
+            self.logger.debug(
+                'Config server not running when gathering sharded cluster fixture pids.')
+        if self.shards is not None:
+            for shard in self.shards:
+                out.extend(shard.pids())
+        else:
+            self.logger.debug('No shards when gathering sharded cluster fixture pids.')
+        return out
+
     def setup(self):
         """Set up the sharded cluster."""
         if self.configsvr is None:
@@ -82,19 +107,6 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         for shard in self.shards:
             shard.setup()
 
-    def refresh_logical_session_cache(self, target):
-        """Refresh logical session cache with no timeout."""
-        primary = (target.mongo_client()
-                   if self.num_rs_nodes_per_shard is None else target.get_primary().mongo_client())
-        try:
-            primary.admin.command({"refreshLogicalSessionCacheNow": 1})
-        except pymongo.errors.OperationFailure as err:
-            if err.code != self._WRITE_CONCERN_FAILED:
-                raise err
-            self.logger.info("Ignoring write concern timeout for refreshLogicalSessionCacheNow "
-                             "command and continuing to wait")
-            target.await_last_op_committed(target.AWAIT_REPL_TIMEOUT_FOREVER_MINS * 60)
-
     def await_ready(self):
         """Block until the fixture can be used for testing."""
         # Wait for the config server
@@ -150,10 +162,13 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
 
         # Ensure that the sessions collection gets auto-sharded by the config server
         if self.configsvr is not None:
-            self.refresh_logical_session_cache(self.configsvr)
+            primary = self.configsvr.get_primary().mongo_client()
+            primary.admin.command({"refreshLogicalSessionCacheNow": 1})
 
         for shard in self.shards:
-            self.refresh_logical_session_cache(shard)
+            primary = (shard.mongo_client() if self.num_rs_nodes_per_shard is None else
+                       shard.get_primary().mongo_client())
+            primary.admin.command({"refreshLogicalSessionCacheNow": 1})
 
     def _auth_to_db(self, client):
         """Authenticate client for the 'authenticationDatabase'."""
@@ -177,7 +192,7 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         client.admin.command({"balancerStart": 1}, maxTimeMS=timeout_ms)
         self.logger.info("Started the balancer")
 
-    def _do_teardown(self):
+    def _do_teardown(self, mode=None):
         """Shut down the sharded cluster."""
         self.logger.info("Stopping all members of the sharded cluster...")
 
@@ -186,19 +201,22 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
             self.logger.warning("All members of the sharded cluster were expected to be running, "
                                 "but weren't.")
 
-        if self.enable_balancer:
+        # If we're killing or aborting to archive data files, stopping the balancer will execute
+        # server commands that might lead to on-disk changes from the point of failure.
+        if self.enable_balancer and mode not in (interface.TeardownMode.KILL,
+                                                 interface.TeardownMode.ABORT):
             self.stop_balancer()
 
         teardown_handler = interface.FixtureTeardownHandler(self.logger)
 
         if self.configsvr is not None:
-            teardown_handler.teardown(self.configsvr, "config server")
+            teardown_handler.teardown(self.configsvr, "config server", mode=mode)
 
         for mongos in self.mongos:
-            teardown_handler.teardown(mongos, "mongos")
+            teardown_handler.teardown(mongos, "mongos", mode=mode)
 
         for shard in self.shards:
-            teardown_handler.teardown(shard, "shard")
+            teardown_handler.teardown(shard, "shard", mode=mode)
 
         if teardown_handler.was_successful():
             self.logger.info("Successfully stopped all members of the sharded cluster.")
@@ -231,7 +249,6 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         configsvr_options = self.configsvr_options.copy()
 
         auth_options = configsvr_options.pop("auth_options", self.auth_options)
-        mongod_executable = configsvr_options.pop("mongod_executable", self.mongod_executable)
         preserve_dbpath = configsvr_options.pop("preserve_dbpath", self.preserve_dbpath)
         num_nodes = configsvr_options.pop("num_nodes", 1)
 
@@ -246,9 +263,9 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         mongod_options["storageEngine"] = "wiredTiger"
 
         return replicaset.ReplicaSetFixture(
-            mongod_logger, self.job_num, mongod_executable=mongod_executable,
-            mongod_options=mongod_options, preserve_dbpath=preserve_dbpath, num_nodes=num_nodes,
-            auth_options=auth_options, replset_config_options=replset_config_options,
+            mongod_logger, self.job_num, mongod_options=mongod_options,
+            preserve_dbpath=preserve_dbpath, num_nodes=num_nodes, auth_options=auth_options,
+            mixed_bin_versions=None, replset_config_options=replset_config_options,
             **configsvr_options)
 
     def _new_rs_shard(self, index, num_rs_nodes_per_shard):
@@ -259,12 +276,17 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         shard_options = self.shard_options.copy()
 
         auth_options = shard_options.pop("auth_options", self.auth_options)
-        mongod_executable = shard_options.pop("mongod_executable", self.mongod_executable)
         preserve_dbpath = shard_options.pop("preserve_dbpath", self.preserve_dbpath)
 
         replset_config_options = shard_options.pop("replset_config_options", {})
         replset_config_options["configsvr"] = False
 
+        mixed_bin_versions = self.mixed_bin_versions
+        if mixed_bin_versions is not None:
+            start_index = index * num_rs_nodes_per_shard
+            mixed_bin_versions = mixed_bin_versions[start_index:start_index +
+                                                    num_rs_nodes_per_shard]
+
         mongod_options = self.mongod_options.copy()
         mongod_options.update(shard_options.pop("mongod_options", {}))
         mongod_options["shardsvr"] = ""
@@ -272,10 +294,10 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         mongod_options["replSet"] = ShardedClusterFixture._SHARD_REPLSET_NAME_PREFIX + str(index)
 
         return replicaset.ReplicaSetFixture(
-            mongod_logger, self.job_num, mongod_executable=mongod_executable,
-            mongod_options=mongod_options, preserve_dbpath=preserve_dbpath,
-            num_nodes=num_rs_nodes_per_shard, auth_options=auth_options,
-            replset_config_options=replset_config_options, **shard_options)
+            mongod_logger, self.job_num, mongod_options=mongod_options,
+            preserve_dbpath=preserve_dbpath, num_nodes=num_rs_nodes_per_shard,
+            auth_options=auth_options, replset_config_options=replset_config_options,
+            mixed_bin_versions=mixed_bin_versions, **shard_options)
 
     def _new_standalone_shard(self, index):
         """Return a standalone.MongoDFixture configured as a shard in a sharded cluster."""
@@ -284,7 +306,6 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
 
         shard_options = self.shard_options.copy()
 
-        mongod_executable = shard_options.pop("mongod_executable", self.mongod_executable)
         preserve_dbpath = shard_options.pop("preserve_dbpath", self.preserve_dbpath)
 
         mongod_options = self.mongod_options.copy()
@@ -292,9 +313,8 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         mongod_options["shardsvr"] = ""
         mongod_options["dbpath"] = os.path.join(self._dbpath_prefix, "shard{}".format(index))
 
-        return standalone.MongoDFixture(
-            mongod_logger, self.job_num, mongod_executable=mongod_executable,
-            mongod_options=mongod_options, preserve_dbpath=preserve_dbpath, **shard_options)
+        return standalone.MongoDFixture(mongod_logger, self.job_num, mongod_options=mongod_options,
+                                        preserve_dbpath=preserve_dbpath, **shard_options)
 
     def _new_mongos(self, index, total):
         """
@@ -315,8 +335,12 @@ class ShardedClusterFixture(interface.Fixture):  # pylint: disable=too-many-inst
         mongos_options = self.mongos_options.copy()
         mongos_options["configdb"] = self.configsvr.get_internal_connection_string()
 
-        return _MongoSFixture(mongos_logger, self.job_num, mongos_executable=self.mongos_executable,
-                              mongos_options=mongos_options)
+        # The last-stable binary is currently expected to live in '/data/multiversion', which is
+        # part of the PATH.
+        mongos_executable = self.mongos_executable if self.mixed_bin_versions is None else LAST_STABLE_MONGOS_BINARY
+
+        return _MongoSFixture(mongos_logger, self.job_num, dbpath_prefix=self._dbpath_prefix,
+                              mongos_executable=mongos_executable, mongos_options=mongos_options)
 
     def _add_shard(self, client, shard):
         """
@@ -335,18 +359,20 @@ class _MongoSFixture(interface.Fixture):
 
     REGISTERED_NAME = registry.LEAVE_UNREGISTERED  # type: ignore
 
-    def __init__(self, logger, job_num, mongos_executable=None, mongos_options=None):
+    # pylint: disable=too-many-arguments
+    def __init__(self, logger, job_num, dbpath_prefix, mongos_executable=None, mongos_options=None):
         """Initialize _MongoSFixture."""
 
         interface.Fixture.__init__(self, logger, job_num)
 
-        # Command line options override the YAML configuration.
-        self.mongos_executable = utils.default_if_none(config.MONGOS_EXECUTABLE, mongos_executable)
+        # Default to command line options if the YAML configuration is not passed in.
+        self.mongos_executable = utils.default_if_none(mongos_executable, config.MONGOS_EXECUTABLE)
 
         self.mongos_options = utils.default_if_none(mongos_options, {}).copy()
 
         self.mongos = None
         self.port = None
+        self._dbpath_prefix = dbpath_prefix
 
     def setup(self):
         """Set up the sharded cluster."""
@@ -354,6 +380,11 @@ class _MongoSFixture(interface.Fixture):
             self.mongos_options["port"] = core.network.PortAllocator.next_fixture_port(self.job_num)
         self.port = self.mongos_options["port"]
 
+        if config.ALWAYS_USE_LOG_FILES:
+            self.mongos_options["logpath"] = self._dbpath_prefix + "/mongos-{port}.log".format(
+                port=self.port)
+            self.mongos_options["logappend"] = ""
+
         mongos = core.programs.mongos_program(self.logger, executable=self.mongos_executable,
                                               **self.mongos_options)
         try:
@@ -367,6 +398,14 @@ class _MongoSFixture(interface.Fixture):
 
         self.mongos = mongos
 
+    def pids(self):
+        """:return: pids owned by this fixture if any."""
+        if self.mongos is not None:
+            return [self.mongos.pid]
+        else:
+            self.logger.debug('Mongos not running when gathering mongos fixture pids.')
+        return []
+
     def await_ready(self):
         """Block until the fixture can be used for testing."""
         deadline = time.time() + standalone.MongoDFixture.AWAIT_READY_TIMEOUT_SECS
@@ -387,24 +426,30 @@ class _MongoSFixture(interface.Fixture):
                 client = self.mongo_client(timeout_millis=500)
                 client.admin.command("ping")
                 break
-            except pymongo.errors.ConnectionFailure as exc:
+            except pymongo.errors.ConnectionFailure:
                 remaining = deadline - time.time()
                 if remaining <= 0.0:
                     raise errors.ServerFailure(
                         "Failed to connect to mongos on port {} after {} seconds".format(
-                            self.port, standalone.MongoDFixture.AWAIT_READY_TIMEOUT_SECS)) from exc
+                            self.port, standalone.MongoDFixture.AWAIT_READY_TIMEOUT_SECS))
 
                 self.logger.info("Waiting to connect to mongos on port %d.", self.port)
                 time.sleep(0.1)  # Wait a little bit before trying again.
 
         self.logger.info("Successfully contacted the mongos on port %d.", self.port)
 
-    def _do_teardown(self):
+    def _do_teardown(self, mode=None):
         if self.mongos is None:
             self.logger.warning("The mongos fixture has not been set up yet.")
             return  # Teardown is still a success even if nothing is running.
 
-        self.logger.info("Stopping mongos on port %d with pid %d...", self.port, self.mongos.pid)
+        if mode == interface.TeardownMode.ABORT:
+            self.logger.info(
+                "Attempting to send SIGABRT from resmoke to mongos on port %d with pid %d...",
+                self.port, self.mongos.pid)
+        else:
+            self.logger.info("Stopping mongos on port %d with pid %d...", self.port,
+                             self.mongos.pid)
         if not self.is_running():
             exit_code = self.mongos.poll()
             msg = ("mongos on port {:d} was expected to be running, but wasn't. "
@@ -412,10 +457,12 @@ class _MongoSFixture(interface.Fixture):
             self.logger.warning(msg)
             raise errors.ServerFailure(msg)
 
-        self.mongos.stop()
+        self.mongos.stop(mode=mode)
         exit_code = self.mongos.wait()
 
-        if exit_code == 0:
+        # Python's subprocess module returns negative versions of system calls.
+        # pylint: disable=invalid-unary-operand-type
+        if exit_code == 0 or (mode is not None and exit_code == -(mode.value)):
             self.logger.info("Successfully stopped the mongos on port {:d}".format(self.port))
         else:
             self.logger.warning("Stopped the mongos on port {:d}. "
