@@ -6,9 +6,12 @@
  * distinct field exactly once among matching documents and also provides any requested sort. The
  * test queries below show most $match/$sort/$group combinations where that is possible.
  *
- * The sharding and $facet passthrough suites modifiy aggregation pipelines in a way that prevents
- * the DISTINCT_SCAN optimization from being applied, which breaks the test.
- * @tags: [assumes_unsharded_collection, do_not_wrap_aggregations_in_facets]
+ * @tags: [
+ * # The sharding and $facet passthrough suites modifiy aggregation pipelines in a way that
+ * # prevents the DISTINCT_SCAN optimization from being applied, which breaks the test.
+ * assumes_unsharded_collection, do_not_wrap_aggregations_in_facets,
+ * # Index filter commands do not support causal consistency.
+ * does_not_support_causal_consistency]
  */
 
 (function() {
@@ -300,18 +303,32 @@ explain = coll.explain().aggregate(pipeline);
 assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
 
 //
-// Verify that we _do not_ attempt a DISTINCT_SCAN to satisfy a sort on a multikey field, even
-// when the field we are grouping by is not multikey.
+// Verify that we use a DISTINCT_SCAN to satisfy a sort on a multikey field if the bounds
+// are [minKey, maxKey].
 //
 pipeline = [{$sort: {aa: 1, mkB: 1}}, {$group: {_id: "$aa", accum: {$first: "$mkB"}}}];
 assertResultsMatchWithAndWithoutHintandIndexes(
     pipeline, [{_id: null, accum: null}, {_id: 1, accum: [1, 3]}, {_id: 2, accum: []}]);
 explain = coll.explain().aggregate(pipeline);
-assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), tojson(explain));
+assert.neq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
+assert.eq({aa: 1, mkB: 1, c: 1}, getAggPlanStage(explain, "DISTINCT_SCAN").keyPattern);
+assert.neq(null, getAggPlanStage(explain, "FETCH"));
+assert.eq(null, getAggPlanStage(explain, "SORT"));
 
 //
-// Verify that with dotted paths we _do not_ attempt a DISTINCT_SCAN to satisfy a sort on a
-// multikey field, even when the field we are grouping by is not multikey.
+// Verify that we _do not_ attempt a DISTINCT_SCAN because "mkB" is multikey, and we don't use
+// DISTINCT_SCAN for a compound group key.
+//
+pipeline = [{$sort: {aa: 1, mkB: 1}}, {$group: {_id: {aa: "$aa", mkB: "$mkB"}}}];
+assertResultsMatchWithAndWithoutHintandIndexes(
+    pipeline,
+    [{_id: {aa: 1, mkB: [1, 3]}}, {_id: {}}, {_id: {aa: 1, mkB: 2}}, {_id: {aa: 2, mkB: []}}]);
+explain = coll.explain().aggregate(pipeline);
+assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
+
+//
+// Verify that with dotted paths we use a DISTINCT_SCAN to satisfy a sort on a multikey field if the
+// bounds are [minKey, maxKey].
 //
 pipeline =
     [{$sort: {"foo.a": 1, "mkFoo.b": 1}}, {$group: {_id: "$foo.a", accum: {$first: "$mkFoo.b"}}}];
@@ -319,6 +336,58 @@ assertResultsMatchWithAndWithoutHintandIndexes(
     pipeline,
     [{_id: null, accum: null}, {_id: 1, accum: 1}, {_id: 2, accum: 2}, {_id: 3, accum: [4, 3]}]);
 explain = coll.explain().aggregate(pipeline);
+assert.neq(null, getAggPlanStage(explain, "DISTINCT_SCAN"));
+assert.eq(
+    {"foo.a": 1, "mkFoo.b": 1}, getAggPlanStage(explain, "DISTINCT_SCAN").keyPattern, explain);
+assert.neq(null, getAggPlanStage(explain, "FETCH"));
+assert.eq(null, getAggPlanStage(explain, "SORT"));
+
+//
+// Verify that we _do not_ attempt a DISTINCT_SCAN to satisfy a sort on a multikey field if
+// the bounds are not [minKey, maxKey].
+//
+pipeline = [
+    {$match: {mkB: {$ne: 9999}}},
+    {$sort: {aa: 1, mkB: 1}},
+    {$group: {_id: "$aa", accum: {$first: "$mkB"}}}
+];
+assertResultsMatchWithAndWithoutHintandIndexes(
+    pipeline, [{_id: 1, accum: [1, 3]}, {_id: null, accum: null}, {_id: 2, accum: []}]);
+explain = coll.explain().aggregate(pipeline);
+assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
+
+pipeline = [
+    {$match: {mkB: {$gt: -5}}},
+    {$sort: {aa: 1, mkB: 1}},
+    {$group: {_id: "$aa", accum: {$first: "$mkB"}}}
+];
+assertResultsMatchWithAndWithoutHintandIndexes(pipeline, [{_id: 1, accum: [1, 3]}]);
+explain = coll.explain().aggregate(pipeline);
+assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
+
+pipeline = [
+    {$match: {aa: {$ne: 9999}}},
+    {$sort: {aa: 1, mkB: 1}},
+    {$group: {_id: "$aa", accum: {$first: "$mkB"}}}
+];
+assertResultsMatchWithAndWithoutHintandIndexes(
+    pipeline, [{_id: 1, accum: [1, 3]}, {_id: null, accum: null}, {_id: 2, accum: []}]);
+explain = coll.explain().aggregate(pipeline);
+assert.neq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
+
+//
+// Verify that with dotted paths we _do not_ attempt a DISTINCT_SCAN to satisfy a sort on a
+// multikey field if the bounds are not [minKey, maxKey].
+//
+pipeline = [
+    {$match: {"mkFoo.b": {$ne: 20000}}},
+    {$sort: {"foo.a": 1, "mkFoo.b": 1}},
+    {$group: {_id: "$foo.a", accum: {$first: "$mkFoo.b"}}}
+];
+assertResultsMatchWithAndWithoutHintandIndexes(
+    pipeline,
+    [{_id: null, accum: null}, {_id: 1, accum: 1}, {_id: 2, accum: 2}, {_id: 3, accum: [4, 3]}]);
+explain = coll.explain().aggregate(pipeline);
 assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
 
 //
@@ -474,18 +543,6 @@ assertResultsMatchWithAndWithoutHintandIndexes(pipeline, [{_id: 1}, {_id: 2}, {_
 explain = coll.explain().aggregate(pipeline);
 assert.eq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
 
-//
-// Verify that a $sort-$group pipeline with a $first accumulator can use DISTINCT_SCAN, even when
-// the group _id field is a singleton object instead of a fieldPath.
-//
-pipeline = [{$sort: {a: 1, b: 1}}, {$group: {_id: {v: "$a"}, accum: {$first: "$b"}}}];
-assertResultsMatchWithAndWithoutHintandIndexes(
-    pipeline, [{_id: {v: null}, accum: null}, {_id: {v: 1}, accum: 1}, {_id: {v: 2}, accum: 2}]);
-explain = coll.explain().aggregate(pipeline);
-assert.neq(null, getAggPlanStage(explain, "DISTINCT_SCAN"), explain);
-assert.eq({a: 1, b: 1, c: 1}, getAggPlanStage(explain, "DISTINCT_SCAN").keyPattern);
-assert.eq(null, getAggPlanStage(explain, "SORT"), explain);
-
 ////////////////////////////////////////////////////////////////////////////////////////////////
 // We execute all the collation-related tests three times with three different configurations
 // (no index, index without collation, index with collation).
