@@ -13,13 +13,15 @@ auditTest(
     function(m) {
         testDB = m.getDB(testDBName);
         assert.commandWorked(testDB.dropDatabase());
+        const beforeCmd = Date.now();
         assert.commandWorked(testDB.createCollection('foo'));
 
-        auditColl = getAuditEventsCollection(m);
+        const beforeLoad = Date.now();
+        auditColl = getAuditEventsCollection(m, testDBName);
         assert.eq(1, auditColl.count({
             atype: "createDatabase",
-            ts: withinTheLastFewSeconds(),
-            'params.ns': testDBName,
+            ts: withinInterval(beforeCmd, beforeLoad),
+            'param.ns': testDBName,
             result: 0,
         }), "FAILED, audit log: " + tojson(auditColl.find().toArray()));
     },
