@@ -12,7 +12,7 @@
  * Unfortunately, there aren't very many assertions we can make here, so this is mostly to test that
  * the server doesn't deadlock or crash.
  *
- * @tags: [requires_capped]
+ * @tags: [requires_capped, requires_fcv_44]
  */
 load('jstests/concurrency/fsm_libs/extend_workload.js');           // for extendWorkload
 load('jstests/concurrency/fsm_workloads/agg_base.js');             // for $config
@@ -65,10 +65,11 @@ var $config = extendWorkload($config, function($config, $super) {
         });
 
         const allowedErrorCodes = [
-            ErrorCodes.CommandFailed,  // indexes of target collection changed during processing.
-            17017,  // $out is not supported to an existing *sharded* output collection.
-            17152,  // namespace is capped so it can't be used for $out.
-            28769,  // $out collection cannot be sharded.
+            ErrorCodes.CommandFailed,     // indexes of target collection changed during processing.
+            ErrorCodes.IllegalOperation,  // $out is not supported to an existing *sharded* output
+                                          // collection.
+            17152,                        // namespace is capped so it can't be used for $out.
+            28769,                        // $out collection cannot be sharded.
         ];
         assertWhenOwnDB.commandWorkedOrFailedWithCode(res, allowedErrorCodes);
 
@@ -132,12 +133,10 @@ var $config = extendWorkload($config, function($config, $super) {
 
     /**
      * If being run against a mongos, shards '$config.data.outputCollName'. This is never undone,
-     * and all subsequent $out's to this collection should fail. Collection sharding is restricted
-     * to a single thread as multiple concurrent invocations can result in command timeout /
-     * failure.
+     * and all subsequent $out's to this collection should fail.
      */
     $config.states.shardCollection = function shardCollection(db, unusedCollName) {
-        if (isMongos(db) && this.tid === 0) {
+        if (isMongos(db)) {
             assertWhenOwnDB.commandWorked(db.adminCommand({enableSharding: db.getName()}));
             assertWhenOwnDB.commandWorked(db.adminCommand(
                 {shardCollection: db[this.outputCollName].getFullName(), key: {_id: 'hashed'}}));
