@@ -10,19 +10,6 @@ coll.drop();
 coll.getDB().system.profile.drop();
 assert.commandWorked(coll.getDB().runCommand({profile: 2}));
 
-/**
- * Asserts that array 'results' contains a profiler generated document that corresponds to a
- * truncated command that matches a regular expression 'truncatedCommandRegexp'. Outputs a message
- * 'message' in case such document is not present.
- */
-function assertContainsTruncatedCommand(results, truncatedCommandRegexp, message) {
-    const document = results.find(
-        element => element.hasOwnProperty('ns') && element.hasOwnProperty('millis') &&
-            element.hasOwnProperty('command') && 'string' === typeof (element.command.$truncated) &&
-            element.command.$truncated.match(truncatedCommandRegexp));
-    assert(document, message + ` Retrieved documents: ${tojson(results)}`);
-}
-
 var str = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa";
 var hugeStr = str;
 while (hugeStr.length < 2 * 1024 * 1024) {
@@ -32,33 +19,48 @@ while (hugeStr.length < 2 * 1024 * 1024) {
 // Test query with large string element.
 coll.find({a: hugeStr}).itcount();
 var results = coll.getDB().system.profile.find().toArray();
-assertContainsTruncatedCommand(results,
-                               /filter: { a: "a+\.\.\." }//* string value is truncated*/,
-                               "Document corresponding to 'find' command not found.");
+assert.eq(1, results.length);
+var result = results[0];
+assert(result.hasOwnProperty('ns'));
+assert(result.hasOwnProperty('millis'));
+assert(result.hasOwnProperty('command'));
+assert.eq('string', typeof (result.command.$truncated));
+// String value is truncated.
+assert(result.command.$truncated.match(/filter: { a: "a+\.\.\." }/));
 
 assert.commandWorked(coll.getDB().runCommand({profile: 0}));
 coll.getDB().system.profile.drop();
 assert.commandWorked(coll.getDB().runCommand({profile: 2}));
 
 // Test update with large string element in query portion.
-assert.writeOK(coll.update({a: hugeStr}, {}));
+assert.commandWorked(coll.update({a: hugeStr}, {}));
 var results = coll.getDB().system.profile.find().toArray();
-assertContainsTruncatedCommand(
-    results,
-    /^{ q: { a: "a+\.\.\." }, u: {}, multi: false, upsert: false }$//* string value is truncated*/,
-    "Document corresponding to 'update' command not found.");
+assert.eq(1, results.length);
+var result = results[0];
+assert(result.hasOwnProperty('ns'));
+assert(result.hasOwnProperty('millis'));
+assert(result.hasOwnProperty('command'));
+assert.eq('string', typeof (result.command.$truncated));
+// String value is truncated.
+assert(result.command.$truncated.match(
+    /^{ q: { a: "a+\.\.\." }, u: {}, multi: false, upsert: false }$/));
 
 assert.commandWorked(coll.getDB().runCommand({profile: 0}));
 coll.getDB().system.profile.drop();
 assert.commandWorked(coll.getDB().runCommand({profile: 2}));
 
 // Test update with large string element in update portion.
-assert.writeOK(coll.update({}, {a: hugeStr}));
+assert.commandWorked(coll.update({}, {a: hugeStr}));
 var results = coll.getDB().system.profile.find().toArray();
-assertContainsTruncatedCommand(
-    results,
-    /^{ q: {}, u: { a: "a+\.\.\." }, multi: false, upsert: false }$//* string value is truncated*/,
-    "Document corresponding to 'update' command not found.");
+assert.eq(1, results.length);
+var result = results[0];
+assert(result.hasOwnProperty('ns'));
+assert(result.hasOwnProperty('millis'));
+assert(result.hasOwnProperty('command'));
+assert.eq('string', typeof (result.command.$truncated));
+// String value is truncated.
+assert(result.command.$truncated.match(
+    /^{ q: {}, u: { a: "a+\.\.\." }, multi: false, upsert: false }$/));
 
 assert.commandWorked(coll.getDB().runCommand({profile: 0}));
 coll.getDB().system.profile.drop();
@@ -71,9 +73,13 @@ for (var i = 0; i < 100 * 1000; ++i) {
 }
 coll.find(doc).itcount();
 var results = coll.getDB().system.profile.find().toArray();
-assertContainsTruncatedCommand(
-    results,
-    /filter: { a0: 1\.0, a1: .*\.\.\.$//* query object itself is truncated*/,
-    "Document corresponding to 'find' command not found.");
+assert.eq(1, results.length);
+var result = results[0];
+assert(result.hasOwnProperty('ns'));
+assert(result.hasOwnProperty('millis'));
+assert(result.hasOwnProperty('command'));
+assert.eq('string', typeof (result.command.$truncated));
+// Query object itself is truncated.
+assert(result.command.$truncated.match(/filter: { a0: 1\.0, a1: .*\.\.\.$/));
 
 assert.commandWorked(coll.getDB().runCommand({profile: 0}));
