@@ -2,11 +2,18 @@
 //   assumes_superuser_permissions,
 //   does_not_support_stepdowns,
 //   requires_fastcount,
+//   requires_fcv_44,
 //   requires_getmore,
 //   requires_non_retryable_commands,
 //   requires_non_retryable_writes,
+//   uses_map_reduce_with_temp_collections,
 //   requires_emptycapped,
 // ]
+//
+// Tagged as 'requires_fcv_44', since this test cannot run against versions less then 4.4. This is
+// because 'planCacheListPlans' and 'planCacheListQueryShapes' were deleted in 4.4, and thus not
+// tested here. But this test asserts that all commands are covered, so will fail against a version
+// of the server which implements these commands.
 
 /*
  * Declaratively-defined tests for views for all database commands. This file contains a map of test
@@ -66,19 +73,21 @@
 // Pre-written reasons for skipping a test.
 const isAnInternalCommand = "internal command";
 const isUnrelated = "is unrelated";
+const wasRemovedInBinaryVersion44 =
+    "must define test coverage for the multiversion passthrough suite";
 
 let viewsCommandTests = {
     _addShard: {skip: isAnInternalCommand},
-    _cloneCatalogData: {skip: isAnInternalCommand},
+    _cloneCatalogData: {skip: wasRemovedInBinaryVersion44},
     _cloneCollectionOptionsFromPrimaryShard: {skip: isAnInternalCommand},
     _configsvrAddShard: {skip: isAnInternalCommand},
     _configsvrAddShardToZone: {skip: isAnInternalCommand},
+    _configsvrBalancerCollectionStatus: {skip: isAnInternalCommand},
     _configsvrBalancerStart: {skip: isAnInternalCommand},
     _configsvrBalancerStatus: {skip: isAnInternalCommand},
     _configsvrBalancerStop: {skip: isAnInternalCommand},
     _configsvrClearJumboFlag: {skip: isAnInternalCommand},
     _configsvrCommitChunkMerge: {skip: isAnInternalCommand},
-    _configsvrCommitChunksMerge: {skip: isAnInternalCommand},
     _configsvrCommitChunkMigration: {skip: isAnInternalCommand},
     _configsvrCommitChunkSplit: {skip: isAnInternalCommand},
     _configsvrCommitMovePrimary: {skip: isAnInternalCommand},
@@ -87,29 +96,32 @@ let viewsCommandTests = {
     _configsvrDropCollection: {skip: isAnInternalCommand},
     _configsvrDropDatabase: {skip: isAnInternalCommand},
     _configsvrEnableSharding: {skip: isAnInternalCommand},
+    _configsvrEnsureChunkVersionIsGreaterThan: {skip: isAnInternalCommand},
     _configsvrMoveChunk: {skip: isAnInternalCommand},
     _configsvrMovePrimary: {skip: isAnInternalCommand},
+    _configsvrRefineCollectionShardKey: {skip: isAnInternalCommand},
+    _configsvrRenameCollection: {skip: isAnInternalCommand},
     _configsvrRemoveShard: {skip: isAnInternalCommand},
     _configsvrRemoveShardFromZone: {skip: isAnInternalCommand},
-    _configsvrRepairShardedCollectionChunksHistory: {skip: isAnInternalCommand},
     _configsvrShardCollection: {skip: isAnInternalCommand},
     _configsvrUpdateZoneKeyRange: {skip: isAnInternalCommand},
-    _cpuProfilerStart: {skip: isAnInternalCommand},
-    _cpuProfilerStop: {skip: isAnInternalCommand},
     _flushDatabaseCacheUpdates: {skip: isUnrelated},
     _flushRoutingTableCacheUpdates: {skip: isUnrelated},
     _getNextSessionMods: {skip: isAnInternalCommand},
     _getUserCacheGeneration: {skip: isAnInternalCommand},
     _hashBSONElement: {skip: isAnInternalCommand},
     _isSelf: {skip: isAnInternalCommand},
+    _killOperations: {skip: isUnrelated},
     _mergeAuthzCollections: {skip: isAnInternalCommand},
     _migrateClone: {skip: isAnInternalCommand},
-    _movePrimary: {skip: isAnInternalCommand},
+    _movePrimary: {skip: wasRemovedInBinaryVersion44},
     _recvChunkAbort: {skip: isAnInternalCommand},
     _recvChunkCommit: {skip: isAnInternalCommand},
     _recvChunkStart: {skip: isAnInternalCommand},
     _recvChunkStatus: {skip: isAnInternalCommand},
-    _shardsvrSetAllowMigrations: {skip: isAnInternalCommand},
+    _shardsvrCloneCatalogData: {skip: isAnInternalCommand},
+    _shardsvrMovePrimary: {skip: isAnInternalCommand},
+    _shardsvrRenameCollection: {skip: isAnInternalCommand},
     _shardsvrShardCollection: {skip: isAnInternalCommand},
     _transferMods: {skip: isAnInternalCommand},
     abortTransaction: {skip: isUnrelated},
@@ -123,15 +135,17 @@ let viewsCommandTests = {
         skipSharded: true,
     },
     authenticate: {skip: isUnrelated},
-    autoSplitVector: {
-        command: {
-            splitVector: "test.view",
-            keyPattern: {x: 1},
-            maxChunkSize: 1,
-        },
-        expectFailure: true,
-    },
     availableQueryOptions: {skip: isAnInternalCommand},
+    balancerCollectionStatus: {
+        command: {balancerCollectionStatus: "test.view"},
+        setup: function(conn) {
+            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
+        },
+        skipStandalone: true,
+        expectFailure: true,
+        isAdminCommand: true,
+        expectedErrorCode: ErrorCodes.NamespaceNotSharded,
+    },
     balancerStart: {skip: isUnrelated},
     balancerStatus: {skip: isUnrelated},
     balancerStop: {skip: isUnrelated},
@@ -152,7 +166,7 @@ let viewsCommandTests = {
         expectedErrorCode: ErrorCodes.NamespaceNotSharded,
     },
     clearLog: {skip: isUnrelated},
-    cloneCollection: {skip: "Tested in noPassthroughWithMongod/clonecollection.js"},
+    cloneCollection: {skip: wasRemovedInBinaryVersion44},
     cloneCollectionAsCapped: {
         command: {cloneCollectionAsCapped: "view", toCollection: "testcapped", size: 10240},
         expectFailure: true,
@@ -217,7 +231,7 @@ let viewsCommandTests = {
             assert.eq(hash1, hash3, "hash should be the same again after removing 'view2'");
         }
     },
-    dbStats: {skip: "TODO(SERVER-25948)"},
+    dbStats: {command: {dbStats: 1}},
     delete: {command: {delete: "view", deletes: [{q: {x: 1}, limit: 1}]}, expectFailure: true},
     distinct: {command: {distinct: "view", key: "_id"}},
     driverOIDTest: {skip: isUnrelated},
@@ -266,14 +280,15 @@ let viewsCommandTests = {
         expectFailure: true
     },
     getCmdLineOpts: {skip: isUnrelated},
+    getDefaultRWConcern: {skip: isUnrelated},
     getDiagnosticData: {skip: isUnrelated},
     getFreeMonitoringStatus: {skip: isUnrelated},
     getLastError: {skip: isUnrelated},
     getLog: {skip: isUnrelated},
     getMore: {
         setup: function(conn) {
-            assert.writeOK(conn.collection.remove({}));
-            assert.writeOK(conn.collection.insert([{_id: 1}, {_id: 2}, {_id: 3}]));
+            assert.commandWorked(conn.collection.remove({}));
+            assert.commandWorked(conn.collection.insert([{_id: 1}, {_id: 2}, {_id: 3}]));
         },
         command: function(conn) {
             function testGetMoreForCommand(cmd) {
@@ -308,9 +323,21 @@ let viewsCommandTests = {
     getParameter: {skip: isUnrelated},
     getShardMap: {skip: isUnrelated},
     getShardVersion: {
-        command: {getShardVersion: "test.view"},
+        command: function(conn) {
+            // getShardVersion was updated to be allowed on views in v4.4, but in v4.2 would return
+            // CommandNotSupportedOnView. Ignore a CommandNotSupportedOnView error code so that the
+            // test passes in the mixed-version replica sets passthrough. The v4.2 behavior is
+            // tested in the v4.2 branch.
+            try {
+                assert.commandWorked(conn.adminCommand({getShardVersion: "test.view"}));
+            } catch (e) {
+                if (e.code == ErrorCodes.CommandNotSupportedOnView) {
+                    return;
+                }
+                throw e;
+            }
+        },
         isAdminCommand: true,
-        expectFailure: true,
         skipSharded: true,  // mongos is tested in views/views_sharded.js
     },
     getnonce: {skip: isUnrelated},
@@ -319,17 +346,17 @@ let viewsCommandTests = {
     grantRolesToRole: {skip: isUnrelated},
     grantRolesToUser: {skip: isUnrelated},
     handshake: {skip: isUnrelated},
-    hello: {skip: isUnrelated},
     hostInfo: {skip: isUnrelated},
     httpClientRequest: {skip: isAnInternalCommand},
     insert: {command: {insert: "view", documents: [{x: 1}]}, expectFailure: true},
+    internalRenameIfOptionsAndIndexesMatch: {skip: isAnInternalCommand},
     invalidateUserCache: {skip: isUnrelated},
     isdbgrid: {skip: isUnrelated},
     isMaster: {skip: isUnrelated},
     killCursors: {
         setup: function(conn) {
-            assert.writeOK(conn.collection.remove({}));
-            assert.writeOK(conn.collection.insert([{_id: 1}, {_id: 2}, {_id: 3}]));
+            assert.commandWorked(conn.collection.remove({}));
+            assert.commandWorked(conn.collection.insert([{_id: 1}, {_id: 2}, {_id: 3}]));
         },
         command: function(conn) {
             // First get and check a partial result for an aggregate command.
@@ -369,7 +396,6 @@ let viewsCommandTests = {
     listShards: {skip: isUnrelated},
     lockInfo: {skip: isUnrelated},
     logApplicationMessage: {skip: isUnrelated},
-    logMessage: {skip: isUnrelated},
     logRotate: {skip: isUnrelated},
     logout: {skip: isUnrelated},
     makeSnapshot: {skip: isAnInternalCommand},
@@ -400,15 +426,13 @@ let viewsCommandTests = {
     planCacheClear: {command: {planCacheClear: "view"}, expectFailure: true},
     planCacheClearFilters: {command: {planCacheClearFilters: "view"}, expectFailure: true},
     planCacheListFilters: {command: {planCacheListFilters: "view"}, expectFailure: true},
-    planCacheListPlans: {command: {planCacheListPlans: "view"}, expectFailure: true},
-    planCacheListQueryShapes: {command: {planCacheListQueryShapes: "view"}, expectFailure: true},
     planCacheSetFilter: {command: {planCacheSetFilter: "view"}, expectFailure: true},
     prepareTransaction: {skip: isUnrelated},
     profile: {skip: isUnrelated},
+    refineCollectionShardKey: {skip: isUnrelated},
     refreshLogicalSessionCacheNow: {skip: isAnInternalCommand},
     reapLogicalSessionCacheNow: {skip: isAnInternalCommand},
     refreshSessions: {skip: isUnrelated},
-    restartCatalog: {skip: isAnInternalCommand},
     reIndex: {command: {reIndex: "view"}, expectFailure: true},
     removeShard: {skip: isUnrelated},
     removeShardFromZone: {skip: isUnrelated},
@@ -427,14 +451,8 @@ let viewsCommandTests = {
             skipSharded: true,
         }
     ],
-    repairCursor: {command: {repairCursor: "view"}, expectFailure: true},
-    repairShardedCollectionChunksHistory: {
-        command: {repairShardedCollectionChunksHistory: "test.view"},
-        skipStandalone: true,
-        isAdminCommand: true,
-        expectFailure: true,
-        expectedErrorCode: ErrorCodes.NamespaceNotFound,
-    },
+    repairCursor: {skip: wasRemovedInBinaryVersion44},
+    repairDatabase: {skip: isUnrelated},
     replSetAbortPrimaryCatchUp: {skip: isUnrelated},
     replSetFreeze: {skip: isUnrelated},
     replSetGetConfig: {skip: isUnrelated},
@@ -471,17 +489,9 @@ let viewsCommandTests = {
     saslContinue: {skip: isUnrelated},
     saslStart: {skip: isUnrelated},
     serverStatus: {command: {serverStatus: 1}, skip: isUnrelated},
-    setAllowMigrations: {
-        command: {setAllowMigrations: "test.view", allowMigrations: false},
-        setup: function(conn) {
-            assert.commandWorked(conn.adminCommand({enableSharding: "test"}));
-        },
-        expectedErrorCode: ErrorCodes.NamespaceNotSharded,
-        skipStandalone: true,
-        expectFailure: true,
-        isAdminCommand: true
-    },
+    setIndexCommitQuorum: {skip: isUnrelated},
     setCommittedSnapshot: {skip: isAnInternalCommand},
+    setDefaultRWConcern: {skip: isUnrelated},
     setFeatureCompatibilityVersion: {skip: isUnrelated},
     setFreeMonitoring: {skip: isUnrelated},
     setParameter: {skip: isUnrelated},
@@ -534,10 +544,7 @@ let viewsCommandTests = {
     startSession: {skip: isAnInternalCommand},
     stopRecordingTraffic: {skip: isUnrelated},
     top: {skip: "tested in views/views_stats.js"},
-    touch: {
-        command: {touch: "view", data: true},
-        expectFailure: true,
-    },
+    touch: {skip: wasRemovedInBinaryVersion44},
     unsetSharding: {skip: isAnInternalCommand},
     update: {command: {update: "view", updates: [{q: {x: 1}, u: {x: 2}}]}, expectFailure: true},
     updateRole: {
@@ -563,6 +570,7 @@ let viewsCommandTests = {
     voteAbortTransaction: {skip: isUnrelated},
     waitForFailPoint: {skip: isUnrelated},
     whatsmyuri: {skip: isUnrelated},
+    whatsmysni: {skip: isUnrelated},
 
     // Percona commands
     auditGetOptions: {skip: isUnrelated},
@@ -628,7 +636,7 @@ for (let command of commands) {
         // 'collection'.
         assert.commandWorked(dbHandle.dropDatabase());
         assert.commandWorked(dbHandle.runCommand({create: "view", viewOn: "collection"}));
-        assert.writeOK(dbHandle.collection.insert({x: 1}));
+        assert.commandWorked(dbHandle.collection.insert({x: 1}));
         if (subtest.setup !== undefined)
             subtest.setup(dbHandle);
 
