@@ -3,6 +3,8 @@
  * opened cursors and kills them on cleanup.
  */
 
+load("jstests/libs/fixture_helpers.js");  // For FixtureHelpers.
+
 /**
  * Enumeration of the possible types of change streams.
  */
@@ -42,15 +44,19 @@ const runCommandChangeStreamPassthroughAware =
  *     - single collection streams: drop, rename, and dropDatabase.
  *     - whole DB streams: dropDatabase.
  *     - whole cluster streams: none.
+ * Returns the invalidate document if there was one, or null otherwise.
  */
 function assertInvalidateOp({cursor, opType}) {
     if (!isChangeStreamPassthrough() ||
         (changeStreamPassthroughType() == ChangeStreamWatchMode.kDb && opType == "dropDatabase")) {
         assert.soon(() => cursor.hasNext());
-        assert.eq(cursor.next().operationType, "invalidate");
+        const invalidate = cursor.next();
+        assert.eq(invalidate.operationType, "invalidate");
         assert(cursor.isExhausted());
         assert(cursor.isClosed());
+        return invalidate;
     }
+    return null;
 }
 
 function pruneOptionalFields(event, expected) {
@@ -210,17 +216,8 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
     function assertChangeIsExpected(
         expectedChanges, numChangesSeen, observedChanges, expectInvalidate) {
         if (expectedChanges) {
-            const lastObservedChange = Object.assign({}, observedChanges[numChangesSeen]);
-            if (expectedChanges[numChangesSeen]._id == null) {
-                delete lastObservedChange._id;  // Remove the resume token, if present.
-            }
-            if (expectedChanges[numChangesSeen].clusterTime == null) {
-                delete lastObservedChange.clusterTime;  // Remove the cluster time, if present.
-            }
-            assert.docEq(lastObservedChange,
-                         expectedChanges[numChangesSeen],
-                         "Change did not match expected change. Expected changes: " +
-                             tojson(expectedChanges));
+            assertChangeStreamEventEq(observedChanges[numChangesSeen],
+                                      expectedChanges[numChangesSeen]);
         } else if (!expectInvalidate) {
             assert(!isInvalidated(observedChanges[numChangesSeen]),
                    "Change was invalidated when it should not have been. Number of changes seen: " +
@@ -237,8 +234,14 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
      *
      * Returns a list of the changes seen.
      */
-    self.assertNextChangesEqual = function(
-        {cursor, expectedChanges, expectedNumChanges, expectInvalidate, skipFirstBatch}) {
+    self.assertNextChangesEqual = function({
+        cursor,
+        expectedChanges,
+        expectedNumChanges,
+        expectInvalidate,
+        skipFirstBatch,
+        ignoreOrder
+    }) {
         expectInvalidate = expectInvalidate || false;
         skipFirstBatch = skipFirstBatch || false;
 
@@ -263,8 +266,22 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
         }
 
         let changes = self.getNextChanges(cursor, expectedNumChanges, skipFirstBatch);
-        for (let i = 0; i < changes.length; i++) {
-            assertChangeIsExpected(expectedChanges, i, changes, expectInvalidate);
+        if (ignoreOrder) {
+            const errMsgFunc = () => `${tojson(changes)} != ${tojson(expectedChanges)}`;
+            assert.eq(changes.length, expectedNumChanges, errMsgFunc);
+            for (let i = 0; i < changes.length; i++) {
+                assert(expectedChanges.some(expectedChange => {
+                    return _convertExceptionToReturnStatus(() => {
+                        assertChangeStreamEventEq(changes[i], expectedChange);
+                        return true;
+                    })();
+                }),
+                       errMsgFunc);
+            }
+        } else {
+            for (let i = 0; i < changes.length; i++) {
+                assertChangeIsExpected(expectedChanges, i, changes, expectInvalidate);
+            }
         }
 
         // If we expect invalidation, the final change should have operation type "invalidate".
@@ -281,16 +298,22 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
     };
 
     /**
-     * Iterates through the change stream and asserts that the next 'expected.length' changes are
-     * among the expected ones. The order of the change events from the cursor relative to their
-     * order in the list of expected changes is ignored, however.
+     * Iterates through the change stream and asserts that the next changes are the expected ones.
+     * The order of the change events from the cursor relative to their order in the list of
+     * expected changes is ignored, however.
+     *
+     * Returns a list of the changes seen.
      */
-    self.assertNextChangesEqualUnordered = function(cursor, expected) {
-        const changes = self.getNextChanges(cursor, expected.length).map(event => {
-            return pruneOptionalFields(event, expected[0]);
+    self.assertNextChangesEqualUnordered = function(
+        {cursor, expectedChanges, expectedNumChanges, expectInvalidate, skipFirstBatch}) {
+        return self.assertNextChangesEqual({
+            cursor: cursor,
+            expectedChanges: expectedChanges,
+            expectedNumChanges: expectedNumChanges,
+            expectInvalidate: expectInvalidate,
+            skipFirstBatch: skipFirstBatch,
+            ignoreOrder: true
         });
-
-        assert.sameMembers(changes, expected);
     };
 
     /**
@@ -339,7 +362,6 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
                 }));
             }
         }
-
     };
 
     /**
@@ -413,27 +435,27 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
 }
 
 /**
- * Asserts that the given pipeline will eventually return an error with the provided code,
- * either in the initial aggregate, or a subsequent getMore. Throws an exception if there are
- * any results from running the pipeline, or if it doesn't throw an error within the window of
- * assert.soon().  If 'doNotModifyInPassthroughs' is 'true' and the test is running in a
- * $changeStream upconversion passthrough, then this stream will not be modified and will run as
- * though no passthrough were active.
+ * Asserts that the given pipeline will eventually return an error with the provided code, either in
+ * the initial aggregate, or a subsequent getMore. Throws an exception if there are any results from
+ * running the pipeline, or if it doesn't throw an error within the window of assert.soon(). If
+ * 'doNotModifyInPassthroughs' is 'true' and the test is running in a $changeStream upconversion
+ * passthrough, then this stream will not be modified and will run as though no passthrough were
+ * active.
  */
 ChangeStreamTest.assertChangeStreamThrowsCode = function assertChangeStreamThrowsCode(
     {db, collName, pipeline, expectedCode, doNotModifyInPassthroughs}) {
     try {
+        // Run a passthrough-aware initial 'aggregate' command to open the change stream.
         const res = assert.commandWorked(runCommandChangeStreamPassthroughAware(
             db,
             {aggregate: collName, pipeline: pipeline, cursor: {batchSize: 1}},
             doNotModifyInPassthroughs));
 
-        // Extract the collection name from the cursor since the change stream may be on the whole
-        // database. The 'collName' parameter will be the integer 1 in that case and the getMore
-        // command requires 'collection' to be a string.
-        const getMoreCollName = getCollectionNameFromFullNamespace(res.cursor.ns);
-        assert.commandWorked(
-            db.runCommand({getMore: res.cursor.id, collection: getMoreCollName, batchSize: 1}));
+        // Create a cursor using the command response, and begin issuing getMores. We expect
+        // csCursor.hasNext() to throw the expected code before assert.soon() times out.
+        const csCursor = new DBCommandCursor(db, res, 1);
+        assert.soon(() => csCursor.hasNext());
+        assert(false, `Unexpected result from cursor: ${tojson(csCursor.next())}`);
     } catch (error) {
         assert.eq(error.code, expectedCode, `Caught unexpected error: ${tojson(error)}`);
         return true;
