@@ -64,13 +64,17 @@ function configuredForTxnOverride() {
     return TestData.networkErrorAndTxnOverrideConfig.wrapCRUDinTransactions;
 }
 
+function configuredForBackgroundReconfigs() {
+    assert(TestData.networkErrorAndTxnOverrideConfig, TestData);
+    return TestData.networkErrorAndTxnOverrideConfig.backgroundReconfigs;
+}
+
 // Commands assumed to not be blindly retryable.
 const kNonRetryableCommands = new Set([
     // Commands that take write concern and do not support txnNumbers.
     "_configsvrAddShard",
     "_configsvrAddShardToZone",
     "_configsvrCommitChunkMerge",
-    "_configsvrCommitChunksMerge",
     "_configsvrCommitChunkMigration",
     "_configsvrCommitChunkSplit",
     "_configsvrCreateDatabase",
@@ -86,9 +90,7 @@ const kNonRetryableCommands = new Set([
     "appendOplogNote",
     "applyOps",
     "captrunc",
-    "cleanupOrphaned",
     "clone",
-    "cloneCollection",
     "cloneCollectionAsCapped",
     "convertToCapped",
     "create",
@@ -109,7 +111,6 @@ const kNonRetryableCommands = new Set([
     "grantRolesToRole",
     "grantRolesToUser",
     "mapreduce.shardedfinish",
-    "moveChunk",
     "renameCollection",
     "revokePrivilegesFromRole",
     "revokeRolesFromRole",
@@ -128,8 +129,18 @@ const kAcceptableNonRetryableCommands = new Set([
     "drop",
     "dropDatabase",  // Already ignores NamespaceNotFound errors, so not handled below.
     "dropIndexes",
+    "moveChunk",
 ]);
 
+// The following read operations defined in the CRUD specification are retryable.
+// Note that estimatedDocumentCount() and countDocuments() use the count command.
+const kRetryableReadCommands = new Set(["find", "aggregate", "distinct", "count"]);
+
+// Returns true if the command name is that of a retryable read command.
+function isRetryableReadCmdName(cmdName) {
+    return kRetryableReadCommands.has(cmdName);
+}
+
 // Returns if the given failed response is a safe response to ignore when retrying the
 // given command type.
 function isAcceptableRetryFailedResponse(cmdName, res) {
@@ -158,8 +169,6 @@ const kNonFailoverTolerantCommands = new Set([
     "planCacheClear",  // The plan cache isn't replicated.
     "planCacheClearFilters",
     "planCacheListFilters",
-    "planCacheListPlans",
-    "planCacheListQueryShapes",
     "planCacheSetFilter",
     "profile",       // Not replicated, so can't tolerate failovers.
     "setParameter",  // Not replicated, so can't tolerate failovers.
@@ -190,6 +199,20 @@ function canRetryNetworkErrorForCommand(cmdName, cmdObj) {
     return true;
 }
 
+// Returns if the given command should retry a read error when reconfigs are present.
+function canRetryReadErrorDuringBackgroundReconfig(cmdName) {
+    if (!configuredForBackgroundReconfigs()) {
+        return false;
+    }
+    return isRetryableReadCmdName(cmdName);
+}
+
+// When running the reconfig command on a node, it will drop its snapshot. Read commands issued
+// to this node before it updates its snapshot will fail with ReadConcernMajorityNotAvailableYet.
+function isRetryableReadCode(code) {
+    return code === ErrorCodes.ReadConcernMajorityNotAvailableYet;
+}
+
 // Several commands that use the plan executor swallow the actual error code from a failed plan
 // into their error message and instead return OperationFailed.
 //
@@ -205,9 +228,24 @@ function isRetryableShardCollectionResponse(res) {
     // shardCollection can bury the original error code in the error message.
     return RetryableWritesUtil.errmsgContainsRetryableCodeName(res.errmsg) ||
         // shardCollection creates collections on each shard that will receive a chunk using
-        // _cloneCollectionsOptionsFromPrimaryShard, which may fail with either of the following
-        // codes if interupted by a failover.
-        res.code === ErrorCodes.CallbackCanceled || res.code === 17405;
+        // _cloneCollectionsOptionsFromPrimaryShard, which may fail with the following code if
+        // interupted by a failover.
+        res.code === ErrorCodes.CallbackCanceled;
+}
+
+// Returns true if the given response could have come from moveChunk being interrupted by a
+// failover.
+function isRetryableMoveChunkResponse(res) {
+    return (res.code === ErrorCodes.OperationFailed &&
+            (RetryableWritesUtil.errmsgContainsRetryableCodeName(res.errmsg) ||
+             // The transaction number is bumped by the migration coordinator when its commit or
+             // abort decision is being made durable.
+             res.errmsg.includes("TransactionTooOld") ||
+             // The range deletion task may have been interrupted. This error can occur even when
+             // _waitForDelete=false.
+             res.errmsg.includes("operation was interrupted"))) ||
+        // This error may occur when the node is shutting down.
+        res.code === ErrorCodes.CallbackCanceled;
 }
 
 function hasError(res) {
@@ -334,10 +372,6 @@ function validateCmdNetworkErrorCompatibility(cmdName, cmdObj) {
                 " because it may return incomplete results if interrupted by a stepdown." +
                 logSuffix);
         }
-    } else if (cmdName === "mapReduce" || cmdName === "mapreduce") {
-        throw new Error(
-            "Refusing to run a test that issues a mapReduce command, because it calls " +
-            " std::terminate() if interrupted by a stepdown." + logSuffix);
     }
 }
 
@@ -454,7 +488,8 @@ function appendReadAndWriteConcern(conn, dbName, cmdName, cmdObj) {
             let writeConcern = cmdObj.writeConcern;
             if (typeof writeConcern !== "object" || writeConcern === null ||
                 (writeConcern.hasOwnProperty("w") &&
-                 bsonWoCompare({_: writeConcern.w}, {_: kDefaultWriteConcernW}) !== 0)) {
+                 bsonWoCompare({_: writeConcern.w}, {_: kDefaultWriteConcernW}) !== 0 &&
+                 bsonWoCompare({_: writeConcern.w}, {_: 1}) !== 0)) {
                 throw new Error("Cowardly refusing to override write concern of command: " +
                                 tojson(cmdObj));
             }
@@ -579,6 +614,16 @@ function isCommandNonTxnGetMore(cmdName, cmdObj) {
     return cmdName === "getMore" && nonTxnAggCursorSet[cmdObj.getMore];
 }
 
+function isNamespaceSystemDotProfile(cmdObj) {
+    // No operations on system.profile are permitted inside transactions (see SERVER-46900).
+    for (let val of Object.values(cmdObj)) {
+        if (typeof val === 'string' && val.endsWith('system.profile')) {
+            return true;
+        }
+    }
+    return false;
+}
+
 function setupTransactionCommand(conn, dbName, cmdName, cmdObj, lsid) {
     // We want to overwrite whatever read and write concern is already set.
     delete cmdObj.readConcern;
@@ -588,8 +633,10 @@ function setupTransactionCommand(conn, dbName, cmdName, cmdObj, lsid) {
     // use transactions.
     const driverSession = conn.getDB(dbName).getSession();
     const commandSupportsTransaction = TransactionsUtil.commandSupportsTxn(dbName, cmdName, cmdObj);
-    if (commandSupportsTransaction && driverSession.getSessionId() !== null &&
-        !isCommandNonTxnGetMore(cmdName, cmdObj)) {
+    const isSystemDotProfile = isNamespaceSystemDotProfile(cmdObj);
+
+    if (commandSupportsTransaction && !isSystemDotProfile &&
+        driverSession.getSessionId() !== null && !isCommandNonTxnGetMore(cmdName, cmdObj)) {
         if (isNested()) {
             // Nested commands should never start a new transaction.
         } else if (ops.length === 0) {
@@ -608,7 +655,10 @@ function setupTransactionCommand(conn, dbName, cmdName, cmdObj, lsid) {
         continueTransaction(conn, dbName, cmdName, cmdObj);
 
     } else {
-        if (ops.length > 0 && !isNested()) {
+        if (ops.length > 0 && !isNested() && !isSystemDotProfile) {
+            // Operations on system.profile must be allowed to execute in parallel with open
+            // transactions, so operations on system.profile should not commit the current open
+            // transaction.
             logMsgFull('setupTransactionCommand',
                        `Committing transaction ${txnOptions.txnNumber} on session` +
                            ` ${tojsononeline(lsid)} to run a command that does not support` +
@@ -689,17 +739,23 @@ function retryWithTxnOverride(res, conn, dbName, cmdName, cmdObj, lsid, logError
         assert.gt(ops.length, 0);
         abortTransaction(conn, lsid, txnOptions.txnNumber);
 
-        // If the command inserted data and is not supported in a transaction, we assume it
-        // failed because the collection did not exist. We will create the collection and retry
-        // the entire transaction. We should not receive this error in this override for any
-        // other reason.
-        // Tests that expect collections to not exist will have to be skipped.
-        if (kCmdsThatInsert.has(cmdName) &&
-            includesErrorCode(res, ErrorCodes.OperationNotSupportedInTransaction)) {
-            const collName = cmdObj[cmdName];
-            createCollectionExplicitly(conn, dbName, collName, lsid);
+        const fcv =
+            conn.getDB("admin").system.version.findOne({_id: 'featureCompatibilityVersion'});
+        if (fcv.version === "4.2" && !fcv.hasOwnProperty("targetVersion")) {
+            // With FCV 4.2, implicit collection creation via commands that insert are not
+            // permitted inside multi-document transactions.
+            // If the command inserted data and is not supported in a transaction, we assume it
+            // failed because the collection did not exist. We will create the collection and retry
+            // the entire transaction. We should not receive this error in this override for any
+            // other reason.
+            // Tests that expect collections to not exist will have to be skipped.
+            if (kCmdsThatInsert.has(cmdName) &&
+                includesErrorCode(res, ErrorCodes.OperationNotSupportedInTransaction)) {
+                const collName = cmdObj[cmdName];
+                createCollectionExplicitly(conn, dbName, collName, lsid);
 
-            return retryEntireTransaction(conn, lsid);
+                return retryEntireTransaction(conn, lsid);
+            }
         }
 
         // Transaction statements cannot be retried, but retryable codes are expected to succeed
@@ -800,8 +856,8 @@ function shouldRetryWithNetworkErrorOverride(
     if (!res.ok) {
         if (RetryableWritesUtil.isRetryableCode(res.code)) {
             // Don't decrement retries, because the command returned before the connection was
-            // closed, so a subsequent attempt will receive a network error (or NotWritablePrimary
-            // error) and need to retry.
+            // closed, so a subsequent attempt will receive a network error (or NotMaster error)
+            // and need to retry.
             logError("Retrying failed response with retryable code");
             return kContinue;
         }
@@ -823,12 +879,6 @@ function shouldRetryWithNetworkErrorOverride(
             return kContinue;
         }
 
-        // Thrown when an index build is interrupted during its collection scan.
-        if (cmdName === "createIndexes" && res.codeName === "InterruptedDueToReplStateChange") {
-            logError("Retrying because of interrupted collection scan");
-            return kContinue;
-        }
-
         // Some sharding commands return raw responses from all contacted shards and there won't
         // be a top level code if shards returned more than one error code, in which case retry
         // if any error is retryable.
@@ -844,6 +894,12 @@ function shouldRetryWithNetworkErrorOverride(
             return kContinue;
         }
 
+        // Check for the retryable error codes from an interrupted moveChunk.
+        if (cmdName === "moveChunk" && isRetryableMoveChunkResponse(res)) {
+            logError("Retrying interrupted moveChunk");
+            return kContinue;
+        }
+
         // In a sharded cluster, drop may bury the original error code in the error message if
         // interrupted.
         if (cmdName === "drop" && RetryableWritesUtil.errmsgContainsRetryableCodeName(res.errmsg)) {
@@ -876,6 +932,19 @@ function shouldRetryWithNetworkErrorOverride(
     return res;
 }
 
+function shouldRetryForBackgroundReconfigOverride(res, cmdName, logError) {
+    assert(configuredForBackgroundReconfigs());
+    // Background reconfigs can interfere with read commands if they are using readConcern: majority
+    // and readPreference: primary. If we're running a read command and it fails with
+    // ReadConcernMajorityNotAvailableYet, retry because it should eventually succeed.
+    if (isRetryableReadCmdName(cmdName) && isRetryableReadCode(res.code)) {
+        logError("Retrying read command after 100ms because of background reconfigs");
+        sleep(100);
+        return kContinue;
+    }
+    return res;
+}
+
 // Processes exceptions if configured for txn override. Retries the entire transaction on
 // transient transaction errors or network errors if configured for network errors as well.
 // If a retry fails, returns the response, or returns null for further exception processing.
@@ -961,6 +1030,7 @@ function runCommandOverrideBody(conn, dbName, cmdName, cmdObj, lsid, clientFunct
     }
 
     const canRetryNetworkError = canRetryNetworkErrorForCommand(cmdName, cmdObj);
+    const canRetryReadError = canRetryReadErrorDuringBackgroundReconfig(cmdName);
     let numNetworkErrorRetries = canRetryNetworkError ? kMaxNumRetries : 0;
     do {
         try {
@@ -991,6 +1061,16 @@ function runCommandOverrideBody(conn, dbName, cmdName, cmdObj, lsid, clientFunct
                 }
             }
 
+            if (canRetryReadError) {
+                const readRetryRes =
+                    shouldRetryForBackgroundReconfigOverride(res, cmdName, logError);
+                if (readRetryRes === kContinue) {
+                    continue;
+                } else {
+                    res = readRetryRes;
+                }
+            }
+
             return res;
 
         } catch (e) {
