@@ -35,19 +35,15 @@ const allHosts = cfg.members.map(x => x.host);
 const mongosDB = mongos.getDB(kDbName);
 const primaryOnly = [primary.name];
 
-function configureReplSetFailpoint(name, modeValue, extraData = undefined) {
-    let data = {
-        shouldCheckForInterrupt: true,
-        nss: kDbName + ".test",
-    };
-    if (extraData) {
-        data = Object.assign({}, data, extraData);
-    }
+function configureReplSetFailpoint(name, modeValue) {
     st.rs0.nodes.forEach(function(node) {
         assert.commandWorked(node.getDB("admin").runCommand({
             configureFailPoint: name,
             mode: modeValue,
-            data: data,
+            data: {
+                shouldCheckForInterrupt: true,
+                nss: kDbName + ".test",
+            },
         }));
     });
 }
@@ -89,7 +85,7 @@ function hasConnPoolStats(args) {
     function checkStats(res, host) {
         var stats = res.hosts[host];
         if (!stats) {
-            jsTestLog("Connection stats for " + host + " are absent in: " + tojson(res));
+            jsTestLog("Connection stats for " + host + " are absent");
             return isAbsent;
         }
 
@@ -132,9 +128,9 @@ function runSubTest(name, fun) {
     updateSetParameters(stepParams);
 }
 
-assert.writeOK(mongosDB.test.insert({x: 1}));
-assert.writeOK(mongosDB.test.insert({x: 2}));
-assert.writeOK(mongosDB.test.insert({x: 3}));
+assert.commandWorked(mongosDB.test.insert({x: 1}));
+assert.commandWorked(mongosDB.test.insert({x: 2}));
+assert.commandWorked(mongosDB.test.insert({x: 3}));
 st.rs0.awaitReplication();
 
 runSubTest("MinSize", function() {
@@ -188,7 +184,7 @@ runSubTest("MaxConnecting", function() {
         ShardingTaskExecutorPoolMaxConnecting: maxPending1,
     });
 
-    configureReplSetFailpoint("waitInHello", "alwaysOn", {delay: 10000});
+    configureReplSetFailpoint("waitInIsMaster", "alwaysOn");
     configureReplSetFailpoint("waitInFindBeforeMakingBatch", "alwaysOn");
     dropConnections();
 
@@ -209,7 +205,7 @@ runSubTest("MaxConnecting", function() {
     hasConnPoolStats({pending: maxPending2});
 
     // Release our pending and walk away
-    configureReplSetFailpoint("waitInHello", "off");
+    configureReplSetFailpoint("waitInIsMaster", "off");
     hasConnPoolStats({
         // Expects the number of pending connections to be zero.
         checkStatsFunc: function(stats) {
@@ -246,7 +242,7 @@ runSubTest("Timeouts", function() {
     hasConnPoolStats({ready: conns});
 
     // Block refreshes and wait for the toRefresh timeout
-    configureReplSetFailpoint("waitInHello", "alwaysOn", {delay: 100000});
+    configureReplSetFailpoint("waitInIsMaster", "alwaysOn");
     sleep(toRefreshTimeoutMS);
 
     // Confirm that we're in pending for all of our conns
@@ -261,7 +257,7 @@ runSubTest("Timeouts", function() {
     sleep(pendingTimeoutMS);
     hasConnPoolStats({});
 
-    configureReplSetFailpoint("waitInHello", "off");
+    configureReplSetFailpoint("waitInIsMaster", "off");
 
     // Reset the min conns to make sure normal refresh doesn't extend the timeout
     updateSetParameters({
