@@ -24,7 +24,6 @@ const isPrimaryOnly = "primary only";
 
 const allCommands = {
     _addShard: {skip: isPrimaryOnly},
-    _cloneCatalogData: {skip: isPrimaryOnly},
     _cloneCollectionOptionsFromPrimaryShard: {skip: isPrimaryOnly},
     _configsvrAddShard: {skip: isPrimaryOnly},
     _configsvrAddShardToZone: {skip: isPrimaryOnly},
@@ -34,7 +33,6 @@ const allCommands = {
     _configsvrBalancerStop: {skip: isPrimaryOnly},
     _configsvrClearJumboFlag: {skip: isPrimaryOnly},
     _configsvrCommitChunkMerge: {skip: isPrimaryOnly},
-    _configsvrCommitChunksMerge: {skip: isPrimaryOnly},
     _configsvrCommitChunkMigration: {skip: isPrimaryOnly},
     _configsvrCommitChunkSplit: {skip: isPrimaryOnly},
     _configsvrCommitMovePrimary: {skip: isPrimaryOnly},
@@ -49,7 +47,6 @@ const allCommands = {
     _configsvrRefineCollectionShardKey: {skip: isPrimaryOnly},
     _configsvrRemoveShard: {skip: isPrimaryOnly},
     _configsvrRemoveShardFromZone: {skip: isPrimaryOnly},
-    _configsvrRepairShardedCollectionChunksHistory: {skip: isAnInternalCommand},
     _configsvrShardCollection: {skip: isPrimaryOnly},
     _configsvrUpdateZoneKeyRange: {skip: isPrimaryOnly},
     _flushDatabaseCacheUpdates: {skip: isPrimaryOnly},
@@ -61,27 +58,24 @@ const allCommands = {
     _killOperations: {skip: isNotAUserDataRead},
     _mergeAuthzCollections: {skip: isPrimaryOnly},
     _migrateClone: {skip: isPrimaryOnly},
-    _movePrimary: {skip: isPrimaryOnly},
     _recvChunkAbort: {skip: isPrimaryOnly},
     _recvChunkCommit: {skip: isPrimaryOnly},
     _recvChunkStart: {skip: isPrimaryOnly},
     _recvChunkStatus: {skip: isPrimaryOnly},
     _shardsvrCloneCatalogData: {skip: isPrimaryOnly},
     _shardsvrMovePrimary: {skip: isPrimaryOnly},
-    _shardsvrSetAllowMigrations: {skip: isPrimaryOnly},
     _shardsvrShardCollection: {skip: isPrimaryOnly},
     _transferMods: {skip: isPrimaryOnly},
     abortTransaction: {skip: isPrimaryOnly},
     aggregate: {
         command: {aggregate: collName, pipeline: [{$match: {}}], cursor: {}},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     appendOplogNote: {skip: isPrimaryOnly},
     applyOps: {skip: isPrimaryOnly},
     auditGetOptions: {skip: isNotAUserDataRead},
     authenticate: {skip: isNotAUserDataRead},
-    autoSplitVector: {skip: isNotAUserDataRead},
     availableQueryOptions: {skip: isNotAUserDataRead},
     buildInfo: {skip: isNotAUserDataRead},
     captrunc: {skip: isPrimaryOnly},
@@ -89,12 +83,11 @@ const allCommands = {
     cleanupOrphaned: {skip: isPrimaryOnly},
     clearLog: {skip: isNotAUserDataRead},
     cloneCollectionAsCapped: {skip: isPrimaryOnly},
-    cloneCollection: {skip: isPrimaryOnly},
     collMod: {skip: isPrimaryOnly},
     collStats: {
         command: {aggregate: collName, pipeline: [{$collStats: {count: {}}}], cursor: {}},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     commitTransaction: {skip: isPrimaryOnly},
     compact: {skip: isNotAUserDataRead},
@@ -107,7 +100,7 @@ const allCommands = {
     count: {
         command: {count: collName},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     cpuload: {skip: isNotAUserDataRead},
     create: {skip: isPrimaryOnly},
@@ -122,17 +115,19 @@ const allCommands = {
     dbCheck: {skip: isPrimaryOnly},
     dbHash: {
         command: {dbHash: 1},
+        expectFailure: true,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     dbStats: {
         command: {dbStats: 1},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     delete: {skip: isPrimaryOnly},
     distinct: {
         command: {distinct: collName, key: "a"},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     driverOIDTest: {skip: isNotAUserDataRead},
     drop: {skip: isPrimaryOnly},
@@ -149,14 +144,14 @@ const allCommands = {
     explain: {
         command: {count: collName},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     features: {skip: isNotAUserDataRead},
     filemd5: {skip: isNotAUserDataRead},
     find: {
         command: {find: collName, filter: {a: 1}},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     findAndModify: {skip: isPrimaryOnly},
     flushRouterConfig: {skip: isNotAUserDataRead},
@@ -169,7 +164,7 @@ const allCommands = {
             near: [-42, 42],
         },
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary
     },
     getCmdLineOpts: {skip: isNotAUserDataRead},
     getDatabaseVersion: {skip: isNotAUserDataRead},
@@ -181,7 +176,7 @@ const allCommands = {
     getMore: {
         command: {getMore: NumberLong(123), collection: collName},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary
     },
     getParameter: {skip: isNotAUserDataRead},
     getShardMap: {skip: isNotAUserDataRead},
@@ -191,7 +186,6 @@ const allCommands = {
     grantPrivilegesToRole: {skip: isPrimaryOnly},
     grantRolesToRole: {skip: isPrimaryOnly},
     grantRolesToUser: {skip: isPrimaryOnly},
-    hello: {skip: isNotAUserDataRead},
     hostInfo: {skip: isNotAUserDataRead},
     httpClientRequest: {skip: isNotAUserDataRead},
     insert: {skip: isPrimaryOnly},
@@ -206,23 +200,22 @@ const allCommands = {
     listCollections: {
         command: {listCollections: 1},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary
     },
     listCommands: {command: {listCommands: 1}},
     listDatabases: {
         command: {listDatabases: 1},
         isAdminCommand: true,
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary
     },
     listIndexes: {
         command: {listIndexes: collName},
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary
     },
     lockInfo: {skip: isPrimaryOnly},
     logApplicationMessage: {skip: isNotAUserDataRead},
-    logMessage: {skip: isNotAUserDataRead},
     logRotate: {skip: isNotAUserDataRead},
     logout: {skip: isNotAUserDataRead},
     makeSnapshot: {skip: isNotAUserDataRead},
@@ -234,7 +227,7 @@ const allCommands = {
             out: {inline: 1}
         },
         expectFailure: true,
-        expectedErrorCode: ErrorCodes.NotPrimaryOrSecondary,
+        expectedErrorCode: ErrorCodes.NotMasterOrSecondary,
     },
     "mapreduce.shardedfinish": {skip: isAnInternalCommand},
     mergeChunks: {skip: isPrimaryOnly},
@@ -243,8 +236,6 @@ const allCommands = {
     planCacheClear: {skip: isNotAUserDataRead},
     planCacheClearFilters: {skip: isNotAUserDataRead},
     planCacheListFilters: {skip: isNotAUserDataRead},
-    planCacheListPlans: {skip: isNotAUserDataRead},
-    planCacheListQueryShapes: {skip: isNotAUserDataRead},
     planCacheSetFilter: {skip: isNotAUserDataRead},
     prepareTransaction: {skip: isPrimaryOnly},
     profile: {skip: isPrimaryOnly},
@@ -253,8 +244,7 @@ const allCommands = {
     refreshSessions: {skip: isNotAUserDataRead},
     reIndex: {skip: isNotAUserDataRead},
     renameCollection: {skip: isPrimaryOnly},
-    repairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
-    repairCursor: {skip: isNotAUserDataRead},
+    repairDatabase: {skip: isNotAUserDataRead},
     replSetAbortPrimaryCatchUp: {skip: isNotAUserDataRead},
     replSetFreeze: {skip: isNotAUserDataRead},
     replSetGetConfig: {skip: isNotAUserDataRead},
@@ -269,11 +259,9 @@ const allCommands = {
     replSetStepUp: {skip: isNotAUserDataRead},
     replSetSyncFrom: {skip: isNotAUserDataRead},
     replSetTest: {skip: isNotAUserDataRead},
-    replSetTestEgress: {skip: isNotAUserDataRead},
     replSetUpdatePosition: {skip: isNotAUserDataRead},
     replSetResizeOplog: {skip: isNotAUserDataRead},
     resetError: {skip: isNotAUserDataRead},
-    restartCatalog: {skip: isNotAUserDataRead},
     revokePrivilegesFromRole: {skip: isPrimaryOnly},
     revokeRolesFromRole: {skip: isPrimaryOnly},
     revokeRolesFromUser: {skip: isPrimaryOnly},
@@ -299,7 +287,6 @@ const allCommands = {
     startSession: {skip: isNotAUserDataRead},
     stopRecordingTraffic: {skip: isNotAUserDataRead},
     top: {skip: isNotAUserDataRead},
-    touch: {skip: isNotAUserDataRead},
     unsetSharding: {skip: isNotAUserDataRead},
     update: {skip: isPrimaryOnly},
     updateRole: {skip: isPrimaryOnly},
@@ -346,7 +333,7 @@ assert.commandWorked(secondary.adminCommand({replSetMaintenance: 1}));
 
 // Run all tests against the RECOVERING node.
 AllCommandsTest.testAllCommands(secondary, allCommands, function(test) {
-    const testDb = secondaryDb.getSiblingDB(dbName);
+    const testDb = secondaryDb.getSiblingDB("test");
     let cmdDb = testDb;
 
     if (test.isAdminCommand) {
