@@ -5,7 +5,7 @@
  * enableMajorityReadConcern=true.
  * Rollback after restarting with enableMajorityReadConcern=true succeeds if the common point is at
  * least the stable timestamp, i.e. we do not attempt to roll back operations that were included in
- * @tags: [requires_persistence]
+ * @tags: [requires_persistence, requires_fcv_44]
  */
 (function() {
 "use strict";
@@ -42,24 +42,16 @@ jsTest.log("Restart the rollback node with enableMajorityReadConcern=true");
 rollbackTest.restartNode(0, 15, {enableMajorityReadConcern: "true"});
 const replSetGetStatusResponse =
     assert.commandWorked(rollbackNode.adminCommand({replSetGetStatus: 1}));
-assert.eq(replSetGetStatusResponse.lastStableCheckpointTimestamp,
+assert.eq(replSetGetStatusResponse.lastStableRecoveryTimestamp,
           operationTime,
           tojson(replSetGetStatusResponse));
 
 // The rollback crashes because the common point is before the stable timestamp.
 jsTest.log("Attempt to roll back. This will fassert.");
 rollbackTest.transitionToSyncSourceOperationsBeforeRollback();
-
-// The first rollback attempt with EMRC=true will fassert, so we expect the actual rollback to occur
-// with EMRC=false. Before the second rollback (via refetch) occurs, we must ensure that the sync
-// source's lastApplied is greater than the rollback node's. Otherwise, the rollback node will never
-// transition to SECONDARY since the rollback node's lastApplied will be less than the
-// initialDataTS. See SERVER-48518 for a more detailed explanation of this behavior.
-rollbackTest.awaitPrimaryAppliedSurpassesRollbackApplied();
-
 rollbackTest.transitionToSyncSourceOperationsDuringRollback();
 assert.soon(() => {
-    return rawMongoProgramOutput().indexOf("Fatal Assertion 51121") !== -1;
+    return rawMongoProgramOutput().search(/Fatal assertion.+51121/) != -1;
 });
 
 jsTest.log(
