@@ -96,6 +96,7 @@
 (function() {
 "use strict";
 
+load("jstests/core/txns/libs/prepare_helpers.js");
 load("jstests/libs/write_concern_util.js");
 load("jstests/replsets/libs/rollback_test.js");
 
@@ -144,10 +145,15 @@ assert.soonNoExcept(() => {
     const res = P2.adminCommand({replSetStepUp: 1});
     return res.ok;
 }, "Failed to step up P2", ReplSetTest.kDefaultTimeoutMS);
-checkLog.contains(P2, "transition to primary complete; database writes are now permitted");
+// "Transition to primary complete; database writes are now permitted".
+checkLog.containsJson(P2, 21331);
 jsTestLog("P2 stepped up");
 
-// Write to P1 to ensure TS 4 (P1's delete timestamp) > TS 3 (P2's step-up timestamp).
+reply = assert.commandWorked(testDB.runCommand({delete: "test", deletes: [{q: {}, limit: 0}]}));
+jsTestLog(`Deleted from P1 at ${reply.operationTime}`);
+
+// Ensure P1's lastApplied > P2's, even if P1's set-up entry was written at the same timestamp as
+// P2's delete timestamp.
 assert.soon(() => {
     testDB.runCommand({insert: "otherCollection", documents: [{}]});
     function lastApplied(node) {
@@ -160,17 +166,15 @@ assert.soon(() => {
     return timestampCmp(P1applied, P2applied) > 0;
 }, "P1's lastApplied never surpassed P2's");
 
-reply = assert.commandWorked(testDB.runCommand({delete: "test", deletes: [{q: {}, limit: 0}]}));
-jsTestLog(`Deleted from P1 at ${reply.operationTime}.` +
-          ` Reconnecting P1 to P2, so P1 rolls back.`);
-
+jsTestLog("Reconnecting P1 to P2, so P1 rolls back");
 P1.reconnect([P2]);
-checkLog.contains(P1, "Rollback using the 'rollbackViaRefetch' method");
-checkLog.contains(P1, "Finding the Common Point");
-checkLog.contains(
-    P1,
-    "We cannot transition to SECONDARY state because our 'lastApplied' optime is less than the" +
-        " initial data timestamp and enableMajorityReadConcern = false");
+// "Rollback using the 'rollbackViaRefetch' method".
+checkLog.containsJson(P1, 21103);
+// "Finding the Common Point".
+checkLog.containsJson(P1, 21682);
+// "We cannot transition to SECONDARY state because our 'lastApplied' optime is less than the
+// initial data timestamp and enableMajorityReadConcern = false".
+checkLog.containsJson(P1, 4851800);
 
 reply = assert.commandWorked(
     P2.getDB(jsTestName()).runCommand({insert: "anotherCollection", documents: [{}]}));
