@@ -12,10 +12,12 @@
  * @tags: [uses_transactions, uses_prepare_transaction]
  */
 (function() {
+
 "use strict";
 load('jstests/libs/parallelTester.js');
 load("jstests/libs/curop_helpers.js");  // for waitForCurOpByFailPoint().
 load("jstests/core/txns/libs/prepare_helpers.js");
+load("jstests/libs/fail_point_util.js");
 
 const dbName = "test";
 const collName = "coll";
@@ -45,14 +47,13 @@ const joinStepDownThread = startParallelShell(() => {
 
     const freezeSecs = 24 * 60 * 60;  // 24 hours
     assert.commandFailedWithCode(db.adminCommand({"replSetStepDown": freezeSecs, "force": true}),
-                                 ErrorCodes.NotWritablePrimary);
+                                 ErrorCodes.NotMaster);
 }, primary.port);
 
 waitForCurOpByFailPointNoNS(primaryDB, "stepdownHangBeforeRSTLEnqueue");
 
 jsTestLog("Force reconfig to swap the electable node");
-// We must specify the node in getReplSetConfigFromNode, because we do not have a writable primary.
-const newConfig = rst.getReplSetConfigFromNode(0);
+const newConfig = rst.getReplSetConfigFromNode();
 const oldPrimaryId = rst.getNodeId(primary);
 const newPrimaryId = rst.getNodeId(secondary);
 newConfig.members[newPrimaryId].priority = 1;
@@ -89,8 +90,7 @@ TestData.clusterTimeAfterPrepare =
 rst.awaitReplication();
 
 jsTestLog("Do a read that hits a prepare conflict on the old primary");
-assert.commandWorked(
-    primary.adminCommand({configureFailPoint: "WTPrintPrepareConflictLog", mode: "alwaysOn"}));
+const wTPrintPrepareConflictLogFailPoint = configureFailPoint(primary, "WTPrintPrepareConflictLog");
 
 const joinReadThread = startParallelShell(() => {
     db.getMongo().setSlaveOk(true);
@@ -105,7 +105,7 @@ const joinReadThread = startParallelShell(() => {
 }, primary.port);
 
 jsTestLog("Wait to hit a prepare conflict");
-checkLog.contains(primary, "WTPrintPrepareConflictLog fail point enabled");
+wTPrintPrepareConflictLogFailPoint.wait();
 
 jsTestLog("Allow step down to complete");
 assert.commandWorked(
