@@ -11,6 +11,16 @@
 
 var st = new ShardingTest({shards: 4, chunkSize: 1});
 
+// Double the balancer interval to produce fewer migrations per unit time so that the test does not
+// run out of stale shard version retries.
+st._configServers.forEach((conn) => {
+    conn.adminCommand({
+        configureFailPoint: 'overrideBalanceRoundInterval',
+        mode: 'alwaysOn',
+        data: {intervalMs: 2000}
+    });
+});
+
 assert.commandWorked(st.s0.adminCommand({enableSharding: 'TestDB'}));
 st.ensurePrimaryShard('TestDB', st.shard0.shardName);
 assert.commandWorked(st.s0.adminCommand({shardCollection: 'TestDB.TestColl', key: {Counter: 1}}));
@@ -76,7 +86,7 @@ while (docsInserted < numDocs) {
         docsInserted++;
     }
 
-    assert.writeOK(coll.insert(bulk));
+    assert.commandWorked(coll.insert(bulk));
 
     if (docsInserted % 10000 == 0) {
         print("Inserted " + docsInserted + " documents.");
