@@ -36,10 +36,10 @@ assert.commandWorked(
 st.printShardingStatus();
 
 // Insert some data into each of the consolidated ranges
-assert.writeOK(coll.insert({_id: 0}));
-assert.writeOK(coll.insert({_id: 10}));
-assert.writeOK(coll.insert({_id: 40}));
-assert.writeOK(coll.insert({_id: 110}));
+assert.commandWorked(coll.insert({_id: 0}));
+assert.commandWorked(coll.insert({_id: 10}));
+assert.commandWorked(coll.insert({_id: 40}));
+assert.commandWorked(coll.insert({_id: 110}));
 
 var staleCollection = staleMongos.getCollection(coll + "");
 
@@ -56,6 +56,12 @@ assert.commandFailed(admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 20
 assert.commandFailed(
     admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 105}, {_id: MaxKey}]}));
 
+// Make sure merging single chunks is invalid
+assert.commandFailed(admin.runCommand({mergeChunks: coll + "", bounds: [{_id: MinKey}, {_id: 0}]}));
+assert.commandFailed(admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 20}, {_id: 40}]}));
+assert.commandFailed(
+    admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 110}, {_id: MaxKey}]}));
+
 // Make sure merging over holes is invalid
 assert.commandFailed(admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 0}, {_id: 40}]}));
 assert.commandFailed(admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 40}, {_id: 110}]}));
@@ -69,32 +75,6 @@ assert.eq(4, staleCollection.find().itcount());
 
 jsTest.log("Trying merges that should succeed...");
 
-// Merging single chunks should be treated as a no-op
-// (or fail because 'the range specifies one single chunk' in multiversion test environments)
-try {
-    assert.commandWorked(
-        admin.runCommand({mergeChunks: coll + "", bounds: [{_id: MinKey}, {_id: 0}]}));
-} catch (e) {
-    if (!e.message.match(/could not merge chunks, collection .* already contains chunk for/)) {
-        throw e;
-    }
-}
-try {
-    assert.commandWorked(
-        admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 20}, {_id: 40}]}));
-} catch (e) {
-    if (!e.message.match(/could not merge chunks, collection .* already contains chunk for/)) {
-        throw e;
-    }
-}
-try {
-    assert.commandWorked(
-        admin.runCommand({mergeChunks: coll + "", bounds: [{_id: 110}, {_id: MaxKey}]}));
-} catch (e) {
-    if (!e.message.match(/could not merge chunks, collection .* already contains chunk for/)) {
-        throw e;
-    }
-}
 // Make sure merge including the MinKey works
 assert.commandWorked(
     admin.runCommand({mergeChunks: coll + "", bounds: [{_id: MinKey}, {_id: 10}]}));
