@@ -1,9 +1,3 @@
-/*
- * @tags: [
- *     requires_fcv_42, # autoSplitVector not present in older v4.0 binaries
- * ]
- */
-
 (function() {
 
 var s = new ShardingTest({name: "presplit", shards: 2, mongos: 1, other: {chunkSize: 1}});
@@ -25,7 +19,7 @@ while (inserted < (20 * 1024 * 1024)) {
     bulk.insert({_id: num++, s: bigString});
     inserted += bigString.length;
 }
-assert.writeOK(bulk.execute());
+assert.commandWorked(bulk.execute());
 
 // Make sure that there's only one chunk holding all the data.
 s.printChunks();
@@ -37,7 +31,7 @@ s.adminCommand({shardcollection: "test.foo", key: {_id: 1}});
 
 // Make sure the collection's original chunk got split
 s.printChunks();
-assert.lte(20, s.config.chunks.count({"ns": "test.foo"}), "many chunks assertion");
+assert.lt(20, s.config.chunks.count({"ns": "test.foo"}), "many chunks assertion");
 assert.eq(num, primary.foo.count());
 
 s.printChangeLog();
