@@ -13,8 +13,17 @@
  * - checkResults: A function that asserts whether the command should succeed or fail. If the
  *                 command is expected to succeed, the function should assert the expected results
  *                 *when the the collection has been dropped and recreated as empty.*
- * - behavior: Must be one of "unshardedOnly", "targetsPrimaryUsesConnectionVersioning" or
- * "versioned". Determines what system profiler checks are performed.
+ * - behavior: Must be "unshardedOnly", or "versioned". Determines what system profiler checks are
+ * performed.
+ *
+ * Tagged as 'requires_fcv_44', since this test cannot run against versions less then 4.4. This is
+ * because 'planCacheListPlans' and 'planCacheListQueryShapes' were deleted in 4.4, and thus not
+ * tested here. But this test asserts that all commands are covered, so will fail against a version
+ * of the server which implements these commands.
+ *
+ * @tags: [
+ *   requires_fcv_44,
+ * ]
  */
 (function() {
 "use strict";
@@ -31,22 +40,20 @@ let validateTestCase = function(test) {
     assert(test.setUp && typeof (test.setUp) === "function");
     assert(test.command && typeof (test.command) === "object");
     assert(test.checkResults && typeof (test.checkResults) === "function");
-    assert(test.behavior === "unshardedOnly" ||
-           test.behavior === "targetsPrimaryUsesConnectionVersioning" ||
-           test.behavior === "versioned");
+    assert(test.behavior === "unshardedOnly" || test.behavior === "versioned");
 };
 
 let testCases = {
     _addShard: {skip: "primary only"},
-    _cloneCatalogData: {skip: "primary only"},
+    _shardsvrCloneCatalogData: {skip: "primary only"},
     _configsvrAddShard: {skip: "primary only"},
     _configsvrAddShardToZone: {skip: "primary only"},
+    _configsvrBalancerCollectionStatus: {skip: "primary only"},
     _configsvrBalancerStart: {skip: "primary only"},
     _configsvrBalancerStatus: {skip: "primary only"},
     _configsvrBalancerStop: {skip: "primary only"},
     _configsvrClearJumboFlag: {skip: "primary only"},
     _configsvrCommitChunkMerge: {skip: "primary only"},
-    _configsvrCommitChunksMerge: {skip: "primary only"},
     _configsvrCommitChunkMigration: {skip: "primary only"},
     _configsvrCommitChunkSplit: {skip: "primary only"},
     _configsvrCommitMovePrimary: {skip: "primary only"},
@@ -61,9 +68,10 @@ let testCases = {
     _getUserCacheGeneration: {skip: "does not return user data"},
     _hashBSONElement: {skip: "does not return user data"},
     _isSelf: {skip: "does not return user data"},
+    _killOperations: {skip: "does not return user data"},
     _mergeAuthzCollections: {skip: "primary only"},
     _migrateClone: {skip: "primary only"},
-    _movePrimary: {skip: "primary only"},
+    _shardsvrMovePrimary: {skip: "primary only"},
     _recvChunkAbort: {skip: "primary only"},
     _recvChunkCommit: {skip: "primary only"},
     _recvChunkStart: {skip: "primary only"},
@@ -74,7 +82,7 @@ let testCases = {
     addShardToZone: {skip: "primary only"},
     aggregate: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {aggregate: coll, pipeline: [{$match: {x: 1}}], cursor: {batchSize: 10}},
         checkResults: function(res) {
@@ -88,6 +96,7 @@ let testCases = {
     authSchemaUpgrade: {skip: "primary only"},
     authenticate: {skip: "does not return user data"},
     availableQueryOptions: {skip: "does not return user data"},
+    balancerCollectionStatus: {skip: "primary only"},
     balancerStart: {skip: "primary only"},
     balancerStatus: {skip: "primary only"},
     balancerStop: {skip: "primary only"},
@@ -98,7 +107,6 @@ let testCases = {
     clearJumboFlag: {skip: "primary only"},
     clearLog: {skip: "does not return user data"},
     clone: {skip: "primary only"},
-    cloneCollection: {skip: "primary only"},
     cloneCollectionAsCapped: {skip: "primary only"},
     collMod: {skip: "primary only"},
     collStats: {skip: "does not return user data"},
@@ -111,7 +119,7 @@ let testCases = {
     convertToCapped: {skip: "primary only"},
     count: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {count: coll, query: {x: 1}},
         checkResults: function(res) {
@@ -132,8 +140,8 @@ let testCases = {
     delete: {skip: "primary only"},
     distinct: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {distinct: coll, key: "x"},
         checkResults: function(res) {
@@ -160,7 +168,7 @@ let testCases = {
     filemd5: {skip: "does not return user data"},
     find: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {find: coll, filter: {x: 1}},
         checkResults: function(res) {
@@ -176,6 +184,7 @@ let testCases = {
     fsyncUnlock: {skip: "does not return user data"},
     geoSearch: {skip: "not supported in mongos"},
     getCmdLineOpts: {skip: "does not return user data"},
+    getDefaultRWConcern: {skip: "does not return user data"},
     getDiagnosticData: {skip: "does not return user data"},
     getLastError: {skip: "primary only"},
     getLog: {skip: "does not return user data"},
@@ -189,7 +198,6 @@ let testCases = {
     grantRolesToRole: {skip: "primary only"},
     grantRolesToUser: {skip: "primary only"},
     handshake: {skip: "does not return user data"},
-    hello: {skip: "does not return user data"},
     hostInfo: {skip: "does not return user data"},
     insert: {skip: "primary only"},
     invalidateUserCache: {skip: "does not return user data"},
@@ -207,14 +215,13 @@ let testCases = {
     listShards: {skip: "does not return user data"},
     lockInfo: {skip: "primary only"},
     logApplicationMessage: {skip: "primary only"},
-    logMessage: {skip: "does not return user data"},
     logRotate: {skip: "does not return user data"},
     logout: {skip: "does not return user data"},
     makeSnapshot: {skip: "does not return user data"},
     mapReduce: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {
             mapReduce: coll,
@@ -230,7 +237,7 @@ let testCases = {
             assert.commandWorked(res);
             assert.eq(0, res.results.length, tojson(res));
         },
-        behavior: "targetsPrimaryUsesConnectionVersioning"
+        behavior: "versioned"
     },
     mergeChunks: {skip: "primary only"},
     moveChunk: {skip: "primary only"},
@@ -241,19 +248,16 @@ let testCases = {
     planCacheClear: {skip: "does not return user data"},
     planCacheClearFilters: {skip: "does not return user data"},
     planCacheListFilters: {skip: "does not return user data"},
-    planCacheListPlans: {skip: "does not return user data"},
-    planCacheListQueryShapes: {skip: "does not return user data"},
     planCacheSetFilter: {skip: "does not return user data"},
     profile: {skip: "primary only"},
     reapLogicalSessionCacheNow: {skip: "does not return user data"},
+    refineCollectionShardKey: {skip: "primary only"},
     refreshLogicalSessionCacheNow: {skip: "does not return user data"},
     refreshSessions: {skip: "does not return user data"},
     refreshSessionsInternal: {skip: "does not return user data"},
     removeShard: {skip: "primary only"},
     removeShardFromZone: {skip: "primary only"},
     renameCollection: {skip: "primary only"},
-    repairCursor: {skip: "does not return user data"},
-    repairShardedCollectionChunksHistory: {skip: "does not return user data"},
     replSetAbortPrimaryCatchUp: {skip: "does not return user data"},
     replSetFreeze: {skip: "does not return user data"},
     replSetGetConfig: {skip: "does not return user data"},
@@ -271,7 +275,6 @@ let testCases = {
     replSetUpdatePosition: {skip: "does not return user data"},
     replSetResizeOplog: {skip: "does not return user data"},
     resetError: {skip: "does not return user data"},
-    restartCatalog: {skip: "internal-only command"},
     resync: {skip: "primary only"},
     revokePrivilegesFromRole: {skip: "primary only"},
     revokeRolesFromRole: {skip: "primary only"},
@@ -280,8 +283,9 @@ let testCases = {
     saslContinue: {skip: "primary only"},
     saslStart: {skip: "primary only"},
     serverStatus: {skip: "does not return user data"},
-    setAllowMigrations: {skip: "primary only"},
     setCommittedSnapshot: {skip: "does not return user data"},
+    setDefaultRWConcern: {skip: "primary only"},
+    setIndexCommitQuorum: {skip: "primary only"},
     setFeatureCompatibilityVersion: {skip: "primary only"},
     setFreeMonitoring: {skip: "primary only"},
     setParameter: {skip: "does not return user data"},
@@ -299,7 +303,6 @@ let testCases = {
     startSession: {skip: "does not return user data"},
     stopRecordingTraffic: {skip: "does not return user data"},
     top: {skip: "does not return user data"},
-    touch: {skip: "does not return user data"},
     unsetSharding: {skip: "does not return user data"},
     update: {skip: "primary only"},
     updateRole: {skip: "primary only"},
@@ -316,8 +319,8 @@ let testCases = {
     createBackup: {skip: "does not return user data"},
 };
 
-commandsRemovedFromMongosIn42.forEach(function(cmd) {
-    testCases[cmd] = {skip: "must define test coverage for 4.0 backwards compatibility"};
+commandsRemovedFromMongosIn44.forEach(function(cmd) {
+    testCases[cmd] = {skip: "must define test coverage for 4.2 backwards compatibility"};
 });
 
 let scenarios = {
@@ -345,19 +348,6 @@ let scenarios = {
         if (test.behavior === "unshardedOnly") {
             profilerHasZeroMatchingEntriesOrThrow(
                 {profileDB: primaryShardSecondary.getDB(db), filter: commandProfile});
-        } else if (test.behavior === "targetsPrimaryUsesConnectionVersioning") {
-            // Check that the primary shard primary received the request without a shardVersion
-            // field and returned success.
-            profilerHasSingleMatchingEntryOrThrow({
-                profileDB: primaryShardPrimary.getDB(db),
-                filter: Object.extend({
-                    "command.shardVersion": {"$exists": false},
-                    "command.$readPreference": {$exists: false},
-                    "command.readConcern": {"level": "local"},
-                    "errCode": {"$exists": false}
-                },
-                                      commandProfile)
-            });
         } else if (test.behavior == "versioned") {
             // Check that the primary shard secondary returned stale shardVersion.
             profilerHasSingleMatchingEntryOrThrow({
@@ -415,19 +405,6 @@ let scenarios = {
         if (test.behavior === "unshardedOnly") {
             profilerHasZeroMatchingEntriesOrThrow(
                 {profileDB: primaryShardSecondary.getDB(db), filter: commandProfile});
-        } else if (test.behavior === "targetsPrimaryUsesConnectionVersioning") {
-            // Check that the primary shard primary received the request without a shardVersion
-            // field and returned success.
-            profilerHasSingleMatchingEntryOrThrow({
-                profileDB: primaryShardPrimary.getDB(db),
-                filter: Object.extend({
-                    "command.shardVersion": {"$exists": false},
-                    "command.$readPreference": {$exists: false},
-                    "command.readConcern": {"level": "local"},
-                    "errCode": {"$exists": false},
-                },
-                                      commandProfile)
-            });
         } else if (test.behavior == "versioned") {
             // Check that the primary shard secondary returned stale shardVersion.
             profilerHasSingleMatchingEntryOrThrow({
@@ -499,19 +476,6 @@ let scenarios = {
                 {profileDB: donorShardSecondary.getDB(db), filter: commandProfile});
             profilerHasZeroMatchingEntriesOrThrow(
                 {profileDB: recipientShardSecondary.getDB(db), filter: commandProfile});
-        } else if (test.behavior === "targetsPrimaryUsesConnectionVersioning") {
-            // Check that the recipient shard primary received the request without a
-            // shardVersion field and returned success.
-            profilerHasSingleMatchingEntryOrThrow({
-                profileDB: recipientShardPrimary.getDB(db),
-                filter: Object.extend({
-                    "command.shardVersion": {"$exists": false},
-                    "command.$readPreference": {$exists: false},
-                    "command.readConcern": {"level": "local"},
-                    "errCode": {"$exists": false},
-                },
-                                      commandProfile)
-            });
         } else if (test.behavior == "versioned") {
             // Check that the donor shard secondary returned stale shardVersion.
             profilerHasSingleMatchingEntryOrThrow({
