@@ -13,6 +13,8 @@
  * - setUp: A function that does any set up (inserts, etc.) needed to check the command's results.
  * - command: The command to run, with all required options. Note, this field is also used to
  *            identify the operation in the system profiler.
+ * - filter: [OPTIONAL] When specified, used instead of 'command' to identify the operation in the
+ *           system profiler.
  * - checkResults: A function that asserts whether the command should succeed or fail. If the
  *                 command is expected to succeed, the function should assert the expected results.
  *                 *when the range has not been deleted from the donor.*
@@ -20,6 +22,15 @@
  *                                     results for the command run with read concern 'available'.
  * - behavior: Must be one of "unshardedOnly", "targetsPrimaryUsesConnectionVersioning" or
  * "versioned". Determines what system profiler checks are performed.
+ *
+ * Tagged as 'requires_fcv_44', since this test cannot run against versions less then 4.4. This is
+ * because 'planCacheListPlans' and 'planCacheListQueryShapes' were deleted in 4.4, and thus not
+ * tested here. But this test asserts that all commands are covered, so will fail against a version
+ * of the server which implements these commands.
+ *
+ * @tags: [
+ *   requires_fcv_44,
+ * ]
  */
 (function() {
 "use strict";
@@ -45,15 +56,15 @@ let validateTestCase = function(test) {
 
 let testCases = {
     _addShard: {skip: "primary only"},
-    _cloneCatalogData: {skip: "primary only"},
+    _shardsvrCloneCatalogData: {skip: "primary only"},
     _configsvrAddShard: {skip: "primary only"},
     _configsvrAddShardToZone: {skip: "primary only"},
+    _configsvrBalancerCollectionStatus: {skip: "primary only"},
     _configsvrBalancerStart: {skip: "primary only"},
     _configsvrBalancerStatus: {skip: "primary only"},
     _configsvrBalancerStop: {skip: "primary only"},
     _configsvrClearJumboFlag: {skip: "primary only"},
     _configsvrCommitChunkMerge: {skip: "primary only"},
-    _configsvrCommitChunksMerge: {skip: "primary only"},
     _configsvrCommitChunkMigration: {skip: "primary only"},
     _configsvrCommitChunkSplit: {skip: "primary only"},
     _configsvrCommitMovePrimary: {skip: "primary only"},
@@ -68,9 +79,10 @@ let testCases = {
     _getUserCacheGeneration: {skip: "does not return user data"},
     _hashBSONElement: {skip: "does not return user data"},
     _isSelf: {skip: "does not return user data"},
+    _killOperations: {skip: "does not return user data"},
     _mergeAuthzCollections: {skip: "primary only"},
     _migrateClone: {skip: "primary only"},
-    _movePrimary: {skip: "primary only"},
+    _shardsvrMovePrimary: {skip: "primary only"},
     _recvChunkAbort: {skip: "primary only"},
     _recvChunkCommit: {skip: "primary only"},
     _recvChunkStart: {skip: "primary only"},
@@ -81,7 +93,7 @@ let testCases = {
     addShardToZone: {skip: "primary only"},
     aggregate: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {aggregate: coll, pipeline: [{$match: {x: 1}}], cursor: {batchSize: 10}},
         checkResults: function(res) {
@@ -101,6 +113,7 @@ let testCases = {
     authSchemaUpgrade: {skip: "primary only"},
     authenticate: {skip: "does not return user data"},
     availableQueryOptions: {skip: "does not return user data"},
+    balancerCollectionStatus: {skip: "primary only"},
     balancerStart: {skip: "primary only"},
     balancerStatus: {skip: "primary only"},
     balancerStop: {skip: "primary only"},
@@ -111,7 +124,6 @@ let testCases = {
     clearJumboFlag: {skip: "primary only"},
     clearLog: {skip: "does not return user data"},
     clone: {skip: "primary only"},
-    cloneCollection: {skip: "primary only"},
     cloneCollectionAsCapped: {skip: "primary only"},
     commitTransaction: {skip: "primary only"},
     collMod: {skip: "primary only"},
@@ -124,7 +136,7 @@ let testCases = {
     convertToCapped: {skip: "primary only"},
     count: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {count: coll, query: {x: 1}},
         checkResults: function(res) {
@@ -151,8 +163,8 @@ let testCases = {
     delete: {skip: "primary only"},
     distinct: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {distinct: coll, key: "x"},
         checkResults: function(res) {
@@ -184,7 +196,7 @@ let testCases = {
     filemd5: {skip: "does not return user data"},
     find: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {find: coll, filter: {x: 1}},
         checkResults: function(res) {
@@ -206,6 +218,7 @@ let testCases = {
     fsyncUnlock: {skip: "does not return user data"},
     geoSearch: {skip: "not supported in mongos"},
     getCmdLineOpts: {skip: "does not return user data"},
+    getDefaultRWConcern: {skip: "does not return user data"},
     getDiagnosticData: {skip: "does not return user data"},
     getLastError: {skip: "primary only"},
     getLog: {skip: "does not return user data"},
@@ -219,7 +232,6 @@ let testCases = {
     grantRolesToRole: {skip: "primary only"},
     grantRolesToUser: {skip: "primary only"},
     handshake: {skip: "does not return user data"},
-    hello: {skip: "does not return user data"},
     hostInfo: {skip: "does not return user data"},
     insert: {skip: "primary only"},
     invalidateUserCache: {skip: "does not return user data"},
@@ -237,14 +249,13 @@ let testCases = {
     listShards: {skip: "does not return user data"},
     lockInfo: {skip: "primary only"},
     logApplicationMessage: {skip: "primary only"},
-    logMessage: {skip: "does not return user data"},
     logRotate: {skip: "does not return user data"},
     logout: {skip: "does not return user data"},
     makeSnapshot: {skip: "does not return user data"},
     mapReduce: {
         setUp: function(mongosConn) {
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
-            assert.writeOK(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
+            assert.commandWorked(mongosConn.getCollection(nss).insert({x: 1}));
         },
         command: {
             mapReduce: coll,
@@ -256,6 +267,36 @@ let testCases = {
             },
             out: {inline: 1}
         },
+        filter: {
+            aggregate: coll,
+            "pipeline": [
+                {
+                    "$project": {
+                        "emits": {
+                            "$_internalJsEmit": {
+                                "eval":
+                                    "function() {\n                emit(this.x, 1);\n            }",
+                                "this": "$$ROOT"
+                            }
+                        },
+                        "_id": false
+                    }
+                },
+                {"$unwind": {"path": "$emits"}},
+                {
+                    "$group": {
+                        "_id": "$emits.k",
+                        "value": {
+                            "$_internalJsReduce": {
+                                "data": "$emits",
+                                "eval":
+                                    "function(key, values) {\n                return Array.sum(values);\n            }"
+                            }
+                        }
+                    }
+                }
+            ],
+        },
         checkResults: function(res) {
             assert.commandWorked(res);
             assert.eq(1, res.results.length, tojson(res));
@@ -263,9 +304,12 @@ let testCases = {
             assert.eq(2, res.results[0].value, tojson(res));
         },
         checkAvailableReadConcernResults: function(res) {
-            assert.commandFailed(res);
+            assert.commandWorked(res);
+            assert.eq(1, res.results.length, tojson(res));
+            assert.eq(1, res.results[0]._id, tojson(res));
+            assert.eq(2, res.results[0].value, tojson(res));
         },
-        behavior: "targetsPrimaryUsesConnectionVersioning"
+        behavior: "versioned"
     },
     mergeChunks: {skip: "primary only"},
     moveChunk: {skip: "primary only"},
@@ -276,19 +320,16 @@ let testCases = {
     planCacheClear: {skip: "does not return user data"},
     planCacheClearFilters: {skip: "does not return user data"},
     planCacheListFilters: {skip: "does not return user data"},
-    planCacheListPlans: {skip: "does not return user data"},
-    planCacheListQueryShapes: {skip: "does not return user data"},
     planCacheSetFilter: {skip: "does not return user data"},
     profile: {skip: "primary only"},
     reapLogicalSessionCacheNow: {skip: "does not return user data"},
+    refineCollectionShardKey: {skip: "primary only"},
     refreshLogicalSessionCacheNow: {skip: "does not return user data"},
     refreshSessions: {skip: "does not return user data"},
     refreshSessionsInternal: {skip: "does not return user data"},
     removeShard: {skip: "primary only"},
     removeShardFromZone: {skip: "primary only"},
     renameCollection: {skip: "primary only"},
-    repairCursor: {skip: "does not return user data"},
-    repairShardedCollectionChunksHistory: {skip: "does not return user data"},
     replSetAbortPrimaryCatchUp: {skip: "does not return user data"},
     replSetFreeze: {skip: "does not return user data"},
     replSetGetConfig: {skip: "does not return user data"},
@@ -306,7 +347,6 @@ let testCases = {
     replSetUpdatePosition: {skip: "does not return user data"},
     replSetResizeOplog: {skip: "does not return user data"},
     resetError: {skip: "does not return user data"},
-    restartCatalog: {skip: "internal-only command"},
     resync: {skip: "primary only"},
     revokePrivilegesFromRole: {skip: "primary only"},
     revokeRolesFromRole: {skip: "primary only"},
@@ -315,8 +355,9 @@ let testCases = {
     saslContinue: {skip: "primary only"},
     saslStart: {skip: "primary only"},
     serverStatus: {skip: "does not return user data"},
-    setAllowMigrations: {skip: "primary only"},
     setCommittedSnapshot: {skip: "does not return user data"},
+    setDefaultRWConcern: {skip: "primary only"},
+    setIndexCommitQuorum: {skip: "primary only"},
     setFeatureCompatibilityVersion: {skip: "primary only"},
     setFreeMonitoring: {skip: "primary only"},
     setParameter: {skip: "does not return user data"},
@@ -334,7 +375,6 @@ let testCases = {
     startSession: {skip: "does not return user data"},
     stopRecordingTraffic: {skip: "does not return user data"},
     top: {skip: "does not return user data"},
-    touch: {skip: "does not return user data"},
     unsetSharding: {skip: "does not return user data"},
     update: {skip: "primary only"},
     updateRole: {skip: "primary only"},
@@ -351,8 +391,8 @@ let testCases = {
     createBackup: {skip: "does not return user data"},
 };
 
-commandsRemovedFromMongosIn42.forEach(function(cmd) {
-    testCases[cmd] = {skip: "must define test coverage for 4.0 backwards compatibility"};
+commandsRemovedFromMongosIn44.forEach(function(cmd) {
+    testCases[cmd] = {skip: "must define test coverage for 4.2 backwards compatibility"};
 });
 
 // Set the secondaries to priority 0 to prevent the primaries from stepping down.
@@ -441,19 +481,19 @@ for (let command of commands) {
     test.checkAvailableReadConcernResults(availableReadConcernRes);
 
     let defaultReadConcernRes = staleMongos.getDB(db).runCommand(cmdReadPrefSecondary);
-    if (command === 'mapReduce') {
-        // mapReduce is always sent to a primary, which defaults to 'local' readConcern
-        test.checkResults(defaultReadConcernRes);
-    } else {
-        // Secondaries default to the 'available' readConcern
-        test.checkAvailableReadConcernResults(defaultReadConcernRes);
-    }
+    // Secondaries default to the 'available' readConcern
+    test.checkAvailableReadConcernResults(defaultReadConcernRes);
 
     let localReadConcernRes = staleMongos.getDB(db).runCommand(cmdPrefSecondaryConcernLocal);
     test.checkResults(localReadConcernRes);
 
     // Build the query to identify the operation in the system profiler.
-    let commandProfile = buildCommandProfile(test.command, true /* sharded */);
+    let filter = test.command;
+    if (test.filter != undefined) {
+        filter = test.filter;
+    }
+
+    let commandProfile = buildCommandProfile(filter, true /* sharded */);
 
     if (test.behavior === "unshardedOnly") {
         // Check that neither the donor nor recipient shard secondaries received either request.
@@ -493,7 +533,10 @@ for (let command of commands) {
             filter: Object.extend({
                 "command.shardVersion": {"$exists": true},
                 "command.$readPreference": {"mode": "secondary"},
-                "command.readConcern": {"$exists": false},
+                "$or": [
+                    {"command.readConcern": {"$exists": false}},
+                    {"command.readConcern": {"provenance": "implicitDefault"}},
+                ],
                 "errCode": {"$ne": ErrorCodes.StaleConfig},
             },
                                   commandProfile)
