@@ -1,9 +1,3 @@
-/*
- * @tags: [
- *     requires_fcv_42, # autoSplitVector not present in older v4.0 binaries
- * ]
- */
-
 (function() {
 'use strict';
 
@@ -23,7 +17,7 @@ var bulk = db.data.initializeUnorderedBulkOp();
 for (var i = 0; i < numDocs; i++) {
     bulk.insert({_id: i, s: bigString});
 }
-assert.writeOK(bulk.execute());
+assert.commandWorked(bulk.execute());
 
 var avgObjSize = db.data.stats().avgObjSize;
 var dataSize = db.data.stats().size;
@@ -34,10 +28,10 @@ s.ensurePrimaryShard('test', s.shard1.shardName);
 var res = s.adminCommand({shardcollection: "test.data", key: {_id: 1}});
 printjson(res);
 
-// number of chunks should be approx equal to the total data size / chunk size
+// number of chunks should be approx equal to the total data size / half the chunk size
 var numChunks = s.config.chunks.find({ns: 'test.data'}).itcount();
-var guess = Math.ceil(dataSize / (1024 * 1024 + avgObjSize));
-assert.lte(Math.abs(numChunks - guess), 2, "not right number of chunks");
+var guess = Math.ceil(dataSize / (512 * 1024 + avgObjSize));
+assert(Math.abs(numChunks - guess) < 2, "not right number of chunks");
 
 s.stop();
 })();
