@@ -1,4 +1,5 @@
 // Check if this build supports the authenticationMechanisms startup parameter.
+load("jstests/libs/logv2_helpers.js");
 var conn = MongoRunner.runMongod({
     auth: "",
     sslMode: "requireSSL",
@@ -18,6 +19,8 @@ MongoRunner.stopMongod(conn);
 const SERVER_CERT = "jstests/libs/server.pem";
 const CA_CERT = "jstests/libs/ca.pem";
 
+const SERVER_USER = "C=US,ST=New York,L=New York City,O=MongoDB,OU=Kernel,CN=server";
+const INTERNAL_USER = "C=US,ST=New York,L=New York City,O=MongoDB,OU=Kernel,CN=internal";
 const CLIENT_USER = "CN=client,OU=KernelUser,O=MongoDB,L=New York City,ST=New York,C=US";
 const INVALID_CLIENT_USER = "C=US,ST=New York,L=New York City,O=MongoDB,OU=KernelUser,CN=invalid";
 
@@ -25,6 +28,18 @@ function authAndTest(mongo) {
     external = mongo.getDB("$external");
     test = mongo.getDB("test");
 
+    // It should be impossible to create users with the same name as the server's subject
+    assert.throws(function() {
+        external.createUser(
+            {user: SERVER_USER, roles: [{'role': 'userAdminAnyDatabase', 'db': 'admin'}]});
+    }, [], "Created user with same name as the server's x.509 subject");
+
+    // It should be impossible to create users with names recognized as cluster members
+    assert.throws(function() {
+        external.createUser(
+            {user: INTERNAL_USER, roles: [{'role': 'userAdminAnyDatabase', 'db': 'admin'}]});
+    }, [], "Created user which would be recognized as a cluster member");
+
     // Add user using localhost exception
     external.createUser({
         user: CLIENT_USER,
@@ -35,6 +50,12 @@ function authAndTest(mongo) {
         ]
     });
 
+    // It should be impossible to create users with an internal name
+    assert.throws(function() {
+        external.createUser(
+            {user: SERVER_USER, roles: [{'role': 'userAdminAnyDatabase', 'db': 'admin'}]});
+    });
+
     // Localhost exception should not be in place anymore
     assert.throws(function() {
         test.foo.findOne();
@@ -54,28 +75,23 @@ function authAndTest(mongo) {
     // Check that there's a "Successfully authenticated" message that includes the client IP
     const log =
         assert.commandWorked(external.getSiblingDB("admin").runCommand({getLog: "global"})).log;
-    const successRegex = new RegExp(`Successfully authenticated as principal ${CLIENT_USER} on ` +
-                                    `\\$external from client (?:\\d{1,3}\\.){3}\\d{1,3}:\\d+`);
 
-    assert(log.some((line) => successRegex.test(line)));
+    if (isJsonLog(mongo)) {
+        function checkAuthSuccess(element, index, array) {
+            const logJson = JSON.parse(element);
 
-    let createServerUser = function() {
-        // It should be impossible to create users with the same name as the server's subject,
-        // unless guardrails are explicitly overridden
-        external.createUser(
-            {user: SERVER_USER, roles: [{'role': 'userAdminAnyDatabase', 'db': 'admin'}]});
-    };
-    assert.throws(
-        createServerUser, [], "Created user with same name as the server's x.509 subject");
+            return logJson.id === 20429 && logJson.attr.user === CLIENT_USER &&
+                logJson.attr.db === "$external" &&
+                /(?:\d{1,3}\.){3}\d{1,3}:\d+/.test(logJson.attr.client);
+        }
+        assert(log.some(checkAuthSuccess));
+    } else {
+        const successRegex =
+            new RegExp(`Successfully authenticated as principal ${CLIENT_USER} on ` +
+                       `\\$external from client (?:\\d{1,3}\\.){3}\\d{1,3}:\\d+`);
 
-    let createInternalUser = function() {
-        // It should be impossible to create users with names recognized as cluster members,
-        // unless guardrails are explicitly overridden
-        external.createUser(
-            {user: INTERNAL_USER, roles: [{'role': 'userAdminAnyDatabase', 'db': 'admin'}]});
-    };
-    assert.throws(
-        createInternalUser, [], "Created user which would be recognized as a cluster member");
+        assert(log.some((line) => successRegex.test(line)));
+    }
 
     // Check that we can add a user and read data
     test.createUser(
@@ -88,34 +104,27 @@ function authAndTest(mongo) {
     }, [], "read after logout");
 }
 
-const x509_options = {
-    sslMode: "requireSSL",
-    sslPEMKeyFile: SERVER_CERT,
-    sslCAFile: CA_CERT
-};
+print("1. Testing x.509 auth to mongod");
+var x509_options = {sslMode: "requireSSL", sslPEMKeyFile: SERVER_CERT, sslCAFile: CA_CERT};
 
-{
-    print("1. Testing x.509 auth to mongod");
-    const mongo = MongoRunner.runMongod(Object.merge(x509_options, {auth: ""}));
+var mongo = MongoRunner.runMongod(Object.merge(x509_options, {auth: ""}));
 
-    authAndTest(mongo);
-    MongoRunner.stopMongod(mongo);
-}
+authAndTest(mongo);
+MongoRunner.stopMongod(mongo);
 
-{
-    print("2. Testing x.509 auth to mongos");
-    var st = new ShardingTest({
-        shards: 1,
-        mongos: 1,
-        other: {
-            keyFile: 'jstests/libs/key1',
-            configOptions: x509_options,
-            mongosOptions: x509_options,
-            shardOptions: x509_options,
-            useHostname: false,
-        }
-    });
+print("2. Testing x.509 auth to mongos");
 
-    authAndTest(new Mongo("localhost:" + st.s0.port));
-    st.stop();
-}
+var st = new ShardingTest({
+    shards: 1,
+    mongos: 1,
+    other: {
+        keyFile: 'jstests/libs/key1',
+        configOptions: x509_options,
+        mongosOptions: x509_options,
+        shardOptions: x509_options,
+        useHostname: false
+    }
+});
+
+authAndTest(new Mongo("localhost:" + st.s0.port));
+st.stop();
