@@ -96,7 +96,6 @@ add_percona_yum_repo(){
     return
 }
 
-
 get_sources(){
     cd "${WORKDIR}"
     if [ "${SOURCE}" = 0 ]
@@ -105,13 +104,11 @@ get_sources(){
         return 0
     fi
     PRODUCT=percona-server-mongodb
-    JEMALLOC_TAG=psmdb-3.2.11-3.1
-
-    echo "PRODUCT=${PRODUCT}" > percona-server-mongodb-42.properties
-    echo "PSM_BRANCH=${PSM_BRANCH}" >> percona-server-mongodb-42.properties
-    echo "JEMALLOC_TAG=${JEMALLOC_TAG}" >> percona-server-mongodb-42.properties
-    echo "BUILD_NUMBER=${BUILD_NUMBER}" >> percona-server-mongodb-42.properties
-    echo "BUILD_ID=${BUILD_ID}" >> percona-server-mongodb-42.properties
+    echo "PRODUCT=${PRODUCT}" > percona-server-mongodb-44.properties
+    echo "PSM_BRANCH=${PSM_BRANCH}" >> percona-server-mongodb-44.properties
+    echo "JEMALLOC_TAG=${JEMALLOC_TAG}" >> percona-server-mongodb-44.properties
+    echo "BUILD_NUMBER=${BUILD_NUMBER}" >> percona-server-mongodb-44.properties
+    echo "BUILD_ID=${BUILD_ID}" >> percona-server-mongodb-44.properties
     git clone "$REPO"
     retval=$?
     if [ $retval != 0 ]
@@ -146,12 +143,12 @@ get_sources(){
     #
 
     PRODUCT_FULL=${PRODUCT}-${PSM_VER}-${PSM_RELEASE}
-    echo "PRODUCT_FULL=${PRODUCT_FULL}" >> ${WORKDIR}/percona-server-mongodb-42.properties
-    echo "VERSION=${PSM_VER}" >> ${WORKDIR}/percona-server-mongodb-42.properties
-    echo "RELEASE=${PSM_RELEASE}" >> ${WORKDIR}/percona-server-mongodb-42.properties
-    echo "MONGO_TOOLS_TAG=${MONGO_TOOLS_TAG}" >> ${WORKDIR}/percona-server-mongodb-42.properties
+    echo "PRODUCT_FULL=${PRODUCT_FULL}" >> ${WORKDIR}/percona-server-mongodb-44.properties
+    echo "VERSION=${PSM_VER}" >> ${WORKDIR}/percona-server-mongodb-44.properties
+    echo "RELEASE=${PSM_RELEASE}" >> ${WORKDIR}/percona-server-mongodb-44.properties
+    echo "MONGO_TOOLS_TAG=${MONGO_TOOLS_TAG}" >> ${WORKDIR}/percona-server-mongodb-44.properties
 
-    echo "REVISION=${REVISION}" >> ${WORKDIR}/percona-server-mongodb-42.properties
+    echo "REVISION=${REVISION}" >> ${WORKDIR}/percona-server-mongodb-44.properties
     rm -fr debian rpm
     cp -a percona-packaging/manpages .
     cp -a percona-packaging/docs/* .
@@ -163,14 +160,13 @@ get_sources(){
     git clone https://github.com/mongodb/mongo-tools.git
     cd mongo-tools
     git checkout $MONGO_TOOLS_TAG
+    sed -i 's|VersionStr="$(go run release/release.go get-version)"|VersionStr="$PSMDB_TOOLS_REVISION"|' set_goenv.sh
+    sed -i 's|GitCommit="$(git rev-parse HEAD)"|GitCommit="$PSMDB_TOOLS_COMMIT_HASH"|' set_goenv.sh
     echo "export PSMDB_TOOLS_COMMIT_HASH=\"$(git rev-parse HEAD)\"" > set_tools_revision.sh
     echo "export PSMDB_TOOLS_REVISION=\"${PSM_VER}-${PSM_RELEASE}\"" >> set_tools_revision.sh
     chmod +x set_tools_revision.sh
     cd ${WORKDIR}
-    #
-    #source ${WORKDIR}/percona-server-mongodb-40.properties
-    #
-    source percona-server-mongodb-42.properties
+    source percona-server-mongodb-44.properties
     #
 
     mv percona-server-mongodb ${PRODUCT}-${PSM_VER}-${PSM_RELEASE}
@@ -179,12 +175,12 @@ get_sources(){
             cd aws-sdk-cpp
                 git reset --hard
                 git clean -xdf
-                git checkout 1.9.379
-                git submodule update --init --recursive
+                git checkout 1.7.91
                 mkdir build
+                sed -i 's:AWS_EVENT_STREAM_SHA:AWS_EVENT_STREAM_TAG:g' third-party/cmake/BuildAwsEventStream.cmake
     cd ../../
     tar --owner=0 --group=0 --exclude=.* -czf ${PRODUCT}-${PSM_VER}-${PSM_RELEASE}.tar.gz ${PRODUCT}-${PSM_VER}-${PSM_RELEASE}
-    echo "UPLOAD=UPLOAD/experimental/BUILDS/${PRODUCT}-4.2/${PRODUCT}-${PSM_VER}-${PSM_RELEASE}/${PSM_BRANCH}/${REVISION}/${BUILD_ID}" >> percona-server-mongodb-42.properties
+    echo "UPLOAD=UPLOAD/experimental/BUILDS/${PRODUCT}-4.4/${PRODUCT}-${PSM_VER}-${PSM_RELEASE}/${PSM_BRANCH}/${REVISION}/${BUILD_ID}" >> percona-server-mongodb-44.properties
     mkdir $WORKDIR/source_tarball
     mkdir $CURDIR/source_tarball
     cp ${PRODUCT}-${PSM_VER}-${PSM_RELEASE}.tar.gz $WORKDIR/source_tarball
@@ -211,25 +207,6 @@ get_system(){
     return
 }
 
-install_python3_7_12() {
-    if [ x"${DEBIAN}" = xxenial ]; then
-        wget https://www.python.org/ftp/python/3.7.12/Python-3.7.12.tgz -O /tmp/Python-3.7.12.tgz
-        CUR_DIR=$PWD
-        cd /tmp
-        tar -zxf Python-3.7.12.tgz
-        cd Python-3.7.12/
-        ./configure --enable-optimizations
-        make -j${NCPU} build_all
-        make altinstall
-        cd $CUR_DIR
-    fi
-}
-
-switch_to_vault_repo() {
-    sed -i 's/mirrorlist/#mirrorlist/g' /etc/yum.repos.d/CentOS-Linux-*
-    sed -i 's|#baseurl=http://mirror.centos.org|baseurl=http://vault.centos.org|g' /etc/yum.repos.d/CentOS-Linux-*
-}
-
 install_golang() {
     wget https://dl.google.com/go/go1.11.4.linux-amd64.tar.gz -O /tmp/golang1.11.tar.gz
     tar --transform=s,go,go1.11, -zxf /tmp/golang1.11.tar.gz
@@ -266,15 +243,6 @@ install_gcc_8_deb(){
     if [ x"${DEBIAN}" = xfocal -o x"${DEBIAN}" = xbionic -o x"${DEBIAN}" = xdisco -o x"${DEBIAN}" = xbuster ]; then
         apt-get -y install gcc-8 g++-8
     fi
-    if [ x"${DEBIAN}" = xbullseye ]; then
-        wget https://jenkins.percona.com/downloads/gcc8/gcc-8.3.0_Debian-bullseye-x64.tar.gz -O /tmp/gcc-8.3.0_Debian-bullseye-x64.tar.gz
-        tar -zxf /tmp/gcc-8.3.0_Debian-bullseye-x64.tar.gz
-        rm -rf /usr/local/gcc-8.3.0
-        mv gcc-8.3.0 /usr/local/
-    fi
-    if [ x"${DEBIAN}" = xjammy ]; then
-        apt-get -y install gcc-10 g++-10
-    fi
     if [ x"${DEBIAN}" = xstretch ]; then
         wget https://jenkins.percona.com/downloads/gcc8/gcc-8.3.0_Debian-stretch-x64.tar.gz -O /tmp/gcc-8.3.0_Debian-stretch-x64.tar.gz
         tar -zxf /tmp/gcc-8.3.0_Debian-stretch-x64.tar.gz
@@ -288,12 +256,6 @@ set_compiler(){
         if [ x"${DEBIAN}" = xfocal -o x"${DEBIAN}" = xbionic -o x"${DEBIAN}" = xdisco -o x"${DEBIAN}" = xbuster ]; then
             export CC=/usr/bin/gcc-8
             export CXX=/usr/bin/g++-8
-	elif [ x"${DEBIAN}" = xbullseye ]; then
-            export CC=/usr/local/gcc-8.3.0/bin/gcc-8.3
-            export CXX=/usr/local/gcc-8.3.0/bin/g++-8.3
-        elif [ x"${DEBIAN}" = xjammy ]; then
-            export CC=/usr/bin/gcc-10
-            export CXX=/usr/bin/g++-10
         else
             export CC=/usr/local/gcc-8.3.0/bin/gcc-8.3
             export CXX=/usr/local/gcc-8.3.0/bin/g++-8.3
@@ -313,12 +275,6 @@ fix_rules(){
     if [ x"${DEBIAN}" = xfocal -o x"${DEBIAN}" = xbionic -o x"${DEBIAN}" = xdisco -o x"${DEBIAN}" = xbuster ]; then
         sed -i 's|CC = gcc-5|CC = /usr/bin/gcc-8|' debian/rules
         sed -i 's|CXX = g++-5|CXX = /usr/bin/g++-8|' debian/rules
-    elif [ x"${DEBIAN}" = xbullseye ]; then
-        sed -i 's|CC = gcc-5|CC = /usr/local/gcc-8.3.0/bin/gcc-8.3|' debian/rules
-        sed -i 's|CXX = g++-5|CXX = /usr/local/gcc-8.3.0/bin/g++-8.3|' debian/rules
-    elif [ x"${DEBIAN}" = xjammy ]; then
-        sed -i 's|CC = gcc-5|CC = /usr/bin/gcc-10|' debian/rules
-        sed -i 's|CXX = g++-5|CXX = /usr/bin/g++-10|' debian/rules
     else
         sed -i 's|CC = gcc-5|CC = /usr/local/gcc-8.3.0/bin/gcc-8.3|' debian/rules
         sed -i 's|CXX = g++-5|CXX = /usr/local/gcc-8.3.0/bin/g++-8.3|' debian/rules
@@ -332,28 +288,24 @@ aws_sdk_build(){
         cd aws-sdk-cpp
             git reset --hard
             git clean -xdf
-            git checkout 1.9.379
-            git submodule update --init --recursive
+            git checkout 1.7.91
             mkdir build
+            sed -i 's:AWS_EVENT_STREAM_SHA:AWS_EVENT_STREAM_TAG:g' third-party/cmake/BuildAwsEventStream.cmake
             cd build
             CMAKE_CMD="cmake"
             if [ -f /etc/redhat-release ]; then
                 RHEL=$(rpm --eval %rhel)
-                if [ x"$RHEL" = x7 ]; then
+                if [ x"$RHEL" = x6 ]; then
                     CMAKE_CMD="cmake3"
                 fi
             fi
             set_compiler
-            CMAKE_CXX_FLAGS=""
-            if [ x"${DEBIAN}" = xjammy ]; then
-                CMAKE_CXX_FLAGS="-Wno-maybe-uninitialized -Wno-error=deprecated-declarations -Wno-error=uninitialized "
-            fi
             if [ -z "${CC}" -a -z "${CXX}" ]; then
-                ${CMAKE_CMD} .. -DCMAKE_CXX_FLAGS="${CMAKE_CXX_FLAGS}" -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3;transfer" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON || exit $?
+                ${CMAKE_CMD} .. -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON || exit $?
             else
-                ${CMAKE_CMD} CC=${CC} CXX=${CXX} .. -DCMAKE_CXX_FLAGS="${CMAKE_CXX_FLAGS}" -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3;transfer" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON || exit $?
+                ${CMAKE_CMD} CC=${CC} CXX=${CXX} .. -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON || exit $?
             fi
-            make -j${NCPU} || exit $?
+            make -j4 || exit $?
             make install
     cd ${WORKDIR}
 }
@@ -371,10 +323,6 @@ install_deps() {
     fi
     CURPLACE=$(pwd)
     if [ "x$OS" = "xrpm" ]; then
-      if [ x"$RHEL" = x8 ]; then
-          switch_to_vault_repo
-      fi
-      yum -y update
       yum -y install wget
       add_percona_yum_repo
       wget http://jenkins.percona.com/yum-repo/percona-dev.repo
@@ -392,48 +340,38 @@ install_deps() {
         yum -y install scons make rpm-build rpmbuild percona-devtoolset-gcc percona-devtoolset-binutils 
         yum -y install percona-devtoolset-gcc-c++ percona-devtoolset-libstdc++-devel percona-devtoolset-valgrind-devel
         yum -y install python27 python27-devel rpmlint libcurl-devel e2fsprogs-devel expat-devel lz4-devel git cmake3
-        yum -y install openldap-devel krb5-devel
-        wget https://bootstrap.pypa.io/get-pip.py -O get-pip.py
+        yum -y install openldap-devel krb5-devel xz-devel
+        wget https://bootstrap.pypa.io/get-pip.py
         python2.7 get-pip.py
         rm -rf /usr/bin/python2
         ln -s /usr/bin/python2.7 /usr/bin/python2
+        wget http://curl.haxx.se/download/curl-7.26.0.tar.gz
+        tar -xvzf curl-7.26.0.tar.gz
+        cd curl-7.26.0
+          ./configure
+          make
+          make install
+        cd ../
       elif [ x"$RHEL" = x7 ]; then
         yum -y install epel-release
         yum -y install rpmbuild rpm-build libpcap-devel gcc make cmake gcc-c++ openssl-devel
         yum -y install cyrus-sasl-devel snappy-devel zlib-devel bzip2-devel scons rpmlint
-        yum -y install rpm-build git python-pip python-devel libopcodes libcurl-devel rpmlint e2fsprogs-devel expat-devel lz4-devel which
-        yum -y install openldap-devel krb5-devel
-        pip install --upgrade pip==20.3.4
-        pip3.6 install --user typing pyyaml regex Cheetah3
-        pip2.7 install --user "setuptools==44.1.1" --upgrade # https://setuptools.pypa.io/en/latest/python%202%20sunset.html
-        pip2.7 install --user "Markdown==3.1" # https://github.com/Python-Markdown/markdown/issues/929
-        pip2.7 install --user typing Cheetah Cheetah3
-        pip2.7 install --user "pyyaml==5.4.1" # https://github.com/yaml/pyyaml/issues/476
-        pip2.7 install --user "regex==2021.11.10"
+        yum -y install rpm-build git python-pip python-devel libopcodes libcurl-devel rpmlint e2fsprogs-devel expat-devel lz4-devel
+        yum -y install openldap-devel krb5-devel xz-devel
       else
         yum -y install bzip2-devel libpcap-devel snappy-devel gcc gcc-c++ rpm-build rpmlint
         yum -y install cmake cyrus-sasl-devel make openssl-devel zlib-devel libcurl-devel git
-        yum -y install python2-scons python2-pip python36-devel which
+        yum -y install python2-scons python2-pip python36-devel
         yum -y install redhat-rpm-config python2-devel e2fsprogs-devel expat-devel lz4-devel
-        yum -y install openldap-devel krb5-devel
+        yum -y install openldap-devel krb5-devel xz-devel
       fi
-      install_gcc_8_centos
       if [ "x${RHEL}" == "x8" ]; then
         /usr/bin/pip3.6 install --user typing pyyaml regex Cheetah3
-        /usr/bin/pip2.7 install --user typing Cheetah
-        /usr/bin/pip2.7 install --user "pyyaml==5.4.1" # https://github.com/yaml/pyyaml/issues/476
-        /usr/bin/pip2.7 install --user "regex==2021.11.10"
+        /usr/bin/pip2.7 install --user typing pyyaml regex Cheetah
       fi
 #
-      wget http://curl.haxx.se/download/curl-7.66.0.tar.gz
-      tar -xvzf curl-7.66.0.tar.gz
-      cd curl-7.66.0
-        ./configure
-        make
-        make install
-      cd ../
       install_golang
-#      install_gcc_8_centos
+      install_gcc_8_centos
       if [ -f /opt/rh/devtoolset-8/enable ]; then
         source /opt/rh/devtoolset-8/enable
         source /opt/rh/rh-python36/enable
@@ -441,21 +379,11 @@ install_deps() {
       pip install --upgrade pip
 
     else
-      apt-get -y update
-      DEBIAN_FRONTEND=noninteractive apt-get -y install curl wget lsb-release apt-transport-https software-properties-common
+      apt-get -y install lsb-release wget apt-transport-https
       export DEBIAN=$(lsb_release -sc)
       export ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
-      if [ x"${DEBIAN}" = xjessie ]; then
-        cat >/etc/apt/sources.list.d/stretch.list <<EOL
-deb http://httpredir.debian.org/debian stretch main
-EOL
-        apt-get -y update
-        DEBIAN_FRONTEND=noninteractive apt-get -y install cmake
-        rm -f /etc/apt/sources.list.d/stretch.list
-        apt-get -y update
-      fi
       wget https://repo.percona.com/apt/percona-release_latest.$(lsb_release -sc)_all.deb && dpkg -i percona-release_latest.$(lsb_release -sc)_all.deb
-      if [ x"${DEBIAN}" = "xbionic" -o x"${DEBIAN}" = "xfocal" ]; then
+      if [ x"${DEBIAN}" = "xxenial" -o x"${DEBIAN}" = "xbionic" -o x"${DEBIAN}" = "xfocal" ]; then
         add-apt-repository -y ppa:deadsnakes/ppa
       elif [ x"${DEBIAN}" = "xstretch" -o x"${DEBIAN}" = "xbuster" ]; then
         wget https://people.debian.org/~paravoid/python-all/unofficial-python-all.asc
@@ -464,16 +392,9 @@ EOL
       fi
       percona-release enable tools testing
       apt-get update
-      if [ x"${DEBIAN}" = "xbullseye" -o x"${DEBIAN}" = "xjammy" ]; then
-        INSTALL_LIST="python3 python3-dev python3-pip"
-      elif [ x"${DEBIAN}" = "xxenial" ]; then
-        INSTALL_LIST="build-essential zlib1g-dev libncurses5-dev libgdbm-dev libnss3-dev libreadline-dev libffi-dev libsqlite3-dev"
-      else
-        INSTALL_LIST="python3.7 python3.7-dev dh-systemd"
-      fi
-      INSTALL_LIST="${INSTALL_LIST} git valgrind scons liblz4-dev devscripts debhelper debconf libpcap-dev libbz2-dev libsnappy-dev pkg-config zlib1g-dev libzlcore-dev libsasl2-dev gcc g++ cmake curl"
-      INSTALL_LIST="${INSTALL_LIST} libssl-dev libcurl4-openssl-dev libldap2-dev libkrb5-dev liblzma-dev patchelf libexpat1-dev"
-      if [ x"${DEBIAN}" != "xstretch" -a x"${DEBIAN}" != "xbullseye" -a x"${DEBIAN}" != "xxenial" -a x"${DEBIAN}" != "xjammy" ]; then
+      INSTALL_LIST="python3.7 python3.7-dev valgrind scons liblz4-dev devscripts debhelper debconf libpcap-dev libbz2-dev libsnappy-dev pkg-config zlib1g-dev libzlcore-dev dh-systemd libsasl2-dev gcc g++ cmake curl"
+      INSTALL_LIST="${INSTALL_LIST} libssl-dev libcurl4-openssl-dev libldap2-dev libkrb5-dev liblzma-dev patchelf"
+      if [ x"${DEBIAN}" != "xstretch" ]; then
         INSTALL_LIST="${INSTALL_LIST} python3.7-distutils"
       fi
       until apt-get -y install dirmngr; do
@@ -487,20 +408,9 @@ EOL
       apt-get -y install libext2fs-dev || apt-get -y install e2fslibs-dev
       install_golang
       install_gcc_8_deb
-      wget https://bootstrap.pypa.io/get-pip.py -O get-pip.py
-      if [ x"${DEBIAN}" = "xbullseye" ]; then
-        update-alternatives --install /usr/bin/python python /usr/bin/python3.9 1
-      elif [ x"${DEBIAN}" = "xjammy" ]; then
-        update-alternatives --install /usr/bin/python python /usr/bin/python3.10 1
-      elif [ x"${DEBIAN}" = "xxenial" ]; then
-        install_python3_7_12
-        update-alternatives --install /usr/bin/python python /usr/local/bin/python3.7 1
-        ln -sf /usr/local/bin/python3.7 /usr/bin/python3
-        sed -i 's/python3 /python3.5 /g' /usr/bin/lsb_release
-      else
-        update-alternatives --install /usr/bin/python python /usr/bin/python3.7 1
-        ln -sf /usr/bin/python3.7 /usr/bin/python3
-      fi
+      wget https://bootstrap.pypa.io/get-pip.py
+      update-alternatives --install /usr/bin/python python /usr/bin/python3.7 1
+      ln -sf /usr/bin/python3.7 /usr/bin/python3
       python get-pip.py
       easy_install pip
       pip install setuptools
@@ -575,9 +485,6 @@ build_srpm(){
         echo "It is not possible to build src rpm here"
         exit 1
     fi
-    if [ x"$RHEL" = x8 ]; then
-        switch_to_vault_repo
-    fi
     cd $WORKDIR
     get_tar "source_tarball"
     rm -fr rpmbuild
@@ -589,8 +496,6 @@ build_srpm(){
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/percona-packaging' --strip=1
     SPEC_TMPL=$(find percona-packaging/redhat -name 'percona-server-mongodb.spec.template' | sort | tail -n1)
     #
-    wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
-    mv call-home.sh rpmbuild/SOURCES
     cp -av percona-packaging/conf/* rpmbuild/SOURCES
     cp -av percona-packaging/redhat/mongod.* rpmbuild/SOURCES
     #
@@ -609,20 +514,6 @@ build_srpm(){
         source /opt/rh/devtoolset-8/enable
         source /opt/rh/rh-python36/enable
     fi
-
-    cd ${WORKDIR}/rpmbuild/SPECS
-    line_number=$(grep -n SOURCE999 percona-server-mongodb.spec | awk -F ':' '{print $1}')
-    cp ../SOURCES/call-home.sh ./
-    awk -v n=$line_number 'NR <= n {print > "part1.txt"} NR > n {print > "part2.txt"}' percona-server-mongodb.spec
-    head -n -1 part1.txt > temp && mv temp part1.txt
-    echo "cat <<'CALLHOME' > /tmp/call-home.sh" >> part1.txt
-    cat call-home.sh >> part1.txt
-    echo "CALLHOME" >> part1.txt
-    cat part2.txt >> part1.txt
-    rm -f call-home.sh part2.txt
-    mv part1.txt percona-server-mongodb.spec
-    cd ${WORKDIR}
-
     rpmbuild -bs --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .generic" rpmbuild/SPECS/$(basename ${SPEC_TMPL%.template})
     mkdir -p ${WORKDIR}/srpm
     mkdir -p ${CURDIR}/srpm
@@ -642,9 +533,6 @@ build_rpm(){
         echo "It is not possible to build rpm here"
         exit 1
     fi
-    if [ x"$RHEL" = x8 ]; then
-        switch_to_vault_repo
-    fi
     SRC_RPM=$(basename $(find $WORKDIR/srpm -name 'percona-server-mongodb*.src.rpm' | sort | tail -n1))
     if [ -z $SRC_RPM ]
     then
@@ -700,8 +588,8 @@ build_rpm(){
         export CXX=/opt/rh/devtoolset-8/root/usr/bin/g++
     fi
     #
-    echo "RHEL=${RHEL}" >> percona-server-mongodb-42.properties
-    echo "ARCH=${ARCH}" >> percona-server-mongodb-42.properties
+    echo "RHEL=${RHEL}" >> percona-server-mongodb-44.properties
+    echo "ARCH=${ARCH}" >> percona-server-mongodb-44.properties
     #
     file /usr/bin/scons
     #
@@ -728,7 +616,7 @@ build_rpm(){
 build_source_deb(){
     if [ $SDEB = 0 ]
     then
-        echo "source deb package will not be created"
+        echo "Source deb package will not be created"
         return;
     fi
     if [ "x$OS" = "xrpm" ]
@@ -755,20 +643,12 @@ build_source_deb(){
     sed -i 's:@@LOGDIR@@:mongodb:g' ${BUILDDIR}/debian/mongod.default
     sed -i 's:@@LOGDIR@@:mongodb:g' ${BUILDDIR}/debian/percona-server-mongodb-helper.sh
     #
-    if [ x"${DEBIAN}" = "xbullseye" -o x"${DEBIAN}" = "xxenial" ]; then
-        sed -i 's:dh-systemd,::' ${BUILDDIR}/debian/control
-    fi
-    #
     mv ${BUILDDIR}/debian/mongod.default ${BUILDDIR}/debian/percona-server-mongodb-server.mongod.default
     mv ${BUILDDIR}/debian/mongod.service ${BUILDDIR}/debian/percona-server-mongodb-server.mongod.service
     #
     mv ${TARFILE} ${PRODUCT}_${VERSION}.orig.tar.gz
     cd ${BUILDDIR}
     pip install --upgrade pip
-
-    # PyYAML pkg installation fix, more info: https://github.com/yaml/pyyaml/issues/724
-    pip install pyyaml==5.4.1 --no-build-isolation
-
     pip install -r etc/pip/dev-requirements.txt
     pip install -r etc/pip/evgtest-requirements.txt
 
@@ -793,12 +673,12 @@ build_source_deb(){
 build_deb(){
     if [ $DEB = 0 ]
     then
-        echo "source deb package will not be created"
+        echo "Deb package will not be created"
         return;
     fi
     if [ "x$OS" = "xrmp" ]
     then
-        echo "It is not possible to build source deb here"
+        echo "It is not possible to build deb here"
         exit 1
     fi
     for file in 'dsc' 'orig.tar.gz' 'changes' 'debian.tar*'
@@ -811,8 +691,8 @@ build_deb(){
     export DEBIAN=$(lsb_release -sc)
     export ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
     #
-    echo "DEBIAN=${DEBIAN}" >> percona-server-mongodb-42.properties
-    echo "ARCH=${ARCH}" >> percona-server-mongodb-42.properties
+    echo "DEBIAN=${DEBIAN}" >> percona-server-mongodb-44.properties
+    echo "ARCH=${ARCH}" >> percona-server-mongodb-44.properties
 
     #
     DSC=$(basename $(find . -name '*.dsc' | sort | tail -n1))
@@ -821,22 +701,14 @@ build_deb(){
     #
     cd ${PRODUCT}-${VERSION}
     pip install --upgrade pip
-
-    # PyYAML pkg installation fix, more info: https://github.com/yaml/pyyaml/issues/724
-    pip install pyyaml==5.4.1 --no-build-isolation
-
     pip install -r etc/pip/dev-requirements.txt
     pip install -r etc/pip/evgtest-requirements.txt
     #
     cp -av percona-packaging/debian/rules debian/
     set_compiler
     fix_rules
-    if [ x"${DEBIAN}" = "xbullseye" -o x"${DEBIAN}" = "xxenial" ]; then
-        sed -i 's:dh-systemd,::' debian/control
-        sed -i 's:etc/:/etc/:g' debian/percona-server-mongodb-server.conffiles
-    fi
-    sed -i 's|VersionStr="$(git describe)"|VersionStr="$PSMDB_TOOLS_REVISION"|' mongo-tools/set_goenv.sh
-    sed -i 's|Gitspec="$(git rev-parse HEAD)"|Gitspec="$PSMDB_TOOLS_COMMIT_HASH"|' mongo-tools/set_goenv.sh
+    sed -i 's|VersionStr="$(go run release/release.go get-version)"|VersionStr="$PSMDB_TOOLS_REVISION"|' mongo-tools/set_goenv.sh
+    sed -i 's|GitCommit="$(git rev-parse HEAD)"|GitCommit="$PSMDB_TOOLS_COMMIT_HASH"|' mongo-tools/set_goenv.sh
     sed -i 's|go build|go build -a -x|' mongo-tools/build.sh
     sed -i 's|exit $ec||' mongo-tools/build.sh
     dch -m -D "${DEBIAN}" --force-distribution -v "${VERSION}-${RELEASE}.${DEBIAN}" 'Update distribution'
@@ -845,19 +717,6 @@ build_deb(){
       CURL_LINKFLAGS=$(pkg-config libcurl --static --libs)
       export LINKFLAGS="${LINKFLAGS} ${CURL_LINKFLAGS}"
     fi
-
-    cd debian/
-        wget https://raw.githubusercontent.com/Percona-Lab/telemetry-agent/phase-0/call-home.sh
-        sed -i 's:exit 0::' percona-server-mongodb-server.postinst
-        echo "cat <<'CALLHOME' > /tmp/call-home.sh" >> percona-server-mongodb-server.postinst
-        cat call-home.sh >> percona-server-mongodb-server.postinst
-        echo "CALLHOME" >> percona-server-mongodb-server.postinst
-        echo 'bash +x /tmp/call-home.sh -f "PRODUCT_FAMILY_PSMDB" -v '"${PSM_VER}-${PSM_RELEASE}"' -d "PACKAGE" &>/dev/null || :' >> percona-server-mongodb-server.postinst
-        echo "rm -rf /tmp/call-home.sh" >> percona-server-mongodb-server.postinst
-        echo "exit 0" >> percona-server-mongodb-server.postinst
-        rm -f call-home.sh
-    cd ../
-
     dpkg-buildpackage -rfakeroot -us -uc -b
     mkdir -p $CURDIR/deb
     mkdir -p $WORKDIR/deb
@@ -871,9 +730,6 @@ build_tarball(){
         echo "Binary tarball will not be created"
         return;
     fi
-    if [ x"$RHEL" = x8 ]; then
-        switch_to_vault_repo
-    fi
     get_tar "source_tarball"
     cd $WORKDIR
     TARFILE=$(basename $(find . -name 'percona-server-mongodb*.tar.gz' | sort | tail -n1))
@@ -916,7 +772,8 @@ build_tarball(){
     PSMDIR_ABS=${WORKDIR}/${PSMDIR}
     TOOLSDIR=${PSMDIR}/mongo-tools
     TOOLSDIR_ABS=${WORKDIR}/${TOOLSDIR}
-    TOOLS_TAGS="ssl sasl gssapi"
+    TOOLS_TAGS="ssl sasl"
+    NJOBS=4
 
     tar xzf $TARFILE
     rm -f $TARFILE
@@ -944,10 +801,6 @@ build_tarball(){
         pip3.6 install --user -r etc/pip/evgtest-requirements.txt
     else
         pip install --upgrade pip
-
-        # PyYAML pkg installation fix, more info: https://github.com/yaml/pyyaml/issues/724
-        pip install pyyaml==5.4.1 --no-build-isolation
-
         pip install --user -r etc/pip/dev-requirements.txt
         pip install --user -r etc/pip/evgtest-requirements.txt
     fi
@@ -963,24 +816,17 @@ build_tarball(){
             cd aws-sdk-cpp
             git reset --hard
             git clean -xdf
-            git checkout 1.9.379
-            git submodule update --init --recursive
+            git checkout 1.7.91
             mkdir build
+            sed -i 's:AWS_EVENT_STREAM_SHA:AWS_EVENT_STREAM_TAG:g' third-party/cmake/BuildAwsEventStream.cmake
             cd build
             set_compiler
-            if [ x"${DEBIAN}" = xjammy ]; then
-                CMAKE_CXX_FLAGS="-Wno-maybe-uninitialized -Wno-error=deprecated-declarations -Wno-error=uninitialized "
-            fi
-            CMAKE_CMD="cmake"
-            if [ x"$RHEL" = x7 ]; then
-                CMAKE_CMD="cmake3"
-            fi
             if [ -z "${CC}" -a -z "${CXX}" ]; then
-                ${CMAKE_CMD} .. -DCMAKE_CXX_FLAGS="${CMAKE_CXX_FLAGS}" -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3;transfer" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON -DCMAKE_INSTALL_PREFIX="${INSTALLDIR_AWS}" || exit $?
+                cmake .. -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON -DCMAKE_INSTALL_PREFIX="${INSTALLDIR_AWS}" || exit $?
             else
-                ${CMAKE_CMD} CC=${CC} CXX=${CXX} .. -DCMAKE_CXX_FLAGS="${CMAKE_CXX_FLAGS}" -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3;transfer" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON -DCMAKE_INSTALL_PREFIX="${INSTALLDIR_AWS}" || exit $?
+                cmake CC=${CC} CXX=${CXX} .. -DCMAKE_BUILD_TYPE=Release -DBUILD_ONLY="s3" -DBUILD_SHARED_LIBS=OFF -DMINIMIZE_SIZE=ON -DCMAKE_INSTALL_PREFIX="${INSTALLDIR_AWS}" || exit $?
             fi
-            make -j${NCPU} || exit $?
+            make -j4 || exit $?
             make install
             mkdir -p ${INSTALLDIR}/include/
             mkdir -p ${INSTALLDIR}/lib/
@@ -996,9 +842,9 @@ build_tarball(){
       export LINKFLAGS="${LINKFLAGS} ${CURL_LINKFLAGS}"
     fi
     if [ ${DEBUG} = 0 ]; then
-        buildscripts/scons.py CC=${CC} CXX=${CXX} --disable-warnings-as-errors --release --ssl --opt=on -j${NCPU} --use-sasl-client --wiredtiger --audit --inmemory --hotbackup CPPPATH="${INSTALLDIR}/include ${AWS_LIBS}/include" LIBPATH="${INSTALLDIR}/lib ${AWS_LIBS}/lib ${AWS_LIBS}/lib64" LINKFLAGS="${LINKFLAGS}" ${PSM_TARGETS} || exit $?
+        buildscripts/scons.py CC=${CC} CXX=${CXX} --install-mode=legacy --disable-warnings-as-errors --release --ssl --opt=on -j$NJOBS --use-sasl-client --wiredtiger --audit --inmemory --hotbackup CPPPATH="${INSTALLDIR}/include ${AWS_LIBS}/include" LIBPATH="${INSTALLDIR}/lib ${AWS_LIBS}/lib ${AWS_LIBS}/lib64" LINKFLAGS="${LINKFLAGS}" ${PSM_TARGETS} || exit $?
     else
-        buildscripts/scons.py CC=${CC} CXX=${CXX} --disable-warnings-as-errors --audit --ssl --dbg=on -j${NCPU} --use-sasl-client \
+        buildscripts/scons.py CC=${CC} CXX=${CXX} --install-mode=legacy --disable-warnings-as-errors --audit --ssl --dbg=on -j$NJOBS --use-sasl-client \
         CPPPATH="${INSTALLDIR}/include ${AWS_LIBS}/include" LIBPATH="${INSTALLDIR}/lib ${AWS_LIBS}/lib ${AWS_LIBS}/lib64" LINKFLAGS="${LINKFLAGS}" --wiredtiger --inmemory --hotbackup ${PSM_TARGETS} || exit $?
     fi
     #
@@ -1026,8 +872,8 @@ build_tarball(){
     cp -r $(ls | grep -v build_tools) build_tools/src/github.com/mongodb/mongo-tools/
     cd build_tools/src/github.com/mongodb/mongo-tools
     . ./set_tools_revision.sh
-    sed -i 's|VersionStr="$(git describe)"|VersionStr="$PSMDB_TOOLS_REVISION"|' set_goenv.sh
-    sed -i 's|Gitspec="$(git rev-parse HEAD)"|Gitspec="$PSMDB_TOOLS_COMMIT_HASH"|' set_goenv.sh
+    sed -i 's|VersionStr="$(go run release/release.go get-version)"|VersionStr="$PSMDB_TOOLS_REVISION"|' set_goenv.sh
+    sed -i 's|GitCommit="$(git rev-parse HEAD)"|GitCommit="$PSMDB_TOOLS_COMMIT_HASH"|' set_goenv.sh
     . ./set_goenv.sh
     if [ ${DEBUG} = 0 ]; then
         sed -i 's|go build|go build -a -x|' build.sh
@@ -1048,7 +894,7 @@ build_tarball(){
     if [ ! -d lib/private ]; then
         mkdir -p lib/private
     fi
-    LIBLIST="libcrypto.so libssl.so libpcap.so librtmp.so libssl3.so libsmime3.so libnss3.so libnssutil3.so libplds4.so libplc4.so libnspr4.so libssl3.so libplds4.so libidn.so"
+    LIBLIST="libcrypto.so libssl.so libsasl2.so libgssapi_krb5.so libkrb5.so libkrb5support.so libk5crypto.so librtmp.so libgssapi.so libssl3.so libsmime3.so libnss3.so libnssutil3.so libplds4.so libplc4.so libnspr4.so libssl3.so liblzma.so libidn.so"
     DIRLIST="bin lib/private"
 
     LIBPATH=""
@@ -1063,7 +909,7 @@ build_tarball(){
                     lib_realpath_basename="$(basename $(readlink -f ${libfromelf}))"
                     lib_without_version_suffix=$(echo ${lib_realpath_basename} | awk -F"." 'BEGIN { OFS = "." }{ print $1, $2}')
 
-                    if [ ! -f "lib/private/${lib_realpath_basename}" ] && [ ! -L "lib/private/${lib_without_version_suffix}" ]; then
+                    if [ ! -f "lib/private/${lib_realpath_basename}" ] && [ ! -L "lib/private/${lib_realpath_basename}" ]; then
                     
                         echo "Copying lib ${lib_realpath_basename}"
                         cp ${lib_realpath} lib/private
@@ -1098,20 +944,6 @@ build_tarball(){
         done
     }
 
-    function fix_sasl_lib {
-        # Details are in tickets PSMDB-950 PSMDB-1153
-        patchelf --remove-needed libsasl2.so.3 bin/mongod
-        patchelf --remove-needed libsasl2.so.2 bin/mongod
-        patchelf --remove-needed libsasl2.so.3 bin/mongo
-        patchelf --remove-needed libsasl2.so.2 bin/mongo
-        # Details are in tickets PSMDB-1160
-        if [ "x$OS" = "xrpm" ]
-        then
-            patchelf --remove-needed libsasl2.so bin/mongod
-            patchelf --remove-needed libsasl2.so bin/mongo
-        fi
-    }
-
     function replace_libs {
         local elf_path=$1
         for libpath_sorted in ${LIBPATH}; do
@@ -1156,50 +988,32 @@ build_tarball(){
         done
     }
 
-    function link {
-        # Gather libs
-        for DIR in ${DIRLIST}; do
-            gather_libs ${DIR}
-        done
+    # Gather libs
+    for DIR in ${DIRLIST}; do
+        gather_libs ${DIR}
+    done
 
-        # Set proper runpath
-        set_runpath bin '$ORIGIN/../lib/private/'
-        set_runpath lib/private '$ORIGIN/'
+    # Set proper runpath
+    set_runpath bin '$ORIGIN/../lib/private/'
+    set_runpath lib/private '$ORIGIN/'
 
-        # Replace libs
-        for DIR in ${DIRLIST}; do
-            replace_libs ${DIR}
-        done
+    # Replace libs
+    for DIR in ${DIRLIST}; do
+        replace_libs ${DIR}
+    done
 
-        # Use system libsasl2 for some binaries
-        fix_sasl_lib
+    # Create and replace by sparse file to reduce size
+    create_sparse bin
+    replace_binaries bin
 
-        # Create and replace by sparse file to reduce size
-        create_sparse bin
-        replace_binaries bin
-
-        # Make final check in order to determine any error after linkage
-        for DIR in ${DIRLIST}; do
-            check_libs ${DIR}
-        done
-    }
+    # Make final check in order to determine any error after linkage
+    for DIR in ${DIRLIST}; do
+        check_libs ${DIR}
+    done
 
     cd ${PSMDIR_ABS}
     mv ${PSMDIR} ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}
-
-    if [[ ${DEBUG} = 0 ]]; then
-        cp -r ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX} ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}-minimal
-        cd ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}-minimal
-        find . -type f -exec file '{}' \; | grep ': ELF ' | cut -d':' -f1 | xargs strip --strip-unneeded
-        link
-        cd ${PSMDIR_ABS}
-        tar --owner=0 --group=0 -czf ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}-minimal.tar.gz ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}-minimal
-    fi
-
-    cd ${PSMDIR_ABS}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}
-    link
-    cd ${PSMDIR_ABS}
-    tar --owner=0 --group=0 -czf ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}.tar.gz ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}
+    tar --owner=0 --group=0 -czf ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}.tar.gz ${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}    
     DIRNAME="tarball"
     if [ "${DEBUG}" = 1 ]; then
     DIRNAME="debug"
@@ -1208,10 +1022,6 @@ build_tarball(){
     mkdir -p ${CURDIR}/${DIRNAME}
     cp ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}.tar.gz ${WORKDIR}/${DIRNAME}
     cp ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}.tar.gz ${CURDIR}/${DIRNAME}
-    if [[ ${DEBUG} = 0 ]]; then
-    cp ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}-minimal.tar.gz ${WORKDIR}/${DIRNAME}
-    cp ${WORKDIR}/${PSMDIR}-${ARCH}${GLIBC_VER}${TARBALL_SUFFIX}-minimal.tar.gz ${CURDIR}/${DIRNAME}
-    fi
 }
 
 #main
@@ -1235,9 +1045,9 @@ DEB_RELEASE=1
 REVISION=0
 BRANCH="master"
 REPO="https://github.com/percona/percona-server-mongodb.git"
-PSM_VER="4.2.0"
+PSM_VER="4.4.0"
 PSM_RELEASE="1"
-MONGO_TOOLS_TAG="r4.2.0-rc2"
+MONGO_TOOLS_TAG="master"
 PRODUCT=percona-server-mongodb
 DEBUG=0
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
@@ -1248,12 +1058,6 @@ PSM_BRANCH=${BRANCH}
 if [ ${DEBUG} = 1 ]; then
   TARBALL=1
 fi
-if test -e "/proc/cpuinfo"
-then
-    NCPU="$(grep -c ^processor /proc/cpuinfo)"
-else
-    NCPU=4
-fi
 
 check_workdir
 get_system
