@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -111,7 +110,7 @@ void FetcherTest::setUp() {
     executor::ThreadPoolExecutorTest::setUp();
     clear();
     callbackHook = Fetcher::CallbackFn();
-    fetcher = stdx::make_unique<Fetcher>(&getExecutor(), source, "db", findCmdObj, makeCallback());
+    fetcher = std::make_unique<Fetcher>(&getExecutor(), source, "db", findCmdObj, makeCallback());
     launchExecutorThread();
 }
 
@@ -145,7 +144,7 @@ void FetcherTest::processNetworkResponse(const BSONObj& obj,
                                          ReadyQueueState readyQueueStateAfterProcessing,
                                          FetcherState fetcherStateAfterProcessing) {
     executor::NetworkInterfaceMock::InNetworkGuard guard(getNet());
-    getNet()->scheduleSuccessfulResponse({obj, {}, elapsed});
+    getNet()->scheduleSuccessfulResponse({obj, elapsed});
     finishProcessingNetworkResponse(readyQueueStateAfterProcessing, fetcherStateAfterProcessing);
 }
 
@@ -176,7 +175,7 @@ void FetcherTest::_callback(const StatusWith<Fetcher::QueryResponse>& result,
         cursorId = batchData.cursorId;
         nss = batchData.nss;
         documents = batchData.documents;
-        elapsedMillis = batchData.elapsedMillis;
+        elapsedMillis = duration_cast<Milliseconds>(batchData.elapsed);
         first = batchData.first;
     }
 
@@ -254,21 +253,21 @@ TEST_F(FetcherTest, InvalidConstruction) {
 TEST_F(FetcherTest, NonFindCommand) {
     TaskExecutor& executor = getExecutor();
 
-    Fetcher(&executor,
-            source,
-            "db",
-            BSON("listIndexes"
-                 << "coll"),
-            unreachableCallback);
-    Fetcher(&executor, source, "db", BSON("listCollections" << 1), unreachableCallback);
-    Fetcher(&executor, source, "db", BSON("a" << 1), unreachableCallback);
+    Fetcher f1(&executor,
+               source,
+               "db",
+               BSON("listIndexes"
+                    << "coll"),
+               unreachableCallback);
+    Fetcher f2(&executor, source, "db", BSON("listCollections" << 1), unreachableCallback);
+    Fetcher f3(&executor, source, "db", BSON("a" << 1), unreachableCallback);
 }
 
 TEST_F(FetcherTest, RemoteCommandRequestShouldContainCommandParametersPassedToConstructor) {
     auto metadataObj = BSON("x" << 1);
     Milliseconds timeout(8000);
 
-    fetcher = stdx::make_unique<Fetcher>(
+    fetcher = std::make_unique<Fetcher>(
         &getExecutor(), source, "db", findCmdObj, doNothingCallback, metadataObj, timeout);
 
     ASSERT_EQUALS(source, fetcher->getSource());
@@ -406,8 +405,7 @@ TEST_F(FetcherTest, FindCommandFailed2) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("ok" << 0 << "errmsg"
                                      << "bad hint"
-                                     << "code"
-                                     << int(ErrorCodes::BadValue)),
+                                     << "code" << int(ErrorCodes::BadValue)),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::BadValue, status.code());
@@ -433,10 +431,8 @@ TEST_F(FetcherTest, CursorIdFieldMissing) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("ns"
                                                  << "db.coll"
-                                                 << "firstBatch"
-                                                 << BSONArray())
-                                         << "ok"
-                                         << 1),
+                                                 << "firstBatch" << BSONArray())
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, status.code());
@@ -447,10 +443,8 @@ TEST_F(FetcherTest, CursorIdNotLongNumber) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 123.1 << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSONArray())
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSONArray())
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, status.code());
@@ -470,11 +464,11 @@ TEST_F(FetcherTest, NamespaceFieldMissing) {
 
 TEST_F(FetcherTest, NamespaceNotAString) {
     ASSERT_OK(fetcher->schedule());
-    processNetworkResponse(
-        BSON("cursor" << BSON("id" << 123LL << "ns" << 123 << "firstBatch" << BSONArray()) << "ok"
-                      << 1),
-        ReadyQueueState::kEmpty,
-        FetcherState::kInactive);
+    processNetworkResponse(BSON("cursor"
+                                << BSON("id" << 123LL << "ns" << 123 << "firstBatch" << BSONArray())
+                                << "ok" << 1),
+                           ReadyQueueState::kEmpty,
+                           FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, status.code());
     ASSERT_STRING_CONTAINS(status.reason(), "'cursor.ns' field must be a string");
 }
@@ -483,10 +477,8 @@ TEST_F(FetcherTest, NamespaceEmpty) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 123LL << "ns"
                                                       << ""
-                                                      << "firstBatch"
-                                                      << BSONArray())
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSONArray())
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::BadValue, status.code());
@@ -497,10 +489,8 @@ TEST_F(FetcherTest, NamespaceMissingCollectionName) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 123LL << "ns"
                                                       << "db."
-                                                      << "firstBatch"
-                                                      << BSONArray())
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSONArray())
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::BadValue, status.code());
@@ -511,8 +501,7 @@ TEST_F(FetcherTest, FirstBatchFieldMissing) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll")
-                                         << "ok"
-                                         << 1),
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, status.code());
@@ -523,10 +512,8 @@ TEST_F(FetcherTest, FirstBatchNotAnArray) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << 123)
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << 123)
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, status.code());
@@ -537,10 +524,8 @@ TEST_F(FetcherTest, FirstBatchArrayContainsNonObject) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(8))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(8))
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_EQUALS(ErrorCodes::FailedToParse, status.code());
@@ -552,10 +537,8 @@ TEST_F(FetcherTest, FirstBatchEmptyArray) {
     ASSERT_OK(fetcher->schedule());
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSONArray())
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSONArray())
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_OK(status);
@@ -569,10 +552,8 @@ TEST_F(FetcherTest, FetchOneDocument) {
     const BSONObj doc = BSON("_id" << 1);
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_OK(status);
@@ -597,10 +578,8 @@ TEST_F(FetcherTest, SetNextActionToContinueWhenNextBatchIsNotAvailable) {
     };
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
     ASSERT_OK(status);
@@ -630,10 +609,8 @@ TEST_F(FetcherTest, FetchMultipleBatches) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            Milliseconds(100),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
@@ -651,10 +628,8 @@ TEST_F(FetcherTest, FetchMultipleBatches) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "nextBatch"
-                                                      << BSON_ARRAY(doc2))
-                                         << "ok"
-                                         << 1),
+                                                      << "nextBatch" << BSON_ARRAY(doc2))
+                                         << "ok" << 1),
                            Milliseconds(200),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
@@ -672,10 +647,8 @@ TEST_F(FetcherTest, FetchMultipleBatches) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "nextBatch"
-                                                      << BSON_ARRAY(doc3))
-                                         << "ok"
-                                         << 1),
+                                                      << "nextBatch" << BSON_ARRAY(doc3))
+                                         << "ok" << 1),
                            Milliseconds(300),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
@@ -699,10 +672,8 @@ TEST_F(FetcherTest, ScheduleGetMoreAndCancel) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
 
@@ -716,10 +687,8 @@ TEST_F(FetcherTest, ScheduleGetMoreAndCancel) {
     const BSONObj doc2 = BSON("_id" << 2);
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "nextBatch"
-                                                      << BSON_ARRAY(doc2))
-                                         << "ok"
-                                         << 1),
+                                                      << "nextBatch" << BSON_ARRAY(doc2))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
 
@@ -762,10 +731,8 @@ TEST_F(FetcherTest, CancelDuringCallbackPutsFetcherInShutdown) {
     const BSONObj doc = BSON("_id" << 1);
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kInactive);
 
@@ -783,10 +750,8 @@ TEST_F(FetcherTest, ScheduleGetMoreButShutdown) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
 
@@ -801,10 +766,8 @@ TEST_F(FetcherTest, ScheduleGetMoreButShutdown) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "nextBatch"
-                                                      << BSON_ARRAY(doc2))
-                                         << "ok"
-                                         << 1),
+                                                      << "nextBatch" << BSON_ARRAY(doc2))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
 
@@ -840,10 +803,8 @@ TEST_F(FetcherTest, EmptyGetMoreRequestAfterFirstBatchMakesFetcherInactiveAndKil
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kInactive);
 
@@ -875,9 +836,13 @@ TEST_F(FetcherTest, EmptyGetMoreRequestAfterFirstBatchMakesFetcherInactiveAndKil
 
     // killCursors command request will be canceled by executor on shutdown.
     tearDown();
-    ASSERT_EQUALS(1,
-                  countLogLinesContaining(
-                      "killCursors command task failed: CallbackCanceled: Callback canceled"));
+    ASSERT_EQUALS(
+        1,
+        countBSONFormatLogLinesIsSubset(BSON("msg"
+                                             << "killCursors command task failed"
+                                             << "attr"
+                                             << BSON("error"
+                                                     << "CallbackCanceled: Callback canceled"))));
 }
 
 void setNextActionToNoAction(const StatusWith<Fetcher::QueryResponse>& fetchResult,
@@ -897,10 +862,8 @@ TEST_F(FetcherTest, UpdateNextActionAfterSecondBatch) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
 
@@ -917,10 +880,8 @@ TEST_F(FetcherTest, UpdateNextActionAfterSecondBatch) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "nextBatch"
-                                                      << BSON_ARRAY(doc2))
-                                         << "ok"
-                                         << 1),
+                                                      << "nextBatch" << BSON_ARRAY(doc2))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kInactive);
 
@@ -949,11 +910,16 @@ TEST_F(FetcherTest, UpdateNextActionAfterSecondBatch) {
         ASSERT_EQUALS(cursorId, cursors.front().numberLong());
 
         // Failed killCursors command response should be logged.
-        getNet()->scheduleSuccessfulResponse(noi, {BSON("ok" << false), {}, Milliseconds(0)});
+        getNet()->scheduleSuccessfulResponse(noi, {BSON("ok" << false), Milliseconds(0)});
         getNet()->runReadyNetworkOperations();
     }
 
-    ASSERT_EQUALS(1, countLogLinesContaining("killCursors command failed: UnknownError"));
+    ASSERT_EQUALS(1,
+                  countBSONFormatLogLinesIsSubset(BSON("msg"
+                                                       << "killCursors command failed"
+                                                       << "attr"
+                                                       << BSON("error"
+                                                               << "UnknownError: "))));
 }
 
 /**
@@ -994,10 +960,8 @@ TEST_F(FetcherTest, ShutdownDuringSecondBatch) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
 
@@ -1017,17 +981,19 @@ TEST_F(FetcherTest, ShutdownDuringSecondBatch) {
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "nextBatch"
-                                                      << BSON_ARRAY(doc2))
-                                         << "ok"
-                                         << 1),
+                                                      << "nextBatch" << BSON_ARRAY(doc2))
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
 
     // Fetcher should attempt (unsuccessfully) to schedule a killCursors command.
     ASSERT_EQUALS(1,
-                  countLogLinesContaining("failed to schedule killCursors command: "
-                                          "ShutdownInProgress: Shutdown in progress"));
+                  countBSONFormatLogLinesIsSubset(
+                      BSON("msg"
+                           << "Failed to schedule killCursors command"
+                           << "attr"
+                           << BSON("error"
+                                   << "ShutdownInProgress: Shutdown in progress"))));
 
     ASSERT_EQUALS(ErrorCodes::ShutdownInProgress, status.code());
 }
@@ -1036,15 +1002,15 @@ TEST_F(FetcherTest, FetcherAppliesRetryPolicyToFirstCommandButNotToGetMoreReques
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         3U, executor::RemoteCommandRequest::kNoTimeout);
 
-    fetcher = stdx::make_unique<Fetcher>(&getExecutor(),
-                                         source,
-                                         "db",
-                                         findCmdObj,
-                                         makeCallback(),
-                                         rpc::makeEmptyMetadata(),
-                                         executor::RemoteCommandRequest::kNoTimeout,
-                                         executor::RemoteCommandRequest::kNoTimeout,
-                                         std::move(policy));
+    fetcher = std::make_unique<Fetcher>(&getExecutor(),
+                                        source,
+                                        "db",
+                                        findCmdObj,
+                                        makeCallback(),
+                                        rpc::makeEmptyMetadata(),
+                                        executor::RemoteCommandRequest::kNoTimeout,
+                                        executor::RemoteCommandRequest::kNoTimeout,
+                                        std::move(policy));
 
     callbackHook = appendGetMoreRequest;
 
@@ -1058,10 +1024,8 @@ TEST_F(FetcherTest, FetcherAppliesRetryPolicyToFirstCommandButNotToGetMoreReques
     processNetworkResponse(rs, ReadyQueueState::kHasReadyRequests, FetcherState::kActive);
     processNetworkResponse(BSON("cursor" << BSON("id" << 1LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSON_ARRAY(doc))
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSON_ARRAY(doc))
+                                         << "ok" << 1),
                            ReadyQueueState::kHasReadyRequests,
                            FetcherState::kActive);
     ASSERT_OK(status);
@@ -1079,7 +1043,8 @@ TEST_F(FetcherTest, FetcherAppliesRetryPolicyToFirstCommandButNotToGetMoreReques
 
 bool sharedCallbackStateDestroyed = false;
 class SharedCallbackState {
-    MONGO_DISALLOW_COPYING(SharedCallbackState);
+    SharedCallbackState(const SharedCallbackState&) = delete;
+    SharedCallbackState& operator=(const SharedCallbackState&) = delete;
 
 public:
     SharedCallbackState() {}
@@ -1092,7 +1057,7 @@ TEST_F(FetcherTest, FetcherResetsInternalFinishCallbackFunctionPointerAfterLastC
     auto sharedCallbackData = std::make_shared<SharedCallbackState>();
     auto callbackInvoked = false;
 
-    fetcher = stdx::make_unique<Fetcher>(
+    fetcher = std::make_unique<Fetcher>(
         &getExecutor(),
         source,
         "db",
@@ -1108,10 +1073,8 @@ TEST_F(FetcherTest, FetcherResetsInternalFinishCallbackFunctionPointerAfterLastC
 
     processNetworkResponse(BSON("cursor" << BSON("id" << 0LL << "ns"
                                                       << "db.coll"
-                                                      << "firstBatch"
-                                                      << BSONArray())
-                                         << "ok"
-                                         << 1),
+                                                      << "firstBatch" << BSONArray())
+                                         << "ok" << 1),
                            ReadyQueueState::kEmpty,
                            FetcherState::kInactive);
 
