@@ -27,6 +27,8 @@
  *    it in the license file.
  */
 
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
+
 #include "mongo/platform/basic.h"
 
 #include <memory>
@@ -39,7 +41,7 @@
 #include "mongo/executor/remote_command_response.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/executor/thread_pool_task_executor_test_fixture.h"
-#include "mongo/stdx/memory.h"
+#include "mongo/logv2/log.h"
 #include "mongo/unittest/task_executor_proxy.h"
 #include "mongo/unittest/unittest.h"
 #include "mongo/util/assert_util.h"
@@ -165,10 +167,10 @@ std::vector<ResponseStatus> CallbackResponseSaver::getResponses() const {
     return _responses;
 }
 
-const executor::RemoteCommandRequest request(HostAndPort("h1:12345"),
-                                             "db1",
-                                             BSON("ping" << 1),
-                                             nullptr);
+executor::RemoteCommandRequest makeRemoteCommandRequest() {
+    return executor::RemoteCommandRequest{
+        HostAndPort("h1:12345"), "db1", BSON("ping" << 1), nullptr};
+}
 
 TEST_F(RemoteCommandRetrySchedulerTest, MakeSingleShotRetryPolicy) {
     auto policy = RemoteCommandRetryScheduler::makeNoRetryPolicy();
@@ -202,6 +204,7 @@ TEST_F(RemoteCommandRetrySchedulerTest, MakeRetryPolicy) {
 TEST_F(RemoteCommandRetrySchedulerTest, InvalidConstruction) {
     auto callback = [](const executor::TaskExecutor::RemoteCommandCallbackArgs&) {};
     auto makeRetryPolicy = [] { return RemoteCommandRetryScheduler::makeNoRetryPolicy(); };
+    auto request = makeRemoteCommandRequest();
 
     // Null executor.
     ASSERT_THROWS_CODE_AND_WHAT(
@@ -291,6 +294,7 @@ TEST_F(RemoteCommandRetrySchedulerTest, InvalidConstruction) {
 TEST_F(RemoteCommandRetrySchedulerTest, StartupFailsWhenExecutorIsShutDown) {
     auto callback = [](const executor::TaskExecutor::RemoteCommandCallbackArgs&) {};
     auto policy = RemoteCommandRetryScheduler::makeNoRetryPolicy();
+    auto request = makeRemoteCommandRequest();
 
     RemoteCommandRetryScheduler scheduler(&getExecutor(), request, callback, std::move(policy));
     ASSERT_FALSE(scheduler.isActive());
@@ -304,6 +308,7 @@ TEST_F(RemoteCommandRetrySchedulerTest, StartupFailsWhenExecutorIsShutDown) {
 TEST_F(RemoteCommandRetrySchedulerTest, StartupFailsWhenSchedulerIsShutDown) {
     auto callback = [](const executor::TaskExecutor::RemoteCommandCallbackArgs&) {};
     auto policy = RemoteCommandRetryScheduler::makeNoRetryPolicy();
+    auto request = makeRemoteCommandRequest();
 
     RemoteCommandRetryScheduler scheduler(&getExecutor(), request, callback, std::move(policy));
     ASSERT_FALSE(scheduler.isActive());
@@ -319,8 +324,10 @@ TEST_F(RemoteCommandRetrySchedulerTest,
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         10U, Milliseconds(1));
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     auto net = getNet();
@@ -341,8 +348,10 @@ TEST_F(RemoteCommandRetrySchedulerTest,
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         10U, Milliseconds(1));
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     scheduler.shutdown();
@@ -354,10 +363,12 @@ TEST_F(RemoteCommandRetrySchedulerTest,
 
 TEST_F(RemoteCommandRetrySchedulerTest, SchedulerInvokesCallbackOnNonRetryableErrorInResponse) {
     CallbackResponseSaver callback;
-    auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::NotPrimaryError>(
+    auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::NotMasterError>(
         10U, Milliseconds(1));
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     // This should match one of the non-retryable error codes in the policy.
@@ -373,8 +384,10 @@ TEST_F(RemoteCommandRetrySchedulerTest, SchedulerInvokesCallbackOnFirstSuccessfu
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         10U, Milliseconds(1));
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     // Elapsed time in response is ignored on successful responses.
@@ -392,8 +405,10 @@ TEST_F(RemoteCommandRetrySchedulerTest, SchedulerIgnoresEmbeddedErrorInSuccessfu
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         10U, Milliseconds(1));
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     // Scheduler does not parse document in a successful response for embedded errors.
@@ -413,9 +428,10 @@ TEST_F(RemoteCommandRetrySchedulerTest,
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         3U, executor::RemoteCommandRequest::kNoTimeout);
+    auto request = makeRemoteCommandRequest();
     TaskExecutorWithFailureInScheduleRemoteCommand badExecutor(&getExecutor());
     RemoteCommandRetryScheduler scheduler(
-        &badExecutor, request, stdx::ref(callback), std::move(policy));
+        &badExecutor, request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     processNetworkResponse({ErrorCodes::HostNotFound, "first", Milliseconds(0)});
@@ -434,8 +450,10 @@ TEST_F(RemoteCommandRetrySchedulerTest,
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         3U, executor::RemoteCommandRequest::kNoTimeout);
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     processNetworkResponse({ErrorCodes::HostNotFound, "first", Milliseconds(0)});
@@ -450,8 +468,10 @@ TEST_F(RemoteCommandRetrySchedulerTest, SchedulerShouldRetryUntilSuccessfulRespo
     CallbackResponseSaver callback;
     auto policy = RemoteCommandRetryScheduler::makeRetryPolicy<ErrorCategory::RetriableError>(
         3U, executor::RemoteCommandRequest::kNoTimeout);
+    auto request = makeRemoteCommandRequest();
+
     RemoteCommandRetryScheduler scheduler(
-        &getExecutor(), request, stdx::ref(callback), std::move(policy));
+        &getExecutor(), request, std::ref(callback), std::move(policy));
     start(&scheduler);
 
     processNetworkResponse({ErrorCodes::HostNotFound, "first", Milliseconds(0)});
@@ -494,11 +514,12 @@ public:
 TEST_F(RemoteCommandRetrySchedulerTest,
        SchedulerReturnsCallbackCanceledIfShutdownBeforeSendingRetryCommand) {
     CallbackResponseSaver callback;
-    auto policy = stdx::make_unique<ShutdownSchedulerRetryPolicy>();
+    auto policy = std::make_unique<ShutdownSchedulerRetryPolicy>();
     auto policyPtr = policy.get();
+    auto request = makeRemoteCommandRequest();
     TaskExecutorWithFailureInScheduleRemoteCommand badExecutor(&getExecutor());
     RemoteCommandRetryScheduler scheduler(
-        &badExecutor, request, stdx::ref(callback), std::move(policy));
+        &badExecutor, request, std::ref(callback), std::move(policy));
     policyPtr->scheduler = &scheduler;
     start(&scheduler);
 
@@ -530,13 +551,14 @@ TEST_F(RemoteCommandRetrySchedulerTest,
 
     Status result = getDetectableErrorStatus();
     auto policy = RemoteCommandRetryScheduler::makeNoRetryPolicy();
+    auto request = makeRemoteCommandRequest();
 
     RemoteCommandRetryScheduler scheduler(
         &getExecutor(),
         request,
         [&result,
          sharedCallbackData](const executor::TaskExecutor::RemoteCommandCallbackArgs& rcba) {
-            unittest::log() << "setting result to " << rcba.response.status;
+            LOGV2(20156, "Setting result", "result"_attr = rcba.response.status);
             result = rcba.response.status;
         },
         std::move(policy));
