@@ -41,8 +41,6 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
 #include <syslog.h>
 
 #include <boost/filesystem/path.hpp>
-#include <boost/iostreams/device/file_descriptor.hpp>
-#include <boost/iostreams/stream.hpp>
 #include <boost/scoped_ptr.hpp>
 
 #include "mongo/util/debug_util.h"
@@ -52,7 +50,7 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
  * See this link for explanation of the MONGO_LOG macro:
  * http://www.mongodb.org/about/contributors/reference/server-logging-rules/
  */
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/base/init.h"
 #include "mongo/bson/bson_field.h"
@@ -66,15 +64,17 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
 #include "mongo/db/matcher/matcher.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/logger/auditlog.h"
+#include "mongo/logger/logger.h"
+#include "mongo/logv2/log.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/util/concurrency/mutex.h"
 #include "mongo/util/exit_code.h"
-#include "mongo/util/log.h"
 #include "mongo/util/net/sock.h"
 #include "mongo/util/string_map.h"
 #include "mongo/util/time_support.h"
 
 #include "audit_options.h"
+#include "audit_file.h"
 
 #define PERCONA_AUDIT_STUB {}
 
@@ -82,11 +82,11 @@ namespace mongo {
 
 namespace audit {
 
-    NOINLINE_DECL void realexit( ExitCode rc ) {
+    MONGO_COMPILER_NOINLINE void realexit( ExitCode rc ) {
 #ifdef _COVERAGE
         // Need to make sure coverage data is properly flushed before exit.
         // It appears that ::_exit() does not do this.
-        log() << "calling regular ::exit() so coverage data may flush..." << std::endl;
+        LOGV2(29013, "calling regular ::exit() so coverage data may flush...");
         ::exit( rc );
 #else
         ::_exit( rc );
@@ -105,33 +105,22 @@ namespace audit {
     class WritableAuditLog : public logger::AuditLog {
     public:
         WritableAuditLog(const BSONObj &filter)
-            : _matcher(filter.getOwned(), new ExpressionContext(nullptr, nullptr)) {
+            : _matcher(filter.getOwned(), new ExpressionContext(nullptr, nullptr, NamespaceString())) {
         }
         virtual ~WritableAuditLog() {}
 
-        void append(const BSONObj &obj, const bool affects_durable_state) {
+        void append(const BSONObj &obj) {
             if (_matcher.matches(obj)) {
-                appendMatched(obj, affects_durable_state);
+                appendMatched(obj);
             }
         }
-        virtual void rotate(bool rename, StringData renameSuffix) override {
+        virtual void rotate() {
             // No need to override this method if there is nothing to rotate
             // like it is for 'console' and 'syslog' destinations
         }
 
-        virtual void flush() {
-            // No need to override this method if there is nothing to flush
-            // like it is for 'console' and 'syslog' destinations
-
-        }
-
-        virtual void fsync() {
-            // No need to override this method if there is nothing to fsync
-            // like it is for 'console' and 'syslog' destinations
-        }
-
     protected:
-        virtual void appendMatched(const BSONObj &obj, const bool affects_durable_state) = 0;
+        virtual void appendMatched(const BSONObj &obj) = 0;
 
     private:
         const Matcher _matcher;
@@ -146,21 +135,12 @@ namespace audit {
                     errcode == EINTR);
         }
 
-        typedef boost::iostreams::stream<boost::iostreams::file_descriptor_sink> Sink;
-
     public:
         FileAuditLog(const std::string &file, const BSONObj &filter)
             : WritableAuditLog(filter),
-              _file(new Sink),
+              _file(new AuditFile),
               _fileName(file) {
-            _file->open(file.c_str(), std::ios_base::out | std::ios_base::app | std::ios_base::binary);
-        }
-
-        virtual ~FileAuditLog() {
-            if (_dirty) {
-                flush_inlock();
-                _dirty = false;
-            }
+            _file->open(file.c_str(), false, false);
         }
 
     protected:
@@ -168,7 +148,7 @@ namespace audit {
         // and passess ownership to caller
         virtual AuditLogFormatAdapter *createAdapter(const BSONObj &obj) const = 0;
 
-        virtual void appendMatched(const BSONObj &obj, const bool affects_durable_state) override {
+        virtual void appendMatched(const BSONObj &obj) {
             boost::scoped_ptr<AuditLogFormatAdapter> adapter(createAdapter(obj));
 
             // mongo::File does not have an "atomic append" operation.
@@ -187,137 +167,121 @@ namespace audit {
             // logRotate destroying our pointer.  Welp.
             stdx::lock_guard<SimpleMutex> lck(_mutex);
 
-            _dirty = true;
-            if (affects_durable_state)
-                _fsync_pending = true;
-
-            invariant(_membuf.write(adapter->data(), adapter->size()));
-        }
-
-        virtual void rotate(bool rename, StringData renameSuffix) override {
-            stdx::lock_guard<SimpleMutex> lck(_mutex);
-
-            // Close the current file.
-            _file.reset();
-
-            if (rename) {
-                // Rename the current file
-                // Note: we append a timestamp to the file name.
-                std::string s = _fileName + renameSuffix;
-                int r = std::rename(_fileName.c_str(), s.c_str());
-                if (r != 0) {
-                    error() << "Could not rotate audit log, but continuing normally "
-                            << "(error desc: " << errnoWithDescription() << ")"
-                            << std::endl;
-                }
-            }
-
-            // Open a new file, with the same name as the original.
-            _file.reset(new Sink);
-            _file->open(_fileName.c_str());
-        }
-
-        virtual void flush() override {
-            stdx::lock_guard<SimpleMutex> lck(_mutex);
-
-            if (_dirty) {
-                flush_inlock();
-                _dirty = false;
-            }
-        }
-
-        virtual void fsync() override {
-            stdx::lock_guard<SimpleMutex> lck(_mutex);
-
-            if (_fsync_pending) {
-                if (_dirty) {
-                    flush_inlock();
-                    _dirty = false;
-                }
-
-                fsync_inlock();
-                _fsync_pending = false;
-            }
-        }
-
-    private:
-        std::ostringstream _membuf;
-        boost::scoped_ptr<Sink> _file;
-        const std::string _fileName;
-        SimpleMutex _mutex;
-        bool _dirty = false;
-        bool _fsync_pending = false;
-
-        void flush_inlock() {
             // If pwrite performs a partial write, we don't want to
             // muck about figuring out how much it did write (hard to
             // get out of the File abstraction) and then carefully
             // writing the rest.  Easier to calculate the position
             // first, then repeatedly write to that position if we
             // have to retry.
-            auto pos = _file->tellp();
-            auto data = _membuf.str();
-            _membuf.str({});
+            fileofs pos = _file->len();
 
             int writeRet;
             for (int retries = 10; retries > 0; --retries) {
-                writeRet = 0;
-                _file->seekp(pos);
-                if (_file->write(data.c_str(), data.length()))
+                writeRet = _file->writeReturningError(pos, adapter->data(), adapter->size());
+                if (writeRet == 0) {
                     break;
-                writeRet = errno;
-                if (!ioErrorShouldRetry(writeRet)) {
-                    error() << "Audit system cannot write " << data.length() << " bytes to log file " << _fileName << std::endl;
-                    error() << "Write failed with fatal error " << errnoWithDescription(writeRet) << std::endl;
-                    error() << "As audit cannot make progress, the server will now shut down." << std::endl;
+                } else if (!ioErrorShouldRetry(writeRet)) {
+                    LOGV2_ERROR(29017,
+                        "Audit system cannot write event {event} to log file {file}. "
+                        "Write failed with fatal error {err_desc}. "
+                        "As audit cannot make progress, the server will now shut down.",
+                        "event"_attr = redact(obj),
+                        "file"_attr = _fileName,
+                        "err_desc"_attr = errnoWithDescription(writeRet));
                     realexit(EXIT_AUDIT_ERROR);
                 }
-                warning() << "Audit system cannot write " << data.length() << " bytes to log file " << _fileName << std::endl;
-                warning() << "Write failed with retryable error " << errnoWithDescription(writeRet) << std::endl;
-                warning() << "Audit system will retry this write another " << retries - 1 << " times." << std::endl;
+                LOGV2_WARNING(29018,
+                    "Audit system cannot write event {event} to log file {file}. "
+                    "Write failed with retryable error {err_desc}. "
+                    "Audit system will retry this write another {retries} times.",
+                    "event"_attr = redact(obj),
+                    "file"_attr = _fileName,
+                    "err_desc"_attr = errnoWithDescription(writeRet),
+                    "retries"_attr = retries - 1);
                 if (retries <= 7 && retries > 0) {
                     sleepmillis(1 << ((7 - retries) * 2));
                 }
-                _file->clear();
             }
 
             if (writeRet != 0) {
-                error() << "Audit system cannot write " << data.length() << " bytes to log file " << _fileName << std::endl;
-                error() << "Write failed with fatal error " << errnoWithDescription(writeRet) << std::endl;
-                error() << "As audit cannot make progress, the server will now shut down." << std::endl;
+                LOGV2_ERROR(29019,
+                    "Audit system cannot write event {event} to log file {file}. "
+                    "Write failed with fatal error {err_desc}. "
+                    "As audit cannot make progress, the server will now shut down.",
+                    "event"_attr = redact(obj),
+                    "file"_attr = _fileName,
+                    "err_desc"_attr = errnoWithDescription(writeRet));
                 realexit(EXIT_AUDIT_ERROR);
             }
 
-            _file->flush();
-        }
-
-        void fsync_inlock() {
             int fsyncRet;
             for (int retries = 10; retries > 0; --retries) {
-                fsyncRet = ::fsync((*_file)->handle());
+                fsyncRet = _file->fsyncReturningError();
                 if (fsyncRet == 0) {
                     break;
                 } else if (!ioErrorShouldRetry(fsyncRet)) {
-                    error() << "Audit system cannot fsync log file " << _fileName << std::endl;
-                    error() << "Fsync failed with fatal error " << errnoWithDescription(fsyncRet) << std::endl;
-                    error() << "As audit cannot make progress, the server will now shut down." << std::endl;
+                    LOGV2_ERROR(29020,
+                        "Audit system cannot fsync event {event} to log file {file}. "
+                        "Fsync failed with fatal error {err_desc}. "
+                        "As audit cannot make progress, the server will now shut down.",
+                        "event"_attr = redact(obj),
+                        "file"_attr = _fileName,
+                        "err_desc"_attr = errnoWithDescription(fsyncRet));
                     realexit(EXIT_AUDIT_ERROR);
                 }
-                warning() << "Audit system cannot fsync log file " << _fileName << std::endl;
-                warning() << "Fsync failed with retryable error " << errnoWithDescription(fsyncRet) << std::endl;
-                warning() << "Audit system will retry this fsync another " << retries - 1 << " times." << std::endl;
+                LOGV2_WARNING(29021,
+                    "Audit system cannot fsync event {event} to log file {file}. "
+                    "Fsync failed with retryable error {err_desc}. "
+                    "Audit system will retry this fsync another {retries} times.",
+                    "event"_attr = redact(obj),
+                    "file"_attr = _fileName,
+                    "err_desc"_attr = errnoWithDescription(fsyncRet),
+                    "retries"_attr = retries - 1);
                 if (retries <= 7 && retries > 0) {
                     sleepmillis(1 << ((7 - retries) * 2));
                 }
             }
 
             if (fsyncRet != 0) {
-                error() << "Audit system cannot fsync log file " << _fileName << std::endl;
-                error() << "Fsync failed with fatal error " << errnoWithDescription(fsyncRet) << std::endl;
-                error() << "As audit cannot make progress, the server will now shut down." << std::endl;
+                LOGV2_ERROR(29022,
+                    "Audit system cannot fsync event {event} to log file {file}. "
+                    "Fsync failed with fatal error {err_desc}. "
+                    "As audit cannot make progress, the server will now shut down.",
+                    "event"_attr = redact(obj),
+                    "file"_attr = _fileName,
+                    "err_desc"_attr = errnoWithDescription(fsyncRet));
                 realexit(EXIT_AUDIT_ERROR);
             }
         }
+
+        virtual void rotate() {
+            stdx::lock_guard<SimpleMutex> lck(_mutex);
+
+            // Close the current file.
+            _file.reset();
+
+            // Rename the current file
+            // Note: we append a timestamp to the file name.
+            std::stringstream ss;
+            ss << _fileName << "." << terseCurrentTime(false);
+            std::string s = ss.str();
+            int r = std::rename(_fileName.c_str(), s.c_str());
+            if (r != 0) {
+                LOGV2_ERROR(29016,
+                            "Could not rotate audit log, but continuing normally "
+                            "(error desc: {err_desc})",
+                            "err_desc"_attr = errnoWithDescription());
+            }
+
+            // Open a new file, with the same name as the original.
+            _file.reset(new AuditFile);
+            _file->open(_fileName.c_str(), false, false);
+        }
+
+    private:
+        boost::scoped_ptr<AuditFile> _file;
+        const std::string _fileName;
+        SimpleMutex _mutex;
     };    
 
     // Writes audit events to a json file
@@ -383,7 +347,7 @@ namespace audit {
         }
 
     private:
-        virtual void appendMatched(const BSONObj &obj, const bool affects_durable_state) override {
+        virtual void appendMatched(const BSONObj &obj) {
             std::cout << obj.jsonString() << std::endl;
         }
 
@@ -397,7 +361,7 @@ namespace audit {
         }
 
     private:
-        virtual void appendMatched(const BSONObj &obj, const bool affects_durable_state) override {
+        virtual void appendMatched(const BSONObj &obj) {
             syslog(LOG_MAKEPRI(LOG_USER, LOG_INFO), "%s", obj.jsonString().c_str());
         }
 
@@ -414,7 +378,7 @@ namespace audit {
         }
 
     protected:
-        void appendMatched(const BSONObj &obj, const bool affects_durable_state) override {
+        void appendMatched(const BSONObj &obj) {
             verify(!obj.jsonString().empty());
         }
 
@@ -423,14 +387,11 @@ namespace audit {
     static std::shared_ptr<WritableAuditLog> _auditLog;
 
     static void _setGlobalAuditLog(WritableAuditLog *log) {
+        _auditLog.reset(log);
+
         // Sets the audit log in the general logging framework which
         // will rotate() the audit log when the server log rotates.
         setAuditLog(log);
-
-        // Must be the last line because this function is also used
-        // for cleanup (log can be nullptr)
-        // Otherwise race condition exists during cleanup.
-        _auditLog.reset(log);
     }
 
     static bool _auditEnabledOnCommandLine() {
@@ -441,14 +402,14 @@ namespace audit {
         if (!_auditEnabledOnCommandLine()) {
             // Write audit events into the void for debug builds, so we get
             // coverage on the code that generates audit log objects.
-            DEV {
-                log() << "Initializing dev null audit..." << std::endl;
+            if (kDebugBuild) {
+                LOGV2(29014, "Initializing dev null audit...");
                 _setGlobalAuditLog(new VoidAuditLog(fromjson(auditOptions.filter)));
             }
             return Status::OK();
         }
 
-        log() << "Initializing audit..." << std::endl;
+        LOGV2(29015, "Initializing audit...");
         const BSONObj filter = fromjson(auditOptions.filter);
         if (auditOptions.destination == "console")
             _setGlobalAuditLog(new ConsoleAuditLog(filter));
@@ -576,13 +537,12 @@ namespace audit {
     static void _auditEvent(Client* client,
                             StringData atype,
                             const BSONObj& params,
-                            ErrorCodes::Error result = ErrorCodes::OK,
-                            const bool affects_durable_state = true) {
+                            ErrorCodes::Error result = ErrorCodes::OK) {
         BSONObjBuilder builder;
         appendCommonInfo(builder, atype, client);
         builder << AuditFields::param(params);
         builder << AuditFields::result(static_cast<int>(result));
-        _auditLog->append(builder.done(), affects_durable_state);
+        _auditLog->append(builder.done());
     }
 
     static void _auditAuthz(Client* client,
@@ -595,7 +555,7 @@ namespace audit {
             const BSONObj params = !ns.empty() ?
                 BSON("command" << command << "ns" << ns << "args" << args) :
                 BSON("command" << command << "args" << args);
-            _auditEvent(client, "authCheck", params, result, false);
+            _auditEvent(client, "authCheck", params, result);
         }
     }
 
@@ -621,7 +581,7 @@ namespace audit {
         const BSONObj params = BSON("user" << user.getUser() <<
                                     "db" << user.getDB() <<
                                     "mechanism" << mechanism);
-        _auditEvent(client, "authenticate", params, result, false);
+        _auditEvent(client, "authenticate", params, result);
     }
 
     void logCommandAuthzCheck(Client* client,
@@ -747,7 +707,7 @@ namespace audit {
         }
 
         const BSONObj params = BSON("msg" << msg);
-        _auditEvent(client, "applicationMessage", params, ErrorCodes::OK, false);
+        _auditEvent(client, "applicationMessage", params);
     }
 
     void logShutdown(Client* client) {
@@ -757,10 +717,6 @@ namespace audit {
 
         const BSONObj params = BSONObj();
         _auditEvent(client, "shutdown", params);
-
-        // This is always the last event
-        // Destroy audit log here
-        _setGlobalAuditLog(nullptr);
     }
 
     void logCreateIndex(Client* client,
@@ -1090,6 +1046,18 @@ namespace audit {
         _auditEvent(client, "revokePrivilegesFromRole", params.done());
     }
 
+    void logRefineCollectionShardKey(Client* client,
+                                     StringData ns,
+                                     const BSONObj& keyPattern) {
+        if (!_auditLog) {
+            return;
+        }
+
+        const BSONObj params = BSON("ns" << ns <<
+                                    "key" << keyPattern);
+        _auditEvent(client, "refineCollectionShardKey", params);
+    }
+
     void writeImpersonatedUsersToMetadata(OperationContext* txn,
                                           BSONObjBuilder* metadata) PERCONA_AUDIT_STUB
 
@@ -1105,23 +1073,6 @@ namespace audit {
             std::vector<RoleName>* parsedRoleNames,
             bool* fieldIsPresent) PERCONA_AUDIT_STUB
 
-
-    void flushAuditLog() {
-        if (!_auditLog) {
-            return;
-        }
-
-        _auditLog->flush();
-    }
-
-    void fsyncAuditLog() {
-        if (!_auditLog) {
-            return;
-        }
-
-        _auditLog->fsync();
-    }
-
 }  // namespace audit
 }  // namespace mongo
 
