@@ -31,7 +31,7 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
     it in the license file.
 ======= */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kAccessControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 #include "mongo/db/auth/external/external_sasl_authentication_session.h"
 
@@ -48,7 +48,7 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
 #include "mongo/db/ldap/ldap_manager.h"
 #include "mongo/db/ldap/ldap_manager_impl.h"
 #include "mongo/db/ldap_options.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/str.h"
 #include "mongo/util/net/socket_utils.h"
 
@@ -156,12 +156,6 @@ StatusWith<std::tuple<bool, std::string>> OpenLDAPServerMechanism::stepImpl(
         const char* authn_id = authz_id + std::strlen(authz_id) + 1; // authentication id
         const char* pw = authn_id + std::strlen(authn_id) + 1; // password
 
-        if(strlen(pw) == 0) {
-            return Status(ErrorCodes::LDAPLibraryError,
-                          "Failed to authenticate '{}'; No password provided."_format(
-                              authn_id));
-        }
-
         // transform user to DN
         std::string mappedUser;
         {
@@ -171,13 +165,23 @@ StatusWith<std::tuple<bool, std::string>> OpenLDAPServerMechanism::stepImpl(
                 return mapRes;
         }
 
-        auto uri = ldapGlobalParams.ldapURIList();
+        const char* ldapprot = "ldaps";
+        if (ldapGlobalParams.ldapTransportSecurity == "none")
+            ldapprot = "ldap";
+        auto uri = "{}://{}/"_format(ldapprot, ldapGlobalParams.ldapServers.get());
         int res = ldap_initialize(&_ld, uri.c_str());
         if (res != LDAP_SUCCESS) {
             return Status(ErrorCodes::LDAPLibraryError,
                           "Cannot initialize LDAP structure for {}; LDAP error: {}"_format(
                               uri, ldap_err2string(res)));
         }
+        const int ldap_version = LDAP_VERSION3;
+        res = ldap_set_option(_ld, LDAP_OPT_PROTOCOL_VERSION, &ldap_version);
+        if (res != LDAP_OPT_SUCCESS) {
+            return Status(ErrorCodes::LDAPLibraryError,
+                          "Cannot set LDAP version option; LDAP error: {}"_format(
+                              ldap_err2string(res)));
+        }
 
         Status status = LDAPbind(_ld, mappedUser.c_str(), pw);
         if (!status.isOK())
@@ -198,7 +202,8 @@ StringData OpenLDAPServerMechanism::getPrincipalName() const {
 namespace {
 
 int saslServerLog(void* context, int priority, const char* message) throw() {
-    log() << "SASL server message: (" << priority << ") " << message;
+    LOGV2(29052, "SASL server message: ({priority}) {msg}",
+          "priority"_attr = priority, "msg"_attr = message);
     return SASL_OK;  // do nothing
 }
 
@@ -213,7 +218,7 @@ MONGO_INITIALIZER(SaslExternalLDAPServerMechanism)(InitializerContext*) {
     };
     int result = sasl_server_init(saslServerGlobalCallbacks, saslGlobalParams.serviceName.c_str());
     if (result != SASL_OK) {
-        error() << "SASL server initialization failed";
+        LOGV2_ERROR(29030, "SASL server initialization failed");
         return getInitializationError(result);
     }
     return Status::OK();
@@ -224,6 +229,7 @@ MONGO_INITIALIZER(SaslExternalLDAPServerMechanism)(InitializerContext*) {
  * depending on current server configuration. */
 class ExternalLDAPServerFactory : public ServerFactoryBase {
 public:
+    using ServerFactoryBase::ServerFactoryBase;
     using policy_type = PLAINPolicy;
 
     static constexpr bool isInternal = false;
@@ -253,7 +259,8 @@ public:
 
     bool canMakeMechanismForUser(const User* user) const final {
         auto credentials = user->getCredentials();
-        return credentials.isExternal;
+        return credentials.isExternal && (credentials.scram<SHA1Block>().isValid() ||
+                                          credentials.scram<SHA256Block>().isValid());
     }
 };
 
