@@ -79,7 +79,7 @@ public:
              const BSONObj& cmdObj,
              std::string& errmsg,
              BSONObjBuilder& result) override;
-    void redactForLogging(mutablebson::Document* cmdObj) const override;
+    void snipForLogging(mutablebson::Document* cmdObj) const override;
 } createBackupCmd;
 
 bool CreateBackupCommand::errmsgRun(mongo::OperationContext* opCtx,
@@ -162,14 +162,6 @@ bool CreateBackupCommand::errmsgRun(mongo::OperationContext* opCtx,
                 s3params.accessKeyId = elem.String();
             else if (elem.fieldNameStringData() == "secretAccessKey"_sd)
                 s3params.secretAccessKey = elem.String();
-            else if (elem.fieldNameStringData() == "threadPoolSize"_sd) {
-                int i = elem.Double();
-                if (i < 1 || i > 256) {
-                    errmsg = "threadPoolSize must be an integer in range [1..256] (default is 4)";
-                    return false;
-                }
-                s3params.threadPoolSize = i;
-            }
             else {
                 errmsg = str::stream()
                     << "s3 subobject contains usupported field or field's name is misspelled: "
@@ -202,7 +194,7 @@ bool CreateBackupCommand::errmsgRun(mongo::OperationContext* opCtx,
     return true;
 }
 
-void CreateBackupCommand::redactForLogging(mutablebson::Document* cmdObj) const {
+void CreateBackupCommand::snipForLogging(mutablebson::Document* cmdObj) const {
     namespace mmb = mutablebson;
     const auto s3FieldName = "s3"_sd;
     mmb::Element s3Element = mmb::findFirstChildNamed(cmdObj->root(), s3FieldName);
