@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -45,6 +45,7 @@
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/index_builds_coordinator.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/op_observer.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/query/plan_yield_policy.h"
@@ -63,7 +64,7 @@ Status emptyCapped(OperationContext* opCtx, const NamespaceString& collectionNam
         !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, collectionName);
 
     if (userInitiatedWritesAndNotPrimary) {
-        return Status(ErrorCodes::NotWritablePrimary,
+        return Status(ErrorCodes::NotMaster,
                       str::stream()
                           << "Not primary while truncating collection: " << collectionName);
     }
@@ -71,7 +72,8 @@ Status emptyCapped(OperationContext* opCtx, const NamespaceString& collectionNam
     Database* db = autoDb.getDb();
     uassert(ErrorCodes::NamespaceNotFound, "no such database", db);
 
-    Collection* collection = db->getCollection(opCtx, collectionName);
+    Collection* collection =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, collectionName);
     uassert(ErrorCodes::CommandNotSupportedOnView,
             str::stream() << "emptycapped not supported on view: " << collectionName.ns(),
             collection || !ViewCatalog::get(db)->lookup(opCtx, collectionName.ns()));
@@ -82,11 +84,6 @@ Status emptyCapped(OperationContext* opCtx, const NamespaceString& collectionNam
                       str::stream() << "Cannot truncate a system collection: " << collectionName);
     }
 
-    if (collectionName.isVirtualized()) {
-        return Status(ErrorCodes::IllegalOperation,
-                      str::stream() << "Cannot truncate a virtual collection: " << collectionName);
-    }
-
     if ((repl::ReplicationCoordinator::get(opCtx)->getReplicationMode() !=
          repl::ReplicationCoordinator::modeNone) &&
         collectionName.isOplog()) {
@@ -96,8 +93,7 @@ Status emptyCapped(OperationContext* opCtx, const NamespaceString& collectionNam
     }
 
     BackgroundOperation::assertNoBgOpInProgForNs(collectionName.ns());
-    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(
-        collection->uuid().get());
+    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(collection->uuid());
 
     WriteUnitOfWork wuow(opCtx);
 
@@ -116,14 +112,12 @@ Status emptyCapped(OperationContext* opCtx, const NamespaceString& collectionNam
 
 void cloneCollectionAsCapped(OperationContext* opCtx,
                              Database* db,
-                             const std::string& shortFrom,
-                             const std::string& shortTo,
+                             const NamespaceString& fromNss,
+                             const NamespaceString& toNss,
                              long long size,
                              bool temp) {
-    NamespaceString fromNss(db->name(), shortFrom);
-    NamespaceString toNss(db->name(), shortTo);
-
-    Collection* fromCollection = db->getCollection(opCtx, fromNss);
+    Collection* fromCollection =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, fromNss);
     if (!fromCollection) {
         uassert(ErrorCodes::CommandNotSupportedOnView,
                 str::stream() << "cloneCollectionAsCapped not supported for views: " << fromNss,
@@ -141,11 +135,12 @@ void cloneCollectionAsCapped(OperationContext* opCtx,
     uassert(ErrorCodes::NamespaceExists,
             str::stream() << "cloneCollectionAsCapped failed - destination collection " << toNss
                           << " already exists. source collection: " << fromNss,
-            !db->getCollection(opCtx, toNss));
+            !CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, toNss));
 
     // create new collection
     {
-        auto options = DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, fromNss);
+        auto options =
+            DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, fromCollection->getCatalogId());
         // The capped collection will get its own new unique id, as the conversion isn't reversible,
         // so it can't be rolled back.
         options.uuid.reset();
@@ -160,7 +155,8 @@ void cloneCollectionAsCapped(OperationContext* opCtx,
         uassertStatusOK(createCollection(opCtx, toNss.db().toString(), cmd.done()));
     }
 
-    Collection* toCollection = db->getCollection(opCtx, toNss);
+    Collection* toCollection =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, toNss);
     invariant(toCollection);  // we created above
 
     // how much data to ignore because it won't fit anyway
@@ -239,46 +235,58 @@ void cloneCollectionAsCapped(OperationContext* opCtx,
     MONGO_UNREACHABLE;
 }
 
-void convertToCapped(OperationContext* opCtx,
-                     const NamespaceString& collectionName,
-                     long long size) {
-    StringData dbname = collectionName.db();
-    StringData shortSource = collectionName.coll();
+void convertToCapped(OperationContext* opCtx, const NamespaceString& ns, long long size) {
+    StringData dbname = ns.db();
+    StringData shortSource = ns.coll();
 
-    AutoGetDb autoDb(opCtx, collectionName.db(), MODE_X);
+    AutoGetCollection autoColl(opCtx, ns, MODE_X);
 
     bool userInitiatedWritesAndNotPrimary = opCtx->writesAreReplicated() &&
-        !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, collectionName);
+        !repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, ns);
 
-    uassert(ErrorCodes::NotWritablePrimary,
-            str::stream() << "Not primary while converting " << collectionName
-                          << " to a capped collection",
+    uassert(ErrorCodes::NotMaster,
+            str::stream() << "Not primary while converting " << ns << " to a capped collection",
             !userInitiatedWritesAndNotPrimary);
 
-    Database* const db = autoDb.getDb();
+    Database* const db = autoColl.getDb();
     uassert(
         ErrorCodes::NamespaceNotFound, str::stream() << "database " << dbname << " not found", db);
 
-    BackgroundOperation::assertNoBgOpInProgForDb(dbname);
-    IndexBuildsCoordinator::get(opCtx)->assertNoBgOpInProgForDb(dbname);
+    BackgroundOperation::assertNoBgOpInProgForNs(ns);
+    if (Collection* coll = autoColl.getCollection()) {
+        IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(coll->uuid());
+    }
 
     // Generate a temporary collection name that will not collide with any existing collections.
-    auto tmpNameResult =
-        db->makeUniqueCollectionNamespace(opCtx, "tmp%%%%%.convertToCapped." + shortSource);
-    uassertStatusOKWithContext(tmpNameResult,
-                               str::stream()
-                                   << "Cannot generate temporary collection namespace to convert "
-                                   << collectionName << " to a capped collection");
+    boost::optional<Lock::CollectionLock> collLock;
+    const auto tempNs = [&] {
+        while (true) {
+            auto tmpNameResult =
+                db->makeUniqueCollectionNamespace(opCtx, "tmp%%%%%.convertToCapped." + shortSource);
+            uassertStatusOKWithContext(
+                tmpNameResult,
+                str::stream() << "Cannot generate temporary collection namespace to convert " << ns
+                              << " to a capped collection");
 
-    const auto& longTmpName = tmpNameResult.getValue();
-    const auto shortTmpName = longTmpName.coll().toString();
+            collLock.emplace(opCtx, tmpNameResult.getValue(), MODE_X);
+            if (!CollectionCatalog::get(opCtx).lookupCollectionByNamespace(
+                    opCtx, tmpNameResult.getValue())) {
+                return std::move(tmpNameResult.getValue());
+            }
 
-    cloneCollectionAsCapped(opCtx, db, shortSource.toString(), shortTmpName, size, true);
+            // The temporary collection was created by someone else between the name being
+            // generated and acquiring the lock on the collection, so try again with a new
+            // temporary collection name.
+            collLock.reset();
+        }
+    }();
+
+    cloneCollectionAsCapped(opCtx, db, ns, tempNs, size, true);
 
     RenameCollectionOptions options;
     options.dropTarget = true;
     options.stayTemp = false;
-    uassertStatusOK(renameCollection(opCtx, longTmpName, collectionName, options));
+    uassertStatusOK(renameCollection(opCtx, tempNs, ns, options));
 }
 
 }  // namespace mongo
