@@ -26,17 +26,19 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
 #include "collection_catalog.h"
 
 #include "mongo/db/catalog/database.h"
+#include "mongo/db/catalog/uncommitted_collections.h"
 #include "mongo/db/concurrency/lock_manager_defs.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/storage/recovery_unit.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/log.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
@@ -56,7 +58,7 @@ public:
     }
 
     void rollback() override {
-        _catalog->registerCollection(_uuid, std::move(_coll));
+        _catalog->registerCollection(_uuid, std::move(&_coll));
     }
 
 private:
@@ -76,7 +78,12 @@ CollectionCatalog::iterator::iterator(StringData dbName,
     stdx::lock_guard<Latch> lock(_catalog->_catalogLock);
     _mapIter = _catalog->_orderedCollections.lower_bound(std::make_pair(_dbName, minUuid));
 
-    if (_mapIter != _catalog->_orderedCollections.end() && _mapIter->first.first == _dbName) {
+    // Start with the first collection that is visible outside of its transaction.
+    while (!_exhausted() && !_mapIter->second->isCommitted()) {
+        _mapIter++;
+    }
+
+    if (!_exhausted()) {
         _uuid = _mapIter->first.second;
     }
 }
@@ -85,7 +92,7 @@ CollectionCatalog::iterator::iterator(
     std::map<std::pair<std::string, CollectionUUID>, Collection*>::const_iterator mapIter)
     : _mapIter(mapIter) {}
 
-const CollectionCatalog::iterator::value_type CollectionCatalog::iterator::operator*() {
+CollectionCatalog::iterator::value_type CollectionCatalog::iterator::operator*() {
     stdx::lock_guard<Latch> lock(_catalog->_catalogLock);
     _repositionIfNeeded();
     if (_exhausted()) {
@@ -106,6 +113,11 @@ CollectionCatalog::iterator CollectionCatalog::iterator::operator++() {
         _mapIter++;  // If the position was not updated, increment iterator to next element.
     }
 
+    // Skip any collections that are not yet visible outside of their respective transactions.
+    while (!_exhausted() && !_mapIter->second->isCommitted()) {
+        _mapIter++;
+    }
+
     if (_exhausted()) {
         // If the iterator is at the end of the map or now points to an entry that does not
         // correspond to the correct database.
@@ -145,8 +157,14 @@ bool CollectionCatalog::iterator::_repositionIfNeeded() {
 
     _genNum = _catalog->_generationNumber;
     // If the map has been modified, find the entry the iterator was on, or the one right after it.
+    // The entry the iterator was on must have been for a collection visible outside of its
+    // transaction.
     _mapIter = _catalog->_orderedCollections.lower_bound(std::make_pair(_dbName, *_uuid));
 
+    while (!_exhausted() && !_mapIter->second->isCommitted()) {
+        _mapIter++;
+    }
+
     if (_exhausted()) {
         return true;
     }
@@ -233,51 +251,58 @@ void CollectionCatalog::onOpenCatalog(OperationContext* opCtx) {
     _shadowCatalog.reset();
 }
 
-Collection* CollectionCatalog::lookupCollectionByUUID(CollectionUUID uuid) const {
-    size_t hash = _catalog.hash_function()(uuid);
-    stdx::lock_guard<Latch> lock(_catalogLock);
-    return _lookupCollectionByUUID(lock, uuid, hash);
-}
-
-std::pair<Collection*, boost::optional<NamespaceString>>
-CollectionCatalog::lookupCollectionByUUIDAndVerifyNamespace(CollectionUUID uuid,
-                                                            const NamespaceString& nss) const {
-    size_t hash = _catalog.hash_function()(uuid);
-    stdx::lock_guard<Latch> lock(_catalogLock);
-    auto coll = _lookupCollectionByUUID(lock, uuid, hash);
-    if (coll) {
-        bool renamed = coll->ns() != nss;
-        if (renamed) {
-            return {nullptr, coll->ns()};
-        } else {
-            return {coll, boost::none};
-        }
+Collection* CollectionCatalog::lookupCollectionByUUID(OperationContext* opCtx,
+                                                      CollectionUUID uuid) const {
+    if (auto coll = UncommittedCollections::getForTxn(opCtx, uuid)) {
+        return coll;
     }
-    return {nullptr, boost::none};
+
+    stdx::lock_guard<Latch> lock(_catalogLock);
+    auto coll = _lookupCollectionByUUID(lock, uuid);
+    return (coll && coll->isCommitted()) ? coll : nullptr;
 }
 
-Collection* CollectionCatalog::_lookupCollectionByUUID(WithLock,
-                                                       CollectionUUID uuid,
-                                                       size_t hash) const {
-    auto foundIt = _catalog.find(uuid, hash);
+void CollectionCatalog::makeCollectionVisible(CollectionUUID uuid) {
+    stdx::lock_guard<Latch> lock(_catalogLock);
+    auto coll = _lookupCollectionByUUID(lock, uuid);
+    coll->setCommitted(true);
+}
+
+bool CollectionCatalog::isCollectionAwaitingVisibility(CollectionUUID uuid) const {
+    stdx::lock_guard<Latch> lock(_catalogLock);
+    auto coll = _lookupCollectionByUUID(lock, uuid);
+    return coll && !coll->isCommitted();
+}
+
+Collection* CollectionCatalog::_lookupCollectionByUUID(WithLock, CollectionUUID uuid) const {
+    auto foundIt = _catalog.find(uuid);
     return foundIt == _catalog.end() ? nullptr : foundIt->second.get();
 }
 
-Collection* CollectionCatalog::lookupCollectionByNamespace(const NamespaceString& nss) const {
-    size_t hash = _collections.hash_function()(nss);
+Collection* CollectionCatalog::lookupCollectionByNamespace(OperationContext* opCtx,
+                                                           const NamespaceString& nss) const {
+    if (auto coll = UncommittedCollections::getForTxn(opCtx, nss)) {
+        return coll;
+    }
+
     stdx::lock_guard<Latch> lock(_catalogLock);
-    auto it = _collections.find(nss, hash);
-    return it == _collections.end() ? nullptr : it->second;
+    auto it = _collections.find(nss);
+    auto coll = (it == _collections.end() ? nullptr : it->second);
+    return (coll && coll->isCommitted()) ? coll : nullptr;
 }
 
-boost::optional<NamespaceString> CollectionCatalog::lookupNSSByUUID(CollectionUUID uuid) const {
-    size_t hash = _catalog.hash_function()(uuid);
+boost::optional<NamespaceString> CollectionCatalog::lookupNSSByUUID(OperationContext* opCtx,
+                                                                    CollectionUUID uuid) const {
+    if (auto coll = UncommittedCollections::getForTxn(opCtx, uuid)) {
+        return coll->ns();
+    }
+
     stdx::lock_guard<Latch> lock(_catalogLock);
-    auto foundIt = _catalog.find(uuid, hash);
+    auto foundIt = _catalog.find(uuid);
     if (foundIt != _catalog.end()) {
-        NamespaceString ns = foundIt->second->ns();
-        invariant(!ns.isEmpty());
-        return ns;
+        boost::optional<NamespaceString> ns = foundIt->second->ns();
+        invariant(!ns.get().isEmpty());
+        return _collections.find(ns.get())->second->isCommitted() ? ns : boost::none;
     }
 
     // Only in the case that the catalog is closed and a UUID is currently unknown, resolve it
@@ -292,29 +317,49 @@ boost::optional<NamespaceString> CollectionCatalog::lookupNSSByUUID(CollectionUU
 }
 
 boost::optional<CollectionUUID> CollectionCatalog::lookupUUIDByNSS(
-    const NamespaceString& nss) const {
-    stdx::lock_guard<Latch> lock(_catalogLock);
-    auto minUuid = UUID::parse("00000000-0000-0000-0000-000000000000").getValue();
-    auto it = _orderedCollections.lower_bound(std::make_pair(nss.db().toString(), minUuid));
+    OperationContext* opCtx, const NamespaceString& nss) const {
+    if (auto coll = UncommittedCollections::getForTxn(opCtx, nss)) {
+        return coll->uuid();
+    }
 
-    // The entry _mapIter points to is valid if it's not at the end of _orderedCollections and
-    // the entry's database is the same as dbName.
-    while (it != _orderedCollections.end() && it->first.first == nss.db()) {
-        if (it->second->ns() == nss) {
-            return it->first.second;
-        }
-        ++it;
+    stdx::lock_guard<Latch> lock(_catalogLock);
+    auto it = _collections.find(nss);
+    if (it != _collections.end()) {
+        boost::optional<CollectionUUID> uuid = it->second->uuid();
+        return it->second->isCommitted() ? uuid : boost::none;
     }
     return boost::none;
 }
 
+NamespaceString CollectionCatalog::resolveNamespaceStringOrUUID(OperationContext* opCtx,
+                                                                NamespaceStringOrUUID nsOrUUID) {
+    if (auto& nss = nsOrUUID.nss()) {
+        uassert(ErrorCodes::InvalidNamespace,
+                str::stream() << "Namespace " << *nss << " is not a valid collection name",
+                nss->isValid());
+        return std::move(*nss);
+    }
+
+    auto resolvedNss = lookupNSSByUUID(opCtx, *nsOrUUID.uuid());
+
+    uassert(ErrorCodes::NamespaceNotFound,
+            str::stream() << "Unable to resolve " << nsOrUUID.toString(),
+            resolvedNss && resolvedNss->isValid());
+
+    uassert(ErrorCodes::NamespaceNotFound,
+            str::stream() << "UUID " << nsOrUUID.toString() << " specified in " << nsOrUUID.dbname()
+                          << " resolved to a collection in a different database: " << *resolvedNss,
+            resolvedNss->db() == nsOrUUID.dbname());
+
+    return std::move(*resolvedNss);
+}
+
 bool CollectionCatalog::checkIfCollectionSatisfiable(CollectionUUID uuid,
                                                      CollectionInfoFn predicate) const {
     invariant(predicate);
 
-    size_t hash = _catalog.hash_function()(uuid);
     stdx::lock_guard<Latch> lock(_catalogLock);
-    auto collection = _lookupCollectionByUUID(lock, uuid, hash);
+    auto collection = _lookupCollectionByUUID(lock, uuid);
 
     if (!collection) {
         return false;
@@ -331,7 +376,9 @@ std::vector<CollectionUUID> CollectionCatalog::getAllCollectionUUIDsFromDb(
 
     std::vector<CollectionUUID> ret;
     while (it != _orderedCollections.end() && it->first.first == dbName) {
-        ret.push_back(it->first.second);
+        if (it->second->isCommitted()) {
+            ret.push_back(it->first.second);
+        }
         ++it;
     }
     return ret;
@@ -348,7 +395,9 @@ std::vector<NamespaceString> CollectionCatalog::getAllCollectionNamesFromDb(
     for (auto it = _orderedCollections.lower_bound(std::make_pair(dbName.toString(), minUuid));
          it != _orderedCollections.end() && it->first.first == dbName;
          ++it) {
-        ret.push_back(it->second->ns());
+        if (it->second->isCommitted()) {
+            ret.push_back(it->second->ns());
+        }
     }
     return ret;
 }
@@ -360,27 +409,47 @@ std::vector<std::string> CollectionCatalog::getAllDbNames() const {
     auto iter = _orderedCollections.upper_bound(std::make_pair("", maxUuid));
     while (iter != _orderedCollections.end()) {
         auto dbName = iter->first.first;
-        ret.push_back(dbName);
+        if (iter->second->isCommitted()) {
+            ret.push_back(dbName);
+        } else {
+            // If the first collection found for `dbName` is not yet committed, increment the
+            // iterator to find the next visible collection (possibly under a different `dbName`).
+            iter++;
+            continue;
+        }
+        // Move on to the next database after `dbName`.
         iter = _orderedCollections.upper_bound(std::make_pair(dbName, maxUuid));
     }
     return ret;
 }
 
-void CollectionCatalog::registerCollection(CollectionUUID uuid, std::unique_ptr<Collection> coll) {
+void CollectionCatalog::registerCollection(CollectionUUID uuid, std::unique_ptr<Collection>* coll) {
+    auto ns = (*coll)->ns();
     stdx::lock_guard<Latch> lock(_catalogLock);
+    if (_collections.find(ns) != _collections.end()) {
+        LOGV2(20279,
+              "Conflicted creating a collection. ns: {coll_ns} ({coll_uuid}).",
+              "Conflicted creating a collection",
+              "namespace"_attr = (*coll)->ns(),
+              "uuid"_attr = (*coll)->uuid());
+        throw WriteConflictException();
+    }
 
-    LOG(1) << "Registering collection " << coll->ns() << " with UUID " << uuid;
+    LOGV2_DEBUG(20280,
+                1,
+                "Registering collection {ns} with UUID {uuid}",
+                "Registering collection",
+                "namespace"_attr = ns,
+                "uuid"_attr = uuid);
 
-    auto ns = coll->ns();
     auto dbName = ns.db().toString();
     auto dbIdPair = std::make_pair(dbName, uuid);
 
     // Make sure no entry related to this uuid.
     invariant(_catalog.find(uuid) == _catalog.end());
-    invariant(_collections.find(ns) == _collections.end());
     invariant(_orderedCollections.find(dbIdPair) == _orderedCollections.end());
 
-    _catalog[uuid] = std::move(coll);
+    _catalog[uuid] = std::move(*coll);
     _collections[ns] = _catalog[uuid].get();
     _orderedCollections[dbIdPair] = _catalog[uuid].get();
 
@@ -401,7 +470,7 @@ std::unique_ptr<Collection> CollectionCatalog::deregisterCollection(CollectionUU
     auto dbName = ns.db().toString();
     auto dbIdPair = std::make_pair(dbName, uuid);
 
-    LOG(1) << "Deregistering collection " << ns << " with UUID " << uuid;
+    LOGV2_DEBUG(20281, 1, "Deregistering collection", "namespace"_attr = ns, "uuid"_attr = uuid);
 
     // Make sure collection object exists.
     invariant(_collections.find(ns) != _collections.end());
@@ -421,22 +490,23 @@ std::unique_ptr<Collection> CollectionCatalog::deregisterCollection(CollectionUU
     return coll;
 }
 
-RecoveryUnit::Change* CollectionCatalog::makeFinishDropCollectionChange(
+std::unique_ptr<RecoveryUnit::Change> CollectionCatalog::makeFinishDropCollectionChange(
     std::unique_ptr<Collection> coll, CollectionUUID uuid) {
-    return new FinishDropCollectionChange(this, std::move(coll), uuid);
+    return std::make_unique<FinishDropCollectionChange>(this, std::move(coll), uuid);
 }
 
 void CollectionCatalog::deregisterAllCollections() {
     stdx::lock_guard<Latch> lock(_catalogLock);
 
-    LOG(0) << "Deregistering all the collections";
+    LOGV2(20282, "Deregistering all the collections");
     for (auto& entry : _catalog) {
         auto uuid = entry.first;
         auto ns = entry.second->ns();
         auto dbName = ns.db().toString();
         auto dbIdPair = std::make_pair(dbName, uuid);
 
-        LOG(1) << "Deregistering collection " << ns << " with UUID " << uuid;
+        LOGV2_DEBUG(
+            20283, 1, "Deregistering collection", "namespace"_attr = ns, "uuid"_attr = uuid);
 
         entry.second.reset();
     }
