@@ -30,6 +30,7 @@
 #pragma once
 
 #include <boost/optional.hpp>
+#include <functional>
 #include <string>
 
 #include "mongo/base/owned_pointer_vector.h"
@@ -40,13 +41,11 @@
 #include "mongo/db/storage/kv/kv_prefix.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/stdx/functional.h"
 #include "mongo/util/debug_util.h"
 
 namespace mongo {
 class CollatorInterface;
 class CollectionCatalogEntry;
-class CollectionInfoCache;
 class IndexAccessMethod;
 class IndexBuildInterceptor;
 class IndexDescriptor;
@@ -65,6 +64,8 @@ public:
 
     virtual void init(std::unique_ptr<IndexAccessMethod> accessMethod) = 0;
 
+    virtual const std::string& getIdent() const = 0;
+
     virtual IndexDescriptor* descriptor() = 0;
 
     virtual const IndexDescriptor* descriptor() const = 0;
@@ -91,12 +92,15 @@ public:
 
     virtual void setIsReady(const bool newIsReady) = 0;
 
+    virtual void setDropped() = 0;
+    virtual bool isDropped() const = 0;
+
     // --
 
     /**
      * Returns true if this index is multikey and false otherwise.
      */
-    virtual bool isMultikey(OperationContext* opCtx) const = 0;
+    virtual bool isMultikey() const = 0;
 
     /**
      * Returns the path components that cause this index to be multikey if this index supports
@@ -124,13 +128,17 @@ public:
      * namespace, index name, and multikey paths on the OperationContext rather than set the index
      * as multikey here.
      */
-    virtual void setMultikey(OperationContext* const opCtx,
-                             const std::vector<BSONObj>& multikeyMetadataKeys,
-                             const MultikeyPaths& multikeyPaths) = 0;
+    virtual void setMultikey(OperationContext* const opCtx, const MultikeyPaths& multikeyPaths) = 0;
 
     // if this ready is ready for queries
     virtual bool isReady(OperationContext* const opCtx) const = 0;
 
+    /**
+     * Returns true if this index is not ready, and it is not currently in the process of being
+     * built either.
+     */
+    virtual bool isFrozen() const = 0;
+
     virtual KVPrefix getPrefix() const = 0;
 
     /**
@@ -140,8 +148,6 @@ public:
     virtual boost::optional<Timestamp> getMinimumVisibleSnapshot() = 0;
 
     virtual void setMinimumVisibleSnapshot(const Timestamp name) = 0;
-
-    virtual void setNs(NamespaceString ns) = 0;
 };
 
 class IndexCatalogEntryContainer {
