@@ -27,16 +27,17 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kIndex
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kIndex
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/catalog/index_catalog_entry_impl.h"
 
 #include <algorithm>
+#include <memory>
 
 #include "mongo/base/init.h"
-#include "mongo/db/catalog/collection_info_cache_impl.h"
+#include "mongo/db/catalog/collection.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/index/index_access_method.h"
@@ -45,13 +46,15 @@
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/matcher/expression_parser.h"
 #include "mongo/db/multi_key_path_tracker.h"
+#include "mongo/db/op_observer.h"
 #include "mongo/db/operation_context.h"
+#include "mongo/db/pipeline/expression_context.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
 #include "mongo/db/transaction_participant.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
@@ -59,23 +62,29 @@ namespace mongo {
 using std::string;
 
 IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
-                                             const NamespaceString& nss,
+                                             const std::string& ident,
                                              std::unique_ptr<IndexDescriptor> descriptor,
-                                             CollectionInfoCache* const infoCache)
-    : _ns(nss),
+                                             CollectionQueryInfo* const queryInfo,
+                                             bool isFrozen)
+    : _ident(ident),
       _descriptor(std::move(descriptor)),
-      _infoCache(infoCache),
+      _queryInfo(queryInfo),
       _ordering(Ordering::make(_descriptor->keyPattern())),
       _isReady(false),
-      _prefix(DurableCatalog::get(opCtx)->getIndexPrefix(opCtx, nss, _descriptor->indexName())) {
+      _isFrozen(isFrozen),
+      _isDropped(false),
+      _prefix(DurableCatalog::get(opCtx)->getIndexPrefix(
+          opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName())) {
     _descriptor->_cachedEntry = this;
 
     _isReady = _catalogIsReady(opCtx);
 
     {
         stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-        _isMultikey.store(_catalogIsMultikey(opCtx, &_indexMultikeyPaths));
-        _indexTracksMultikeyPathsInCatalog = !_indexMultikeyPaths.empty();
+        const bool isMultikey = _catalogIsMultikey(opCtx, &_indexMultikeyPaths);
+        _isMultikeyForRead.store(isMultikey);
+        _isMultikeyForWrite.store(isMultikey);
+        _indexTracksPathLevelMultikeyInfo = !_indexMultikeyPaths.empty();
     }
 
     const BSONObj& collation = _descriptor->collation();
@@ -92,20 +101,24 @@ IndexCatalogEntryImpl::IndexCatalogEntryImpl(OperationContext* const opCtx,
     if (_descriptor->isPartial()) {
         const BSONObj& filter = _descriptor->partialFilterExpression();
 
-        boost::intrusive_ptr<ExpressionContext> expCtx(
-            new ExpressionContext(opCtx, _collator.get()));
+        _expCtxForFilter = make_intrusive<ExpressionContext>(
+            opCtx, CollatorInterface::cloneCollator(_collator.get()), ns());
 
         // Parsing the partial filter expression is not expected to fail here since the
         // expression would have been successfully parsed upstream during index creation.
         StatusWithMatchExpression statusWithMatcher =
             MatchExpressionParser::parse(filter,
-                                         std::move(expCtx),
+                                         _expCtxForFilter,
                                          ExtensionsCallbackNoop(),
                                          MatchExpressionParser::kBanAllSpecialFeatures);
         invariant(statusWithMatcher.getStatus());
         _filterExpression = std::move(statusWithMatcher.getValue());
-        LOG(2) << "have filter expression for " << _ns << " " << _descriptor->indexName() << " "
-               << redact(filter);
+        LOGV2_DEBUG(20350,
+                    2,
+                    "have filter expression for {ns} {descriptor_indexName} {filter}",
+                    "ns"_attr = ns(),
+                    "descriptor_indexName"_attr = _descriptor->indexName(),
+                    "filter"_attr = redact(filter));
     }
 }
 
@@ -115,6 +128,10 @@ IndexCatalogEntryImpl::~IndexCatalogEntryImpl() {
     _descriptor.reset();
 }
 
+const NamespaceString& IndexCatalogEntryImpl::ns() const {
+    return _descriptor->parentNS();
+}
+
 void IndexCatalogEntryImpl::init(std::unique_ptr<IndexAccessMethod> accessMethod) {
     invariant(!_accessMethod);
     _accessMethod = std::move(accessMethod);
@@ -133,57 +150,23 @@ bool IndexCatalogEntryImpl::isReady(OperationContext* opCtx) const {
         }
     }
 
-    DEV invariant(_isReady == _catalogIsReady(opCtx));
+    if (kDebugBuild)
+        invariant(_isReady == _catalogIsReady(opCtx));
     return _isReady;
 }
 
-bool IndexCatalogEntryImpl::isMultikey(OperationContext* opCtx) const {
-    auto ret = _isMultikey.load();
-    if (ret) {
-        return true;
-    }
+bool IndexCatalogEntryImpl::isFrozen() const {
+    invariant(!_isFrozen || !_isReady);
+    return _isFrozen;
+}
 
-    // Multikey updates are only persisted, to disk and in memory, when the transaction
-    // commits. In the case of multi-statement transactions, a client attempting to read their own
-    // transactions writes can return wrong results if their writes include multikey changes.
-    //
-    // To accomplish this, the write-path will persist multikey changes on the `Session` object
-    // and the read-path will query this state before determining there is no interesting multikey
-    // state. Note, it's always legal, though potentially wasteful, to return `true`.
-    auto txnParticipant = TransactionParticipant::get(opCtx);
-    if (!txnParticipant || !txnParticipant.transactionIsOpen()) {
-        return false;
-    }
-
-    invariant(txnParticipant);
-
-    for (const MultikeyPathInfo& path : txnParticipant.getUncommittedMultikeyPathInfos()) {
-        if (path.nss == NamespaceString(_ns) && path.indexName == _descriptor->indexName()) {
-            return true;
-        }
-    }
-
-    return false;
+bool IndexCatalogEntryImpl::isMultikey() const {
+    return _isMultikeyForRead.load();
 }
 
 MultikeyPaths IndexCatalogEntryImpl::getMultikeyPaths(OperationContext* opCtx) const {
     stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-
-    auto txnParticipant = TransactionParticipant::get(opCtx);
-    if (!txnParticipant || !txnParticipant.transactionIsOpen()) {
-        return _indexMultikeyPaths;
-    }
-
-    invariant(txnParticipant);
-
-    MultikeyPaths ret = _indexMultikeyPaths;
-    for (const MultikeyPathInfo& path : txnParticipant.getUncommittedMultikeyPathInfos()) {
-        if (path.nss == NamespaceString(_ns) && path.indexName == _descriptor->indexName()) {
-            MultikeyPathTracker::mergeMultikeyPaths(&ret, path.multikeyPaths);
-        }
-    }
-
-    return ret;
+    return _indexMultikeyPaths;
 }
 
 // ---
@@ -199,19 +182,14 @@ void IndexCatalogEntryImpl::setIsReady(bool newIsReady) {
 }
 
 void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
-                                        const std::vector<BSONObj>& multikeyMetadataKeys,
                                         const MultikeyPaths& multikeyPaths) {
-    // An index can either track path-level multikey information in the catalog or as metadata keys
-    // in the index itself, but not both.
-    invariant(!(_indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.size() > 0));
-    // If the index is already set as multikey and we don't have any path-level information to
-    // update, then there's nothing more for us to do.
-    bool hasNoPathLevelInfo = (!_indexTracksMultikeyPathsInCatalog && multikeyMetadataKeys.empty());
-    if (hasNoPathLevelInfo && _isMultikey.load()) {
+    if (!_indexTracksPathLevelMultikeyInfo && _isMultikeyForWrite.load()) {
+        // If the index is already set as multikey and we don't have any path-level information to
+        // update, then there's nothing more for us to do.
         return;
     }
 
-    if (_indexTracksMultikeyPathsInCatalog) {
+    if (_indexTracksPathLevelMultikeyInfo) {
         stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
         invariant(multikeyPaths.size() == _indexMultikeyPaths.size());
 
@@ -235,7 +213,7 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
         }
     }
 
-    MultikeyPaths paths = _indexTracksMultikeyPathsInCatalog ? multikeyPaths : MultikeyPaths{};
+    MultikeyPaths paths = _indexTracksPathLevelMultikeyInfo ? multikeyPaths : MultikeyPaths{};
 
     // On a primary, we can simply assign this write the same timestamp as the index creation,
     // insert, or update that caused this index to become multikey. This is because if two
@@ -258,48 +236,27 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
     // it multikey, that write will be marked as "isTrackingMultikeyPathInfo" on the applier's
     // OperationContext and we can safely defer that write to the end of the batch.
     if (MultikeyPathTracker::get(opCtx).isTrackingMultikeyPathInfo()) {
-        MultikeyPathTracker::get(opCtx).addMultikeyPathInfo(
-            {_ns, _descriptor->indexName(), multikeyMetadataKeys, std::move(paths)});
+        MultikeyPathInfo info;
+        info.nss = ns();
+        info.indexName = _descriptor->indexName();
+        info.multikeyPaths = paths;
+        MultikeyPathTracker::get(opCtx).addMultikeyPathInfo(info);
         return;
     }
 
-    // If multikeyMetadataKeys is non-empty, we must insert these keys into the index itself. We do
-    // not have to account for potential dupes, since all metadata keys are indexed against a single
-    // RecordId. An attempt to write a duplicate key will therefore be ignored.
-    if (!multikeyMetadataKeys.empty()) {
-        static const auto kMultikeyMetadataKeyId =
-            RecordId{RecordId::ReservedId::kWildcardMultikeyMetadataId};
-        uassertStatusOK(accessMethod()->insertKeys(
-            opCtx, multikeyMetadataKeys, kMultikeyMetadataKeyId, {}, {}, nullptr));
+    if (opCtx->inMultiDocumentTransaction()) {
+        auto status = _setMultikeyInMultiDocumentTransaction(opCtx, paths);
+        // Retry without side transaction.
+        if (!status.isOK()) {
+            _catalogSetMultikey(opCtx, paths);
+        }
+    } else {
+        _catalogSetMultikey(opCtx, paths);
     }
+}
 
-    // It's possible that the index type (e.g. ascending/descending index) supports tracking
-    // path-level multikey information, but this particular index doesn't.
-    // CollectionCatalogEntry::setIndexIsMultikey() requires that we discard the path-level
-    // multikey information in order to avoid unintentionally setting path-level multikey
-    // information on an index created before 3.4.
-    bool indexMetadataHasChanged;
-
-    // The commit handler for a transaction that sets the multikey flag. When the recovery unit
-    // commits, update the multikey paths if needed and clear the plan cache if the index metadata
-    // has changed.
-    auto onMultikeyCommitFn = [this, multikeyPaths](bool indexMetadataHasChanged) {
-        _isMultikey.store(true);
-
-        if (_indexTracksMultikeyPathsInCatalog) {
-            stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
-            for (size_t i = 0; i < multikeyPaths.size(); ++i) {
-                _indexMultikeyPaths[i].insert(multikeyPaths[i].begin(), multikeyPaths[i].end());
-            }
-        }
-
-        if (indexMetadataHasChanged && _infoCache) {
-            LOG(1) << _ns << ": clearing plan cache - index " << _descriptor->keyPattern()
-                   << " set to multi key.";
-            _infoCache->clearQueryCache();
-        }
-    };
-
+Status IndexCatalogEntryImpl::_setMultikeyInMultiDocumentTransaction(
+    OperationContext* opCtx, const MultikeyPaths& multikeyPaths) {
     // If we are inside a multi-document transaction, we write the on-disk multikey update in a
     // separate transaction so that it will not generate prepare conflicts with other operations
     // that try to set the multikey flag. In general, it should always be safe to update the
@@ -307,93 +264,127 @@ void IndexCatalogEntryImpl::setMultikey(OperationContext* opCtx,
     // multikey flag write and the parent transaction. We can do this write separately and commit it
     // before the parent transaction commits.
     auto txnParticipant = TransactionParticipant::get(opCtx);
-    if (opCtx->inMultiDocumentTransaction()) {
-        TransactionParticipant::SideTransactionBlock sideTxn(opCtx);
-        writeConflictRetry(opCtx, "set index multikey", _ns.ns(), [&] {
+
+    TransactionParticipant::SideTransactionBlock sideTxn(opCtx);
+
+    // If the index is not visible within the side transaction, the index may have been created,
+    // but not committed, in the parent transaction. Therefore, we abandon the side transaction
+    // and set the multikey flag in the parent transaction.
+    if (!_catalogIsPresent(opCtx)) {
+        return {ErrorCodes::SnapshotUnavailable, "index not visible in side transaction"};
+    }
+
+    writeConflictRetry(
+        opCtx, "set index multikey", ns().ns(), [&] {
             WriteUnitOfWork wuow(opCtx);
 
-            // If we have a prepare optime for recovery, then we always use that. During recovery of
-            // prepared transactions, the logical clock may not yet be initialized, so we use the
-            // prepare timestamp of the transaction for this write. This is safe since the prepare
-            // timestamp is always <= the commit timestamp of a transaction, which satisfies the
-            // correctness requirement for multikey writes i.e. they must occur at or before the
-            // first write that set the multikey flag.
+            // If we have a prepare optime for recovery, then we always use that. This is safe since
+            // the prepare timestamp is always <= the commit timestamp of a transaction, which
+            // satisfies the correctness requirement for multikey writes i.e. they must occur at or
+            // before the first write that set the multikey flag. This only occurs when
+            // reconstructing prepared transactions, and not during replication recovery oplog
+            // application.
             auto recoveryPrepareOpTime = txnParticipant.getPrepareOpTimeForRecovery();
-            // We might replay a prepared transaction behind the oldest timestamp during initial
-            // sync or behind the stable timestamp during rollback. During initial sync, we
-            // may not have a stable timestamp. Therefore, we need to round up
-            // the multi-key write timestamp to the max of the three so that we don't write
-            // behind the oldest/stable timestamp. This code path is only hit during initial
-            // sync/recovery when reconstructing prepared transactions and so we don't expect
-            // the oldest/stable timestamp to advance concurrently.
-            Timestamp writeTs = recoveryPrepareOpTime.isNull()
-                ? LogicalClock::get(opCtx)->getClusterTime().asTimestamp()
-                : std::max({recoveryPrepareOpTime.getTimestamp(),
-                            opCtx->getServiceContext()->getStorageEngine()->getOldestTimestamp(),
-                            opCtx->getServiceContext()->getStorageEngine()->getStableTimestamp()});
+            if (!recoveryPrepareOpTime.isNull()) {
+                // We might replay a prepared transaction behind the oldest timestamp during initial
+                // sync. So round up to the oldest timestamp for the multikey write if the prepare
+                // timestamp is behind the oldest timestamp.
+                auto status = opCtx->recoveryUnit()->setTimestamp(
+                    std::max(opCtx->getServiceContext()->getStorageEngine()->getOldestTimestamp(),
+                             recoveryPrepareOpTime.getTimestamp()));
+                if (status.code() == ErrorCodes::BadValue) {
+                    LOGV2(20352,
+                          "Temporarily could not timestamp the multikey catalog write, retrying",
+                          "reason"_attr = status.reason());
+                    throw WriteConflictException();
+                }
+                fassert(31164, status);
+            } else {
+                // If there is no recovery prepare OpTime, then this node must be a primary. We
+                // write a noop oplog entry to get a properly ordered timestamp.
+                invariant(opCtx->writesAreReplicated());
 
-            auto status = opCtx->recoveryUnit()->setTimestamp(writeTs);
-            if (status.code() == ErrorCodes::BadValue) {
-                log() << "Temporarily could not timestamp the multikey catalog write, retrying. "
-                      << status.reason();
-                throw WriteConflictException();
+                auto msg = BSON("msg"
+                                << "Setting index to multikey"
+                                << "coll" << ns().ns() << "index" << _descriptor->indexName());
+                opCtx->getClient()->getServiceContext()->getOpObserver()->onOpMessage(opCtx, msg);
             }
-            fassert(31164, status);
-            indexMetadataHasChanged = DurableCatalog::get(opCtx)->setIndexIsMultikey(
-                opCtx, _ns, _descriptor->indexName(), paths);
-            opCtx->recoveryUnit()->onCommit(
-                [onMultikeyCommitFn, indexMetadataHasChanged](boost::optional<Timestamp>) {
-                    onMultikeyCommitFn(indexMetadataHasChanged);
-                });
+
+            _catalogSetMultikey(opCtx, multikeyPaths);
+
             wuow.commit();
         });
-    } else {
-        indexMetadataHasChanged = DurableCatalog::get(opCtx)->setIndexIsMultikey(
-            opCtx, _ns, _descriptor->indexName(), paths);
-    }
 
-    opCtx->recoveryUnit()->onCommit(
-        [onMultikeyCommitFn, indexMetadataHasChanged](boost::optional<Timestamp>) {
-            onMultikeyCommitFn(indexMetadataHasChanged);
-        });
-
-    // Within a multi-document transaction, reads should be able to see the effect of previous
-    // writes done within that transaction. If a previous write in a transaction has set the index
-    // to be multikey, then a subsequent read MUST know that fact in order to return correct
-    // results. This is true in general for multikey writes. Since we don't update the in-memory
-    // multikey flag until after the transaction commits, we track extra information here to let
-    // subsequent readers within the same transaction know if this index was set as multikey by a
-    // previous write in the transaction.
-    if (opCtx->inMultiDocumentTransaction()) {
-        invariant(txnParticipant);
-        txnParticipant.addUncommittedMultikeyPathInfo(MultikeyPathInfo{
-            _ns, _descriptor->indexName(), multikeyMetadataKeys, std::move(paths)});
-    }
-}
-
-void IndexCatalogEntryImpl::setNs(NamespaceString ns) {
-    _ns = ns;
-    _descriptor->setNs(std::move(ns));
+    return Status::OK();
 }
 
 // ----
 
 bool IndexCatalogEntryImpl::_catalogIsReady(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->isIndexReady(opCtx, _ns, _descriptor->indexName());
+    return DurableCatalog::get(opCtx)->isIndexReady(
+        opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName());
 }
 
 bool IndexCatalogEntryImpl::_catalogIsPresent(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->isIndexPresent(opCtx, _ns, _descriptor->indexName());
+    return DurableCatalog::get(opCtx)->isIndexPresent(
+        opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName());
 }
 
 bool IndexCatalogEntryImpl::_catalogIsMultikey(OperationContext* opCtx,
                                                MultikeyPaths* multikeyPaths) const {
-    return DurableCatalog::get(opCtx)->isIndexMultikey(
-        opCtx, _ns, _descriptor->indexName(), multikeyPaths);
+    return DurableCatalog::get(opCtx)->isIndexMultikey(opCtx,
+                                                       _descriptor->getCollection()->getCatalogId(),
+                                                       _descriptor->indexName(),
+                                                       multikeyPaths);
+}
+
+void IndexCatalogEntryImpl::_catalogSetMultikey(OperationContext* opCtx,
+                                                const MultikeyPaths& multikeyPaths) {
+    // It's possible that the index type (e.g. ascending/descending index) supports tracking
+    // path-level multikey information, but this particular index doesn't.
+    // CollectionCatalogEntry::setIndexIsMultikey() requires that we discard the path-level
+    // multikey information in order to avoid unintentionally setting path-level multikey
+    // information on an index created before 3.4.
+    auto indexMetadataHasChanged =
+        DurableCatalog::get(opCtx)->setIndexIsMultikey(opCtx,
+                                                       _descriptor->getCollection()->getCatalogId(),
+                                                       _descriptor->indexName(),
+                                                       multikeyPaths);
+
+    // In the absense of using the storage engine to read from the catalog, we must set multikey
+    // prior to the storage engine transaction committing.
+    //
+    // Moreover, there must not be an `onRollback` handler to reset this back to false. Given a long
+    // enough pause in processing `onRollback` handlers, a later writer that successfully flipped
+    // multikey can be undone. Alternatively, one could use a counter instead of a boolean to avoid
+    // that problem.
+    _isMultikeyForRead.store(true);
+    if (_indexTracksPathLevelMultikeyInfo) {
+        stdx::lock_guard<Latch> lk(_indexMultikeyPathsMutex);
+        for (size_t i = 0; i < multikeyPaths.size(); ++i) {
+            _indexMultikeyPaths[i].insert(multikeyPaths[i].begin(), multikeyPaths[i].end());
+        }
+    }
+    if (indexMetadataHasChanged && _queryInfo) {
+        LOGV2_DEBUG(4718705,
+                    1,
+                    "Index set to multi key, clearing query plan cache",
+                    "namespace"_attr = ns(),
+                    "keyPattern"_attr = _descriptor->keyPattern());
+        _queryInfo->clearQueryCache();
+    }
+
+    opCtx->recoveryUnit()->onCommit([this](boost::optional<Timestamp>) {
+        // Writers must attempt to flip multikey until it's confirmed a storage engine
+        // transaction successfully commits. Only after this point may a writer optimize out
+        // flipping multikey.
+        _isMultikeyForWrite.store(true);
+    });
 }
 
 KVPrefix IndexCatalogEntryImpl::_catalogGetPrefix(OperationContext* opCtx) const {
-    return DurableCatalog::get(opCtx)->getIndexPrefix(opCtx, _ns, _descriptor->indexName());
+    return DurableCatalog::get(opCtx)->getIndexPrefix(
+        opCtx, _descriptor->getCollection()->getCatalogId(), _descriptor->indexName());
 }
 
 }  // namespace mongo
