@@ -46,33 +46,34 @@
 namespace mongo {
 
 class CollatorInterface;
-class CollectionInfoCache;
+class CollectionQueryInfo;
 class IndexAccessMethod;
 class IndexDescriptor;
 class MatchExpression;
 class OperationContext;
+class ExpressionContext;
 
 class IndexCatalogEntryImpl : public IndexCatalogEntry {
     IndexCatalogEntryImpl(const IndexCatalogEntryImpl&) = delete;
     IndexCatalogEntryImpl& operator=(const IndexCatalogEntryImpl&) = delete;
 
 public:
-    explicit IndexCatalogEntryImpl(
-        OperationContext* opCtx,
-        const NamespaceString& nss,
-        std::unique_ptr<IndexDescriptor> descriptor,  // ownership passes to me
-        CollectionInfoCache* infoCache);              // not owned, optional
+    IndexCatalogEntryImpl(OperationContext* opCtx,
+                          const std::string& ident,
+                          std::unique_ptr<IndexDescriptor> descriptor,  // ownership passes to me
+                          CollectionQueryInfo* queryInfo,               // not owned, optional
+                          bool isFrozen);
 
     ~IndexCatalogEntryImpl() final;
 
-    const NamespaceString& ns() const final {
-        return _ns;
-    }
-
-    void setNs(NamespaceString ns) final;
+    const NamespaceString& ns() const final;
 
     void init(std::unique_ptr<IndexAccessMethod> accessMethod) final;
 
+    const std::string& getIdent() const final {
+        return _ident;
+    }
+
     IndexDescriptor* descriptor() final {
         return _descriptor.get();
     }
@@ -119,12 +120,20 @@ public:
 
     void setIsReady(bool newIsReady) final;
 
+    void setDropped() final {
+        _isDropped.store(true);
+    }
+
+    bool isDropped() const final {
+        return _isDropped.load();
+    }
+
     // --
 
     /**
      * Returns true if this index is multikey, and returns false otherwise.
      */
-    bool isMultikey(OperationContext* opCtx) const final;
+    bool isMultikey() const final;
 
     /**
      * Returns the path components that cause this index to be multikey if this index supports
@@ -152,13 +161,13 @@ public:
      * namespace, index name, and multikey paths on the OperationContext rather than set the index
      * as multikey here.
      */
-    void setMultikey(OperationContext* opCtx,
-                     const std::vector<BSONObj>& multikeyMetadataKeys,
-                     const MultikeyPaths& multikeyPaths) final;
+    void setMultikey(OperationContext* opCtx, const MultikeyPaths& multikeyPaths) final;
 
     // if this ready is ready for queries
     bool isReady(OperationContext* opCtx) const final;
 
+    bool isFrozen() const final;
+
     KVPrefix getPrefix() const final {
         return _prefix;
     }
@@ -178,7 +187,12 @@ public:
     void setMinimumVisibleSnapshot(Timestamp newMinimumVisibleSnapshot) final;
 
 private:
-    class SetMultikeyChange;
+    /**
+     * Sets this index to be multikey when we are running inside a multi-document transaction.
+     * Used by setMultikey() only.
+     */
+    Status _setMultikeyInMultiDocumentTransaction(OperationContext* opCtx,
+                                                  const MultikeyPaths& multikeyPaths);
 
     bool _catalogIsReady(OperationContext* opCtx) const;
     bool _catalogIsPresent(OperationContext* opCtx) const;
@@ -190,15 +204,20 @@ private:
      */
     bool _catalogIsMultikey(OperationContext* opCtx, MultikeyPaths* multikeyPaths) const;
 
+    /**
+     * Sets on-disk multikey flag for this index.
+     */
+    void _catalogSetMultikey(OperationContext* opCtx, const MultikeyPaths& multikeyPaths);
+
     KVPrefix _catalogGetPrefix(OperationContext* opCtx) const;
 
     // -----
 
-    NamespaceString _ns;
+    const std::string _ident;
 
     std::unique_ptr<IndexDescriptor> _descriptor;  // owned here
 
-    CollectionInfoCache* _infoCache;  // not owned here
+    CollectionQueryInfo* _queryInfo;  // not owned here
 
     std::unique_ptr<IndexAccessMethod> _accessMethod;
 
@@ -206,19 +225,27 @@ private:
 
     std::unique_ptr<CollatorInterface> _collator;
     std::unique_ptr<MatchExpression> _filterExpression;
+    // Special ExpressionContext used to evaluate the partial filter expression.
+    boost::intrusive_ptr<ExpressionContext> _expCtxForFilter;
 
     // cached stuff
 
     Ordering _ordering;  // TODO: this might be b-tree specific
     bool _isReady;       // cache of NamespaceDetails info
+    bool _isFrozen;
+    AtomicWord<bool> _isDropped;  // Whether the index drop is committed.
 
-    // Set to true if this index can track path-level multikey information in the catalog. This
-    // member is effectively const after IndexCatalogEntry::init() is called.
-    bool _indexTracksMultikeyPathsInCatalog = false;
+    // Set to true if this index supports path-level multikey tracking.
+    // '_indexTracksPathLevelMultikeyInfo' is effectively const after IndexCatalogEntry::init() is
+    // called.
+    bool _indexTracksPathLevelMultikeyInfo = false;
 
-    // Set to true if this index is multikey. '_isMultikey' serves as a cache of the information
-    // stored in the NamespaceDetails or DurableCatalog.
-    AtomicWord<bool> _isMultikey;
+    // Set to true if this index may contain multikey data.
+    AtomicWord<bool> _isMultikeyForRead;
+
+    // Set to true after a transaction commit successfully updates multikey on the catalog data. At
+    // this point, future writers do not need to update the catalog.
+    AtomicWord<bool> _isMultikeyForWrite;
 
     // Controls concurrent access to '_indexMultikeyPaths'. We acquire this mutex rather than the
     // RESOURCE_METADATA lock as a performance optimization so that it is cheaper to detect whether
