@@ -57,13 +57,10 @@ struct InsertDeleteOptions;
 class IndexCatalogImpl : public IndexCatalog {
 public:
     explicit IndexCatalogImpl(Collection* collection);
-    ~IndexCatalogImpl() override;
 
     // must be called before used
     Status init(OperationContext* opCtx) override;
 
-    bool ok() const override;
-
     // ---- accessors -----
 
     bool haveAnyIndexes() const override;
@@ -168,18 +165,16 @@ public:
 
     std::vector<std::shared_ptr<const IndexCatalogEntry>> getAllReadyEntriesShared() const override;
 
-    /**
-     * Returns a not-ok Status if there are any unfinished index builds. No new indexes should
-     * be built when in this state.
-     */
-    Status checkUnfinished() const override;
-
     using IndexIterator = IndexCatalog::IndexIterator;
     std::unique_ptr<IndexIterator> getIndexIterator(
         OperationContext* const opCtx, const bool includeUnfinishedIndexes) const override;
 
     // ---- index set modifiers ------
 
+    IndexCatalogEntry* createIndexEntry(OperationContext* opCtx,
+                                        std::unique_ptr<IndexDescriptor> descriptor,
+                                        CreateIndexEntryFlags flags) override;
+
     /**
      * Call this only on an empty collection from inside a WriteUnitOfWork. Index creation on an
      * empty collection can be rolled back as part of a larger WUOW. Returns the full specification
@@ -206,24 +201,19 @@ public:
      */
     void dropAllIndexes(OperationContext* opCtx,
                         bool includingIdIndex,
-                        stdx::function<void(const IndexDescriptor*)> onDropFn) override;
+                        std::function<void(const IndexDescriptor*)> onDropFn) override;
     void dropAllIndexes(OperationContext* opCtx, bool includingIdIndex) override;
 
-    /**
-     * Drops the index.
-     *
-     * The caller must hold the collection X lock and ensure no index builds are in progress on the
-     * collection.
-     */
+
     Status dropIndex(OperationContext* opCtx, const IndexDescriptor* desc) override;
+    Status dropUnfinishedIndex(OperationContext* const opCtx,
+                               const IndexDescriptor* const desc) override;
 
-    /**
-     * will drop all incompleted indexes and return specs
-     * after this, the indexes can be rebuilt
-     */
-    std::vector<BSONObj> getAndClearUnfinishedIndexes(OperationContext* opCtx) override;
+    Status dropIndexEntry(OperationContext* opCtx, IndexCatalogEntry* entry) override;
 
 
+    void deleteIndexFromDisk(OperationContext* opCtx, const std::string& indexName) override;
+
     struct IndexKillCriteria {
         std::string ns;
         std::string name;
@@ -235,7 +225,7 @@ public:
     /**
      * Returns true if the index 'idx' is multikey, and returns false otherwise.
      */
-    bool isMultikey(OperationContext* opCtx, const IndexDescriptor* idx) override;
+    bool isMultikey(const IndexDescriptor* const idx) override;
 
     /**
      * Returns the path components that cause the index 'idx' to be multikey if the index supports
@@ -250,72 +240,8 @@ public:
 
     void setMultikeyPaths(OperationContext* const opCtx,
                           const IndexDescriptor* desc,
-                          const std::vector<BSONObj>& multikeyMetadataKeys,
                           const MultikeyPaths& multikeyPaths) override;
 
-    // --- these probably become private?
-
-    class IndexBuildBlock : public IndexCatalog::IndexBuildBlockInterface {
-        IndexBuildBlock(const IndexBuildBlock&) = delete;
-        IndexBuildBlock& operator=(const IndexBuildBlock&) = delete;
-
-    public:
-        IndexBuildBlock(IndexCatalogImpl* catalog,
-                        const NamespaceString& nss,
-                        const BSONObj& spec,
-                        IndexBuildMethod method);
-
-        ~IndexBuildBlock();
-
-        /**
-         * Being called in a 'WriteUnitOfWork' has no effect.
-         */
-        void deleteTemporaryTables(OperationContext* opCtx);
-
-        /**
-         * Must be called from within a `WriteUnitOfWork`
-         */
-        Status init(OperationContext* opCtx, Collection* collection);
-
-        /**
-         * Must be called from within a `WriteUnitOfWork`
-         */
-        void success(OperationContext* opCtx, Collection* collection);
-
-        /**
-         * index build failed, clean up meta data
-         *
-         * Must be called from within a `WriteUnitOfWork`
-         */
-        void fail(OperationContext* opCtx, const Collection* collection);
-
-        IndexCatalogEntry* getEntry() {
-            return _entry;
-        }
-
-        const std::string& getIndexName() const {
-            return _indexName;
-        }
-
-        const BSONObj& getSpec() const {
-            return _spec;
-        }
-
-    private:
-        IndexCatalogImpl* const _catalog;
-        const NamespaceString _nss;
-
-        BSONObj _spec;
-        IndexBuildMethod _method;
-
-        std::string _indexName;
-        std::string _indexNamespace;
-
-        IndexCatalogEntry* _entry;
-
-        std::unique_ptr<IndexBuildInterceptor> _indexBuildInterceptor;
-    };
-
     // ----- data modifiers ------
 
     /**
@@ -353,9 +279,6 @@ public:
         return _getAccessMethodName(keyPattern);
     }
 
-    std::unique_ptr<IndexCatalog::IndexBuildBlockInterface> createIndexBuildBlock(
-        OperationContext* opCtx, const BSONObj& spec, IndexBuildMethod method) override;
-
     std::string::size_type getLongestIndexNameLength(OperationContext* opCtx) const override;
 
     // public static helpers
@@ -370,8 +293,6 @@ public:
                                     const IndexDescriptor* desc,
                                     InsertDeleteOptions* options) const override;
 
-    void setNs(NamespaceString ns) override;
-
     void indexBuildSuccess(OperationContext* opCtx, IndexCatalogEntry* index) override;
 
 private:
@@ -385,12 +306,10 @@ private:
      */
     std::string _getAccessMethodName(const BSONObj& keyPattern) const;
 
-    void _checkMagic() const;
-
     Status _indexKeys(OperationContext* opCtx,
                       IndexCatalogEntry* index,
-                      const std::vector<BSONObj>& keys,
-                      const BSONObjSet& multikeyMetadataKeys,
+                      const std::vector<KeyString::Value>& keys,
+                      const KeyStringSet& multikeyMetadataKeys,
                       const MultikeyPaths& multikeyPaths,
                       const BSONObj& obj,
                       RecordId loc,
@@ -417,7 +336,7 @@ private:
 
     void _unindexKeys(OperationContext* opCtx,
                       IndexCatalogEntry* index,
-                      const std::vector<BSONObj>& keys,
+                      const std::vector<KeyString::Value>& keys,
                       const BSONObj& obj,
                       RecordId loc,
                       bool logIfError,
@@ -430,27 +349,6 @@ private:
                         bool logIfError,
                         int64_t* keysDeletedOut);
 
-    /**
-     * this does no sanity checks
-     */
-    Status _dropIndex(OperationContext* opCtx, IndexCatalogEntry* entry);
-
-    // just does disk hanges
-    // doesn't change memory state, etc...
-    void _deleteIndexFromDisk(OperationContext* opCtx,
-                              const std::string& indexName,
-                              const std::string& indexNamespace);
-
-    // descriptor ownership passes to _setupInMemoryStructures
-    // initFromDisk: Avoids registering a change to undo this operation when set to true.
-    //               You must set this flag if calling this function outside of a UnitOfWork.
-    // isReadyIndex: The index will be directly available for query usage without needing to
-    //               complete the IndexBuildBlock process.
-    IndexCatalogEntry* _setupInMemoryStructures(OperationContext* opCtx,
-                                                std::unique_ptr<IndexDescriptor> descriptor,
-                                                bool initFromDisk,
-                                                bool isReadyIndex);
-
     /**
      * Applies a set of transformations to the user-provided index object 'spec' to make it
      * conform to the standard for insertion.  Removes the '_id' field if it exists, applies
@@ -495,16 +393,9 @@ private:
                            const std::vector<std::string>& indexNamesToDrop,
                            bool haveIdIndex);
 
-    int _magic;
     Collection* const _collection;
 
     IndexCatalogEntryContainer _readyIndexes;
     IndexCatalogEntryContainer _buildingIndexes;
-
-    // These are the index specs of indexes that were "leftover".
-    // "Leftover" means they were unfinished when a mongod shut down.
-    // Certain operations are prohibited until someone fixes.
-    // Retrieve by calling getAndClearUnfinishedIndexes().
-    std::vector<BSONObj> _unfinishedIndexes;
 };
 }  // namespace mongo
