@@ -29,6 +29,7 @@
 
 #pragma once
 
+#include <functional>
 #include <iosfwd>
 #include <memory>
 #include <set>
@@ -41,22 +42,22 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/db/background.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog/index_build_block.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/index/index_access_method.h"
+#include "mongo/db/index/index_build_interceptor.h"
 #include "mongo/db/record_id.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/stdx/functional.h"
-#include "mongo/util/fail_point_service.h"
+#include "mongo/util/fail_point.h"
 
 namespace mongo {
 
-MONGO_FAIL_POINT_DECLARE(leaveIndexBuildUnfinishedForShutdown);
+extern FailPoint leaveIndexBuildUnfinishedForShutdown;
 
 class Collection;
 class MatchExpression;
 class NamespaceString;
 class OperationContext;
-class ProgressMeterHolder;
 
 /**
  * Builds one or more indexes.
@@ -77,17 +78,6 @@ public:
     MultiIndexBlock() = default;
     ~MultiIndexBlock();
 
-    /**
-     * Ensures the index build state is cleared correctly after index build success or failure.
-     *
-     * Must be called before object destruction if init() has been called; and safe to call if
-     * init() has not been called.
-     *
-     * By only requiring this call after init(), we allow owners of the object to exit without
-     * further handling if they never use the object.
-     */
-    void cleanUpAfterBuild(OperationContext* opCtx, Collection* collection);
-
     static bool areHybridIndexBuildsEnabled();
 
     /**
@@ -100,6 +90,14 @@ public:
      */
     void ignoreUniqueConstraint();
 
+    /**
+     * Sets an index build UUID associated with the indexes for this builder. This call is required
+     * for two-phase index builds.
+     */
+    void setTwoPhaseBuildUUID(UUID indexBuildUUID) {
+        _buildUUID = indexBuildUUID;
+    }
+
     /**
      * Prepares the index(es) for building and returns the canonicalized form of the requested index
      * specifications.
@@ -110,9 +108,9 @@ public:
      *
      * Does not need to be called inside of a WriteUnitOfWork (but can be due to nesting).
      *
-     * Requires holding an exclusive database lock.
+     * Requires holding an exclusive lock on the collection.
      */
-    using OnInitFn = stdx::function<Status(std::vector<BSONObj>& specs)>;
+    using OnInitFn = std::function<Status(std::vector<BSONObj>& specs)>;
     StatusWith<std::vector<BSONObj>> init(OperationContext* opCtx,
                                           Collection* collection,
                                           const std::vector<BSONObj>& specs,
@@ -174,11 +172,6 @@ public:
     Status dumpInsertsFromBulk(OperationContext* opCtx);
     Status dumpInsertsFromBulk(OperationContext* opCtx, std::set<RecordId>* const dupRecords);
 
-    /**
-     * Check whether all the side writes have been applied from all the side write tables.
-     */
-    bool areAllWritesApplied(OperationContext* opCtx);
-
     /**
      * For background indexes using an IndexBuildInterceptor to capture inserts during a build,
      * drain these writes into the index. If intent locks are held on the collection, more writes
@@ -191,9 +184,22 @@ public:
      *
      * Must not be in a WriteUnitOfWork.
      */
-    Status drainBackgroundWrites(
-        OperationContext* opCtx,
-        RecoveryUnit::ReadSource readSource = RecoveryUnit::ReadSource::kUnset);
+    Status drainBackgroundWrites(OperationContext* opCtx,
+                                 RecoveryUnit::ReadSource readSource,
+                                 IndexBuildInterceptor::DrainYieldPolicy drainYieldPolicy);
+
+
+    /**
+     * Retries key generation and insertion for all records skipped during the collection scanning
+     * phase.
+     *
+     * Index builds ignore key generation errors on secondaries. In steady-state replication, all
+     * writes from the primary are eventually applied, so an index build should always succeed when
+     * the primary commits. In two-phase index builds, a secondary may become primary in the middle
+     * of an index build, so it must ensure that before it finishes, it has indexed all documents in
+     * a collection, requiring a call to this function upon completion.
+     */
+    Status retrySkippedRecords(OperationContext* opCtx, Collection* collection);
 
     /**
      * Check any constraits that may have been temporarily violated during the index build for
@@ -214,10 +220,10 @@ public:
      * `onCreateEach` will be called after each index has been marked as "ready".
      * `onCommit` will be called after all indexes have been marked "ready".
      *
-     * Requires holding an exclusive database lock.
+     * Requires holding an exclusive lock on the collection.
      */
-    using OnCommitFn = stdx::function<void()>;
-    using OnCreateEachFn = stdx::function<void(const BSONObj& spec)>;
+    using OnCommitFn = std::function<void()>;
+    using OnCreateEachFn = std::function<void(const BSONObj& spec)>;
     Status commit(OperationContext* opCtx,
                   Collection* collection,
                   OnCreateEachFn onCreateEach,
@@ -231,45 +237,34 @@ public:
     static OnCommitFn kNoopOnCommitFn;
 
     /**
-     * Returns true if this index builder was added to the index catalog successfully.
-     * In addition to having commit() return without errors, the enclosing WUOW has to be committed
-     * for the indexes to show up in the index catalog.
+     * Ensures the index build state is cleared correctly after index build failure.
+     *
+     * Must be called before object destruction if init() has been called; and safe to call if
+     * init() has not been called.
+     *
+     * By only requiring this call after init(), we allow owners of the object to exit without
+     * further handling if they never use the object.
+     *
+     * `onCleanUp` will be called after all indexes have been removed from the catalog.
      */
-    bool isCommitted() const;
+    using OnCleanUpFn = std::function<void()>;
+    void abortIndexBuild(OperationContext* opCtx,
+                         Collection* collection,
+                         OnCleanUpFn onCleanUp) noexcept;
 
     /**
-     * Signals the index build to abort.
-     *
-     * In-progress inserts and commits will still run to completion. However, subsequent index build
-     * operations will fail an IndexBuildAborted error.
-     *
-     * Aborts the uncommitted index build and prevents further inserts or commit attempts from
-     * proceeding. On destruction, all traces of uncommitted index builds will be removed.
-     *
-     * If the index build has already been aborted (using abort() or abortWithoutCleanup()),
-     * this function does nothing.
-     *
-     * If this index build has been committed successfully, this function has no effect.
-     *
-     * May be called from any thread.
+     * Not all index aborts need this function, in particular index builds that do not need
+     * to timestamp catalog writes. This is a no-op.
      */
-    void abort(StringData reason);
+    static OnCleanUpFn kNoopOnCleanUpFn;
 
     /**
      * May be called at any time after construction but before a successful commit(). Suppresses
-     * the default behavior on destruction of removing all traces of uncommitted index builds.
+     * the default behavior on destruction of removing all traces of uncommitted index builds. Does
+     * not perform any storage engine writes. May delete internal tables, but this is not
+     * transactional.
      *
-     * The most common use of this is if the indexes were already dropped via some other
-     * mechanism such as the whole collection being dropped. In that case, it would be invalid
-     * to try to remove the indexes again. Also, replication uses this to ensure that indexes
-     * that are being built on shutdown are resumed on startup.
-     *
-     * Do not use this unless you are really sure you need to.
-     *
-     * Does not matter whether it is called inside of a WriteUnitOfWork. Will not be rolled
-     * back.
-     *
-     * Must be called from owning thread.
+     * This should only be used during shutdown or rollback.
      */
     void abortWithoutCleanup(OperationContext* opCtx);
 
@@ -281,63 +276,17 @@ public:
 
     void setIndexBuildMethod(IndexBuildMethod indexBuildMethod);
 
-    /**
-     * State transitions:
-     *
-     * Uninitialized --> Running --> Committed
-     *       |              |           ^
-     *       |              |           |
-     *       \--------------+------> Aborted
-     *
-     * It is possible for abort() to skip intermediate states. For example, calling abort() when the
-     * index build has not been initialized will transition from Uninitialized directly to Aborted.
-     *
-     * In the case where we are in the midst of committing the WUOW for a successful commit() call,
-     * we may transition temporarily to Aborted before finally ending at Committed. See comments for
-     * MultiIndexBlock::abort().
-     *
-     * For testing only. Callers should not have to query the state of the MultiIndexBlock directly.
-     */
-    enum class State { kUninitialized, kRunning, kCommitted, kAborted };
-    State getState_forTest() const;
-
 private:
     struct IndexToBuild {
-        std::unique_ptr<IndexCatalog::IndexBuildBlockInterface> block;
+        std::unique_ptr<IndexBuildBlock> block;
 
-        IndexAccessMethod* real = NULL;           // owned elsewhere
+        IndexAccessMethod* real = nullptr;        // owned elsewhere
         const MatchExpression* filterExpression;  // might be NULL, owned elsewhere
         std::unique_ptr<IndexAccessMethod::BulkBuilder> bulk;
 
         InsertDeleteOptions options;
     };
 
-    Status _dumpInsertsFromBulk(std::set<RecordId>* dupRecords,
-                                std::vector<BSONObj>* dupKeysInserted);
-
-    /**
-     * Returns the current state.
-     */
-    State _getState() const;
-
-    /**
-     * Updates the current state to a non-Aborted state.
-     */
-    void _setState(State newState);
-
-    /**
-     * Updates the current state to Aborted with the given reason.
-     */
-    void _setStateToAbortedIfNotCommitted(StringData reason);
-
-    /**
-     * Performs a collection scan on the given collection and inserts the relevant index keys into
-     * the external sorter.
-     */
-    Status _doCollectionScan(OperationContext* opCtx,
-                             Collection* collection,
-                             ProgressMeterHolder* progress);
-
     // Is set during init() and ensures subsequent function calls act on the same Collection.
     boost::optional<UUID> _collectionUUID;
 
@@ -349,25 +298,12 @@ private:
 
     bool _ignoreUnique = false;
 
-    bool _needToCleanup = true;
-
     // Set to true when no work remains to be done, the object can safely destruct without leaving
     // incorrect state set anywhere.
     bool _buildIsCleanedUp = true;
 
-    // Duplicate key constraints should be checked at least once in the MultiIndexBlock.
-    bool _constraintsChecked = false;
-
-    // Protects member variables of this class declared below.
-    mutable Mutex _mutex = MONGO_MAKE_LATCH("MultiIndexBlock::_mutex");
-
-    State _state = State::kUninitialized;
-    std::string _abortReason;
+    // A unique identifier associating this index build with a two-phase index build within a
+    // replica set.
+    boost::optional<UUID> _buildUUID;
 };
-
-// For unit tests that need to check MultiIndexBlock states.
-// The ASSERT_*() macros use this function to print the value of 'state' when the predicate fails.
-std::ostream& operator<<(std::ostream& os, const MultiIndexBlock::State& state);
-
-logger::LogstreamBuilder& operator<<(logger::LogstreamBuilder& out, const IndexBuildMethod& method);
 }  // namespace mongo
