@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -39,6 +39,7 @@
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/catalog/drop_collection.h"
 #include "mongo/db/catalog/index_catalog.h"
+#include "mongo/db/catalog/list_indexes.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
@@ -57,67 +58,23 @@
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/durable_catalog.h"
 #include "mongo/db/views/view_catalog.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 namespace {
 
+MONGO_FAIL_POINT_DEFINE(useRenameCollectionPathThroughConfigsvr);
 MONGO_FAIL_POINT_DEFINE(writeConflictInRenameCollCopyToTmp);
 
 boost::optional<NamespaceString> getNamespaceFromUUID(OperationContext* opCtx, const UUID& uuid) {
-    return CollectionCatalog::get(opCtx).lookupNSSByUUID(uuid);
+    return CollectionCatalog::get(opCtx).lookupNSSByUUID(opCtx, uuid);
 }
 
 bool isCollectionSharded(OperationContext* opCtx, const NamespaceString& nss) {
     auto* const css = CollectionShardingState::get(opCtx, nss);
-    const auto metadata = css->getCurrentMetadata();
-    return metadata->isSharded();
-}
-
-Status checkAllIndexesCanBeRecreatedInNewNS(OperationContext* opCtx,
-                                            Collection* coll,
-                                            const NamespaceString& newNss) {
-    const auto checkIndexNamespace =
-        serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-        serverGlobalParams.featureCompatibility.getVersion() !=
-            ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo42;
-    std::string::size_type longestIndexNameLength =
-        checkIndexNamespace ? coll->getIndexCatalog()->getLongestIndexNameLength(opCtx) : 0;
-    return newNss.checkLengthForRename(longestIndexNameLength);
-}
-
-void dropIndexesThatWillBeTooLongAfterDropPendingRename(OperationContext* opCtx, Collection* coll) {
-    // Compile a list of any indexes that would become too long following the
-    // drop-pending rename. In the case that this collection drop gets rolled back, this
-    // will incur a performance hit, since those indexes will have to be rebuilt from
-    // scratch, but data integrity is maintained.
-    std::vector<const IndexDescriptor*> indexesToDrop;
-    auto indexIter = coll->getIndexCatalog()->getIndexIterator(opCtx, true);
-
-    // Determine which index names are too long. Since we don't have the collection
-    // rename optime at this time, use the maximum optime to check the index names.
-    auto longDpns = coll->ns().makeDropPendingNamespace(repl::OpTime::max());
-    while (indexIter->more()) {
-        auto index = indexIter->next()->descriptor();
-        auto status = longDpns.checkLengthForRename(index->indexName().size());
-        if (!status.isOK()) {
-            indexesToDrop.push_back(index);
-        }
-    }
-
-    // Drop the offending indexes.
-    auto opObserver = opCtx->getServiceContext()->getOpObserver();
-    for (auto&& index : indexesToDrop) {
-        log() << "Collection " << coll->ns() << " contains an index namespace '"
-              << index->indexNamespace()
-              << "' that would be too long after drop-pending rename. Dropping index "
-                 "immediately.";
-        fassert(50941, coll->getIndexCatalog()->dropIndex(opCtx, index));
-        opObserver->onDropIndex(
-            opCtx, coll->ns(), coll->uuid(), index->indexName(), index->infoObj());
-    }
+    return css->getCollectionDescription().isSharded();
 }
 
 // From a replicated to an unreplicated collection or vice versa.
@@ -138,44 +95,41 @@ Status checkSourceAndTargetNamespaces(OperationContext* opCtx,
 
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     if (opCtx->writesAreReplicated() && !replCoord->canAcceptWritesFor(opCtx, source))
-        return Status(ErrorCodes::NotWritablePrimary,
+        return Status(ErrorCodes::NotMaster,
                       str::stream() << "Not primary while renaming collection " << source << " to "
                                     << target);
 
-    if (isCollectionSharded(opCtx, source))
-        return {ErrorCodes::IllegalOperation, "source namespace cannot be sharded"};
+    // TODO: SERVER-42638 Replace checks of cm() with cm()->distributionMode() == sharded
+    if (!MONGO_unlikely(useRenameCollectionPathThroughConfigsvr.shouldFail())) {
+        if (isCollectionSharded(opCtx, source))
+            return {ErrorCodes::IllegalOperation, "source namespace cannot be sharded"};
+    }
 
     if (isReplicatedChanged(opCtx, source, target))
         return {ErrorCodes::IllegalOperation,
                 "Cannot rename collections between a replicated and an unreplicated database"};
 
     auto db = DatabaseHolder::get(opCtx)->getDb(opCtx, source.db());
-    if (!db)
-        return Status(ErrorCodes::NamespaceNotFound, "source namespace does not exist");
+    if (!db || db->isDropPending(opCtx))
+        return Status(ErrorCodes::NamespaceNotFound,
+                      str::stream()
+                          << "Database " << source.db() << " does not exist or is drop pending");
 
-    {
-        auto& dss = DatabaseShardingState::get(db);
-        auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, &dss);
-        dss.checkDbVersion(opCtx, dssLock);
-    }
-
-    Collection* const sourceColl = db->getCollection(opCtx, source);
+    Collection* const sourceColl =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, source);
     if (!sourceColl) {
         if (ViewCatalog::get(db)->lookup(opCtx, source.ns()))
             return Status(ErrorCodes::CommandNotSupportedOnView,
                           str::stream() << "cannot rename view: " << source);
-        return Status(ErrorCodes::NamespaceNotFound, "source namespace does not exist");
+        return Status(ErrorCodes::NamespaceNotFound,
+                      str::stream() << "Source collection " << source.ns() << " does not exist");
     }
 
-    auto status = checkAllIndexesCanBeRecreatedInNewNS(opCtx, sourceColl, target);
-    if (!status.isOK())
-        return status;
-
     BackgroundOperation::assertNoBgOpInProgForNs(source.ns());
-    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(
-        sourceColl->uuid().get());
+    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(sourceColl->uuid());
 
-    Collection* targetColl = db->getCollection(opCtx, target);
+    Collection* targetColl =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, target);
 
     if (!targetColl) {
         if (ViewCatalog::get(db)->lookup(opCtx, target.ns()))
@@ -220,9 +174,16 @@ Status renameTargetCollectionToTmp(OperationContext* opCtx,
 
         wunit.commit();
 
-        log() << "Successfully renamed the target " << targetNs << " (" << targetUUID << ") to "
-              << tmpName << " so that the source " << sourceNs << " (" << sourceUUID
-              << ") could be renamed to " << targetNs;
+        LOGV2(20397,
+              "Successfully renamed the target {targetNs} ({targetUUID}) to {tmpName} so that the "
+              "source {sourceNs} ({sourceUUID}) could be renamed to {targetNs2}",
+              "Successfully renamed the target so that the source could be renamed",
+              "existingTargetNamespace"_attr = targetNs,
+              "existingTargetUUID"_attr = targetUUID,
+              "renamedExistingTarget"_attr = tmpName,
+              "sourceNamespace"_attr = sourceNs,
+              "sourceUUID"_attr = sourceUUID,
+              "newTargetNamespace"_attr = targetNs);
 
         return Status::OK();
     });
@@ -274,12 +235,6 @@ Status renameCollectionAndDropTarget(OperationContext* opCtx,
         // Target collection exists - drop it.
         invariant(options.dropTarget);
 
-        // Check for index builds on the target collection before dropping any indexes or the
-        // collection.
-        BackgroundOperation::assertNoBgOpInProgForNs(targetColl->ns().ns());
-        IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(
-            targetColl->uuid().get());
-
         // If this rename collection is replicated, check for long index names in the target
         // collection that may exceed the MMAPv1 namespace limit when the target collection
         // is renamed with a drop-pending namespace.
@@ -288,9 +243,12 @@ Status renameCollectionAndDropTarget(OperationContext* opCtx,
         if (!isOplogDisabledForNamespace) {
             invariant(opCtx->writesAreReplicated());
             invariant(renameOpTimeFromApplyOps.isNull());
-            dropIndexesThatWillBeTooLongAfterDropPendingRename(opCtx, targetColl);
         }
 
+        BackgroundOperation::assertNoBgOpInProgForNs(targetColl->ns().ns());
+        IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(
+            targetColl->uuid());
+
         auto numRecords = targetColl->numRecords(opCtx);
         auto opObserver = opCtx->getServiceContext()->getOpObserver();
         auto renameOpTime = opObserver->preRenameCollection(
@@ -300,10 +258,15 @@ Status renameCollectionAndDropTarget(OperationContext* opCtx,
             // 'renameOpTime' must be null because a valid 'renameOpTimeFromApplyOps' implies
             // replicated writes are not enabled.
             if (!renameOpTime.isNull()) {
-                severe() << "renameCollection: " << source << " to " << target
-                         << " (with dropTarget=true) - unexpected renameCollection oplog entry"
-                         << " written to the oplog with optime " << renameOpTime;
-                fassertFailed(40616);
+                LOGV2_FATAL(
+                    40616,
+                    "renameCollection: {from} to {to} (with dropTarget=true) - unexpected "
+                    "renameCollection oplog entry written to the oplog with optime {renameOpTime}",
+                    "renameCollection (with dropTarget=true): unexpected renameCollection oplog "
+                    "entry written to the oplog",
+                    "from"_attr = source,
+                    "to"_attr = target,
+                    "renameOpTime"_attr = renameOpTime);
             }
             renameOpTime = renameOpTimeFromApplyOps;
         }
@@ -334,6 +297,13 @@ Status renameCollectionWithinDB(OperationContext* opCtx,
     DisableDocumentValidation validationDisabler(opCtx);
 
     Lock::DBLock dbWriteLock(opCtx, source.db(), MODE_IX);
+
+    {
+        auto dss = DatabaseShardingState::get(opCtx, source.db());
+        auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, dss);
+        dss->checkDbVersion(opCtx, dssLock);
+    }
+
     boost::optional<Lock::CollectionLock> sourceLock;
     boost::optional<Lock::CollectionLock> targetLock;
     // To prevent deadlock, always lock system.views collection in the end because concurrent
@@ -356,8 +326,10 @@ Status renameCollectionWithinDB(OperationContext* opCtx,
         return status;
 
     auto db = DatabaseHolder::get(opCtx)->getDb(opCtx, source.db());
-    Collection* const sourceColl = db->getCollection(opCtx, source);
-    Collection* const targetColl = db->getCollection(opCtx, target);
+    Collection* const sourceColl =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, source);
+    Collection* const targetColl =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, target);
 
     AutoStatsTracker statsTracker(opCtx,
                                   source,
@@ -384,13 +356,20 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
 
     Lock::DBLock dbWriteLock(opCtx, source.db(), MODE_X);
 
+    {
+        auto dss = DatabaseShardingState::get(opCtx, source.db());
+        auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, dss);
+        dss->checkDbVersion(opCtx, dssLock);
+    }
+
     auto status = checkSourceAndTargetNamespaces(
         opCtx, source, target, options, /* targetExistsAllowed */ true);
     if (!status.isOK())
         return status;
 
     auto db = DatabaseHolder::get(opCtx)->getDb(opCtx, source.db());
-    Collection* const sourceColl = db->getCollection(opCtx, source);
+    Collection* const sourceColl =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, source);
 
     AutoStatsTracker statsTracker(opCtx,
                                   source,
@@ -399,7 +378,8 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
                                   db->getProfilingLevel());
 
     return writeConflictRetry(opCtx, "renameCollection", target.ns(), [&] {
-        Collection* targetColl = db->getCollection(opCtx, target);
+        Collection* targetColl =
+            CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, target);
         WriteUnitOfWork wuow(opCtx);
         if (targetColl) {
             if (sourceColl->uuid() == targetColl->uuid()) {
@@ -428,7 +408,7 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
             if (!uuidToDrop || uuidToDrop != targetColl->uuid()) {
                 // We need to rename the targetColl to a temporary name.
                 auto status = renameTargetCollectionToTmp(
-                    opCtx, source, sourceColl->uuid().get(), db, target, targetColl->uuid().get());
+                    opCtx, source, sourceColl->uuid(), db, target, targetColl->uuid());
                 if (!status.isOK())
                     return status;
                 targetColl = nullptr;
@@ -443,7 +423,8 @@ Status renameCollectionWithinDBForApplyOps(OperationContext* opCtx,
             auto collToDropBasedOnUUID = getNamespaceFromUUID(opCtx, uuidToDrop.get());
             if (collToDropBasedOnUUID && !collToDropBasedOnUUID->isDropPendingNamespace()) {
                 invariant(collToDropBasedOnUUID->db() == target.db());
-                targetColl = db->getCollection(opCtx, *collToDropBasedOnUUID);
+                targetColl = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(
+                    opCtx, *collToDropBasedOnUUID);
             }
         }
 
@@ -495,18 +476,18 @@ Status renameBetweenDBs(OperationContext* opCtx,
         targetDBLock.emplace(opCtx, target.db(), MODE_X);
     }
 
+    {
+        auto dss = DatabaseShardingState::get(opCtx, source.db());
+        auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, dss);
+        dss->checkDbVersion(opCtx, dssLock);
+    }
+
     DisableDocumentValidation validationDisabler(opCtx);
 
     auto sourceDB = DatabaseHolder::get(opCtx)->getDb(opCtx, source.db());
     if (!sourceDB)
         return Status(ErrorCodes::NamespaceNotFound, "source namespace does not exist");
 
-    {
-        auto& dss = DatabaseShardingState::get(sourceDB);
-        auto dssLock = DatabaseShardingState::DSSLock::lockShared(opCtx, &dss);
-        dss.checkDbVersion(opCtx, dssLock);
-    }
-
     boost::optional<AutoStatsTracker> statsTracker(boost::in_place_init,
                                                    opCtx,
                                                    source,
@@ -514,7 +495,8 @@ Status renameBetweenDBs(OperationContext* opCtx,
                                                    AutoStatsTracker::LogMode::kUpdateCurOp,
                                                    sourceDB->getProfilingLevel());
 
-    Collection* const sourceColl = sourceDB->getCollection(opCtx, source);
+    Collection* const sourceColl =
+        CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, source);
     if (!sourceColl) {
         if (sourceDB && ViewCatalog::get(sourceDB)->lookup(opCtx, source.ns()))
             return Status(ErrorCodes::CommandNotSupportedOnView,
@@ -522,27 +504,27 @@ Status renameBetweenDBs(OperationContext* opCtx,
         return Status(ErrorCodes::NamespaceNotFound, "source namespace does not exist");
     }
 
-    if (isCollectionSharded(opCtx, source))
-        return {ErrorCodes::IllegalOperation, "source namespace cannot be sharded"};
+    // TODO: SERVER-42638 Replace checks of cm() with cm()->distributionMode() == sharded
+    if (!MONGO_unlikely(useRenameCollectionPathThroughConfigsvr.shouldFail())) {
+        if (isCollectionSharded(opCtx, source))
+            return {ErrorCodes::IllegalOperation, "source namespace cannot be sharded"};
+    }
 
     if (isReplicatedChanged(opCtx, source, target))
         return {ErrorCodes::IllegalOperation,
                 "Cannot rename collections between a replicated and an unreplicated database"};
 
-    auto status = checkAllIndexesCanBeRecreatedInNewNS(opCtx, sourceColl, target);
-    if (!status.isOK())
-        return status;
-
     BackgroundOperation::assertNoBgOpInProgForNs(source.ns());
-    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(
-        sourceColl->uuid().get());
+    IndexBuildsCoordinator::get(opCtx)->assertNoIndexBuildInProgForCollection(sourceColl->uuid());
 
     auto targetDB = DatabaseHolder::get(opCtx)->getDb(opCtx, target.db());
 
     // Check if the target namespace exists and if dropTarget is true.
     // Return a non-OK status if target exists and dropTarget is not true or if the collection
     // is sharded.
-    Collection* targetColl = targetDB ? targetDB->getCollection(opCtx, target) : nullptr;
+    Collection* targetColl = targetDB
+        ? CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, target)
+        : nullptr;
     if (targetColl) {
         if (sourceColl->uuid() == targetColl->uuid()) {
             invariant(source == target);
@@ -579,18 +561,17 @@ Status renameBetweenDBs(OperationContext* opCtx,
     }
     const auto& tmpName = tmpNameResult.getValue();
 
-    log() << "Attempting to create temporary collection: " << tmpName
-          << " with the contents of collection: " << source;
-
-    // Check if all the source collection's indexes can be recreated in the temporary collection.
-    status = checkAllIndexesCanBeRecreatedInNewNS(opCtx, sourceColl, tmpName);
-    if (!status.isOK())
-        return status;
+    LOGV2(20398,
+          "Attempting to create temporary collection: {tmpName} with the contents of collection: "
+          "{source}",
+          "Attempting to create temporary collection",
+          "temporaryCollection"_attr = tmpName,
+          "sourceCollection"_attr = source);
 
     Collection* tmpColl = nullptr;
     {
         auto collectionOptions =
-            DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, sourceColl->ns());
+            DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, sourceColl->getCatalogId());
 
         // Renaming across databases will result in a new UUID.
         collectionOptions.uuid = UUID::gen();
@@ -604,76 +585,81 @@ Status renameBetweenDBs(OperationContext* opCtx,
 
     // Dismissed on success
     auto tmpCollectionDropper = makeGuard([&] {
-        BSONObjBuilder unusedResult;
         Status status = Status::OK();
         try {
-            status =
-                dropCollection(opCtx,
-                               tmpName,
-                               unusedResult,
-                               {},
-                               DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
+            status = dropCollectionForApplyOps(
+                opCtx,
+                tmpName,
+                {},
+                DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
         } catch (...) {
             status = exceptionToStatus();
         }
         if (!status.isOK()) {
             // Ignoring failure case when dropping the temporary collection during cleanup because
             // the rename operation has already failed for another reason.
-            log() << "Unable to drop temporary collection " << tmpName << " while renaming from "
-                  << source << " to " << target << ": " << status;
+            LOGV2(20399,
+                  "Unable to drop temporary collection {tmpName} while renaming from {source} to "
+                  "{target}: {error}",
+                  "Unable to drop temporary collection while renaming",
+                  "tempCollection"_attr = tmpName,
+                  "source"_attr = source,
+                  "target"_attr = target,
+                  "error"_attr = status);
         }
     });
 
-    // Copy the index descriptions from the source collection, adjusting the ns field.
-    {
-        std::vector<BSONObj> indexesToCopy;
-        std::unique_ptr<IndexCatalog::IndexIterator> sourceIndIt =
-            sourceColl->getIndexCatalog()->getIndexIterator(opCtx, true);
-        while (sourceIndIt->more()) {
-            auto descriptor = sourceIndIt->next()->descriptor();
-            if (descriptor->isIdIndex()) {
-                continue;
-            }
-
-            const BSONObj currIndex = descriptor->infoObj();
-
-            // Process the source index, adding fields in the same order as they were originally.
-            BSONObjBuilder newIndex;
-            for (auto&& elem : currIndex) {
-                if (elem.fieldNameStringData() == "ns") {
-                    newIndex.append("ns", tmpName.ns());
-                } else {
-                    newIndex.append(elem);
-                }
-            }
-            indexesToCopy.push_back(newIndex.obj());
+    // Copy the index descriptions from the source collection, adjusting the 'ns' field if
+    // necessary.
+    std::vector<BSONObj> indexesToCopy;
+    for (auto sourceIndIt = sourceColl->getIndexCatalog()->getIndexIterator(opCtx, true);
+         sourceIndIt->more();) {
+        auto descriptor = sourceIndIt->next()->descriptor();
+        if (descriptor->isIdIndex()) {
+            continue;
         }
 
-        // Create indexes using the namespace-adjusted index specs on the empty temporary collection
-        // that was just created. Since each index build is possibly replicated to downstream nodes,
-        // each createIndex oplog entry must have a distinct timestamp to support correct rollback
-        // operation. This is achieved by writing the createIndexes oplog entry *before* creating
-        // the index. Using IndexCatalog::createIndexOnEmptyCollection() for the index creation
-        // allows us to add and commit the index within a single WriteUnitOfWork and avoids the
-        // possibility of seeing the index in an unfinished state. For more information on assigning
-        // timestamps to multiple index builds, please see SERVER-35780 and SERVER-35070.
-        status = writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
+        const BSONObj currIndex = descriptor->infoObj();
+
+        // In FCV 4.4, there is no 'ns' field to modify.
+        if (serverGlobalParams.featureCompatibility.isVersionInitialized() &&
+            serverGlobalParams.featureCompatibility.getVersion() ==
+                ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44) {
+            indexesToCopy.push_back(currIndex);
+            continue;
+        }
+
+        // In FCV 4.2, we must rename the 'ns' field if it exists.
+        // Process the source index, adding fields in the same order as they were originally.
+        BSONObjBuilder newIndex;
+        for (auto&& elem : currIndex) {
+            if (elem.fieldNameStringData() == "ns") {
+                newIndex.append("ns", tmpName.ns());
+            } else {
+                newIndex.append(elem);
+            }
+        }
+        indexesToCopy.push_back(newIndex.obj());
+    }
+
+    // Create indexes using the index specs on the empty temporary collection that was just created.
+    // Since each index build is possibly replicated to downstream nodes, each createIndex oplog
+    // entry must have a distinct timestamp to support correct rollback operation. This is achieved
+    // by writing the createIndexes oplog entry *before* creating the index. Using
+    // IndexCatalog::createIndexOnEmptyCollection() for the index creation allows us to add and
+    // commit the index within a single WriteUnitOfWork and avoids the possibility of seeing the
+    // index in an unfinished state. For more information on assigning timestamps to multiple index
+    // builds, please see SERVER-35780 and SERVER-35070.
+    if (!indexesToCopy.empty()) {
+        Status status = writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
             WriteUnitOfWork wunit(opCtx);
-            auto tmpIndexCatalog = tmpColl->getIndexCatalog();
-            auto opObserver = opCtx->getServiceContext()->getOpObserver();
-            for (const auto& indexToCopy : indexesToCopy) {
-                opObserver->onCreateIndex(opCtx,
-                                          tmpName,
-                                          *(tmpColl->uuid()),
-                                          indexToCopy,
-                                          false  // fromMigrate
-                );
-                auto indexResult =
-                    tmpIndexCatalog->createIndexOnEmptyCollection(opCtx, indexToCopy);
-                if (!indexResult.isOK()) {
-                    return indexResult.getStatus();
-                }
-            };
+            auto fromMigrate = false;
+            try {
+                IndexBuildsCoordinator::get(opCtx)->createIndexesOnEmptyCollection(
+                    opCtx, tmpColl->uuid(), indexesToCopy, fromMigrate);
+            } catch (DBException& ex) {
+                return ex.toStatus();
+            }
             wunit.commit();
             return Status::OK();
         });
@@ -684,7 +670,7 @@ Status renameBetweenDBs(OperationContext* opCtx,
 
     {
         NamespaceStringOrUUID tmpCollUUID =
-            NamespaceStringOrUUID(std::string(tmpName.db()), *tmpColl->uuid());
+            NamespaceStringOrUUID(std::string(tmpName.db()), tmpColl->uuid());
         statsTracker.reset();
 
         // Copy over all the data from source collection to temporary collection. For this we can
@@ -706,13 +692,13 @@ Status renameBetweenDBs(OperationContext* opCtx,
             opCtx->checkForInterrupt();
             // Cursor is left one past the end of the batch inside writeConflictRetry.
             auto beginBatchId = record->id;
-            status = writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
+            Status status = writeConflictRetry(opCtx, "renameCollection", tmpName.ns(), [&] {
+                WriteUnitOfWork wunit(opCtx);
                 // Need to reset cursor if it gets a WCE midway through.
                 if (!record || (beginBatchId != record->id)) {
                     record = cursor->seekExact(beginBatchId);
                 }
                 for (int i = 0; record && (i < internalInsertMaxBatchSize.load()); i++) {
-                    WriteUnitOfWork wunit(opCtx);
                     const InsertStatement stmt(record->data.releaseToBson());
                     OpDebug* const opDebug = nullptr;
                     auto status = tmpColl->insertDocument(opCtx, stmt, opDebug, true);
@@ -720,12 +706,6 @@ Status renameBetweenDBs(OperationContext* opCtx,
                         return status;
                     }
                     record = cursor->next();
-
-                    // Used to make sure that a WCE can be handled by this logic without data loss.
-                    if (MONGO_FAIL_POINT(writeConflictInRenameCollCopyToTmp)) {
-                        throw WriteConflictException();
-                    }
-                    wunit.commit();
                 }
 
                 // Time to yield; make a safe copy of the current record before releasing our
@@ -739,6 +719,11 @@ Status renameBetweenDBs(OperationContext* opCtx,
                     writeConflictRetry(
                         opCtx, "retryRestoreCursor", ns, [&cursor] { cursor->restore(); });
                 });
+                // Used to make sure that a WCE can be handled by this logic without data loss.
+                if (MONGO_unlikely(writeConflictInRenameCollCopyToTmp.shouldFail())) {
+                    throw WriteConflictException();
+                }
+                wunit.commit();
                 return Status::OK();
             });
             if (!status.isOK())
@@ -751,22 +736,108 @@ Status renameBetweenDBs(OperationContext* opCtx,
     // Getting here means we successfully built the target copy. We now do the final
     // in-place rename and remove the source collection.
     invariant(tmpName.db() == target.db());
-    status = renameCollectionWithinDB(opCtx, tmpName, target, options);
+    Status status = renameCollectionWithinDB(opCtx, tmpName, target, options);
     if (!status.isOK())
         return status;
 
     tmpCollectionDropper.dismiss();
-
-    BSONObjBuilder unusedResult;
-    return dropCollection(opCtx,
-                          source,
-                          unusedResult,
-                          {},
-                          DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
+    return dropCollectionForApplyOps(
+        opCtx, source, {}, DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
 }
 
 }  // namespace
 
+void doLocalRenameIfOptionsAndIndexesHaveNotChanged(OperationContext* opCtx,
+                                                    const NamespaceString& sourceNs,
+                                                    const NamespaceString& targetNs,
+                                                    bool dropTarget,
+                                                    bool stayTemp,
+                                                    std::list<BSONObj> originalIndexes,
+                                                    BSONObj originalCollectionOptions) {
+    AutoGetDb dbLock(opCtx, targetNs.db(), MODE_X);
+    auto collection = dbLock.getDb()
+        ? CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, targetNs)
+        : nullptr;
+    BSONObj collectionOptions = {};
+    if (collection) {
+        // We do not include the UUID field in the options comparison. It is ok if the target
+        // collection was dropped and recreated, as long as the new target collection has the same
+        // options and indexes as the original one did. This is mainly to support concurrent $out
+        // to the same collection.
+        collectionOptions = DurableCatalog::get(opCtx)
+                                ->getCollectionOptions(opCtx, collection->getCatalogId())
+                                .toBSON()
+                                .removeField("uuid");
+    }
+
+    uassert(ErrorCodes::CommandFailed,
+            str::stream() << "collection options of target collection " << targetNs.ns()
+                          << " changed during processing. Original options: "
+                          << originalCollectionOptions << ", new options: " << collectionOptions,
+            SimpleBSONObjComparator::kInstance.evaluate(
+                originalCollectionOptions.removeField("uuid") == collectionOptions));
+
+    auto currentIndexes =
+        listIndexesEmptyListIfMissing(opCtx, targetNs, false /* includeBuildUUIDs */);
+    uassert(ErrorCodes::CommandFailed,
+            str::stream() << "indexes of target collection " << targetNs.ns()
+                          << " changed during processing.",
+            originalIndexes.size() == currentIndexes.size() &&
+                std::equal(originalIndexes.begin(),
+                           originalIndexes.end(),
+                           currentIndexes.begin(),
+                           SimpleBSONObjComparator::kInstance.makeEqualTo()));
+
+    validateAndRunRenameCollection(opCtx, sourceNs, targetNs, dropTarget, stayTemp);
+}
+void validateAndRunRenameCollection(OperationContext* opCtx,
+                                    const NamespaceString& source,
+                                    const NamespaceString& target,
+                                    bool dropTarget,
+                                    bool stayTemp) {
+    uassert(ErrorCodes::InvalidNamespace,
+            str::stream() << "Invalid source namespace: " << source.ns(),
+            source.isValid());
+    uassert(ErrorCodes::InvalidNamespace,
+            str::stream() << "Invalid target namespace: " << target.ns(),
+            target.isValid());
+
+    if ((repl::ReplicationCoordinator::get(opCtx)->getReplicationMode() !=
+         repl::ReplicationCoordinator::modeNone)) {
+        uassert(ErrorCodes::IllegalOperation,
+                "can't rename live oplog while replicating",
+                !source.isOplog());
+        uassert(ErrorCodes::IllegalOperation,
+                "can't rename to live oplog while replicating",
+                !target.isOplog());
+    }
+
+    uassert(ErrorCodes::IllegalOperation,
+            "If either the source or target of a rename is an oplog name, both must be",
+            source.isOplog() == target.isOplog());
+
+    Status sourceStatus = userAllowedWriteNS(source);
+    uassert(ErrorCodes::IllegalOperation,
+            "error with source namespace: " + sourceStatus.reason(),
+            sourceStatus.isOK());
+    Status targetStatus = userAllowedWriteNS(target);
+    uassert(ErrorCodes::IllegalOperation,
+            "error with target namespace: " + targetStatus.reason(),
+            targetStatus.isOK());
+
+    if (source.isServerConfigurationCollection()) {
+        uasserted(ErrorCodes::IllegalOperation,
+                  "renaming the server configuration "
+                  "collection (admin.system.version) is not "
+                  "allowed");
+    }
+
+    RenameCollectionOptions options;
+    options.dropTarget = dropTarget;
+    options.stayTemp = stayTemp;
+    uassertStatusOK(renameCollection(opCtx, source, target, options));
+}
+
 Status renameCollection(OperationContext* opCtx,
                         const NamespaceString& source,
                         const NamespaceString& target,
@@ -784,9 +855,13 @@ Status renameCollection(OperationContext* opCtx,
             "renaming system.views collection or renaming to system.views is not allowed");
     }
 
-    const std::string dropTargetMsg =
-        options.dropTarget ? " and drop " + target.toString() + "." : ".";
-    log() << "renameCollectionForCommand: rename " << source << " to " << target << dropTargetMsg;
+    StringData dropTargetMsg = options.dropTarget ? "yes"_sd : "no"_sd;
+    LOGV2(20400,
+          "renameCollectionForCommand: rename {source} to {target}{dropTargetMsg}",
+          "renameCollectionForCommand",
+          "sourceNamespace"_attr = source,
+          "targetNamespace"_attr = target,
+          "dropTarget"_attr = dropTargetMsg);
 
     if (source.db() == target.db())
         return renameCollectionWithinDB(opCtx, source, target, options);
@@ -797,7 +872,7 @@ Status renameCollection(OperationContext* opCtx,
 
 Status renameCollectionForApplyOps(OperationContext* opCtx,
                                    const std::string& dbName,
-                                   const BSONElement& ui,
+                                   const OptionalCollectionUUID& uuidToRename,
                                    const BSONObj& cmd,
                                    const repl::OpTime& renameOpTime) {
 
@@ -819,10 +894,8 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
 
     NamespaceString sourceNss(sourceNsElt.valueStringData());
     NamespaceString targetNss(targetNsElt.valueStringData());
-    OptionalCollectionUUID uuidToRename;
-    if (!ui.eoo()) {
-        uuidToRename = uassertStatusOK(UUID::parse(ui));
-        auto nss = CollectionCatalog::get(opCtx).lookupNSSByUUID(uuidToRename.get());
+    if (uuidToRename) {
+        auto nss = CollectionCatalog::get(opCtx).lookupNSSByUUID(opCtx, uuidToRename.get());
         if (nss)
             sourceNss = *nss;
     }
@@ -866,12 +939,11 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
 
         // Downgrade renameCollection to dropCollection.
         if (dropTargetNss) {
-            BSONObjBuilder unusedResult;
-            return dropCollection(opCtx,
-                                  *dropTargetNss,
-                                  unusedResult,
-                                  renameOpTime,
-                                  DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
+            return dropCollectionForApplyOps(
+                opCtx,
+                *dropTargetNss,
+                renameOpTime,
+                DropCollectionSystemCollectionMode::kAllowSystemCollectionDrops);
         }
 
         return Status(ErrorCodes::NamespaceNotFound,
@@ -881,11 +953,16 @@ Status renameCollectionForApplyOps(OperationContext* opCtx,
                           << sourceNss.toString());
     }
 
-    const std::string dropTargetMsg =
-        uuidToDrop ? " and drop " + uuidToDrop->toString() + "." : ".";
+    const std::string uuidToDropString = uuidToDrop ? uuidToDrop->toString() : "<none>";
     const std::string uuidString = uuidToRename ? uuidToRename->toString() : "UUID unknown";
-    log() << "renameCollectionForApplyOps: rename " << sourceNss << " (" << uuidString << ") to "
-          << targetNss << dropTargetMsg;
+    LOGV2(20401,
+          "renameCollectionForApplyOps: rename {sourceNss} ({uuidString}) to "
+          "{targetNss}{dropTargetMsg}",
+          "renameCollectionForApplyOps",
+          "sourceNamespace"_attr = sourceNss,
+          "uuid"_attr = uuidString,
+          "targetNamespace"_attr = targetNss,
+          "uuidToDrop"_attr = uuidToDropString);
 
     if (sourceNss.db() == targetNss.db()) {
         return renameCollectionWithinDBForApplyOps(
@@ -906,8 +983,12 @@ Status renameCollectionForRollback(OperationContext* opCtx,
                                "have the same database. source: "
                             << *source << ". target: " << target);
 
-    log() << "renameCollectionForRollback: rename " << *source << " (" << uuid << ") to " << target
-          << ".";
+    LOGV2(20402,
+          "renameCollectionForRollback: rename {source} ({uuid}) to {target}.",
+          "renameCollectionForRollback",
+          "source"_attr = *source,
+          "uuid"_attr = uuid,
+          "target"_attr = target);
 
     return renameCollectionWithinDB(opCtx, *source, target, {});
 }
