@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -47,7 +47,7 @@
 #include "mongo/db/repl/optime.h"
 #include "mongo/util/background.h"
 
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
@@ -118,7 +118,7 @@ std::unique_ptr<DbCheckRun> singleCollectionRun(OperationContext* opCtx,
     auto maxSize = invocation.getMaxSize();
     auto maxRate = invocation.getMaxCountPerSecond();
     auto info = DbCheckCollectionInfo{nss, start, end, maxCount, maxSize, maxRate};
-    auto result = stdx::make_unique<DbCheckRun>();
+    auto result = std::make_unique<DbCheckRun>();
     result->push_back(info);
     return result;
 }
@@ -132,7 +132,7 @@ std::unique_ptr<DbCheckRun> fullDatabaseRun(OperationContext* opCtx,
     // Read the list of collections in a database-level lock.
     AutoGetDb agd(opCtx, StringData(dbName), MODE_S);
     auto db = agd.getDb();
-    auto result = stdx::make_unique<DbCheckRun>();
+    auto result = std::make_unique<DbCheckRun>();
 
     uassert(ErrorCodes::NamespaceNotFound, "Database " + dbName + " not found", agd.getDb());
 
@@ -210,7 +210,7 @@ protected:
             }
 
             if (_done) {
-                log() << "dbCheck terminated due to stepdown";
+                LOGV2(20451, "dbCheck terminated due to stepdown");
                 return;
             }
         }
@@ -334,17 +334,12 @@ private:
             return false;
         }
 
-        auto collection = db->getCollection(opCtx, info.nss);
+        auto collection =
+            CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, info.nss);
         if (!collection) {
             return false;
         }
 
-        auto uuid = collection->uuid();
-        // Check if UUID exists.
-        if (!uuid) {
-            return false;
-        }
-
         auto [prev, next] = getPrevAndNextUUIDs(opCtx, collection);
 
         // Find and report collection metadata.
@@ -353,7 +348,7 @@ private:
 
         DbCheckOplogCollection entry;
         entry.setNss(collection->ns());
-        entry.setUuid(*collection->uuid());
+        entry.setUuid(collection->uuid());
         if (prev) {
             entry.setPrev(*prev);
         }
@@ -375,7 +370,7 @@ private:
         collectionInfo.options = entry.getOptions();
 
         auto hle = dbCheckCollectionEntry(
-            collection->ns(), *collection->uuid(), collectionInfo, collectionInfo, optime);
+            collection->ns(), collection->uuid(), collectionInfo, collectionInfo, optime);
 
         HealthLog::get(opCtx).log(*hle);
 
@@ -467,25 +462,18 @@ private:
                         const NamespaceString& nss,
                         OptionalCollectionUUID uuid,
                         const BSONObj& obj) {
+        repl::MutableOplogEntry oplogEntry;
+        oplogEntry.setOpType(repl::OpTypeEnum::kCommand);
+        oplogEntry.setNss(nss);
+        oplogEntry.setUuid(uuid);
+        oplogEntry.setObject(obj);
         return writeConflictRetry(
             opCtx, "dbCheck oplog entry", NamespaceString::kRsOplogNamespace.ns(), [&] {
                 auto const clockSource = opCtx->getServiceContext()->getFastClockSource();
-                const auto wallClockTime = clockSource->now();
+                oplogEntry.setWallClockTime(clockSource->now());
 
                 WriteUnitOfWork uow(opCtx);
-                repl::OpTime result = repl::logOp(opCtx,
-                                                  "c",
-                                                  nss,
-                                                  uuid,
-                                                  obj,
-                                                  nullptr,
-                                                  false,
-                                                  wallClockTime,
-                                                  {},
-                                                  kUninitializedStmtId,
-                                                  {},
-                                                  OplogSlot(),
-                                                  {});
+                repl::OpTime result = repl::logOp(opCtx, &oplogEntry);
                 uow.commit();
                 return result;
             });
