@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -87,9 +87,9 @@
 #include "mongo/db/stats/storage_stats.h"
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/logv2/log.h"
 #include "mongo/scripting/engine.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/md5.hpp"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/version.h"
@@ -165,6 +165,41 @@ public:
 
 } cmdDropDatabase;
 
+static const char* repairRemovedMessage =
+    "This command has been removed. If you would like to compact your data, use the 'compact' "
+    "command. If you would like to rebuild indexes, use the 'reIndex' command. If you need to "
+    "recover data, please see the documentation for repairing your database offline: "
+    "http://dochub.mongodb.org/core/repair";
+
+class CmdRepairDatabase : public ErrmsgCommandDeprecated {
+public:
+    AllowedOnSecondary secondaryAllowed(ServiceContext*) const override {
+        return AllowedOnSecondary::kAlways;
+    }
+    virtual bool maintenanceMode() const {
+        return false;
+    }
+
+    std::string help() const override {
+        return repairRemovedMessage;
+    }
+    virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
+        return false;
+    }
+
+    CmdRepairDatabase() : ErrmsgCommandDeprecated("repairDatabase") {}
+
+    bool errmsgRun(OperationContext* opCtx,
+                   const string& dbname,
+                   const BSONObj& cmdObj,
+                   string& errmsg,
+                   BSONObjBuilder& result) {
+
+        uasserted(ErrorCodes::CommandNotFound, repairRemovedMessage);
+        return false;
+    }
+} cmdRepairDatabase;
+
 /* drop collection */
 class CmdDrop : public ErrmsgCommandDeprecated {
 public:
@@ -198,11 +233,6 @@ public:
                            BSONObjBuilder& result) {
         const NamespaceString nsToDrop(CommandHelpers::parseNsCollectionRequired(dbname, cmdObj));
 
-        if (nsToDrop.isVirtualized()) {
-            errmsg = "can't drop a virtual collection";
-            return false;
-        }
-
         if (nsToDrop.isOplog()) {
             if (repl::ReplicationCoordinator::get(opCtx)->isReplEnabled()) {
                 errmsg = "can't drop live oplog while replicating";
@@ -225,7 +255,6 @@ public:
             dropCollection(opCtx,
                            nsToDrop,
                            result,
-                           {},
                            DropCollectionSystemCollectionMode::kDisallowSystemCollectionDrops));
         return true;
     }
@@ -288,10 +317,10 @@ public:
         const NamespaceString ns = cmd.getNamespace();
 
         if (cmd.getAutoIndexId()) {
-            const char* deprecationWarning =
-                "the autoIndexId option is deprecated and will be removed in a future release";
-            warning() << deprecationWarning;
-            result.append("note", deprecationWarning);
+#define DEPR_23800 "The autoIndexId option is deprecated and will be removed in a future release"
+            LOGV2_WARNING(23800, DEPR_23800);
+            result.append("note", DEPR_23800);
+#undef DEPR_23800
         }
 
         // Ensure that the 'size' field is present if 'capped' is set to true.
@@ -313,7 +342,9 @@ public:
         if (cmd.getTemp()) {
             uassert(ErrorCodes::InvalidOptions,
                     str::stream() << "the 'temp' field is an invalid option",
-                    opCtx->getClient()->isInDirectClient());
+                    opCtx->getClient()->isInDirectClient() ||
+                        (opCtx->getClient()->session()->getTags() |
+                         transport::Session::kInternalClient));
         }
 
         // Validate _id index spec and fill in missing fields.
@@ -330,7 +361,7 @@ public:
 
             // Perform index spec validation.
             idIndexSpec = uassertStatusOK(index_key_validate::validateIndexSpec(
-                opCtx, idIndexSpec, ns, serverGlobalParams.featureCompatibility));
+                opCtx, idIndexSpec, serverGlobalParams.featureCompatibility));
             uassertStatusOK(index_key_validate::validateIdIndexSpec(idIndexSpec));
 
             // Validate or fill in _id index collation.
@@ -424,10 +455,10 @@ public:
         AutoGetCollectionForReadCommand ctx(opCtx, nss);
         Collection* collection = ctx.getCollection();
 
-        const auto metadata = CollectionShardingState::get(opCtx, nss)->getCurrentMetadata();
+        const auto collDesc = CollectionShardingState::get(opCtx, nss)->getCollectionDescription();
 
-        if (metadata->isSharded()) {
-            const ShardKeyPattern shardKeyPattern(metadata->getKeyPattern());
+        if (collDesc.isSharded()) {
+            const ShardKeyPattern shardKeyPattern(collDesc.getKeyPattern());
             uassert(ErrorCodes::BadValue,
                     "keyPattern must be empty or must be an object that equals the shard key",
                     keyPattern.isEmpty() ||
@@ -467,7 +498,7 @@ public:
                 result.append("millis", timer.millis());
                 return 1;
             }
-            exec = InternalPlanner::collectionScan(opCtx, ns, collection, PlanExecutor::YIELD_AUTO);
+            exec = InternalPlanner::collectionScan(opCtx, ns, collection, PlanExecutor::NO_YIELD);
         } else if (min.isEmpty() || max.isEmpty()) {
             errmsg = "only one of min or max specified";
             return false;
@@ -482,7 +513,7 @@ public:
                                                                          keyPattern,
                                                                          true);  // requireSingleKey
 
-            if (idx == NULL) {
+            if (idx == nullptr) {
                 errmsg = "couldn't find valid index containing key pattern";
                 return false;
             }
@@ -497,7 +528,7 @@ public:
                                               min,
                                               max,
                                               BoundInclusion::kIncludeStartKeyOnly,
-                                              PlanExecutor::YIELD_AUTO);
+                                              PlanExecutor::NO_YIELD);
         }
 
         long long avgObjSize = collection->dataSize(opCtx) / numRecords;
@@ -526,7 +557,10 @@ public:
         }
 
         if (PlanExecutor::FAILURE == state) {
-            warning() << "Internal error while reading " << ns;
+            LOGV2_WARNING(23801,
+                          "Internal error while reading {namespace}",
+                          "Internal error while reading",
+                          "namespace"_attr = ns);
             uassertStatusOK(WorkingSetCommon::getMemberObjectStatus(obj).withContext(
                 "Executor error while reading during dataSize command"));
         }
@@ -703,7 +737,7 @@ public:
             result.append("avgObjSize", 0);
             result.appendNumber("dataSize", 0);
             result.appendNumber("storageSize", 0);
-            result.appendNumber("numExtents", 0);
+            result.appendNumber("totalSize", 0);
             result.appendNumber("indexes", 0);
             result.appendNumber("indexSize", 0);
             result.appendNumber("scaleFactor", scale);
