@@ -32,8 +32,8 @@
 namespace mongo {
 namespace feature_compatibility_version_documentation {
 constexpr StringData kCompatibilityLink =
-    "https://docs.mongodb.com/manual/release-notes/4.2-compatibility/#feature-compatibility"_sd;
+    "https://docs.mongodb.com/master/release-notes/4.4-compatibility/#feature-compatibility"_sd;
 constexpr StringData kUpgradeLink =
-    "https://docs.mongodb.com/manual/release-notes/4.2/#upgrade-procedures"_sd;
+    "https://docs.mongodb.com/master/release-notes/4.4/#upgrade-procedures"_sd;
 }  // namespace feature_compatibility_version_documentation
 }  // namespace mongo
