@@ -27,8 +27,6 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
-
 #include "mongo/platform/basic.h"
 
 #include "mongo/bson/util/bson_extract.h"
@@ -36,7 +34,6 @@
 #include "mongo/db/commands.h"
 #include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/log_process_details.h"
-#include "mongo/util/log.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/ramlog.h"
 
@@ -159,10 +156,10 @@ public:
                      const string& ns,
                      const BSONObj& cmdObj,
                      BSONObjBuilder& result) {
-        // Sort the command names before building the result BSON.
+        // sort the commands before building the result BSON
         std::vector<Command*> commands;
         for (const auto command : globalCommandRegistry()->allCommands()) {
-            // Don't show oldnames
+            // don't show oldnames
             if (command.first == command.second->getName())
                 commands.push_back(command.second);
         }
@@ -171,20 +168,20 @@ public:
         });
 
         BSONObjBuilder b(result.subobjStart("commands"));
-        for (const auto& command : commands) {
-            BSONObjBuilder temp(b.subobjStart(command->getName()));
-            temp.append("help", command->help());
+        for (const auto& c : commands) {
+            BSONObjBuilder temp(b.subobjStart(c->getName()));
+            temp.append("help", c->help());
+            temp.append("requiresAuth", c->requiresAuth());
             temp.append("slaveOk",
-                        command->secondaryAllowed(opCtx->getServiceContext()) ==
+                        c->secondaryAllowed(opCtx->getServiceContext()) ==
                             Command::AllowedOnSecondary::kAlways);
-            temp.append("adminOnly", command->adminOnly());
-            // Optionally indicates that the command can be forced to run on a secondary.
-            if (command->secondaryAllowed(opCtx->getServiceContext()) ==
+            temp.append("adminOnly", c->adminOnly());
+            // optionally indicates that the command can be forced to run on a slave/secondary
+            if (c->secondaryAllowed(opCtx->getServiceContext()) ==
                 Command::AllowedOnSecondary::kOptIn)
                 temp.append("slaveOverrideOk", true);
             temp.done();
         }
-
         b.done();
 
         return 1;
@@ -192,46 +189,5 @@ public:
 
 } listCommandsCmd;
 
-class CmdLogMessage : public BasicCommand {
-public:
-    CmdLogMessage() : BasicCommand("logMessage") {}
-
-    std::string help() const final {
-        return "Send a message to the server log";
-    }
-
-    bool supportsWriteConcern(const BSONObj& cmd) const final {
-        return false;
-    }
-
-    AllowedOnSecondary secondaryAllowed(ServiceContext*) const final {
-        return AllowedOnSecondary::kAlways;
-    }
-
-    bool adminOnly() const final {
-        return true;
-    }
-
-    void addRequiredPrivileges(const std::string& dbname,
-                               const BSONObj& cmdObj,
-                               std::vector<Privilege>* out) const final {
-        out->push_back(
-            Privilege(ResourcePattern::forClusterResource(), ActionType::applicationMessage));
-    }
-
-    bool run(OperationContext* opCtx,
-             const std::string& ns,
-             const BSONObj& cmdObj,
-             BSONObjBuilder& result) final {
-        auto msgElem = cmdObj["logMessage"];
-        uassert(ErrorCodes::BadValue, "logMessage must be a string", msgElem.type() == String);
-
-        log() << "logMessage: " << msgElem.valueStringData();
-        return true;
-    }
-};
-
-MONGO_REGISTER_TEST_COMMAND(CmdLogMessage);
-
 }  // namespace
 }  // namespace mongo
