@@ -27,13 +27,14 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/commands/run_aggregate.h"
 
 #include <boost/optional.hpp>
+#include <memory>
 #include <vector>
 
 #include "mongo/db/auth/authorization_session.h"
@@ -42,20 +43,20 @@
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/exec/change_stream_proxy.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/pipeline/accumulator.h"
-#include "mongo/db/pipeline/document.h"
 #include "mongo/db/pipeline/document_source.h"
 #include "mongo/db/pipeline/document_source_exchange.h"
 #include "mongo/db/pipeline/document_source_geo_near.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
-#include "mongo/db/pipeline/lite_parsed_pipeline.h"
-#include "mongo/db/pipeline/mongo_process_interface.h"
 #include "mongo/db/pipeline/pipeline.h"
 #include "mongo/db/pipeline/pipeline_d.h"
+#include "mongo/db/pipeline/process_interface/mongo_process_interface.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
+#include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/query/cursor_response.h"
 #include "mongo/db/query/find_common.h"
 #include "mongo/db/query/get_executor.h"
@@ -70,8 +71,7 @@
 #include "mongo/db/storage/storage_options.h"
 #include "mongo/db/views/view.h"
 #include "mongo/db/views/view_catalog.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/string_map.h"
 
@@ -83,13 +83,8 @@ using std::shared_ptr;
 using std::string;
 using std::stringstream;
 using std::unique_ptr;
-using stdx::make_unique;
 
 namespace {
-Counter64 allowDiskUseCounter;
-ServerStatusMetricField<Counter64> allowDiskUseMetric{"commands.aggregate.allowDiskUseTrue",
-                                                      &allowDiskUseCounter};
-
 /**
  * Returns true if this PlanExecutor is for a Pipeline.
  */
@@ -116,8 +111,7 @@ bool canOptimizeAwayPipeline(const Pipeline* pipeline,
         pipeline->getSources().empty() &&
         // For exchange we will create a number of pipelines consisting of a single
         // DocumentSourceExchange stage, so cannot not optimize it away.
-        !request.getExchangeSpec() &&
-        !QueryPlannerCommon::hasNode(exec->getCanonicalQuery()->root(), MatchExpression::TEXT);
+        !request.getExchangeSpec();
 }
 
 /**
@@ -176,15 +170,15 @@ bool handleCursorCommand(OperationContext* opCtx,
     auto exec = cursor->getExecutor();
     invariant(exec);
 
-    BSONObj next;
     bool stashedResult = false;
     for (int objCount = 0; objCount < batchSize; objCount++) {
         // The initial getNext() on a PipelineProxyStage may be very expensive so we don't
         // do it when batchSize is 0 since that indicates a desire for a fast return.
         PlanExecutor::ExecState state;
+        Document nextDoc;
 
         try {
-            state = exec->getNext(&next, nullptr);
+            state = exec->getNext(&nextDoc, nullptr);
         } catch (const ExceptionFor<ErrorCodes::CloseChangeStream>&) {
             // This exception is thrown when a $changeStream stage encounters an event
             // that invalidates the cursor. We should close the cursor and return without
@@ -205,26 +199,32 @@ bool handleCursorCommand(OperationContext* opCtx,
 
         if (PlanExecutor::ADVANCED != state) {
             // We should always have a valid status member object at this point.
-            auto status = WorkingSetCommon::getMemberObjectStatus(next);
+            auto status = WorkingSetCommon::getMemberObjectStatus(nextDoc);
             invariant(!status.isOK());
-            warning() << "Aggregate command executor error: " << PlanExecutor::statestr(state)
-                      << ", status: " << status
-                      << ", stats: " << redact(Explain::getWinningPlanStats(exec));
+            LOGV2_WARNING(
+                23799,
+                "Aggregate command executor error: {state}, status: {error}, stats: {stats}",
+                "Aggregate command executor error",
+                "state"_attr = PlanExecutor::statestr(state),
+                "error"_attr = status,
+                "stats"_attr = redact(Explain::getWinningPlanStats(exec)));
 
             uassertStatusOK(status.withContext("PlanExecutor error during aggregation"));
         }
 
         // If adding this object will cause us to exceed the message size limit, then we stash it
         // for later.
+
+        auto* expCtx = exec->getExpCtx().get();
+        BSONObj next = expCtx->needsMerge ? nextDoc.toBsonWithMetaData(expCtx->sortKeyFormat)
+                                          : nextDoc.toBson();
         if (!FindCommon::haveSpaceForNext(next, objCount, responseBuilder.bytesUsed())) {
-            exec->enqueue(next);
+            exec->enqueue(nextDoc);
             stashedResult = true;
             break;
         }
 
-        // TODO SERVER-38539: We need to set both the latestOplogTimestamp and the PBRT until the
-        // former is removed in a future release.
-        responseBuilder.setLatestOplogTimestamp(exec->getLatestOplogTimestamp());
+        // If this executor produces a postBatchResumeToken, add it to the cursor response.
         responseBuilder.setPostBatchResumeToken(exec->getPostBatchResumeToken());
         responseBuilder.append(next);
     }
@@ -235,9 +235,6 @@ bool handleCursorCommand(OperationContext* opCtx,
         // For empty batches, or in the case where the final result was added to the batch rather
         // than being stashed, we update the PBRT to ensure that it is the most recent available.
         if (!stashedResult) {
-            // TODO SERVER-38539: We need to set both the latestOplogTimestamp and the PBRT until
-            // the former is removed in a future release.
-            responseBuilder.setLatestOplogTimestamp(exec->getLatestOplogTimestamp());
             responseBuilder.setPostBatchResumeToken(exec->getPostBatchResumeToken());
         }
         // If a time limit was set on the pipeline, remaining time is "rolled over" to the
@@ -295,50 +292,13 @@ StatusWith<StringMap<ExpressionContext::ResolvedNamespace>> resolveInvolvedNames
             continue;
         }
 
-        // If 'ns' refers to a view namespace, then we resolve its definition.
-        auto resolveViewDefinition = [&](const NamespaceString& ns) -> Status {
-            auto resolvedView = viewCatalog->resolveView(opCtx, ns);
-            if (!resolvedView.isOK()) {
-                return {ErrorCodes::FailedToParse,
-                        str::stream() << "Failed to resolve view '" << involvedNs.ns()
-                                      << "': " << resolvedView.getStatus().toString()};
-            }
-
-            resolvedNamespaces[ns.coll()] = {resolvedView.getValue().getNamespace(),
-                                             resolvedView.getValue().getPipeline()};
-
-            // We parse the pipeline corresponding to the resolved view in case we must resolve
-            // other view namespaces that are also involved.
-            LiteParsedPipeline resolvedViewLitePipeline(
-                {resolvedView.getValue().getNamespace(), resolvedView.getValue().getPipeline()});
-
-            const auto& resolvedViewInvolvedNamespaces =
-                resolvedViewLitePipeline.getInvolvedNamespaces();
-            involvedNamespacesQueue.insert(involvedNamespacesQueue.end(),
-                                           resolvedViewInvolvedNamespaces.begin(),
-                                           resolvedViewInvolvedNamespaces.end());
-            return Status::OK();
-        };
-
-        // If the involved namespace is not in the same database as the aggregation, it must be
-        // from a $merge to a collection in a different database.
         if (involvedNs.db() != request.getNamespaceString().db()) {
-            // SERVER-51886: It is not correct to assume that we are reading from a collection
-            // because the collection targeted by $out/$merge on a given database can have the same
-            // name as a view on the source database. As such, we determine whether the collection
-            // name references a view on the aggregation request's database. Note that the inverse
-            // scenario (mistaking a view for a collection) is not an issue because $merge/$out
-            // cannot target a view.
-            auto nssToCheck = NamespaceString(request.getNamespaceString().db(), involvedNs.coll());
-            if (viewCatalog && viewCatalog->lookup(opCtx, nssToCheck.ns())) {
-                auto status = resolveViewDefinition(nssToCheck);
-                if (!status.isOK()) {
-                    return status;
-                }
-            } else {
-                resolvedNamespaces[involvedNs.coll()] = {involvedNs, std::vector<BSONObj>{}};
-            }
-        } else if (!db || CollectionCatalog::get(opCtx).lookupCollectionByNamespace(involvedNs)) {
+            // If the involved namespace is not in the same database as the aggregation, it must be
+            // from a $merge to a collection in a different database. Since we cannot write to
+            // views, simply assume that the namespace is a collection.
+            resolvedNamespaces[involvedNs.coll()] = {involvedNs, std::vector<BSONObj>{}};
+        } else if (!db ||
+                   CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, involvedNs)) {
             // If the aggregation database exists and 'involvedNs' refers to a collection namespace,
             // then we resolve it as an empty pipeline in order to read directly from the underlying
             // collection. If the database doesn't exist, then we still resolve it as an empty
@@ -346,10 +306,26 @@ StatusWith<StringMap<ExpressionContext::ResolvedNamespace>> resolveInvolvedNames
             // snapshot of the view catalog.
             resolvedNamespaces[involvedNs.coll()] = {involvedNs, std::vector<BSONObj>{}};
         } else if (viewCatalog->lookup(opCtx, involvedNs.ns())) {
-            auto status = resolveViewDefinition(involvedNs);
-            if (!status.isOK()) {
-                return status;
+            // If 'involvedNs' refers to a view namespace, then we resolve its definition.
+            auto resolvedView = viewCatalog->resolveView(opCtx, involvedNs);
+            if (!resolvedView.isOK()) {
+                return resolvedView.getStatus().withContext(
+                    str::stream() << "Failed to resolve view '" << involvedNs.ns());
             }
+
+            resolvedNamespaces[involvedNs.coll()] = {resolvedView.getValue().getNamespace(),
+                                                     resolvedView.getValue().getPipeline()};
+
+            // We parse the pipeline corresponding to the resolved view in case we must resolve
+            // other view namespaces that are also involved.
+            LiteParsedPipeline resolvedViewLitePipeline(resolvedView.getValue().getNamespace(),
+                                                        resolvedView.getValue().getPipeline());
+
+            const auto& resolvedViewInvolvedNamespaces =
+                resolvedViewLitePipeline.getInvolvedNamespaces();
+            involvedNamespacesQueue.insert(involvedNamespacesQueue.end(),
+                                           resolvedViewInvolvedNamespaces.begin(),
+                                           resolvedViewInvolvedNamespaces.end());
         } else {
             // 'involvedNs' is neither a view nor a collection, so resolve it as an empty pipeline
             // to treat it as reading from a non-existent collection.
@@ -372,7 +348,7 @@ Status collatorCompatibleWithPipeline(OperationContext* opCtx,
         return Status::OK();
     }
     for (auto&& potentialViewNs : liteParsedPipeline.getInvolvedNamespaces()) {
-        if (db->getCollection(opCtx, potentialViewNs)) {
+        if (CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, potentialViewNs)) {
             continue;
         }
 
@@ -389,23 +365,6 @@ Status collatorCompatibleWithPipeline(OperationContext* opCtx,
     return Status::OK();
 }
 
-/**
- * Resolves the collator to either the user-specified collation or, if none was specified, to the
- * collection-default collation.
- */
-std::unique_ptr<CollatorInterface> resolveCollator(OperationContext* opCtx,
-                                                   const AggregationRequest& request,
-                                                   const Collection* collection) {
-    if (!request.getCollation().isEmpty()) {
-        return uassertStatusOK(CollatorFactoryInterface::get(opCtx->getServiceContext())
-                                   ->makeFromBSON(request.getCollation()));
-    }
-
-    return (collection && collection->getDefaultCollator()
-                ? collection->getDefaultCollator()->clone()
-                : nullptr);
-}
-
 boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
     OperationContext* opCtx,
     const AggregationRequest& request,
@@ -453,8 +412,9 @@ void _adjustChangeStreamReadConcern(OperationContext* opCtx) {
     }
 
     // Wait for read concern again since we changed the original read concern.
-    uassertStatusOK(waitForReadConcern(
-        opCtx, readConcernArgs, true, PrepareConflictBehavior::kIgnoreConflicts));
+    uassertStatusOK(waitForReadConcern(opCtx, readConcernArgs, true));
+    setPrepareConflictBehaviorForReadConcern(
+        opCtx, readConcernArgs, PrepareConflictBehavior::kIgnoreConflicts);
 }
 
 /**
@@ -489,7 +449,7 @@ std::vector<std::unique_ptr<Pipeline, PipelineDeleter>> createExchangePipelinesI
             // DocumentSourceExchange.
             boost::intrusive_ptr<DocumentSource> consumer = new DocumentSourceExchange(
                 expCtx, exchange, idx, expCtx->mongoProcessInterface->getResourceYielder());
-            pipelines.emplace_back(uassertStatusOK(Pipeline::create({consumer}, expCtx)));
+            pipelines.emplace_back(Pipeline::create({consumer}, expCtx));
         }
     } else {
         pipelines.emplace_back(std::move(pipeline));
@@ -506,25 +466,37 @@ std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> createOuterPipelineProxyExe
     const NamespaceString& nss,
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline,
     bool hasChangeStream) {
+    boost::intrusive_ptr<ExpressionContext> expCtx(pipeline->getContext());
+
     // Transfer ownership of the Pipeline to the PipelineProxyStage.
-    auto ws = make_unique<WorkingSet>();
+    auto ws = std::make_unique<WorkingSet>();
     auto proxy = hasChangeStream
-        ? make_unique<ChangeStreamProxyStage>(opCtx, std::move(pipeline), ws.get())
-        : make_unique<PipelineProxyStage>(opCtx, std::move(pipeline), ws.get());
+        ? std::make_unique<ChangeStreamProxyStage>(expCtx.get(), std::move(pipeline), ws.get())
+        : std::make_unique<PipelineProxyStage>(expCtx.get(), std::move(pipeline), ws.get());
 
     // This PlanExecutor will simply forward requests to the Pipeline, so does not need
     // to yield or to be registered with any collection's CursorManager to receive
     // invalidations. The Pipeline may contain PlanExecutors which *are* yielding
     // PlanExecutors and which *are* registered with their respective collection's
     // CursorManager
-    return uassertStatusOK(
-        PlanExecutor::make(opCtx, std::move(ws), std::move(proxy), nss, PlanExecutor::NO_YIELD));
+    return uassertStatusOK(PlanExecutor::make(
+        std::move(expCtx), std::move(ws), std::move(proxy), nullptr, PlanExecutor::NO_YIELD, nss));
 }
 }  // namespace
 
+Status runAggregate(OperationContext* opCtx,
+                    const NamespaceString& nss,
+                    const AggregationRequest& request,
+                    const BSONObj& cmdObj,
+                    const PrivilegeVector& privileges,
+                    rpc::ReplyBuilderInterface* result) {
+    return runAggregate(opCtx, nss, request, {request}, cmdObj, privileges, result);
+}
+
 Status runAggregate(OperationContext* opCtx,
                     const NamespaceString& origNss,
                     const AggregationRequest& request,
+                    const LiteParsedPipeline& liteParsedPipeline,
                     const BSONObj& cmdObj,
                     const PrivilegeVector& privileges,
                     rpc::ReplyBuilderInterface* result) {
@@ -545,22 +517,12 @@ Status runAggregate(OperationContext* opCtx,
 
     std::vector<unique_ptr<PlanExecutor, PlanExecutor::Deleter>> execs;
     boost::intrusive_ptr<ExpressionContext> expCtx;
-    const LiteParsedPipeline liteParsedPipeline(request);
     auto curOp = CurOp::get(opCtx);
     {
-
-        try {
-            // Check whether the parsed pipeline supports the given read concern.
-            liteParsedPipeline.assertSupportsReadConcern(
-                opCtx, request.getExplain(), serverGlobalParams.enableMajorityReadConcern);
-        } catch (const DBException& ex) {
-            // If we are in a multi-document transaction, we intercept the 'readConcern'
-            // assertion in order to provide a more descriptive error message and code.
-            if (opCtx->inMultiDocumentTransaction()) {
-                return {ErrorCodes::OperationNotSupportedInTransaction,
-                        ex.toStatus("Operation not permitted in transaction").reason()};
-            }
-            return ex.toStatus();
+        // If we are in a transaction, check whether the parsed pipeline supports
+        // being in a transaction.
+        if (opCtx->inMultiDocumentTransaction()) {
+            liteParsedPipeline.assertSupportsMultiDocumentTransaction(request.getExplain());
         }
 
         const auto& pipelineInvolvedNamespaces = liteParsedPipeline.getInvolvedNamespaces();
@@ -586,7 +548,8 @@ Status runAggregate(OperationContext* opCtx,
             // If the user specified an explicit collation, adopt it; otherwise, use the simple
             // collation. We do not inherit the collection's default collation or UUID, since
             // the stream may be resuming from a point before the current UUID existed.
-            collatorToUse.emplace(resolveCollator(opCtx, request, nullptr));
+            collatorToUse.emplace(
+                PipelineD::resolveCollator(opCtx, request.getCollation(), nullptr));
 
             // Obtain collection locks on the execution namespace; that is, the oplog.
             ctx.emplace(opCtx, nss, AutoGetCollection::ViewMode::kViewsForbidden);
@@ -597,12 +560,16 @@ Status runAggregate(OperationContext* opCtx,
                                  Top::LockType::NotLocked,
                                  AutoStatsTracker::LogMode::kUpdateTopAndCurOp,
                                  0);
-            collatorToUse.emplace(resolveCollator(opCtx, request, nullptr));
+            collatorToUse.emplace(
+                PipelineD::resolveCollator(opCtx, request.getCollation(), nullptr));
         } else {
             // This is a regular aggregation. Lock the collection or view.
             ctx.emplace(opCtx, nss, AutoGetCollection::ViewMode::kViewsPermitted);
-            collatorToUse.emplace(resolveCollator(opCtx, request, ctx->getCollection()));
-            uuid = ctx->getCollection() ? ctx->getCollection()->uuid() : boost::none;
+            collatorToUse.emplace(
+                PipelineD::resolveCollator(opCtx, request.getCollation(), ctx->getCollection()));
+            if (ctx->getCollection()) {
+                uuid = ctx->getCollection()->uuid();
+            }
         }
 
         Collection* collection = ctx ? ctx->getCollection() : nullptr;
@@ -655,14 +622,32 @@ Status runAggregate(OperationContext* opCtx,
         invariant(collatorToUse);
         expCtx = makeExpressionContext(opCtx, request, std::move(*collatorToUse), uuid);
 
-        expCtx->startExpressionCounters();
-        auto pipeline = uassertStatusOK(Pipeline::parse(request.getPipeline(), expCtx));
-        expCtx->stopExpressionCounters();
-
-        if (request.shouldAllowDiskUse()) {
-            allowDiskUseCounter.increment();
+        // If this is a shard server, we need to use the correct sort key format for the mongoS that
+        // generated this query. We determine the version by checking for the "use44SortKeys" flag
+        // in the aggregation request.
+        // TODO (SERVER-43361): This check will be unnecessary after branching for 4.5.
+        if (expCtx->fromMongos) {
+            if (request.getUse44SortKeys()) {
+                // This request originated with 4.4-or-newer mongoS, which can understand the new
+                // sort key format. Note: it's possible that merging will actually occur on a mongoD
+                // (for pipelines that merge on a shard), but if the mongoS is 4.4 or newer, all
+                // shard servers must also be 4.4 or newer.
+                expCtx->sortKeyFormat = SortKeyFormat::k44SortKey;
+            } else {
+                // This request originated with an older mongoS that will not understand the new
+                // sort key format. We must use the older format, which differs depending on whether
+                // or not the pipeline is a change stream. Non-$changeStream pipelines may still
+                // merge on a mongoD, but a 4.4 mongoD can still understand the 4.2 sort key format.
+                expCtx->sortKeyFormat = liteParsedPipeline.hasChangeStream()
+                    ? SortKeyFormat::k42ChangeStreamSortKey
+                    : SortKeyFormat::k42SortKey;
+            }
+        } else {
+            // Use default value for the ExpressionContext's 'sortKeyFormat' member variable.
         }
 
+        auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
+
         // Check that the view's collation matches the collation of any views involved in the
         // pipeline.
         if (!pipelineInvolvedNamespaces.empty()) {
@@ -716,10 +701,6 @@ Status runAggregate(OperationContext* opCtx,
                                                           std::move(attachExecutorCallback.second),
                                                           pipeline.get());
 
-            // Optimize again, since there may be additional optimizations that can be done after
-            // adding the initial cursor stage.
-            pipeline->optimizePipeline();
-
             auto pipelines =
                 createExchangePipelinesIfNeeded(opCtx, expCtx, request, std::move(pipeline), uuid);
             for (auto&& pipelineIt : pipelines) {
@@ -773,7 +754,8 @@ Status runAggregate(OperationContext* opCtx,
             repl::ReadConcernArgs::get(opCtx),
             cmdObj,
             lockPolicy,
-            privileges);
+            privileges,
+            expCtx->needsMerge);
         if (expCtx->tailableMode == TailableModeEnum::kTailable) {
             cursorParams.setTailable(true);
         } else if (expCtx->tailableMode == TailableModeEnum::kTailableAndAwaitData) {
@@ -826,7 +808,7 @@ Status runAggregate(OperationContext* opCtx,
         // For an optimized away pipeline, signal the cache that a query operation has completed.
         // For normal pipelines this is done in DocumentSourceCursor.
         if (ctx && ctx->getCollection()) {
-            ctx->getCollection()->infoCache()->notifyOfQuery(opCtx, stats.indexesUsed);
+            CollectionQueryInfo::get(ctx->getCollection()).notifyOfQuery(opCtx, stats);
         }
     }
 
