@@ -27,23 +27,20 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
 #include <string>
 
 #include "mongo/db/commands/shutdown.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/transaction_coordinator_service.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
-namespace {
-MONGO_FAIL_POINT_DEFINE(hangInShutdownBeforeStepdown);
-}  // namespace
-
 Status stepDownForShutdown(OperationContext* opCtx,
                            const Milliseconds& waitTime,
                            bool forceShutdown) noexcept {
@@ -52,20 +49,15 @@ Status stepDownForShutdown(OperationContext* opCtx,
     // for any secondaries. Ignore stepdown.
     if (replCoord->getConfig().getNumMembers() != 1) {
         try {
-            if (MONGO_FAIL_POINT(hangInShutdownBeforeStepdown)) {
-                log() << "hangInShutdownBeforeStepdown failpoint enabled";
-                MONGO_FAIL_POINT_PAUSE_WHILE_SET(hangInShutdownBeforeStepdown);
-            }
-
             replCoord->stepDown(opCtx, false /* force */, waitTime, Seconds(120));
-        } catch (const ExceptionFor<ErrorCodes::NotWritablePrimary>&) {
-            // Ignore NotWritablePrimary errors.
+        } catch (const ExceptionFor<ErrorCodes::NotMaster>&) {
+            // Ignore not master errors.
         } catch (const DBException& e) {
             if (!forceShutdown) {
                 return e.toStatus();
             }
             // Ignore stepDown errors on force shutdown.
-            log() << "Error stepping down during force shutdown " << e.toStatus();
+            LOGV2_WARNING(4719000, "Error stepping down during force shutdown", "error"_attr = e);
         }
 
         // Even if the ReplicationCoordinator failed to step down, ensure we still shut down the
@@ -99,6 +91,19 @@ public:
             timeoutSecs = cmdObj["timeoutSecs"].numberLong();
         }
 
+        // This code may race with a new index build starting up. We may get 0 active index builds
+        // from the IndexBuildsCoordinator shutdown to proceed, but there is nothing to prevent a
+        // new index build from starting after that check.
+        if (!force) {
+            auto indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
+            auto numIndexBuilds = indexBuildsCoord->getActiveIndexBuildCount(opCtx);
+            uassert(ErrorCodes::ConflictingOperationInProgress,
+                    str::stream() << "Index builds in progress while processing shutdown command "
+                                     "without {force: true}: "
+                                  << numIndexBuilds,
+                    numIndexBuilds == 0U);
+        }
+
         uassertStatusOK(stepDownForShutdown(opCtx, Seconds(timeoutSecs), force));
 
         // Never returns
