@@ -27,43 +27,51 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/catalog/collection.h"
-
+#include "mongo/db/catalog/collection_validation.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/db_raii.h"
 #include "mongo/db/query/internal_plans.h"
 #include "mongo/db/storage/durable_catalog.h"
 #include "mongo/db/storage/record_store.h"
-#include "mongo/db/views/view_catalog.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
-using std::endl;
-using std::string;
-using std::stringstream;
-
+// Sets the 'valid' result field to false and returns immediately.
 MONGO_FAIL_POINT_DEFINE(validateCmdCollectionNotValid);
 
 namespace {
 
-// Protects `_validationQueue`
+// Protects the state below.
 Mutex _validationMutex;
 
-// Wakes up `_validationQueue`
+// Holds the set of full `databaseName.collectionName` namespace strings in progress. Validation
+// commands register themselves in this data structure so that subsequent commands on the same
+// namespace will wait rather than run in parallel.
+std::set<std::string> _validationsInProgress;
+
+// This is waited upon if there is found to already be a validation command running on the targeted
+// namespace, as _validationsInProgress would indicate. This is signaled when a validation command
+// finishes on any namespace.
 stdx::condition_variable _validationNotifier;
 
-// Holds the set of full `database.collections` namespace strings in progress.
-std::set<std::string> _validationsInProgress;
 }  // namespace
 
+/**
+ * Example validate command:
+ *   {
+ *       validate: "collectionNameWithoutTheDBPart",
+ *       full: <bool>  // If true, a more thorough (and slower) collection validation is performed.
+ *       background: <bool>  // If true, performs validation on the checkpoint of the collection.
+ *   }
+ */
 class ValidateCmd : public BasicCommand {
 public:
     ValidateCmd() : BasicCommand("validate") {}
@@ -73,9 +81,11 @@ public:
     }
 
     std::string help() const override {
-        return "Validate contents of a namespace by scanning its data structures for correctness.  "
-               "Slow.\n"
-               "Add full:true option to do a more thorough check";
+        return str::stream() << "Validate contents of a namespace by scanning its data structures "
+                             << "for correctness.\nThis is a slow operation.\n"
+                             << "\tAdd {full: true} option to do a more thorough check.\n"
+                             << "\tAdd {background: true} to validate in the background.\n"
+                             << "Cannot specify both {full: true, background: true}.";
     }
 
     virtual bool supportsWriteConcern(const BSONObj& cmd) const override {
@@ -101,56 +111,44 @@ public:
         actions.addAction(ActionType::validate);
         out->push_back(Privilege(parseResourcePattern(dbname, cmdObj), actions));
     }
-    //{ validate: "collectionnamewithoutthedbpart" [, full: <bool> } */
 
     bool run(OperationContext* opCtx,
-             const string& dbname,
+             const std::string& dbname,
              const BSONObj& cmdObj,
              BSONObjBuilder& result) {
-        if (MONGO_FAIL_POINT(validateCmdCollectionNotValid)) {
+        if (MONGO_unlikely(validateCmdCollectionNotValid.shouldFail())) {
             result.appendBool("valid", false);
             return true;
         }
 
         const NamespaceString nss(CommandHelpers::parseNsCollectionRequired(dbname, cmdObj));
+        bool background = cmdObj["background"].trueValue();
 
-        const bool full = cmdObj["full"].trueValue();
-
-        ValidateCmdLevel level = kValidateNormal;
-
-        if (full) {
-            level = kValidateFull;
+        // Background validation is not supported on the ephemeralForTest storage engine due to its
+        // lack of support for timestamps. Switch the mode to foreground validation instead.
+        if (background && storageGlobalParams.engine == "ephemeralForTest") {
+            LOGV2(4775400,
+                  "ephemeralForTest does not support background validation, switching to "
+                  "foreground validation");
+            background = false;
         }
 
-        if (cmdObj["scandata"]) {
-            const char* deprecationWarning =
-                "the scandata option is deprecated and will be removed in a future release";
-            result.append("note", deprecationWarning);
-        }
-
-        if (!nss.isNormal() && full) {
-            uasserted(ErrorCodes::CommandFailed,
-                      "Can only run full validate on a regular collection");
+        const bool fullValidate = cmdObj["full"].trueValue();
+        if (background && fullValidate) {
+            uasserted(ErrorCodes::CommandNotSupported,
+                      str::stream() << "Running the validate command with both { background: true }"
+                                    << " and { full: true } is not supported.");
         }
 
         if (!serverGlobalParams.quiet.load()) {
-            LOG(0) << "CMD: validate " << nss.ns();
+            LOGV2(20514,
+                  "CMD: validate",
+                  "namespace"_attr = nss,
+                  "background"_attr = background,
+                  "full"_attr = fullValidate);
         }
 
-        AutoGetDb ctx(opCtx, nss.db(), MODE_IX);
-        Lock::CollectionLock collLk(opCtx, nss, MODE_X);
-        Collection* collection = ctx.getDb() ? ctx.getDb()->getCollection(opCtx, nss) : NULL;
-        if (!collection) {
-            if (ctx.getDb() && ViewCatalog::get(ctx.getDb())->lookup(opCtx, nss.ns())) {
-                uasserted(ErrorCodes::CommandNotSupportedOnView, "Cannot validate a view");
-            }
-
-            uasserted(ErrorCodes::NamespaceNotFound, "ns not found");
-        }
-
-        result.append("ns", nss.ns());
-
-        // Only one validation per collection can be in progress, the rest wait in order.
+        // Only one validation per collection can be in progress, the rest wait.
         {
             stdx::unique_lock<Latch> lock(_validationMutex);
             try {
@@ -161,7 +159,7 @@ public:
                 CommandHelpers::appendCommandStatusNoThrow(
                     result,
                     {ErrorCodes::CommandFailed,
-                     str::stream() << "Exception during validation: " << e.toString()});
+                     str::stream() << "Exception thrown during validation: " << e.toString()});
                 return false;
             }
 
@@ -174,50 +172,26 @@ public:
             _validationNotifier.notify_all();
         });
 
-        // TODO SERVER-30357: Add support for background validation.
-        const bool background = false;
+        auto options = (fullValidate) ? CollectionValidation::ValidateOptions::kFullValidation
+                                      : CollectionValidation::ValidateOptions::kNoFullValidation;
 
-        ValidateResults results;
-        Status status = collection->validate(opCtx, level, background, &results, &result);
+        ValidateResults validateResults;
+        Status status = CollectionValidation::validate(
+            opCtx, nss, options, background, &validateResults, &result);
         if (!status.isOK()) {
             return CommandHelpers::appendCommandStatusNoThrow(result, status);
         }
 
-        CollectionOptions opts =
-            DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, collection->ns());
-
-        // All collections must have a UUID.
-        if (!opts.uuid) {
-            results.errors.push_back(str::stream() << "UUID missing on collection " << nss.ns()
-                                                   << " but SchemaVersion=3.6");
-            results.valid = false;
+        result.appendBool("valid", validateResults.valid);
+        if (validateResults.readTimestamp) {
+            result.append("readTimestamp", validateResults.readTimestamp.get());
         }
+        result.append("warnings", validateResults.warnings);
+        result.append("errors", validateResults.errors);
+        result.append("extraIndexEntries", validateResults.extraIndexEntries);
+        result.append("missingIndexEntries", validateResults.missingIndexEntries);
 
-        if (!full) {
-            results.warnings.push_back(
-                "Some checks omitted for speed. use {full:true} option to do more thorough scan.");
-        }
-        result.appendBool("valid", results.valid);
-
-        static constexpr std::size_t kMaxErrorWarningSizeBytes = 2 * 1024 * 1024;
-        auto appendRangeSizeLimited = [&result](StringData fieldName, const auto& values) {
-            std::size_t usedSize = 0;
-            BSONArrayBuilder arr(result.subarrayStart(fieldName));
-            for (auto it = values.begin(), end = values.end();
-                 it != end && usedSize < kMaxErrorWarningSizeBytes;
-                 ++it) {
-                arr.append(*it);
-                usedSize += it->size();
-            }
-        };
-
-        appendRangeSizeLimited("warnings"_sd, results.warnings);
-        appendRangeSizeLimited("errors"_sd, results.errors);
-
-        result.append("extraIndexEntries", results.extraIndexEntries);
-        result.append("missingIndexEntries", results.missingIndexEntries);
-
-        if (!results.valid) {
+        if (!validateResults.valid) {
             result.append("advice",
                           "A corrupt namespace has been detected. See "
                           "http://dochub.mongodb.org/core/data-recovery for recovery steps.");
