@@ -28,19 +28,20 @@
  */
 
 #include "mongo/base/init.h"
+#include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/mutable/document.h"
 #include "mongo/bson/mutable/element.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/client.h"
 #include "mongo/db/commands.h"
-#include "mongo/db/commands/update_metrics.h"
 #include "mongo/db/commands/write_commands/write_commands_common.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/json.h"
 #include "mongo/db/lasterror.h"
 #include "mongo/db/matcher/extensions_callback_real.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/ops/delete_request.h"
 #include "mongo/db/ops/parsed_delete.h"
 #include "mongo/db/ops/parsed_update.h"
@@ -95,7 +96,8 @@ void serializeReply(OperationContext* opCtx,
     if (continueOnError && !result.results.empty()) {
         const auto& lastResult = result.results.back();
         if (lastResult == ErrorCodes::StaleConfig ||
-            lastResult == ErrorCodes::CannotImplicitlyCreateCollection) {
+            lastResult == ErrorCodes::CannotImplicitlyCreateCollection ||
+            lastResult == ErrorCodes::StaleDbVersion) {
             // For ordered:false commands we need to duplicate these error results for all ops
             // after we stopped. See handleError() in write_ops_exec.cpp for more info.
             auto err = result.results.back();
@@ -243,11 +245,6 @@ private:
         }
     }
 
-    bool supportsReadConcern(repl::ReadConcernLevel level) const final {
-        return level == repl::ReadConcernLevel::kLocalReadConcern ||
-            level == repl::ReadConcernLevel::kSnapshotReadConcern;
-    }
-
     bool supportsWriteConcern() const final {
         return true;
     }
@@ -312,7 +309,7 @@ private:
 
     std::unique_ptr<CommandInvocation> parse(OperationContext*,
                                              const OpMsgRequest& request) override {
-        return stdx::make_unique<Invocation>(this, request);
+        return std::make_unique<Invocation>(this, request);
     }
 
     void snipForLogging(mutablebson::Document* cmdObj) const final {
@@ -326,18 +323,46 @@ private:
 
 class CmdUpdate final : public WriteCommand {
 public:
-    CmdUpdate() : WriteCommand("update"), _updateMetrics{"update"} {}
+    CmdUpdate() : WriteCommand("update") {}
 
 private:
     class Invocation final : public InvocationBase {
     public:
-        Invocation(const WriteCommand* cmd,
-                   const OpMsgRequest& request,
-                   UpdateMetrics* updateMetrics)
-            : InvocationBase(cmd, request),
-              _batch(UpdateOp::parse(request)),
-              _updateMetrics{updateMetrics} {
-            invariant(_updateMetrics);
+        Invocation(const WriteCommand* cmd, const OpMsgRequest& request)
+            : InvocationBase(cmd, request), _batch(UpdateOp::parse(request)) {}
+
+        bool supportsReadMirroring() const override {
+            // TODO SERVER-46533 Remove this and uncomment below when update can be safely mirrored.
+            // Disable readMIrrroing for update
+            return false;
+
+            // This would translate to a find command with no filter!
+            // Based on the documentation for `update`, this vector should never be empty.
+            // return !_batch.getUpdates().empty();
+        }
+
+        void appendMirrorableRequest(BSONObjBuilder* bob) const override {
+            auto extractQueryDetails = [](const write_ops::Update& query,
+                                          BSONObjBuilder* bob) -> void {
+                auto updates = query.getUpdates();
+                // `supportsReadMirroring()` is responsible for this validation.
+                invariant(!updates.empty());
+
+                // Current design ignores contents of `updates` array except for the first entry.
+                // Assuming identical collation for all elements in `updates`, future design could
+                // use the disjunction primitive (i.e, `$or`) to compile all queries into a single
+                // filter. Such a design also requires a sound way of combining hints.
+                bob->append("filter", updates.front().getQ());
+                if (!updates.front().getHint().isEmpty())
+                    bob->append("hint", updates.front().getHint());
+                if (updates.front().getCollation())
+                    bob->append("collation", *updates.front().getCollation());
+            };
+
+            bob->append("find", _batch.getNamespace().coll());
+            extractQueryDetails(_batch, bob);
+            bob->append("batchSize", 1);
+            bob->append("singleBatch", true);
         }
 
     private:
@@ -358,23 +383,14 @@ private:
                            std::move(reply),
                            &result);
 
-            // Collect metrics.
-            auto updates = _batch.getUpdates();
-            for (auto&& update : updates) {
-                // If this was a pipeline style update, record that pipeline-style was used and
-                // which stages were being used.
+            // If this was a pipeline style update, record which stages were being used.
+            for (auto&& update : _batch.getUpdates()) {
                 auto& updateMod = update.getU();
                 if (updateMod.type() == write_ops::UpdateModification::Type::kPipeline) {
                     AggregationRequest request(_batch.getNamespace(),
                                                updateMod.getUpdatePipeline());
                     LiteParsedPipeline pipeline(request);
                     pipeline.tickGlobalStageCounters();
-                    _updateMetrics->incrementExecutedWithAggregationPipeline();
-                }
-
-                // If this command had arrayFilters option, record that it was used.
-                if (update.getArrayFilters()) {
-                    _updateMetrics->incrementExecutedWithArrayFilters();
                 }
             }
         }
@@ -411,20 +427,17 @@ private:
             AutoGetCollection collection(opCtx, _batch.getNamespace(), MODE_IX);
 
             auto exec = uassertStatusOK(getExecutorUpdate(
-                opCtx, &CurOp::get(opCtx)->debug(), collection.getCollection(), &parsedUpdate));
+                &CurOp::get(opCtx)->debug(), collection.getCollection(), &parsedUpdate, verbosity));
             auto bodyBuilder = result->getBodyBuilder();
             Explain::explainStages(
                 exec.get(), collection.getCollection(), verbosity, BSONObj(), &bodyBuilder);
         }
 
         write_ops::Update _batch;
-
-        // Update related command execution metrics.
-        UpdateMetrics* const _updateMetrics;
     };
 
     std::unique_ptr<CommandInvocation> parse(OperationContext*, const OpMsgRequest& request) {
-        return stdx::make_unique<Invocation>(this, request, &_updateMetrics);
+        return std::make_unique<Invocation>(this, request);
     }
 
     void snipForLogging(mutablebson::Document* cmdObj) const final {
@@ -434,9 +447,6 @@ private:
     std::string help() const final {
         return "update documents";
     }
-
-    // Update related command execution metrics.
-    UpdateMetrics _updateMetrics;
 } cmdUpdate;
 
 class CmdDelete final : public WriteCommand {
@@ -480,6 +490,7 @@ private:
             deleteRequest.setCollation(write_ops::collationOf(_batch.getDeletes()[0]));
             deleteRequest.setMulti(_batch.getDeletes()[0].getMulti());
             deleteRequest.setYieldPolicy(PlanExecutor::YIELD_AUTO);
+            deleteRequest.setHint(_batch.getDeletes()[0].getHint());
             deleteRequest.setExplain();
 
             ParsedDelete parsedDelete(opCtx, &deleteRequest);
@@ -491,7 +502,7 @@ private:
 
             // Explain the plan tree.
             auto exec = uassertStatusOK(getExecutorDelete(
-                opCtx, &CurOp::get(opCtx)->debug(), collection.getCollection(), &parsedDelete));
+                &CurOp::get(opCtx)->debug(), collection.getCollection(), &parsedDelete, verbosity));
             auto bodyBuilder = result->getBodyBuilder();
             Explain::explainStages(
                 exec.get(), collection.getCollection(), verbosity, BSONObj(), &bodyBuilder);
@@ -502,7 +513,7 @@ private:
 
     std::unique_ptr<CommandInvocation> parse(OperationContext*,
                                              const OpMsgRequest& request) override {
-        return stdx::make_unique<Invocation>(this, request);
+        return std::make_unique<Invocation>(this, request);
     }
 
     void snipForLogging(mutablebson::Document* cmdObj) const final {
