@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -39,15 +39,15 @@
 #include "mongo/db/concurrency/lock_manager_test_help.h"
 #include "mongo/db/concurrency/locker.h"
 #include "mongo/db/service_context_test_fixture.h"
-#include "mongo/unittest/death_test.h"
+#include "mongo/transport/session.h"
+#include "mongo/transport/transport_layer_mock.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/timer.h"
 
 namespace mongo {
 
-class LockerImplTest : public unittest::Test, public ScopedGlobalServiceContextForTest {};
+class LockerImplTest : public ServiceContextTest {};
 
 TEST_F(LockerImplTest, LockNoConflict) {
     const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
@@ -94,8 +94,11 @@ TEST_F(LockerImplTest, ConflictWithTimeout) {
 
     LockerImpl locker2;
     locker2.lockGlobal(MODE_IX);
-    ASSERT_THROWS_CODE(
-        locker2.lock(resId, MODE_S, Date_t::now()), AssertionException, ErrorCodes::LockTimeout);
+
+    auto opCtx = makeOperationContext();
+    ASSERT_THROWS_CODE(locker2.lock(opCtx.get(), resId, MODE_S, Date_t::now()),
+                       AssertionException,
+                       ErrorCodes::LockTimeout);
 
     ASSERT(locker2.getLockMode(resId) == MODE_NONE);
 
@@ -117,7 +120,8 @@ TEST_F(LockerImplTest, ConflictUpgradeWithTimeout) {
     locker2.lock(resId, MODE_S);
 
     // Try upgrading locker 1, which should block and timeout
-    ASSERT_THROWS_CODE(locker1.lock(resId, MODE_X, Date_t::now() + Milliseconds(1)),
+    auto opCtx = makeOperationContext();
+    ASSERT_THROWS_CODE(locker1.lock(opCtx.get(), resId, MODE_X, Date_t::now() + Milliseconds(1)),
                        AssertionException,
                        ErrorCodes::LockTimeout);
 
@@ -126,6 +130,13 @@ TEST_F(LockerImplTest, ConflictUpgradeWithTimeout) {
 }
 
 TEST_F(LockerImplTest, FailPointInLockFailsGlobalNonIntentLocksIfTheyCannotBeImmediatelyGranted) {
+    transport::TransportLayerMock transportLayer;
+    transport::SessionHandle session = transportLayer.createSession();
+
+    auto newClient = getServiceContext()->makeClient("userClient", session);
+    AlternativeClientRegion acr(newClient);
+    auto newOpCtx = cc().makeOperationContext();
+
     LockerImpl locker1;
     locker1.lockGlobal(MODE_IX);
 
@@ -134,32 +145,42 @@ TEST_F(LockerImplTest, FailPointInLockFailsGlobalNonIntentLocksIfTheyCannotBeImm
 
         // MODE_S attempt.
         LockerImpl locker2;
-        ASSERT_THROWS_CODE(locker2.lockGlobal(MODE_S), DBException, ErrorCodes::LockTimeout);
+        ASSERT_THROWS_CODE(
+            locker2.lockGlobal(newOpCtx.get(), MODE_S), DBException, ErrorCodes::LockTimeout);
 
         // MODE_X attempt.
         LockerImpl locker3;
-        ASSERT_THROWS_CODE(locker3.lockGlobal(MODE_X), DBException, ErrorCodes::LockTimeout);
+        ASSERT_THROWS_CODE(
+            locker3.lockGlobal(newOpCtx.get(), MODE_X), DBException, ErrorCodes::LockTimeout);
     }
 
     locker1.unlockGlobal();
 }
 
 TEST_F(LockerImplTest, FailPointInLockFailsNonIntentLocksIfTheyCannotBeImmediatelyGranted) {
+    transport::TransportLayerMock transportLayer;
+    transport::SessionHandle session = transportLayer.createSession();
+
+    auto newClient = getServiceContext()->makeClient("userClient", session);
+    AlternativeClientRegion acr(newClient);
+    auto newOpCtx = cc().makeOperationContext();
+
     // Granted MODE_X lock, fail incoming MODE_S and MODE_X.
     const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
 
     LockerImpl locker1;
-    locker1.lockGlobal(MODE_IX);
-    locker1.lock(resId, MODE_X);
+    locker1.lockGlobal(newOpCtx.get(), MODE_IX);
+    locker1.lock(newOpCtx.get(), resId, MODE_X);
 
     {
         FailPointEnableBlock failWaitingNonPartitionedLocks("failNonIntentLocksIfWaitNeeded");
 
         // MODE_S attempt.
         LockerImpl locker2;
-        locker2.lockGlobal(MODE_IS);
-        ASSERT_THROWS_CODE(
-            locker2.lock(resId, MODE_S, Date_t::max()), DBException, ErrorCodes::LockTimeout);
+        locker2.lockGlobal(newOpCtx.get(), MODE_IS);
+        ASSERT_THROWS_CODE(locker2.lock(newOpCtx.get(), resId, MODE_S, Date_t::max()),
+                           DBException,
+                           ErrorCodes::LockTimeout);
 
         // The timed out MODE_S attempt shouldn't be present in the map of lock requests because it
         // won't ever be granted.
@@ -168,9 +189,10 @@ TEST_F(LockerImplTest, FailPointInLockFailsNonIntentLocksIfTheyCannotBeImmediate
 
         // MODE_X attempt.
         LockerImpl locker3;
-        locker3.lockGlobal(MODE_IX);
-        ASSERT_THROWS_CODE(
-            locker3.lock(resId, MODE_X, Date_t::max()), DBException, ErrorCodes::LockTimeout);
+        locker3.lockGlobal(newOpCtx.get(), MODE_IX);
+        ASSERT_THROWS_CODE(locker3.lock(newOpCtx.get(), resId, MODE_X, Date_t::max()),
+                           DBException,
+                           ErrorCodes::LockTimeout);
 
         // The timed out MODE_X attempt shouldn't be present in the map of lock requests because it
         // won't ever be granted.
@@ -880,7 +902,8 @@ TEST_F(LockerImplTest, OverrideLockRequestTimeout) {
 
     // locker2's attempt to acquire FirstDB with unlimited wait time should timeout after 1000
     // milliseconds and throw because _maxLockRequestTimeout is set to 1000 milliseconds.
-    ASSERT_THROWS_CODE(locker2.lock(resIdFirstDB, MODE_X, Date_t::max()),
+    auto opCtx = makeOperationContext();
+    ASSERT_THROWS_CODE(locker2.lock(opCtx.get(), resIdFirstDB, MODE_X, Date_t::max()),
                        AssertionException,
                        ErrorCodes::LockTimeout);
 
@@ -916,7 +939,8 @@ TEST_F(LockerImplTest, DoNotWaitForLockAcquisition) {
 
     // locker2's attempt to acquire FirstDB with unlimited wait time should fail immediately and
     // throw because _maxLockRequestTimeout was set to 0.
-    ASSERT_THROWS_CODE(locker2.lock(resIdFirstDB, MODE_X, Date_t::max()),
+    auto opCtx = makeOperationContext();
+    ASSERT_THROWS_CODE(locker2.lock(opCtx.get(), resIdFirstDB, MODE_X, Date_t::max()),
                        AssertionException,
                        ErrorCodes::LockTimeout);
 
@@ -987,7 +1011,8 @@ TEST_F(LockerImplTest, GetLockerInfoShouldReportPendingLocks) {
     LockerImpl conflictingLocker;
     conflictingLocker.lockGlobal(MODE_IS);
     conflictingLocker.lock(dbId, MODE_IS);
-    ASSERT_EQ(LOCK_WAITING, conflictingLocker.lockBegin(nullptr, collectionId, MODE_IS));
+    ASSERT_EQ(LOCK_WAITING,
+              conflictingLocker.lockBeginForTest(nullptr /* opCtx */, collectionId, MODE_IS));
 
     // Assert the held locks show up in the output of getLockerInfo().
     Locker::LockerInfo lockerInfo;
@@ -1005,7 +1030,8 @@ TEST_F(LockerImplTest, GetLockerInfoShouldReportPendingLocks) {
     ASSERT(successfulLocker.unlock(dbId));
     ASSERT(successfulLocker.unlockGlobal());
 
-    conflictingLocker.lockComplete(collectionId, MODE_IS, Date_t::now());
+    conflictingLocker.lockCompleteForTest(
+        nullptr /* opCtx */, collectionId, MODE_IS, Date_t::now());
 
     conflictingLocker.getLockerInfo(&lockerInfo, boost::none);
     ASSERT_FALSE(lockerInfo.waitingResource.isValid());
@@ -1144,20 +1170,4 @@ TEST_F(LockerImplTest, ConvertLockPendingUnlockAndUnlock) {
     locker.unlockGlobal();
 }
 
-// This test exercises the lock dumping code in ~LockerImpl in case locks are held on destruction.
-DEATH_TEST_F(LockerImplTest,
-             LocksHeldOnDestructionCausesALocksDump,
-             "Operation ending while holding locks.") {
-    const ResourceId resId(RESOURCE_COLLECTION, "TestDB.collection"_sd);
-
-    LockerImpl locker;
-    locker.lockGlobal(MODE_IX);
-    locker.lock(resId, MODE_X);
-
-    ASSERT(locker.isLockHeldForMode(resId, MODE_X));
-    ASSERT(locker.isLockHeldForMode(resId, MODE_S));
-
-    // 'locker' destructor should invariant because locks are still held.
-}
-
 }  // namespace mongo
