@@ -29,7 +29,7 @@
 
 // CHECK_LOG_REDACTION
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -46,15 +46,19 @@
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/locker.h"
 #include "mongo/db/json.h"
+#include "mongo/db/prepare_conflict_tracker.h"
 #include "mongo/db/query/getmore_request.h"
 #include "mongo/platform/random.h"
 #include "mongo/db/query/plan_summary_stats.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/metadata/client_metadata.h"
+#include "mongo/rpc/metadata/client_metadata_ismaster.h"
 #include "mongo/rpc/metadata/impersonated_user_metadata.h"
 #include "mongo/util/hex.h"
-#include "mongo/util/log.h"
+#include "mongo/util/log_with_sampling.h"
 #include "mongo/util/net/socket_utils.h"
 #include "mongo/util/str.h"
+#include <mongo/db/stats/timer_stats.h>
 
 namespace mongo {
 
@@ -76,6 +80,10 @@ const std::vector<const char*> kDollarQueryModifiers = {
     "$maxTimeMS",
 };
 
+TimerStats oplogGetMoreStats;
+ServerStatusMetricField<TimerStats> displayBatchesReceived("repl.network.oplogGetMoresProcessed",
+                                                           &oplogGetMoreStats);
+
 }  // namespace
 
 BSONObj upconvertQueryEntry(const BSONObj& query,
@@ -248,18 +256,20 @@ void CurOp::reportCurrentOpForClient(OperationContext* opCtx,
     infoBuilder->append("host", hostName);
 
     client->reportState(*infoBuilder);
-    if (auto clientMetadata = ClientMetadata::get(client)) {
-        auto appName = clientMetadata->getApplicationName();
+    const auto& clientMetadata = ClientMetadataIsMasterState::get(client).getClientMetadata();
+
+    if (clientMetadata) {
+        auto appName = clientMetadata.get().getApplicationName();
         if (!appName.empty()) {
             infoBuilder->append("appName", appName);
         }
 
-        auto clientMetadataDocument = clientMetadata->getDocument();
+        auto clientMetadataDocument = clientMetadata.get().getDocument();
         infoBuilder->append("clientMetadata", clientMetadataDocument);
     }
 
     // Fill out the rest of the BSONObj with opCtx specific details.
-    infoBuilder->appendBool("active", static_cast<bool>(clientOpCtx));
+    infoBuilder->appendBool("active", client->hasAnyActiveCurrentOp());
     infoBuilder->append("currentOpTime",
                         opCtx->getServiceContext()->getPreciseClockSource()->now().toString());
 
@@ -290,7 +300,12 @@ void CurOp::reportCurrentOpForClient(OperationContext* opCtx,
     }
 
     if (clientOpCtx) {
-        infoBuilder->append("opid", clientOpCtx->getOpID());
+        infoBuilder->append("opid", static_cast<int>(clientOpCtx->getOpID()));
+
+        if (auto opKey = clientOpCtx->getOperationKey()) {
+            opKey->appendToBuilder(infoBuilder, "operationKey");
+        }
+
         if (clientOpCtx->isKillPending()) {
             infoBuilder->append("killPending", true);
         }
@@ -300,7 +315,7 @@ void CurOp::reportCurrentOpForClient(OperationContext* opCtx,
             lsid->serialize(&lsidBuilder);
         }
 
-        CurOp::get(clientOpCtx)->reportState(infoBuilder, truncateOps);
+        CurOp::get(clientOpCtx)->reportState(clientOpCtx, infoBuilder, truncateOps);
     }
 
 #ifndef MONGO_CONFIG_USE_RAW_LATCHES
@@ -369,7 +384,11 @@ void CurOp::setGenericOpRequestDetails(OperationContext* opCtx,
 
 void CurOp::setMessage_inlock(StringData message) {
     if (_progressMeter.isActive()) {
-        error() << "old _message: " << redact(_message) << " new message:" << redact(message);
+        LOGV2_ERROR(20527,
+                    "Changing message from {old} to {new}",
+                    "Updating message",
+                    "old"_attr = redact(_message),
+                    "new"_attr = redact(message));
         verify(!_progressMeter.isActive());
     }
     _message = message.toString();  // copy
@@ -407,16 +426,12 @@ void CurOp::raiseDbProfileLevel(int dbProfileLevel) {
 }
 
 bool CurOp::completeAndLogOperation(OperationContext* opCtx,
-                                    logger::LogComponent component,
+                                    logv2::LogComponent component,
                                     boost::optional<size_t> responseLength,
                                     boost::optional<long long> slowMsOverride,
                                     bool forceLog) {
-    // Log the operation if it is eligible according to the current slowMS and sampleRate settings.
-    const bool shouldLogOp = (forceLog || shouldLog(component, logger::LogSeverity::Debug(1)));
     const long long slowMs = slowMsOverride.value_or(serverGlobalParams.slowMS);
 
-    const auto client = opCtx->getClient();
-
     // Record the size of the response returned to the client, if applicable.
     if (responseLength) {
         _debug.responseLength = *responseLength;
@@ -426,10 +441,19 @@ bool CurOp::completeAndLogOperation(OperationContext* opCtx,
     _end = curTimeMicros64();
     _debug.executionTimeMicros = durationCount<Microseconds>(elapsedTimeExcludingPauses());
 
-    const bool shouldSample =
-        client->getPrng().nextCanonicalDouble() < serverGlobalParams.sampleRate;
+    const auto executionTimeMillis = _debug.executionTimeMicros / 1000;
 
-    if (shouldLogOp || (shouldSample && _debug.executionTimeMicros > slowMs * 1000LL)) {
+    if (_debug.isReplOplogFetching) {
+        oplogGetMoreStats.recordMillis(executionTimeMillis);
+    }
+
+    bool shouldLogSlowOp, shouldSample;
+
+    // Log the operation if it is eligible according to the current slowMS and sampleRate settings.
+    std::tie(shouldLogSlowOp, shouldSample) = shouldLogSlowOpWithSampling(
+        opCtx, component, Milliseconds(executionTimeMillis), Milliseconds(slowMs));
+
+    if (forceLog || shouldLogSlowOp) {
         auto lockerInfo = opCtx->lockState()->getLockerInfo(_lockStatsBase);
         if (_debug.storageStats == nullptr && opCtx->lockState()->wasGlobalLockTaken() &&
             opCtx->getServiceContext()->getStorageEngine()) {
@@ -451,18 +475,34 @@ bool CurOp::completeAndLogOperation(OperationContext* opCtx,
                 if (lk.isLocked()) {
                     _debug.storageStats = opCtx->recoveryUnit()->getOperationStatistics();
                 } else {
-                    warning(component) << "Unable to gather storage statistics for a slow "
-                                          "operation due to lock aquire timeout";
+                    LOGV2_WARNING_OPTIONS(
+                        20525,
+                        {component},
+                        "Failed to gather storage statistics for {opId} due to {reason}",
+                        "Failed to gather storage statistics for slow operation",
+                        "opId"_attr = opCtx->getOpID(),
+                        "error"_attr = "lock acquire timeout"_sd);
                 }
-            } catch (const ExceptionForCat<ErrorCategory::Interruption>&) {
-                warning(component) << "Unable to gather storage statistics for a slow "
-                                      "operation due to interrupt";
+            } catch (const ExceptionForCat<ErrorCategory::Interruption>& ex) {
+                LOGV2_WARNING_OPTIONS(
+                    20526,
+                    {component},
+                    "Failed to gather storage statistics for {opId} due to {reason}",
+                    "Failed to gather storage statistics for slow operation",
+                    "opId"_attr = opCtx->getOpID(),
+                    "error"_attr = redact(ex));
             }
         }
-        log(component) << _debug.report(client,
-                                        *this,
-                                        (lockerInfo ? &lockerInfo->stats : nullptr),
-                                        opCtx->lockState()->getFlowControlStats());
+
+        // Gets the time spent blocked on prepare conflicts.
+        auto prepareConflictDurationMicros =
+            PrepareConflictTracker::get(opCtx).getPrepareConflictDuration();
+        _debug.prepareConflictDurationMillis =
+            duration_cast<Milliseconds>(prepareConflictDurationMicros);
+
+        logv2::DynamicAttributes attr;
+        _debug.report(opCtx, (lockerInfo ? &lockerInfo->stats : nullptr), &attr);
+        LOGV2_OPTIONS(51803, {component}, "Slow query", attr);
     }
 
     // Return 'true' if this operation should also be added to the profiler.
@@ -488,6 +528,11 @@ Command::ReadWriteType CurOp::getReadWriteType() const {
 
 namespace {
 
+BSONObj appendCommentField(OperationContext* opCtx, const BSONObj& cmdObj) {
+    return opCtx->getComment() && !cmdObj["comment"] ? cmdObj.addField(*opCtx->getComment())
+                                                     : cmdObj;
+}
+
 /**
  * Appends {<name>: obj} to the provided builder.  If obj is greater than maxSize, appends a string
  * summary of obj as { <name>: { $truncated: "obj" } }. If a comment parameter is present, add it to
@@ -561,7 +606,7 @@ BSONObj CurOp::truncateAndSerializeGenericCursor(GenericCursor* cursor,
     return serialized;
 }
 
-void CurOp::reportState(BSONObjBuilder* builder, bool truncateOps) {
+void CurOp::reportState(OperationContext* opCtx, BSONObjBuilder* builder, bool truncateOps) {
     if (_start) {
         builder->append("secs_running", durationCount<Seconds>(elapsedTimeTotal()));
         builder->append("microsecs_running", durationCount<Microseconds>(elapsedTimeTotal()));
@@ -576,7 +621,9 @@ void CurOp::reportState(BSONObjBuilder* builder, bool truncateOps) {
     // is true, limit the size of each op to 1000 bytes. Otherwise, do not truncate.
     const boost::optional<size_t> maxQuerySize{truncateOps, 1000};
 
-    appendAsObjOrString("command", _opDescription, maxQuerySize, builder);
+    appendAsObjOrString(
+        "command", appendCommentField(opCtx, _opDescription), maxQuerySize, builder);
+
 
     if (!_planSummary.empty()) {
         builder->append("planSummary", _planSummary);
@@ -609,11 +656,19 @@ void CurOp::reportState(BSONObjBuilder* builder, bool truncateOps) {
     }
 
     builder->append("numYields", _numYields);
+
+    if (_debug.dataThroughputLastSecond) {
+        builder->append("dataThroughputLastSecond", *_debug.dataThroughputLastSecond);
+    }
+
+    if (_debug.dataThroughputAverage) {
+        builder->append("dataThroughputAverage", *_debug.dataThroughputAverage);
+    }
 }
 
 bool CurOp::_shouldDBProfileWithRateLimit() {
     // Pseudo RNG for rate limiter feature
-    static PseudoRandom _prng(std::unique_ptr<SecureRandom>(SecureRandom::create())->nextInt64());
+    static PseudoRandom _prng(SecureRandom().nextInt64());
 
     const int64_t RATE_LIMIT_MULTIPLIER = 1LL << 52;
     static_assert( RATE_LIMIT_MAX * RATE_LIMIT_MULTIPLIER <= std::numeric_limits<int64_t>::max(),
@@ -659,10 +714,10 @@ StringData getProtoString(int op) {
     if (y)                                   \
     s << " " x ":" << (*y)
 
-string OpDebug::report(Client* client,
-                       const CurOp& curop,
-                       const SingleThreadedLockStats* lockStats,
-                       FlowControlTicketholder::CurOp flowControlStats) const {
+string OpDebug::report(OperationContext* opCtx, const SingleThreadedLockStats* lockStats) const {
+    Client* client = opCtx->getClient();
+    auto& curop = *CurOp::get(opCtx);
+    auto flowControlStats = opCtx->lockState()->getFlowControlStats();
     StringBuilder s;
     if (iscommand)
         s << "command ";
@@ -671,14 +726,15 @@ string OpDebug::report(Client* client,
 
     s << curop.getNS();
 
-    if (auto clientMetadata = ClientMetadata::get(client)) {
-        auto appName = clientMetadata->getApplicationName();
+    const auto& clientMetadata = ClientMetadataIsMasterState::get(client).getClientMetadata();
+    if (clientMetadata) {
+        auto appName = clientMetadata.get().getApplicationName();
         if (!appName.empty()) {
             s << " appName: \"" << str::escape(appName) << '\"';
         }
     }
 
-    auto query = curop.opDescription();
+    auto query = appendCommentField(opCtx, curop.opDescription());
     if (!query.isEmpty()) {
         s << " command: ";
         if (iscommand) {
@@ -709,10 +765,22 @@ string OpDebug::report(Client* client,
         s << " planSummary: " << curop.getPlanSummary().toString();
     }
 
+    if (prepareConflictDurationMillis > Milliseconds::zero()) {
+        s << " prepareConflictDuration: " << prepareConflictDurationMillis;
+    }
+
+    if (dataThroughputLastSecond) {
+        s << " dataThroughputLastSecond: " << *dataThroughputLastSecond << " MB/sec";
+    }
+
+    if (dataThroughputAverage) {
+        s << " dataThroughputAverage: " << *dataThroughputAverage << " MB/sec";
+    }
+
     OPDEBUG_TOSTRING_HELP(nShards);
     OPDEBUG_TOSTRING_HELP(cursorid);
     if (mongotCursorId) {
-        s << " mongot: " << makeSearchBetaObject().toString();
+        s << " mongot: " << makeMongotDebugStatsObject().toString();
     }
     OPDEBUG_TOSTRING_HELP(ntoreturn);
     OPDEBUG_TOSTRING_HELP(ntoskip);
@@ -773,6 +841,17 @@ string OpDebug::report(Client* client,
         s << " flowControl:" << flowControlObj.toString();
     }
 
+    {
+        const auto& readConcern = repl::ReadConcernArgs::get(opCtx);
+        if (readConcern.isSpecified()) {
+            s << " readConcern:" << readConcern.toBSONInner();
+        }
+    }
+
+    if (writeConcern && !writeConcern->usedDefault) {
+        s << " writeConcern:" << writeConcern->toBSON();
+    }
+
     if (storageStats) {
         s << " storage:" << storageStats->toBSON().toString();
     }
@@ -790,6 +869,174 @@ string OpDebug::report(Client* client,
     return s.str();
 }
 
+#define OPDEBUG_TOATTR_HELP(x) \
+    if (x >= 0)                \
+    pAttrs->add(#x, x)
+#define OPDEBUG_TOATTR_HELP_BOOL(x) \
+    if (x)                          \
+    pAttrs->add(#x, x)
+#define OPDEBUG_TOATTR_HELP_ATOMIC(x, y) \
+    if (auto __y = y.load(); __y > 0)    \
+    pAttrs->add(x, __y)
+#define OPDEBUG_TOATTR_HELP_OPTIONAL(x, y) \
+    if (y)                                 \
+    pAttrs->add(x, *y)
+
+void OpDebug::report(OperationContext* opCtx,
+                     const SingleThreadedLockStats* lockStats,
+                     logv2::DynamicAttributes* pAttrs) const {
+    Client* client = opCtx->getClient();
+    auto& curop = *CurOp::get(opCtx);
+    auto flowControlStats = opCtx->lockState()->getFlowControlStats();
+
+    if (iscommand) {
+        pAttrs->add("type", "command");
+    } else {
+        pAttrs->add("type", networkOpToString(networkOp));
+    }
+
+    pAttrs->addDeepCopy("ns", curop.getNS());
+
+    const auto& clientMetadata = ClientMetadataIsMasterState::get(client).getClientMetadata();
+    if (clientMetadata) {
+        StringData appName = clientMetadata.get().getApplicationName();
+        if (!appName.empty()) {
+            pAttrs->add("appName", appName);
+        }
+    }
+
+    auto query = appendCommentField(opCtx, curop.opDescription());
+    if (!query.isEmpty()) {
+        if (iscommand) {
+            const Command* curCommand = curop.getCommand();
+            if (curCommand) {
+                mutablebson::Document cmdToLog(query, mutablebson::Document::kInPlaceDisabled);
+                curCommand->snipForLogging(&cmdToLog);
+                pAttrs->add("command", redact(cmdToLog.getObject()));
+            } else {
+                // Should not happen but we need to handle curCommand == NULL gracefully.
+                // We don't know what the request payload is intended to be, so it might be
+                // sensitive, and we don't know how to redact it properly without a 'Command*'.
+                // So we just don't log it at all.
+                pAttrs->add("command", "unrecognized");
+            }
+        } else {
+            pAttrs->add("command", redact(query));
+        }
+    }
+
+    auto originatingCommand = curop.originatingCommand();
+    if (!originatingCommand.isEmpty()) {
+        pAttrs->add("originatingCommand", redact(originatingCommand));
+    }
+
+    if (!curop.getPlanSummary().empty()) {
+        pAttrs->addDeepCopy("planSummary", curop.getPlanSummary().toString());
+    }
+
+    if (prepareConflictDurationMillis > Milliseconds::zero()) {
+        pAttrs->add("prepareConflictDuration", prepareConflictDurationMillis);
+    }
+
+    if (dataThroughputLastSecond) {
+        pAttrs->add("dataThroughputLastSecondMBperSec", *dataThroughputLastSecond);
+    }
+
+    if (dataThroughputAverage) {
+        pAttrs->add("dataThroughputAverageMBPerSec", *dataThroughputAverage);
+    }
+
+    OPDEBUG_TOATTR_HELP(nShards);
+    OPDEBUG_TOATTR_HELP(cursorid);
+    if (mongotCursorId) {
+        pAttrs->add("mongot", makeMongotDebugStatsObject());
+    }
+    OPDEBUG_TOATTR_HELP(ntoreturn);
+    OPDEBUG_TOATTR_HELP(ntoskip);
+    OPDEBUG_TOATTR_HELP_BOOL(exhaust);
+
+    OPDEBUG_TOATTR_HELP_OPTIONAL("keysExamined", additiveMetrics.keysExamined);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("docsExamined", additiveMetrics.docsExamined);
+    OPDEBUG_TOATTR_HELP_BOOL(hasSortStage);
+    OPDEBUG_TOATTR_HELP_BOOL(usedDisk);
+    OPDEBUG_TOATTR_HELP_BOOL(fromMultiPlanner);
+    if (replanReason) {
+        bool replanned = true;
+        OPDEBUG_TOATTR_HELP_BOOL(replanned);
+        pAttrs->add("replanReason", redact(*replanReason));
+    }
+    OPDEBUG_TOATTR_HELP_OPTIONAL("nMatched", additiveMetrics.nMatched);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("nModified", additiveMetrics.nModified);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("ninserted", additiveMetrics.ninserted);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("ndeleted", additiveMetrics.ndeleted);
+    OPDEBUG_TOATTR_HELP_BOOL(upsert);
+    OPDEBUG_TOATTR_HELP_BOOL(cursorExhausted);
+
+    OPDEBUG_TOATTR_HELP_OPTIONAL("keysInserted", additiveMetrics.keysInserted);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("keysDeleted", additiveMetrics.keysDeleted);
+    OPDEBUG_TOATTR_HELP_ATOMIC("prepareReadConflicts", additiveMetrics.prepareReadConflicts);
+    OPDEBUG_TOATTR_HELP_ATOMIC("writeConflicts", additiveMetrics.writeConflicts);
+
+    pAttrs->add("numYields", curop.numYields());
+    OPDEBUG_TOATTR_HELP(nreturned);
+
+    if (queryHash) {
+        pAttrs->addDeepCopy("queryHash", unsignedIntToFixedLengthHex(*queryHash));
+        invariant(planCacheKey);
+        pAttrs->addDeepCopy("planCacheKey", unsignedIntToFixedLengthHex(*planCacheKey));
+    }
+
+    if (!errInfo.isOK()) {
+        pAttrs->add("ok", 0);
+        if (!errInfo.reason().empty()) {
+            pAttrs->add("errMsg", redact(errInfo.reason()));
+        }
+        pAttrs->addDeepCopy("errName", errInfo.codeString());
+        pAttrs->add("errCode", static_cast<int>(errInfo.code()));
+    }
+
+    if (responseLength > 0) {
+        pAttrs->add("reslen", responseLength);
+    }
+
+    if (lockStats) {
+        BSONObjBuilder locks;
+        lockStats->report(&locks);
+        pAttrs->add("locks", locks.obj());
+    }
+
+    BSONObj flowControlObj = makeFlowControlObject(flowControlStats);
+    if (flowControlObj.nFields() > 0) {
+        pAttrs->add("flowControl", flowControlObj);
+    }
+
+    {
+        const auto& readConcern = repl::ReadConcernArgs::get(opCtx);
+        if (readConcern.isSpecified()) {
+            pAttrs->add("readConcern", readConcern.toBSONInner());
+        }
+    }
+
+    if (writeConcern && !writeConcern->usedDefault) {
+        pAttrs->add("writeConcern", writeConcern->toBSON());
+    }
+
+    if (storageStats) {
+        pAttrs->add("storage", storageStats->toBSON());
+    }
+
+    if (iscommand) {
+        pAttrs->add("protocol", getProtoString(networkOp));
+    }
+
+    if (remoteOpWaitTime) {
+        pAttrs->add("remoteOpWaitMillis", durationCount<Milliseconds>(*remoteOpWaitTime));
+    }
+
+    pAttrs->add("durationMillis", (executionTimeMicros / 1000));
+}
+
+
 #define OPDEBUG_APPEND_NUMBER(x) \
     if (x != -1)                 \
     b.appendNumber(#x, (x))
@@ -803,10 +1050,11 @@ string OpDebug::report(Client* client,
     if (y)                            \
     b.appendNumber(x, (*y))
 
-void OpDebug::append(const CurOp& curop,
+void OpDebug::append(OperationContext* opCtx,
                      const SingleThreadedLockStats& lockStats,
                      FlowControlTicketholder::CurOp flowControlStats,
                      BSONObjBuilder& b) const {
+    auto& curop = *CurOp::get(opCtx);
     const size_t maxElementSize = 50 * 1024;
 
     b.append("op", logicalOpToString(logicalOp));
@@ -814,7 +1062,8 @@ void OpDebug::append(const CurOp& curop,
     NamespaceString nss = NamespaceString(curop.getNS());
     b.append("ns", nss.ns());
 
-    appendAsObjOrString("command", curop.opDescription(), maxElementSize, &b);
+    appendAsObjOrString(
+        "command", appendCommentField(opCtx, curop.opDescription()), maxElementSize, &b);
 
     auto originatingCommand = curop.originatingCommand();
     if (!originatingCommand.isEmpty()) {
@@ -824,7 +1073,7 @@ void OpDebug::append(const CurOp& curop,
     OPDEBUG_APPEND_NUMBER(nShards);
     OPDEBUG_APPEND_NUMBER(cursorid);
     if (mongotCursorId) {
-        b.append("mongot", makeSearchBetaObject());
+        b.append("mongot", makeMongotDebugStatsObject());
     }
     OPDEBUG_APPEND_BOOL(exhaust);
 
@@ -850,6 +1099,9 @@ void OpDebug::append(const CurOp& curop,
     OPDEBUG_APPEND_ATOMIC("prepareReadConflicts", additiveMetrics.prepareReadConflicts);
     OPDEBUG_APPEND_ATOMIC("writeConflicts", additiveMetrics.writeConflicts);
 
+    OPDEBUG_APPEND_OPTIONAL("dataThroughputLastSecond", dataThroughputLastSecond);
+    OPDEBUG_APPEND_OPTIONAL("dataThroughputAverage", dataThroughputAverage);
+
     b.appendNumber("numYield", curop.numYields());
     OPDEBUG_APPEND_NUMBER(nreturned);
 
@@ -870,6 +1122,17 @@ void OpDebug::append(const CurOp& curop,
         flowControlBuilder.appendElements(flowControlMetrics);
     }
 
+    {
+        const auto& readConcern = repl::ReadConcernArgs::get(opCtx);
+        if (readConcern.isSpecified()) {
+            readConcern.appendInfo(&b);
+        }
+    }
+
+    if (writeConcern && !writeConcern->usedDefault) {
+        b.append("writeConcern", writeConcern->toBSON());
+    }
+
     if (storageStats) {
         b.append("storage", storageStats->toBSON());
     }
@@ -931,7 +1194,7 @@ BSONObj OpDebug::makeFlowControlObject(FlowControlTicketholder::CurOp stats) con
     return builder.obj();
 }
 
-BSONObj OpDebug::makeSearchBetaObject() const {
+BSONObj OpDebug::makeMongotDebugStatsObject() const {
     BSONObjBuilder cursorBuilder;
     invariant(mongotCursorId);
     cursorBuilder.append("cursorid", mongotCursorId.get());
@@ -1039,4 +1302,32 @@ string OpDebug::AdditiveMetrics::report() const {
     return s.str();
 }
 
+void OpDebug::AdditiveMetrics::report(logv2::DynamicAttributes* pAttrs) const {
+    OPDEBUG_TOATTR_HELP_OPTIONAL("keysExamined", keysExamined);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("docsExamined", docsExamined);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("nMatched", nMatched);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("nModified", nModified);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("ninserted", ninserted);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("ndeleted", ndeleted);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("keysInserted", keysInserted);
+    OPDEBUG_TOATTR_HELP_OPTIONAL("keysDeleted", keysDeleted);
+    OPDEBUG_TOATTR_HELP_ATOMIC("prepareReadConflicts", prepareReadConflicts);
+    OPDEBUG_TOATTR_HELP_ATOMIC("writeConflicts", writeConflicts);
+}
+
+BSONObj OpDebug::AdditiveMetrics::reportBSON() const {
+    BSONObjBuilder b;
+    OPDEBUG_APPEND_OPTIONAL("keysExamined", keysExamined);
+    OPDEBUG_APPEND_OPTIONAL("docsExamined", docsExamined);
+    OPDEBUG_APPEND_OPTIONAL("nMatched", nMatched);
+    OPDEBUG_APPEND_OPTIONAL("nModified", nModified);
+    OPDEBUG_APPEND_OPTIONAL("ninserted", ninserted);
+    OPDEBUG_APPEND_OPTIONAL("ndeleted", ndeleted);
+    OPDEBUG_APPEND_OPTIONAL("keysInserted", keysInserted);
+    OPDEBUG_APPEND_OPTIONAL("keysDeleted", keysDeleted);
+    OPDEBUG_APPEND_ATOMIC("prepareReadConflicts", prepareReadConflicts);
+    OPDEBUG_APPEND_ATOMIC("writeConflicts", writeConflicts);
+    return b.obj();
+}
+
 }  // namespace mongo
