@@ -36,6 +36,9 @@
 #include "mongo/db/cursor_id.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/write_concern_options.h"
+#include "mongo/logv2/attribute_storage.h"
+#include "mongo/logv2/log_component.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/util/progress_meter.h"
 #include "mongo/util/time_support.h"
@@ -119,6 +122,9 @@ public:
          * the format: "<field1>:<value1> <field2>:<value2> ...".
          */
         std::string report() const;
+        BSONObj reportBSON() const;
+
+        void report(logv2::DynamicAttributes* pAttrs) const;
 
         boost::optional<long long> keysExamined;
         boost::optional<long long> docsExamined;
@@ -147,10 +153,11 @@ public:
 
     OpDebug() = default;
 
-    std::string report(Client* client,
-                       const CurOp& curop,
-                       const SingleThreadedLockStats* lockStats,
-                       FlowControlTicketholder::CurOp flowControlStats) const;
+    std::string report(OperationContext* opCtx, const SingleThreadedLockStats* lockStats) const;
+
+    void report(OperationContext* opCtx,
+                const SingleThreadedLockStats* lockStats,
+                logv2::DynamicAttributes* pAttrs) const;
 
     /**
      * Appends information about the current operation to "builder"
@@ -158,7 +165,7 @@ public:
      * @param curop reference to the CurOp that owns this OpDebug
      * @param lockStats lockStats object containing locking information about the operation
      */
-    void append(const CurOp& curop,
+    void append(OperationContext* opCtx,
                 const SingleThreadedLockStats& lockStats,
                 FlowControlTicketholder::CurOp flowControlStats,
                 BSONObjBuilder& builder) const;
@@ -174,9 +181,9 @@ public:
     BSONObj makeFlowControlObject(FlowControlTicketholder::CurOp flowControlStats) const;
 
     /**
-     * Make object from $searchBeta stats with non-populated values omitted.
+     * Make object from $search stats with non-populated values omitted.
      */
-    BSONObj makeSearchBetaObject() const;
+    BSONObj makeMongotDebugStatsObject() const;
 
     // -------------------
 
@@ -195,7 +202,7 @@ public:
     long long ntoskip{-1};
     bool exhaust{false};
 
-    // For searchBeta.
+    // For search using mongot.
     boost::optional<long long> mongotCursorId{boost::none};
     boost::optional<long long> msWaitingForMongot{boost::none};
 
@@ -233,6 +240,13 @@ public:
     // Shard targeting info.
     int nShards{-1};
 
+    // Stores the duration of time spent blocked on prepare conflicts.
+    Milliseconds prepareConflictDurationMillis{0};
+
+    // Stores the amount of the data processed by the throttle cursors in MB/sec.
+    boost::optional<float> dataThroughputLastSecond;
+    boost::optional<float> dataThroughputAverage;
+
     // Used to track the amount of time spent waiting for a response from remote operations.
     boost::optional<Microseconds> remoteOpWaitTime;
 
@@ -243,6 +257,13 @@ public:
     std::shared_ptr<StorageStats> storageStats;
 
     bool waitingForFlowControl{false};
+
+    // Records the WC that was waited on during the operation. (The WC in opCtx can't be used
+    // because it's only set while the Command itself executes.)
+    boost::optional<WriteConcernOptions> writeConcern;
+
+    // Whether this is an oplog getMore operation for replication oplog fetching.
+    bool isReplOplogFetching{false};
 };
 
 /**
@@ -319,7 +340,7 @@ public:
      * operation should also be profiled.
      */
     bool completeAndLogOperation(OperationContext* opCtx,
-                                 logger::LogComponent logComponent,
+                                 logv2::LogComponent logComponent,
                                  boost::optional<size_t> responseLength = boost::none,
                                  boost::optional<long long> slowMsOverride = boost::none,
                                  bool forceLog = false);
@@ -616,7 +637,7 @@ public:
      * If called from a thread other than the one executing the operation associated with this
      * CurOp, it is necessary to lock the associated Client object before executing this method.
      */
-    void reportState(BSONObjBuilder* builder, bool truncateOps = false);
+    void reportState(OperationContext* opCtx, BSONObjBuilder* builder, bool truncateOps = false);
 
     /**
      * Sets the message for this CurOp.
