@@ -42,25 +42,30 @@ struct CollectionScanParams {
         BACKWARD = -1,
     };
 
-    // The RecordId to which we should seek to as the first document of the scan.
-    RecordId start;
-
     // If present, the collection scan will seek directly to the RecordId of an oplog entry as
     // close to 'minTs' as possible without going higher. Must only be set on forward oplog scans.
+    // This field cannot be used in conjunction with 'resumeAfterRecordId'.
     boost::optional<Timestamp> minTs;
 
     // If present, the collection scan will stop and return EOF the first time it sees a document
-    // that does not pass the filter and has 'ts' greater than 'maxTs'.
+    // that does not pass the filter and has 'ts' greater than 'maxTs'. Must only be set on forward
+    // oplog scans.
+    // This field cannot be used in conjunction with 'resumeAfterRecordId'.
     boost::optional<Timestamp> maxTs;
 
+    // If true, the collection scan will return a token that can be used to resume the scan.
+    bool requestResumeToken = false;
+
+    // If present, the collection scan will seek to the exact RecordId, or return KeyNotFound if it
+    // does not exist. Must only be set on forward collection scans.
+    // This field cannot be used in conjunction with 'minTs' or 'maxTs'.
+    boost::optional<RecordId> resumeAfterRecordId;
+
     Direction direction = FORWARD;
 
     // Do we want the scan to be 'tailable'?  Only meaningful if the collection is capped.
     bool tailable = false;
 
-    // Should we assert that the specified minTS has not fallen off the oplog?
-    bool assertMinTsHasNotFallenOffOplog = false;
-
     // Should we keep track of the timestamp of the latest oplog entry we've seen? This information
     // is needed to merge cursors from the oplog in order of operation time when reading the oplog
     // across a sharded cluster.
