@@ -27,10 +27,11 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/exec/geo_near.h"
 
+#include "mongo/logv2/log.h"
 #include <memory>
 #include <vector>
 
@@ -39,9 +40,8 @@
 
 #include "mongo/base/owned_pointer_vector.h"
 #include "mongo/db/bson/dotted_path_support.h"
+#include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/exec/fetch.h"
-#include "mongo/db/exec/index_scan.h"
-#include "mongo/db/exec/working_set_computed_data.h"
 #include "mongo/db/geo/geoconstants.h"
 #include "mongo/db/geo/geoparser.h"
 #include "mongo/db/geo/hash.h"
@@ -49,8 +49,6 @@
 #include "mongo/db/matcher/expression.h"
 #include "mongo/db/query/expression_index.h"
 #include "mongo/db/query/expression_index_knobs_gen.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
 
 #include <algorithm>
 
@@ -81,14 +79,14 @@ namespace {
 struct StoredGeometry {
     static StoredGeometry* parseFrom(const BSONElement& element) {
         if (!element.isABSONObj())
-            return NULL;
+            return nullptr;
 
         unique_ptr<StoredGeometry> stored(new StoredGeometry);
 
         // GeoNear stage can only be run with an existing index
         // Therefore, it is always safe to skip geometry validation
         if (!stored->geometry.parseFromStorage(element, true).isOK())
-            return NULL;
+            return nullptr;
         stored->element = element;
         return stored.release();
     }
@@ -127,12 +125,15 @@ static void extractGeometries(const BSONObj& doc,
                     // Valid geometry element
                     geometries->push_back(std::move(stored));
                 } else {
-                    warning() << "geoNear stage read non-geometry element " << redact(nextEl)
-                              << " in array " << redact(el);
+                    LOGV2_WARNING(23760,
+                                  "geoNear stage read non-geometry element {nextEl} in array {el}",
+                                  "nextEl"_attr = redact(nextEl),
+                                  "el"_attr = redact(el));
                 }
             }
         } else {
-            warning() << "geoNear stage read non-geometry element " << redact(el);
+            LOGV2_WARNING(
+                23761, "geoNear stage read non-geometry element {el}", "el"_attr = redact(el));
         }
     }
 }
@@ -152,11 +153,11 @@ static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams
 
     // Extract all the geometries out of this document for the near query
     std::vector<std::unique_ptr<StoredGeometry>> geometries;
-    extractGeometries(member->obj.value(), nearParams.nearQuery->field, &geometries);
+    extractGeometries(member->doc.value().toBson(), nearParams.nearQuery->field, &geometries);
 
     // Compute the minimum distance of all the geometries in the document
     double minDistance = -1;
-    BSONObj minDistanceObj;
+    Value minDistanceMetadata;
     for (auto it = geometries.begin(); it != geometries.end(); ++it) {
         StoredGeometry& stored = **it;
 
@@ -176,7 +177,7 @@ static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams
 
         if (minDistance < 0 || nextDistance < minDistance) {
             minDistance = nextDistance;
-            minDistanceObj = stored.element.Obj();
+            minDistanceMetadata = Value{stored.element};
         }
     }
 
@@ -190,14 +191,14 @@ static StatusWith<double> computeGeoNearDistance(const GeoNearParams& nearParams
             // Hack for nearSphere
             // TODO: Remove nearSphere?
             invariant(SPHERE == queryCRS);
-            member->addComputed(new GeoDistanceComputedData(minDistance / kRadiusOfEarthInMeters));
+            member->metadata().setGeoNearDistance(minDistance / kRadiusOfEarthInMeters);
         } else {
-            member->addComputed(new GeoDistanceComputedData(minDistance));
+            member->metadata().setGeoNearDistance(minDistance);
         }
     }
 
     if (nearParams.addPointMeta) {
-        member->addComputed(new GeoNearPointComputedData(minDistanceObj));
+        member->metadata().setGeoNearPoint(minDistanceMetadata);
     }
 
     return StatusWith<double>(minDistance);
@@ -247,7 +248,7 @@ static R2Annulus twoDDistanceBounds(const GeoNearParams& nearParams,
         // Reset the full bounds based on our index bounds
         GeoHashConverter::Parameters hashParams;
         Status status = GeoHashConverter::parseParameters(twoDIndex->infoObj(), &hashParams);
-        invariant(status.isOK());  // The index status should always be valid
+        invariantStatusOK(status);  // The index status should always be valid
 
         // The biggest distance possible in this indexed collection is the diagonal of the
         // square indexed region.
@@ -266,54 +267,32 @@ static R2Annulus twoDDistanceBounds(const GeoNearParams& nearParams,
     return fullBounds;
 }
 
-class GeoNear2DStage::DensityEstimator {
-public:
-    DensityEstimator(PlanStage::Children* children,
-                     BSONObj infoObj,
-                     const GeoNearParams* nearParams,
-                     const R2Annulus& fullBounds)
-        : _children(children), _nearParams(nearParams), _fullBounds(fullBounds), _currentLevel(0) {
-        GeoHashConverter::Parameters hashParams;
-        Status status = GeoHashConverter::parseParameters(std::move(infoObj), &hashParams);
-        // The index status should always be valid.
-        invariant(status.isOK());
+GeoNear2DStage::DensityEstimator::DensityEstimator(PlanStage::Children* children,
+                                                   BSONObj infoObj,
+                                                   const GeoNearParams* nearParams,
+                                                   const R2Annulus& fullBounds)
+    : _children(children), _nearParams(nearParams), _fullBounds(fullBounds), _currentLevel(0) {
+    GeoHashConverter::Parameters hashParams;
+    Status status = GeoHashConverter::parseParameters(std::move(infoObj), &hashParams);
+    // The index status should always be valid.
+    invariantStatusOK(status);
 
-        _converter.reset(new GeoHashConverter(hashParams));
-        _centroidCell = _converter->hash(_nearParams->nearQuery->centroid->oldPoint);
+    _converter.reset(new GeoHashConverter(hashParams));
+    _centroidCell = _converter->hash(_nearParams->nearQuery->centroid->oldPoint);
 
-        // Since appendVertexNeighbors(level, output) requires level < hash.getBits(),
-        // we have to start to find documents at most GeoHash::kMaxBits - 1. Thus the finest
-        // search area is 16 * finest cell area at GeoHash::kMaxBits.
-        _currentLevel = std::max(0, hashParams.bits - 1);
-    }
-
-    PlanStage::StageState work(OperationContext* opCtx,
-                               WorkingSet* workingSet,
-                               const IndexDescriptor* twoDIndex,
-                               WorkingSetID* out,
-                               double* estimatedDistance);
-
-private:
-    void buildIndexScan(OperationContext* opCtx,
-                        WorkingSet* workingSet,
-                        const IndexDescriptor* twoDIndex);
-
-    PlanStage::Children* _children;    // Points to PlanStage::_children in the NearStage.
-    const GeoNearParams* _nearParams;  // Not owned here.
-    const R2Annulus& _fullBounds;
-    IndexScan* _indexScan = nullptr;  // Owned in PlanStage::_children.
-    unique_ptr<GeoHashConverter> _converter;
-    GeoHash _centroidCell;
-    unsigned _currentLevel;
-};
+    // Since appendVertexNeighbors(level, output) requires level < hash.getBits(),
+    // we have to start to find documents at most GeoHash::kMaxBits - 1. Thus the finest
+    // search area is 16 * finest cell area at GeoHash::kMaxBits.
+    _currentLevel = std::max(0, hashParams.bits - 1);
+}
 
 // Initialize the internal states
-void GeoNear2DStage::DensityEstimator::buildIndexScan(OperationContext* opCtx,
+void GeoNear2DStage::DensityEstimator::buildIndexScan(ExpressionContext* expCtx,
                                                       WorkingSet* workingSet,
                                                       const IndexDescriptor* twoDIndex) {
     // Scan bounds on 2D indexes are only over the 2D field - other bounds aren't applicable.
     // This is handled in query planning.
-    IndexScanParams scanParams(opCtx, twoDIndex);
+    IndexScanParams scanParams(expCtx->opCtx, twoDIndex);
     scanParams.bounds = _nearParams->baseBounds;
 
     // The "2d" field is always the first in the index
@@ -344,20 +323,20 @@ void GeoNear2DStage::DensityEstimator::buildIndexScan(OperationContext* opCtx,
     IndexBoundsBuilder::intersectize(oil, &scanParams.bounds.fields[twoDFieldPosition]);
 
     invariant(!_indexScan);
-    _indexScan = new IndexScan(opCtx, scanParams, workingSet, NULL);
+    _indexScan = new IndexScan(expCtx, scanParams, workingSet, nullptr);
     _children->emplace_back(_indexScan);
 }
 
 // Return IS_EOF is we find a document in it's ancestor cells and set estimated distance
 // from the nearest document.
-PlanStage::StageState GeoNear2DStage::DensityEstimator::work(OperationContext* opCtx,
+PlanStage::StageState GeoNear2DStage::DensityEstimator::work(ExpressionContext* expCtx,
                                                              WorkingSet* workingSet,
                                                              const IndexDescriptor* twoDIndex,
                                                              WorkingSetID* out,
                                                              double* estimatedDistance) {
     if (!_indexScan) {
         // Setup index scan stage for current level.
-        buildIndexScan(opCtx, workingSet, twoDIndex);
+        buildIndexScan(expCtx, workingSet, twoDIndex);
     }
 
     WorkingSetID workingSetID;
@@ -435,7 +414,7 @@ PlanStage::StageState GeoNear2DStage::initialize(OperationContext* opCtx,
 
     double estimatedDistance;
     PlanStage::StageState state =
-        _densityEstimator->work(opCtx, workingSet, indexDescriptor(), out, &estimatedDistance);
+        _densityEstimator->work(expCtx(), workingSet, indexDescriptor(), out, &estimatedDistance);
 
     if (state == PlanStage::IS_EOF) {
         // 2d index only works with legacy points as centroid. $nearSphere will project
@@ -444,8 +423,8 @@ PlanStage::StageState GeoNear2DStage::initialize(OperationContext* opCtx,
 
         // Estimator finished its work, we need to finish initialization too.
         if (SPHERE == _nearParams.nearQuery->centroid->crs) {
-            // Estimated distance is in degrees, convert it to meters.
-            _boundsIncrement = deg2rad(estimatedDistance) * kRadiusOfEarthInMeters * 3;
+            // Estimated distance is in degrees, convert it to meters multiplied by 3.
+            _boundsIncrement = (estimatedDistance * kRadiusOfEarthInMeters * 3) * (M_PI / 180);
             // Limit boundsIncrement to ~20KM, so that the first circle won't be too aggressive.
             _boundsIncrement = std::min(_boundsIncrement, kMaxEarthDistanceInMeters / 1000.0);
         } else {
@@ -462,7 +441,7 @@ PlanStage::StageState GeoNear2DStage::initialize(OperationContext* opCtx,
         invariant(_boundsIncrement > 0.0);
 
         // Clean up
-        _densityEstimator.reset(NULL);
+        _densityEstimator.reset(nullptr);
     }
 
     return state;
@@ -471,10 +450,10 @@ PlanStage::StageState GeoNear2DStage::initialize(OperationContext* opCtx,
 static const string kTwoDIndexNearStage("GEO_NEAR_2D");
 
 GeoNear2DStage::GeoNear2DStage(const GeoNearParams& nearParams,
-                               OperationContext* opCtx,
+                               ExpressionContext* expCtx,
                                WorkingSet* workingSet,
                                const IndexDescriptor* twoDIndex)
-    : NearStage(opCtx, kTwoDIndexNearStage.c_str(), STAGE_GEO_NEAR_2D, workingSet, twoDIndex),
+    : NearStage(expCtx, kTwoDIndexNearStage.c_str(), STAGE_GEO_NEAR_2D, workingSet, twoDIndex),
       _nearParams(nearParams),
       _fullBounds(twoDDistanceBounds(nearParams, twoDIndex)),
       _currBounds(_fullBounds.center(), -1, _fullBounds.getInner()),
@@ -531,7 +510,7 @@ public:
 
     unique_ptr<MatchExpression> shallowClone() const final {
         MONGO_UNREACHABLE;
-        return NULL;
+        return nullptr;
     }
 
 private:
@@ -545,12 +524,12 @@ private:
 // Helper class to maintain ownership of a match expression alongside an index scan
 class FetchStageWithMatch final : public FetchStage {
 public:
-    FetchStageWithMatch(OperationContext* opCtx,
+    FetchStageWithMatch(ExpressionContext* expCtx,
                         WorkingSet* ws,
-                        PlanStage* child,
+                        std::unique_ptr<PlanStage> child,
                         MatchExpression* filter,
                         const Collection* collection)
-        : FetchStage(opCtx, ws, child, filter, collection), _matcher(filter) {}
+        : FetchStage(expCtx, ws, std::move(child), filter, collection), _matcher(filter) {}
 
 private:
     // Owns matcher
@@ -562,7 +541,7 @@ static double min2DBoundsIncrement(const GeoNearExpression& query,
                                    const IndexDescriptor* twoDIndex) {
     GeoHashConverter::Parameters hashParams;
     Status status = GeoHashConverter::parseParameters(twoDIndex->infoObj(), &hashParams);
-    invariant(status.isOK());  // The index status should always be valid
+    invariantStatusOK(status);  // The index status should always be valid
     GeoHashConverter hasher(hashParams);
 
     // The hasher error is the diagonal of a 2D hash region - it's generally not helpful
@@ -596,7 +575,7 @@ GeoNear2DStage::nextInterval(OperationContext* opCtx,
                              const Collection* collection) {
     // The search is finished if we searched at least once and all the way to the edge
     if (_currBounds.getInner() >= 0 && _currBounds.getOuter() == _fullBounds.getOuter()) {
-        return StatusWith<CoveredInterval*>(NULL);
+        return StatusWith<CoveredInterval*>(nullptr);
     }
 
     //
@@ -723,7 +702,7 @@ GeoNear2DStage::nextInterval(OperationContext* opCtx,
         .transitional_ignore();
 
     // 2D indexes support covered search over additional fields they contain
-    IndexScan* scan = new IndexScan(opCtx, scanParams, workingSet, _nearParams.filter);
+    auto scan = std::make_unique<IndexScan>(expCtx(), scanParams, workingSet, _nearParams.filter);
 
     MatchExpression* docMatcher = nullptr;
 
@@ -734,8 +713,8 @@ GeoNear2DStage::nextInterval(OperationContext* opCtx,
     }
 
     // FetchStage owns index scan
-    _children.emplace_back(
-        new FetchStageWithMatch(opCtx, workingSet, scan, docMatcher, collection));
+    _children.emplace_back(std::make_unique<FetchStageWithMatch>(
+        expCtx(), workingSet, std::move(scan), docMatcher, collection));
 
     return StatusWith<CoveredInterval*>(new CoveredInterval(
         _children.back().get(), nextBounds.getInner(), nextBounds.getOuter(), isLastInterval));
@@ -769,10 +748,10 @@ static int getFieldPosition(const IndexDescriptor* index, const string& fieldNam
 static const string kS2IndexNearStage("GEO_NEAR_2DSPHERE");
 
 GeoNear2DSphereStage::GeoNear2DSphereStage(const GeoNearParams& nearParams,
-                                           OperationContext* opCtx,
+                                           ExpressionContext* expCtx,
                                            WorkingSet* workingSet,
                                            const IndexDescriptor* s2Index)
-    : NearStage(opCtx, kS2IndexNearStage.c_str(), STAGE_GEO_NEAR_2DSPHERE, workingSet, s2Index),
+    : NearStage(expCtx, kS2IndexNearStage.c_str(), STAGE_GEO_NEAR_2DSPHERE, workingSet, s2Index),
       _nearParams(nearParams),
       _fullBounds(geoNearDistanceBounds(*nearParams.nearQuery)),
       _currBounds(_fullBounds.center(), -1, _fullBounds.getInner()),
@@ -813,27 +792,12 @@ S2Region* buildS2Region(const R2Annulus& sphereBounds) {
         regions.push_back(new S2Cap(innerCap));
     }
 
-    // 'kEpsilon' is about 9 times the double-precision roundoff relative error.
-    const double kEpsilon = 1e-15;
-
     // We only need to max bound if this is not a full search of the Earth
     // Using the constant here is important since we use the min of kMaxEarthDistance
     // and the actual bounds passed in to set up the search area.
-    if ((outer * (1 + kEpsilon)) < kMaxEarthDistanceInMeters) {
-        // SERVER-52953: The cell covering returned by S2 may have a matching point along its
-        // boundary. In certain cases, this boundary point is not contained within the covering,
-        // which means that this search will not match said point. As such, we avoid this issue by
-        // finding a covering for the region expanded over a very small radius because this covering
-        // is guaranteed to contain the boundary point.
-        auto angle = S1Angle::Radians((outer * (1 + kEpsilon)) / kRadiusOfEarthInMeters);
-        S2Cap outerCap = S2Cap::FromAxisAngle(latLng.ToPoint(), angle);
-
-        // If 'outer' is sufficiently small, the computation of the S2Cap's height from 'angle' may
-        // underflow, resulting in a height less than 'kEpsilon' and an empty cap. As such, we
-        // guarantee that 'outerCap' has a height of at least 'kEpsilon'.
-        if (outerCap.height() < kEpsilon) {
-            outerCap = S2Cap::FromAxisHeight(latLng.ToPoint(), kEpsilon);
-        }
+    if (outer < kMaxEarthDistanceInMeters) {
+        S2Cap outerCap = S2Cap::FromAxisAngle(latLng.ToPoint(),
+                                              S1Angle::Radians(outer / kRadiusOfEarthInMeters));
         regions.push_back(new S2Cap(outerCap));
     }
 
@@ -847,50 +811,26 @@ S2Region* buildS2Region(const R2Annulus& sphereBounds) {
 }
 }  // namespace
 
-// Estimate the density of data by search the nearest cells level by level around center.
-class GeoNear2DSphereStage::DensityEstimator {
-public:
-    DensityEstimator(PlanStage::Children* children,
-                     const GeoNearParams* nearParams,
-                     const S2IndexingParams& indexParams,
-                     const R2Annulus& fullBounds)
-        : _children(children),
-          _nearParams(nearParams),
-          _indexParams(indexParams),
-          _fullBounds(fullBounds),
-          _currentLevel(0) {
-        // cellId.AppendVertexNeighbors(level, output) requires level < finest,
-        // so we use the minimum of max_level - 1 and the user specified finest
-        int level = std::min(S2::kMaxCellLevel - 1, gInternalQueryS2GeoFinestLevel.load());
-        _currentLevel = std::max(0, level);
-    }
-
-    // Search for a document in neighbors at current level.
-    // Return IS_EOF is such document exists and set the estimated distance to the nearest doc.
-    PlanStage::StageState work(OperationContext* opCtx,
-                               WorkingSet* workingSet,
-                               const IndexDescriptor* s2Index,
-                               WorkingSetID* out,
-                               double* estimatedDistance);
-
-private:
-    void buildIndexScan(OperationContext* opCtx,
-                        WorkingSet* workingSet,
-                        const IndexDescriptor* s2Index);
-
-    PlanStage::Children* _children;    // Points to PlanStage::_children in the NearStage.
-    const GeoNearParams* _nearParams;  // Not owned here.
-    const S2IndexingParams _indexParams;
-    const R2Annulus& _fullBounds;
-    int _currentLevel;
-    IndexScan* _indexScan = nullptr;  // Owned in PlanStage::_children.
-};
+GeoNear2DSphereStage::DensityEstimator::DensityEstimator(PlanStage::Children* children,
+                                                         const GeoNearParams* nearParams,
+                                                         const S2IndexingParams& indexParams,
+                                                         const R2Annulus& fullBounds)
+    : _children(children),
+      _nearParams(nearParams),
+      _indexParams(indexParams),
+      _fullBounds(fullBounds),
+      _currentLevel(0) {
+    // cellId.AppendVertexNeighbors(level, output) requires level < finest,
+    // so we use the minimum of max_level - 1 and the user specified finest
+    int level = std::min(S2::kMaxCellLevel - 1, gInternalQueryS2GeoFinestLevel.load());
+    _currentLevel = std::max(0, level);
+}
 
 // Setup the index scan stage for neighbors at this level.
-void GeoNear2DSphereStage::DensityEstimator::buildIndexScan(OperationContext* opCtx,
+void GeoNear2DSphereStage::DensityEstimator::buildIndexScan(ExpressionContext* expCtx,
                                                             WorkingSet* workingSet,
                                                             const IndexDescriptor* s2Index) {
-    IndexScanParams scanParams(opCtx, s2Index);
+    IndexScanParams scanParams(expCtx->opCtx, s2Index);
     scanParams.bounds = _nearParams->baseBounds;
 
     // Because the planner doesn't yet set up 2D index bounds, do it ourselves here
@@ -913,18 +853,18 @@ void GeoNear2DSphereStage::DensityEstimator::buildIndexScan(OperationContext* op
 
     // Index scan
     invariant(!_indexScan);
-    _indexScan = new IndexScan(opCtx, scanParams, workingSet, NULL);
+    _indexScan = new IndexScan(expCtx, scanParams, workingSet, nullptr);
     _children->emplace_back(_indexScan);
 }
 
-PlanStage::StageState GeoNear2DSphereStage::DensityEstimator::work(OperationContext* opCtx,
+PlanStage::StageState GeoNear2DSphereStage::DensityEstimator::work(ExpressionContext* expCtx,
                                                                    WorkingSet* workingSet,
                                                                    const IndexDescriptor* s2Index,
                                                                    WorkingSetID* out,
                                                                    double* estimatedDistance) {
     if (!_indexScan) {
         // Setup index scan stage for current level.
-        buildIndexScan(opCtx, workingSet, s2Index);
+        buildIndexScan(expCtx, workingSet, s2Index);
     }
 
     WorkingSetID workingSetID;
@@ -1005,7 +945,7 @@ PlanStage::StageState GeoNear2DSphereStage::initialize(OperationContext* opCtx,
 
     double estimatedDistance;
     PlanStage::StageState state =
-        _densityEstimator->work(opCtx, workingSet, indexDescriptor(), out, &estimatedDistance);
+        _densityEstimator->work(expCtx(), workingSet, indexDescriptor(), out, &estimatedDistance);
 
     if (state == IS_EOF) {
         // We find a document in 4 neighbors at current level, but didn't at previous level.
@@ -1019,7 +959,7 @@ PlanStage::StageState GeoNear2DSphereStage::initialize(OperationContext* opCtx,
         invariant(_boundsIncrement > 0.0);
 
         // Clean up
-        _densityEstimator.reset(NULL);
+        _densityEstimator.reset(nullptr);
     }
 
     return state;
@@ -1031,7 +971,7 @@ GeoNear2DSphereStage::nextInterval(OperationContext* opCtx,
                                    const Collection* collection) {
     // The search is finished if we searched at least once and all the way to the edge
     if (_currBounds.getInner() >= 0 && _currBounds.getOuter() == _fullBounds.getOuter()) {
-        return StatusWith<CoveredInterval*>(NULL);
+        return StatusWith<CoveredInterval*>(nullptr);
     }
 
     //
@@ -1093,10 +1033,11 @@ GeoNear2DSphereStage::nextInterval(OperationContext* opCtx,
     OrderedIntervalList* coveredIntervals = &scanParams.bounds.fields[s2FieldPosition];
     ExpressionMapping::S2CellIdsToIntervalsWithParents(cover, _indexParams, coveredIntervals);
 
-    IndexScan* scan = new IndexScan(opCtx, scanParams, workingSet, nullptr);
+    auto scan = std::make_unique<IndexScan>(expCtx(), scanParams, workingSet, nullptr);
 
     // FetchStage owns index scan
-    _children.emplace_back(new FetchStage(opCtx, workingSet, scan, _nearParams.filter, collection));
+    _children.emplace_back(std::make_unique<FetchStage>(
+        expCtx(), workingSet, std::move(scan), _nearParams.filter, collection));
 
     return StatusWith<CoveredInterval*>(new CoveredInterval(
         _children.back().get(), nextBounds.getInner(), nextBounds.getOuter(), isLastInterval));
