@@ -48,10 +48,9 @@ namespace mongo {
 class BSONObjBuilder;
 class MatchExpression;
 struct UpdateTicket;
+struct InsertResult;
 struct InsertDeleteOptions;
 
-bool failIndexKeyTooLongParam();
-
 /**
  * An IndexAccessMethod is the interface through which all the mutation, lookup, and
  * traversal of index entries is done. The class is designed so that the underlying index
@@ -67,8 +66,6 @@ class IndexAccessMethod {
     IndexAccessMethod& operator=(const IndexAccessMethod&) = delete;
 
 public:
-    using KeyHandlerFn = std::function<Status(const BSONObj&)>;
-
     IndexAccessMethod() = default;
     virtual ~IndexAccessMethod() = default;
 
@@ -91,43 +88,22 @@ public:
                           const BSONObj& obj,
                           const RecordId& loc,
                           const InsertDeleteOptions& options,
-                          KeyHandlerFn&& onDuplicateKey,
-                          int64_t* numInserted) = 0;
+                          InsertResult* result) = 0;
 
-    /**
-     * Inserts the specified keys into the index, and determines whether these keys should cause the
-     * index to become multikey. If so, this method also handles the task of marking the index as
-     * multikey in the catalog, and sets the path-level multikey information if applicable.
-     */
-    virtual Status insertKeysAndUpdateMultikeyPaths(
-        OperationContext* opCtx,
-        const std::vector<BSONObj>& keys,
-        const std::vector<BSONObj>& multikeyMetadataKeys,
-        const MultikeyPaths& multikeyPaths,
-        const RecordId& loc,
-        const InsertDeleteOptions& options,
-        KeyHandlerFn&& onDuplicateKey,
-        int64_t* numInserted) = 0;
-
-    /**
-     * Inserts the specified keys into the index. Does not attempt to determine whether the
-     * insertion of these keys should cause the index to become multikey. The 'numInserted' output
-     * parameter, if non-nullptr, will be reset to the number of keys inserted by this function
-     * call, or to zero in the case of either a non-OK return Status or an empty 'keys' argument.
-     */
     virtual Status insertKeys(OperationContext* opCtx,
-                              const std::vector<BSONObj>& keys,
+                              const std::vector<KeyString::Value>& keys,
+                              const std::vector<KeyString::Value>& multikeyMetadataKeys,
+                              const MultikeyPaths& multikeyPaths,
                               const RecordId& loc,
                               const InsertDeleteOptions& options,
-                              KeyHandlerFn&& onDuplicateKey,
-                              int64_t* numInserted) = 0;
+                              InsertResult* result) = 0;
 
     /**
      * Analogous to insertKeys above, but remove the keys instead of inserting them.
      * 'numDeleted' will be set to the number of keys removed from the index for the provided keys.
      */
     virtual Status removeKeys(OperationContext* opCtx,
-                              const std::vector<BSONObj>& keys,
+                              const std::vector<KeyString::Value>& keys,
                               const RecordId& loc,
                               const InsertDeleteOptions& options,
                               int64_t* numDeleted) = 0;
@@ -142,7 +118,7 @@ public:
                                const BSONObj& to,
                                const RecordId& loc,
                                const InsertDeleteOptions& options,
-                               UpdateTicket* ticket) = 0;
+                               UpdateTicket* ticket) const = 0;
 
     /**
      * Perform a validated update.  The keys for the 'from' object will be removed, and the keys
@@ -178,19 +154,6 @@ public:
      */
     virtual Status initializeAsEmpty(OperationContext* opCtx) = 0;
 
-    /**
-     * Try to page-in the pages that contain the keys generated from 'obj'.
-     * This can be used to speed up future accesses to an index by trying to ensure the
-     * appropriate pages are not swapped out.
-     * See prefetch.cpp.
-     */
-    virtual Status touch(OperationContext* opCtx, const BSONObj& obj) = 0;
-
-    /**
-     * this pages in the entire index
-     */
-    virtual Status touch(OperationContext* opCtx) const = 0;
-
     /**
      * Walk the entire index, checking the internal structure for consistency.
      * Set numKeys to the number of keys in the index.
@@ -227,9 +190,7 @@ public:
     /**
      * Sets this index as multikey with the provided paths.
      */
-    virtual void setIndexIsMultikey(OperationContext* opCtx,
-                                    std::vector<BSONObj> multikeyMetadataKeys,
-                                    MultikeyPaths paths) = 0;
+    virtual void setIndexIsMultikey(OperationContext* opCtx, MultikeyPaths paths) = 0;
 
     //
     // Bulk operations support
@@ -237,7 +198,7 @@ public:
 
     class BulkBuilder {
     public:
-        using Sorter = mongo::Sorter<BSONObj, RecordId>;
+        using Sorter = mongo::Sorter<KeyString::Value, mongo::NullValue>;
 
         virtual ~BulkBuilder() = default;
 
@@ -284,14 +245,22 @@ public:
      * @param mayInterrupt - Is this commit interruptible (will cancel)
      * @param dupsAllowed - If false and 'dupRecords' is not null, append with the RecordIds of
      *                      the uninserted duplicates.
-     * @param onDuplicateKey - Will be called for each duplicate key inserted into the index.
+     *                      If true and 'dupKeys' is not null, append with the keys of the inserted
+     *                      duplicates.
      * @param dupRecords - If not null, is filled with the RecordIds of uninserted duplicate keys.
+     *                     If null, duplicate keys will return errors.
+     * @param dupKeys - If not null and 'dupsAllowed' is true, is filled with the keys of inserted
+     *                  duplicates.
+     *                  If null, duplicates are inserted but not recorded.
+     *
+     * It is invalid and contradictory to pass both 'dupRecords' and 'dupKeys'.
      */
+
     virtual Status commitBulk(OperationContext* opCtx,
                               BulkBuilder* bulk,
                               bool dupsAllowed,
-                              KeyHandlerFn&& onDuplicateKey,
-                              std::set<RecordId>* dupRecords) = 0;
+                              std::set<RecordId>* dupRecords,
+                              std::vector<BSONObj>* dupKeys) = 0;
 
     /**
      * Specifies whether getKeys should relax the index constraints or not, in order of most
@@ -307,10 +276,10 @@ public:
     };
 
     /**
-     * Specifies whether getKeys is being used in the context of creating new keys or deleting
-     * existing keys.
+     * Specifies whether getKeys is being used in the context of creating new keys, deleting
+     * or validating existing keys.
      */
-    enum class GetKeysContext { kRemovingKeys, kReadOrAddKeys };
+    enum class GetKeysContext { kRemovingKeys, kAddingKeys, kValidatingKeys };
 
     /**
      * Fills 'keys' with the keys that should be generated for 'obj' on this index.
@@ -328,21 +297,31 @@ public:
      * BSONObjSet with any multikey metadata keys generated while processing the document. These
      * keys are not associated with the document itself, but instead represent multi-key path
      * information that must be stored in a reserved keyspace within the index.
+     *
+     * If any key generation errors are encountered and suppressed due to the provided GetKeysMode,
+     * 'onSuppressedErrorFn' is called.
      */
+    using OnSuppressedErrorFn =
+        std::function<void(Status status, const BSONObj& obj, boost::optional<RecordId> loc)>;
     virtual void getKeys(const BSONObj& obj,
                          GetKeysMode mode,
                          GetKeysContext context,
-                         BSONObjSet* keys,
-                         BSONObjSet* multikeyMetadataKeys,
-                         MultikeyPaths* multikeyPaths) const = 0;
+                         KeyStringSet* keys,
+                         KeyStringSet* multikeyMetadataKeys,
+                         MultikeyPaths* multikeyPaths,
+                         boost::optional<RecordId> id,
+                         OnSuppressedErrorFn onSuppressedError) const = 0;
+
+    static OnSuppressedErrorFn kNoopOnSuppressedErrorFn;
 
     /**
      * Given the set of keys, multikeyMetadataKeys and multikeyPaths generated by a particular
      * document, return 'true' if the index should be marked as multikey and 'false' otherwise.
      */
-    virtual bool shouldMarkIndexAsMultikey(const std::vector<BSONObj>& keys,
-                                           const std::vector<BSONObj>& multikeyMetadataKeys,
-                                           const MultikeyPaths& multikeyPaths) const = 0;
+    virtual bool shouldMarkIndexAsMultikey(
+        size_t numberOfKeys,
+        const std::vector<KeyString::Value>& multikeyMetadataKeys,
+        const MultikeyPaths& multikeyPaths) const = 0;
 
     /**
      * Returns the intersection of 'fields' and the set of multikey metadata paths stored in the
@@ -387,8 +366,17 @@ public:
     static void set(ServiceContext* service,
                     std::unique_ptr<IndexAccessMethodFactory> collectionFactory);
 
-    virtual std::unique_ptr<IndexAccessMethod> make(IndexCatalogEntry* entry,
-                                                    SortedDataInterface* sortedDataInterface) = 0;
+    virtual std::unique_ptr<IndexAccessMethod> make(
+        IndexCatalogEntry* entry, std::unique_ptr<SortedDataInterface> sortedDataInterface) = 0;
+};
+
+/**
+ * Records number of keys inserted and duplicate keys inserted, if applicable.
+ */
+struct InsertResult {
+public:
+    std::int64_t numInserted{0};
+    std::vector<BSONObj> dupsInserted;
 };
 
 /**
@@ -396,20 +384,15 @@ public:
  * prepareUpdate fills out the UpdateStatus and update actually applies it.
  */
 struct UpdateTicket {
-    UpdateTicket()
-        : oldKeys(SimpleBSONObjComparator::kInstance.makeBSONObjSet()),
-          newKeys(oldKeys),
-          newMultikeyMetadataKeys(newKeys) {}
-
     bool _isValid{false};
 
-    BSONObjSet oldKeys;
-    BSONObjSet newKeys;
+    KeyStringSet oldKeys;
+    KeyStringSet newKeys;
 
-    BSONObjSet newMultikeyMetadataKeys;
+    KeyStringSet newMultikeyMetadataKeys;
 
-    std::vector<BSONObj> removed;
-    std::vector<BSONObj> added;
+    std::vector<KeyString::Value> removed;
+    std::vector<KeyString::Value> added;
 
     RecordId loc;
     bool dupsAllowed;
@@ -460,36 +443,28 @@ public:
      * setDifference({BSON("a" << 0.0)}, {BSON("a" << 0LL)}) would result in the pair
      * ( {BSON("a" << 0.0)}, {BSON("a" << 0LL)} ).
      */
-    static std::pair<std::vector<BSONObj>, std::vector<BSONObj>> setDifference(
-        const BSONObjSet& left, const BSONObjSet& right);
+    static std::pair<std::vector<KeyString::Value>, std::vector<KeyString::Value>> setDifference(
+        const KeyStringSet& left, const KeyStringSet& right, Ordering ordering);
 
-    AbstractIndexAccessMethod(IndexCatalogEntry* btreeState, SortedDataInterface* btree);
+    AbstractIndexAccessMethod(IndexCatalogEntry* btreeState,
+                              std::unique_ptr<SortedDataInterface> btree);
 
     Status insert(OperationContext* opCtx,
                   const BSONObj& obj,
                   const RecordId& loc,
                   const InsertDeleteOptions& options,
-                  KeyHandlerFn&& onDuplicateKey,
-                  int64_t* numInserted) final;
+                  InsertResult* result) final;
 
     Status insertKeys(OperationContext* opCtx,
-                      const std::vector<BSONObj>& keys,
+                      const std::vector<KeyString::Value>& keys,
+                      const std::vector<KeyString::Value>& multikeyMetadataKeys,
+                      const MultikeyPaths& multikeyPaths,
                       const RecordId& loc,
                       const InsertDeleteOptions& options,
-                      KeyHandlerFn&& onDuplicateKey,
-                      int64_t* numInserted) final;
-
-    Status insertKeysAndUpdateMultikeyPaths(OperationContext* opCtx,
-                                            const std::vector<BSONObj>& keys,
-                                            const std::vector<BSONObj>& multikeyMetadataKeys,
-                                            const MultikeyPaths& multikeyPaths,
-                                            const RecordId& loc,
-                                            const InsertDeleteOptions& options,
-                                            KeyHandlerFn&& onDuplicateKey,
-                                            int64_t* numInserted) final;
+                      InsertResult* result) final;
 
     Status removeKeys(OperationContext* opCtx,
-                      const std::vector<BSONObj>& keys,
+                      const std::vector<KeyString::Value>& keys,
                       const RecordId& loc,
                       const InsertDeleteOptions& options,
                       int64_t* numDeleted) final;
@@ -500,7 +475,7 @@ public:
                        const BSONObj& to,
                        const RecordId& loc,
                        const InsertDeleteOptions& options,
-                       UpdateTicket* ticket) final;
+                       UpdateTicket* ticket) const final;
 
     Status update(OperationContext* opCtx,
                   const UpdateTicket& ticket,
@@ -513,10 +488,6 @@ public:
 
     Status initializeAsEmpty(OperationContext* opCtx) final;
 
-    Status touch(OperationContext* opCtx, const BSONObj& obj) final;
-
-    Status touch(OperationContext* opCtx) const final;
-
     void validate(OperationContext* opCtx,
                   int64_t* numKeys,
                   ValidateResults* fullResults) const final;
@@ -531,27 +502,27 @@ public:
 
     Status compact(OperationContext* opCtx) final;
 
-    void setIndexIsMultikey(OperationContext* opCtx,
-                            std::vector<BSONObj> multikeyMetadataKeys,
-                            MultikeyPaths paths) final;
+    void setIndexIsMultikey(OperationContext* opCtx, MultikeyPaths paths) final;
 
     std::unique_ptr<BulkBuilder> initiateBulk(size_t maxMemoryUsageBytes) final;
 
     Status commitBulk(OperationContext* opCtx,
                       BulkBuilder* bulk,
                       bool dupsAllowed,
-                      KeyHandlerFn&& onDuplicateKey,
-                      std::set<RecordId>* dupRecords) final;
+                      std::set<RecordId>* dupRecords,
+                      std::vector<BSONObj>* dupKeys) final;
 
     void getKeys(const BSONObj& obj,
                  GetKeysMode mode,
                  GetKeysContext context,
-                 BSONObjSet* keys,
-                 BSONObjSet* multikeyMetadataKeys,
-                 MultikeyPaths* multikeyPaths) const final;
+                 KeyStringSet* keys,
+                 KeyStringSet* multikeyMetadataKeys,
+                 MultikeyPaths* multikeyPaths,
+                 boost::optional<RecordId> id,
+                 OnSuppressedErrorFn onSuppressedError) const final;
 
-    bool shouldMarkIndexAsMultikey(const std::vector<BSONObj>& keys,
-                                   const std::vector<BSONObj>& multikeyMetadataKeys,
+    bool shouldMarkIndexAsMultikey(size_t numberOfKeys,
+                                   const std::vector<KeyString::Value>& multikeyMetadataKeys,
                                    const MultikeyPaths& multikeyPaths) const override;
 
     SortedDataInterface* getSortedDataInterface() const override final;
@@ -573,11 +544,12 @@ protected:
      */
     virtual void doGetKeys(const BSONObj& obj,
                            GetKeysContext context,
-                           BSONObjSet* keys,
-                           BSONObjSet* multikeyMetadataKeys,
-                           MultikeyPaths* multikeyPaths) const = 0;
+                           KeyStringSet* keys,
+                           KeyStringSet* multikeyMetadataKeys,
+                           MultikeyPaths* multikeyPaths,
+                           boost::optional<RecordId> id) const = 0;
 
-    IndexCatalogEntry* const _btreeState;  // owned by IndexCatalogEntry
+    IndexCatalogEntry* const _indexCatalogEntry;  // owned by IndexCatalog
     const IndexDescriptor* const _descriptor;
 
 private:
@@ -589,19 +561,7 @@ private:
      * in the event that a non-fatal 'ErrorCodes::DuplicateKeyValue' is encountered during a
      * background index build.
      */
-    bool isFatalError(OperationContext* opCtx, Status status, BSONObj key);
-
-    /**
-     * Determines whether it's OK to ignore ErrorCodes::KeyTooLong for this OperationContext
-     * TODO SERVER-36385: Remove this function.
-     */
-    bool ignoreKeyTooLong();
-
-    /**
-     * If true, we should check whether the index key exceeds the hardcoded limit.
-     * TODO SERVER-36385: Remove this function.
-     */
-    bool shouldCheckIndexKeySize(OperationContext* opCtx);
+    bool isFatalError(OperationContext* opCtx, Status status, KeyString::Value key);
 
     /**
      * Removes a single key from the index.
@@ -609,7 +569,7 @@ private:
      * Used by remove() only.
      */
     void removeOneKey(OperationContext* opCtx,
-                      const BSONObj& key,
+                      const KeyString::Value& keyString,
                       const RecordId& loc,
                       bool dupsAllowed);
 
