@@ -29,7 +29,7 @@ Copyright (C) 2019-present Percona and/or its affiliates. All rights reserved.
     it in the license file.
 ======= */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kAccessControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kAccessControl
 
 #include "mongo/db/ldap/ldap_manager.h"
 
@@ -71,11 +71,6 @@ ServiceContext::ConstructorActionRegisterer createLDAPManager(
     [](ServiceContext* service) {
         if (!ldapGlobalParams.ldapServers->empty()) {
             auto ldapManager = LDAPManager::create();
-            Status status = ldapManager->initialize();
-            using namespace fmt::literals;
-            uassertStatusOKWithContext(status,
-                "Cannot initialize LDAP server connection (parameters are: {})"_format(
-                    ldapGlobalParams.logString()));
             LDAPManager::set(service, std::move(ldapManager));
         }
     });
