@@ -46,21 +46,24 @@ public:
 
     virtual Status initialize() override;
 
-    virtual void start_threads() override;
-
     virtual Status queryUserRoles(const UserName& userName, stdx::unordered_set<RoleName>& roles) override;
 
     virtual Status mapUserToDN(const std::string& user, std::string& out) override;
 
+    void needReinit() {
+        _reinitPending.store(true);
+    }
+
 private:
+    LDAP* _ldap{nullptr};
     std::unique_ptr<ConnectionPoller> _connPoller;
 
-    LDAP* borrow_search_connection();
-    void return_search_connection(LDAP* ldap);
+    AtomicWord<bool> _reinitPending{true};
+    // guard init/reinit of _ldap
+    Mutex _mutex = MONGO_MAKE_LATCH("LDAPManagerImpl::_mutex");
 
-    Status execQuery(const std::string& ldapurl,
-                     bool entitiesonly,
-                     std::vector<std::string>& results);
+    Status reinitialize();
+    Status execQuery(std::string& ldapurl, std::vector<std::string>& results);
 };
 
 // bind either simple or sasl using global LDAP parameters
