@@ -77,8 +77,7 @@ public:
         managerState = localManagerState.get();
         managerState->setAuthzVersion(AuthorizationManager::schemaVersion26Final);
         auto authzManager = std::make_unique<AuthorizationManagerImpl>(
-            std::move(localManagerState),
-            AuthorizationManagerImpl::InstallMockForTestingOrAuthImpl{});
+            getServiceContext(), std::move(localManagerState));
         authzManager->setAuthEnabled(true);
         AuthorizationManager::set(getServiceContext(), std::move(authzManager));
         Client::releaseCurrent();
@@ -88,7 +87,7 @@ public:
 
         auto localServiceLiaison =
             std::make_unique<MockServiceLiaison>(std::make_shared<MockServiceLiaisonImpl>());
-        auto localSessionsCollection = stdx::make_unique<MockSessionsCollection>(
+        auto localSessionsCollection = std::make_unique<MockSessionsCollection>(
             std::make_shared<MockSessionsCollectionImpl>());
 
         auto localLogicalSessionCache = std::make_unique<LogicalSessionCacheImpl>(
@@ -231,14 +230,7 @@ TEST_F(LogicalSessionIdTest, GenWithMultipleAuthedUsers) {
     addSimpleUser(UserName("simple", "test"));
     addSimpleUser(UserName("simple", "test2"));
 
-    ASSERT_THROWS_WITH_CHECK(makeLogicalSessionId(_opCtx.get()),
-                             AssertionException,
-                             [](const AssertionException& exception) {
-                                 ASSERT_EQ(exception.code(), ErrorCodes::Unauthorized);
-                                 ASSERT_STRING_CONTAINS(
-                                     exception.reason(),
-                                     "docs.mongodb.com/manual/core/authentication");
-                             });
+    ASSERT_THROWS(makeLogicalSessionId(_opCtx.get()), AssertionException);
 }
 
 TEST_F(LogicalSessionIdTest, GenWithoutAuthedUser) {
