@@ -31,12 +31,14 @@
 
 #include "mongo/db/matcher/expression_parser.h"
 
+#include <memory>
 #include <pcrecpp.h>
 
 #include "mongo/base/init.h"
 #include "mongo/bson/bsonmisc.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
+#include "mongo/bson/bsontypes.h"
 #include "mongo/db/matcher/expression_always_boolean.h"
 #include "mongo/db/matcher/expression_array.h"
 #include "mongo/db/matcher/expression_expr.h"
@@ -65,8 +67,6 @@
 #include "mongo/db/matcher/schema/json_schema_parser.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/query_knobs_gen.h"
-#include "mongo/db/stats/counters.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/util/str.h"
 #include "mongo/util/string_map.h"
 
@@ -128,22 +128,19 @@ Status parseSub(StringData name,
                 MatchExpressionParser::AllowedFeatureSet allowedFeatures,
                 DocumentParseLevel currentLevel);
 
-stdx::function<StatusWithMatchExpression(StringData,
-                                         BSONElement,
-                                         const boost::intrusive_ptr<ExpressionContext>&,
-                                         const ExtensionsCallback*,
-                                         MatchExpressionParser::AllowedFeatureSet,
-                                         DocumentParseLevel)>
+std::function<StatusWithMatchExpression(StringData,
+                                        BSONElement,
+                                        const boost::intrusive_ptr<ExpressionContext>&,
+                                        const ExtensionsCallback*,
+                                        MatchExpressionParser::AllowedFeatureSet,
+                                        DocumentParseLevel)>
 retrievePathlessParser(StringData name);
 
-StatusWithMatchExpression parseRegexElement(StringData name,
-                                            BSONElement e,
-                                            const boost::intrusive_ptr<ExpressionContext>& expCtx) {
+StatusWithMatchExpression parseRegexElement(StringData name, BSONElement e) {
     if (e.type() != BSONType::RegEx)
         return {Status(ErrorCodes::BadValue, "not a regex")};
 
-    expCtx->incrementMatchExprCounter("$regex");
-    return {stdx::make_unique<RegexMatchExpression>(name, e.regex(), e.regexFlags())};
+    return {std::make_unique<RegexMatchExpression>(name, e.regex(), e.regexFlags())};
 }
 
 StatusWithMatchExpression parseComparison(
@@ -246,7 +243,7 @@ StatusWithMatchExpression parse(const BSONObj& obj,
                                 const ExtensionsCallback* extensionsCallback,
                                 MatchExpressionParser::AllowedFeatureSet allowedFeatures,
                                 DocumentParseLevel currentLevel) {
-    auto root = stdx::make_unique<AndMatchExpression>();
+    auto root = std::make_unique<AndMatchExpression>();
 
     const DocumentParseLevel nextLevel = (currentLevel == DocumentParseLevel::kPredicateTopLevel)
         ? DocumentParseLevel::kUserDocumentTopLevel
@@ -278,7 +275,6 @@ StatusWithMatchExpression parse(const BSONObj& obj,
                 root->add(parsedExpression.getValue().release());
             }
 
-            expCtx->incrementMatchExprCounter(e.fieldNameStringData());
             continue;
         }
 
@@ -296,7 +292,7 @@ StatusWithMatchExpression parse(const BSONObj& obj,
         }
 
         if (e.type() == BSONType::RegEx) {
-            auto result = parseRegexElement(e.fieldNameStringData(), e, expCtx);
+            auto result = parseRegexElement(e.fieldNameStringData(), e);
             if (!result.isOK())
                 return result;
             root->add(result.getValue().release());
@@ -305,14 +301,13 @@ StatusWithMatchExpression parse(const BSONObj& obj,
 
         auto eq =
             parseComparison(e.fieldNameStringData(),
-                            stdx::make_unique<EqualityMatchExpression>(e.fieldNameStringData(), e),
+                            std::make_unique<EqualityMatchExpression>(e.fieldNameStringData(), e),
                             e,
                             expCtx,
                             allowedFeatures);
         if (!eq.isOK())
             return eq;
 
-        expCtx->incrementMatchExprCounter("$eq");
         root->add(eq.getValue().release());
     }
 
@@ -343,8 +338,12 @@ StatusWithMatchExpression parseWhere(StringData name,
     if ((allowedFeatures & MatchExpressionParser::AllowedFeatures::kJavascript) == 0u) {
         return {Status(ErrorCodes::BadValue, "$where is not allowed in this context")};
     }
+    if (currentLevel == DocumentParseLevel::kUserSubDocument) {
+        return {
+            Status(ErrorCodes::BadValue, "$where can only be applied to the top-level document")};
+    }
 
-    return extensionsCallback->parseWhere(elem);
+    return extensionsCallback->parseWhere(expCtx, elem);
 }
 
 StatusWithMatchExpression parseText(StringData name,
@@ -371,7 +370,7 @@ StatusWithMatchExpression parseDBRef(StringData name,
                                      const ExtensionsCallback* extensionsCallback,
                                      MatchExpressionParser::AllowedFeatureSet allowedFeatures,
                                      DocumentParseLevel currentLevel) {
-    auto eq = stdx::make_unique<EqualityMatchExpression>(elem.fieldName(), elem);
+    auto eq = std::make_unique<EqualityMatchExpression>(elem.fieldName(), elem);
 
     // 'id' is collation-aware. 'ref' and 'db' are compared using binary comparison.
     eq->setCollator("id"_sd == name ? expCtx->getCollator() : nullptr);
@@ -416,7 +415,7 @@ StatusWithMatchExpression parseAlwaysBoolean(
                        str::stream() << T::kName << " must be an integer value of 1")};
     }
 
-    return {stdx::make_unique<T>()};
+    return {std::make_unique<T>()};
 }
 
 StatusWithMatchExpression parseExpr(StringData name,
@@ -434,32 +433,31 @@ StatusWithMatchExpression parseExpr(StringData name,
         return {Status(ErrorCodes::QueryFeatureNotAllowed, "$expr is not allowed in this context")};
     }
 
-    return {stdx::make_unique<ExprMatchExpression>(std::move(elem), expCtx)};
+    return {std::make_unique<ExprMatchExpression>(std::move(elem), expCtx)};
 }
 
-StatusWithMatchExpression parseMOD(StringData name, BSONElement e) {
-    if (e.type() != BSONType::Array)
+StatusWithMatchExpression parseMOD(StringData name, BSONElement elem) {
+    if (elem.type() != BSONType::Array)
         return {Status(ErrorCodes::BadValue, "malformed mod, needs to be an array")};
 
-    BSONObjIterator i(e.Obj());
+    BSONObjIterator iter(elem.Obj());
 
-    if (!i.more())
+    if (!iter.more())
         return {Status(ErrorCodes::BadValue, "malformed mod, not enough elements")};
-    auto d = i.next();
-    if (!d.isNumber())
+    auto divisor = iter.next();
+    if (!divisor.isNumber())
         return {Status(ErrorCodes::BadValue, "malformed mod, divisor not a number")};
 
-    if (!i.more())
+    if (!iter.more())
         return {Status(ErrorCodes::BadValue, "malformed mod, not enough elements")};
-    auto r = i.next();
-    if (!r.isNumber())
+    auto remainder = iter.next();
+    if (!remainder.isNumber())
         return {Status(ErrorCodes::BadValue, "malformed mod, remainder not a number")};
 
-    if (i.more())
+    if (iter.more())
         return {Status(ErrorCodes::BadValue, "malformed mod, too many elements")};
 
-    return {std::make_unique<ModMatchExpression>(
-        name, ModMatchExpression::truncateToLong(d), ModMatchExpression::truncateToLong(r))};
+    return {std::make_unique<ModMatchExpression>(name, divisor.numberInt(), remainder.numberInt())};
 }
 
 StatusWithMatchExpression parseRegexDocument(StringData name, const BSONObj& doc) {
@@ -507,7 +505,7 @@ StatusWithMatchExpression parseRegexDocument(StringData name, const BSONObj& doc
         }
     }
 
-    return {stdx::make_unique<RegexMatchExpression>(name, regex, regexOptions)};
+    return {std::make_unique<RegexMatchExpression>(name, regex, regexOptions)};
 }
 
 Status parseInExpression(InMatchExpression* inExpression,
@@ -522,7 +520,7 @@ Status parseInExpression(InMatchExpression* inExpression,
         }
 
         if (e.type() == BSONType::RegEx) {
-            auto status = inExpression->addRegex(stdx::make_unique<RegexMatchExpression>(""_sd, e));
+            auto status = inExpression->addRegex(std::make_unique<RegexMatchExpression>(""_sd, e));
             if (!status.isOK()) {
                 return status;
             }
@@ -535,7 +533,7 @@ Status parseInExpression(InMatchExpression* inExpression,
 
 template <class T>
 StatusWithMatchExpression parseType(StringData name, BSONElement elt) {
-    auto typeSet = MatcherTypeSet::parse(elt, kTypeAliasMap);
+    auto typeSet = MatcherTypeSet::parse(elt);
     if (!typeSet.isOK()) {
         return typeSet.getStatus();
     }
@@ -545,7 +543,7 @@ StatusWithMatchExpression parseType(StringData name, BSONElement elt) {
                        str::stream() << name << " must match at least one type")};
     }
 
-    return {stdx::make_unique<T>(name, std::move(typeSet.getValue()))};
+    return {std::make_unique<T>(name, std::move(typeSet.getValue()))};
 }
 
 /**
@@ -626,19 +624,19 @@ StatusWithMatchExpression parseBitTest(StringData name, BSONElement e) {
         if (!bitPositions.isOK()) {
             return bitPositions.getStatus();
         }
-        bitTestMatchExpression = stdx::make_unique<T>(name, std::move(bitPositions.getValue()));
+        bitTestMatchExpression = std::make_unique<T>(name, std::move(bitPositions.getValue()));
     } else if (e.isNumber()) {
         // Integer bitmask provided as value.
         auto bitMask = e.parseIntegerElementToNonNegativeLong();
         if (!bitMask.isOK()) {
             return bitMask.getStatus();
         }
-        bitTestMatchExpression = stdx::make_unique<T>(name, bitMask.getValue());
+        bitTestMatchExpression = std::make_unique<T>(name, bitMask.getValue());
     } else if (e.type() == BSONType::BinData) {
         // Binary bitmask provided as value.
         int eBinaryLen;
         auto eBinary = e.binData(eBinaryLen);
-        bitTestMatchExpression = stdx::make_unique<T>(name, eBinary, eBinaryLen);
+        bitTestMatchExpression = std::make_unique<T>(name, eBinary, eBinaryLen);
     } else {
         return Status(ErrorCodes::BadValue,
                       str::stream()
@@ -677,7 +675,7 @@ StatusWithMatchExpression parseInternalSchemaFmod(StringData name, BSONElement e
         return {ErrorCodes::BadValue, str::stream() << path << " has too many elements"};
     }
 
-    return {stdx::make_unique<InternalSchemaFmodMatchExpression>(
+    return {std::make_unique<InternalSchemaFmodMatchExpression>(
         name, d.numberDecimal(), r.numberDecimal())};
 }
 
@@ -700,7 +698,7 @@ StatusWithMatchExpression parseInternalSchemaRootDocEq(
                                      << " must be an object, found type " << elem.type())};
     }
     auto rootDocEq =
-        stdx::make_unique<InternalSchemaRootDocEqMatchExpression>(elem.embeddedObject());
+        std::make_unique<InternalSchemaRootDocEqMatchExpression>(elem.embeddedObject());
     return {std::move(rootDocEq)};
 }
 
@@ -716,7 +714,7 @@ StatusWithMatchExpression parseInternalSchemaSingleIntegerArgument(StringData na
         return parsedInt.getStatus();
     }
 
-    return {stdx::make_unique<T>(name, parsedInt.getValue())};
+    return {std::make_unique<T>(name, parsedInt.getValue())};
 }
 
 /**
@@ -735,7 +733,7 @@ StatusWithMatchExpression parseTopLevelInternalSchemaSingleIntegerArgument(
     if (!parsedInt.isOK()) {
         return parsedInt.getStatus();
     }
-    return {stdx::make_unique<T>(parsedInt.getValue())};
+    return {std::make_unique<T>(parsedInt.getValue())};
 }
 
 /**
@@ -969,7 +967,7 @@ StatusWithMatchExpression parseInternalSchemaAllowedProperties(
         return properties.getStatus();
     }
 
-    return {stdx::make_unique<InternalSchemaAllowedPropertiesMatchExpression>(
+    return {std::make_unique<InternalSchemaAllowedPropertiesMatchExpression>(
         std::move(properties.getValue()),
         namePlaceholder.getValue(),
         std::move(patternProperties.getValue()),
@@ -1024,22 +1022,21 @@ StatusWithMatchExpression parseInternalSchemaMatchArrayIndex(
         return expressionWithPlaceholder.getStatus();
     }
 
-    return {stdx::make_unique<InternalSchemaMatchArrayIndexMatchExpression>(
+    return {std::make_unique<InternalSchemaMatchArrayIndexMatchExpression>(
         path, index.getValue(), std::move(expressionWithPlaceholder.getValue()))};
 }
 
 StatusWithMatchExpression parseGeo(StringData name,
                                    PathAcceptingKeyword type,
                                    const BSONObj& section,
-                                   const boost::intrusive_ptr<ExpressionContext>& expCtx,
                                    MatchExpressionParser::AllowedFeatureSet allowedFeatures) {
     if (PathAcceptingKeyword::WITHIN == type || PathAcceptingKeyword::GEO_INTERSECTS == type) {
-        auto gq = stdx::make_unique<GeoExpression>(name.toString());
+        auto gq = std::make_unique<GeoExpression>(name.toString());
         auto parseStatus = gq->parseFrom(section);
         if (!parseStatus.isOK()) {
             return parseStatus;
         }
-        return {stdx::make_unique<GeoMatchExpression>(name, gq.release(), section)};
+        return {std::make_unique<GeoMatchExpression>(name, gq.release(), section)};
     } else {
         invariant(PathAcceptingKeyword::GEO_NEAR == type);
 
@@ -1048,13 +1045,12 @@ StatusWithMatchExpression parseGeo(StringData name,
                            "$geoNear, $near, and $nearSphere are not allowed in this context")};
         }
 
-        auto nq = stdx::make_unique<GeoNearExpression>(name.toString());
+        auto nq = std::make_unique<GeoNearExpression>(name.toString());
         auto status = nq->parseFrom(section);
         if (!status.isOK()) {
             return status;
         }
-        expCtx->incrementMatchExprCounter(section.firstElementFieldName());
-        return {stdx::make_unique<GeoNearMatchExpression>(name, nq.release(), section)};
+        return {std::make_unique<GeoNearMatchExpression>(name, nq.release(), section)};
     }
 }
 
@@ -1070,7 +1066,7 @@ StatusWithMatchExpression parseTreeTopLevel(
         return {Status(ErrorCodes::BadValue, str::stream() << T::kName << " must be an array")};
     }
 
-    auto temp = stdx::make_unique<T>();
+    auto temp = std::make_unique<T>();
 
     auto arr = elem.Obj();
     if (arr.isEmpty()) {
@@ -1131,7 +1127,7 @@ StatusWithMatchExpression parseElemMatch(StringData name,
         if (!s.isOK())
             return s;
 
-        auto temp = stdx::make_unique<ElemMatchValueMatchExpression>(name);
+        auto temp = std::make_unique<ElemMatchValueMatchExpression>(name);
 
         for (size_t i = 0; i < theAnd.numChildren(); i++) {
             temp->add(theAnd.getChild(i));
@@ -1159,7 +1155,7 @@ StatusWithMatchExpression parseElemMatch(StringData name,
         return {Status(ErrorCodes::BadValue, "$elemMatch cannot contain $where expression")};
     }
 
-    return {stdx::make_unique<ElemMatchObjectMatchExpression>(name, sub.release())};
+    return {std::make_unique<ElemMatchObjectMatchExpression>(name, sub.release())};
 }
 
 StatusWithMatchExpression parseAll(StringData name,
@@ -1171,7 +1167,7 @@ StatusWithMatchExpression parseAll(StringData name,
         return {Status(ErrorCodes::BadValue, "$all needs an array")};
 
     auto arr = e.Obj();
-    auto myAnd = stdx::make_unique<AndMatchExpression>();
+    auto myAnd = std::make_unique<AndMatchExpression>();
     BSONObjIterator i(arr);
 
     if (arr.firstElement().type() == BSONType::Object &&
@@ -1209,20 +1205,20 @@ StatusWithMatchExpression parseAll(StringData name,
         auto e = i.next();
 
         if (e.type() == BSONType::RegEx) {
-            auto expr = stdx::make_unique<RegexMatchExpression>(name, e);
+            auto expr = std::make_unique<RegexMatchExpression>(name, e);
             myAnd->add(expr.release());
         } else if (e.type() == BSONType::Object &&
                    MatchExpressionParser::parsePathAcceptingKeyword(e.Obj().firstElement())) {
             return {Status(ErrorCodes::BadValue, "no $ expressions in $all")};
         } else {
-            auto expr = stdx::make_unique<EqualityMatchExpression>(name, e);
+            auto expr = std::make_unique<EqualityMatchExpression>(name, e);
             expr->setCollator(expCtx->getCollator());
             myAnd->add(expr.release());
         }
     }
 
     if (myAnd->numChildren() == 0) {
-        return {stdx::make_unique<AlwaysFalseMatchExpression>()};
+        return {std::make_unique<AlwaysFalseMatchExpression>()};
     }
 
     return {std::move(myAnd)};
@@ -1275,7 +1271,7 @@ StatusWithMatchExpression parseInternalSchemaFixedArityArgument(
         ++position;
     }
 
-    return {stdx::make_unique<T>(std::move(expressions))};
+    return {std::make_unique<T>(std::move(expressions))};
 }
 
 StatusWithMatchExpression parseNot(StringData name,
@@ -1285,11 +1281,11 @@ StatusWithMatchExpression parseNot(StringData name,
                                    MatchExpressionParser::AllowedFeatureSet allowedFeatures,
                                    DocumentParseLevel currentLevel) {
     if (elem.type() == BSONType::RegEx) {
-        auto regex = parseRegexElement(name, elem, expCtx);
+        auto regex = parseRegexElement(name, elem);
         if (!regex.isOK()) {
             return regex;
         }
-        return {stdx::make_unique<NotMatchExpression>(regex.getValue().release())};
+        return {std::make_unique<NotMatchExpression>(regex.getValue().release())};
     }
 
     if (elem.type() != BSONType::Object) {
@@ -1301,14 +1297,14 @@ StatusWithMatchExpression parseNot(StringData name,
         return {ErrorCodes::BadValue, "$not cannot be empty"};
     }
 
-    auto theAnd = stdx::make_unique<AndMatchExpression>();
+    auto theAnd = std::make_unique<AndMatchExpression>();
     auto parseStatus = parseSub(
         name, notObject, theAnd.get(), expCtx, extensionsCallback, allowedFeatures, currentLevel);
     if (!parseStatus.isOK()) {
         return parseStatus;
     }
 
-    return {stdx::make_unique<NotMatchExpression>(theAnd.release())};
+    return {std::make_unique<NotMatchExpression>(theAnd.release())};
 }
 
 StatusWithMatchExpression parseInternalSchemaBinDataSubType(StringData name, BSONElement e) {
@@ -1333,7 +1329,7 @@ StatusWithMatchExpression parseInternalSchemaBinDataSubType(StringData name, BSO
                           << " value must represent BinData subtype: " << valueAsInt.getValue());
     }
 
-    return {stdx::make_unique<InternalSchemaBinDataSubTypeExpression>(
+    return {std::make_unique<InternalSchemaBinDataSubTypeExpression>(
         name, static_cast<BinDataType>(valueAsInt.getValue()))};
 }
 
@@ -1354,7 +1350,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
 
     if ("$eq"_sd == e.fieldNameStringData()) {
         return parseComparison(
-            name, stdx::make_unique<EqualityMatchExpression>(name, e), e, expCtx, allowedFeatures);
+            name, std::make_unique<EqualityMatchExpression>(name, e), e, expCtx, allowedFeatures);
     }
 
     if ("$not"_sd == e.fieldNameStringData()) {
@@ -1375,16 +1371,16 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
     switch (*parseExpMatchType) {
         case PathAcceptingKeyword::LESS_THAN:
             return parseComparison(
-                name, stdx::make_unique<LTMatchExpression>(name, e), e, expCtx, allowedFeatures);
+                name, std::make_unique<LTMatchExpression>(name, e), e, expCtx, allowedFeatures);
         case PathAcceptingKeyword::LESS_THAN_OR_EQUAL:
             return parseComparison(
-                name, stdx::make_unique<LTEMatchExpression>(name, e), e, expCtx, allowedFeatures);
+                name, std::make_unique<LTEMatchExpression>(name, e), e, expCtx, allowedFeatures);
         case PathAcceptingKeyword::GREATER_THAN:
             return parseComparison(
-                name, stdx::make_unique<GTMatchExpression>(name, e), e, expCtx, allowedFeatures);
+                name, std::make_unique<GTMatchExpression>(name, e), e, expCtx, allowedFeatures);
         case PathAcceptingKeyword::GREATER_THAN_OR_EQUAL:
             return parseComparison(
-                name, stdx::make_unique<GTEMatchExpression>(name, e), e, expCtx, allowedFeatures);
+                name, std::make_unique<GTEMatchExpression>(name, e), e, expCtx, allowedFeatures);
         case PathAcceptingKeyword::NOT_EQUAL: {
             if (BSONType::RegEx == e.type()) {
                 // Just because $ne can be rewritten as the negation of an equality does not mean
@@ -1393,15 +1389,15 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
             }
             StatusWithMatchExpression s =
                 parseComparison(name,
-                                stdx::make_unique<EqualityMatchExpression>(name, e),
+                                std::make_unique<EqualityMatchExpression>(name, e),
                                 e,
                                 expCtx,
                                 allowedFeatures);
-            return {stdx::make_unique<NotMatchExpression>(s.getValue().release())};
+            return {std::make_unique<NotMatchExpression>(s.getValue().release())};
         }
         case PathAcceptingKeyword::EQUALITY:
             return parseComparison(name,
-                                   stdx::make_unique<EqualityMatchExpression>(name, e),
+                                   std::make_unique<EqualityMatchExpression>(name, e),
                                    e,
                                    expCtx,
                                    allowedFeatures);
@@ -1410,7 +1406,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
             if (e.type() != BSONType::Array) {
                 return {Status(ErrorCodes::BadValue, "$in needs an array")};
             }
-            auto temp = stdx::make_unique<InMatchExpression>(name);
+            auto temp = std::make_unique<InMatchExpression>(name);
             auto parseStatus = parseInExpression(temp.get(), e.Obj(), expCtx);
             if (!parseStatus.isOK()) {
                 return parseStatus;
@@ -1422,12 +1418,12 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
             if (e.type() != Array) {
                 return {Status(ErrorCodes::BadValue, "$nin needs an array")};
             }
-            auto temp = stdx::make_unique<InMatchExpression>(name);
+            auto temp = std::make_unique<InMatchExpression>(name);
             auto parseStatus = parseInExpression(temp.get(), e.Obj(), expCtx);
             if (!parseStatus.isOK()) {
                 return parseStatus;
             }
-            return {stdx::make_unique<NotMatchExpression>(temp.release())};
+            return {std::make_unique<NotMatchExpression>(temp.release())};
         }
 
         case PathAcceptingKeyword::SIZE: {
@@ -1454,7 +1450,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
             if (size < 0) {
                 return {Status(ErrorCodes::BadValue, "$size may not be negative")};
             }
-            return {stdx::make_unique<SizeMatchExpression>(name, size)};
+            return {std::make_unique<SizeMatchExpression>(name, size)};
         }
 
         case PathAcceptingKeyword::EXISTS: {
@@ -1462,12 +1458,12 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
                 return {Status(ErrorCodes::BadValue, "$exists can't be eoo")};
             }
 
-            auto existsExpr = stdx::make_unique<ExistsMatchExpression>(name);
+            auto existsExpr = std::make_unique<ExistsMatchExpression>(name);
             if (e.trueValue()) {
                 return {std::move(existsExpr)};
             }
 
-            return {stdx::make_unique<NotMatchExpression>(existsExpr.release())};
+            return {std::make_unique<NotMatchExpression>(existsExpr.release())};
         }
 
         case PathAcceptingKeyword::TYPE:
@@ -1501,7 +1497,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
 
         case PathAcceptingKeyword::WITHIN:
         case PathAcceptingKeyword::GEO_INTERSECTS:
-            return parseGeo(name, *parseExpMatchType, context, expCtx, allowedFeatures);
+            return parseGeo(name, *parseExpMatchType, context, allowedFeatures);
 
         case PathAcceptingKeyword::GEO_NEAR:
             return {Status(ErrorCodes::BadValue,
@@ -1515,7 +1511,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
                                              << typeName(e.type()))};
             }
 
-            auto exprEqExpr = stdx::make_unique<InternalExprEqMatchExpression>(name, e);
+            auto exprEqExpr = std::make_unique<InternalExprEqMatchExpression>(name, e);
             exprEqExpr->setCollator(expCtx->getCollator());
             return {std::move(exprEqExpr)};
         }
@@ -1565,7 +1561,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
                 return parsedSubObjExpr;
             }
 
-            return {stdx::make_unique<InternalSchemaObjectMatchExpression>(
+            return {std::make_unique<InternalSchemaObjectMatchExpression>(
                 name, std::move(parsedSubObjExpr.getValue()))};
         }
 
@@ -1575,7 +1571,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
                         str::stream() << name << " must be a boolean of value true"};
             }
 
-            return {stdx::make_unique<InternalSchemaUniqueItemsMatchExpression>(name)};
+            return {std::make_unique<InternalSchemaUniqueItemsMatchExpression>(name)};
         }
 
         case PathAcceptingKeyword::INTERNAL_SCHEMA_MIN_LENGTH: {
@@ -1653,7 +1649,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
             if (!exprWithPlaceholder.isOK()) {
                 return exprWithPlaceholder.getStatus();
             }
-            return {stdx::make_unique<InternalSchemaAllElemMatchFromIndexMatchExpression>(
+            return {std::make_unique<InternalSchemaAllElemMatchFromIndexMatchExpression>(
                 name, parsedIndex.getValue(), std::move(exprWithPlaceholder.getValue()))};
         }
 
@@ -1662,7 +1658,7 @@ StatusWithMatchExpression parseSubField(const BSONObj& context,
         }
 
         case PathAcceptingKeyword::INTERNAL_SCHEMA_EQ: {
-            return {stdx::make_unique<InternalSchemaEqMatchExpression>(name, e)};
+            return {std::make_unique<InternalSchemaEqMatchExpression>(name, e)};
         }
 
         case PathAcceptingKeyword::INTERNAL_SCHEMA_BIN_DATA_ENCRYPTED_TYPE: {
@@ -1705,8 +1701,7 @@ Status parseSub(StringData name,
         if (firstElt.isABSONObj()) {
             if (MatchExpressionParser::parsePathAcceptingKeyword(firstElt) ==
                 PathAcceptingKeyword::GEO_NEAR) {
-                auto s =
-                    parseGeo(name, PathAcceptingKeyword::GEO_NEAR, sub, expCtx, allowedFeatures);
+                auto s = parseGeo(name, PathAcceptingKeyword::GEO_NEAR, sub, allowedFeatures);
                 if (s.isOK()) {
                     root->add(s.getValue().release());
                 }
@@ -1723,7 +1718,6 @@ Status parseSub(StringData name,
         if (!s.isOK())
             return s.getStatus();
 
-        expCtx->incrementMatchExprCounter(deep.fieldNameStringData());
         if (s.getValue())
             root->add(s.getValue().release());
     }
@@ -1750,29 +1744,29 @@ StatusWithMatchExpression MatchExpressionParser::parse(
 namespace {
 // Maps from query operator string name to function.
 std::unique_ptr<StringMap<
-    stdx::function<StatusWithMatchExpression(StringData,
-                                             BSONElement,
-                                             const boost::intrusive_ptr<ExpressionContext>&,
-                                             const ExtensionsCallback*,
-                                             MatchExpressionParser::AllowedFeatureSet,
-                                             DocumentParseLevel)>>>
+    std::function<StatusWithMatchExpression(StringData,
+                                            BSONElement,
+                                            const boost::intrusive_ptr<ExpressionContext>&,
+                                            const ExtensionsCallback*,
+                                            MatchExpressionParser::AllowedFeatureSet,
+                                            DocumentParseLevel)>>>
     pathlessOperatorMap;
 
 MONGO_INITIALIZER(PathlessOperatorMap)(InitializerContext* context) {
-    pathlessOperatorMap = stdx::make_unique<StringMap<
-        stdx::function<StatusWithMatchExpression(StringData,
-                                                 BSONElement,
-                                                 const boost::intrusive_ptr<ExpressionContext>&,
-                                                 const ExtensionsCallback*,
-                                                 MatchExpressionParser::AllowedFeatureSet,
-                                                 DocumentParseLevel)>>>(
+    pathlessOperatorMap = std::make_unique<StringMap<
+        std::function<StatusWithMatchExpression(StringData,
+                                                BSONElement,
+                                                const boost::intrusive_ptr<ExpressionContext>&,
+                                                const ExtensionsCallback*,
+                                                MatchExpressionParser::AllowedFeatureSet,
+                                                DocumentParseLevel)>>>(
         StringMap<
-            stdx::function<StatusWithMatchExpression(StringData,
-                                                     BSONElement,
-                                                     const boost::intrusive_ptr<ExpressionContext>&,
-                                                     const ExtensionsCallback*,
-                                                     MatchExpressionParser::AllowedFeatureSet,
-                                                     DocumentParseLevel)>>{
+            std::function<StatusWithMatchExpression(StringData,
+                                                    BSONElement,
+                                                    const boost::intrusive_ptr<ExpressionContext>&,
+                                                    const ExtensionsCallback*,
+                                                    MatchExpressionParser::AllowedFeatureSet,
+                                                    DocumentParseLevel)>>{
             {"_internalSchemaAllowedProperties", &parseInternalSchemaAllowedProperties},
             {"_internalSchemaCond",
              &parseInternalSchemaFixedArityArgument<InternalSchemaCondMatchExpression>},
@@ -1806,7 +1800,7 @@ std::unique_ptr<StringMap<PathAcceptingKeyword>> queryOperatorMap;
 
 MONGO_INITIALIZER(MatchExpressionParser)(InitializerContext* context) {
     queryOperatorMap =
-        stdx::make_unique<StringMap<PathAcceptingKeyword>>(StringMap<PathAcceptingKeyword>{
+        std::make_unique<StringMap<PathAcceptingKeyword>>(StringMap<PathAcceptingKeyword>{
             // TODO: SERVER-19565 Add $eq after auditing callers.
             {"_internalExprEq", PathAcceptingKeyword::INTERNAL_EXPR_EQ},
             {"_internalSchemaAllElemMatchFromIndex",
@@ -1860,12 +1854,12 @@ MONGO_INITIALIZER(MatchExpressionParser)(InitializerContext* context) {
  * Returns the proper parser for the indicated pathless operator. Returns 'null' if 'name'
  * doesn't represent a known type.
  */
-stdx::function<StatusWithMatchExpression(StringData,
-                                         BSONElement,
-                                         const boost::intrusive_ptr<ExpressionContext>&,
-                                         const ExtensionsCallback*,
-                                         MatchExpressionParser::AllowedFeatureSet,
-                                         DocumentParseLevel)>
+std::function<StatusWithMatchExpression(StringData,
+                                        BSONElement,
+                                        const boost::intrusive_ptr<ExpressionContext>&,
+                                        const ExtensionsCallback*,
+                                        MatchExpressionParser::AllowedFeatureSet,
+                                        DocumentParseLevel)>
 retrievePathlessParser(StringData name) {
     auto func = pathlessOperatorMap->find(name);
     if (func == pathlessOperatorMap->end()) {
@@ -1873,35 +1867,6 @@ retrievePathlessParser(StringData name) {
     }
     return func->second;
 }
-
-MONGO_INITIALIZER_WITH_PREREQUISITES(MatchExpressionCounters,
-                                     ("PathlessOperatorMap", "MatchExpressionParser"))
-(InitializerContext* context) {
-    static const std::set<std::string> exceptionsSet{"within",   // deprecated
-                                                     "geoNear",  // aggregation stage
-                                                     "db",       // $-prefixed field names
-                                                     "id",
-                                                     "ref",
-                                                     "options"};
-
-    for (auto&& [name, keyword] : *queryOperatorMap) {
-        if (name[0] == '_' || exceptionsSet.count(name) > 0) {
-            continue;
-        }
-        operatorCountersMatchExpressions.addMatchExprCounter("$" + name);
-    }
-    for (auto&& [name, fn] : *pathlessOperatorMap) {
-        if (name[0] == '_' || exceptionsSet.count(name) > 0) {
-            continue;
-        }
-        operatorCountersMatchExpressions.addMatchExprCounter("$" + name);
-    }
-    operatorCountersMatchExpressions.addMatchExprCounter("$not");
-    operatorCountersMatchExpressions.addMatchExprCounter("$eq");
-    return Status::OK();
-}
-
-
 }  // namespace
 
 boost::optional<PathAcceptingKeyword> MatchExpressionParser::parsePathAcceptingKeyword(
