@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kControl
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kControl
 
 #include "mongo/db/mongod_options.h"
 
@@ -56,7 +56,9 @@
 #include "mongo/db/server_options_base.h"
 #include "mongo/db/server_options_nongeneral_gen.h"
 #include "mongo/db/server_options_server_helpers.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
+#include "mongo/logv2/log_domain_global.h"
+#include "mongo/logv2/log_manager.h"
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/options_parser/startup_options.h"
 #include "mongo/util/str.h"
@@ -104,17 +106,20 @@ void printMongodHelp(const moe::OptionSection& options) {
 };
 
 namespace {
-void sysRuntimeInfo() {
+
+void appendSysInfo(BSONObjBuilder* obj) {
+    auto o = BSONObjBuilder(obj->subobjStart("sysinfo"));
 #if defined(_SC_PAGE_SIZE)
-    log() << "  page size: " << (int)sysconf(_SC_PAGE_SIZE);
+    o.append("_SC_PAGE_SIZE", (long long)sysconf(_SC_PAGE_SIZE));
 #endif
 #if defined(_SC_PHYS_PAGES)
-    log() << "  _SC_PHYS_PAGES: " << sysconf(_SC_PHYS_PAGES);
+    o.append("_SC_PHYS_PAGES", (long long)sysconf(_SC_PHYS_PAGES));
 #endif
 #if defined(_SC_AVPHYS_PAGES)
-    log() << "  _SC_AVPHYS_PAGES: " << sysconf(_SC_AVPHYS_PAGES);
+    o.append("_SC_AVPHYS_PAGES", (long long)sysconf(_SC_AVPHYS_PAGES));
 #endif
 }
+
 }  // namespace
 
 bool handlePreValidationMongodOptions(const moe::Environment& params,
@@ -124,20 +129,20 @@ bool handlePreValidationMongodOptions(const moe::Environment& params,
         return false;
     }
     if (params.count("version") && params["version"].as<bool>() == true) {
-        setPlainConsoleLogger();
         auto&& vii = VersionInfoInterface::instance();
-        log() << mongodVersion(vii);
-        vii.logBuildInfo();
+        std::cout << mongodVersion(vii) << std::endl;
+        vii.logBuildInfo(&std::cout);
         return false;
     }
     if (params.count("sysinfo") && params["sysinfo"].as<bool>() == true) {
-        setPlainConsoleLogger();
-        sysRuntimeInfo();
+        BSONObjBuilder obj;
+        appendSysInfo(&obj);
+        std::cout << tojson(obj.done(), ExtendedRelaxedV2_0_0, true) << std::endl;
         return false;
     }
 
     if (params.count("master") || params.count("slave")) {
-        severe() << "Master/slave replication is no longer supported";
+        LOGV2_FATAL_CONTINUE(20881, "Master/slave replication is no longer supported");
         return false;
     }
 
@@ -168,12 +173,9 @@ Status validateMongodOptions(const moe::Environment& params) {
     if (params.count("security.enableEncryption")) {
         // One of the master key sources must be provided
         if (!params.count("security.encryptionKeyFile") &&
-            !params.count("security.vault.serverName") &&
-            !params.count("security.kmip.serverName")) {
+            !params.count("security.vault.serverName")) {
             return Status(ErrorCodes::BadValue,
-                          "security.encryptionKeyFile, security.vault.serverName or "
-                          "security.encryption.kmip.serverName is required when "
-                          "security.enableEncryption is specified");
+                          "security.encryptionKeyFile or security.vault.serverName is required when security.enableEncryption is specified");
         }
     }
 
@@ -264,20 +266,6 @@ Status canonicalizeMongodOptions(moe::Environment* params) {
         }
     }
 
-    // "storage.indexBuildRetry" comes from the config file, so override it if "noIndexBuildRetry"
-    // is set since that comes from the command line.
-    if (params->count("noIndexBuildRetry")) {
-        Status ret = params->set("storage.indexBuildRetry",
-                                 moe::Value(!(*params)["noIndexBuildRetry"].as<bool>()));
-        if (!ret.isOK()) {
-            return ret;
-        }
-        ret = params->remove("noIndexBuildRetry");
-        if (!ret.isOK()) {
-            return ret;
-        }
-    }
-
     // "sharding.clusterRole" comes from the config file, so override it if "configsvr" or
     // "shardsvr" are set since those come from the command line.
     if (params->count("configsvr")) {
@@ -491,36 +479,6 @@ Status storeMongodOptions(const moe::Environment& params) {
         encryptionGlobalParams.vaultDisableTLS = params["security.vault.disableTLSForTesting"].as<bool>();
     }
 
-    if (params.count("security.kmip.serverName")) {
-        encryptionGlobalParams.kmipServerName = params["security.kmip.serverName"].as<std::string>();
-    }
-
-    if (params.count("security.kmip.port")) {
-        encryptionGlobalParams.kmipPort = params["security.kmip.port"].as<int>();
-    }
-
-    if (params.count("security.kmip.serverCAFile")) {
-        encryptionGlobalParams.kmipServerCAFile = params["security.kmip.serverCAFile"].as<std::string>();
-    }
-
-    if (params.count("security.kmip.clientCertificateFile")) {
-        encryptionGlobalParams.kmipClientCertificateFile = params["security.kmip.clientCertificateFile"].as<std::string>();
-    }
-
-    if (params.count("security.kmip.clientCertificatePassword")) {
-        encryptionGlobalParams.kmipClientCertificatePassword =
-            params["security.kmip.clientCertificatePassword"].as<std::string>();
-    }
-
-    if (params.count("security.kmip.keyIdentifier")) {
-        encryptionGlobalParams.kmipKeyIdentifier = params["security.kmip.keyIdentifier"].as<std::string>();
-    }
-
-    if (params.count("security.kmip.rotateMasterKey")) {
-        encryptionGlobalParams.kmipRotateMasterKey =
-            params["security.kmip.rotateMasterKey"].as<bool>();
-    }
-
     if (params.count("security.ldap.authz.queryTemplate")) {
         ldapGlobalParams.ldapQueryTemplate = params["security.ldap.authz.queryTemplate"].as<std::string>();
     }
@@ -529,10 +487,6 @@ Status storeMongodOptions(const moe::Environment& params) {
         serverGlobalParams.cpu = params["cpu"].as<bool>();
     }
 
-    if (params.count("storage.indexBuildRetry")) {
-        serverGlobalParams.indexBuildRetry = params["storage.indexBuildRetry"].as<bool>();
-    }
-
     if (params.count("storage.journal.enabled")) {
         storageGlobalParams.dur = params["storage.journal.enabled"].as<bool>();
     }
@@ -629,6 +583,16 @@ Status storeMongodOptions(const moe::Environment& params) {
         invariant(replSettings.getOplogSizeBytes() > 0);
     }
 
+    if (params.count("storage.oplogMinRetentionHours")) {
+        storageGlobalParams.oplogMinRetentionHours.store(
+            params["storage.oplogMinRetentionHours"].as<double>());
+        if (storageGlobalParams.oplogMinRetentionHours.load() < 0) {
+            return Status(ErrorCodes::BadValue,
+                          "bad --oplogMinRetentionHours, argument must be greater or equal to 0");
+        }
+        invariant(storageGlobalParams.oplogMinRetentionHours.load() >= 0);
+    }
+
     if (params.count("cacheSize")) {
         long x = params["cacheSize"].as<long>();
         if (x <= 0) {
@@ -662,10 +626,9 @@ Status storeMongodOptions(const moe::Environment& params) {
 
             if (params.count("replication.enableMajorityReadConcern") &&
                 !params["replication.enableMajorityReadConcern"].as<bool>()) {
-                warning()
-                    << "Config servers require majority read concern, but it was explicitly "
-                       "disabled. The override is being ignored and the process is continuing "
-                       "with majority read concern enabled.";
+                LOGV2_WARNING(20879,
+                              "Ignoring read concern override as config server requires majority "
+                              "read concern");
             }
             serverGlobalParams.enableMajorityReadConcern = true;
 
@@ -719,11 +682,9 @@ Status storeMongodOptions(const moe::Environment& params) {
 
     // Check if we are 32 bit and have not explicitly specified any journaling options
     if (sizeof(void*) == 4 && !params.count("storage.journal.enabled")) {
-        // trying to make this stand out more like startup warnings
-        log() << endl;
-        warning() << "32-bit servers don't have journaling enabled by default. "
-                  << "Please use --journal if you want durability.";
-        log() << endl;
+        LOGV2_WARNING(20880,
+                      "32-bit servers don't have journaling enabled by default. Please use "
+                      "--journal if you want durability");
     }
 
     bool isClusterRoleShard = params.count("shardsvr");
