@@ -38,15 +38,19 @@
 #include "mongo/base/string_data.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/repl/optime.h"
+#include "mongo/logv2/log_attr.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
 
-const size_t MaxDatabaseNameLen = 128;  // max str len for the db name, including null char
-
 class NamespaceString {
 public:
+    constexpr static size_t MaxDatabaseNameLen =
+        128;  // max str len for the db name, including null char
+    constexpr static size_t MaxNSCollectionLenFCV42 = 120U;
+    constexpr static size_t MaxNsCollectionLen = 255;
+
     // Reserved system namespaces
 
     // Namespace for the admin database
@@ -95,14 +99,20 @@ public:
     // Namespace for storing the persisted state of transaction coordinators.
     static const NamespaceString kTransactionCoordinatorsNamespace;
 
+    // Namespace for storing the persisted state of migration coordinators.
+    static const NamespaceString kMigrationCoordinatorsNamespace;
+
     // Namespace for replica set configuration settings.
     static const NamespaceString kSystemReplSetNamespace;
 
     // Namespace for index build entries.
     static const NamespaceString kIndexBuildEntryNamespace;
 
-    // Namespace used for storing retryable findAndModify images.
-    static const NamespaceString kConfigImagesNamespace;
+    // Namespace for pending range deletions.
+    static const NamespaceString kRangeDeletionNamespace;
+
+    // Namespace for balancer settings and default read and write concerns.
+    static const NamespaceString kConfigSettingsNamespace;
 
     /**
      * Constructs an empty NamespaceString.
@@ -159,28 +169,6 @@ public:
      */
     static NamespaceString makeListCollectionsNSS(StringData dbName);
 
-    /**
-     * Note that these values are derived from the mmap_v1 implementation and that is the only
-     * reason they are constrained as such.
-     */
-    enum MaxNsLenValue {
-        // Maximum possible length of name any namespace, including special ones like $extra.
-        // This includes rum for the NUL byte so it can be used when sizing buffers.
-        MaxNsLenWithNUL = 128,
-
-        // MaxNsLenWithNUL excluding the NUL byte. Use this when comparing std::string lengths.
-        MaxNsLen = MaxNsLenWithNUL - 1,
-
-        // Maximum allowed length of fully qualified namespace name of any real collection.
-        // Does not include NUL so it can be directly compared to std::string lengths.
-        MaxNsCollectionLen = MaxNsLen - 7 /*strlen(".$extra")*/,
-
-        // The maximum namespace length of sharded collections is less than that of unsharded ones
-        // since the namespace of the cached chunks metadata, local to each shard, is composed by
-        // the namespace of the related sharded collection (i.e., config.cache.chunks.<ns>).
-        MaxNsShardedCollectionLen = 100 /* MaxNsCollectionLen - len(ChunkType::ShardNSPrefix) */,
-    };
-
     /**
      * NOTE: DollarInDbNameBehavior::allow is deprecated.
      *
@@ -228,6 +216,9 @@ public:
     bool isSystem() const {
         return coll().startsWith("system.");
     }
+    bool isNormalCollection() const {
+        return !isSystem() && !(isLocal() && coll().startsWith("replset."));
+    }
     bool isAdminDB() const {
         return db() == kAdminDb;
     }
@@ -252,9 +243,6 @@ public:
     bool isOplog() const {
         return oplog(_ns);
     }
-    bool isSpecial() const {
-        return special(_ns);
-    }
     bool isOnInternalDb() const {
         if (db() == kAdminDb)
             return true;
@@ -264,25 +252,27 @@ public:
             return true;
         return false;
     }
-    bool isNormal() const {
-        return normal(_ns);
-    }
+
     bool isOrphanCollection() const {
         return db() == kOrphanCollectionDb && coll().startsWith(kOrphanCollectionPrefix);
     }
 
     /**
-     * Returns whether the NamespaceString references a special collection that cannot be used for
-     * generic data storage.
+     * Returns whether the specified namespace is used for internal purposes only and can
+     * never be marked as anything other than UNSHARDED.
      */
-    bool isVirtualized() const {
-        return virtualized(_ns);
-    }
+    bool isNamespaceAlwaysUnsharded() const;
 
     /**
      * Returns whether a namespace is replicated, based only on its string value. One notable
      * omission is that map reduce `tmp.mr` collections may or may not be replicated. Callers must
      * decide how to handle that case separately.
+     *
+     * Note: This function considers "replicated" to be any namespace that should be timestamped.
+     * Not all collections that are timestamped are replicated explicitly through the oplog.
+     * Drop-pending collections are a notable example. Please use
+     * ReplicationCoordinator::isOplogDisabledForNS to determine if a namespace gets logged in the
+     * oplog.
      */
     bool isReplicated() const;
 
@@ -314,9 +304,6 @@ public:
      *
      * Example:
      *     test.foo -> test.system.drop.<timestamp seconds>i<timestamp increment>t<term>.foo
-     *
-     * Original collection name may be truncated so that the generated namespace length does not
-     * exceed MaxNsCollectionLen.
      */
     NamespaceString makeDropPendingNamespace(const repl::OpTime& opTime) const;
 
@@ -326,12 +313,6 @@ public:
      */
     StatusWith<repl::OpTime> getDropPendingNamespaceOpTime() const;
 
-    /**
-     * Checks if this namespace is valid as a target namespace for a rename operation, given
-     * the length of the longest index name in the source collection.
-     */
-    Status checkLengthForRename(const std::string::size_type longestIndexNameLength) const;
-
     /**
      * Returns true if the namespace is valid. Special namespaces for internal use are considered as
      * valid.
@@ -349,19 +330,6 @@ public:
         return {db(), "$cmd"};
     }
 
-    /**
-     * Returns index namespace for an index in this collection namespace.
-     */
-    NamespaceString makeIndexNamespace(StringData indexName) const;
-
-    /**
-     * @return true if ns is 'normal'.  A "$" is used for namespaces holding index data,
-     * which do not contain BSON objects in their records. ("oplog.$main" is the exception)
-     */
-    static bool normal(StringData ns) {
-        return !virtualized(ns);
-    }
-
     /**
      * @return true if the ns is an oplog one, otherwise false.
      */
@@ -369,17 +337,6 @@ public:
         return ns.startsWith("local.oplog.");
     }
 
-    static bool special(StringData ns) {
-        return !normal(ns) || ns.substr(ns.find('.')).startsWith(".system.");
-    }
-
-    /**
-     * Check if `ns` references a special collection that cannot be used for generic data storage.
-     */
-    static bool virtualized(StringData ns) {
-        return ns.find('$') != std::string::npos && ns != "local.oplog.$main";
-    }
-
     /**
      * samples:
      *   good
@@ -451,6 +408,10 @@ public:
         return H::combine(std::move(h), nss._ns);
     }
 
+    friend auto logAttrs(const NamespaceString& nss) {
+        return "namespace"_attr = nss;
+    }
+
 private:
     std::string _ns;
     size_t _dotIndex;
@@ -474,10 +435,20 @@ public:
         return _uuid;
     }
 
+    /**
+     * Returns database name if this object was initialized with a UUID.
+     */
     const std::string& dbname() const {
         return _dbname;
     }
 
+    /**
+     * Returns database name derived from either '_nss' or '_dbname'.
+     */
+    StringData db() const {
+        return _nss ? _nss->db() : StringData(_dbname);
+    }
+
     std::string toString() const;
 
 private:
@@ -502,10 +473,13 @@ StringBuilder& operator<<(StringBuilder& builder, const NamespaceStringOrUUID& n
 inline StringData nsToDatabaseSubstring(StringData ns) {
     size_t i = ns.find('.');
     if (i == std::string::npos) {
-        massert(10078, "nsToDatabase: db too long", ns.size() < MaxDatabaseNameLen);
+        massert(
+            10078, "nsToDatabase: db too long", ns.size() < NamespaceString::MaxDatabaseNameLen);
         return ns;
     }
-    massert(10088, "nsToDatabase: db too long", i < static_cast<size_t>(MaxDatabaseNameLen));
+    massert(10088,
+            "nsToDatabase: db too long",
+            i < static_cast<size_t>(NamespaceString::MaxDatabaseNameLen));
     return ns.substr(0, i);
 }
 
