@@ -88,7 +88,7 @@ private:
 
 class DocumentSourceGroup final : public DocumentSource {
 public:
-    using Accumulators = std::vector<boost::intrusive_ptr<Accumulator>>;
+    using Accumulators = std::vector<boost::intrusive_ptr<AccumulatorState>>;
     using GroupsMap = ValueUnorderedMap<Accumulators>;
 
     static constexpr StringData kStageName = "$group"_sd;
@@ -96,7 +96,6 @@ public:
     boost::intrusive_ptr<DocumentSource> optimize() final;
     DepsTracker::State getDependencies(DepsTracker* deps) const final;
     Value serialize(boost::optional<ExplainOptions::Verbosity> explain = boost::none) const final;
-    GetNextResult getNext() final;
     const char* getSourceName() const final;
     GetModPathsReturn getModifiedPaths() const final;
     StringMap<boost::intrusive_ptr<Expression>> getIdFields() const;
@@ -126,12 +125,12 @@ public:
                                      DiskUseRequirement::kWritesTmpData,
                                      FacetRequirement::kAllowed,
                                      TransactionRequirement::kAllowed,
-                                     LookupRequirement::kAllowed);
+                                     LookupRequirement::kAllowed,
+                                     UnionRequirement::kAllowed);
         constraints.canSwapWithMatch = true;
         return constraints;
     }
 
-
     /**
      * Add an accumulator, which will become a field in each Document that results from grouping.
      */
@@ -179,6 +178,7 @@ public:
         const;
 
 protected:
+    GetNextResult doGetNext() final;
     void doDispose() final;
 
 private:
