@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -39,10 +39,8 @@
 #include "mongo/base/init.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/service_context.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/duration.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -59,14 +57,16 @@ std::unique_ptr<_timelib_time, TimeZone::TimelibTimeDeleter> createTimelibTime()
 // Converts a date to a number of seconds, being careful to round appropriately for negative numbers
 // of seconds.
 long long seconds(Date_t date) {
-    // We want the division below to truncate toward -inf rather than 0
-    // eg Dec 31, 1969 23:59:58.001 should be -2 seconds rather than -1
-    // This is needed to get the correct values from coerceToTM
-    constexpr auto needsRounding = -1999 / 1000 != -2;  // This is implementaiton defined.
-    if (auto millis = date.toMillisSinceEpoch(); millis < 0 && millis % 1000 != 0 && needsRounding)
-        return durationCount<Seconds>(Milliseconds(millis)) - 1ll;
-    else
-        return durationCount<Seconds>(Milliseconds(millis));
+    auto millis = date.toMillisSinceEpoch();
+    if (millis < 0) {
+        // We want the division below to truncate toward -inf rather than 0
+        // eg Dec 31, 1969 23:59:58.001 should be -2 seconds rather than -1
+        // This is needed to get the correct values from coerceToTM
+        if (-1999 / 1000 != -2) {  // this is implementation defined
+            millis -= 1000 - 1;
+        }
+    }
+    return durationCount<Seconds>(Milliseconds(millis));
 }
 
 //
@@ -181,8 +181,7 @@ void TimeZoneDatabase::loadTimeZoneInfo(
                                << entry.id << "\": " << timelib_get_error_message(errorCode)});
         }
 
-        invariant(errorCode == TIMELIB_ERROR_NO_ERROR ||
-                  errorCode == TIMELIB_ERROR_EMPTY_POSIX_STRING);
+        invariant(errorCode == TIMELIB_ERROR_NO_ERROR);
         _timeZones[entry.id] = TimeZone{tzInfo};
     }
 }
@@ -191,7 +190,7 @@ TimeZone TimeZoneDatabase::utcZone() {
     return TimeZone{nullptr};
 }
 
-static timelib_tzinfo* timezonedatabase_gettzinfowrapper(const char* tz_id,
+static timelib_tzinfo* timezonedatabase_gettzinfowrapper(char* tz_id,
                                                          const _timelib_tzdb* db,
                                                          int* error) {
     return nullptr;
@@ -316,7 +315,7 @@ boost::optional<Seconds> TimeZoneDatabase::parseUtcOffset(StringData offsetSpec)
         // ±HH
         if (offsetSpec.size() == 3 && isdigit(offsetSpec[1]) && isdigit(offsetSpec[2])) {
             int offset;
-            if (parseNumberFromStringWithBase(offsetSpec.substr(1, 2), 10, &offset).isOK()) {
+            if (NumberParser().base(10)(offsetSpec.substr(1, 2), &offset).isOK()) {
                 return duration_cast<Seconds>(Hours(bias * offset));
             }
             return boost::none;
@@ -326,7 +325,7 @@ boost::optional<Seconds> TimeZoneDatabase::parseUtcOffset(StringData offsetSpec)
         if (offsetSpec.size() == 5 && isdigit(offsetSpec[1]) && isdigit(offsetSpec[2]) &&
             isdigit(offsetSpec[3]) && isdigit(offsetSpec[4])) {
             int offset;
-            if (parseNumberFromStringWithBase(offsetSpec.substr(1, 4), 10, &offset).isOK()) {
+            if (NumberParser().base(10)(offsetSpec.substr(1, 4), &offset).isOK()) {
                 return duration_cast<Seconds>(Hours(bias * (offset / 100L)) +
                                               Minutes(bias * (offset % 100)));
             }
@@ -337,10 +336,10 @@ boost::optional<Seconds> TimeZoneDatabase::parseUtcOffset(StringData offsetSpec)
         if (offsetSpec.size() == 6 && isdigit(offsetSpec[1]) && isdigit(offsetSpec[2]) &&
             offsetSpec[3] == ':' && isdigit(offsetSpec[4]) && isdigit(offsetSpec[5])) {
             int hourOffset, minuteOffset;
-            if (!parseNumberFromStringWithBase(offsetSpec.substr(1, 2), 10, &hourOffset).isOK()) {
+            if (!NumberParser().base(10)(offsetSpec.substr(1, 2), &hourOffset).isOK()) {
                 return boost::none;
             }
-            if (!parseNumberFromStringWithBase(offsetSpec.substr(4, 2), 10, &minuteOffset).isOK()) {
+            if (!NumberParser().base(10)(offsetSpec.substr(4, 2), &minuteOffset).isOK()) {
                 return boost::none;
             }
             return duration_cast<Seconds>(Hours(bias * hourOffset) + Minutes(bias * minuteOffset));
@@ -364,6 +363,16 @@ TimeZone TimeZoneDatabase::getTimeZone(StringData timeZoneId) const {
               str::stream() << "unrecognized time zone identifier: \"" << timeZoneId << "\"");
 }
 
+std::vector<std::string> TimeZoneDatabase::getTimeZoneStrings() const {
+    std::vector<std::string> timeZoneStrings = {};
+
+    for (auto const& timezone : _timeZones) {
+        timeZoneStrings.push_back(timezone.first);
+    }
+
+    return timeZoneStrings;
+}
+
 void TimeZone::adjustTimeZone(timelib_time* timelibTime) const {
     if (isTimeZoneIDZone()) {
         timelib_set_timezone(timelibTime, _tzInfo.get());
@@ -553,11 +562,9 @@ void TimeZone::validateFromStringFormat(StringData format) {
     return validateFormat(format, kDateFromStringFormatMap);
 }
 
-StatusWith<std::string> TimeZone::formatDate(StringData format, Date_t date) const {
+std::string TimeZone::formatDate(StringData format, Date_t date) const {
     StringBuilder formatted;
-    if (auto status = outputDateWithFormat(formatted, format, date); status != Status::OK())
-        return status;
-    else
-        return formatted.str();
+    outputDateWithFormat(formatted, format, date);
+    return formatted.str();
 }
 }  // namespace mongo
