@@ -387,16 +387,12 @@ TEST(NewYorkTimeBeforeEpoch, DoesComputeISOWeek) {
 TEST(UTCTimeBeforeEpoch, DoesFormatDate) {
     // Tuesday, Dec 30, 1969 13:42:23:211
     auto date = Date_t::fromMillisSinceEpoch(-123456789LL);
-    auto result = TimeZoneDatabase::utcZone().formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "1969/12/30 13:42:23:211, dayOfYear: 364, dayOfWeek: 3, week: 52, isoYear: 1970, "
-        "isoWeek: 01, isoDayOfWeek: 2, percent: %",
-        result.getValue());
+    ASSERT_EQ(TimeZoneDatabase::utcZone().formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                                     date),
+              "1969/12/30 13:42:23:211, dayOfYear: 364, dayOfWeek: 3, week: 52, isoYear: 1970, "
+              "isoWeek: 01, isoDayOfWeek: 2, percent: %");
 }
 
 TEST(NewYorkTimeBeforeEpoch, DoesFormatDate) {
@@ -404,28 +400,23 @@ TEST(NewYorkTimeBeforeEpoch, DoesFormatDate) {
 
     // Tuesday, Dec 30, 1969 13:42:23:211
     auto date = Date_t::fromMillisSinceEpoch(-123456789LL);
-    auto result = newYorkZone.formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "1969/12/30 08:42:23:211, dayOfYear: 364, dayOfWeek: 3, week: 52, isoYear: 1970, "
-        "isoWeek: 01, isoDayOfWeek: 2, percent: %",
-        result.getValue());
+    ASSERT_EQ(newYorkZone.formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                     date),
+              "1969/12/30 08:42:23:211, dayOfYear: 364, dayOfWeek: 3, week: 52, isoYear: 1970, "
+              "isoWeek: 01, isoDayOfWeek: 2, percent: %");
 }
 
 TEST(UTCTimeBeforeEpoch, DoesOutputFormatDate) {
     // Tuesday, Dec 30, 1969 13:42:23:211
     auto date = Date_t::fromMillisSinceEpoch(-123456789LL);
     std::ostringstream os;
-    ASSERT_OK(TimeZoneDatabase::utcZone().outputDateWithFormat(
-        os,
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date));
+    TimeZoneDatabase::utcZone().outputDateWithFormat(os,
+                                                     "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                                     date);
     ASSERT_EQ(os.str(),
               "1969/12/30 13:42:23:211, dayOfYear: 364, dayOfWeek: 3, week: 52, isoYear: 1970, "
               "isoWeek: 01, isoDayOfWeek: 2, percent: %");
@@ -437,11 +428,11 @@ TEST(NewYorkTimeBeforeEpoch, DoesOutputFormatDate) {
     // Tuesday, Dec 30, 1969 13:42:23:211
     auto date = Date_t::fromMillisSinceEpoch(-123456789LL);
     std::ostringstream os;
-    ASSERT_OK(newYorkZone.outputDateWithFormat(os,
-                                               "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-                                               "dayOfWeek: %w, week: %U, isoYear: %G, "
-                                               "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-                                               date));
+    newYorkZone.outputDateWithFormat(os,
+                                     "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                     date);
     ASSERT_EQ(os.str(),
               "1969/12/30 08:42:23:211, dayOfYear: 364, dayOfWeek: 3, week: 52, isoYear: 1970, "
               "isoWeek: 01, isoDayOfWeek: 2, percent: %");
@@ -576,16 +567,12 @@ TEST(NewYorkTimeAtEpoch, DoesComputeISOWeek) {
 TEST(UTCTimeAtEpoch, DoesFormatDate) {
     // Thu, Jan 1, 1970 00:00:00:000
     auto date = Date_t::fromMillisSinceEpoch(0);
-    auto result = TimeZoneDatabase::utcZone().formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "1970/01/01 00:00:00:000, dayOfYear: 001, dayOfWeek: 5, week: 00, isoYear: 1970, "
-        "isoWeek: 01, isoDayOfWeek: 4, percent: %",
-        result.getValue());
+    ASSERT_EQ(TimeZoneDatabase::utcZone().formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                                     date),
+              "1970/01/01 00:00:00:000, dayOfYear: 001, dayOfWeek: 5, week: 00, isoYear: 1970, "
+              "isoWeek: 01, isoDayOfWeek: 4, percent: %");
 }
 
 TEST(NewYorkTimeAtEpoch, DoesFormatDate) {
@@ -593,27 +580,22 @@ TEST(NewYorkTimeAtEpoch, DoesFormatDate) {
 
     // Thu, Jan 1, 1970 00:00:00:000Z
     auto date = Date_t::fromMillisSinceEpoch(0);
-    auto result = newYorkZone.formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "1969/12/31 19:00:00:000, dayOfYear: 365, dayOfWeek: 4, week: 52, isoYear: 1970, "
-        "isoWeek: 01, isoDayOfWeek: 3, percent: %",
-        result.getValue());
+    ASSERT_EQ(newYorkZone.formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                     date),
+              "1969/12/31 19:00:00:000, dayOfYear: 365, dayOfWeek: 4, week: 52, isoYear: 1970, "
+              "isoWeek: 01, isoDayOfWeek: 3, percent: %");
 }
 
 TEST(UTCTimeAtEpoch, DoesOutputFormatDate) {
     auto date = Date_t::fromMillisSinceEpoch(0);
     std::ostringstream os;
-    ASSERT_OK(TimeZoneDatabase::utcZone().outputDateWithFormat(
-        os,
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date));
+    TimeZoneDatabase::utcZone().outputDateWithFormat(os,
+                                                     "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                                     date);
     ASSERT_EQ(os.str(),
               "1970/01/01 00:00:00:000, dayOfYear: 001, dayOfWeek: 5, week: 00, isoYear: 1970, "
               "isoWeek: 01, isoDayOfWeek: 4, percent: %");
@@ -623,11 +605,11 @@ TEST(NewYorkTimeAtEpoch, DoesOutputFormatDate) {
     auto newYorkZone = kDefaultTimeZoneDatabase.getTimeZone("America/New_York");
     auto date = Date_t::fromMillisSinceEpoch(0);
     std::ostringstream os;
-    ASSERT_OK(newYorkZone.outputDateWithFormat(os,
-                                               "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-                                               "dayOfWeek: %w, week: %U, isoYear: %G, "
-                                               "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-                                               date));
+    newYorkZone.outputDateWithFormat(os,
+                                     "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                     date);
     ASSERT_EQ(os.str(),
               "1969/12/31 19:00:00:000, dayOfYear: 365, dayOfWeek: 4, week: 52, isoYear: 1970, "
               "isoWeek: 01, isoDayOfWeek: 3, percent: %");
@@ -897,60 +879,47 @@ TEST(NewYorkTimeAfterEpoch, DoesComputeISOWeek) {
 TEST(UTCTimeAfterEpoch, DoesFormatDate) {
     // Tue, Jun 6, 2017 19:38:43:234.
     auto date = Date_t::fromMillisSinceEpoch(1496777923234LL);
-    auto result = TimeZoneDatabase::utcZone().formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "2017/06/06 19:38:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
-        "isoWeek: 23, isoDayOfWeek: 2, percent: %",
-        result.getValue());
+    ASSERT_EQ(TimeZoneDatabase::utcZone().formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                                     date),
+              "2017/06/06 19:38:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
+              "isoWeek: 23, isoDayOfWeek: 2, percent: %");
 }
 
 TEST(NewYorkTimeAfterEpoch, DoesFormatDate) {
     // 2017-06-06T19:38:43:234Z (Tuesday).
     auto date = Date_t::fromMillisSinceEpoch(1496777923234LL);
     auto newYorkZone = kDefaultTimeZoneDatabase.getTimeZone("America/New_York");
-    auto result = newYorkZone.formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "2017/06/06 15:38:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
-        "isoWeek: 23, isoDayOfWeek: 2, percent: %",
-        result.getValue());
+    ASSERT_EQ(newYorkZone.formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                     date),
+              "2017/06/06 15:38:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
+              "isoWeek: 23, isoDayOfWeek: 2, percent: %");
 }
 
 TEST(UTCOffsetAfterEpoch, DoesFormatDate) {
     // 2017-06-06T19:38:43:234Z (Tuesday).
     auto date = Date_t::fromMillisSinceEpoch(1496777923234LL);
     auto offsetSpec = kDefaultTimeZoneDatabase.getTimeZone("+02:30");
-    auto result = offsetSpec.formatDate(
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date);
-    ASSERT_OK(result);
-    ASSERT_EQ(
-        "2017/06/06 22:08:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
-        "isoWeek: 23, isoDayOfWeek: 2, percent: %",
-        result.getValue());
+    ASSERT_EQ(offsetSpec.formatDate("%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                    "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                    "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                    date),
+              "2017/06/06 22:08:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
+              "isoWeek: 23, isoDayOfWeek: 2, percent: %");
 }
 
 TEST(UTCTimeAfterEpoch, DoesOutputFormatDate) {
     // Tue, Jun 6, 2017 19:38:43:234.
     auto date = Date_t::fromMillisSinceEpoch(1496777923234LL);
     std::ostringstream os;
-    ASSERT_OK(TimeZoneDatabase::utcZone().outputDateWithFormat(
-        os,
-        "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-        "dayOfWeek: %w, week: %U, isoYear: %G, "
-        "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-        date));
+    TimeZoneDatabase::utcZone().outputDateWithFormat(os,
+                                                     "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                                     date);
     ASSERT_EQ(os.str(),
               "2017/06/06 19:38:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
               "isoWeek: 23, isoDayOfWeek: 2, percent: %");
@@ -961,11 +930,11 @@ TEST(NewYorkTimeAfterEpoch, DoesOutputFormatDate) {
     auto date = Date_t::fromMillisSinceEpoch(1496777923234LL);
     std::ostringstream os;
     auto newYorkZone = kDefaultTimeZoneDatabase.getTimeZone("America/New_York");
-    ASSERT_OK(newYorkZone.outputDateWithFormat(os,
-                                               "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
-                                               "dayOfWeek: %w, week: %U, isoYear: %G, "
-                                               "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
-                                               date));
+    newYorkZone.outputDateWithFormat(os,
+                                     "%Y/%m/%d %H:%M:%S:%L, dayOfYear: %j, "
+                                     "dayOfWeek: %w, week: %U, isoYear: %G, "
+                                     "isoWeek: %V, isoDayOfWeek: %u, percent: %%",
+                                     date);
     ASSERT_EQ(os.str(),
               "2017/06/06 15:38:43:234, dayOfYear: 157, dayOfWeek: 3, week: 23, isoYear: 2017, "
               "isoWeek: 23, isoDayOfWeek: 2, percent: %");
@@ -996,42 +965,37 @@ TEST(DateFormat, ThrowsUserExceptionIfGivenUnmatchedPercent) {
                        18535);
 }
 
-TEST(DateFormat, ProducesNonOKStatusGivenDateBeforeYear0) {
+TEST(DateFormat, ThrowsUserExceptionIfGivenDateBeforeYear0) {
     const long long kMillisPerYear = 31556926000;
 
-    ASSERT_EQ(18537,
-              TimeZoneDatabase::utcZone()
-                  .formatDate("%Y", Date_t::fromMillisSinceEpoch(-(kMillisPerYear * 1971)))
-                  .getStatus()
-                  .code());
+    ASSERT_THROWS_CODE(TimeZoneDatabase::utcZone().formatDate(
+                           "%Y", Date_t::fromMillisSinceEpoch(-(kMillisPerYear * 1971))),
+                       AssertionException,
+                       18537);
 
-    ASSERT_EQ(18537,
-              TimeZoneDatabase::utcZone()
-                  .formatDate("%G", Date_t::fromMillisSinceEpoch(-(kMillisPerYear * 1971)))
-                  .getStatus()
-                  .code());
+    ASSERT_THROWS_CODE(TimeZoneDatabase::utcZone().formatDate(
+                           "%G", Date_t::fromMillisSinceEpoch(-(kMillisPerYear * 1971))),
+                       AssertionException,
+                       18537);
 
-    auto result = TimeZoneDatabase::utcZone().formatDate(
-        "%Y", Date_t::fromMillisSinceEpoch(-(kMillisPerYear * 1970)));
-    ASSERT_OK(result);
-    ASSERT_EQ("0000", result.getValue());
+    ASSERT_EQ("0000",
+              TimeZoneDatabase::utcZone().formatDate(
+                  "%Y", Date_t::fromMillisSinceEpoch(-(kMillisPerYear * 1970))));
 }
 
-TEST(DateFormat, ProducesNonOKStatusIfGivenDateAfterYear9999) {
-    ASSERT_EQ(18537,
-              TimeZoneDatabase::utcZone().formatDate("%Y", Date_t::max()).getStatus().code());
+TEST(DateFormat, ThrowsUserExceptionIfGivenDateAfterYear9999) {
+    ASSERT_THROWS_CODE(
+        TimeZoneDatabase::utcZone().formatDate("%Y", Date_t::max()), AssertionException, 18537);
 
-    ASSERT_EQ(18537,
-              TimeZoneDatabase::utcZone().formatDate("%G", Date_t::max()).getStatus().code());
+    ASSERT_THROWS_CODE(
+        TimeZoneDatabase::utcZone().formatDate("%G", Date_t::max()), AssertionException, 18537);
 }
 
 TEST(DateFromString, CorrectlyParsesStringThatMatchesFormat) {
     auto input = "2017-07-04T10:56:02Z";
     auto format = "%Y-%m-%dT%H:%M:%SZ"_sd;
     auto date = kDefaultTimeZoneDatabase.fromString(input, kDefaultTimeZone, format);
-    auto result = TimeZoneDatabase::utcZone().formatDate(format, date);
-    ASSERT_OK(result);
-    ASSERT_EQ(input, result.getValue());
+    ASSERT_EQ(TimeZoneDatabase::utcZone().formatDate(format, date), input);
 }
 
 TEST(DateFromString, RejectsStringWithInvalidYearFormat) {
