@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/db/query/planner_ixselect.h"
 
@@ -42,13 +42,12 @@
 #include "mongo/db/matcher/expression_geo.h"
 #include "mongo/db/matcher/expression_internal_expr_eq.h"
 #include "mongo/db/matcher/expression_text.h"
-#include "mongo/db/query/canonical_query_encoder.h"
 #include "mongo/db/query/collation/collator_interface.h"
 #include "mongo/db/query/index_tag.h"
 #include "mongo/db/query/indexability.h"
 #include "mongo/db/query/planner_wildcard_helpers.h"
 #include "mongo/db/query/query_planner_common.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
@@ -174,15 +173,12 @@ bool QueryPlannerIXSelect::notEqualsNullCanUseIndex(const IndexEntry& index,
     }
 }
 
-bool QueryPlannerIXSelect::canUseIndexForNin(const InMatchExpression* ime) {
-    const std::vector<BSONElement>& inList = ime->getEqualities();
-    auto containsNull = [](const BSONElement& elt) { return elt.type() == jstNULL; };
-    auto containsEmptyArray = [](const BSONElement& elt) {
-        return elt.type() == Array && elt.embeddedObject().isEmpty();
-    };
-    return ime->getRegexes().empty() && inList.size() == 2 &&
-        std::any_of(inList.begin(), inList.end(), containsNull) &&
-        std::any_of(inList.begin(), inList.end(), containsEmptyArray);
+static double fieldWithDefault(const BSONObj& infoObj, const string& name, double def) {
+    BSONElement e = infoObj[name];
+    if (e.isNumber()) {
+        return e.numberDouble();
+    }
+    return def;
 }
 
 /**
@@ -256,11 +252,15 @@ void QueryPlannerIXSelect::getFields(const MatchExpression* node,
     // Leaf nodes with a path and some array operators.
     if (Indexability::nodeCanUseIndexOnOwnField(node)) {
         out->insert(prefix + node->path().toString());
-    } else if (Indexability::arrayUsesIndexOnChildren(node) && !node->path().empty()) {
+    } else if (Indexability::arrayUsesIndexOnChildren(node)) {
         // If the array uses an index on its children, it's something like
         // {foo : {$elemMatch: {bar: 1}}}, in which case the predicate is really over foo.bar.
-        // Note we skip empty path components since they are not allowed in index key patterns.
-        prefix += node->path().toString() + ".";
+        //
+        // When we have {foo: {$all: [{$elemMatch: {a: 1}}], the path of the embedded elemMatch
+        // is empty. We don't want to append a dot in that case as the field would be foo..a.
+        if (!node->path().empty()) {
+            prefix += node->path().toString() + ".";
+        }
 
         for (size_t i = 0; i < node->numChildren(); ++i) {
             getFields(node->getChild(i), prefix, out);
@@ -287,15 +287,24 @@ std::vector<IndexEntry> QueryPlannerIXSelect::findIndexesByHint(
         auto hintName = firstHintElt.valueStringData();
         for (auto&& entry : allIndices) {
             if (entry.identifier.catalogName == hintName) {
-                LOG(5) << "Hint by name specified, restricting indices to "
-                       << entry.keyPattern.toString();
+                LOGV2_DEBUG(20952,
+                            5,
+                            "Hint by name specified, restricting indices to {keyPattern}",
+                            "Hint by name specified, restricting indices",
+                            "name"_attr = entry.identifier.catalogName,
+                            "keyPattern"_attr = entry.keyPattern);
                 out.push_back(entry);
             }
         }
     } else {
         for (auto&& entry : allIndices) {
             if (SimpleBSONObjComparator::kInstance.evaluate(entry.keyPattern == hintedIndex)) {
-                LOG(5) << "Hint specified, restricting indices to " << hintedIndex.toString();
+                LOGV2_DEBUG(20953,
+                            5,
+                            "Hint specified, restricting indices to {keyPattern}",
+                            "Hint specified, restricting indices",
+                            "name"_attr = entry.identifier.catalogName,
+                            "keyPattern"_attr = entry.keyPattern);
                 out.push_back(entry);
             }
         }
@@ -396,19 +405,28 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
 
         // There are restrictions on when we can use the index if the expression is a NOT.
         if (exprtype == MatchExpression::NOT) {
-            // Don't allow indexed NOT on special index types such as geo or text indices. The
-            // exception is that $** indexes may be used for {$ne: null} queries.
+            // Don't allow indexed NOT on special index types such as geo or text indices. There are
+            // two exceptions to this rule:
+            // - Wildcard indexes can answer {$ne: null} queries. We allow wildcard indexes to pass
+            //   the test here because we subsequently enforce that {$ne:null} is the only accepted
+            //   negation predicate for sparse indexes, and wildcard indexes are always sparse.
+            // - Any non-hashed field in a compound hashed index can answer negated predicates. We
+            //   have already determined that the specific field under consideration is not hashed,
+            //   so we can safely permit a hashed index to pass the test below.
             //
             // TODO: SERVER-30994 should remove this check entirely and allow $not on the
             // 'non-special' fields of non-btree indices (e.g. {a: 1, geo: "2dsphere"}).
             if (INDEX_BTREE != index.type && INDEX_WILDCARD != index.type &&
-                !isChildOfElemMatchValue) {
+                INDEX_HASHED != index.type && !isChildOfElemMatchValue) {
                 return false;
             }
 
             const auto* child = node->getChild(0);
             const MatchExpression::MatchType childtype = child->matchType();
-            const bool isNotEqualsNull = isQueryNegatingEqualToNull(node);
+            const bool isNotEqualsNull =
+                (childtype == MatchExpression::EQ &&
+                 static_cast<const ComparisonMatchExpression*>(child)->getData().type() ==
+                     BSONType::jstNULL);
 
             // The type being INDEX_WILDCARD implies that the index is sparse.
             invariant(!(index.type == INDEX_WILDCARD && !index.sparse));
@@ -431,6 +449,10 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
                 return false;
             }
 
+            if (isEqualsArrayOrNotInArray(child)) {
+                return false;
+            }
+
             // Most of the time we can't use a multikey index for a $ne: null query, however there
             // are a few exceptions around $elemMatch.
             const bool canUseIndexForNeNull =
@@ -442,12 +464,6 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             // If it's a negated $in, it can't have any REGEX's inside.
             if (MatchExpression::MATCH_IN == childtype) {
                 InMatchExpression* ime = static_cast<InMatchExpression*>(node->getChild(0));
-
-                if (canUseIndexForNin(ime)) {
-                    // This is a case that we know is supported.
-                    return true;
-                }
-
                 if (!ime->getRegexes().empty()) {
                     return false;
                 }
@@ -458,10 +474,6 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
                     return false;
                 }
             }
-
-            if (isEqualsArrayOrNotInArray(child)) {
-                return false;
-            }
         }
 
         // If this is an $elemMatch value, make sure _all_ of the children can use the index.
@@ -470,17 +482,7 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             newContext.fullPathToParentElemMatch = fullPathToNode;
             newContext.innermostParentElemMatch = static_cast<ElemMatchValueMatchExpression*>(node);
 
-            FieldRef path(fullPathToNode);
-            // If the index path has at least two components, and the last component of the path is
-            // numeric, this component could be an array index because the preceding path component
-            // may contain an array. Currently it is not known whether the preceding path component
-            // could be an array because indexes which positionally index array elements are not
-            // considered multikey.
-            if (path.numParts() > 1 && path.isNumericPathComponentStrict(path.numParts() - 1)) {
-                return false;
-            }
-
-            auto* children = node->getChildVector();
+            auto&& children = node->getChildVector();
             if (!std::all_of(children->begin(), children->end(), [&](MatchExpression* child) {
                     const auto newPath = fullPathToNode.toString() + child->path();
                     return _compatible(
@@ -533,18 +535,7 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
         // above.
         MONGO_UNREACHABLE;
     } else if (IndexNames::HASHED == indexedFieldType) {
-        if (index.sparse && !nodeIsSupportedBySparseIndex(node, isChildOfElemMatchValue)) {
-            return false;
-        }
-
-        if (ComparisonMatchExpressionBase::isEquality(exprtype)) {
-            return true;
-        }
-        if (exprtype == MatchExpression::MATCH_IN) {
-            const InMatchExpression* expr = static_cast<const InMatchExpression*>(node);
-            return expr->getRegexes().empty();
-        }
-        return false;
+        return nodeIsSupportedByHashedIndex(node);
     } else if (IndexNames::GEO_2DSPHERE == indexedFieldType) {
         if (exprtype == MatchExpression::GEO) {
             // within or intersect.
@@ -581,7 +572,7 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
             const CapWithCRS* cap = gc.getCapGeometryHack();
 
             // 2d indices can answer centerSphere queries.
-            if (NULL == cap) {
+            if (nullptr == cap) {
                 return false;
             }
 
@@ -597,8 +588,11 @@ bool QueryPlannerIXSelect::_compatible(const BSONElement& keyPatternElt,
     } else if (IndexNames::GEO_HAYSTACK == indexedFieldType) {
         return false;
     } else {
-        warning() << "Unknown indexing for node " << node->debugString() << " and field "
-                  << keyPatternElt.toString();
+        LOGV2_WARNING(20954,
+                      "Unknown indexing for node {node} and field {field}",
+                      "Unknown indexing for given node and field",
+                      "node"_attr = node->debugString(),
+                      "field"_attr = keyPatternElt.toString());
         verify(0);
     }
 }
@@ -611,6 +605,11 @@ bool QueryPlannerIXSelect::nodeIsSupportedBySparseIndex(const MatchExpression* q
     // cannot answer), so this function only needs to check if the query performs an equality to
     // null.
 
+    // Equality to null inside an $elemMatch implies a match on literal 'null'.
+    if (isInElemMatch) {
+        return true;
+    }
+
     // Otherwise, we can't use a sparse index for $eq (or $lte, or $gte) with a null element.
     //
     // We can use a sparse index for $_internalExprEq with a null element. Expression language
@@ -619,30 +618,10 @@ bool QueryPlannerIXSelect::nodeIsSupportedBySparseIndex(const MatchExpression* q
     const auto typ = queryExpr->matchType();
     if (typ == MatchExpression::EQ) {
         const auto* queryExprEquality = static_cast<const EqualityMatchExpression*>(queryExpr);
-        // Equality to null inside an $elemMatch implies a match on literal 'null'.
-        return isInElemMatch || !queryExprEquality->getData().isNull();
+        return !queryExprEquality->getData().isNull();
     } else if (queryExpr->matchType() == MatchExpression::MATCH_IN) {
         const auto* queryExprIn = static_cast<const InMatchExpression*>(queryExpr);
-        // Equality to null inside an $elemMatch implies a match on literal 'null'.
-        return isInElemMatch || !queryExprIn->hasNull();
-    } else if (queryExpr->matchType() == MatchExpression::NOT) {
-        const auto* child = queryExpr->getChild(0);
-        const MatchExpression::MatchType childtype = child->matchType();
-        const bool isNotEqualsNull =
-            (childtype == MatchExpression::EQ &&
-             static_cast<const ComparisonMatchExpression*>(child)->getData().type() ==
-                 BSONType::jstNULL);
-
-        // Prevent negated predicates from using sparse indices. Doing so would cause us to
-        // miss documents which do not contain the indexed fields. The only case where we may
-        // use a sparse index for a negation is when the query is {$ne: null}. This is due to
-        // the behavior of {$eq: null} matching documents where the field does not exist OR the
-        // field is equal to literal null. The negation of {$eq: null} therefore matches
-        // documents where the field does exist AND the field is not equal to literal
-        // null. Since the field must exist, it is safe to use a sparse index.
-        if (!isNotEqualsNull) {
-            return false;
-        }
+        return !queryExprIn->hasNull();
     }
 
     return true;
@@ -675,6 +654,24 @@ bool QueryPlannerIXSelect::nodeIsSupportedByWildcardIndex(const MatchExpression*
     return true;
 }
 
+bool QueryPlannerIXSelect::nodeIsSupportedByHashedIndex(const MatchExpression* queryExpr) {
+    // Hashed fields can answer simple equality predicates.
+    if (ComparisonMatchExpressionBase::isEquality(queryExpr->matchType())) {
+        return true;
+    }
+    // An $in can be answered so long as its operand contains only simple equalities.
+    if (queryExpr->matchType() == MatchExpression::MATCH_IN) {
+        const InMatchExpression* expr = static_cast<const InMatchExpression*>(queryExpr);
+        return expr->getRegexes().empty();
+    }
+    // {$exists:false} produces a single point-interval index bound on [null,null].
+    if (queryExpr->matchType() == MatchExpression::NOT) {
+        return queryExpr->getChild(0)->matchType() == MatchExpression::EXISTS;
+    }
+    // {$exists:true} can be answered using [MinKey, MaxKey] bounds.
+    return (queryExpr->matchType() == MatchExpression::EXISTS);
+}
+
 // static
 // This is the public method which does not accept an ElemMatchContext.
 void QueryPlannerIXSelect::rateIndices(MatchExpression* node,
@@ -705,7 +702,7 @@ void QueryPlannerIXSelect::_rateIndices(MatchExpression* node,
             fullPath = prefix + node->path().toString();
         }
 
-        verify(NULL == node->getTag());
+        verify(nullptr == node->getTag());
         node->setTag(new RelevantTag());
         auto rt = static_cast<RelevantTag*>(node->getTag());
         rt->path = fullPath;
@@ -738,8 +735,7 @@ void QueryPlannerIXSelect::_rateIndices(MatchExpression* node,
             childRt->path = rt->path;
             node->getChild(0)->setTag(childRt);
         }
-    } else if (Indexability::arrayUsesIndexOnChildren(node) && !node->path().empty()) {
-        // Note we skip empty path components since they are not allowed in index key patterns.
+    } else if (Indexability::arrayUsesIndexOnChildren(node)) {
         const auto newPath = prefix + node->path().toString();
         ElemMatchContext newContext;
         // Note this StringData is unowned and references the string declared on the stack here.
@@ -750,7 +746,12 @@ void QueryPlannerIXSelect::_rateIndices(MatchExpression* node,
 
         // If the array uses an index on its children, it's something like
         // {foo: {$elemMatch: {bar: 1}}}, in which case the predicate is really over foo.bar.
-        prefix += node->path().toString() + ".";
+        //
+        // When we have {foo: {$all: [{$elemMatch: {a: 1}}], the path of the embedded elemMatch
+        // is empty. We don't want to append a dot in that case as the field would be foo..a.
+        if (!node->path().empty()) {
+            prefix += node->path().toString() + ".";
+        }
         for (size_t i = 0; i < node->numChildren(); ++i) {
             _rateIndices(node->getChild(i), prefix, indices, collator, newContext);
         }
@@ -1031,7 +1032,7 @@ static void stripInvalidAssignmentsToTextIndex(MatchExpression* node,
         MatchExpression* child = node->getChild(i);
         RelevantTag* tag = static_cast<RelevantTag*>(child->getTag());
 
-        if (NULL == tag) {
+        if (nullptr == tag) {
             // 'child' could be a logical operator.  Maybe there are some assignments hiding
             // inside.
             stripInvalidAssignmentsToTextIndex(child, idx, prefixPaths);
