@@ -121,7 +121,7 @@ std::unique_ptr<HealthLogEntry> dbCheckHealthLogEntry(const NamespaceString& nss
                                                       const std::string& msg,
                                                       OplogEntriesEnum operation,
                                                       const BSONObj& data) {
-    auto entry = stdx::make_unique<HealthLogEntry>();
+    auto entry = std::make_unique<HealthLogEntry>();
     entry->setNss(nss);
     entry->setTimestamp(Date_t::now());
     entry->setSeverity(severity);
@@ -235,21 +235,24 @@ std::string hashCollectionInfo(const DbCheckCollectionInformation& info) {
 std::pair<boost::optional<UUID>, boost::optional<UUID>> getPrevAndNextUUIDs(
     OperationContext* opCtx, Collection* collection) {
     const CollectionCatalog& catalog = CollectionCatalog::get(opCtx);
-    const UUID uuid = *collection->uuid();
+    const UUID uuid = collection->uuid();
 
     std::vector<CollectionUUID> collectionUUIDs =
         catalog.getAllCollectionUUIDsFromDb(collection->ns().db());
     auto uuidIt = std::find(collectionUUIDs.begin(), collectionUUIDs.end(), uuid);
     invariant(uuidIt != collectionUUIDs.end());
 
+    auto prevIt = std::prev(uuidIt);
+    auto nextIt = std::next(uuidIt);
+
     boost::optional<UUID> prevUUID;
     boost::optional<UUID> nextUUID;
 
-    if (uuidIt != collectionUUIDs.begin()) {
-        prevUUID = *std::prev(uuidIt);
+    if (prevIt != collectionUUIDs.end()) {
+        prevUUID = *prevIt;
     }
 
-    if (auto nextIt = std::next(uuidIt); nextIt != collectionUUIDs.end()) {
+    if (nextIt != collectionUUIDs.end()) {
         nextUUID = *nextIt;
     }
 
@@ -356,14 +359,14 @@ std::vector<BSONObj> collectionIndexInfo(OperationContext* opCtx, Collection* co
 
     // List the indices,
     auto durableCatalog = DurableCatalog::get(opCtx);
-    durableCatalog->getAllIndexes(opCtx, collection->ns(), &names);
+    durableCatalog->getAllIndexes(opCtx, collection->getCatalogId(), &names);
 
     // and get the info for each one.
     for (const auto& name : names) {
-        result.push_back(durableCatalog->getIndexSpec(opCtx, collection->ns(), name));
+        result.push_back(durableCatalog->getIndexSpec(opCtx, collection->getCatalogId(), name));
     }
 
-    auto comp = stdx::make_unique<SimpleBSONObjComparator>();
+    auto comp = std::make_unique<SimpleBSONObjComparator>();
 
     std::sort(result.begin(), result.end(), SimpleBSONObjComparator::LessThan());
 
@@ -371,7 +374,9 @@ std::vector<BSONObj> collectionIndexInfo(OperationContext* opCtx, Collection* co
 }
 
 BSONObj collectionOptions(OperationContext* opCtx, Collection* collection) {
-    return DurableCatalog::get(opCtx)->getCollectionOptions(opCtx, collection->ns()).toBSON();
+    return DurableCatalog::get(opCtx)
+        ->getCollectionOptions(opCtx, collection->getCatalogId())
+        .toBSON();
 }
 
 AutoGetDbForDbCheck::AutoGetDbForDbCheck(OperationContext* opCtx, const NamespaceString& nss)
@@ -383,7 +388,9 @@ AutoGetCollectionForDbCheck::AutoGetCollectionForDbCheck(OperationContext* opCtx
     : _agd(opCtx, nss), _collLock(opCtx, nss, MODE_S) {
     std::string msg;
 
-    _collection = _agd.getDb() ? _agd.getDb()->getCollection(opCtx, nss) : nullptr;
+    _collection = _agd.getDb()
+        ? CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss)
+        : nullptr;
 
     // If the collection gets deleted after the check is launched, record that in the health log.
     if (!_collection) {
@@ -458,7 +465,7 @@ Status dbCheckDatabaseOnSecondary(OperationContext* opCtx,
                                   const DbCheckOplogCollection& entry) {
     // dbCheckCollectionResult-specific stuff.
     auto uuid = uassertStatusOK(UUID::parse(entry.getUuid().toString()));
-    auto collection = CollectionCatalog::get(opCtx).lookupCollectionByUUID(uuid);
+    auto collection = CollectionCatalog::get(opCtx).lookupCollectionByUUID(opCtx, uuid);
 
     if (!collection) {
         Status status(ErrorCodes::NamespaceNotFound, "Could not find collection for dbCheck");
@@ -506,27 +513,27 @@ Status dbCheckDatabaseOnSecondary(OperationContext* opCtx,
 namespace repl {
 
 /*
- * The corresponding command run on the secondary.
+ * The corresponding command run during command application.
  */
 Status dbCheckOplogCommand(OperationContext* opCtx,
-                           const char* ns,
-                           const BSONElement& ui,
-                           BSONObj& cmd,
-                           const repl::OpTime& optime,
                            const repl::OplogEntry& entry,
-                           OplogApplication::Mode mode,
-                           boost::optional<Timestamp> stableTimestampForRecovery) {
+                           OplogApplication::Mode mode) {
+    const auto& cmd = entry.getObject();
+    OpTime opTime;
+    if (!opCtx->writesAreReplicated()) {
+        opTime = entry.getOpTime();
+    }
     auto type = OplogEntries_parse(IDLParserErrorContext("type"), cmd.getStringField("type"));
     IDLParserErrorContext ctx("o");
 
     switch (type) {
         case OplogEntriesEnum::Batch: {
             auto invocation = DbCheckOplogBatch::parse(ctx, cmd);
-            return dbCheckBatchOnSecondary(opCtx, optime, invocation);
+            return dbCheckBatchOnSecondary(opCtx, opTime, invocation);
         }
         case OplogEntriesEnum::Collection: {
             auto invocation = DbCheckOplogCollection::parse(ctx, cmd);
-            return dbCheckDatabaseOnSecondary(opCtx, optime, invocation);
+            return dbCheckDatabaseOnSecondary(opCtx, opTime, invocation);
         }
     }
 
