@@ -31,6 +31,7 @@
 #pragma once
 
 #include <cstdint>
+#include <functional>
 #include <iosfwd>
 #include <memory>
 
@@ -38,11 +39,11 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/client/fetcher.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/repl/callback_completion_guard.h"
-#include "mongo/db/repl/collection_cloner.h"
 #include "mongo/db/repl/data_replicator_external_state.h"
-#include "mongo/db/repl/database_cloner.h"
+#include "mongo/db/repl/initial_sync_shared_data.h"
 #include "mongo/db/repl/multiapplier.h"
 #include "mongo/db/repl/oplog_applier.h"
 #include "mongo/db/repl/oplog_buffer.h"
@@ -53,9 +54,8 @@
 #include "mongo/dbtests/mock/mock_dbclient_connection.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
-#include "mongo/stdx/functional.h"
 #include "mongo/util/concurrency/thread_pool.h"
-#include "mongo/util/fail_point_service.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/net/hostandport.h"
 
 namespace mongo {
@@ -63,17 +63,13 @@ namespace repl {
 
 // TODO: Remove forward declares once we remove rs_initialsync.cpp and other dependents.
 // Failpoint which fails initial sync and leaves an oplog entry in the buffer.
-MONGO_FAIL_POINT_DECLARE(failInitSyncWithBufferedEntriesLeft);
+extern FailPoint failInitSyncWithBufferedEntriesLeft;
 
 // Failpoint which causes the initial sync function to hang before copying databases.
-MONGO_FAIL_POINT_DECLARE(initialSyncHangBeforeCopyingDatabases);
-
-// Failpoint which causes the initial sync function to hang before calling shouldRetry on a failed
-// operation.
-MONGO_FAIL_POINT_DECLARE(initialSyncHangBeforeGettingMissingDocument);
+extern FailPoint initialSyncHangBeforeCopyingDatabases;
 
 // Failpoint which stops the applier.
-MONGO_FAIL_POINT_DECLARE(rsSyncApplyStop);
+extern FailPoint rsSyncApplyStop;
 
 struct InitialSyncState;
 struct MemberState;
@@ -82,32 +78,30 @@ class StorageInterface;
 
 struct InitialSyncerOptions {
     /** Function to return optime of last operation applied on this node */
-    using GetMyLastOptimeFn = stdx::function<OpTime()>;
+    using GetMyLastOptimeFn = std::function<OpTime()>;
 
     /** Function to update optime of last operation applied on this node */
-    using SetMyLastOptimeFn = stdx::function<void(
+    using SetMyLastOptimeFn = std::function<void(
         const OpTimeAndWallTime&, ReplicationCoordinator::DataConsistency consistency)>;
 
     /** Function to reset all optimes on this node (e.g. applied & durable). */
-    using ResetOptimesFn = stdx::function<void()>;
+    using ResetOptimesFn = std::function<void()>;
 
     /** Function to sets this node into a specific follower mode. */
-    using SetFollowerModeFn = stdx::function<bool(const MemberState&)>;
+    using SetFollowerModeFn = std::function<bool(const MemberState&)>;
 
-    // Error and retry values
+    // Retry values
     Milliseconds syncSourceRetryWait{1000};
     Milliseconds initialSyncRetryWait{1000};
-    Seconds blacklistSyncSourcePenaltyForNetworkConnectionError{10};
-    Minutes blacklistSyncSourcePenaltyForOplogStartMissing{10};
 
     // InitialSyncer waits this long before retrying getApplierBatchCallback() if there are
     // currently no operations available to apply or if the 'rsSyncApplyStop' failpoint is active.
-    // This default value is based on the duration in SyncTail::OpQueueBatcher::run().
+    // This default value is based on the duration in OplogBatcher::run().
     Milliseconds getApplierBatchCallbackRetryWait{1000};
 
     // Replication settings
-    NamespaceString localOplogNS = NamespaceString("local.oplog.rs");
-    NamespaceString remoteOplogNS = NamespaceString("local.oplog.rs");
+    NamespaceString localOplogNS = NamespaceString::kRsOplogNamespace;
+    NamespaceString remoteOplogNS = NamespaceString::kRsOplogNamespace;
 
     GetMyLastOptimeFn getMyLastOptime;
     SetMyLastOptimeFn setMyLastOptime;
@@ -145,32 +139,72 @@ public:
     /**
      * Callback function to report last applied optime of initial sync.
      */
-    typedef stdx::function<void(const StatusWith<OpTimeAndWallTime>& lastApplied)> OnCompletionFn;
+    typedef std::function<void(const StatusWith<OpTimeAndWallTime>& lastApplied)> OnCompletionFn;
 
     /**
      * Callback completion guard for initial syncer.
      */
     using OnCompletionGuard = CallbackCompletionGuard<StatusWith<OpTimeAndWallTime>>;
 
-    using StartCollectionClonerFn = DatabaseCloner::StartCollectionClonerFn;
-
     /**
      * Type of function to create a database client
      *
      * Used for testing only.
      */
-    using CreateClientFn = stdx::function<std::unique_ptr<DBClientConnection>()>;
+    using CreateClientFn = std::function<std::unique_ptr<DBClientConnection>()>;
+
+    /**
+     * Type of function to create an OplogFetcher.
+     */
+    using CreateOplogFetcherFn = std::function<std::unique_ptr<OplogFetcher>(
+        executor::TaskExecutor* executor,
+        OpTime lastFetched,
+        HostAndPort source,
+        ReplSetConfig config,
+        std::unique_ptr<OplogFetcher::OplogFetcherRestartDecision> oplogFetcherRestartDecision,
+        int requiredRBID,
+        bool requireFresherSyncSource,
+        DataReplicatorExternalState* dataReplicatorExternalState,
+        OplogFetcher::EnqueueDocumentsFn enqueueDocumentsFn,
+        OplogFetcher::OnShutdownCallbackFn onShutdownCallbackFn,
+        const int batchSize,
+        OplogFetcher::StartingPoint startingPoint)>;
 
     struct InitialSyncAttemptInfo {
         int durationMillis;
         Status status;
         HostAndPort syncSource;
+        int rollBackId;
+        int operationsRetried;
+        int totalTimeUnreachableMillis;
 
         std::string toString() const;
         BSONObj toBSON() const;
         void append(BSONObjBuilder* builder) const;
     };
 
+    class OplogFetcherRestartDecisionInitialSyncer
+        : public OplogFetcher::OplogFetcherRestartDecision {
+
+    public:
+        OplogFetcherRestartDecisionInitialSyncer(InitialSyncSharedData* sharedData,
+                                                 std::size_t maxFetcherRestarts)
+            : _sharedData(sharedData), _defaultDecision(maxFetcherRestarts){};
+
+        bool shouldContinue(OplogFetcher* fetcher, Status status) final;
+
+        void fetchSuccessful(OplogFetcher* fetcher) final;
+
+    private:
+        InitialSyncSharedData* _sharedData;
+
+        // We delegate to the default strategy when it's a non-network error.
+        OplogFetcher::OplogFetcherRestartDecisionDefault _defaultDecision;
+
+        // The operation, if any, currently being retried because of a network error.
+        InitialSyncSharedData::RetryableOperation _retryingOperation;
+    };
+
     struct Stats {
         std::uint32_t failedInitialSyncAttempts{0};
         std::uint32_t maxFailedInitialSyncAttempts{0};
@@ -224,25 +258,53 @@ public:
     BSONObj getInitialSyncProgress() const;
 
     /**
-     * Overrides how executor schedules database work.
-     *
-     * For testing only.
+     * Cancels the current initial sync attempt if the initial syncer is active.
      */
-    void setScheduleDbWorkFn_forTest(const DatabaseCloner::ScheduleDbWorkFn& scheduleDbWorkFn);
+    void cancelCurrentAttempt();
 
     /**
-     * Overrides how executor starts a collection cloner.
      *
-     * For testing only.
+     * Overrides how the initial syncer creates the client.
+     *
+     * For testing only
      */
-    void setStartCollectionClonerFn(const StartCollectionClonerFn& startCollectionCloner);
+    void setCreateClientFn_forTest(const CreateClientFn& createClientFn);
 
     /**
-     * Allows a different client class to be injected for the databases cloner.
+     *
+     * Overrides how the initial syncer creates the OplogFetcher.
      *
      * For testing only.
      */
-    void setDbsClonerCreateClientFn_forTest(const CreateClientFn& createClientFn);
+    void setCreateOplogFetcherFn_forTest(const CreateOplogFetcherFn& createOplogFetcherFn);
+
+    /**
+     *
+     * Get a raw pointer to the OplogFetcher. Block up to 10s until the underlying OplogFetcher has
+     * started. It is the caller's responsibility to not reuse this pointer beyond the lifetime of
+     * the underlying OplogFetcher.
+     *
+     * For testing only.
+     */
+    OplogFetcher* getOplogFetcher_forTest() const;
+
+    /**
+     *
+     * Provides a separate executor for the cloners, so network operations based on
+     * TaskExecutor::scheduleRemoteCommand() can use the NetworkInterfaceMock while the cloners
+     * are stopped on a failpoint.
+     *
+     * For testing only
+     */
+    void setClonerExecutor_forTest(executor::TaskExecutor* clonerExec);
+
+    /**
+     *
+     * Wait for the cloner thread to finish.
+     *
+     * For testing only
+     */
+    void waitForCloner_forTest();
 
     // State transitions:
     // PreStart --> Running --> ShuttingDown --> Complete
@@ -262,7 +324,18 @@ public:
      */
     Date_t getWallClockTime_forTest() const;
 
+    /**
+     * Sets the allowed outage duration in _sharedData.
+     * For testing only.
+     */
+    void setAllowedOutageDuration_forTest(Milliseconds allowedOutageDuration);
+
 private:
+    enum LastOplogEntryFetcherRetryStrategy {
+        kFetcherHandlesRetries,
+        kInitialSyncerHandlesRetries
+    };
+
     /**
      * Returns true if we are still processing initial sync tasks (_state is either Running or
      * Shutdown).
@@ -334,7 +407,7 @@ private:
      *         |                              |
      *         |                              |
      *         V                              V
-     *    _oplogFetcherCallback()         _databasesClonerCallback
+     *    _oplogFetcherCallback()         _allDatabaseClonerCallback
      *         |                              |
      *         |                              |
      *         |                              V
@@ -344,28 +417,19 @@ private:
      *         |            (no ops to apply) |       | (have ops to apply)
      *         |                              |       |
      *         |                              |       V
-     *         |                              |   _getNextApplierBatchCallback()<-----+
-     *         |                              |       |                       ^       |
-     *         |                              |       |                       |       |
-     *         |                              |       |      (no docs fetched |       |
-     *         |                              |       |       and end ts not  |       |
-     *         |                              |       |       reached)        |       |
-     *         |                              |       |                       |       |
-     *         |                              |       V                       |       |
-     *         |                              |   _multiApplierCallback()-----+       |
-     *         |                              |       |       |                       |
-     *         |                              |       |       |                       |
-     *         |                              |       |       | (docs fetched)        | (end ts not
-     *         |                              |       |       |                       |  reached)
-     *         |                              |       |       V                       |
-     *         |                              |       |   _lastOplogEntryFetcherCallbackAfter-
-     *         |                              |       |       FetchingMissingDocuments()
-     *         |                              |       |       |
-     *         |                              |       |       |
-     *         |                           (reached end timestamp)
-     *         |                              |       |       |
-     *         |                              V       V       V
-     *         |                          _rollbackCheckerCheckForRollbackCallback()
+     *         |                              |   _getNextApplierBatchCallback()
+     *         |                              |       |                       ^
+     *         |                              |       |                       |
+     *         |                              |       |             (end ts not reached)
+     *         |                              |       |                       |
+     *         |                              |       V                       |
+     *         |                              |   _multiApplierCallback()-----+
+     *         |                              |       |
+     *         |                              |       |
+     *         |                        (reached end timestamp)
+     *         |                              |       |
+     *         |                              V       V
+     *         |                _rollbackCheckerCheckForRollbackCallback()
      *         |                              |
      *         |                              |
      *         +------------------------------+
@@ -477,8 +541,8 @@ private:
     /**
      * Callback for DatabasesCloner.
      */
-    void _databasesClonerCallback(const Status& status,
-                                  std::shared_ptr<OnCompletionGuard> onCompletionGuard);
+    void _allDatabaseClonerCallback(const Status& status,
+                                    std::shared_ptr<OnCompletionGuard> onCompletionGuard);
 
     /**
      * Callback for second '_lastOplogEntryFetcher' callback. This is scheduled to obtain the stop
@@ -504,16 +568,6 @@ private:
                                std::uint32_t numApplied,
                                std::shared_ptr<OnCompletionGuard> onCompletionGuard);
 
-    /**
-     * Callback for third '_lastOplogEntryFetcher' callback. This is scheduled after MultiApplier
-     * completed successfully and missing documents were fetched from the sync source while
-     * DataReplicatorExternalState::_multiApply() was processing operations.
-     * This callback will update InitialSyncState::stopTimestamp on success.
-     */
-    void _lastOplogEntryFetcherCallbackAfterFetchingMissingDocuments(
-        const StatusWith<Fetcher::QueryResponse>& result,
-        std::shared_ptr<OnCompletionGuard> onCompletionGuard);
-
     /**
      * Callback for rollback checker's last replSetGetRBID command after cloning data and applying
      * operations.
@@ -544,19 +598,24 @@ private:
      * Returns a status even though it always returns OK, to conform the interface OplogFetcher
      * expects for the EnqueueDocumentsFn.
      */
-    Status _enqueueDocuments(Fetcher::Documents::const_iterator begin,
-                             Fetcher::Documents::const_iterator end,
+    Status _enqueueDocuments(OplogFetcher::Documents::const_iterator begin,
+                             OplogFetcher::Documents::const_iterator end,
                              const OplogFetcher::DocumentsInfo& info);
 
     void _appendInitialSyncProgressMinimal_inlock(BSONObjBuilder* bob) const;
     BSONObj _getInitialSyncProgress_inlock() const;
 
-    StatusWith<MultiApplier::Operations> _getNextApplierBatch_inlock();
+    StatusWith<std::vector<OplogEntry>> _getNextApplierBatch_inlock();
 
     /**
      * Schedules a fetcher to get the last oplog entry from the sync source.
+     *
+     * If 'retryStrategy' is 'kFetcherHandlesRetries', the fetcher will retry up to the server
+     * parameter 'numInitialSyncOplogFindAttempts' times. Otherwise any failures must be handled by
+     * the caller.
      */
-    Status _scheduleLastOplogEntryFetcher_inlock(Fetcher::CallbackFn callback);
+    Status _scheduleLastOplogEntryFetcher_inlock(Fetcher::CallbackFn callback,
+                                                 LastOplogEntryFetcherRetryStrategy retryStrategy);
 
     /**
      * Checks the current oplog application progress (begin and end timestamps).
@@ -578,6 +637,19 @@ private:
     void _scheduleRollbackCheckerCheckForRollback_inlock(
         const stdx::lock_guard<Latch>& lock, std::shared_ptr<OnCompletionGuard> onCompletionGuard);
 
+    /**
+     * Check if a status is one which means there's a retriable error and we should retry the
+     * current operation, and records whether an operation is currently being retried.  Note this
+     * can only handle one operation at a time (i.e. it should not be used in both parts of the
+     * "split" section of Initial Sync)
+     */
+    bool _shouldRetryError(WithLock lk, Status status);
+
+    /**
+     * Indicates we are no longer handling a retriable error.
+     */
+    void _clearRetriableError(WithLock lk);
+
     /**
      * Checks the given status (or embedded status inside the callback args) and current data
      * replicator shutdown state. If the given status is not OK or if we are shutting down, returns
@@ -640,9 +712,14 @@ private:
     const InitialSyncerOptions _opts;                                           // (R)
     std::unique_ptr<DataReplicatorExternalState> _dataReplicatorExternalState;  // (R)
     executor::TaskExecutor* _exec;                                              // (R)
-    ThreadPool* _writerPool;                                                    // (R)
-    StorageInterface* _storage;                                                 // (R)
-    ReplicationProcess* _replicationProcess;                                    // (S)
+    // The executor that the Cloner thread runs on.  In production code this is the same as _exec,
+    // but for unit testing, _exec is single-threaded and our NetworkInterfaceMock runs it in
+    // lockstep with the unit test code.  If we pause the cloners using failpoints
+    // NetworkInterfaceMock is unaware of this and this causes our unit tests to deadlock.
+    executor::TaskExecutor* _clonerExec;      // (R)
+    ThreadPool* _writerPool;                  // (R)
+    StorageInterface* _storage;               // (R)
+    ReplicationProcess* _replicationProcess;  // (S)
 
     // This is invoked with the final status of the initial sync. If startup() fails, this callback
     // is never invoked. The caller gets the last applied optime when the initial sync completes
@@ -669,6 +746,9 @@ private:
     // Handle to currently scheduled _getNextApplierBatchCallback() task.
     executor::TaskExecutor::CallbackHandle _getNextApplierBatchHandle;  // (M)
 
+    // The operation, if any, currently being retried because of a network error.
+    InitialSyncSharedData::RetryableOperation _retryingOperation;  // (M)
+
     std::unique_ptr<InitialSyncState> _initialSyncState;   // (M)
     std::unique_ptr<OplogFetcher> _oplogFetcher;           // (S)
     std::unique_ptr<Fetcher> _beginFetchingOpTimeFetcher;  // (S)
@@ -676,11 +756,12 @@ private:
     std::unique_ptr<Fetcher> _fCVFetcher;                  // (S)
     std::unique_ptr<MultiApplier> _applier;                // (M)
     HostAndPort _syncSource;                               // (M)
+    std::unique_ptr<DBClientConnection> _client;           // (M)
     OpTime _lastFetched;                                   // (MX)
     OpTimeAndWallTime _lastApplied;                        // (MX)
-    std::unique_ptr<OplogBuffer> _oplogBuffer;             // (M)
-    std::unique_ptr<OplogApplier::Observer> _observer;     // (S)
-    std::unique_ptr<OplogApplier> _oplogApplier;           // (M)
+
+    std::unique_ptr<OplogBuffer> _oplogBuffer;    // (M)
+    std::unique_ptr<OplogApplier> _oplogApplier;  // (M)
 
     // Used to signal changes in _state.
     mutable stdx::condition_variable _stateCondition;
@@ -688,17 +769,22 @@ private:
     // Current initial syncer state. See comments for State enum class for details.
     State _state = State::kPreStart;  // (M)
 
-    // Passed to DatabasesCloner.
-    CreateClientFn _dbsClonerCreateClientFn;  // (M)
+    // Used to create the DBClientConnection for the cloners
+    CreateClientFn _createClientFn;
 
-
-    // Passed to CollectionCloner via DatabasesCloner.
-    DatabaseCloner::ScheduleDbWorkFn _scheduleDbWorkFn;  // (M)
-    StartCollectionClonerFn _startCollectionClonerFn;    // (M)
+    // Used to create the OplogFetcher for the InitialSyncer.
+    CreateOplogFetcherFn _createOplogFetcherFn;
 
     // Contains stats on the current initial sync request (includes all attempts).
     // To access these stats in a user-readable format, use getInitialSyncProgress().
     Stats _stats;  // (M)
+
+    // Data shared by cloners and fetcher.  Follow InitialSyncSharedData synchronization rules.
+    std::unique_ptr<InitialSyncSharedData> _sharedData;  // (S)
+
+    // Amount of time an outage is allowed to continue before the initial sync attempt is marked
+    // as failed.
+    Milliseconds _allowedOutageDuration;  // (M)
 };
 
 }  // namespace repl
