@@ -31,6 +31,7 @@
 
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/simple_bsonobj_comparator.h"
+#include "mongo/db/catalog/collection_options.h"
 #include "mongo/db/logical_session_id.h"
 #include "mongo/db/repl/apply_ops_gen.h"
 #include "mongo/db/repl/oplog_entry_gen.h"
@@ -59,73 +60,194 @@ public:
         o.parseProtected(ctxt, bsonObject);
         return o;
     }
-
-    const BSONObj& getPostImageDocumentKey() const {
-        return _postImageDocumentKey;
+    const BSONObj& getPreImageDocumentKey() const {
+        return _preImageDocumentKey;
+    }
+    void setPreImageDocumentKey(BSONObj value) {
+        _preImageDocumentKey = std::move(value);
     }
 
-    void setPostImageDocumentKey(BSONObj value) {
-        _postImageDocumentKey = std::move(value);
+    const BSONObj& getPreImage() const {
+        return _fullPreImage;
+    }
+
+    void setPreImage(BSONObj value) {
+        _fullPreImage = std::move(value);
     }
 
 private:
-    // Stores the post image _id + shard key values.
-    BSONObj _postImageDocumentKey;
+    BSONObj _preImageDocumentKey;
+    BSONObj _fullPreImage;
 };
 
 /**
- * A parsed oplog entry that privately inherits from the OplogEntryBase parsed by the IDL.
+ * Mutable class used on primary to build up oplog entries progressively.
+ */
+class MutableOplogEntry : public OplogEntryBase {
+public:
+    // Current oplog version, should be the value of the v field in all oplog entries.
+    static const int kOplogVersion;
+
+    // Helpers to generate ReplOperation.
+    static ReplOperation makeInsertOperation(const NamespaceString& nss,
+                                             UUID uuid,
+                                             const BSONObj& docToInsert);
+    static ReplOperation makeUpdateOperation(const NamespaceString nss,
+                                             UUID uuid,
+                                             const BSONObj& update,
+                                             const BSONObj& criteria);
+    static ReplOperation makeDeleteOperation(const NamespaceString& nss,
+                                             UUID uuid,
+                                             const BSONObj& docToDelete);
+
+    static ReplOperation makeCreateCommand(const NamespaceString nss,
+                                           const mongo::CollectionOptions& options,
+                                           const BSONObj& idIndex);
+
+    static ReplOperation makeCreateIndexesCommand(const NamespaceString nss,
+                                                  CollectionUUID uuid,
+                                                  const BSONObj& indexDoc);
+
+    static BSONObj makeCreateCollCmdObj(const NamespaceString& collectionName,
+                                        const mongo::CollectionOptions& options,
+                                        const BSONObj& idIndex);
+
+    static StatusWith<MutableOplogEntry> parse(const BSONObj& object);
+
+    MutableOplogEntry() : OplogEntryBase() {}
+
+    void setSessionId(boost::optional<LogicalSessionId> value) & {
+        getOperationSessionInfo().setSessionId(std::move(value));
+    }
+
+    void setTxnNumber(boost::optional<std::int64_t> value) & {
+        getOperationSessionInfo().setTxnNumber(std::move(value));
+    }
+
+    void setOpType(OpTypeEnum value) & {
+        getDurableReplOperation().setOpType(std::move(value));
+    }
+
+    void setNss(NamespaceString value) & {
+        getDurableReplOperation().setNss(std::move(value));
+    }
+
+    void setUuid(boost::optional<UUID> value) & {
+        getDurableReplOperation().setUuid(std::move(value));
+    }
+
+    void setObject(BSONObj value) & {
+        getDurableReplOperation().setObject(std::move(value));
+    }
+
+    void setObject2(boost::optional<BSONObj> value) & {
+        getDurableReplOperation().setObject2(std::move(value));
+    }
+
+    void setUpsert(boost::optional<bool> value) & {
+        getDurableReplOperation().setUpsert(std::move(value));
+    }
+
+    void setPreImageOpTime(boost::optional<OpTime> value) {
+        getDurableReplOperation().setPreImageOpTime(std::move(value));
+    }
+
+    const boost::optional<OpTime>& getPreImageOpTime() const {
+        return getDurableReplOperation().getPreImageOpTime();
+    }
+
+    void setTimestamp(Timestamp value) & {
+        getOpTimeAndWallTimeBase().setTimestamp(std::move(value));
+    }
+
+    void setTerm(boost::optional<std::int64_t> value) & {
+        getOpTimeAndWallTimeBase().setTerm(std::move(value));
+    }
+
+    void setWallClockTime(Date_t value) & {
+        getOpTimeAndWallTimeBase().setWallClockTime(std::move(value));
+    }
+
+    /**
+     * Sets the OpTime of the oplog entry.
+     */
+    void setOpTime(const OpTime& opTime) &;
+
+    /**
+     * Returns the OpTime of the oplog entry.
+     */
+    OpTime getOpTime() const;
+
+    /**
+     * Same as setFromMigrate but only set when it is true.
+     */
+    void setFromMigrateIfTrue(bool value) & {
+        if (value)
+            setFromMigrate(value);
+    }
+};
+
+/**
+ * A parsed oplog entry that privately inherits from the MutableOplogEntry.
  * This class is immutable. All setters are hidden.
  */
-class OplogEntry : private OplogEntryBase {
+class OplogEntry : private MutableOplogEntry {
 public:
     // Make field names accessible.
-    using OplogEntryBase::k_idFieldName;
-    using OplogEntryBase::kDurableReplOperationFieldName;
-    using OplogEntryBase::kFromMigrateFieldName;
-    using OplogEntryBase::kHashFieldName;
-    using OplogEntryBase::kNssFieldName;
-    using OplogEntryBase::kObject2FieldName;
-    using OplogEntryBase::kObjectFieldName;
-    using OplogEntryBase::kOperationSessionInfoFieldName;
-    using OplogEntryBase::kOpTypeFieldName;
-    using OplogEntryBase::kPostImageOpTimeFieldName;
-    using OplogEntryBase::kPreImageOpTimeFieldName;
-    using OplogEntryBase::kPrevWriteOpTimeInTransactionFieldName;
-    using OplogEntryBase::kSessionIdFieldName;
-    using OplogEntryBase::kStatementIdFieldName;
-    using OplogEntryBase::kTermFieldName;
-    using OplogEntryBase::kTimestampFieldName;
-    using OplogEntryBase::kTxnNumberFieldName;
-    using OplogEntryBase::kUpsertFieldName;
-    using OplogEntryBase::kUuidFieldName;
-    using OplogEntryBase::kVersionFieldName;
-    using OplogEntryBase::kWallClockTimeFieldName;
-    // Make serialize(), toBSON() and getters accessible.
-    using OplogEntryBase::get_id;
-    using OplogEntryBase::getDurableReplOperation;
-    using OplogEntryBase::getFromMigrate;
-    using OplogEntryBase::getHash;
-    using OplogEntryBase::getNeedsRetryImage;
-    using OplogEntryBase::getNss;
-    using OplogEntryBase::getObject;
-    using OplogEntryBase::getObject2;
-    using OplogEntryBase::getOperationSessionInfo;
-    using OplogEntryBase::getOpType;
-    using OplogEntryBase::getPostImageOpTime;
-    using OplogEntryBase::getPreImageOpTime;
-    using OplogEntryBase::getPrevWriteOpTimeInTransaction;
-    using OplogEntryBase::getSessionId;
-    using OplogEntryBase::getStatementId;
-    using OplogEntryBase::getTerm;
-    using OplogEntryBase::getTimestamp;
-    using OplogEntryBase::getTxnNumber;
-    using OplogEntryBase::getUpsert;
-    using OplogEntryBase::getUuid;
-    using OplogEntryBase::getVersion;
-    using OplogEntryBase::getWallClockTime;
-    using OplogEntryBase::serialize;
-    using OplogEntryBase::toBSON;
+    using MutableOplogEntry::k_idFieldName;
+    using MutableOplogEntry::kDurableReplOperationFieldName;
+    using MutableOplogEntry::kFromMigrateFieldName;
+    using MutableOplogEntry::kHashFieldName;
+    using MutableOplogEntry::kNssFieldName;
+    using MutableOplogEntry::kObject2FieldName;
+    using MutableOplogEntry::kObjectFieldName;
+    using MutableOplogEntry::kOperationSessionInfoFieldName;
+    using MutableOplogEntry::kOplogVersion;
+    using MutableOplogEntry::kOpTypeFieldName;
+    using MutableOplogEntry::kPostImageOpTimeFieldName;
+    using MutableOplogEntry::kPreImageOpTimeFieldName;
+    using MutableOplogEntry::kPrevWriteOpTimeInTransactionFieldName;
+    using MutableOplogEntry::kSessionIdFieldName;
+    using MutableOplogEntry::kStatementIdFieldName;
+    using MutableOplogEntry::kTermFieldName;
+    using MutableOplogEntry::kTimestampFieldName;
+    using MutableOplogEntry::kTxnNumberFieldName;
+    using MutableOplogEntry::kUpsertFieldName;
+    using MutableOplogEntry::kUuidFieldName;
+    using MutableOplogEntry::kVersionFieldName;
+    using MutableOplogEntry::kWallClockTimeFieldName;
+
+    // Make serialize() and getters accessible.
+    using MutableOplogEntry::get_id;
+    using MutableOplogEntry::getDurableReplOperation;
+    using MutableOplogEntry::getFromMigrate;
+    using MutableOplogEntry::getHash;
+    using MutableOplogEntry::getNss;
+    using MutableOplogEntry::getObject;
+    using MutableOplogEntry::getObject2;
+    using MutableOplogEntry::getOperationSessionInfo;
+    using MutableOplogEntry::getOpType;
+    using MutableOplogEntry::getPostImageOpTime;
+    using MutableOplogEntry::getPreImageOpTime;
+    using MutableOplogEntry::getPrevWriteOpTimeInTransaction;
+    using MutableOplogEntry::getSessionId;
+    using MutableOplogEntry::getStatementId;
+    using MutableOplogEntry::getTerm;
+    using MutableOplogEntry::getTimestamp;
+    using MutableOplogEntry::getTxnNumber;
+    using MutableOplogEntry::getUpsert;
+    using MutableOplogEntry::getUuid;
+    using MutableOplogEntry::getVersion;
+    using MutableOplogEntry::getWallClockTime;
+    using MutableOplogEntry::serialize;
+
+    // Make helper functions accessible.
+    using MutableOplogEntry::getOpTime;
+    using MutableOplogEntry::makeCreateCommand;
+    using MutableOplogEntry::makeCreateIndexesCommand;
+    using MutableOplogEntry::makeDeleteOperation;
+    using MutableOplogEntry::makeInsertOperation;
+    using MutableOplogEntry::makeUpdateOperation;
 
     enum class CommandType {
         kNotCommand,
@@ -147,21 +269,6 @@ public:
         kAbortTransaction,
     };
 
-    // Current oplog version, should be the value of the v field in all oplog entries.
-    static const int kOplogVersion;
-
-    // Helpers to generate ReplOperation.
-    static ReplOperation makeInsertOperation(const NamespaceString& nss,
-                                             boost::optional<UUID> uuid,
-                                             const BSONObj& docToInsert);
-    static ReplOperation makeUpdateOperation(const NamespaceString nss,
-                                             boost::optional<UUID> uuid,
-                                             const BSONObj& update,
-                                             const BSONObj& criteria);
-    static ReplOperation makeDeleteOperation(const NamespaceString& nss,
-                                             boost::optional<UUID> uuid,
-                                             const BSONObj& docToDelete);
-
     // Get the in-memory size in bytes of a ReplOperation.
     static size_t getDurableReplOperationSize(const DurableReplOperation& op);
 
@@ -178,12 +285,11 @@ public:
                const boost::optional<BSONObj>& o2Field,
                const OperationSessionInfo& sessionInfo,
                const boost::optional<bool>& isUpsert,
-               const boost::optional<mongo::Date_t>& wallClockTime,
+               const mongo::Date_t& wallClockTime,
                const boost::optional<StmtId>& statementId,
                const boost::optional<OpTime>& prevWriteOpTimeInTransaction,
                const boost::optional<OpTime>& preImageOpTime,
-               const boost::optional<OpTime>& postImageOpTime,
-               const boost::optional<repl::RetryImageEnum>& needsRetryImage = boost::none);
+               const boost::optional<OpTime>& postImageOpTime);
 
     // DEPRECATED: This constructor can throw. Use static parse method instead.
     explicit OplogEntry(BSONObj raw);
@@ -210,6 +316,11 @@ public:
         return getObject()[ApplyOpsCommandInfoBase::kPartialTxnFieldName].booleanSafe();
     }
 
+    /**
+     * Returns whether if the oplog entry is the last applyOps in a multiple-entry transaction.
+     */
+    bool isEndOfLargeTransaction() const;
+
     /**
      * Returns if this is a prepared 'commitTransaction' oplog entry.
      */
@@ -227,17 +338,23 @@ public:
             !isPartialTransaction() && !getObject().getBoolField("prepare");
     }
 
+    /**
+     * Returns whether the oplog entry represents a single oplog entry transaction.
+     */
+    bool isSingleOplogEntryTransaction() const;
+
+    /**
+     * Returns whether the oplog entry represents an applyOps with a command inside. This is only
+     * for transactions with only one oplog entry.
+     */
+    bool isSingleOplogEntryTransactionWithCommand() const;
+
     /**
      * Returns if the oplog entry is for a CRUD operation.
      */
     static bool isCrudOpType(OpTypeEnum opType);
     bool isCrudOpType() const;
 
-    /**
-     * Returns true if the oplog entry is for an Update or Delete operation.
-     */
-    bool isUpdateOrDelete() const;
-
     /**
      * Returns if the operation should be prepared. Must be called on an 'applyOps' entry.
      */
@@ -282,16 +399,15 @@ public:
         return _raw;
     }
 
-    /**
-     * Returns the OpTime of the oplog entry.
-     */
-    OpTime getOpTime() const;
-
     /**
      * Serializes the oplog entry to a string.
      */
     std::string toString() const;
 
+    BSONObj toBSON() const {
+        return _raw;
+    }
+
 private:
     BSONObj _raw;  // Owned.
     CommandType _commandType = CommandType::kNotCommand;
