@@ -26,7 +26,7 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kReplication
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -34,18 +34,20 @@
 
 #include "mongo/db/bson/bson_helper.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/catalog_raii.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/repl/storage_interface.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 namespace repl {
 
 constexpr StringData ReplicationConsistencyMarkersImpl::kDefaultMinValidNamespace;
 constexpr StringData ReplicationConsistencyMarkersImpl::kDefaultOplogTruncateAfterPointNamespace;
+constexpr StringData ReplicationConsistencyMarkersImpl::kDefaultInitialSyncIdNamespace;
 
 namespace {
 const BSONObj kInitialSyncFlag(BSON(MinValidDocument::kInitialSyncFlagFieldName << true));
@@ -59,15 +61,18 @@ ReplicationConsistencyMarkersImpl::ReplicationConsistencyMarkersImpl(
           storageInterface,
           NamespaceString(ReplicationConsistencyMarkersImpl::kDefaultMinValidNamespace),
           NamespaceString(
-              ReplicationConsistencyMarkersImpl::kDefaultOplogTruncateAfterPointNamespace)) {}
+              ReplicationConsistencyMarkersImpl::kDefaultOplogTruncateAfterPointNamespace),
+          NamespaceString(ReplicationConsistencyMarkersImpl::kDefaultInitialSyncIdNamespace)) {}
 
 ReplicationConsistencyMarkersImpl::ReplicationConsistencyMarkersImpl(
     StorageInterface* storageInterface,
     NamespaceString minValidNss,
-    NamespaceString oplogTruncateAfterPointNss)
+    NamespaceString oplogTruncateAfterPointNss,
+    NamespaceString initialSyncIdNss)
     : _storageInterface(storageInterface),
       _minValidNss(minValidNss),
-      _oplogTruncateAfterPointNss(oplogTruncateAfterPointNss) {}
+      _oplogTruncateAfterPointNss(oplogTruncateAfterPointNss),
+      _initialSyncIdNss(initialSyncIdNss) {}
 
 boost::optional<MinValidDocument> ReplicationConsistencyMarkersImpl::_getMinValidDocument(
     OperationContext* opCtx) const {
@@ -93,7 +98,7 @@ void ReplicationConsistencyMarkersImpl::_updateMinValidDocument(
 }
 
 void ReplicationConsistencyMarkersImpl::initializeMinValidDocument(OperationContext* opCtx) {
-    LOG(3) << "Initializing minValid document";
+    LOGV2_DEBUG(21282, 3, "Initializing minValid document");
 
     // This initializes the values of the required fields if they are not already set.
     // If one of the fields is already set, the $max will prefer the existing value since it
@@ -114,22 +119,28 @@ void ReplicationConsistencyMarkersImpl::initializeMinValidDocument(OperationCont
 bool ReplicationConsistencyMarkersImpl::getInitialSyncFlag(OperationContext* opCtx) const {
     auto doc = _getMinValidDocument(opCtx);
     if (!doc) {
-        LOG(3) << "No min valid document found, returning initial sync flag value of false.";
+        LOGV2_DEBUG(
+            21283, 3, "No min valid document found, returning initial sync flag value of false");
         return false;
     }
 
     boost::optional<bool> flag = doc->getInitialSyncFlag();
     if (!flag) {
-        LOG(3) << "No initial sync flag set, returning initial sync flag value of false.";
+        LOGV2_DEBUG(
+            21284, 3, "No initial sync flag set, returning initial sync flag value of false");
         return false;
     }
 
-    LOG(3) << "returning initial sync flag value of " << flag.get();
+    LOGV2_DEBUG(21285,
+                3,
+                "returning initial sync flag value of {flag}",
+                "Returning initial sync flag value",
+                "flag"_attr = flag.get());
     return flag.get();
 }
 
 void ReplicationConsistencyMarkersImpl::setInitialSyncFlag(OperationContext* opCtx) {
-    LOG(3) << "setting initial sync flag";
+    LOGV2_DEBUG(21286, 3, "Setting initial sync flag");
     TimestampedBSONObj update;
     update.obj = BSON("$set" << kInitialSyncFlag);
 
@@ -139,11 +150,11 @@ void ReplicationConsistencyMarkersImpl::setInitialSyncFlag(OperationContext* opC
     update.timestamp = Timestamp();
 
     _updateMinValidDocument(opCtx, update);
-    opCtx->recoveryUnit()->waitUntilDurable();
+    opCtx->recoveryUnit()->waitUntilDurable(opCtx);
 }
 
 void ReplicationConsistencyMarkersImpl::clearInitialSyncFlag(OperationContext* opCtx) {
-    LOG(3) << "clearing initial sync flag";
+    LOGV2_DEBUG(21287, 3, "Clearing initial sync flag");
 
     auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     OpTimeAndWallTime opTimeAndWallTime = replCoord->getMyLastAppliedOpTimeAndWallTime();
@@ -155,22 +166,27 @@ void ReplicationConsistencyMarkersImpl::clearInitialSyncFlag(OperationContext* o
                                        << MinValidDocument::kMinValidTermFieldName << time.getTerm()
                                        << MinValidDocument::kAppliedThroughFieldName << time));
 
-    // We clear the initial sync flag at the 'lastAppliedOpTime'. This is unnecessary, since there
-    // should not be any stable checkpoints being taken that this write could inadvertantly enter.
-    // This 'lastAppliedOpTime' will be the first stable timestamp candidate, so it will be in the
-    // first stable checkpoint taken after initial sync. This provides more clarity than providing
-    // no timestamp.
-    update.timestamp = time.getTimestamp();
+    // As we haven't yet updated our initialDataTimestamp from
+    // Timestamp::kAllowUnstableCheckpointsSentinel to lastAppliedTimestamp, we are only allowed to
+    // take unstable checkpoints. And, this "lastAppliedTimestamp" will be the first stable
+    // checkpoint taken after initial sync. So, no way this minValid update can be part of a stable
+    // checkpoint taken earlier than lastAppliedTimestamp. So, it's safe to make it as an
+    // non-timestamped write. Also, this has to be non-timestamped write because we may have readers
+    // at lastAppliedTimestamp, commiting the storage writes before or at such timestamps is
+    // illegal.
+    update.timestamp = Timestamp();
 
     _updateMinValidDocument(opCtx, update);
 
     // Make sure to clear the oplogTrucateAfterPoint in case it is stale. Otherwise, we risk the
     // possibility of deleting oplog entries that we want to keep. It is safe to clear this
     // here since we are consistent at the top of our oplog at this point.
+    invariant(!isOplogTruncateAfterPointBeingUsedForPrimary(),
+              "Clearing the truncate point while primary is unsafe: it is asynchronously updated.");
     setOplogTruncateAfterPoint(opCtx, Timestamp());
 
     if (getGlobalServiceContext()->getStorageEngine()->isDurable()) {
-        opCtx->recoveryUnit()->waitUntilDurable();
+        opCtx->recoveryUnit()->waitUntilDurable(opCtx);
         replCoord->setMyLastDurableOpTimeAndWallTime(opTimeAndWallTime);
     }
 }
@@ -181,15 +197,24 @@ OpTime ReplicationConsistencyMarkersImpl::getMinValid(OperationContext* opCtx) c
 
     auto minValid = OpTime(doc->getMinValidTimestamp(), doc->getMinValidTerm());
 
-    LOG(3) << "returning minvalid: " << minValid.toString() << "(" << minValid.toBSON() << ")";
+    LOGV2_DEBUG(21288,
+                3,
+                "returning minvalid: {minValidString}({minValidBSON})",
+                "Returning minvalid",
+                "minValidString"_attr = minValid.toString(),
+                "minValidBSON"_attr = minValid.toBSON());
 
     return minValid;
 }
 
 void ReplicationConsistencyMarkersImpl::setMinValid(OperationContext* opCtx,
                                                     const OpTime& minValid) {
-    LOG(3) << "setting minvalid to exactly: " << minValid.toString() << "(" << minValid.toBSON()
-           << ")";
+    LOGV2_DEBUG(21289,
+                3,
+                "setting minvalid to exactly: {minValidString}({minValidBSON})",
+                "Setting minvalid to exactly",
+                "minValidString"_attr = minValid.toString(),
+                "minValidBSON"_attr = minValid.toBSON());
     TimestampedBSONObj update;
     update.obj =
         BSON("$set" << BSON(MinValidDocument::kMinValidTimestampFieldName
@@ -198,6 +223,7 @@ void ReplicationConsistencyMarkersImpl::setMinValid(OperationContext* opCtx,
 
     // This method is only used with storage engines that do not support recover to stable
     // timestamp. As a result, their timestamps do not matter.
+    invariant(!opCtx->getServiceContext()->getStorageEngine()->supportsRecoverToStableTimestamp());
     update.timestamp = Timestamp();
 
     _updateMinValidDocument(opCtx, update);
@@ -205,8 +231,12 @@ void ReplicationConsistencyMarkersImpl::setMinValid(OperationContext* opCtx,
 
 void ReplicationConsistencyMarkersImpl::setMinValidToAtLeast(OperationContext* opCtx,
                                                              const OpTime& minValid) {
-    LOG(3) << "setting minvalid to at least: " << minValid.toString() << "(" << minValid.toBSON()
-           << ")";
+    LOGV2_DEBUG(21290,
+                3,
+                "setting minvalid to at least: {minValidString}({minValidBSON})",
+                "Setting minvalid to at least",
+                "minValidString"_attr = minValid.toString(),
+                "minValidBSON"_attr = minValid.toBSON());
 
     auto& termField = MinValidDocument::kMinValidTermFieldName;
     auto& tsField = MinValidDocument::kMinValidTimestampFieldName;
@@ -246,7 +276,12 @@ void ReplicationConsistencyMarkersImpl::setAppliedThrough(OperationContext* opCt
                                                           const OpTime& optime,
                                                           bool setTimestamp) {
     invariant(!optime.isNull());
-    LOG(3) << "setting appliedThrough to: " << optime.toString() << "(" << optime.toBSON() << ")";
+    LOGV2_DEBUG(21291,
+                3,
+                "setting appliedThrough to: {appliedThroughString}({appliedThroughBSON})",
+                "Setting appliedThrough",
+                "appliedThroughString"_attr = optime.toString(),
+                "appliedThroughBSON"_attr = optime.toBSON());
 
     // We set the 'appliedThrough' to the provided timestamp. The 'appliedThrough' is only valid
     // in checkpoints that contain all writes through this timestamp since it indicates the top of
@@ -262,7 +297,11 @@ void ReplicationConsistencyMarkersImpl::setAppliedThrough(OperationContext* opCt
 
 void ReplicationConsistencyMarkersImpl::clearAppliedThrough(OperationContext* opCtx,
                                                             const Timestamp& writeTimestamp) {
-    LOG(3) << "clearing appliedThrough at: " << writeTimestamp.toString();
+    LOGV2_DEBUG(21292,
+                3,
+                "clearing appliedThrough at: {writeTimestamp}",
+                "Clearing appliedThrough",
+                "writeTimestamp"_attr = writeTimestamp.toString());
 
     TimestampedBSONObj update;
     update.timestamp = writeTimestamp;
@@ -277,15 +316,80 @@ OpTime ReplicationConsistencyMarkersImpl::getAppliedThrough(OperationContext* op
 
     auto appliedThrough = doc->getAppliedThrough();
     if (!appliedThrough) {
-        LOG(3) << "No appliedThrough OpTime set, returning empty appliedThrough OpTime.";
+        LOGV2_DEBUG(
+            21293, 3, "No appliedThrough OpTime set, returning empty appliedThrough OpTime");
         return {};
     }
-    LOG(3) << "returning appliedThrough: " << appliedThrough->toString() << "("
-           << appliedThrough->toBSON() << ")";
+    LOGV2_DEBUG(21294,
+                3,
+                "returning appliedThrough: {appliedThroughString}({appliedThroughBSON})",
+                "Returning appliedThrough",
+                "appliedThroughString"_attr = appliedThrough->toString(),
+                "appliedThroughBSON"_attr = appliedThrough->toBSON());
 
     return appliedThrough.get();
 }
 
+void ReplicationConsistencyMarkersImpl::ensureFastCountOnOplogTruncateAfterPoint(
+    OperationContext* opCtx) {
+    LOGV2_DEBUG(
+        21295,
+        3,
+        "Updating cached fast-count on collection {oplogTruncateAfterPointNamespace} in case an "
+        "unclean shutdown caused it to become incorrect.",
+        "Updating cached fast-count on oplog truncate after point collection in case an unclean "
+        "shutdown caused it to become incorrect",
+        "oplogTruncateAfterPointNamespace"_attr = _oplogTruncateAfterPointNss);
+
+    auto result = _storageInterface->findSingleton(opCtx, _oplogTruncateAfterPointNss);
+
+    if (result.getStatus() == ErrorCodes::NamespaceNotFound) {
+        return;
+    }
+
+    if (result.getStatus() == ErrorCodes::CollectionIsEmpty) {
+        // The count is updated before successful commit of a write, so unclean shutdown can leave
+        // the value incorrectly set to one.
+        invariant(
+            _storageInterface->setCollectionCount(opCtx, _oplogTruncateAfterPointNss, 0).isOK());
+        return;
+    }
+
+    if (result.getStatus() == ErrorCodes::TooManyMatchingDocuments) {
+        fassert(51265,
+                {result.getStatus().code(),
+                 str::stream() << "More than one document was found in the '"
+                               << kDefaultOplogTruncateAfterPointNamespace
+                               << "' collection. Users should not write to this collection. Please "
+                                  "delete the excess documents"});
+    }
+    fassert(51266, result.getStatus());
+
+    // We can safely set a count of one. We know that we only ever write one document, and the
+    // success of findSingleton above confirms only one document exists in the collection.
+    invariant(_storageInterface->setCollectionCount(opCtx, _oplogTruncateAfterPointNss, 1).isOK());
+}
+
+void ReplicationConsistencyMarkersImpl::_upsertOplogTruncateAfterPointDocument(
+    OperationContext* opCtx, const BSONObj& updateSpec) {
+    fassert(40512,
+            _storageInterface->upsertById(
+                opCtx, _oplogTruncateAfterPointNss, kOplogTruncateAfterPointId["_id"], updateSpec));
+}
+
+void ReplicationConsistencyMarkersImpl::setOplogTruncateAfterPoint(OperationContext* opCtx,
+                                                                   const Timestamp& timestamp) {
+    LOGV2_DEBUG(21296,
+                3,
+                "setting oplog truncate after point to: {oplogTruncateAfterPoint}",
+                "Setting oplog truncate after point",
+                "oplogTruncateAfterPoint"_attr = timestamp.toBSON());
+    _upsertOplogTruncateAfterPointDocument(
+        opCtx,
+        BSON("$set" << BSON(OplogTruncateAfterPointDocument::kOplogTruncateAfterPointFieldName
+                            << timestamp)));
+}
+
 boost::optional<OplogTruncateAfterPointDocument>
 ReplicationConsistencyMarkersImpl::_getOplogTruncateAfterPointDocument(
     OperationContext* opCtx) const {
@@ -307,34 +411,111 @@ ReplicationConsistencyMarkersImpl::_getOplogTruncateAfterPointDocument(
     return oplogTruncateAfterPoint;
 }
 
-void ReplicationConsistencyMarkersImpl::_upsertOplogTruncateAfterPointDocument(
-    OperationContext* opCtx, const BSONObj& updateSpec) {
-    fassert(40512,
-            _storageInterface->upsertById(
-                opCtx, _oplogTruncateAfterPointNss, kOplogTruncateAfterPointId["_id"], updateSpec));
-}
-
-void ReplicationConsistencyMarkersImpl::setOplogTruncateAfterPoint(OperationContext* opCtx,
-                                                                   const Timestamp& timestamp) {
-    LOG(3) << "setting oplog truncate after point to: " << timestamp.toBSON();
-    _upsertOplogTruncateAfterPointDocument(
-        opCtx,
-        BSON("$set" << BSON(OplogTruncateAfterPointDocument::kOplogTruncateAfterPointFieldName
-                            << timestamp)));
-}
-
 Timestamp ReplicationConsistencyMarkersImpl::getOplogTruncateAfterPoint(
     OperationContext* opCtx) const {
-    auto doc = _getOplogTruncateAfterPointDocument(opCtx);
-    if (!doc) {
-        LOG(3) << "Returning empty oplog truncate after point since document did not exist";
-        return {};
+    auto truncatePointDoc = _getOplogTruncateAfterPointDocument(opCtx);
+    if (!truncatePointDoc) {
+        LOGV2_DEBUG(
+            21297, 3, "Returning empty oplog truncate after point since document did not exist");
+        return Timestamp();
+    }
+    Timestamp truncatePointTimestamp = truncatePointDoc->getOplogTruncateAfterPoint();
+
+    LOGV2_DEBUG(21298,
+                3,
+                "Returning oplog truncate after point: {oplogTruncateAfterPoint}",
+                "Returning oplog truncate after point",
+                "oplogTruncateAfterPoint"_attr = truncatePointTimestamp);
+    return truncatePointTimestamp;
+}
+
+void ReplicationConsistencyMarkersImpl::startUsingOplogTruncateAfterPointForPrimary() {
+    stdx::lock_guard<Latch> lk(_truncatePointIsPrimaryMutex);
+    // There is only one path to stepup and it is not called redundantly.
+    invariant(!_isPrimary);
+    _isPrimary = true;
+}
+
+void ReplicationConsistencyMarkersImpl::stopUsingOplogTruncateAfterPointForPrimary() {
+    stdx::lock_guard<Latch> lk(_truncatePointIsPrimaryMutex);
+    _isPrimary = false;
+}
+
+bool ReplicationConsistencyMarkersImpl::isOplogTruncateAfterPointBeingUsedForPrimary() const {
+    stdx::lock_guard<Latch> lk(_truncatePointIsPrimaryMutex);
+    return _isPrimary;
+}
+
+void ReplicationConsistencyMarkersImpl::setOplogTruncateAfterPointToTopOfOplog(
+    OperationContext* opCtx) {
+    auto timestamp = _storageInterface->getLatestOplogTimestamp(opCtx);
+    LOGV2_DEBUG(21551,
+                3,
+                "Initializing oplog truncate after point: {oplogTruncateAfterPoint}",
+                "Initializing oplog truncate after point",
+                "oplogTruncateAfterPoint"_attr = timestamp);
+    setOplogTruncateAfterPoint(opCtx, timestamp);
+}
+
+boost::optional<OpTimeAndWallTime>
+ReplicationConsistencyMarkersImpl::refreshOplogTruncateAfterPointIfPrimary(
+    OperationContext* opCtx) {
+
+    if (!isOplogTruncateAfterPointBeingUsedForPrimary()) {
+        // Stepdown clears the truncate point, after which the truncate point is set manually as
+        // needed, so nothing should be done here -- else we might truncate something we should not.
+        return boost::none;
     }
 
-    Timestamp out = doc->getOplogTruncateAfterPoint();
+    // Temporarily allow writes if kIgnoreConflicts is set on the recovery unit so the truncate
+    // point can be updated. The kIgnoreConflicts setting only allows reads.
+    auto originalBehavior = opCtx->recoveryUnit()->getPrepareConflictBehavior();
+    if (originalBehavior == PrepareConflictBehavior::kIgnoreConflicts) {
+        opCtx->recoveryUnit()->setPrepareConflictBehavior(
+            PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
+    }
+    ON_BLOCK_EXIT([&] { opCtx->recoveryUnit()->setPrepareConflictBehavior(originalBehavior); });
 
-    LOG(3) << "returning oplog truncate after point: " << out;
-    return out;
+    // The locks necessary to write to the oplog truncate after point's collection and read from the
+    // oplog collection must be taken up front so that the mutex can also be taken around both
+    // operations without causing deadlocks.
+    AutoGetCollection autoTruncateColl(opCtx, _oplogTruncateAfterPointNss, MODE_IX);
+    AutoGetCollection autoOplogColl(opCtx, NamespaceString::kRsOplogNamespace, MODE_IS);
+    stdx::lock_guard<Latch> lk(_refreshOplogTruncateAfterPointMutex);
+
+    // Update the oplogTruncateAfterPoint to the storage engine's reported oplog timestamp with no
+    // holes behind it in-memory (only, not on disk, despite the name).
+    auto truncateTimestamp = _storageInterface->getAllDurableTimestamp(opCtx->getServiceContext());
+
+    if (truncateTimestamp != Timestamp(StorageEngine::kMinimumTimestamp)) {
+        setOplogTruncateAfterPoint(opCtx, truncateTimestamp);
+    } else {
+        // The all_durable timestamp has not yet been set: there have been no oplog writes since
+        // this server instance started up. In this case, we will return the current
+        // oplogTruncateAfterPoint without updating it, since there's nothing to update.
+        truncateTimestamp = getOplogTruncateAfterPoint(opCtx);
+
+        // A primary cannot have an unset oplogTruncateAfterPoint because it is initialized on
+        // step-up.
+        invariant(!truncateTimestamp.isNull());
+    }
+
+    // Reset the snapshot so that it is ensured to see the latest oplog entries.
+    opCtx->recoveryUnit()->abandonSnapshot();
+
+    // Fetch the oplog entry <= timestamp. all_durable may be set to a value between oplog entries.
+    // We need an oplog entry in order to return term and wallclock time for an OpTimeAndWallTime
+    // result.
+    auto truncateOplogEntryBSON = _storageInterface->findOplogEntryLessThanOrEqualToTimestamp(
+        opCtx, autoOplogColl.getCollection(), truncateTimestamp);
+
+    // The truncate point moves the Durable timestamp forward, so it should always exist in the
+    // oplog.
+    invariant(truncateOplogEntryBSON, "Found no oplog entry lte " + truncateTimestamp.toString());
+
+    return fassert(
+        4455501,
+        OpTimeAndWallTime::parseOpTimeAndWallTimeFromOplogEntry(truncateOplogEntryBSON.get()));
 }
 
 Status ReplicationConsistencyMarkersImpl::createInternalCollections(OperationContext* opCtx) {
@@ -346,9 +527,47 @@ Status ReplicationConsistencyMarkersImpl::createInternalCollections(OperationCon
                                   << " Error: " << status.toString()};
         }
     }
-
     return Status::OK();
 }
 
+void ReplicationConsistencyMarkersImpl::setInitialSyncIdIfNotSet(OperationContext* opCtx) {
+    auto status =
+        _storageInterface->createCollection(opCtx, _initialSyncIdNss, CollectionOptions());
+    if (!status.isOK() && status.code() != ErrorCodes::NamespaceExists) {
+        LOGV2_FATAL(
+            4608500, "Failed to create collection", "namespace"_attr = _initialSyncIdNss.ns());
+        fassertFailedWithStatus(4608502, status);
+    }
+
+    auto prevId = _storageInterface->findSingleton(opCtx, _initialSyncIdNss);
+    if (prevId.getStatus() == ErrorCodes::CollectionIsEmpty) {
+        auto doc = BSON("_id" << UUID::gen() << "wallTime"
+                              << opCtx->getServiceContext()->getPreciseClockSource()->now());
+        fassert(4608503,
+                _storageInterface->insertDocument(opCtx,
+                                                  _initialSyncIdNss,
+                                                  TimestampedBSONObj{doc, Timestamp()},
+                                                  OpTime::kUninitializedTerm));
+    } else if (!prevId.isOK()) {
+        fassertFailedWithStatus(4608504, prevId.getStatus());
+    }
+}
+
+void ReplicationConsistencyMarkersImpl::clearInitialSyncId(OperationContext* opCtx) {
+    fassert(4608501, _storageInterface->dropCollection(opCtx, _initialSyncIdNss));
+}
+
+BSONObj ReplicationConsistencyMarkersImpl::getInitialSyncId(OperationContext* opCtx) {
+    auto idStatus = _storageInterface->findSingleton(opCtx, _initialSyncIdNss);
+    if (idStatus.isOK()) {
+        return idStatus.getValue();
+    }
+    if (idStatus.getStatus() != ErrorCodes::CollectionIsEmpty &&
+        idStatus.getStatus() != ErrorCodes::NamespaceNotFound) {
+        uassertStatusOK(idStatus);
+    }
+    return BSONObj();
+}
+
 }  // namespace repl
 }  // namespace mongo
