@@ -41,6 +41,9 @@
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/split_horizon.h"
 #include "mongo/db/repl/sync_source_selector.h"
+#include "mongo/db/storage/storage_engine_init.h"
+#include "mongo/executor/task_executor.h"
+#include "mongo/rpc/topology_version_gen.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
 
@@ -72,7 +75,7 @@ namespace repl {
 class BackgroundSync;
 class IsMasterResponse;
 class OpTime;
-struct OpTimeAndWallTime;
+class OpTimeAndWallTime;
 class ReadConcernArgs;
 class ReplSetConfig;
 class ReplSetHeartbeatArgsV1;
@@ -113,7 +116,8 @@ public:
      * components of the replication system to start up whatever threads and do whatever
      * initialization they need.
      */
-    virtual void startup(OperationContext* opCtx) = 0;
+    virtual void startup(OperationContext* opCtx,
+                         LastStorageEngineShutdownState lastStorageEngineShutdownState) = 0;
 
     /**
      * Start terminal shutdown.  This causes the topology coordinator to refuse to vote in any
@@ -131,6 +135,15 @@ public:
      */
     virtual void shutdown(OperationContext* opCtx) = 0;
 
+    /**
+     * Performs some bookkeeping to make sure that it's a clean shutdown (i.e. dataset is
+     * consistent with top of the oplog).
+     * This should be called after calling shutdown() and should make sure that that are no
+     * active readers while executing this method as this does perform timestamped storage
+     * writes at lastAppliedTimestamp.
+     */
+    virtual void markAsCleanShutdownIfPossible(OperationContext* opCtx) = 0;
+
     /**
      * Returns a reference to the parsed command line arguments that are related to replication.
      */
@@ -199,7 +212,7 @@ public:
      *     the data has been sufficiently replicated
      * ErrorCodes::ExceededTimeLimit if the opCtx->getMaxTimeMicrosRemaining is reached before
      *     the data has been sufficiently replicated
-     * ErrorCodes::NotWritablePrimary if the node is not a writable primary
+     * ErrorCodes::NotMaster if the node is not Primary/Master
      * ErrorCodes::UnknownReplWriteConcern if the writeConcern.wMode contains a write concern
      *     mode that is not known
      * ErrorCodes::ShutdownInProgress if we are mid-shutdown
@@ -250,13 +263,15 @@ public:
      * The result of this function should be consistent with canAcceptWritesForDatabase()
      * for the database the namespace refers to, with additional checks on the collection.
      */
-    virtual bool canAcceptWritesFor(OperationContext* opCtx, const NamespaceString& ns) = 0;
+    virtual bool canAcceptWritesFor(OperationContext* opCtx,
+                                    const NamespaceStringOrUUID& nsOrUUID) = 0;
 
     /**
      * Version which does not check for the RSTL.  Do not use in new code. Without the RSTL held,
      * the return value may be inaccurate by the time the function returns.
      */
-    virtual bool canAcceptWritesFor_UNSAFE(OperationContext* opCtx, const NamespaceString& ns) = 0;
+    virtual bool canAcceptWritesFor_UNSAFE(OperationContext* opCtx,
+                                           const NamespaceStringOrUUID& nsOrUUID) = 0;
 
     /**
      * Checks if the current replica set configuration can satisfy the given write concern.
@@ -279,15 +294,10 @@ public:
         const CommitQuorumOptions& commitQuorum) const = 0;
 
     /**
-     * Checks if the 'commitQuorum' has been satisfied by the 'commitReadyMembers', if it has been
-     * satisfied, return true.
-     *
-     * Prior to checking if the 'commitQuorum' is satisfied by 'commitReadyMembers', it calls
-     * 'checkIfCommitQuorumCanBeSatisfied()' with all the replica set members.
+     * Checks if the 'members' list can satisfy the 'commitQuorum'.
      */
-    virtual StatusWith<bool> checkIfCommitQuorumIsSatisfied(
-        const CommitQuorumOptions& commitQuorum,
-        const std::vector<HostAndPort>& commitReadyMembers) const = 0;
+    virtual bool isCommitQuorumSatisfied(const CommitQuorumOptions& commitQuorum,
+                                         const std::vector<mongo::HostAndPort>& members) const = 0;
 
     /**
      * Returns Status::OK() if it is valid for this node to serve reads on the given collection
@@ -481,7 +491,7 @@ public:
      * mode. This is used for transitioning to RS_ROLLBACK so that we can conflict with readers
      * holding the RSTL in intent mode.
      */
-    virtual Status setFollowerModeStrict(OperationContext* opCtx, const MemberState& newState) = 0;
+    virtual Status setFollowerModeRollback(OperationContext* opCtx) = 0;
 
     /**
      * Step-up
@@ -591,13 +601,6 @@ public:
      */
     virtual Status resyncData(OperationContext* opCtx, bool waitUntilCompleted) = 0;
 
-    /**
-     * Handles an incoming isMaster command for a replica set node.  Should not be
-     * called on a standalone node.
-     */
-    virtual void fillIsMasterForReplSet(IsMasterResponse* result,
-                                        const SplitHorizon::Parameters& horizonParams) = 0;
-
     /**
      * Adds to "result" a description of the slaveInfo data structure used to map RIDs to their
      * last known optimes.
@@ -611,8 +614,11 @@ public:
 
     /**
      * Handles an incoming replSetGetConfig command. Adds BSON to 'result'.
+     *
+     * If commitmentStatus is true, adds a boolean 'commitmentStatus' field to 'result' indicating
+     * whether the current config is committed.
      */
-    virtual void processReplSetGetConfig(BSONObjBuilder* result) = 0;
+    virtual void processReplSetGetConfig(BSONObjBuilder* result, bool commitmentStatus = false) = 0;
 
     /**
      * Processes the ReplSetMetadata returned from a command run against another
@@ -645,12 +651,9 @@ public:
     /**
      * Toggles maintenanceMode to the value expressed by 'activate'
      * return Status::OK if the change worked, NotSecondary if it failed because we are
-     * PRIMARY, and OperationFailed if we are not currently in maintenance mode.
-     *
-     * Takes the ReplicationStateTransitionLock (RSTL) in X mode, since the state can potentially
-     * change to and from RECOVERING.
+     * PRIMARY, and OperationFailed if we are not currently in maintenance mode
      */
-    virtual Status setMaintenanceMode(OperationContext* opCtx, bool activate) = 0;
+    virtual Status setMaintenanceMode(bool activate) = 0;
 
     /**
      * Retrieves the current count of maintenanceMode and returns 'true' if greater than 0.
@@ -686,7 +689,7 @@ public:
      */
     struct ReplSetReconfigArgs {
         BSONObj newConfigObj;
-        bool force;
+        bool force = false;
     };
 
     /**
@@ -697,6 +700,22 @@ public:
                                           const ReplSetReconfigArgs& args,
                                           BSONObjBuilder* resultObj) = 0;
 
+    /**
+     * Install the new config returned by the callback "getNewConfig".
+     */
+    using GetNewConfigFn = std::function<StatusWith<ReplSetConfig>(const ReplSetConfig& oldConfig,
+                                                                   long long currentTerm)>;
+    virtual Status doReplSetReconfig(OperationContext* opCtx,
+                                     GetNewConfigFn getNewConfig,
+                                     bool force) = 0;
+
+    /**
+     * Waits until the following two conditions are satisfied:
+     *  (1) The current config has propagated to a majority of nodes.
+     *  (2) Any operations committed in the previous config are committed in the current config.
+     */
+    virtual Status awaitConfigCommitment(OperationContext* opCtx, bool waitForOplogCommitment) = 0;
+
     /*
      * Handles an incoming replSetInitiate command. If "configObj" is empty, generates a default
      * configuration to use.
@@ -711,7 +730,7 @@ public:
      * Returns Status::OK() if all updates are processed correctly, NodeNotFound
      * if any updating node cannot be found in the config, InvalidReplicaSetConfig if the
      * "configVersion" sent in any of the updates doesn't match our config version, or
-     * NotPrimaryOrSecondary if we are in state REMOVED or otherwise don't have a valid
+     * NotMasterOrSecondary if we are in state REMOVED or otherwise don't have a valid
      * replica set config.
      * If a non-OK status is returned, it is unspecified whether none or some of the updates
      * were applied.
@@ -732,13 +751,6 @@ public:
      */
     virtual std::vector<HostAndPort> getHostsWrittenTo(const OpTime& op, bool durablyWritten) = 0;
 
-    /**
-     * Returns a vector of the members other than ourself in the replica set, as specified in
-     * the replica set config.  Invalid to call if we are not in replica set mode.  Returns
-     * an empty vector if we do not have a valid config.
-     */
-    virtual std::vector<HostAndPort> getOtherNodesInReplSet() const = 0;
-
     /**
      * Returns a BSONObj containing a representation of the current default write concern.
      */
@@ -804,7 +816,13 @@ public:
     /**
      * Returns the current term.
      */
-    virtual long long getTerm() = 0;
+    virtual long long getTerm() const = 0;
+
+    /**
+     * Returns the TopologyVersion. It is possible to return a stale value. This is safe because
+     * we expect the 'processId' field to never change and 'counter' should always be increasing.
+     */
+    virtual TopologyVersion getTopologyVersion() const = 0;
 
     /**
      * Attempts to update the current term for the V1 election protocol. If the term changes and
@@ -849,15 +867,8 @@ public:
     virtual void appendConnectionStats(executor::ConnectionPoolStats* stats) const = 0;
 
     /**
-     * Gets the number of uncommitted snapshots currently held.
-     * Warning: This value can change at any time and may not even be accurate at the time of
-     * return. It should not be used when an exact amount is needed.
-     */
-    virtual size_t getNumUncommittedSnapshots() = 0;
-
-    /**
-     * Creates a CallbackWaiter that waits for w:majority write concern to be satisfied up to opTime
-     * before setting the 'wMajorityWriteAvailabilityDate' election candidate metric.
+     * Creates a waiter that waits for w:majority write concern to be satisfied up to opTime before
+     * setting the 'wMajorityWriteAvailabilityDate' election candidate metric.
      */
     virtual void createWMajorityWriteAvailabilityDateWaiter(OpTime opTime) = 0;
 
@@ -899,10 +910,17 @@ public:
     virtual void signalDropPendingCollectionsRemovedFromStorage() = 0;
 
     /**
-     * Returns true if logOp() should not append an entry to the oplog for the namespace for this
-     * operation.
+     * Returns true if logOp() should not append an entry to the oplog for this operation.
      */
-    bool isOplogDisabledFor(OperationContext* opCtx, const NamespaceString& nss);
+    bool isOplogDisabledFor(OperationContext* opCtx, const NamespaceString& nss) const;
+
+    /**
+     * Returns true if logOp() should never append an entry to the oplog for this namespace. logOp()
+     * may not want to append an entry to the oplog for other reasons, even if the namespace is
+     * allowed to be replicated in the oplog (e.g. being a secondary).
+     */
+    static bool isOplogDisabledForNS(const NamespaceString& nss);
+
 
     /**
      * Returns the stable timestamp that the storage engine recovered to on startup. If the
@@ -921,6 +939,10 @@ public:
      */
     virtual void attemptToAdvanceStableTimestamp() = 0;
 
+    /**
+     * If our state is RECOVERING and lastApplied is at least minValid, transition to SECONDARY.
+     */
+    virtual void finishRecoveryIfEligible(OperationContext* opCtx) = 0;
 
     /**
      * Field name of the newPrimaryMsg within the 'o' field in the new term oplog entry.
@@ -947,6 +969,75 @@ public:
         const size_t numOpsKilled,
         const size_t numOpsRunning) const = 0;
 
+    /**
+     * Increment the server TopologyVersion and fulfill the promise of any currently waiting
+     * isMaster request.
+     */
+    virtual void incrementTopologyVersion() = 0;
+
+    /**
+     * Constructs and returns an IsMasterResponse. Will block until the given deadline waiting for a
+     * significant topology change if the 'counter' field of 'clientTopologyVersion' is equal to the
+     * current TopologyVersion 'counter' from the TopologyCoordinator. Returns immediately if
+     * 'clientTopologyVersion' < TopologyVersion of the TopologyCoordinator or if the processId
+     * differs.
+     */
+    virtual std::shared_ptr<const IsMasterResponse> awaitIsMasterResponse(
+        OperationContext* opCtx,
+        const SplitHorizon::Parameters& horizonParams,
+        boost::optional<TopologyVersion> clientTopologyVersion,
+        boost::optional<Date_t> deadline) = 0;
+
+    /**
+     * The futurized version of `awaitIsMasterResponse()`:
+     * * The future is ready for all cases that `awaitIsMasterResponse()` returns immediately.
+     * * For cases that `awaitIsMasterResponse()` blocks, calling `get()` on the future is blocking.
+     */
+    virtual SharedSemiFuture<std::shared_ptr<const IsMasterResponse>> getIsMasterResponseFuture(
+        const SplitHorizon::Parameters& horizonParams,
+        boost::optional<TopologyVersion> clientTopologyVersion) = 0;
+
+    /**
+     * Returns the OpTime that consists of the timestamp of the latest oplog entry and the current
+     * term.
+     * This function returns a non-ok status if:
+     * 1. It is called on secondaries.
+     * 2. OperationContext times out or is interrupted.
+     * 3. Oplog collection does not exist.
+     * 4. Oplog collection is empty.
+     * 5. Getting latest oplog timestamp is not supported by the storage engine.
+     */
+    virtual StatusWith<OpTime> getLatestWriteOpTime(OperationContext* opCtx) const noexcept = 0;
+
+    /**
+     * Returns the HostAndPort of the current primary, or an empty HostAndPort if there is no
+     * primary. Note that the primary can change at any time and thus the result may be immediately
+     * stale unless run from the primary with the RSTL held.
+     */
+    virtual HostAndPort getCurrentPrimaryHostAndPort() const = 0;
+
+    /*
+     * Cancels the callback referenced in the given callback handle.
+     * This function expects the activeHandle to be valid.
+     */
+    virtual void cancelCbkHandle(executor::TaskExecutor::CallbackHandle activeHandle) = 0;
+
+    using OnRemoteCmdScheduledFn = std::function<void(executor::TaskExecutor::CallbackHandle)>;
+    using OnRemoteCmdCompleteFn = std::function<void(executor::TaskExecutor::CallbackHandle)>;
+    /**
+     * Runs the given command 'cmdObj' on primary and waits till the response for that command is
+     * received. If the node is primary, then the command will be executed using DBDirectClient to
+     * avoid tcp network calls. Otherwise, the node will execute the remote command using the repl
+     * task executor (AsyncDBClient).
+     * - 'OnRemoteCmdScheduled' will be called once the remote command is scheduled.
+     * - 'OnRemoteCmdComplete' will be called once the response for the remote command is received.
+     */
+    virtual BSONObj runCmdOnPrimaryAndAwaitResponse(OperationContext* opCtx,
+                                                    const std::string& dbName,
+                                                    const BSONObj& cmdObj,
+                                                    OnRemoteCmdScheduledFn onRemoteCmdScheduled,
+                                                    OnRemoteCmdCompleteFn onRemoteCmdComplete) = 0;
+
 protected:
     ReplicationCoordinator();
 };
