@@ -31,12 +31,12 @@
 
 #include <boost/optional.hpp>
 #include <cstddef>
+#include <functional>
 
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/repl/member_state.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/executor/task_executor.h"
-#include "mongo/stdx/functional.h"
 #include "mongo/util/concurrency/thread_pool.h"
 #include "mongo/util/time_support.h"
 
@@ -101,6 +101,15 @@ public:
      */
     virtual void shutdown(OperationContext* opCtx) = 0;
 
+    /**
+     * Clears appliedThrough to indicate that the dataset is consistent with top of the
+     * oplog on shutdown.
+     * This should be called after calling shutdown() and should be called holding RSTL
+     * in mode X to make sure that that are no active readers while executing this method
+     * as this does perform timestamped minvalid writes at lastAppliedTimestamp.
+     */
+    virtual void clearAppliedThroughIfCleanShutdown(OperationContext* opCtx) = 0;
+
     /**
      * Returns task executor for scheduling tasks to be run asynchronously.
      */
@@ -155,10 +164,12 @@ public:
     virtual StatusWith<BSONObj> loadLocalConfigDocument(OperationContext* opCtx) = 0;
 
     /**
-     * Stores the replica set config document in local storage, or returns an error.
+     * Stores the replica set config document in local storage and writes a no-op in the oplog, or
+     * returns an error.
      */
-    virtual Status storeLocalConfigDocument(OperationContext* opCtx, const BSONObj& config) = 0;
-
+    virtual Status storeLocalConfigDocument(OperationContext* opCtx,
+                                            const BSONObj& config,
+                                            bool writeOplog) = 0;
 
     /**
      * Creates the collection for "lastVote" documents and initializes it, or returns an error.
@@ -217,6 +228,20 @@ public:
      */
     virtual void shardingOnStepDownHook() = 0;
 
+    /**
+     * Stops asynchronous updates to and then clears the oplogTruncateAfterPoint.
+     *
+     * Safe to call when there are no oplog writes, and therefore no oplog holes that must be
+     * tracked by the oplogTruncateAfterPoint.
+     *
+     * Only primaries update the truncate point asynchronously; other replication states update the
+     * truncate point manually as necessary. This function should be called whenever replication
+     * leaves state PRIMARY: stepdown; and shutdown while in state PRIMARY. Otherwise, we might
+     * leave a stale oplogTruncateAfterPoint set and cause unnecessary oplog truncation during
+     * startup if the server gets restarted.
+     */
+    virtual void stopAsyncUpdatesOfAndClearOplogTruncateAfterPoint() = 0;
+
     /**
      * Notifies the bgsync and syncSourceFeedback threads to choose a new sync source.
      */
@@ -232,6 +257,11 @@ public:
      */
     virtual void startProducerIfStopped() = 0;
 
+    /**
+     * True if we have discovered that no sync source's oplog overlaps with ours.
+     */
+    virtual bool tooStale() = 0;
+
     /**
      * Drops all snapshots and clears the "committed" snapshot.
      */
