@@ -74,6 +74,9 @@ public:
     virtual bool isInitialSyncFlagSet(OperationContext* opCtx) override;
 
     virtual void shutdown(OperationContext* opCtx);
+
+    virtual void clearAppliedThroughIfCleanShutdown(OperationContext* opCtx);
+
     virtual executor::TaskExecutor* getTaskExecutor() const override;
     virtual ThreadPool* getDbWorkThreadPool() const override;
     virtual Status initializeReplSetStorage(OperationContext* opCtx, const BSONObj& config);
@@ -83,7 +86,9 @@ public:
     virtual bool isSelf(const HostAndPort& host, ServiceContext* service);
     Status createLocalLastVoteCollection(OperationContext* opCtx) final;
     virtual StatusWith<BSONObj> loadLocalConfigDocument(OperationContext* opCtx);
-    virtual Status storeLocalConfigDocument(OperationContext* opCtx, const BSONObj& config);
+    virtual Status storeLocalConfigDocument(OperationContext* opCtx,
+                                            const BSONObj& config,
+                                            bool writeOplog);
     virtual StatusWith<LastVote> loadLocalLastVoteDocument(OperationContext* opCtx);
     virtual Status storeLocalLastVoteDocument(OperationContext* opCtx, const LastVote& lastVote);
     virtual void setGlobalTimestamp(ServiceContext* service, const Timestamp& newTime);
@@ -93,9 +98,11 @@ public:
     virtual HostAndPort getClientHostAndPort(const OperationContext* opCtx);
     virtual void closeConnections();
     virtual void shardingOnStepDownHook();
+    virtual void stopAsyncUpdatesOfAndClearOplogTruncateAfterPoint() override;
     virtual void signalApplierToChooseNewSyncSource();
     virtual void stopProducer();
     virtual void startProducerIfStopped();
+    virtual bool tooStale();
     void dropAllSnapshots() final;
     void updateCommittedSnapshot(const OpTime& newCommitPoint) final;
     void updateLastAppliedSnapshot(const OpTime& optime) final;
@@ -109,7 +116,7 @@ public:
     virtual std::size_t getOplogFetcherInitialSyncMaxFetcherRestarts() const override;
 
     // Methods from JournalListener.
-    virtual JournalListener::Token getToken();
+    virtual JournalListener::Token getToken(OperationContext* opCtx);
     virtual void onDurable(const JournalListener::Token& token);
 
     virtual void setupNoopWriter(Seconds waitTime);
@@ -166,9 +173,9 @@ private:
     // replication.
     SyncSourceFeedback _syncSourceFeedback;
 
-    // The OplogBuffer is used to hold operations read from the sync source.
-    // BackgroundSync adds operations to the OplogBuffer while the applier thread consumes these
-    // operations in batches during oplog application.
+    // The OplogBuffer is used to hold operations read from the sync source. During oplog
+    // application, Backgrounds Sync adds operations to the OplogBuffer while the applier's
+    // OplogBatcher consumes these operations from the buffer in batches.
     std::unique_ptr<OplogBuffer> _oplogBuffer;
 
     // The BackgroundSync class is responsible for pulling ops off the network from the sync source
@@ -195,7 +202,7 @@ private:
     // Task executor used to run replication tasks.
     std::unique_ptr<executor::TaskExecutor> _taskExecutor;
 
-    // Used by repl::multiApply() to apply the sync source's operations in parallel.
+    // Used by repl::applyOplogBatch() to apply the sync source's operations in parallel.
     // Also used by database and collection cloners to perform storage operations.
     // Cloners and oplog application run in separate phases of initial sync so it is fine to share
     // this thread pool.
