@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kReplication
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kTest
 
 #include "mongo/platform/basic.h"
 
@@ -41,10 +41,10 @@
 #include "mongo/db/repl/replication_coordinator_test_fixture.h"
 #include "mongo/db/repl/topology_coordinator.h"
 #include "mongo/executor/network_interface_mock.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/metadata/repl_set_metadata.h"
-#include "mongo/unittest/ensure_fcv.h"
+#include "mongo/unittest/log_test.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 namespace repl {
@@ -53,7 +53,27 @@ namespace {
 using executor::NetworkInterfaceMock;
 using executor::RemoteCommandRequest;
 using executor::RemoteCommandResponse;
-using unittest::EnsureFCV;
+
+TEST(ReplSetHeartbeatArgs, AcceptsUnknownField) {
+    ReplSetHeartbeatArgsV1 hbArgs;
+    hbArgs.setConfigTerm(1);
+    hbArgs.setConfigVersion(1);
+    hbArgs.setHeartbeatVersion(1);
+    hbArgs.setTerm(1);
+    hbArgs.setSenderHost(HostAndPort("host:1"));
+    hbArgs.setSetName("replSet");
+    BSONObjBuilder bob;
+    hbArgs.addToBSON(&bob);
+    bob.append("unknownField", 1);  // append an unknown field.
+    BSONObj cmdObj = bob.obj();
+    ASSERT_OK(hbArgs.initialize(cmdObj));
+
+    // The serialized object should be the same as the original except for the unknown field.
+    BSONObjBuilder bob2;
+    hbArgs.addToBSON(&bob2);
+    bob2.append("unknownField", 1);
+    ASSERT_BSONOBJ_EQ(bob2.obj(), cmdObj);
+}
 
 class ReplCoordHBV1Test : public ReplCoordTest {
 protected:
@@ -74,6 +94,7 @@ ReplSetHeartbeatResponse ReplCoordHBV1Test::receiveHeartbeatFrom(const ReplSetCo
                                                                  const HostAndPort& source) {
     ReplSetHeartbeatArgsV1 hbArgs;
     hbArgs.setConfigVersion(rsConfig.getConfigVersion());
+    hbArgs.setConfigTerm(rsConfig.getConfigTerm());
     hbArgs.setSetName(rsConfig.getReplSetName());
     hbArgs.setSenderHost(source);
     hbArgs.setSenderId(sourceId);
@@ -87,7 +108,8 @@ ReplSetHeartbeatResponse ReplCoordHBV1Test::receiveHeartbeatFrom(const ReplSetCo
 
 TEST_F(ReplCoordHBV1Test,
        NodeJoinsExistingReplSetWhenReceivingAConfigContainingTheNodeViaHeartbeat) {
-    logger::globalLogDomain()->setMinimumLoggedSeverity(logger::LogSeverity::Debug(3));
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{logv2::LogComponent::kDefault,
+                                                              logv2::LogSeverity::Debug(3)};
     ReplSetConfig rsConfig = assertMakeRSConfig(BSON("_id"
                                                      << "mySet"
                                                      << "version" << 3 << "members"
@@ -154,9 +176,465 @@ TEST_F(ReplCoordHBV1Test,
     ASSERT_TRUE(getExternalState()->threadsStarted());
 }
 
+class ReplCoordHBV1ReconfigTest : public ReplCoordHBV1Test {
+public:
+    void setUp() {
+        BSONObj configBson = BSON("_id"
+                                  << "mySet"
+                                  << "version" << initConfigVersion << "term" << initConfigTerm
+                                  << "members" << members << "protocolVersion" << 1);
+        ReplSetConfig rsConfig = assertMakeRSConfig(configBson);
+        assertStartSuccess(configBson, HostAndPort("h2", 1));
+        ASSERT_OK(getReplCoord()->setFollowerMode(MemberState::RS_SECONDARY));
+
+        // Black hole initial heartbeat requests.
+        NetworkInterfaceMock* net = getNet();
+        net->enterNetwork();
+        net->blackHole(net->getNextReadyRequest());
+        net->blackHole(net->getNextReadyRequest());
+        net->exitNetwork();
+    }
+
+    BSONObj makeConfigObj(long long version, boost::optional<long long> term) {
+        BSONObjBuilder bob;
+        bob.appendElements(BSON("_id"
+                                << "mySet"
+                                << "version" << version << "members" << members << "protocolVersion"
+                                << 1));
+        if (term) {
+            bob.append("term", *term);
+        }
+        return bob.obj();
+    }
+
+    ReplSetConfig makeRSConfigWithVersionAndTerm(long long version, long long term) {
+        return assertMakeRSConfig(makeConfigObj(version, term));
+    }
+
+    unittest::MinimumLoggedSeverityGuard severityGuard{logv2::LogComponent::kDefault,
+                                                       logv2::LogSeverity::Debug(3)};
+
+    int initConfigVersion = 2;
+    int initConfigTerm = 2;
+    long long UninitializedTerm = OpTime::kUninitializedTerm;
+    BSONArray members = BSON_ARRAY(BSON("_id" << 1 << "host"
+                                              << "h1:1")
+                                   << BSON("_id" << 2 << "host"
+                                                 << "h2:1")
+                                   << BSON("_id" << 3 << "host"
+                                                 << "h3:1"));
+};
+
+
+TEST_F(ReplCoordHBV1ReconfigTest,
+       NodeSchedulesHeartbeatToFetchConfigIfItHearsAboutConfigWithNewerVersionAndWillInstallIt) {
+    // Config with newer version and same term.
+    ReplSetConfig rsConfig =
+        makeRSConfigWithVersionAndTerm((initConfigVersion + 1), initConfigTerm);
+
+    // Receive a heartbeat request that tells us about a newer config.
+    receiveHeartbeatFrom(rsConfig, 1, HostAndPort("h1", 1));
+
+    getNet()->enterNetwork();
+    ReplSetHeartbeatArgsV1 hbArgs;
+    auto noi = getNet()->getNextReadyRequest();
+    const RemoteCommandRequest& hbrequest = noi->getRequest();
+    ASSERT_EQUALS(HostAndPort("h1", 1), hbrequest.target);
+    ASSERT_OK(hbArgs.initialize(hbrequest.cmdObj));
+    ASSERT_EQUALS("mySet", hbArgs.getSetName());
+    ASSERT_EQUALS(initConfigVersion, hbArgs.getConfigVersion());
+    ASSERT_EQUALS(initConfigTerm, hbArgs.getConfigTerm());
+    ASSERT_EQUALS(OpTime::kInitialTerm, hbArgs.getTerm());
+
+    // Construct the heartbeat response containing the newer config.
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_PRIMARY);
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfigTerm(rsConfig.getConfigTerm());
+    // The smallest valid optime in PV1.
+    OpTime opTime(Timestamp(), 0);
+    hbResp.setAppliedOpTimeAndWallTime({opTime, Date_t()});
+    hbResp.setDurableOpTimeAndWallTime({opTime, Date_t()});
+    BSONObjBuilder responseBuilder;
+    responseBuilder << "ok" << 1;
+    hbResp.addToBSON(&responseBuilder);
+    // Add the raw config object.
+    responseBuilder << "config" << makeConfigObj(initConfigVersion + 1, initConfigTerm);
+    auto origResObj = responseBuilder.obj();
+
+    // Schedule and deliver the heartbeat response.
+    getNet()->scheduleResponse(noi, getNet()->now(), makeResponseStatus(origResObj));
+    getNet()->runReadyNetworkOperations();
+
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigVersion(), initConfigVersion + 1);
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigTerm(), initConfigTerm);
+}
+
+TEST_F(ReplCoordHBV1ReconfigTest,
+       NodeSchedulesHeartbeatToFetchConfigIfItHearsAboutConfigWithNewerTermAndWillInstallIt) {
+    // Config with newer term and same version.
+    ReplSetConfig rsConfig = makeRSConfigWithVersionAndTerm(initConfigVersion, initConfigTerm + 1);
+
+    // Receive a heartbeat request that tells us about a newer config.
+    receiveHeartbeatFrom(rsConfig, 1, HostAndPort("h1", 1));
+
+    getNet()->enterNetwork();
+    ReplSetHeartbeatArgsV1 hbArgs;
+    auto noi = getNet()->getNextReadyRequest();
+    const RemoteCommandRequest& hbrequest = noi->getRequest();
+    ASSERT_EQUALS(HostAndPort("h1", 1), hbrequest.target);
+    ASSERT_OK(hbArgs.initialize(hbrequest.cmdObj));
+    ASSERT_EQUALS("mySet", hbArgs.getSetName());
+    ASSERT_EQUALS(initConfigVersion, hbArgs.getConfigVersion());
+    ASSERT_EQUALS(initConfigTerm, hbArgs.getConfigTerm());
+    ASSERT_EQUALS(OpTime::kInitialTerm, hbArgs.getTerm());
+
+    // Construct the heartbeat response containing the newer config.
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_PRIMARY);
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfigTerm(rsConfig.getConfigTerm());
+    // The smallest valid optime in PV1.
+    OpTime opTime(Timestamp(), 0);
+    hbResp.setAppliedOpTimeAndWallTime({opTime, Date_t()});
+    hbResp.setDurableOpTimeAndWallTime({opTime, Date_t()});
+    BSONObjBuilder responseBuilder;
+    responseBuilder << "ok" << 1;
+    hbResp.addToBSON(&responseBuilder);
+    // Add the raw config object.
+    responseBuilder << "config" << makeConfigObj(initConfigVersion, initConfigTerm + 1);
+    auto origResObj = responseBuilder.obj();
+
+    // Schedule and deliver the heartbeat response.
+    getNet()->scheduleResponse(noi, getNet()->now(), makeResponseStatus(origResObj));
+    getNet()->runReadyNetworkOperations();
+
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigTerm(), initConfigTerm + 1);
+}
+
+TEST_F(ReplCoordHBV1ReconfigTest,
+       NodeShouldntScheduleHeartbeatToFetchConfigIfItHearsAboutSameConfig) {
+    // Config with same term and same version. Shouldn't schedule any heartbeats.
+    receiveHeartbeatFrom(getReplCoord()->getReplicaSetConfig_forTest(), 1, HostAndPort("h1", 1));
+    getNet()->enterNetwork();
+    ASSERT_FALSE(getNet()->hasReadyRequests());
+}
+
+TEST_F(
+    ReplCoordHBV1ReconfigTest,
+    NodeSchedulesHeartbeatToFetchConfigIfItHearsAboutConfigWithNewerTermAndLowerVersionAndWillInstallIt) {
+    // Config with newer term and lower version.
+    ReplSetConfig rsConfig =
+        makeRSConfigWithVersionAndTerm((initConfigVersion - 1), (initConfigTerm + 1));
+
+    // Receive a heartbeat request that tells us about a newer config.
+    receiveHeartbeatFrom(rsConfig, 1, HostAndPort("h1", 1));
+
+    getNet()->enterNetwork();
+    ReplSetHeartbeatArgsV1 hbArgs;
+    auto noi = getNet()->getNextReadyRequest();
+    const RemoteCommandRequest& hbrequest = noi->getRequest();
+    ASSERT_EQUALS(HostAndPort("h1", 1), hbrequest.target);
+    ASSERT_OK(hbArgs.initialize(hbrequest.cmdObj));
+    ASSERT_EQUALS("mySet", hbArgs.getSetName());
+    ASSERT_EQUALS(initConfigVersion, hbArgs.getConfigVersion());
+    ASSERT_EQUALS(initConfigTerm, hbArgs.getConfigTerm());
+    ASSERT_EQUALS(OpTime::kInitialTerm, hbArgs.getTerm());
+
+    // Construct the heartbeat response containing the newer config.
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_PRIMARY);
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfigTerm(rsConfig.getConfigTerm());
+    // The smallest valid optime in PV1.
+    OpTime opTime(Timestamp(), 0);
+    hbResp.setAppliedOpTimeAndWallTime({opTime, Date_t()});
+    hbResp.setDurableOpTimeAndWallTime({opTime, Date_t()});
+    BSONObjBuilder responseBuilder;
+    responseBuilder << "ok" << 1;
+    hbResp.addToBSON(&responseBuilder);
+    // Add the raw config object.
+    responseBuilder << "config" << makeConfigObj((initConfigVersion - 1), (initConfigTerm + 1));
+    auto origResObj = responseBuilder.obj();
+
+    // Schedule and deliver the heartbeat response.
+    getNet()->scheduleResponse(noi, getNet()->now(), makeResponseStatus(origResObj));
+    getNet()->runReadyNetworkOperations();
+
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigVersion(), (initConfigVersion - 1));
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigTerm(), (initConfigTerm + 1));
+}
+
+TEST_F(
+    ReplCoordHBV1ReconfigTest,
+    NodeSchedulesHeartbeatToFetchConfigIfItHearsAboutConfigWithNewerVersionAndUninitializedTermAndWillInstallIt) {
+    // Config with version and uninitialized term.
+    ReplSetConfig rsConfig =
+        makeRSConfigWithVersionAndTerm(initConfigVersion + 1, UninitializedTerm);
+
+    // Receive a heartbeat request that tells us about a newer config.
+    receiveHeartbeatFrom(rsConfig, 1, HostAndPort("h1", 1));
+
+    getNet()->enterNetwork();
+    ReplSetHeartbeatArgsV1 hbArgs;
+    auto noi = getNet()->getNextReadyRequest();
+    const RemoteCommandRequest& hbrequest = noi->getRequest();
+    ASSERT_EQUALS(HostAndPort("h1", 1), hbrequest.target);
+    ASSERT_OK(hbArgs.initialize(hbrequest.cmdObj));
+    ASSERT_EQUALS("mySet", hbArgs.getSetName());
+    ASSERT_EQUALS(initConfigVersion, hbArgs.getConfigVersion());
+    ASSERT_EQUALS(initConfigTerm, hbArgs.getConfigTerm());
+    ASSERT_EQUALS(OpTime::kInitialTerm, hbArgs.getTerm());
+
+    // Construct the heartbeat response containing the newer config.
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_PRIMARY);
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfigTerm(rsConfig.getConfigTerm());
+    // The smallest valid optime in PV1.
+    OpTime opTime(Timestamp(), 0);
+    hbResp.setAppliedOpTimeAndWallTime({opTime, Date_t()});
+    hbResp.setDurableOpTimeAndWallTime({opTime, Date_t()});
+    BSONObjBuilder responseBuilder;
+    responseBuilder << "ok" << 1;
+    hbResp.addToBSON(&responseBuilder);
+    // Add the raw config object.
+    responseBuilder << "config" << makeConfigObj((initConfigVersion + 1), UninitializedTerm);
+    auto origResObj = responseBuilder.obj();
+
+    // Schedule and deliver the heartbeat response.
+    getNet()->scheduleResponse(noi, getNet()->now(), makeResponseStatus(origResObj));
+    getNet()->runReadyNetworkOperations();
+
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigVersion(), (initConfigVersion + 1));
+    ASSERT_EQ(getReplCoord()->getConfig().getConfigTerm(), UninitializedTerm);
+}
+
+TEST_F(ReplCoordHBV1ReconfigTest,
+       NodeSchedulesHeartbeatToFetchNewerConfigAndInstallsConfigWithNoTermField) {
+    // Config with newer version.
+    ReplSetConfig rsConfig =
+        makeRSConfigWithVersionAndTerm(initConfigVersion + 1, UninitializedTerm);
+
+    // Receive a heartbeat request that tells us about a newer config.
+    receiveHeartbeatFrom(rsConfig, 1, HostAndPort("h1", 1));
+
+    getNet()->enterNetwork();
+    ReplSetHeartbeatArgsV1 hbArgs;
+    auto noi = getNet()->getNextReadyRequest();
+    const RemoteCommandRequest& hbrequest = noi->getRequest();
+    ASSERT_EQUALS(HostAndPort("h1", 1), hbrequest.target);
+    ASSERT_OK(hbArgs.initialize(hbrequest.cmdObj));
+    ASSERT_EQUALS("mySet", hbArgs.getSetName());
+    ASSERT_EQUALS(initConfigVersion, hbArgs.getConfigVersion());
+    ASSERT_EQUALS(initConfigTerm, hbArgs.getConfigTerm());
+    ASSERT_EQUALS(OpTime::kInitialTerm, hbArgs.getTerm());
+
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_PRIMARY);
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfig(rsConfig);
+    // The smallest valid optime in PV1.
+    OpTime opTime(Timestamp(), 0);
+    hbResp.setAppliedOpTimeAndWallTime({opTime, Date_t()});
+    hbResp.setDurableOpTimeAndWallTime({opTime, Date_t()});
+    BSONObjBuilder responseBuilder;
+    responseBuilder << "ok" << 1;
+    hbResp.addToBSON(&responseBuilder);
+    auto origResObj = responseBuilder.obj();
+
+    // Construct a heartbeat response object that omits the top-level 't' field and the 'term' field
+    // from the config object. This simulates the case of receiving a heartbeat response from a 4.2
+    // node.
+    BSONObjBuilder finalRes;
+    for (auto field : origResObj.getFieldNames<std::set<std::string>>()) {
+        if (field == "t") {
+            continue;
+        } else if (field == "config") {
+            finalRes.append("config", makeConfigObj(initConfigVersion + 1, boost::none /* term */));
+        } else {
+            finalRes.append(origResObj[field]);
+        }
+    }
+
+    // Make sure the response has no term fields.
+    auto finalResObj = finalRes.obj();
+    ASSERT_FALSE(finalResObj.hasField("t"));
+    ASSERT_TRUE(finalResObj.hasField("config"));
+    ASSERT_TRUE(finalResObj["config"].isABSONObj());
+    ASSERT_FALSE(finalResObj.getObjectField("config").hasField("term"));
+
+    // Schedule and deliver the heartbeat response.
+    getNet()->scheduleResponse(noi, getNet()->now(), makeResponseStatus(finalResObj));
+    getNet()->runReadyNetworkOperations();
+
+    // We should have installed the newer config, even though it had no term attached.
+    auto myConfig = getReplCoord()->getConfig();
+    ASSERT_EQ(myConfig.getConfigVersion(), initConfigVersion + 1);
+    ASSERT_EQ(myConfig.getConfigTerm(), UninitializedTerm);
+}
+
+TEST_F(ReplCoordHBV1Test, RejectHeartbeatReconfigDuringElection) {
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{
+        logv2::LogComponent::kReplicationHeartbeats, logv2::LogSeverity::Debug(1)};
+
+    auto term = 1;
+    auto version = 1;
+    auto members = BSON_ARRAY(member(1, "h1:1") << member(2, "h2:1") << member(3, "h3:1"));
+    auto configObj = configWithMembers(version, term, members);
+    assertStartSuccess(configObj, {"h1", 1});
+
+    OpTime time1(Timestamp(100, 1), 0);
+    replCoordSetMyLastAppliedAndDurableOpTime(time1, getNet()->now());
+    ASSERT_OK(getReplCoord()->setFollowerMode(MemberState::RS_SECONDARY));
+
+    simulateEnoughHeartbeatsForAllNodesUp();
+    simulateSuccessfulDryRun();
+
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_SECONDARY);
+    // Attach a config with a higher version and the same term.
+    ReplSetConfig rsConfig = assertMakeRSConfig(configWithMembers(version + 1, term, members));
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfig(rsConfig);
+    hbResp.setAppliedOpTimeAndWallTime({time1, getNet()->now()});
+    hbResp.setDurableOpTimeAndWallTime({time1, getNet()->now()});
+    auto hbRespObj = (BSONObjBuilder(hbResp.toBSON()) << "ok" << 1).obj();
+
+    startCapturingLogMessages();
+    getReplCoord()->handleHeartbeatResponse_forTest(hbRespObj, 1);
+    getNet()->enterNetwork();
+    getNet()->runReadyNetworkOperations();
+    getNet()->exitNetwork();
+    stopCapturingLogMessages();
+    ASSERT_EQUALS(1,
+                  countTextFormatLogLinesContaining(
+                      "Not scheduling a heartbeat reconfig when running for election"));
+
+    auto net = getNet();
+    net->enterNetwork();
+    while (net->hasReadyRequests()) {
+        auto noi = net->getNextReadyRequest();
+        auto& request = noi->getRequest();
+        LOGV2(482571, "processing", "to"_attr = request.target, "cmd"_attr = request.cmdObj);
+        if (request.cmdObj.firstElementFieldNameStringData() != "replSetRequestVotes") {
+            net->blackHole(noi);
+        } else {
+            auto response = BSON("ok" << 1 << "term" << term << "voteGranted" << true << "reason"
+                                      << "");
+            net->scheduleResponse(noi, net->now(), makeResponseStatus(response));
+        }
+        net->runReadyNetworkOperations();
+    }
+    net->exitNetwork();
+
+    getReplCoord()->waitForElectionFinish_forTest();
+    ASSERT(getReplCoord()->getMemberState().primary());
+}
+
+TEST_F(ReplCoordHBV1Test, AwaitIsMasterReturnsResponseOnReconfigViaHeartbeat) {
+    init();
+    assertStartSuccess(BSON("_id"
+                            << "mySet"
+                            << "version" << 2 << "members"
+                            << BSON_ARRAY(BSON("host"
+                                               << "node1:12345"
+                                               << "_id" << 0)
+                                          << BSON("host"
+                                                  << "node2:12345"
+                                                  << "_id" << 1))),
+                       HostAndPort("node1", 12345));
+
+    // Become primary.
+    ASSERT_OK(getReplCoord()->setFollowerMode(MemberState::RS_SECONDARY));
+    replCoordSetMyLastAppliedOpTime(OpTime(Timestamp(100, 1), 0), Date_t() + Seconds(100));
+    replCoordSetMyLastDurableOpTime(OpTime(Timestamp(100, 1), 0), Date_t() + Seconds(100));
+    simulateSuccessfulV1Election();
+    ASSERT(getReplCoord()->getMemberState().primary());
+
+    auto maxAwaitTime = Milliseconds(5000);
+    auto deadline = getNet()->now() + maxAwaitTime;
+
+    auto currentTopologyVersion = getTopoCoord().getTopologyVersion();
+    auto expectedProcessId = currentTopologyVersion.getProcessId();
+    // A reconfig should increment the TopologyVersion counter.
+    auto expectedCounter = currentTopologyVersion.getCounter() + 1;
+    auto opCtx = makeOperationContext();
+    // awaitIsMasterResponse blocks and waits on a future when the request TopologyVersion equals
+    // the current TopologyVersion of the server.
+    stdx::thread getIsMasterThread([&] {
+        auto response = getReplCoord()->awaitIsMasterResponse(
+            opCtx.get(), {}, currentTopologyVersion, deadline);
+        auto topologyVersion = response->getTopologyVersion();
+        ASSERT_EQUALS(topologyVersion->getCounter(), expectedCounter);
+        ASSERT_EQUALS(topologyVersion->getProcessId(), expectedProcessId);
+
+        // Ensure the isMasterResponse contains the newly added node.
+        const auto hosts = response->getHosts();
+        ASSERT_EQUALS(3, hosts.size());
+        ASSERT_EQUALS("node3", hosts[2].host());
+    });
+
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{logv2::LogComponent::kDefault,
+                                                              logv2::LogSeverity::Debug(3)};
+    ReplSetConfig rsConfig =
+        assertMakeRSConfig(BSON("_id"
+                                << "mySet"
+                                << "version" << 3 << "protocolVersion" << 1 << "members"
+                                << BSON_ARRAY(BSON("_id" << 0 << "host"
+                                                         << "node1:12345"
+                                                         << "priority" << 3)
+                                              << BSON("_id" << 1 << "host"
+                                                            << "node2:12345")
+                                              << BSON("_id" << 2 << "host"
+                                                            << "node3:12345"))));
+    const Date_t startDate = getNet()->now();
+
+    enterNetwork();
+    NetworkInterfaceMock* net = getNet();
+    ASSERT_FALSE(net->hasReadyRequests());
+    exitNetwork();
+    receiveHeartbeatFrom(rsConfig, 1, HostAndPort("node2", 12345));
+
+    enterNetwork();
+    NetworkInterfaceMock::NetworkOperationIterator noi = net->getNextReadyRequest();
+    ReplSetHeartbeatResponse hbResp;
+    hbResp.setSetName("mySet");
+    hbResp.setState(MemberState::RS_PRIMARY);
+    hbResp.setConfigVersion(rsConfig.getConfigVersion());
+    hbResp.setConfig(rsConfig);
+    // The smallest valid optime in PV1.
+    OpTime opTime(Timestamp(), 0);
+    hbResp.setAppliedOpTimeAndWallTime({opTime, Date_t()});
+    hbResp.setDurableOpTimeAndWallTime({opTime, Date_t()});
+    BSONObjBuilder responseBuilder;
+    responseBuilder << "ok" << 1;
+    hbResp.addToBSON(&responseBuilder);
+    net->scheduleResponse(
+        noi, startDate + Milliseconds(200), makeResponseStatus(responseBuilder.obj()));
+    assertRunUntil(startDate + Milliseconds(200));
+
+    // Because the new config is stored using an out-of-band thread, we need to perform some
+    // extra synchronization to let the executor finish the heartbeat reconfig.  We know that
+    // after the out-of-band thread completes, it schedules new heartbeats.  We assume that no
+    // other network operations get scheduled during or before the reconfig, though this may
+    // cease to be true in the future.
+    noi = net->getNextReadyRequest();
+
+    exitNetwork();
+    getIsMasterThread.join();
+}
+
 TEST_F(ReplCoordHBV1Test,
        ArbiterJoinsExistingReplSetWhenReceivingAConfigContainingTheArbiterViaHeartbeat) {
-    logger::globalLogDomain()->setMinimumLoggedSeverity(logger::LogSeverity::Debug(3));
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{logv2::LogComponent::kDefault,
+                                                              logv2::LogSeverity::Debug(3)};
     ReplSetConfig rsConfig =
         assertMakeRSConfig(BSON("_id"
                                 << "mySet"
@@ -229,7 +707,8 @@ TEST_F(ReplCoordHBV1Test,
        NodeDoesNotJoinExistingReplSetWhenReceivingAConfigNotContainingTheNodeViaHeartbeat) {
     // Tests that a node in RS_STARTUP will not transition to RS_REMOVED if it receives a
     // configuration that does not contain it.
-    logger::globalLogDomain()->setMinimumLoggedSeverity(logger::LogSeverity::Debug(3));
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{logv2::LogComponent::kDefault,
+                                                              logv2::LogSeverity::Debug(3)};
     ReplSetConfig rsConfig = assertMakeRSConfig(BSON("_id"
                                                      << "mySet"
                                                      << "version" << 3 << "members"
@@ -311,7 +790,8 @@ TEST_F(ReplCoordHBV1Test,
 TEST_F(ReplCoordHBV1Test,
        NodeChangesToRecoveringStateWhenAllNodesRespondToHeartbeatsWithUnauthorized) {
     // Tests that a node that only has auth error heartbeats is recovering
-    logger::globalLogDomain()->setMinimumLoggedSeverity(logger::LogSeverity::Debug(3));
+    auto severityGuard = unittest::MinimumLoggedSeverityGuard{logv2::LogComponent::kDefault,
+                                                              logv2::LogSeverity::Debug(3)};
     assertStartSuccess(BSON("_id"
                             << "mySet"
                             << "version" << 1 << "members"
@@ -326,7 +806,10 @@ TEST_F(ReplCoordHBV1Test,
     enterNetwork();
     const NetworkInterfaceMock::NetworkOperationIterator noi = getNet()->getNextReadyRequest();
     const RemoteCommandRequest& request = noi->getRequest();
-    log() << request.target.toString() << " processing " << request.cmdObj;
+    LOGV2(21492,
+          "{request_target} processing {request_cmdObj}",
+          "request_target"_attr = request.target.toString(),
+          "request_cmdObj"_attr = request.cmdObj);
     getNet()->scheduleResponse(
         noi,
         getNet()->now(),
@@ -336,8 +819,10 @@ TEST_F(ReplCoordHBV1Test,
 
     if (request.target != HostAndPort("node2", 12345) &&
         request.cmdObj.firstElement().fieldNameStringData() != "replSetHeartbeat") {
-        error() << "Black holing unexpected request to " << request.target << ": "
-                << request.cmdObj;
+        LOGV2_ERROR(21496,
+                    "Black holing unexpected request to {request_target}: {request_cmdObj}",
+                    "request_target"_attr = request.target,
+                    "request_cmdObj"_attr = request.cmdObj);
         getNet()->blackHole(noi);
     }
     getNet()->runReadyNetworkOperations();
@@ -384,9 +869,11 @@ TEST_F(ReplCoordHBV1Test, IgnoreTheContentsOfMetadataWhenItsReplicaSetIdDoesNotM
                                       {opTime, Date_t()},
                                       opTime,
                                       rsConfig.getConfigVersion(),
+                                      0,
                                       unexpectedId,
                                       1,
-                                      -1);
+                                      -1,
+                                      true);
         uassertStatusOK(metadata.writeToMetadata(&responseBuilder));
 
         heartbeatResponse = makeResponseStatus(responseBuilder.obj());
@@ -400,11 +887,16 @@ TEST_F(ReplCoordHBV1Test, IgnoreTheContentsOfMetadataWhenItsReplicaSetIdDoesNotM
         const RemoteCommandRequest& request = noi->getRequest();
         if (request.target == host2 &&
             request.cmdObj.firstElement().fieldNameStringData() == "replSetHeartbeat") {
-            log() << request.target.toString() << " processing " << request.cmdObj;
+            LOGV2(21493,
+                  "{request_target} processing {request_cmdObj}",
+                  "request_target"_attr = request.target.toString(),
+                  "request_cmdObj"_attr = request.cmdObj);
             net->scheduleResponse(noi, net->now(), heartbeatResponse);
         } else {
-            log() << "blackholing request to " << request.target.toString() << ": "
-                  << request.cmdObj;
+            LOGV2(21494,
+                  "blackholing request to {request_target}: {request_cmdObj}",
+                  "request_target"_attr = request.target.toString(),
+                  "request_cmdObj"_attr = request.cmdObj);
             net->blackHole(noi);
         }
         net->runReadyNetworkOperations();
@@ -418,7 +910,7 @@ TEST_F(ReplCoordHBV1Test, IgnoreTheContentsOfMetadataWhenItsReplicaSetIdDoesNotM
     ASSERT_OK(getReplCoord()->processReplSetGetStatus(
         &statusBuilder, ReplicationCoordinator::ReplSetGetStatusResponseStyle::kBasic));
     auto statusObj = statusBuilder.obj();
-    unittest::log() << "replica set status = " << statusObj;
+    LOGV2(21495, "replica set status = {statusObj}", "statusObj"_attr = statusObj);
 
     ASSERT_EQ(mongo::Array, statusObj["members"].type());
     auto members = statusObj["members"].Array();
@@ -428,6 +920,10 @@ TEST_F(ReplCoordHBV1Test, IgnoreTheContentsOfMetadataWhenItsReplicaSetIdDoesNotM
     ASSERT_EQ(host2, HostAndPort(member["name"].String()));
     ASSERT_EQ(MemberState(MemberState::RS_DOWN).toString(),
               MemberState(member["state"].numberInt()).toString());
+    ASSERT_EQ(member["lastHeartbeatMessage"].String(),
+              std::string(str::stream()
+                          << "replica set IDs do not match, ours: " << rsConfig.getReplicaSetId()
+                          << "; remote node's: " << unexpectedId));
 }
 
 TEST_F(ReplCoordHBV1Test,
@@ -460,9 +956,11 @@ TEST_F(ReplCoordHBV1Test,
         {commitPoint, Date_t() + Seconds(commitPoint.getSecs())},  // committed OpTime
         commitPoint,                                               // visibleOpTime
         config.getConfigVersion(),
-        {},  // replset id
-        1,   // currentPrimaryIndex,
-        1);  // currentSyncSourceIndex
+        0,
+        {},     // replset id
+        1,      // currentPrimaryIndex,
+        1,      // currentSyncSourceIndex
+        true);  // isPrimary
 
     auto net = getNet();
     BSONObjBuilder responseBuilder;
@@ -507,84 +1005,6 @@ TEST_F(ReplCoordHBV1Test,
     }
 }
 
-TEST_F(ReplCoordHBV1Test, LastCommittedOpTimeOnlyUpdatesFromHeartbeatInFCV42) {
-    EnsureFCV guard(ServerGlobalParams::FeatureCompatibility::Version::kFullyDowngradedTo40);
-
-    // Ensure that the metadata is processed if it is contained in a heartbeat response.
-    assertStartSuccess(BSON("_id"
-                            << "mySet"
-                            << "version" << 2 << "members"
-                            << BSON_ARRAY(BSON("host"
-                                               << "node1:12345"
-                                               << "_id" << 0)
-                                          << BSON("host"
-                                                  << "node2:12345"
-                                                  << "_id" << 1))
-                            << "protocolVersion" << 1),
-                       HostAndPort("node1", 12345));
-    ASSERT_OK(getReplCoord()->setFollowerMode(MemberState::RS_SECONDARY));
-    ASSERT_EQUALS(OpTime(), getReplCoord()->getLastCommittedOpTime());
-
-    auto config = getReplCoord()->getConfig();
-
-    auto lastAppliedOpTime = OpTime({11, 1}, 2);
-    auto commitPoint = OpTime({15, 1}, 2);
-    replCoordSetMyLastAppliedOpTime(lastAppliedOpTime, Date_t() + Seconds(100));
-
-    // Node 1 is the current primary.
-    rpc::ReplSetMetadata metadata(
-        2,                                                         // term
-        {commitPoint, Date_t() + Seconds(commitPoint.getSecs())},  // committed OpTime
-        commitPoint,                                               // visibleOpTime
-        config.getConfigVersion(),
-        {},  // replset id
-        1,   // currentPrimaryIndex,
-        1);  // currentSyncSourceIndex
-
-    auto net = getNet();
-    BSONObjBuilder responseBuilder;
-    ASSERT_OK(metadata.writeToMetadata(&responseBuilder));
-
-    ReplSetHeartbeatResponse hbResp;
-    hbResp.setConfigVersion(config.getConfigVersion());
-    hbResp.setSetName(config.getReplSetName());
-    hbResp.setState(MemberState::RS_PRIMARY);
-    responseBuilder.appendElements(hbResp.toBSON());
-    auto hbRespObj = responseBuilder.obj();
-    {
-        net->enterNetwork();
-        ASSERT_TRUE(net->hasReadyRequests());
-        auto noi = net->getNextReadyRequest();
-        auto& request = noi->getRequest();
-        ASSERT_EQUALS(config.getMemberAt(1).getHostAndPort(), request.target);
-        ASSERT_EQUALS("replSetHeartbeat", request.cmdObj.firstElement().fieldNameStringData());
-
-        net->scheduleResponse(noi, net->now(), makeResponseStatus(hbRespObj));
-        net->runReadyNetworkOperations();
-        net->exitNetwork();
-
-        ASSERT_EQUALS(OpTime(), getReplCoord()->getLastCommittedOpTime());
-        ASSERT_EQUALS(2, getReplCoord()->getTerm());
-    }
-
-    // Set FCV to 4.2, so commit point can be advanced through heartbeats.
-    serverGlobalParams.featureCompatibility.setVersion(
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo42);
-    {
-        net->enterNetwork();
-        net->runUntil(net->now() + config.getHeartbeatInterval());
-        auto noi = net->getNextReadyRequest();
-        auto& request = noi->getRequest();
-        ASSERT_EQUALS("replSetHeartbeat", request.cmdObj.firstElement().fieldNameStringData());
-
-        net->scheduleResponse(noi, net->now(), makeResponseStatus(hbRespObj));
-        net->runReadyNetworkOperations();
-        net->exitNetwork();
-
-        ASSERT_EQUALS(commitPoint, getReplCoord()->getLastCommittedOpTime());
-    }
-}
-
 TEST_F(ReplCoordHBV1Test, LastCommittedOpTimeOnlyUpdatesFromHeartbeatIfNotInStartup) {
     // Ensure that the metadata is not processed if it is contained in a heartbeat response,
     // if we are in STARTUP2.
@@ -613,9 +1033,11 @@ TEST_F(ReplCoordHBV1Test, LastCommittedOpTimeOnlyUpdatesFromHeartbeatIfNotInStar
         {commitPoint, Date_t() + Seconds(commitPoint.getSecs())},  // committed OpTime
         commitPoint,                                               // visibleOpTime
         config.getConfigVersion(),
-        {},  // replset id
-        1,   // currentPrimaryIndex,
-        1);  // currentSyncSourceIndex
+        0,
+        {},     // replset id
+        1,      // currentPrimaryIndex,
+        1,      // currentSyncSourceIndex
+        true);  // isPrimary
 
     auto net = getNet();
     BSONObjBuilder responseBuilder;
