@@ -37,12 +37,15 @@ namespace repl {
 ReplicationCoordinatorNoOp::ReplicationCoordinatorNoOp(ServiceContext* service)
     : _service(service) {}
 
-void ReplicationCoordinatorNoOp::startup(OperationContext* opCtx) {}
+void ReplicationCoordinatorNoOp::startup(
+    OperationContext* opCtx, LastStorageEngineShutdownState lastStorageEngineShutdownState) {}
 
 void ReplicationCoordinatorNoOp::enterTerminalShutdown() {}
 
 void ReplicationCoordinatorNoOp::shutdown(OperationContext* opCtx) {}
 
+void ReplicationCoordinatorNoOp::markAsCleanShutdownIfPossible(OperationContext* opCtx) {}
+
 ReplicationCoordinator::Mode ReplicationCoordinatorNoOp::getReplicationMode() const {
     return modeReplSet;
 }
@@ -78,12 +81,12 @@ bool ReplicationCoordinatorNoOp::canAcceptWritesForDatabase_UNSAFE(OperationCont
 }
 
 bool ReplicationCoordinatorNoOp::canAcceptWritesFor_UNSAFE(OperationContext* opCtx,
-                                                           const NamespaceString& ns) {
+                                                           const NamespaceStringOrUUID& nsOrUUID) {
     MONGO_UNREACHABLE;
 }
 
 bool ReplicationCoordinatorNoOp::canAcceptWritesFor(OperationContext* opCtx,
-                                                    const NamespaceString& ns) {
+                                                    const NamespaceStringOrUUID& nsOrUUID) {
     MONGO_UNREACHABLE;
 }
 
@@ -172,8 +175,7 @@ Status ReplicationCoordinatorNoOp::setFollowerMode(const MemberState&) {
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::setFollowerModeStrict(OperationContext* opCtx,
-                                                         const MemberState&) {
+Status ReplicationCoordinatorNoOp::setFollowerModeRollback(OperationContext* opCtx) {
     MONGO_UNREACHABLE;
 }
 
@@ -279,11 +281,6 @@ Status ReplicationCoordinatorNoOp::processReplSetGetStatus(BSONObjBuilder*,
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::fillIsMasterForReplSet(IsMasterResponse*,
-                                                        const SplitHorizon::Parameters&) {
-    MONGO_UNREACHABLE;
-}
-
 void ReplicationCoordinatorNoOp::appendSlaveInfoData(BSONObjBuilder*) {
     MONGO_UNREACHABLE;
 }
@@ -292,7 +289,8 @@ ReplSetConfig ReplicationCoordinatorNoOp::getConfig() const {
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::processReplSetGetConfig(BSONObjBuilder*) {
+void ReplicationCoordinatorNoOp::processReplSetGetConfig(BSONObjBuilder* result,
+                                                         bool commitmentStatus) {
     MONGO_UNREACHABLE;
 }
 
@@ -304,7 +302,7 @@ void ReplicationCoordinatorNoOp::cancelAndRescheduleElectionTimeout() {
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::setMaintenanceMode(OperationContext*, bool) {
+Status ReplicationCoordinatorNoOp::setMaintenanceMode(bool) {
     MONGO_UNREACHABLE;
 }
 
@@ -324,6 +322,17 @@ Status ReplicationCoordinatorNoOp::processReplSetReconfig(OperationContext*,
     MONGO_UNREACHABLE;
 }
 
+Status ReplicationCoordinatorNoOp::doReplSetReconfig(OperationContext* opCtx,
+                                                     GetNewConfigFn getNewConfig,
+                                                     bool force) {
+    MONGO_UNREACHABLE;
+}
+
+Status ReplicationCoordinatorNoOp::awaitConfigCommitment(OperationContext* opCtx,
+                                                         bool waitForOplogCommitment) {
+    MONGO_UNREACHABLE;
+}
+
 Status ReplicationCoordinatorNoOp::processReplSetInitiate(OperationContext*,
                                                           const BSONObj&,
                                                           BSONObjBuilder*) {
@@ -347,10 +356,6 @@ std::vector<HostAndPort> ReplicationCoordinatorNoOp::getHostsWrittenTo(const OpT
     MONGO_UNREACHABLE;
 }
 
-std::vector<HostAndPort> ReplicationCoordinatorNoOp::getOtherNodesInReplSet() const {
-    MONGO_UNREACHABLE;
-}
-
 Status ReplicationCoordinatorNoOp::checkIfWriteConcernCanBeSatisfied(
     const WriteConcernOptions&) const {
     MONGO_UNREACHABLE;
@@ -361,9 +366,8 @@ Status ReplicationCoordinatorNoOp::checkIfCommitQuorumCanBeSatisfied(
     MONGO_UNREACHABLE;
 }
 
-StatusWith<bool> ReplicationCoordinatorNoOp::checkIfCommitQuorumIsSatisfied(
-    const CommitQuorumOptions& commitQuorum,
-    const std::vector<HostAndPort>& commitReadyMembers) const {
+bool ReplicationCoordinatorNoOp::isCommitQuorumSatisfied(
+    const CommitQuorumOptions& commitQuorum, const std::vector<mongo::HostAndPort>& members) const {
     MONGO_UNREACHABLE;
 }
 
@@ -385,7 +389,7 @@ void ReplicationCoordinatorNoOp::resetLastOpTimesFromOplog(OperationContext*, Da
 
 bool ReplicationCoordinatorNoOp::shouldChangeSyncSource(const HostAndPort&,
                                                         const rpc::ReplSetMetadata&,
-                                                        boost::optional<rpc::OplogQueryMetadata>) {
+                                                        const rpc::OplogQueryMetadata&) {
     MONGO_UNREACHABLE;
 }
 
@@ -422,7 +426,7 @@ Status ReplicationCoordinatorNoOp::processHeartbeatV1(const ReplSetHeartbeatArgs
     MONGO_UNREACHABLE;
 }
 
-long long ReplicationCoordinatorNoOp::getTerm() {
+long long ReplicationCoordinatorNoOp::getTerm() const {
     MONGO_UNREACHABLE;
 }
 
@@ -434,10 +438,6 @@ void ReplicationCoordinatorNoOp::waitUntilSnapshotCommitted(OperationContext*, c
     MONGO_UNREACHABLE;
 }
 
-size_t ReplicationCoordinatorNoOp::getNumUncommittedSnapshots() {
-    MONGO_UNREACHABLE;
-}
-
 void ReplicationCoordinatorNoOp::createWMajorityWriteAvailabilityDateWaiter(OpTime opTime) {
     MONGO_UNREACHABLE;
 }
@@ -466,6 +466,10 @@ void ReplicationCoordinatorNoOp::attemptToAdvanceStableTimestamp() {
     MONGO_UNREACHABLE;
 }
 
+void ReplicationCoordinatorNoOp::finishRecoveryIfEligible(OperationContext* opCtx) {
+    MONGO_UNREACHABLE;
+}
+
 void ReplicationCoordinatorNoOp::updateAndLogStateTransitionMetrics(
     const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
     const size_t numOpsKilled,
@@ -473,5 +477,52 @@ void ReplicationCoordinatorNoOp::updateAndLogStateTransitionMetrics(
     MONGO_UNREACHABLE;
 }
 
+TopologyVersion ReplicationCoordinatorNoOp::getTopologyVersion() const {
+    MONGO_UNREACHABLE;
+}
+
+void ReplicationCoordinatorNoOp::incrementTopologyVersion() {
+    MONGO_UNREACHABLE;
+}
+
+std::shared_ptr<const IsMasterResponse> ReplicationCoordinatorNoOp::awaitIsMasterResponse(
+    OperationContext* opCtx,
+    const SplitHorizon::Parameters& horizonParams,
+    boost::optional<TopologyVersion> clientTopologyVersion,
+    boost::optional<Date_t> deadline) {
+    MONGO_UNREACHABLE;
+}
+
+
+SharedSemiFuture<std::shared_ptr<const IsMasterResponse>>
+ReplicationCoordinatorNoOp::getIsMasterResponseFuture(
+    const SplitHorizon::Parameters& horizonParams,
+    boost::optional<TopologyVersion> clientTopologyVersion) {
+    MONGO_UNREACHABLE;
+}
+
+StatusWith<OpTime> ReplicationCoordinatorNoOp::getLatestWriteOpTime(OperationContext* opCtx) const
+    noexcept {
+    return getMyLastAppliedOpTime();
+}
+
+HostAndPort ReplicationCoordinatorNoOp::getCurrentPrimaryHostAndPort() const {
+    MONGO_UNREACHABLE;
+}
+
+void ReplicationCoordinatorNoOp::cancelCbkHandle(
+    executor::TaskExecutor::CallbackHandle activeHandle) {
+    MONGO_UNREACHABLE;
+}
+
+BSONObj ReplicationCoordinatorNoOp::runCmdOnPrimaryAndAwaitResponse(
+    OperationContext* opCtx,
+    const std::string& dbName,
+    const BSONObj& cmdObj,
+    OnRemoteCmdScheduledFn onRemoteCmdScheduled,
+    OnRemoteCmdCompleteFn onRemoteCmdComplete) {
+    MONGO_UNREACHABLE;
+}
+
 }  // namespace repl
 }  // namespace mongo
