@@ -186,7 +186,6 @@ public:
     virtual Status setIndexIsMultikey(OperationContext* opCtx,
                                       const NamespaceString& nss,
                                       const std::string& indexName,
-                                      const std::vector<BSONObj>& multikeyMetadataKeys,
                                       const MultikeyPaths& paths,
                                       Timestamp ts) = 0;
     /**
@@ -314,6 +313,18 @@ public:
                                   const NamespaceString& nss,
                                   const BSONObj& filter) = 0;
 
+    /**
+     * Searches for an oplog entry with a timestamp <= 'timestamp'. Returns boost::none if no
+     * matches are found.
+     */
+    virtual boost::optional<BSONObj> findOplogEntryLessThanOrEqualToTimestamp(
+        OperationContext* opCtx, Collection* oplog, const Timestamp& timestamp) = 0;
+
+    /**
+     * Fetches the latest oplog entry's timestamp. Bypasses the oplog visibility rules.
+     */
+    virtual Timestamp getLatestOplogTimestamp(OperationContext* opCtx) = 0;
+
     using CollectionSize = uint64_t;
     using CollectionCount = uint64_t;
 
@@ -342,12 +353,6 @@ public:
     virtual StatusWith<OptionalCollectionUUID> getCollectionUUID(OperationContext* opCtx,
                                                                  const NamespaceString& nss) = 0;
 
-    /**
-     * Updates unique indexes belonging to all non-replicated collections. To be called at the
-     * end of initial sync.
-     */
-    virtual Status upgradeNonReplicatedUniqueIndexes(OperationContext* opCtx) = 0;
-
     /**
      * Sets the highest timestamp at which the storage engine is allowed to take a checkpoint.
      * This timestamp can never decrease, and thus should be a timestamp that can never roll back.
@@ -380,6 +385,14 @@ public:
      */
     virtual bool supportsRecoveryTimestamp(ServiceContext* serviceCtx) const = 0;
 
+    /**
+     * Responsible for initializing independent processes for replication that manage
+     * and interact with the storage layer.
+     *
+     * Initializes the OplogCapMaintainerThread to control deletion of oplog stones.
+     */
+    virtual void initializeStorageControlsForReplication(ServiceContext* serviceCtx) const = 0;
+
     /**
      * Returns the stable timestamp that the storage engine recovered to on startup. If the
      * recovery point was not stable, returns "none".
@@ -443,18 +456,6 @@ public:
     virtual boost::optional<Timestamp> getLastStableRecoveryTimestamp(
         ServiceContext* serviceCtx) const = 0;
 
-    /**
-     * DEPRECATED. Use getLastStableRecoveryTimestamp instead!
-     *
-     * Returns a timestamp that is guaranteed to be persisted on disk in a checkpoint. Returns
-     * `Timestamp::min()` if no stable checkpoint has been taken. Returns boost::none if
-     * `supportsRecoverToStableTimestamp` returns false or if this is not a persisted data engine.
-     *
-     * TODO: delete this in v4.4 (SERVER-36194).
-     */
-    virtual boost::optional<Timestamp> getLastStableCheckpointTimestampDeprecated(
-        ServiceContext* serviceCtx) const = 0;
-
     /**
      * Returns the read timestamp of the recovery unit of the given operation context.
      */
