@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kReplication
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -45,7 +45,7 @@
 #include "mongo/db/session_catalog_mongod.h"
 #include "mongo/db/transaction_history_iterator.h"
 #include "mongo/db/transaction_participant.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 using repl::OplogEntry;
@@ -60,23 +60,19 @@ MONGO_FAIL_POINT_DEFINE(skipReconstructPreparedTransactions);
 // conflict error.
 MONGO_FAIL_POINT_DEFINE(applyPrepareTxnOpsFailsWithWriteConflict);
 
-MONGO_FAIL_POINT_DEFINE(hangBeforeSessionCheckOutForApplyPrepare);
-
 // Apply the oplog entries for a prepare or a prepared commit during recovery/initial sync.
 Status _applyOperationsForTransaction(OperationContext* opCtx,
-                                      const repl::MultiApplier::Operations& ops,
+                                      const std::vector<OplogEntry>& ops,
                                       repl::OplogApplication::Mode oplogApplicationMode) noexcept {
     // Apply each the operations via repl::applyOperation.
     for (const auto& op : ops) {
         try {
+            // Presently, it is not allowed to run a prepared transaction with a command
+            // inside. TODO(SERVER-46105)
+            invariant(!op.isCommand());
             AutoGetCollection coll(opCtx, op.getNss(), MODE_IX);
-            const bool isDataConsistent = true;
-            auto status = repl::applyOperation_inlock(opCtx,
-                                                      coll.getDb(),
-                                                      op.toBSON(),
-                                                      false /*alwaysUpsert*/,
-                                                      oplogApplicationMode,
-                                                      isDataConsistent);
+            auto status = repl::applyOperation_inlock(
+                opCtx, coll.getDb(), &op, false /*alwaysUpsert*/, oplogApplicationMode);
             if (!status.isOK()) {
                 return status;
             }
@@ -87,14 +83,26 @@ Status _applyOperationsForTransaction(OperationContext* opCtx,
                  oplogApplicationMode == repl::OplogApplication::Mode::kRecovering);
 
             if (!ignoreException) {
-                LOG(1) << "Error applying operation in transaction. " << redact(ex)
-                       << "- oplog entry: " << redact(op.toBSON());
+                LOGV2_DEBUG(
+                    21845,
+                    1,
+                    "Error applying operation in transaction. {error}- oplog entry: {oplogEntry}",
+                    "Error applying operation in transaction",
+                    "error"_attr = redact(ex),
+                    "oplogEntry"_attr = redact(op.toBSON()));
                 return exceptionToStatus();
             }
-            LOG(1) << "Encountered but ignoring error: " << redact(ex)
-                   << " while applying operations for transaction because we are either in initial "
-                      "sync or recovering mode - oplog entry: "
-                   << redact(op.toBSON());
+            LOGV2_DEBUG(21846,
+                        1,
+                        "Encountered but ignoring error: {error} while applying operations for "
+                        "transaction because we are either in initial "
+                        "sync or recovering mode - oplog entry: {oplogEntry}",
+                        "Encountered but ignoring error while applying operations for transaction "
+                        "because we are either in initial sync or recovering mode",
+                        "error"_attr = redact(ex),
+                        "oplogEntry"_attr = redact(op.toBSON()),
+                        "oplogApplicationMode"_attr =
+                            repl::OplogApplication::modeToString(oplogApplicationMode));
         }
     }
     return Status::OK();
@@ -249,14 +257,16 @@ Status applyAbortTransaction(OperationContext* opCtx,
     MONGO_UNREACHABLE;
 }
 
-repl::MultiApplier::Operations readTransactionOperationsFromOplogChain(
+std::pair<std::vector<OplogEntry>, bool> _readTransactionOperationsFromOplogChain(
     OperationContext* opCtx,
     const OplogEntry& lastEntryInTxn,
-    const std::vector<OplogEntry*>& cachedOps) noexcept {
+    const std::vector<OplogEntry*>& cachedOps,
+    const bool checkForCommands) noexcept {
+    bool isTransactionWithCommand = false;
     // Traverse the oplog chain with its own snapshot and read timestamp.
     ReadSourceScope readSourceScope(opCtx);
 
-    repl::MultiApplier::Operations ops;
+    std::vector<OplogEntry> ops;
 
     // The cachedOps are the ops for this transaction that are from the same oplog application batch
     // as the commit or prepare, those which have not necessarily been written to the oplog.  These
@@ -314,7 +324,35 @@ repl::MultiApplier::Operations readTransactionOperationsFromOplogChain(
 
     // Reconstruct the operations from the prepare or unprepared commit oplog entry.
     repl::ApplyOps::extractOperationsTo(prepareOrUnpreparedCommit, lastEntryInTxnObj, &ops);
-    return ops;
+
+    // It is safe to assume that any commands inside `ops` are real commands to be applied, as
+    // opposed to auxiliary commands such as "commit" and "abort".
+    if (checkForCommands) {
+        for (auto&& op : ops) {
+            if (op.isCommand()) {
+                isTransactionWithCommand = true;
+                break;
+            }
+        }
+    }
+    return std::make_pair(ops, isTransactionWithCommand);
+}
+
+std::vector<OplogEntry> readTransactionOperationsFromOplogChain(
+    OperationContext* opCtx,
+    const OplogEntry& lastEntryInTxn,
+    const std::vector<OplogEntry*>& cachedOps) noexcept {
+    auto result = _readTransactionOperationsFromOplogChain(
+        opCtx, lastEntryInTxn, cachedOps, false /*checkForCommands*/);
+    return std::get<0>(result);
+}
+
+std::pair<std::vector<OplogEntry>, bool> readTransactionOperationsFromOplogChainAndCheckForCommands(
+    OperationContext* opCtx,
+    const OplogEntry& lastEntryInTxn,
+    const std::vector<OplogEntry*>& cachedOps) noexcept {
+    return _readTransactionOperationsFromOplogChain(
+        opCtx, lastEntryInTxn, cachedOps, true /*checkForCommands*/);
 }
 
 namespace {
@@ -345,14 +383,26 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
     // This will prevent hybrid index builds from corrupting an index on secondary nodes if a
     // prepared transaction becomes prepared during a build but commits after the index build
     // commits.
+    // When two-phase index builds are in use, this is both unnecessary and unsafe. Due to locking,
+    // we can guarantee that a transaction prepared on a primary during an index build will always
+    // commit before that index build completes. Because two-phase index builds replicate start and
+    // commit oplog entries, it will never be possible to replicate a prepared transaction, commit
+    // an index build, then commit the transaction, the bug described above.
+    // This blocking behavior can also introduce a deadlock with two-phase index builds on
+    // a secondary if a prepared transaction blocks on an index build, but the index build can't
+    // re-acquire its X lock because of the transaction.
     for (const auto& op : ops) {
+        auto indexBuildsCoord = IndexBuildsCoordinator::get(opCtx);
         auto ns = op.getNss();
         auto uuid = *op.getUuid();
-        if (BackgroundOperation::inProgForNs(ns)) {
-            warning() << "blocking replication until index builds are finished on "
-                      << redact(ns.toString()) << ", due to prepared transaction";
-            BackgroundOperation::awaitNoBgOpInProgForNs(ns);
-            IndexBuildsCoordinator::get(opCtx)->awaitNoIndexBuildInProgressForCollection(uuid);
+        if (indexBuildsCoord->inProgForCollection(uuid, IndexBuildProtocol::kSinglePhase)) {
+            LOGV2_WARNING(21849,
+                          "Blocking replication until single-phase index builds are finished on "
+                          "collection, due to prepared transaction",
+                          "namespace"_attr = redact(ns.toString()),
+                          "uuid"_attr = uuid);
+            indexBuildsCoord->awaitNoIndexBuildInProgressForCollection(
+                opCtx, uuid, IndexBuildProtocol::kSinglePhase);
         }
     }
 
@@ -367,15 +417,10 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
         // The write on transaction table may be applied concurrently, so refreshing state
         // from disk may read that write, causing starting a new transaction on an existing
         // txnNumber. Thus, we start a new transaction without refreshing state from disk.
-        MONGO_FAIL_POINT_PAUSE_WHILE_SET(hangBeforeSessionCheckOutForApplyPrepare);
         MongoDOperationContextSessionWithoutRefresh sessionCheckout(opCtx);
 
         auto txnParticipant = TransactionParticipant::get(opCtx);
 
-        // We reset the recovery unit on retries, so make sure that we ignore prepare conflicts.
-        opCtx->recoveryUnit()->setPrepareConflictBehavior(
-            PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
-
         // Release the WUOW, transaction lock resources and abort storage transaction so that the
         // writeConflictRetry loop will be able to retry applying transactional ops on WCE error.
         auto abortOnError = makeGuard([&txnParticipant, opCtx] {
@@ -389,28 +434,27 @@ Status _applyPrepareTransaction(OperationContext* opCtx,
 
         // Set this in case the application of any ops need to use the prepare timestamp of this
         // transaction. It should be cleared automatically when the transaction finishes.
-        if (mode == repl::OplogApplication::Mode::kRecovering) {
+        if (mode == repl::OplogApplication::Mode::kRecovering ||
+            mode == repl::OplogApplication::Mode::kInitialSync) {
             txnParticipant.setPrepareOpTimeForRecovery(opCtx, entry.getOpTime());
         }
 
         auto status = _applyOperationsForTransaction(opCtx, ops, mode);
 
-        if (MONGO_FAIL_POINT(applyPrepareTxnOpsFailsWithWriteConflict)) {
-            LOG(0) << "Hit applyPrepareTxnOpsFailsWithWriteConflict failpoint";
+        if (MONGO_unlikely(applyPrepareTxnOpsFailsWithWriteConflict.shouldFail())) {
+            LOGV2(4947101, "Hit applyPrepareTxnOpsFailsWithWriteConflict failpoint");
             status = Status(ErrorCodes::WriteConflict,
                             "Prepare transaction apply ops failed due to write conflict");
         }
 
-
         if (status == ErrorCodes::WriteConflict) {
             throw WriteConflictException();
         }
         fassert(31137, status);
 
-        if (MONGO_FAIL_POINT(applyOpsHangBeforePreparingTransaction)) {
-            LOG(0) << "Hit applyOpsHangBeforePreparingTransaction failpoint";
-            MONGO_FAIL_POINT_PAUSE_WHILE_SET_OR_INTERRUPTED(opCtx,
-                                                            applyOpsHangBeforePreparingTransaction);
+        if (MONGO_unlikely(applyOpsHangBeforePreparingTransaction.shouldFail())) {
+            LOGV2(21847, "Hit applyOpsHangBeforePreparingTransaction failpoint");
+            applyOpsHangBeforePreparingTransaction.pauseWhileSet(opCtx);
         }
 
         txnParticipant.prepareTransaction(opCtx, entry.getOpTime());
@@ -461,10 +505,11 @@ Status applyPrepareTransaction(OperationContext* opCtx,
     switch (mode) {
         case repl::OplogApplication::Mode::kRecovering: {
             if (!serverGlobalParams.enableMajorityReadConcern) {
-                error()
-                    << "Cannot replay a prepared transaction when 'enableMajorityReadConcern' is "
-                       "set to false. Restart the server with --enableMajorityReadConcern=true "
-                       "to complete recovery.";
+                LOGV2_ERROR(
+                    21850,
+                    "Cannot replay a prepared transaction when 'enableMajorityReadConcern' is "
+                    "set to false. Restart the server with --enableMajorityReadConcern=true "
+                    "to complete recovery");
                 fassertFailed(51146);
             }
 
@@ -490,8 +535,8 @@ Status applyPrepareTransaction(OperationContext* opCtx,
 }
 
 void reconstructPreparedTransactions(OperationContext* opCtx, repl::OplogApplication::Mode mode) {
-    if (MONGO_FAIL_POINT(skipReconstructPreparedTransactions)) {
-        log() << "Hit skipReconstructPreparedTransactions failpoint";
+    if (MONGO_unlikely(skipReconstructPreparedTransactions.shouldFail())) {
+        LOGV2(21848, "Hit skipReconstructPreparedTransactions failpoint");
         return;
     }
     // Read the transactions table and the oplog collection without a timestamp.
@@ -527,6 +572,7 @@ void reconstructPreparedTransactions(OperationContext* opCtx, repl::OplogApplica
                 opCtx->getServiceContext()->makeClient("reconstruct-prepared-transactions");
             AlternativeClientRegion acr(newClient);
             const auto newOpCtx = cc().makeOperationContext();
+
             _reconstructPreparedTransaction(newOpCtx.get(), prepareOplogEntry, mode);
         }
     }
