@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/db/s/config/sharding_catalog_manager.h"
 
@@ -39,54 +39,77 @@
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/write_concern.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_database.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/client/shard.h"
 #include "mongo/s/database_version_helpers.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/shard_id.h"
-#include "mongo/util/log.h"
+#include "mongo/s/shard_util.h"
 
 namespace mongo {
-
-using std::string;
-using std::vector;
-
 namespace {
 
 const ReadPreferenceSetting kConfigReadSelector(ReadPreference::Nearest, TagSet{});
 
+/**
+ * Selects an optimal shard on which to place a newly created database from the set of available
+ * shards. Will return ShardNotFound if shard could not be found.
+ */
+ShardId selectShardForNewDatabase(OperationContext* opCtx, ShardRegistry* shardRegistry) {
+    std::vector<ShardId> allShardIds;
+
+    // Ensure the shard registry contains the most up-to-date list of available shards
+    shardRegistry->reload(opCtx);
+    shardRegistry->getAllShardIds(opCtx, &allShardIds);
+    uassert(ErrorCodes::ShardNotFound, "No shards found", !allShardIds.empty());
+
+    ShardId candidateShardId = allShardIds[0];
+
+    auto candidateSize =
+        uassertStatusOK(shardutil::retrieveTotalShardSize(opCtx, candidateShardId));
+
+    for (size_t i = 1; i < allShardIds.size(); i++) {
+        const ShardId shardId = allShardIds[i];
+
+        const auto currentSize = uassertStatusOK(shardutil::retrieveTotalShardSize(opCtx, shardId));
+
+        if (currentSize < candidateSize) {
+            candidateSize = currentSize;
+            candidateShardId = shardId;
+        }
+    }
+
+    return candidateShardId;
+}
+
 }  // namespace
 
 DatabaseType ShardingCatalogManager::createDatabase(OperationContext* opCtx,
                                                     StringData dbName,
-                                                    ShardId primaryShard) {
+                                                    const ShardId& primaryShard) {
     invariant(nsIsDbOnly(dbName));
 
-    // The admin, local and config databases should never be explicitly created, in any casing. They
-    // "just exist", i.e. getDatabase will always return an entry for them.
-    uassert(ErrorCodes::InvalidOptions,
-            str::stream() << "cannot manually create database '" << dbName << "'",
-            !dbName.equalCaseInsensitive(NamespaceString::kAdminDb) &&
-                !dbName.equalCaseInsensitive(NamespaceString::kLocalDb) &&
-                !dbName.equalCaseInsensitive(NamespaceString::kConfigDb));
+    // The admin and config databases should never be explicitly created. They "just exist",
+    // i.e. getDatabase will always return an entry for them.
+    if (dbName == NamespaceString::kAdminDb || dbName == NamespaceString::kConfigDb) {
+        uasserted(ErrorCodes::InvalidOptions,
+                  str::stream() << "cannot manually create database '" << dbName << "'");
+    }
 
-    const auto shardPtr = !primaryShard.isValid()
-        ? nullptr
-        : uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, primaryShard));
-    uassert(ErrorCodes::InvalidOptions,
-            str::stream() << "Invalid shard name: " << primaryShard.toString(),
-            (primaryShard.isValid() && shardPtr != nullptr) ||
-                (!primaryShard.isValid() && shardPtr == nullptr));
+    const auto catalogClient = Grid::get(opCtx)->catalogClient();
+    const auto shardRegistry = Grid::get(opCtx)->shardRegistry();
 
     // Check if a database already exists with the same name (case sensitive), and if so, return the
     // existing entry.
 
     BSONObjBuilder queryBuilder;
-    queryBuilder.appendRegex(
-        DatabaseType::name(), (string) "^" + pcrecpp::RE::QuoteMeta(dbName.toString()) + "$", "i");
+    queryBuilder.appendRegex(DatabaseType::name(),
+                             (std::string) "^" + pcrecpp::RE::QuoteMeta(dbName.toString()) + "$",
+                             "i");
 
-    auto docs = uassertStatusOK(Grid::get(opCtx)->catalogClient()->_exhaustiveFindOnConfig(
+    auto docs = uassertStatusOK(catalogClient->_exhaustiveFindOnConfig(
                                     opCtx,
                                     ReadPreferenceSetting{ReadPreference::PrimaryOnly},
                                     repl::ReadConcernLevel::kLocalReadConcern,
@@ -96,57 +119,101 @@ DatabaseType ShardingCatalogManager::createDatabase(OperationContext* opCtx,
                                     1))
                     .value;
 
-    if (!docs.empty()) {
-        auto actualDb = uassertStatusOK(DatabaseType::fromBSON(docs.front()));
+    auto const [primaryShardPtr, database] = [&] {
+        if (!docs.empty()) {
+            auto actualDb = uassertStatusOK(DatabaseType::fromBSON(docs.front()));
 
-        uassert(ErrorCodes::DatabaseDifferCase,
-                str::stream() << "can't have 2 databases that just differ on case "
-                              << " have: " << actualDb.getName()
-                              << " want to add: " << dbName.toString(),
-                actualDb.getName() == dbName.toString());
+            uassert(ErrorCodes::DatabaseDifferCase,
+                    str::stream() << "can't have 2 databases that just differ on case "
+                                  << " have: " << actualDb.getName()
+                                  << " want to add: " << dbName.toString(),
+                    actualDb.getName() == dbName.toString());
 
-        uassert(ErrorCodes::NamespaceExists,
+            uassert(
+                ErrorCodes::NamespaceExists,
                 str::stream() << "database already created on a primary which is different from: "
                               << primaryShard,
                 !primaryShard.isValid() || actualDb.getPrimary() == primaryShard);
 
-        // We did a local read of the database entry above and found that the database already
-        // exists. However, the data may not be majority committed (a previous createDatabase
-        // attempt may have failed with a writeConcern error).
-        // Since the current Client doesn't know the opTime of the last write to the database entry,
-        // make it wait for the last opTime in the system when we wait for writeConcern.
-        repl::ReplClientInfo::forClient(opCtx->getClient()).setLastOpToSystemLastOpTime(opCtx);
-        return actualDb;
+            // We did a local read of the database entry above and found that the database already
+            // exists. However, the data may not be majority committed (a previous createDatabase
+            // attempt may have failed with a writeConcern error).
+            // Since the current Client doesn't know the opTime of the last write to the database
+            // entry, make it wait for the last opTime in the system when we wait for writeConcern.
+            auto& replClient = repl::ReplClientInfo::forClient(opCtx->getClient());
+            replClient.setLastOpToSystemLastOpTime(opCtx);
+
+            WriteConcernResult unusedResult;
+            uassertStatusOK(waitForWriteConcern(opCtx,
+                                                replClient.getLastOp(),
+                                                ShardingCatalogClient::kMajorityWriteConcern,
+                                                &unusedResult));
+            return std::make_pair(
+                uassertStatusOK(shardRegistry->getShard(opCtx, actualDb.getPrimary())), actualDb);
+        } else {
+            // The database does not exist. Insert an entry for the new database into the sharding
+            // catalog.
+            auto const shardPtr = uassertStatusOK(shardRegistry->getShard(
+                opCtx,
+                primaryShard.isValid() ? primaryShard
+                                       : selectShardForNewDatabase(opCtx, shardRegistry)));
+
+            // Pick a primary shard for the new database.
+            DatabaseType db(
+                dbName.toString(), shardPtr->getId(), false, databaseVersion::makeNew());
+
+            LOGV2(21938,
+                  "Registering new database {db} in sharding catalog",
+                  "Registering new database in sharding catalog",
+                  "db"_attr = db);
+
+            // Do this write with majority writeConcern to guarantee that the shard sees the write
+            // when it receives the _flushDatabaseCacheUpdates.
+            uassertStatusOK(
+                catalogClient->insertConfigDocument(opCtx,
+                                                    DatabaseType::ConfigNS,
+                                                    db.toBSON(),
+                                                    ShardingCatalogClient::kMajorityWriteConcern));
+
+            return std::make_pair(shardPtr, db);
+        }
+    }();
+
+    // Note, making the primary shard refresh its databaseVersion here is not required for
+    // correctness, since either:
+    // 1) This is the first time this database is being created. The primary shard will not have a
+    //    databaseVersion already cached.
+    // 2) The database was dropped and is being re-created. Since dropping a database also sends
+    //    _flushDatabaseCacheUpdates to all shards, the primary shard should not have a database
+    //    version cached. (Note, it is possible that dropping a database will skip sending
+    //    _flushDatabaseCacheUpdates if the config server fails over while dropping the database.)
+    // However, routers don't support retrying internally on StaleDbVersion in transactions
+    // (SERVER-39704), so if the first operation run against the database is in a transaction, it
+    // would fail with StaleDbVersion. Making the primary shard refresh here allows that first
+    // transaction to succeed. This allows our transaction passthrough suites and transaction demos
+    // to succeed without additional special logic.
+    auto cmdResponse = uassertStatusOK(primaryShardPtr->runCommandWithFixedRetryAttempts(
+        opCtx,
+        ReadPreferenceSetting{ReadPreference::PrimaryOnly},
+        "admin",
+        BSON("_flushDatabaseCacheUpdates" << dbName),
+        Shard::RetryPolicy::kIdempotent));
+
+    // If the shard had binary version v4.2 when it received the _flushDatabaseCacheUpdates, it will
+    // have responded with NamespaceNotFound, because the shard does not have the database (see
+    // SERVER-34431). Ignore this error, since the _flushDatabaseCacheUpdates is only a nicety for
+    // users testing transactions, and the transaction passthrough suites do not change shard binary
+    // versions.
+    if (cmdResponse.commandStatus != ErrorCodes::NamespaceNotFound) {
+        uassertStatusOK(cmdResponse.commandStatus);
     }
 
-    // The database does not exist. Insert an entry for the new database into the sharding catalog.
-
-    // Pick a primary shard for the new database.
-    const ShardId primaryShardId = shardPtr == nullptr
-        ? uassertStatusOK(_selectShardForNewDatabase(opCtx, Grid::get(opCtx)->shardRegistry()))
-        : shardPtr->getId();
-
-    // Insert an entry for the new database into the sharding catalog.
-    DatabaseType db(
-        dbName.toString(), std::move(primaryShardId), false, databaseVersion::makeNew());
-
-    log() << "Registering new database " << db << " in sharding catalog";
-
-    uassertStatusOK(Grid::get(opCtx)->catalogClient()->insertConfigDocument(
-        opCtx, DatabaseType::ConfigNS, db.toBSON(), ShardingCatalogClient::kLocalWriteConcern));
-
-    return db;
+    return database;
 }
 
 void ShardingCatalogManager::enableSharding(OperationContext* opCtx,
                                             StringData dbName,
-                                            ShardId primaryShard) {
-    invariant(nsIsDbOnly(dbName));
-
-    uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "Enabling sharding on the admin database is not allowed",
-            dbName != NamespaceString::kAdminDb);
-
+                                            const ShardId& primaryShard) {
     // Sharding is enabled automatically on the config db.
     if (dbName == NamespaceString::kConfigDb) {
         return;
@@ -169,7 +236,11 @@ void ShardingCatalogManager::enableSharding(OperationContext* opCtx,
                                                 Milliseconds{30000}),
                             &unusedResult));
 
-    log() << "Enabling sharding for database [" << dbName << "] in config db";
+    LOGV2(21939,
+          "Persisted sharding enabled for database {db}",
+          "Persisted sharding enabled for database",
+          "db"_attr = dbName);
+
     uassertStatusOK(Grid::get(opCtx)->catalogClient()->updateConfigDocument(
         opCtx,
         DatabaseType::ConfigNS,
@@ -257,8 +328,11 @@ Status ShardingCatalogManager::commitMovePrimary(OperationContext* opCtx,
         ShardingCatalogClient::kLocalWriteConcern);
 
     if (!updateStatus.isOK()) {
-        log() << "error committing movePrimary: " << dbname
-              << causedBy(redact(updateStatus.getStatus()));
+        LOGV2(21940,
+              "Error committing movePrimary for {db}: {error}",
+              "Error committing movePrimary",
+              "db"_attr = dbname,
+              "error"_attr = redact(updateStatus.getStatus()));
         return updateStatus.getStatus();
     }
 
