@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -53,13 +53,13 @@
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/repl_set_config.h"
-#include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/add_shard_cmd_gen.h"
 #include "mongo/db/s/add_shard_util.h"
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/type_shard_identity.h"
 #include "mongo/db/wire_version.h"
 #include "mongo/executor/task_executor.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/s/catalog/config_server_version.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
@@ -71,11 +71,9 @@
 #include "mongo/s/cluster_identity_loader.h"
 #include "mongo/s/database_version_helpers.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/shard_util.h"
 #include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
 
@@ -163,7 +161,10 @@ StatusWith<Shard::CommandResponse> ShardingCatalogManager::_runCommandForAddShar
     _executorForAddShard->wait(swCallbackHandle.getValue());
 
     if (response.status == ErrorCodes::ExceededTimeLimit) {
-        LOG(0) << "Operation timed out with status " << redact(response.status);
+        LOGV2(21941,
+              "Operation timed out with {error}",
+              "Operation timed out",
+              "error"_attr = redact(response.status));
     }
 
     if (!response.isOK()) {
@@ -338,14 +339,14 @@ StatusWith<ShardType> ShardingCatalogManager::_validateHostAsShard(
                                                 << connectionString.toString() << " as a shard");
     }
     if (serverGlobalParams.featureCompatibility.getVersion() >
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyDowngradedTo40) {
-        // If the cluster's FCV is 4.2, or upgrading to / downgrading from, the node being added
-        // must be a v4.2 binary.
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyDowngradedTo42) {
+        // If the cluster's FCV is 4.4, or upgrading to / downgrading from, the node being added
+        // must be a v4.4 binary.
         invariant(maxWireVersion == WireVersion::LATEST_WIRE_VERSION);
     } else {
-        // If the cluster's FCV is 4.0, the node being added must be a v4.0 or v4.2 binary.
+        // If the cluster's FCV is 4.2, the node being added must be a v4.2 or v4.4 binary.
         invariant(serverGlobalParams.featureCompatibility.getVersion() ==
-                  ServerGlobalParams::FeatureCompatibility::Version::kFullyDowngradedTo40);
+                  ServerGlobalParams::FeatureCompatibility::Version::kFullyDowngradedTo42);
         invariant(maxWireVersion >= WireVersion::LATEST_WIRE_VERSION - 1);
     }
 
@@ -359,7 +360,7 @@ StatusWith<ShardType> ShardingCatalogManager::_validateHostAsShard(
                                                 << connectionString.toString() << " as a shard");
     }
     if (!isMaster) {
-        return {ErrorCodes::NotWritablePrimary,
+        return {ErrorCodes::NotMaster,
                 str::stream()
                     << connectionString.toString()
                     << " does not have a master. If this is a replica set, ensure that it has a"
@@ -549,23 +550,8 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
         return existingShard.getValue()->getName();
     }
 
-    // Force a reload of the ShardRegistry to ensure that, in case this addShard is to re-add a
-    // replica set that has recently been removed, we have detached the ReplicaSetMonitor for the
-    // set with that setName from the ReplicaSetMonitorManager and will create a new
-    // ReplicaSetMonitor when targeting the set below.
-    // Note: This is necessary because as of 3.4, removeShard is performed by mongos (unlike
-    // addShard), so the ShardRegistry is not synchronously reloaded on the config server when a
-    // shard is removed.
-    if (!Grid::get(opCtx)->shardRegistry()->reload(opCtx)) {
-        // If the first reload joined an existing one, call reload again to ensure the reload is
-        // fresh.
-        Grid::get(opCtx)->shardRegistry()->reload(opCtx);
-    }
-
-    // TODO: Don't create a detached Shard object, create a detached RemoteCommandTargeter instead.
     const std::shared_ptr<Shard> shard{
         Grid::get(opCtx)->shardRegistry()->createConnection(shardConnectionString)};
-    invariant(shard);
     auto targeter = shard->getTargeter();
 
     auto stopMonitoringGuard = makeGuard([&] {
@@ -646,26 +632,9 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
 
     AddShard addShardCmd = add_shard_util::createAddShardCmd(opCtx, shardType.getName());
 
-    auto addShardCmdBSON = [&]() {
-        // In 4.2, use the _addShard command to add the shard, which in turn inserts a
-        // shardIdentity document into the shard and triggers sharding state initialization.
-        // In the unlikely scenario that there's a downgrade to 4.0 between the
-        // construction of this command object and the issuing of the command
-        // on the receiving shard, the user will receive a rather harmless
-        // CommandNotFound error for _addShard, and can simply retry.
-        if (serverGlobalParams.featureCompatibility.getVersion() ==
-            ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo42) {
-            // Needed for IDL toBSON method
-            BSONObj passthroughFields;
-            return addShardCmd.toBSON(passthroughFields);
-        } else {
-            // To support backwards compatibility with v4.0 shards, insert a shardIdentity document
-            // directly.
-            return add_shard_util::createShardIdentityUpsertForAddShard(addShardCmd);
-        }
-    }();
-
-    auto addShardStatus = runCmdOnNewShard(addShardCmdBSON);
+    // Use the _addShard command to add the shard, which in turn inserts a shardIdentity document
+    // into the shard and triggers sharding state initialization.
+    auto addShardStatus = runCmdOnNewShard(addShardCmd.toBSON({}));
 
     if (!addShardStatus.isOK()) {
         return addShardStatus;
@@ -682,16 +651,16 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
 
         BSONObj setFCVCmd;
         switch (serverGlobalParams.featureCompatibility.getVersion()) {
-            case ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo42:
-            case ServerGlobalParams::FeatureCompatibility::Version::kUpgradingTo42:
+            case ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44:
+            case ServerGlobalParams::FeatureCompatibility::Version::kUpgradingTo44:
                 setFCVCmd = BSON(FeatureCompatibilityVersionCommandParser::kCommandName
-                                 << FeatureCompatibilityVersionParser::kVersion42
+                                 << FeatureCompatibilityVersionParser::kVersion44
                                  << WriteConcernOptions::kWriteConcernField
                                  << opCtx->getWriteConcern().toBSON());
                 break;
             default:
                 setFCVCmd = BSON(FeatureCompatibilityVersionCommandParser::kCommandName
-                                 << FeatureCompatibilityVersionParser::kVersion40
+                                 << FeatureCompatibilityVersionParser::kVersion42
                                  << WriteConcernOptions::kWriteConcernField
                                  << opCtx->getWriteConcern().toBSON());
                 break;
@@ -706,7 +675,10 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
             return versionResponse.getValue().commandStatus;
         }
 
-        log() << "going to insert new entry for shard into config.shards: " << shardType.toString();
+        LOGV2(21942,
+              "Going to insert new entry for shard into config.shards: {shardType}",
+              "Going to insert new entry for shard into config.shards",
+              "shardType"_attr = shardType.toString());
 
         Status result = Grid::get(opCtx)->catalogClient()->insertConfigDocument(
             opCtx,
@@ -714,7 +686,11 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
             shardType.toBSON(),
             ShardingCatalogClient::kLocalWriteConcern);
         if (!result.isOK()) {
-            log() << "error adding shard: " << shardType.toBSON() << " err: " << result.reason();
+            LOGV2(21943,
+                  "Error adding shard: {shardType} err: {error}",
+                  "Error adding shard",
+                  "shardType"_attr = shardType.toBSON(),
+                  "error"_attr = result.reason());
             return result;
         }
     }
@@ -732,8 +708,11 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
                 true,
                 ShardingCatalogClient::kLocalWriteConcern);
             if (!status.isOK()) {
-                log() << "adding shard " << shardConnectionString.toString()
-                      << " even though could not add database " << dbName;
+                LOGV2(21944,
+                      "Adding shard {connectionString} even though could not add database {db}",
+                      "Adding shard even though we could not add database",
+                      "connectionString"_attr = shardConnectionString.toString(),
+                      "db"_attr = dbName);
             }
         }
     }
@@ -748,6 +727,10 @@ StatusWith<std::string> ShardingCatalogManager::addShard(
 
     // Ensure the added shard is visible to this process.
     auto shardRegistry = Grid::get(opCtx)->shardRegistry();
+    if (!shardRegistry->reload(opCtx)) {
+        shardRegistry->reload(opCtx);
+    }
+
     if (!shardRegistry->getShard(opCtx, shardType.getName()).isOK()) {
         return {ErrorCodes::OperationFailed,
                 "Could not find shard metadata for shard after adding it. This most likely "
@@ -762,6 +745,23 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
                                                         const ShardId& shardId) {
     const auto name = shardId.toString();
 
+    const auto configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
+
+    Lock::ExclusiveLock shardLock(opCtx->lockState(), _kShardMembershipLock);
+
+    auto findShardResponse = uassertStatusOK(
+        configShard->exhaustiveFindOnConfig(opCtx,
+                                            kConfigReadSelector,
+                                            repl::ReadConcernLevel::kLocalReadConcern,
+                                            ShardType::ConfigNS,
+                                            BSON(ShardType::name() << name),
+                                            BSONObj(),
+                                            1));
+    uassert(ErrorCodes::ShardNotFound,
+            str::stream() << "Shard " << shardId << " does not exist",
+            !findShardResponse.docs.empty());
+    const auto shard = uassertStatusOK(ShardType::fromBSON(findShardResponse.docs[0]));
+
     // Find how many *other* shards exist, which are *not* currently draining
     const auto countOtherNotDrainingShards = uassertStatusOK(_runCountCommandOnConfig(
         opCtx,
@@ -771,6 +771,17 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
             "Operation not allowed because it would remove the last shard",
             countOtherNotDrainingShards > 0);
 
+    // Ensure there are no non-empty zones that only belong to this shard
+    for (auto& zoneName : shard.getTags()) {
+        auto isRequiredByZone = uassertStatusOK(
+            _isShardRequiredByZoneStillInUse(opCtx, kConfigReadSelector, name, zoneName));
+        uassert(ErrorCodes::ZoneStillInUse,
+                str::stream()
+                    << "Operation not allowed because it would remove the only shard for zone "
+                    << zoneName << " which has a chunk range is associated with it",
+                !isRequiredByZone);
+    }
+
     // Figure out if shard is already draining
     const bool isShardCurrentlyDraining =
         uassertStatusOK(_runCountCommandOnConfig(
@@ -778,11 +789,13 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
             ShardType::ConfigNS,
             BSON(ShardType::name() << name << ShardType::draining(true)))) > 0;
 
-    auto* const shardRegistry = Grid::get(opCtx)->shardRegistry();
     auto* const catalogClient = Grid::get(opCtx)->catalogClient();
 
     if (!isShardCurrentlyDraining) {
-        log() << "going to start draining shard: " << name;
+        LOGV2(21945,
+              "Going to start draining shard: {shardId}",
+              "Going to start draining shard",
+              "shardId"_attr = name);
 
         // Record start in changelog
         uassertStatusOK(ShardingLogging::get(opCtx)->logChangeChecked(
@@ -801,12 +814,12 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
                                                 ShardingCatalogClient::kLocalWriteConcern),
             "error starting removeShard");
 
-        shardRegistry->reload(opCtx);
-
         return {RemoveShardProgress::STARTED,
                 boost::optional<RemoveShardProgress::DrainingShardUsage>(boost::none)};
     }
 
+    shardLock.unlock();
+
     // Draining has already started, now figure out how many chunks and databases are still on the
     // shard.
     const auto chunkCount = uassertStatusOK(
@@ -820,9 +833,13 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
 
     if (chunkCount > 0 || databaseCount > 0) {
         // Still more draining to do
-        LOG(0) << "chunkCount: " << chunkCount;
-        LOG(0) << "databaseCount: " << databaseCount;
-        LOG(0) << "jumboCount: " << jumboCount;
+        LOGV2(21946,
+              "removeShard: draining chunkCount {chunkCount}; databaseCount {databaseCount}; "
+              "jumboCount {jumboCount}",
+              "removeShard: draining",
+              "chunkCount"_attr = chunkCount,
+              "databaseCount"_attr = databaseCount,
+              "jumboCount"_attr = jumboCount);
 
         return {RemoveShardProgress::ONGOING,
                 boost::optional<RemoveShardProgress::DrainingShardUsage>(
@@ -830,7 +847,8 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
     }
 
     // Draining is done, now finish removing the shard.
-    log() << "going to remove shard: " << name;
+    LOGV2(
+        21949, "Going to remove shard: {shardId}", "Going to remove shard", "shardId"_attr = name);
     audit::logRemoveShard(opCtx->getClient(), name);
 
     uassertStatusOKWithContext(
@@ -840,11 +858,17 @@ RemoveShardProgress ShardingCatalogManager::removeShard(OperationContext* opCtx,
                                              ShardingCatalogClient::kLocalWriteConcern),
         str::stream() << "error completing removeShard operation on: " << name);
 
+    // The shard which was just removed must be reflected in the shard registry, before the replica
+    // set monitor is removed, otherwise the shard would be referencing a dropped RSM. Check that
+    // this reload did not join an already running reload to be sure it picks up that the shard was
+    // removed.
+    if (!Grid::get(opCtx)->shardRegistry()->reload(opCtx)) {
+        Grid::get(opCtx)->shardRegistry()->reload(opCtx);
+    }
+
     shardConnectionPool.removeHost(name);
     ReplicaSetMonitor::remove(name);
 
-    shardRegistry->reload(opCtx);
-
     // Record finish in changelog
     ShardingLogging::get(opCtx)->logChange(
         opCtx, "removeShard", "", BSON("shard" << name), ShardingCatalogClient::kLocalWriteConcern);
@@ -857,40 +881,6 @@ void ShardingCatalogManager::appendConnectionStats(executor::ConnectionPoolStats
     _executorForAddShard->appendConnectionStats(stats);
 }
 
-// static
-StatusWith<ShardId> ShardingCatalogManager::_selectShardForNewDatabase(
-    OperationContext* opCtx, ShardRegistry* shardRegistry) {
-    std::vector<ShardId> allShardIds;
-
-    shardRegistry->getAllShardIds(opCtx, &allShardIds);
-    if (allShardIds.empty()) {
-        return Status(ErrorCodes::ShardNotFound, "No shards found");
-    }
-
-    ShardId candidateShardId = allShardIds[0];
-
-    auto candidateSizeStatus = shardutil::retrieveTotalShardSize(opCtx, candidateShardId);
-    if (!candidateSizeStatus.isOK()) {
-        return candidateSizeStatus.getStatus();
-    }
-
-    for (size_t i = 1; i < allShardIds.size(); i++) {
-        const ShardId shardId = allShardIds[i];
-
-        const auto sizeStatus = shardutil::retrieveTotalShardSize(opCtx, shardId);
-        if (!sizeStatus.isOK()) {
-            return sizeStatus.getStatus();
-        }
-
-        if (sizeStatus.getValue() < candidateSizeStatus.getValue()) {
-            candidateSizeStatus = sizeStatus;
-            candidateShardId = shardId;
-        }
-    }
-
-    return candidateShardId;
-}
-
 StatusWith<long long> ShardingCatalogManager::_runCountCommandOnConfig(OperationContext* opCtx,
                                                                        const NamespaceString& nss,
                                                                        BSONObj query) {
