@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -33,6 +32,7 @@
 #include "mongo/client/remote_command_targeter_mock.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/dbdirectclient.h"
+#include "mongo/db/logical_session_id_helpers.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/s/migration_chunk_cloner_source_legacy.h"
 #include "mongo/s/catalog/sharding_catalog_client_mock.h"
@@ -95,13 +95,15 @@ protected:
                 ->setFindHostReturnValue(kRecipientConnStr.getServers()[0]);
         }
 
-        auto clockSource = stdx::make_unique<ClockSourceMock>();
+        auto clockSource = std::make_unique<ClockSourceMock>();
 
         // Timestamps of "0 seconds" are not allowed, so we must advance our clock mock to the first
         // real second.
         clockSource->advance(Seconds(1));
 
         operationContext()->getServiceContext()->setFastClockSource(std::move(clockSource));
+
+        _lsid = makeLogicalSessionId(operationContext());
     }
 
     void tearDown() override {
@@ -155,7 +157,8 @@ protected:
             chunkRange,
             1024 * 1024,
             MigrationSecondaryThrottleOptions::create(MigrationSecondaryThrottleOptions::kDefault),
-            false);
+            false,
+            MoveChunkRequest::ForceJumbo::kDoNotForce);
 
         return assertGet(MoveChunkRequest::createFromCommand(kNss, cmdBuilder.obj()));
     }
@@ -167,6 +170,10 @@ protected:
         return BSON("_id" << value << "X" << value);
     }
 
+protected:
+    LogicalSessionId _lsid;
+    TxnNumber _txnNumber{0};
+
 private:
     std::unique_ptr<ShardingCatalogClient> makeShardingCatalogClient(
         std::unique_ptr<DistLockManager> distLockManager) override {
@@ -189,7 +196,7 @@ private:
             }
         };
 
-        return stdx::make_unique<StaticCatalogClient>();
+        return std::make_unique<StaticCatalogClient>();
     }
 
     boost::optional<DBDirectClient> _client;
@@ -214,7 +221,7 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, CorrectDocumentsFetched) {
             onCommand([&](const RemoteCommandRequest& request) { return BSON("ok" << true); });
         });
 
-        ASSERT_OK(cloner.startClone(operationContext()));
+        ASSERT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
         futureStartClone.default_timed_get();
     }
 
@@ -312,7 +319,7 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, CollectionNotFound) {
         kDonorConnStr,
         kRecipientConnStr.getServers()[0]);
 
-    ASSERT_NOT_OK(cloner.startClone(operationContext()));
+    ASSERT_NOT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
     cloner.cancelClone(operationContext());
 }
 
@@ -325,7 +332,7 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, ShardKeyIndexNotFound) {
         kDonorConnStr,
         kRecipientConnStr.getServers()[0]);
 
-    ASSERT_NOT_OK(cloner.startClone(operationContext()));
+    ASSERT_NOT_OK(cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber));
     cloner.cancelClone(operationContext());
 }
 
@@ -351,10 +358,9 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, FailedToEngageRecipientShard) {
             });
         });
 
-        auto startCloneStatus = cloner.startClone(operationContext());
-        // Error is converted to OperationFailed as if it's propagated to the MigrationManager
-        // it will mark the migration source as having network problems.
-        ASSERT_EQ(ErrorCodes::OperationFailed, startCloneStatus.code());
+        auto startCloneStatus =
+            cloner.startClone(operationContext(), UUID::gen(), _lsid, _txnNumber);
+        ASSERT_EQ(ErrorCodes::NetworkTimeout, startCloneStatus.code());
         futureStartClone.default_timed_get();
     }
 
@@ -382,35 +388,5 @@ TEST_F(MigrationChunkClonerSourceLegacyTest, FailedToEngageRecipientShard) {
     cloner.cancelClone(operationContext());
 }
 
-TEST_F(MigrationChunkClonerSourceLegacyTest, RecipientShardFailsWithIsNotMaster) {
-    const std::vector<BSONObj> contents = {createCollectionDocument(99),
-                                           createCollectionDocument(100),
-                                           createCollectionDocument(199),
-                                           createCollectionDocument(200)};
-
-    createShardedCollection(contents);
-
-    MigrationChunkClonerSourceLegacy cloner(
-        createMoveChunkRequest(ChunkRange(BSON("X" << 100), BSON("X" << 200))),
-        kShardKeyPattern,
-        kDonorConnStr,
-        kRecipientConnStr.getServers()[0]);
-
-    {
-        auto futureStartClone = launchAsync([&]() {
-            onCommand([&](const RemoteCommandRequest& request) {
-                return Status(ErrorCodes::NotMaster, "Simulate failover at recipient");
-            });
-        });
-
-        auto startCloneStatus = cloner.startClone(operationContext());
-        // Error is converted to OperationFailed as if it's propagated to the MigrationManager
-        // it will mark the migration source as no longer primary.
-        ASSERT_EQ(ErrorCodes::OperationFailed, startCloneStatus.code());
-        futureStartClone.default_timed_get();
-    }
-    cloner.cancelClone(operationContext());
-}
-
 }  // namespace
 }  // namespace mongo
