@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -41,16 +41,19 @@
 #include "mongo/db/s/sharding_logging.h"
 #include "mongo/db/s/sharding_state_recovery.h"
 #include "mongo/db/s/sharding_statistics.h"
+#include "mongo/logv2/log.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/s/catalog/type_shard_database.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/grid.h"
 #include "mongo/util/exit.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
+MONGO_FAIL_POINT_DEFINE(hangInCloneStage);
+MONGO_FAIL_POINT_DEFINE(hangInCleanStaleDataStage);
+
 using namespace shardmetadatautil;
 
 MovePrimarySourceManager::MovePrimarySourceManager(OperationContext* opCtx,
@@ -66,7 +69,7 @@ MovePrimarySourceManager::MovePrimarySourceManager(OperationContext* opCtx,
 MovePrimarySourceManager::~MovePrimarySourceManager() {}
 
 NamespaceString MovePrimarySourceManager::getNss() const {
-    return _requestArgs.get_movePrimary();
+    return _requestArgs.get_shardsvrMovePrimary().get();
 }
 
 Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
@@ -74,7 +77,12 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
     invariant(_state == kCreated);
     auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
 
-    log() << "Moving " << _dbname << " primary from: " << _fromShard << " to: " << _toShard;
+    LOGV2(22042,
+          "Moving {db} primary from: {fromShard} to: {toShard}",
+          "Moving primary for database",
+          "db"_attr = _dbname,
+          "fromShard"_attr = _fromShard,
+          "toShard"_attr = _toShard);
 
     // Record start in changelog
     uassertStatusOK(ShardingLogging::get(opCtx)->logChangeChecked(
@@ -89,20 +97,25 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
         // data was inserted into the database.
         AutoGetOrCreateDb autoDb(opCtx, getNss().toString(), MODE_X);
 
-        auto& dss = DatabaseShardingState::get(autoDb.getDb());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
+        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
+        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
 
-        dss.setMovePrimarySourceManager(opCtx, this, dssLock);
+        dss->setMovePrimarySourceManager(opCtx, this, dssLock);
     }
 
     _state = kCloning;
 
+    if (MONGO_unlikely(hangInCloneStage.shouldFail())) {
+        LOGV2(4908700, "Hit hangInCloneStage");
+        hangInCloneStage.pauseWhileSet(opCtx);
+    }
+
     auto const shardRegistry = Grid::get(opCtx)->shardRegistry();
     auto fromShardObj = uassertStatusOK(shardRegistry->getShard(opCtx, _fromShard));
     auto toShardObj = uassertStatusOK(shardRegistry->getShard(opCtx, _toShard));
 
     BSONObjBuilder cloneCatalogDataCommandBuilder;
-    cloneCatalogDataCommandBuilder << "_cloneCatalogData" << _dbname << "from"
+    cloneCatalogDataCommandBuilder << "_shardsvrCloneCatalogData" << _dbname << "from"
                                    << fromShardObj->getConnString().toString();
 
 
@@ -115,6 +128,24 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
 
     auto cloneCommandStatus = Shard::CommandResponse::getEffectiveStatus(cloneCommandResponse);
 
+    // If the `toShard` is on v4.2 or earlier, it will not recognize the command name
+    // _shardsvrCloneCatalogData. We will retry the command with the old name _cloneCatalogData.
+    if (cloneCommandStatus == ErrorCodes::CommandNotFound) {
+        BSONObjBuilder legacyCloneCatalogDataCommandBuilder;
+        legacyCloneCatalogDataCommandBuilder << "_cloneCatalogData" << _dbname << "from"
+                                             << fromShardObj->getConnString().toString();
+
+
+        cloneCommandResponse = toShardObj->runCommandWithFixedRetryAttempts(
+            opCtx,
+            ReadPreferenceSetting(ReadPreference::PrimaryOnly),
+            "admin",
+            CommandHelpers::appendMajorityWriteConcern(legacyCloneCatalogDataCommandBuilder.obj()),
+            Shard::RetryPolicy::kIdempotent);
+
+        cloneCommandStatus = Shard::CommandResponse::getEffectiveStatus(cloneCommandResponse);
+    }
+
     uassertStatusOK(cloneCommandStatus);
 
     auto clonedCollsArray = cloneCommandResponse.getValue().response["clonedColls"];
@@ -149,11 +180,11 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
                                     << " was dropped during the movePrimary operation.");
         }
 
-        auto& dss = DatabaseShardingState::get(autoDb.getDb());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
+        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
+        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
 
         // IMPORTANT: After this line, the critical section is in place and needs to be signaled
-        dss.enterCriticalSectionCatchUpPhase(opCtx, dssLock);
+        dss->enterCriticalSectionCatchUpPhase(opCtx, dssLock);
     }
 
     _state = kCriticalSection;
@@ -177,9 +208,10 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
                           << signalStatus.toString()};
     }
 
-    log() << "movePrimary successfully entered critical section";
+    LOGV2(22043, "movePrimary successfully entered critical section");
 
     scopedGuard.dismiss();
+
     return Status::OK();
 }
 
@@ -201,12 +233,12 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
                                     << " was dropped during the movePrimary operation.");
         }
 
-        auto& dss = DatabaseShardingState::get(autoDb.getDb());
-        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
+        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
+        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
 
         // Read operations must begin to wait on the critical section just before we send the
         // commit operation to the config server
-        dss.enterCriticalSectionCommitPhase(opCtx, dssLock);
+        dss->enterCriticalSectionCommitPhase(opCtx, dssLock);
     }
 
     auto configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
@@ -226,9 +258,12 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
         // Need to get the latest optime in case the refresh request goes to a secondary --
         // otherwise the read won't wait for the write that _configsvrCommitMovePrimary may have
         // done
-        log() << "Error occurred while committing the movePrimary. Performing a majority write "
-                 "against the config server to obtain its latest optime"
-              << causedBy(redact(commitStatus));
+        LOGV2(22044,
+              "Error occurred while committing the movePrimary. Performing a majority write "
+              "against the config server to obtain its latest optime: {error}",
+              "Error occurred while committing the movePrimary. Performing a majority write "
+              "against the config server to obtain its latest optime",
+              "error"_attr = redact(commitStatus));
 
         Status validateStatus = ShardingLogging::get(opCtx)->logChangeChecked(
             opCtx,
@@ -262,10 +297,10 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
             }
 
             if (!repl::ReplicationCoordinator::get(opCtx)->canAcceptWritesFor(opCtx, getNss())) {
-                auto& dss = DatabaseShardingState::get(autoDb.getDb());
-                auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
+                auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
+                auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
 
-                dss.setDbVersion(opCtx, boost::none, dssLock);
+                dss->setDbVersion(opCtx, boost::none, dssLock);
                 uassertStatusOK(validateStatus.withContext(
                     str::stream() << "Unable to verify movePrimary commit for database: "
                                   << getNss().ns()
@@ -312,6 +347,11 @@ Status MovePrimarySourceManager::cleanStaleData(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kNeedCleanStaleData);
 
+    if (MONGO_unlikely(hangInCleanStaleDataStage.shouldFail())) {
+        LOGV2(4908701, "Hit hangInCleanStaleDataStage");
+        hangInCleanStaleDataStage.pauseWhileSet(opCtx);
+    }
+
     // Only drop the cloned (unsharded) collections.
     DBDirectClient client(opCtx);
     for (auto& coll : _clonedColls) {
@@ -319,7 +359,11 @@ Status MovePrimarySourceManager::cleanStaleData(OperationContext* opCtx) {
         client.runCommand(_dbname.toString(), BSON("drop" << coll.coll()), dropCollResult);
         Status dropStatus = getStatusFromCommandResult(dropCollResult);
         if (!dropStatus.isOK()) {
-            log() << "failed to drop cloned collection " << coll << causedBy(redact(dropStatus));
+            LOGV2(22045,
+                  "Failed to drop cloned collection {namespace} in movePrimary: {error}",
+                  "Failed to drop cloned collection in movePrimary",
+                  "namespace"_attr = coll,
+                  "error"_attr = redact(dropStatus));
         }
     }
 
@@ -342,11 +386,15 @@ void MovePrimarySourceManager::cleanupOnError(OperationContext* opCtx) {
 
     try {
         _cleanup(opCtx);
-    } catch (const ExceptionForCat<ErrorCategory::NotPrimaryError>& ex) {
+    } catch (const ExceptionForCat<ErrorCategory::NotMasterError>& ex) {
         BSONObjBuilder requestArgsBSON;
         _requestArgs.serialize(&requestArgsBSON);
-        warning() << "Failed to clean up movePrimary: " << redact(requestArgsBSON.obj())
-                  << "due to: " << redact(ex);
+        LOGV2_WARNING(22046,
+                      "Failed to clean up movePrimary with request parameters {request} due to: "
+                      "{error}",
+                      "Failed to clean up movePrimary",
+                      "request"_attr = redact(requestArgsBSON.obj()),
+                      "error"_attr = redact(ex));
     }
 }
 
@@ -358,23 +406,16 @@ void MovePrimarySourceManager::_cleanup(OperationContext* opCtx) {
         UninterruptibleLockGuard noInterrupt(opCtx->lockState());
         AutoGetDb autoDb(opCtx, getNss().toString(), MODE_IX);
 
-        if (autoDb.getDb()) {
-            auto& dss = DatabaseShardingState::get(autoDb.getDb());
-            auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
+        auto dss = DatabaseShardingState::get(opCtx, getNss().toString());
+        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
 
-            dss.clearMovePrimarySourceManager(opCtx, dssLock);
+        dss->clearMovePrimarySourceManager(opCtx, dssLock);
 
-            // Leave the critical section if we're still registered.
-            dss.exitCriticalSection(opCtx, boost::none, dssLock);
-        }
+        // Leave the critical section if we're still registered.
+        dss->exitCriticalSection(opCtx, boost::none, dssLock);
     }
 
     if (_state == kCriticalSection || _state == kCloneCompleted) {
-
-        // TODO SERVER-32608
-        // Wait for writes to 'config.cache.databases' to flush before removing the 'minOpTime
-        // 'recovery' document.
-
         // Clear the 'minOpTime recovery' document so that the next time a node from this shard
         // becomes a primary, it won't have to recover the config server optime.
         ShardingStateRecovery::endMetadataOp(opCtx);
