@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -47,10 +47,10 @@
 #include "mongo/db/s/sharded_connection_info.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/views/view_catalog.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/request_types/set_shard_version_request.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -159,33 +159,6 @@ public:
 
         // Step 3
 
-        // Validate shardName parameter.
-        const auto shardName = cmdObj["shard"].str();
-        const auto storedShardName = shardingState->shardId().toString();
-        uassert(ErrorCodes::BadValue,
-                str::stream() << "received shardName " << shardName
-                              << " which differs from stored shardName " << storedShardName,
-                storedShardName == shardName);
-
-        // Validate config connection string parameter.
-        const auto configdb = cmdObj["configdb"].String();
-        uassert(ErrorCodes::BadValue,
-                "Config server connection string cannot be empty",
-                !configdb.empty());
-
-        const auto givenConnStr = uassertStatusOK(ConnectionString::parse(configdb));
-        uassert(ErrorCodes::InvalidOptions,
-                str::stream() << "Given config server string " << givenConnStr.toString()
-                              << " is not of type SET",
-                givenConnStr.type() == ConnectionString::SET);
-
-        const auto storedConnStr =
-            Grid::get(opCtx)->shardRegistry()->getConfigServerConnectionString();
-        uassert(ErrorCodes::IllegalOperation,
-                str::stream() << "Given config server set name: " << givenConnStr.getSetName()
-                              << " differs from known set name: " << storedConnStr.getSetName(),
-                givenConnStr.getSetName() == storedConnStr.getSetName());
-
         // Validate namespace parameter.
         const NamespaceString nss(cmdObj["setShardVersion"].String());
         uassert(ErrorCodes::InvalidNamespace,
@@ -213,7 +186,7 @@ public:
             autoDb.emplace(opCtx, nss.db(), MODE_IS);
 
             // Slave nodes cannot support set shard version
-            uassert(ErrorCodes::NotWritablePrimary,
+            uassert(ErrorCodes::NotMaster,
                     str::stream() << "setShardVersion with collection version is only supported "
                                      "against primary nodes, but it was received for namespace "
                                   << nss.ns(),
@@ -225,7 +198,8 @@ public:
 
             // Views do not require a shard version check. We do not care about invalid system views
             // for this check, only to validate if a view already exists for this namespace.
-            if (autoDb->getDb() && !autoDb->getDb()->getCollection(opCtx, nss) &&
+            if (autoDb->getDb() &&
+                !CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss) &&
                 ViewCatalog::get(autoDb->getDb())
                     ->lookupWithoutValidatingDurableViews(opCtx, nss.ns())) {
                 return true;
@@ -252,7 +226,7 @@ public:
                         // A migration occurred
                         info->setVersion(nss.ns(), requestedVersion);
                     } else if (authoritative) {
-                        // The collection was dropped and recreated
+                        // The collection was dropped and recreated or had its shard key refined.
                         info->setVersion(nss.ns(), requestedVersion);
                     } else {
                         result.append("ns", nss.ns());
@@ -304,7 +278,7 @@ public:
                     if (critSecSignal) {
                         collLock.reset();
                         autoDb.reset();
-                        log() << "waiting till out of critical section";
+                        LOGV2(22056, "waiting till out of critical section");
                         critSecSignal->waitFor(opCtx, Seconds(10));
                     }
 
@@ -325,7 +299,7 @@ public:
                     if (critSecSignal) {
                         collLock.reset();
                         autoDb.reset();
-                        log() << "waiting till out of critical section";
+                        LOGV2(22057, "waiting till out of critical section");
                         critSecSignal->waitFor(opCtx, Seconds(10));
                     }
 
@@ -363,16 +337,19 @@ public:
 
             if (!status.isOK()) {
                 // The reload itself was interrupted or confused here
-
-                errmsg = str::stream()
-                    << "could not refresh metadata for " << nss.ns()
-                    << " with requested shard version " << requestedVersion.toString()
-                    << ", stored shard version is " << currVersion.toString()
-                    << causedBy(redact(status));
-
-                warning() << errmsg;
+                LOGV2_WARNING(
+                    22058,
+                    "Could not refresh metadata for the namespace {namespace} with the requested "
+                    "shard version {requestedShardVersion}; the current shard version is "
+                    "{currentShardVersion}: {error}",
+                    "Could not refresh metadata",
+                    "namespace"_attr = nss.ns(),
+                    "requestedShardVersion"_attr = requestedVersion,
+                    "currentShardVersion"_attr = currVersion,
+                    "error"_attr = redact(status));
 
                 result.append("ns", nss.ns());
+                result.append("code", status.code());
                 requestedVersion.appendLegacyWithField(&result, "version");
                 currVersion.appendLegacyWithField(&result, "globalVersion");
                 result.appendBool("reloadConfig", true);
@@ -381,14 +358,19 @@ public:
             } else if (!requestedVersion.isWriteCompatibleWith(currVersion)) {
                 // We reloaded a version that doesn't match the version mongos was trying to
                 // set.
-                errmsg = str::stream() << "requested shard version differs from"
-                                       << " config shard version for " << nss.ns()
-                                       << ", requested version is " << requestedVersion.toString()
-                                       << " but found version " << currVersion.toString();
-
                 static Occasionally sampler;
                 if (sampler.tick()) {
-                    warning() << errmsg;
+                    LOGV2_WARNING(
+                        22059,
+                        "Requested shard version differs from the authoritative (current) shard "
+                        "version for the namespace {namespace}; the requested version is "
+                        "{requestedShardVersion}, but the current version is "
+                        "{currentShardVersion}",
+                        "Requested shard version differs from the authoritative (current) shard "
+                        "version for this namespace",
+                        "namespace"_attr = nss.ns(),
+                        "requestedShardVersion"_attr = requestedVersion,
+                        "currentShardVersion"_attr = currVersion);
                 }
 
                 // WARNING: the exact fields below are important for compatibility with mongos
