@@ -79,10 +79,8 @@ public:
      */
     vector<ChunkType> setUpChunkLoaderWithFiveChunks();
 
-    void refreshCollectionEpochOnRemoteLoader();
-
     const KeyPattern kKeyPattern = KeyPattern(BSON(kPattern << 1));
-    const stdx::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)>
+    const std::function<void(OperationContext*, StatusWith<CollectionAndChangedChunks>)>
         kDoNothingCallbackFn = [](
             OperationContext * opCtx,
             StatusWith<CatalogCacheLoader::CollectionAndChangedChunks> swCollAndChunks) noexcept {};
@@ -100,10 +98,9 @@ void ShardServerCatalogCacheLoaderTest::setUp() {
 
     // Create mock remote and real shard loader, retaining a pointer to the mock remote loader so
     // that unit tests can manipulate it to return certain responses.
-    std::unique_ptr<CatalogCacheLoaderMock> mockLoader =
-        stdx::make_unique<CatalogCacheLoaderMock>();
+    std::unique_ptr<CatalogCacheLoaderMock> mockLoader = std::make_unique<CatalogCacheLoaderMock>();
     _remoteLoaderMock = mockLoader.get();
-    _shardLoader = stdx::make_unique<ShardServerCatalogCacheLoader>(std::move(mockLoader));
+    _shardLoader = std::make_unique<ShardServerCatalogCacheLoader>(std::move(mockLoader));
 
     // Set the shard loader to primary mode, and set it for testing.
     _shardLoader->initializeReplicaSetRole(true);
@@ -566,35 +563,5 @@ TEST_F(ShardServerCatalogCacheLoaderTest, PrimaryLoadFromShardedAndFindMixedChun
     }
 }
 
-void ShardServerCatalogCacheLoaderTest::refreshCollectionEpochOnRemoteLoader() {
-    ChunkVersion collectionVersion(1, 2, OID::gen());
-    CollectionType collectionType = makeCollectionType(collectionVersion);
-    vector<ChunkType> chunks = makeFiveChunks(collectionVersion);
-    _remoteLoaderMock->setCollectionRefreshReturnValue(collectionType);
-    _remoteLoaderMock->setChunkRefreshReturnValue(chunks);
-}
-
-TEST_F(ShardServerCatalogCacheLoaderTest, CollAndChunkTasksConsistency) {
-    // Put some metadata in the persisted cache (config.cache.chunks.*)
-    refreshCollectionEpochOnRemoteLoader();
-    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), kDoNothingCallbackFn)->get();
-    _shardLoader->waitForCollectionFlush(operationContext(), kNss);
-
-    // Pause the thread processing the pending updates on metadata
-    FailPointEnableBlock failPoint("hangCollectionFlush");
-
-    // Put a first task in the list of pending updates on metadata (in-memory)
-    refreshCollectionEpochOnRemoteLoader();
-    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), kDoNothingCallbackFn)->get();
-
-    // Bump the shard's term
-    _shardLoader->onStepUp();
-
-    // Putting a second task causes a verification of the contiguous versions in the list pending
-    // updates on metadata
-    refreshCollectionEpochOnRemoteLoader();
-    _shardLoader->getChunksSince(kNss, ChunkVersion::UNSHARDED(), kDoNothingCallbackFn)->get();
-}
-
 }  // namespace
 }  // namespace mongo
