@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -40,15 +40,19 @@
 #include "mongo/db/s/chunk_splitter.h"
 #include "mongo/db/s/database_sharding_state.h"
 #include "mongo/db/s/migration_source_manager.h"
+#include "mongo/db/s/migration_util.h"
+#include "mongo/db/s/operation_sharding_state.h"
+#include "mongo/db/s/range_deletion_task_gen.h"
 #include "mongo/db/s/shard_identity_rollback_notifier.h"
 #include "mongo/db/s/sharding_initialization_mongod.h"
+#include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/type_shard_identity.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/balancer_configuration.h"
 #include "mongo/s/catalog/type_shard_collection.h"
 #include "mongo/s/catalog/type_shard_database.h"
 #include "mongo/s/catalog_cache_loader.h"
 #include "mongo/s/grid.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 namespace {
@@ -114,6 +118,27 @@ private:
     const ShardIdentityType _shardIdentity;
 };
 
+/**
+ * Used to submit a range deletion task once it is certain that the update/insert to
+ * config.rangeDeletions is committed.
+ */
+class SubmitRangeDeletionHandler final : public RecoveryUnit::Change {
+public:
+    SubmitRangeDeletionHandler(OperationContext* opCtx, RangeDeletionTask task)
+        : _opCtx(opCtx), _task(std::move(task)) {}
+
+    void commit(boost::optional<Timestamp>) override {
+        migrationutil::submitRangeDeletionTask(_opCtx, _task).getAsync([](auto) {});
+    }
+
+    void rollback() override {}
+
+private:
+    OperationContext* _opCtx;
+    RangeDeletionTask _task;
+};
+
+
 /**
  * Invalidates the in-memory routing table cache when a collection is dropped, so the next caller
  * with routing information will provoke a routing table refresh and see the drop.
@@ -134,13 +159,14 @@ void onConfigDeleteInvalidateCachedCollectionMetadataAndNotify(OperationContext*
     // Extract which collection entry is being deleted from the _id field.
     std::string deletedCollection;
     fassert(40479,
-            bsonExtractStringField(query, ShardCollectionType::ns.name(), &deletedCollection));
+            bsonExtractStringField(query, ShardCollectionType::kNssFieldName, &deletedCollection));
     const NamespaceString deletedNss(deletedCollection);
 
     // Need the WUOW to retain the lock for CollectionVersionLogOpHandler::commit().
     AutoGetCollection autoColl(opCtx, deletedNss, MODE_IX);
 
-    opCtx->recoveryUnit()->registerChange(new CollectionVersionLogOpHandler(opCtx, deletedNss));
+    opCtx->recoveryUnit()->registerChange(
+        std::make_unique<CollectionVersionLogOpHandler>(opCtx, deletedNss));
 }
 
 /**
@@ -155,16 +181,7 @@ void incrementChunkOnInsertOrUpdate(OperationContext* opCtx,
                                     long dataWritten,
                                     bool fromMigrate) {
     const auto& shardKeyPattern = chunkManager.getShardKeyPattern();
-
-    // Each inserted/updated document should contain the shard key. The only instance in which a
-    // document could not contain a shard key is if the insert/update is performed through mongod
-    // explicitly, as opposed to first routed through mongos.
     BSONObj shardKey = shardKeyPattern.extractShardKeyFromDoc(document);
-    if (shardKey.woCompare(BSONObj()) == 0) {
-        warning() << "inserting document " << document.toString() << " without shard key pattern "
-                  << shardKeyPattern << " into a sharded collection";
-        return;
-    }
 
     // Use the shard key to locate the chunk into which the document was updated, and increment the
     // number of bytes tracked for the chunk.
@@ -194,6 +211,30 @@ void incrementChunkOnInsertOrUpdate(OperationContext* opCtx,
     }
 }
 
+/**
+ * Aborts any ongoing migration for the given namespace if the observed operation was sent with a
+ * shard version. Should only be called when observing index operations.
+ *
+ * TODO SERVER-45017: Update this comment once the check for a shard version is removed.
+ */
+void abortOngoingMigrationIfNeeded(OperationContext* opCtx, const NamespaceString nss) {
+    // Only abort migrations if the request is versioned, meaning it came from a mongos using the
+    // 4.4 index operation protocol.
+    //
+    // TODO SERVER-45017: Remove this check once 4.4. is last-stable, since all index operations
+    // sent from a mongos will have shard versions.
+    if (!OperationShardingState::isOperationVersioned(opCtx)) {
+        return;
+    }
+
+    auto* const csr = CollectionShardingRuntime::get(opCtx, nss);
+    auto csrLock = CollectionShardingRuntime::CSRLock::lockShared(opCtx, csr);
+    auto msm = MigrationSourceManager::get(csr, csrLock);
+    if (msm) {
+        msm->abortDueToConflictingIndexOperation();
+    }
+}
+
 }  // namespace
 
 ShardServerOpObserver::ShardServerOpObserver() = default;
@@ -207,7 +248,7 @@ void ShardServerOpObserver::onInserts(OperationContext* opCtx,
                                       std::vector<InsertStatement>::const_iterator end,
                                       bool fromMigrate) {
     auto* const css = CollectionShardingState::get(opCtx, nss);
-    const auto metadata = css->getCurrentMetadata();
+    const auto collDesc = css->getCollectionDescription();
 
     for (auto it = begin; it != end; ++it) {
         const auto& insertedDoc = it->doc;
@@ -219,15 +260,30 @@ void ShardServerOpObserver::onInserts(OperationContext* opCtx,
                         uassertStatusOK(ShardIdentityType::fromShardIdentityDocument(insertedDoc));
                     uassertStatusOK(shardIdentityDoc.validate());
                     opCtx->recoveryUnit()->registerChange(
-                        new ShardIdentityLogOpHandler(opCtx, std::move(shardIdentityDoc)));
+                        std::make_unique<ShardIdentityLogOpHandler>(opCtx,
+                                                                    std::move(shardIdentityDoc)));
                 }
             }
         }
 
-        if (metadata->isSharded()) {
+        if (nss == NamespaceString::kRangeDeletionNamespace) {
+            if (!isStandaloneOrPrimary(opCtx)) {
+                return;
+            }
+
+            auto deletionTask = RangeDeletionTask::parse(
+                IDLParserErrorContext("ShardServerOpObserver"), insertedDoc);
+
+            if (!deletionTask.getPending()) {
+                opCtx->recoveryUnit()->registerChange(
+                    std::make_unique<SubmitRangeDeletionHandler>(opCtx, deletionTask));
+            }
+        }
+
+        if (collDesc.isSharded()) {
             incrementChunkOnInsertOrUpdate(opCtx,
                                            nss,
-                                           *metadata->getChunkManager(),
+                                           *collDesc->getChunkManager(),
                                            insertedDoc,
                                            insertedDoc.objsize(),
                                            fromMigrate);
@@ -237,7 +293,7 @@ void ShardServerOpObserver::onInserts(OperationContext* opCtx,
 
 void ShardServerOpObserver::onUpdate(OperationContext* opCtx, const OplogUpdateEntryArgs& args) {
     auto* const css = CollectionShardingState::get(opCtx, args.nss);
-    const auto metadata = css->getCurrentMetadata();
+    const auto collDesc = css->getCollectionDescription();
 
     if (args.nss == NamespaceString::kShardConfigCollectionsNamespace) {
         // Notification of routing table changes are only needed on secondaries
@@ -264,7 +320,7 @@ void ShardServerOpObserver::onUpdate(OperationContext* opCtx, const OplogUpdateE
             std::string coll;
             fassert(40477,
                     bsonExtractStringField(
-                        args.updateArgs.criteria, ShardCollectionType::ns.name(), &coll));
+                        args.updateArgs.criteria, ShardCollectionType::kNssFieldName, &coll));
             return NamespaceString(coll);
         }());
 
@@ -278,13 +334,13 @@ void ShardServerOpObserver::onUpdate(OperationContext* opCtx, const OplogUpdateE
             // Need the WUOW to retain the lock for CollectionVersionLogOpHandler::commit()
             AutoGetCollection autoColl(opCtx, updatedNss, MODE_IX);
 
-            if (setField.hasField(ShardCollectionType::refreshing.name()) &&
-                !setField.getBoolField("refreshing")) {
+            auto refreshingField = setField.getField(ShardCollectionType::kRefreshingFieldName);
+            if (refreshingField.isBoolean() && !refreshingField.boolean()) {
                 opCtx->recoveryUnit()->registerChange(
-                    new CollectionVersionLogOpHandler(opCtx, updatedNss));
+                    std::make_unique<CollectionVersionLogOpHandler>(opCtx, updatedNss));
             }
 
-            if (setField.hasField(ShardCollectionType::enterCriticalSectionCounter.name())) {
+            if (setField.hasField(ShardCollectionType::kEnterCriticalSectionCounterFieldName)) {
                 // Force subsequent uses of the namespace to refresh the filtering metadata so they
                 // can synchronize with any work happening on the primary (e.g., migration critical
                 // section).
@@ -321,19 +377,38 @@ void ShardServerOpObserver::onUpdate(OperationContext* opCtx, const OplogUpdateE
 
             if (setField.hasField(ShardDatabaseType::enterCriticalSectionCounter.name())) {
                 AutoGetDb autoDb(opCtx, db, MODE_X);
-                if (autoDb.getDb()) {
-                    auto& dss = DatabaseShardingState::get(autoDb.getDb());
-                    auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
-                    dss.setDbVersion(opCtx, boost::none, dssLock);
-                }
+                auto dss = DatabaseShardingState::get(opCtx, db);
+                auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
+                dss->setDbVersion(opCtx, boost::none, dssLock);
             }
         }
     }
 
-    if (metadata->isSharded()) {
+    if (args.nss == NamespaceString::kRangeDeletionNamespace) {
+        if (!isStandaloneOrPrimary(opCtx))
+            return;
+
+        BSONElement unsetElement;
+        Status status = bsonExtractTypedField(
+            args.updateArgs.update, StringData("$unset"), Object, &unsetElement);
+
+        if (unsetElement.Obj().hasField("pending")) {
+            auto deletionTask = RangeDeletionTask::parse(
+                IDLParserErrorContext("ShardServerOpObserver"), args.updateArgs.updatedDoc);
+
+            if (deletionTask.getDonorShardId() != ShardingState::get(opCtx)->shardId()) {
+                // Range deletion tasks for moved away chunks are scheduled through the
+                // MigrationCoordinator, so only schedule a task for received chunks.
+                opCtx->recoveryUnit()->registerChange(
+                    std::make_unique<SubmitRangeDeletionHandler>(opCtx, deletionTask));
+            }
+        }
+    }
+
+    if (collDesc.isSharded()) {
         incrementChunkOnInsertOrUpdate(opCtx,
                                        args.nss,
-                                       *metadata->getChunkManager(),
+                                       *collDesc->getChunkManager(),
                                        args.updateArgs.updatedDoc,
                                        args.updateArgs.updatedDoc.objsize(),
                                        args.updateArgs.fromMigrate);
@@ -350,7 +425,8 @@ void ShardServerOpObserver::onDelete(OperationContext* opCtx,
                                      const NamespaceString& nss,
                                      OptionalCollectionUUID uuid,
                                      StmtId stmtId,
-                                     const OplogDeleteEntryArgs& args) {
+                                     bool fromMigrate,
+                                     const boost::optional<BSONObj>& deletedDoc) {
     auto& documentKey = getDocumentKey(opCtx);
 
     if (nss == NamespaceString::kShardConfigCollectionsNamespace) {
@@ -368,11 +444,9 @@ void ShardServerOpObserver::onDelete(OperationContext* opCtx,
             bsonExtractStringField(documentKey, ShardDatabaseType::name.name(), &deletedDatabase));
 
         AutoGetDb autoDb(opCtx, deletedDatabase, MODE_X);
-        if (autoDb.getDb()) {
-            auto& dss = DatabaseShardingState::get(autoDb.getDb());
-            auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, &dss);
-            dss.setDbVersion(opCtx, boost::none, dssLock);
-        }
+        auto dss = DatabaseShardingState::get(opCtx, deletedDatabase);
+        auto dssLock = DatabaseShardingState::DSSLock::lockExclusive(opCtx, dss);
+        dss->setDbVersion(opCtx, boost::none, dssLock);
     }
 
     if (nss == NamespaceString::kServerConfigurationNamespace) {
@@ -383,8 +457,9 @@ void ShardServerOpObserver::onDelete(OperationContext* opCtx,
                     uasserted(40070,
                               "cannot delete shardIdentity document while in --shardsvr mode");
                 } else {
-                    warning() << "Shard identity document rolled back.  Will shut down after "
-                                 "finishing rollback.";
+                    LOGV2_WARNING(23779,
+                                  "Shard identity document rolled back.  Will shut down after "
+                                  "finishing rollback.");
                     ShardIdentityRollbackNotifier::get(opCtx)->recordThatRollbackHappened();
                 }
             }
@@ -404,8 +479,9 @@ repl::OpTime ShardServerOpObserver::onDropCollection(OperationContext* opCtx,
 
         // Can't confirm whether there was a ShardIdentity document or not yet, so assume there was
         // one and shut down the process to clear the in-memory sharding state
-        warning() << "admin.system.version collection rolled back. Will shut down after finishing "
-                     "rollback";
+        LOGV2_WARNING(23780,
+                      "admin.system.version collection rolled back. Will shut down after finishing "
+                      "rollback");
 
         ShardIdentityRollbackNotifier::get(opCtx)->recordThatRollbackHappened();
     }
@@ -413,4 +489,36 @@ repl::OpTime ShardServerOpObserver::onDropCollection(OperationContext* opCtx,
     return {};
 }
 
+void ShardServerOpObserver::onStartIndexBuild(OperationContext* opCtx,
+                                              const NamespaceString& nss,
+                                              CollectionUUID collUUID,
+                                              const UUID& indexBuildUUID,
+                                              const std::vector<BSONObj>& indexes,
+                                              bool fromMigrate) {
+    abortOngoingMigrationIfNeeded(opCtx, nss);
+};
+
+void ShardServerOpObserver::onStartIndexBuildSinglePhase(OperationContext* opCtx,
+                                                         const NamespaceString& nss) {
+    abortOngoingMigrationIfNeeded(opCtx, nss);
+}
+
+void ShardServerOpObserver::onDropIndex(OperationContext* opCtx,
+                                        const NamespaceString& nss,
+                                        OptionalCollectionUUID uuid,
+                                        const std::string& indexName,
+                                        const BSONObj& indexInfo) {
+    abortOngoingMigrationIfNeeded(opCtx, nss);
+};
+
+void ShardServerOpObserver::onCollMod(OperationContext* opCtx,
+                                      const NamespaceString& nss,
+                                      OptionalCollectionUUID uuid,
+                                      const BSONObj& collModCmd,
+                                      const CollectionOptions& oldCollOptions,
+                                      boost::optional<IndexCollModInfo> indexInfo) {
+    abortOngoingMigrationIfNeeded(opCtx, nss);
+};
+
+
 }  // namespace mongo
