@@ -251,7 +251,7 @@ TEST(TransactionCoordinatorFuturesUtilTest,
     promises[0].setError(errorStatus1);
     ASSERT(!resultFuture.isReady());
 
-    Status errorStatus2{ErrorCodes::NotWritablePrimary, "dummy error"};
+    Status errorStatus2{ErrorCodes::NotMaster, "dummy error"};
     promises[1].setError(errorStatus2);
     ASSERT(!resultFuture.isReady());
 
@@ -315,7 +315,7 @@ protected:
             }
         };
 
-        return stdx::make_unique<StaticCatalogClient>();
+        return std::make_unique<StaticCatalogClient>();
     }
 
     static std::vector<ShardId> makeThreeShardIdsList() {
@@ -643,11 +643,11 @@ TEST_F(AsyncWorkSchedulerTest, DestroyingSchedulerCapturedInFutureCallback) {
     future.get();
 }
 
-TEST_F(AsyncWorkSchedulerTest, NotifiesRemoteCommandTargeter_CmdResponseNotWritablePrimaryError) {
+TEST_F(AsyncWorkSchedulerTest, NotifiesRemoteCommandTargeter_CmdResponseNotMasterError) {
     ASSERT_EQ(0UL, getShardTargeterMock(kShardIds[1])->getAndClearMarkedDownHosts().size());
 
     scheduleAWSRemoteCommandWithResponse(kShardIds[1],
-                                         BSON("ok" << 0 << "code" << ErrorCodes::NotWritablePrimary
+                                         BSON("ok" << 0 << "code" << ErrorCodes::NotMaster
                                                    << "errmsg"
                                                    << "dummy"));
 
@@ -684,7 +684,7 @@ TEST_F(AsyncWorkSchedulerTest, DoesNotNotifyRemoteCommandTargeter_CmdResponseOth
     ASSERT_EQ(0UL, getShardTargeterMock(kShardIds[1])->getAndClearMarkedDownHosts().size());
 }
 
-TEST_F(AsyncWorkSchedulerTest, NotifiesRemoteCommandTargeter_WCNotPrimaryError) {
+TEST_F(AsyncWorkSchedulerTest, NotifiesRemoteCommandTargeter_WCNotMasterError) {
     ASSERT_EQ(0UL, getShardTargeterMock(kShardIds[1])->getAndClearMarkedDownHosts().size());
 
     scheduleAWSRemoteCommandWithResponse(
