@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -38,7 +38,6 @@
 #include "mongo/db/s/transaction_coordinator_service.h"
 #include "mongo/db/s/transaction_coordinator_test_fixture.h"
 #include "mongo/db/write_concern_options.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
@@ -184,7 +183,8 @@ protected:
         }
 
         // Wait for abort to complete.
-        commitDecisionFuture.get();
+        ASSERT_THROWS_CODE(
+            commitDecisionFuture.get(), AssertionException, ErrorCodes::NoSuchTransaction);
     }
 
     auto* service() const {
@@ -198,17 +198,17 @@ TEST_F(TransactionCoordinatorServiceStepUpStepDownTest, OperationsFailBeforeStep
     ASSERT_THROWS_CODE(service()->createCoordinator(
                            operationContext(), makeLogicalSessionIdForTest(), 0, kCommitDeadline),
                        AssertionException,
-                       ErrorCodes::NotWritablePrimary);
+                       ErrorCodes::NotMaster);
 
     ASSERT_THROWS_CODE(service()->coordinateCommit(
                            operationContext(), makeLogicalSessionIdForTest(), 0, kTwoShardIdSet),
                        AssertionException,
-                       ErrorCodes::NotWritablePrimary);
+                       ErrorCodes::NotMaster);
 
     ASSERT_THROWS_CODE(
         service()->recoverCommit(operationContext(), makeLogicalSessionIdForTest(), 0),
         AssertionException,
-        ErrorCodes::NotWritablePrimary);
+        ErrorCodes::NotMaster);
 }
 
 TEST_F(TransactionCoordinatorServiceStepUpStepDownTest, OperationsBlockBeforeStepUpCompletes) {
@@ -380,8 +380,10 @@ TEST_F(TransactionCoordinatorServiceTest,
     assertAbortSentAndRespondWithSuccess();
     assertAbortSentAndRespondWithSuccess();
 
-    ASSERT_EQ(static_cast<int>(commitDecisionFuture1.get()),
-              static_cast<int>(commitDecisionFuture2.get()));
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture1.get(), AssertionException, ErrorCodes::NoSuchTransaction);
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture2.get(), AssertionException, ErrorCodes::NoSuchTransaction);
 }
 
 TEST_F(TransactionCoordinatorServiceTest,
@@ -423,8 +425,10 @@ TEST_F(TransactionCoordinatorServiceTest, RecoverCommitJoinsOngoingCoordinationT
     assertAbortSentAndRespondWithSuccess();
     assertAbortSentAndRespondWithSuccess();
 
-    ASSERT_EQ(static_cast<int>(commitDecisionFuture1.get()),
-              static_cast<int>(commitDecisionFuture2.get()));
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture1.get(), AssertionException, ErrorCodes::NoSuchTransaction);
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture2.get(), AssertionException, ErrorCodes::NoSuchTransaction);
 }
 
 TEST_F(TransactionCoordinatorServiceTest, RecoverCommitJoinsOngoingCoordinationThatLeadsToCommit) {
@@ -462,7 +466,7 @@ TEST_F(TransactionCoordinatorServiceTest,
         operationContext(), _lsid, _txnNumber + 1, kCommitDeadline);
 
     ASSERT_THROWS_CODE(
-        commitDecisionFuture.get(), AssertionException, ErrorCodes::NoSuchTransaction);
+        commitDecisionFuture.get(), AssertionException, ErrorCodes::TransactionCoordinatorCanceled);
 }
 
 TEST_F(
@@ -484,8 +488,9 @@ TEST_F(
 
     // The old transaction should now be committed.
     if (oldTxnCommitDecisionFuture) {
-        ASSERT_THROWS_CODE(
-            oldTxnCommitDecisionFuture->get(), AssertionException, ErrorCodes::NoSuchTransaction);
+        ASSERT_THROWS_CODE(oldTxnCommitDecisionFuture->get(),
+                           AssertionException,
+                           ErrorCodes::TransactionCoordinatorCanceled);
     }
 
     // Make sure the newly created one works fine too.
@@ -549,8 +554,8 @@ TEST_F(TransactionCoordinatorServiceTest, CoordinatorRetriesOnWriteConcernErrorT
     assertAbortSentAndRespondWithNoSuchTransaction();
 
     // The transaction should now be aborted.
-    ASSERT_EQ(static_cast<int>(commitDecisionFuture.get()),
-              static_cast<int>(txn::CommitDecision::kAbort));
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture.get(), AssertionException, ErrorCodes::NoSuchTransaction);
 }
 
 TEST_F(TransactionCoordinatorServiceTest, CoordinatorRetriesOnWriteConcernErrorToCommit) {
@@ -623,7 +628,8 @@ TEST_F(TransactionCoordinatorServiceTest, CoordinatorAbortsIfDeadlinePassesAndSt
     assertAbortSentAndRespondWithSuccess();
     assertAbortSentAndRespondWithSuccess();
 
-    ASSERT_EQ(int(txn::CommitDecision::kAbort), int(commitDecisionFuture->get()));
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture->get(), AssertionException, ErrorCodes::NoSuchTransaction);
 }
 
 TEST_F(TransactionCoordinatorServiceTest,
@@ -688,8 +694,8 @@ TEST_F(TransactionCoordinatorServiceTestSingleTxn,
     assertAbortSentAndRespondWithSuccess();
     assertAbortSentAndRespondWithSuccess();
 
-    auto commitDecision = commitDecisionFuture.get();
-    ASSERT_EQ(static_cast<int>(commitDecision), static_cast<int>(txn::CommitDecision::kAbort));
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture.get(), AssertionException, ErrorCodes::NoSuchTransaction);
 }
 
 TEST_F(TransactionCoordinatorServiceTestSingleTxn,
@@ -733,8 +739,10 @@ TEST_F(TransactionCoordinatorServiceTestSingleTxn,
 
     abortTransaction(*coordinatorService(), _lsid, _txnNumber, kTwoShardIdSet, kTwoShardIdList[0]);
 
-    ASSERT_EQ(static_cast<int>(commitDecisionFuture1.get()),
-              static_cast<int>(commitDecisionFuture2.get()));
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture1.get(), AssertionException, ErrorCodes::NoSuchTransaction);
+    ASSERT_THROWS_CODE(
+        commitDecisionFuture2.get(), AssertionException, ErrorCodes::NoSuchTransaction);
 }
 
 }  // namespace
