@@ -43,6 +43,7 @@
 #include "mongo/db/index_builds_coordinator_mongod.h"
 #include "mongo/db/logical_clock.h"
 #include "mongo/db/op_observer_registry.h"
+#include "mongo/db/s/collection_sharding_state_factory_shard.h"
 #include "mongo/db/service_entry_point_mongod.h"
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/db/storage/storage_options.h"
@@ -77,13 +78,16 @@ ServiceContextMongoDTest::ServiceContextMongoDTest(std::string engine, RepairAct
 
     storageGlobalParams.dbpath = _tempDir.path();
 
-    initializeStorageEngine(serviceContext, StorageEngineInitFlags::kNone);
+    // Since unit tests start in their own directories, skip lock file and metadata file for faster
+    // startup.
+    initializeStorageEngine(serviceContext,
+                            StorageEngineInitFlags::kAllowNoLockFile |
+                                StorageEngineInitFlags::kSkipMetadataFile);
 
     DatabaseHolder::set(serviceContext, std::make_unique<DatabaseHolderImpl>());
     IndexAccessMethodFactory::set(serviceContext, std::make_unique<IndexAccessMethodFactoryImpl>());
     Collection::Factory::set(serviceContext, std::make_unique<CollectionImpl::FactoryImpl>());
     IndexBuildsCoordinator::set(serviceContext, std::make_unique<IndexBuildsCoordinatorMongod>());
-    getServiceContext()->getStorageEngine()->notifyStartupComplete();
 }
 
 ServiceContextMongoDTest::~ServiceContextMongoDTest() {
@@ -94,8 +98,6 @@ ServiceContextMongoDTest::~ServiceContextMongoDTest() {
         databaseHolder->closeAll(opCtx.get());
     }
 
-    IndexBuildsCoordinator::get(getServiceContext())->shutdown();
-
     shutdownGlobalStorageEngineCleanly(getServiceContext());
 
     std::swap(storageGlobalParams.engine, _stashedStorageParams.engine);
@@ -103,4 +105,29 @@ ServiceContextMongoDTest::~ServiceContextMongoDTest() {
     std::swap(storageGlobalParams.repair, _stashedStorageParams.repair);
 }
 
+void ServiceContextMongoDTest::setUp() {
+    ServiceContextTest::setUp();
+
+    CollectionShardingStateFactory::set(
+        getServiceContext(),
+        std::make_unique<CollectionShardingStateFactoryShard>(getServiceContext()));
+}
+
+void ServiceContextMongoDTest::tearDown() {
+    {
+        // Some tests set the current OperationContext but do not release it until destruction.
+        ServiceContext::UniqueOperationContext uniqueOpCtx;
+        auto opCtx = getClient()->getOperationContext();
+        if (!opCtx) {
+            uniqueOpCtx = getClient()->makeOperationContext();
+            opCtx = uniqueOpCtx.get();
+        }
+        IndexBuildsCoordinator::get(opCtx)->shutdown(opCtx);
+    }
+
+    CollectionShardingStateFactory::clear(getServiceContext());
+
+    ServiceContextTest::tearDown();
+}
+
 }  // namespace mongo
