@@ -114,6 +114,8 @@ class RecoveryUnit {
     RecoveryUnit& operator=(const RecoveryUnit&) = delete;
 
 public:
+    void commitRegisteredChanges(boost::optional<Timestamp> commitTimestamp);
+    void abortRegisteredChanges();
     virtual ~RecoveryUnit() {}
 
     /**
@@ -130,7 +132,10 @@ public:
      *
      * Should be called through WriteUnitOfWork rather than directly.
      */
-    virtual void commitUnitOfWork() = 0;
+    void commitUnitOfWork() {
+        assignNextSnapshotId();
+        doCommitUnitOfWork();
+    }
 
     /**
      * Marks the end of a unit of work and rolls back all changes registered by calls to onRollback
@@ -139,7 +144,10 @@ public:
      *
      * Should be called through WriteUnitOfWork rather than directly.
      */
-    virtual void abortUnitOfWork() = 0;
+    void abortUnitOfWork() {
+        assignNextSnapshotId();
+        doAbortUnitOfWork();
+    }
 
     /**
      * Transitions the active unit of work to the "prepared" state. Must be called after
@@ -185,9 +193,9 @@ public:
      * Waits until all commits that happened before this call are durable in the journal. Returns
      * true, unless the storage engine cannot guarantee durability, which should never happen when
      * isDurable() returned true. This cannot be called from inside a unit of work, and should
-     * fail if it is.
+     * fail if it is. This method invariants if the caller holds any locks, except for repair.
      */
-    virtual bool waitUntilDurable() = 0;
+    virtual bool waitUntilDurable(OperationContext* opCtx) = 0;
 
     /**
      * Unlike `waitUntilDurable`, this method takes a stable checkpoint, making durable any writes
@@ -198,15 +206,18 @@ public:
      * This must not be called by a system taking user writes until after a stable timestamp is
      * passed to the storage engine.
      */
-    virtual bool waitUntilUnjournaledWritesDurable(bool stableCheckpoint = true) {
-        return waitUntilDurable();
+    virtual bool waitUntilUnjournaledWritesDurable(OperationContext* opCtx, bool stableCheckpoint) {
+        return waitUntilDurable(opCtx);
     }
 
     /**
      * If there is an open transaction, it is closed. On return no transaction is active. This
      * cannot be called inside of a WriteUnitOfWork, and should fail if it is.
      */
-    virtual void abandonSnapshot() = 0;
+    void abandonSnapshot() {
+        assignNextSnapshotId();
+        doAbandonSnapshot();
+    }
 
     /**
      * Informs the RecoveryUnit that a snapshot will be needed soon, if one was not already
@@ -260,7 +271,9 @@ public:
      *
      * This is unrelated to Timestamp which must be globally comparable.
      */
-    virtual SnapshotId getSnapshotId() const = 0;
+    SnapshotId getSnapshotId() const {
+        return SnapshotId{_mySnapshotId};
+    }
 
     /**
      * Sets a timestamp to assign to future writes in a transaction.
@@ -278,14 +291,6 @@ public:
         return Status::OK();
     }
 
-    /**
-     * Returns true if a commit timestamp has been assigned to writes in this transaction.
-     * Otherwise, returns false.
-     */
-    virtual bool isTimestamped() const {
-        return false;
-    }
-
     /**
      * Sets a timestamp that will be assigned to all future writes on this RecoveryUnit until
      * clearCommitTimestamp() is called. This must be called either outside of a WUOW or on a
@@ -407,6 +412,26 @@ public:
         kProvided
     };
 
+    static std::string toString(ReadSource rs) {
+        switch (rs) {
+            case ReadSource::kUnset:
+                return "kUnset";
+            case ReadSource::kNoTimestamp:
+                return "kNoTimestamp";
+            case ReadSource::kMajorityCommitted:
+                return "kMajorityCommitted";
+            case ReadSource::kNoOverlap:
+                return "kNoOverlap";
+            case ReadSource::kLastApplied:
+                return "kLastApplied";
+            case ReadSource::kAllDurableSnapshot:
+                return "kAllDurableSnapshot";
+            case ReadSource::kProvided:
+                return "kProvided";
+        }
+        MONGO_UNREACHABLE;
+    }
+
     /**
      * Sets which timestamp to use for read transactions. If 'provided' is supplied, only kProvided
      * is an acceptable input.
@@ -435,6 +460,21 @@ public:
         return false;
     };
 
+    /**
+     * Indicates whether a unit of work is active. Will be true after beginUnitOfWork
+     * is called and before either commitUnitOfWork or abortUnitOfWork gets called.
+     */
+    virtual bool inActiveTxn() const = 0;
+
+    /**
+     * Registers a callback to be called prior to a WriteUnitOfWork committing the storage
+     * transaction. This callback may throw a WriteConflictException which will abort the
+     * transaction.
+     */
+    virtual void registerPreCommitHook(std::function<void(OperationContext*)> callback);
+
+    virtual void runPreCommitHooks(OperationContext* opCtx);
+
     /**
      * A Change is an action that is registerChange()'d while a WriteUnitOfWork exists. The
      * change is either rollback()'d or commit()'d when the WriteUnitOfWork goes out of scope.
@@ -467,7 +507,7 @@ public:
      * The registerChange() method may only be called when a WriteUnitOfWork is active, and
      * may not be called during commit or rollback.
      */
-    virtual void registerChange(Change* change) = 0;
+    virtual void registerChange(std::unique_ptr<Change> change);
 
     /**
      * Registers a callback to be called if the current WriteUnitOfWork rolls back.
@@ -488,7 +528,7 @@ public:
             Callback _callback;
         };
 
-        registerChange(new OnRollbackChange(std::move(callback)));
+        registerChange(std::make_unique<OnRollbackChange>(std::move(callback)));
     }
 
     /**
@@ -510,23 +550,133 @@ public:
             Callback _callback;
         };
 
-        registerChange(new OnCommitChange(std::move(callback)));
+        registerChange(std::make_unique<OnCommitChange>(std::move(callback)));
     }
 
     virtual void setOrderedCommit(bool orderedCommit) = 0;
 
-    void setNoEvictionAfterRollback() {
-        _noEvictionAfterRollback = true;
+    /**
+     * State transitions:
+     *
+     *   /------------------------> Inactive <-----------------------------\
+     *   |                             |                                   |
+     *   |                             |                                   |
+     *   |              /--------------+--------------\                    |
+     *   |              |                             |                    | abandonSnapshot()
+     *   |              |                             |                    |
+     *   |   beginUOW() |                             | _txnOpen()         |
+     *   |              |                             |                    |
+     *   |              V                             V                    |
+     *   |    InactiveInUnitOfWork          ActiveNotInUnitOfWork ---------/
+     *   |              |                             |
+     *   |              |                             |
+     *   |   _txnOpen() |                             | beginUOW()
+     *   |              |                             |
+     *   |              \--------------+--------------/
+     *   |                             |
+     *   |                             |
+     *   |                             V
+     *   |                           Active
+     *   |                             |
+     *   |                             |
+     *   |              /--------------+--------------\
+     *   |              |                             |
+     *   |              |                             |
+     *   |   abortUOW() |                             | commitUOW()
+     *   |              |                             |
+     *   |              V                             V
+     *   |          Aborting                      Committing
+     *   |              |                             |
+     *   |              |                             |
+     *   |              |                             |
+     *   \--------------+-----------------------------/
+     *
+     */
+    enum class State {
+        kInactive,
+        kInactiveInUnitOfWork,
+        kActiveNotInUnitOfWork,
+        kActive,
+        kAborting,
+        kCommitting,
+    };
+
+    std::string toString(State state) const {
+        switch (state) {
+            case State::kInactive:
+                return "Inactive";
+            case State::kInactiveInUnitOfWork:
+                return "InactiveInUnitOfWork";
+            case State::kActiveNotInUnitOfWork:
+                return "ActiveNotInUnitOfWork";
+            case State::kActive:
+                return "Active";
+            case State::kCommitting:
+                return "Committing";
+            case State::kAborting:
+                return "Aborting";
+        }
+        MONGO_UNREACHABLE;
     }
 
-    bool getNoEvictionAfterRollback() const {
-        return _noEvictionAfterRollback;
+    void setMustBeTimestamped() {
+        _mustBeTimestamped = true;
     }
 
 protected:
-    RecoveryUnit() {}
+    RecoveryUnit();
 
-    bool _noEvictionAfterRollback = false;
+    /**
+     * Returns the current state.
+     */
+    State _getState() const {
+        return _state;
+    }
+
+    /**
+     * Transitions to new state.
+     */
+    void _setState(State newState) {
+        _state = newState;
+    }
+
+    /**
+     * Returns true if active.
+     */
+    bool _isActive() const {
+        return State::kActiveNotInUnitOfWork == _state || State::kActive == _state;
+    }
+
+    /**
+     * Returns true if currently managed by a WriteUnitOfWork.
+     */
+    bool _inUnitOfWork() const {
+        return State::kInactiveInUnitOfWork == _state || State::kActive == _state;
+    }
+
+    /**
+     * Returns true if currently running commit or rollback handlers
+     */
+    bool _isCommittingOrAborting() const {
+        return State::kCommitting == _state || State::kAborting == _state;
+    }
+
+    bool _mustBeTimestamped = false;
+
+private:
+    // Sets the snapshot associated with this RecoveryUnit to a new globally unique id number.
+    void assignNextSnapshotId();
+
+    virtual void doAbandonSnapshot() = 0;
+    virtual void doCommitUnitOfWork() = 0;
+    virtual void doAbortUnitOfWork() = 0;
+
+    std::vector<std::function<void(OperationContext*)>> _preCommitHooks;
+
+    typedef std::vector<std::unique_ptr<Change>> Changes;
+    Changes _changes;
+    State _state = State::kInactive;
+    uint64_t _mySnapshotId;
 };
 
 }  // namespace mongo
