@@ -36,6 +36,7 @@
 #include "mongo/bson/bsonobj.h"
 #include "mongo/db/storage/engine_extension.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/db/catalog/index_builds.h"
 #include "mongo/db/storage/temporary_record_store.h"
 #include "mongo/util/functional.h"
 #include "mongo/util/str.h"
@@ -197,21 +198,7 @@ public:
      * are called. Any initialization work that requires the ability to create OperationContexts
      * should be done here rather than in the constructor.
      */
-    virtual void finishInit() {}
-
-    /**
-     * During the startup process, the storage engine is one of the first components to be started
-     * up and fully initialized. But that fully initialized storage engine may not be recognized as
-     * the end for the remaining storage startup tasks that still need to be performed.
-     *
-     * For example, after the storage engine has been fully initialized, we need to access it in
-     * order to set up all of the collections and indexes based on the metadata, or perform some
-     * corrective measures on the data files, etc.
-     *
-     * When all of the storage startup tasks are completed as a whole, then this function is called
-     * by the external force managing the startup process.
-     */
-    virtual void notifyStartupComplete() {}
+    virtual void finishInit() = 0;
 
     /**
      * Returns a new interface to the storage engine's recovery unit.  The recovery
@@ -238,16 +225,17 @@ public:
     /**
      * Returns whether the storage engine supports locking at a database level.
      */
-    virtual bool supportsDBLocking() const {
-        return true;
-    }
+    virtual bool supportsDBLocking() const = 0;
 
     /**
      * Returns whether the storage engine supports capped collections.
      */
-    virtual bool supportsCappedCollections() const {
-        return true;
-    }
+    virtual bool supportsCappedCollections() const = 0;
+
+    /**
+     * Returns whether the storage engine supports checkpoints.
+     */
+    virtual bool supportsCheckpoints() const = 0;
 
     /**
      * Returns whether the engine supports a journalling concept or not.
@@ -265,8 +253,8 @@ public:
      *
      * Must be called with the global lock acquired in exclusive mode.
      */
-    virtual void loadCatalog(OperationContext* opCtx) {}
-    virtual void closeCatalog(OperationContext* opCtx) {}
+    virtual void loadCatalog(OperationContext* opCtx) = 0;
+    virtual void closeCatalog(OperationContext* opCtx) = 0;
 
     /**
      * Closes all file handles associated with a database.
@@ -279,9 +267,13 @@ public:
     virtual Status dropDatabase(OperationContext* opCtx, StringData db) = 0;
 
     /**
-     * @return number of files flushed
+     * Checkpoints the data to disk.
+     *
+     * 'callerHoldsReadLock' signals whether the caller holds a read lock. A write lock may be taken
+     * internally, but will be skipped for callers holding a read lock because a write lock would
+     * conflict. The JournalListener will not be updated in this case.
      */
-    virtual int flushAllFiles(OperationContext* opCtx, bool sync) = 0;
+    virtual void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) = 0;
 
     /**
      * Transitions the storage engine into backup mode.
@@ -302,10 +294,7 @@ public:
      * retried, returns a non-OK status. This function may throw a WriteConflictException, which
      * should trigger a retry by the caller. All other exceptions should be treated as errors.
      */
-    virtual Status beginBackup(OperationContext* opCtx) {
-        return Status(ErrorCodes::CommandNotSupported,
-                      "The current storage engine doesn't support backup mode");
-    }
+    virtual Status beginBackup(OperationContext* opCtx) = 0;
 
     /**
      * Transitions the storage engine out of backup mode.
@@ -314,9 +303,7 @@ public:
      *
      * Storage engines implementing this feature should fassert when unable to leave backup mode.
      */
-    virtual void endBackup(OperationContext* opCtx) {
-        return;
-    }
+    virtual void endBackup(OperationContext* opCtx) = 0;
 
     /**
      * Disables the storage of incremental backup history until a subsequent incremental backup
@@ -344,23 +331,20 @@ public:
     virtual StatusWith<StorageEngine::BackupInformation> beginNonBlockingBackup(
         OperationContext* opCtx, const BackupOptions& options) = 0;
 
-    virtual void endNonBlockingBackup(OperationContext* opCtx) {
-        return;
-    }
+    virtual void endNonBlockingBackup(OperationContext* opCtx) = 0;
 
-    virtual StatusWith<std::vector<std::string>> extendBackupCursor(OperationContext* opCtx) {
-        return Status(ErrorCodes::CommandNotSupported,
-                      "The current storage engine does not support a concurrent mode.");
-    }
+    virtual StatusWith<std::vector<std::string>> extendBackupCursor(OperationContext* opCtx) = 0;
 
     /**
      * Recover as much data as possible from a potentially corrupt RecordStore.
      * This only recovers the record data, not indexes or anything else.
      *
-     * Generally, this method should not be called directly except by the repairDatabase()
-     * free function.
+     * The Collection object for on this namespace will be destructed and invalidated. A new
+     * Collection object will be created and it should be retrieved from the CollectionCatalog.
      */
-    virtual Status repairRecordStore(OperationContext* opCtx, const NamespaceString& nss) = 0;
+    virtual Status repairRecordStore(OperationContext* opCtx,
+                                     RecordId catalogId,
+                                     const NamespaceString& nss) = 0;
 
     /**
      * Creates a temporary RecordStore on the storage engine. This record store will drop itself
@@ -386,13 +370,13 @@ public:
      *
      * Pointer remains owned by the StorageEngine, not the caller.
      */
-    virtual SnapshotManager* getSnapshotManager() const {
-        return nullptr;
-    }
+    virtual SnapshotManager* getSnapshotManager() const = 0;
 
     /**
      * Sets a new JournalListener, which is used by the storage engine to alert the rest of the
      * system about journaled write progress.
+     *
+     * This may only be set once.
      */
     virtual void setJournalListener(JournalListener* jl) = 0;
 
@@ -402,27 +386,26 @@ public:
      * a stable timestamp. In that case StorageEngine::recoverToStableTimestamp() will return
      * a bad status.
      */
-    virtual bool supportsRecoverToStableTimestamp() const {
-        return false;
-    }
+    virtual bool supportsRecoverToStableTimestamp() const = 0;
 
     /**
      * Returns whether the storage engine can provide a recovery timestamp.
      */
-    virtual bool supportsRecoveryTimestamp() const {
-        return false;
-    }
+    virtual bool supportsRecoveryTimestamp() const = 0;
 
     /**
      * Returns true if the storage engine supports the readConcern level "snapshot".
      */
-    virtual bool supportsReadConcernSnapshot() const {
-        return false;
-    }
+    virtual bool supportsReadConcernSnapshot() const = 0;
 
-    virtual bool supportsReadConcernMajority() const {
-        return false;
-    }
+    virtual bool supportsReadConcernMajority() const = 0;
+
+    /**
+     * Returns true if the storage engine uses oplog stones to more finely control
+     * deletion of oplog history, instead of the standard capped collection controls on
+     * the oplog collection size.
+     */
+    virtual bool supportsOplogStones() const = 0;
 
     /**
      * Returns true if the storage engine supports deferring collection drops until the the storage
@@ -442,6 +425,11 @@ public:
      */
     virtual void clearDropPendingState() = 0;
 
+    /**
+     * Returns true if the storage engine supports two phase index builds.
+     */
+    virtual bool supportsTwoPhaseIndexBuild() const = 0;
+
     /**
      * Recovers the storage engine state to the last stable timestamp. "Stable" in this case
      * refers to a timestamp that is guaranteed to never be rolled back. The stable timestamp
@@ -458,18 +446,14 @@ public:
      * It is illegal to call this concurrently with `setStableTimestamp` or
      * `setInitialDataTimestamp`.
      */
-    virtual StatusWith<Timestamp> recoverToStableTimestamp(OperationContext* opCtx) {
-        fassertFailed(40547);
-    }
+    virtual StatusWith<Timestamp> recoverToStableTimestamp(OperationContext* opCtx) = 0;
 
     /**
      * Returns the stable timestamp that the storage engine recovered to on startup. If the
      * recovery point was not stable, returns "none".
      * fasserts if StorageEngine::supportsRecoverToStableTimestamp() would return false.
      */
-    virtual boost::optional<Timestamp> getRecoveryTimestamp() const {
-        MONGO_UNREACHABLE;
-    }
+    virtual boost::optional<Timestamp> getRecoveryTimestamp() const = 0;
 
     /**
      * Returns a timestamp that is guaranteed to exist on storage engine recovery to a stable
@@ -481,27 +465,20 @@ public:
      * boost::none if the recovery time has not yet been established. Replication recoverable
      * rollback may not succeed before establishment, and restart will require resync.
      */
-    virtual boost::optional<Timestamp> getLastStableRecoveryTimestamp() const {
-        MONGO_UNREACHABLE;
-    }
+    virtual boost::optional<Timestamp> getLastStableRecoveryTimestamp() const = 0;
 
     /**
      * Sets the highest timestamp at which the storage engine is allowed to take a checkpoint. This
      * timestamp must not decrease unless force=true is set, in which case we force the stable
      * timestamp, the oldest timestamp, and the commit timestamp backward.
      */
-    virtual void setStableTimestamp(Timestamp stableTimestamp, bool force = false) {}
-
-    /**
-     * Returns the stable timestamp.
-     */
-    virtual Timestamp getStableTimestamp() const = 0;
+    virtual void setStableTimestamp(Timestamp stableTimestamp, bool force = false) = 0;
 
     /**
      * Tells the storage engine the timestamp of the data at startup. This is necessary because
      * timestamps are not persisted in the storage layer.
      */
-    virtual void setInitialDataTimestamp(Timestamp timestamp) {}
+    virtual void setInitialDataTimestamp(Timestamp timestamp) = 0;
 
     /**
      * Returns the initial data timestamp.
@@ -518,13 +495,13 @@ public:
      * of the oplog read timestamp, ensuring the oplog reader's 'read_timestamp' can always be
      * serviced.
      */
-    virtual void setOldestTimestampFromStable() {}
+    virtual void setOldestTimestampFromStable() = 0;
 
     /**
      * Sets the oldest timestamp for which the storage engine must maintain snapshot history
      * through. Additionally, all future writes must be newer or equal to this value.
      */
-    virtual void setOldestTimestamp(Timestamp timestamp) {}
+    virtual void setOldestTimestamp(Timestamp timestamp) = 0;
 
     /**
      * Gets the oldest timestamp for which the storage engine must maintain snapshot history
@@ -538,7 +515,7 @@ public:
      * oplog it must preserve.
      */
     virtual void setOldestActiveTransactionTimestampCallback(
-        OldestActiveTransactionTimestampCallback callback){};
+        OldestActiveTransactionTimestampCallback callback) = 0;
 
     /**
      * Indicates whether the storage engine cache is under pressure.
@@ -547,46 +524,75 @@ public:
      * it against storageGlobalParams.cachePressureThreshold, a dynamic server parameter, to
      * determine whether cache pressure is too high.
      */
-    virtual bool isCacheUnderPressure(OperationContext* opCtx) const {
-        return false;
-    }
+    virtual bool isCacheUnderPressure(OperationContext* opCtx) const = 0;
 
     /**
      * For unit tests only. Sets the cache pressure value with which isCacheUnderPressure()
      * evalutates to 'pressure'.
      */
-    virtual void setCachePressureForTest(int pressure) {}
+    virtual void setCachePressureForTest(int pressure) = 0;
 
     /**
-     *  Notifies the storage engine that a replication batch has completed.
-     *  This means that all the writes associated with the oplog entries in the batch are
-     *  finished and no new writes with timestamps associated with those oplog entries will show
-     *  up in the future.
-     *  This function can be used to ensure oplog visibility rules are not broken, for example.
+     * Prompts an immediate journal flush.
      */
-    virtual void replicationBatchIsComplete() const {};
-
-    // (CollectionName, IndexName)
-    typedef std::pair<std::string, std::string> CollectionIndexNamePair;
+    virtual void triggerJournalFlush() const = 0;
 
     /**
-     * Drop abandoned idents. In the successful case, returns a list of collection, index name
-     * pairs to rebuild.
+     * Initiates if needed and waits for a complete round of journal flushing to execute.
+     *
+     * Can throw ShutdownInProgress if the storage engine is being closed.
      */
-    virtual StatusWith<std::vector<CollectionIndexNamePair>> reconcileCatalogAndIdents(
-        OperationContext* opCtx) {
-        return std::vector<CollectionIndexNamePair>();
+    virtual void waitForJournalFlush(OperationContext* opCtx) const = 0;
+
+    /**
+     * Ensures interruption of the JournalFlusher if it is or will be acquiring a lock.
+     *
+     * TODO: this function will be moved above the Storage Engine layer along with the
+     * JournalFlusher in SERVER-45847.
+     */
+    virtual void interruptJournalFlusherForReplStateChange() const = 0;
+
+    struct IndexIdentifier {
+        const RecordId catalogId;
+        const NamespaceString nss;
+        const std::string indexName;
     };
 
+    /*
+     * ReconcileResult is the result of reconciling abandoned storage engine idents and unfinished
+     * index builds.
+     */
+    struct ReconcileResult {
+        // A list of IndexIdentifiers that must be rebuilt to completion.
+        std::vector<IndexIdentifier> indexesToRebuild;
+
+        // A map of unfinished two-phase indexes that must be restarted in the background, but
+        // not to completion; they will wait for replicated commit or abort operations. This is a
+        // mapping from index build UUID to index build.
+        IndexBuilds indexBuildsToRestart;
+    };
+
+    /**
+     * Drop abandoned idents. If successful, returns a ReconcileResult with indexes that need to be
+     * rebuilt or builds that need to be restarted.
+     * */
+    virtual StatusWith<ReconcileResult> reconcileCatalogAndIdents(OperationContext* opCtx) = 0;
+
     /**
      * Returns the all_durable timestamp. All transactions with timestamps earlier than the
      * all_durable timestamp are committed. Only storage engines that support document level locking
      * must provide an implementation. Other storage engines may provide a no-op implementation.
      *
+     * The all_durable timestamp is the in-memory no holes point. That does not mean that there are
+     * no holes behind it on disk. The all_durable timestamp also might not correspond with any
+     * oplog entry, but instead have a timestamp value between that of two oplog entries.
+     *
      * The all_durable timestamp only includes non-prepared transactions that have been given a
      * commit_timestamp and prepared transactions that have been given a durable_timestamp.
      * Previously, the deprecated all_committed timestamp would also include prepared transactions
      * that were prepared but not committed which could make the stable timestamp briefly jump back.
+     *
+     * Returns kMinimumTimestamp if there have been no new writes since the storage engine started.
      */
     virtual Timestamp getAllDurableTimestamp() const = 0;
 
