@@ -27,9 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
-#define LOG_FOR_RECOVERY(level) \
-    MONGO_LOG_COMPONENT(level, ::mongo::logger::LogComponent::kStorageRecovery)
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/db/storage/storage_engine_impl.h"
 
@@ -40,6 +38,7 @@
 #include "mongo/db/catalog/collection_catalog_helper.h"
 #include "mongo/db/client.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/logical_clock.h"
 #include "mongo/db/operation_context_noop.h"
 #include "mongo/db/server_options.h"
@@ -47,13 +46,17 @@
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/kv/temporary_kv_record_store.h"
 #include "mongo/db/storage/storage_repair_observer.h"
+#include "mongo/db/storage/two_phase_index_build_knobs_gen.h"
 #include "mongo/db/unclean_shutdown.h"
+#include "mongo/logv2/log.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
 
+#define LOGV2_FOR_RECOVERY(ID, DLEVEL, MESSAGE, ...) \
+    LOGV2_DEBUG_OPTIONS(ID, DLEVEL, {logv2::LogComponent::kStorageRecovery}, MESSAGE, ##__VA_ARGS__)
+
 namespace mongo {
 
 using std::string;
@@ -61,7 +64,7 @@ using std::vector;
 
 namespace {
 const std::string catalogInfo = "_mdb_catalog";
-const auto kCatalogLogLevel = logger::LogSeverity::Debug(2);
+const auto kCatalogLogLevel = logv2::LogSeverity::Debug(2);
 }  // namespace
 
 Status StorageEngineImpl::hotBackup(OperationContext* opCtx, const std::string& path) {
@@ -104,11 +107,11 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
         auto repairObserver = StorageRepairObserver::get(getGlobalServiceContext());
         invariant(repairObserver->isIncomplete());
 
-        log() << "Repairing catalog metadata";
+        LOGV2(22246, "Repairing catalog metadata");
         Status status = _engine->repairIdent(opCtx, catalogInfo);
 
         if (status.code() == ErrorCodes::DataModifiedByRepair) {
-            warning() << "Catalog data modified by repair: " << status.reason();
+            LOGV2_WARNING(22264, "Catalog data modified by repair", "error"_attr = status);
             repairObserver->invalidatingModification(str::stream() << "DurableCatalog repaired: "
                                                                    << status.reason());
         } else {
@@ -133,8 +136,8 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
 
     _catalogRecordStore = _engine->getGroupedRecordStore(
         opCtx, catalogInfo, catalogInfo, CollectionOptions(), KVPrefix::kNotPrefixed);
-    if (shouldLog(::mongo::logger::LogComponent::kStorageRecovery, kCatalogLogLevel)) {
-        LOG_FOR_RECOVERY(kCatalogLogLevel) << "loadCatalog:";
+    if (shouldLog(::mongo::logv2::LogComponent::kStorageRecovery, kCatalogLogLevel)) {
+        LOGV2_FOR_RECOVERY(4615631, kCatalogLogLevel.toInt(), "loadCatalog:");
         _dumpCatalog(opCtx);
     }
 
@@ -153,8 +156,7 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
         std::sort(identsKnownToStorageEngine.begin(), identsKnownToStorageEngine.end());
     }
 
-    auto collectionsKnownToCatalog = _catalog->getAllCollections();
-
+    std::vector<DurableCatalog::Entry> catalogEntries = _catalog->getAllCatalogEntries(opCtx);
     if (_options.forRepair) {
         // It's possible that there are collection files on disk that are unknown to the catalog. In
         // a repair context, if we can't find an ident in the catalog, we generate a catalog entry
@@ -162,12 +164,10 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
         // will be dropped in reconcileCatalogAndIdents().
         for (const auto& ident : identsKnownToStorageEngine) {
             if (_catalog->isCollectionIdent(ident)) {
-                bool isOrphan = !std::any_of(collectionsKnownToCatalog.begin(),
-                                             collectionsKnownToCatalog.end(),
-                                             [this, &ident](const auto& coll) {
-                                                 return _catalog->getCollectionIdent(
-                                                            NamespaceString(coll)) == ident;
-                                             });
+                bool isOrphan = !std::any_of(
+                    catalogEntries.begin(),
+                    catalogEntries.end(),
+                    [this, &ident](DurableCatalog::Entry entry) { return entry.ident == ident; });
                 if (isOrphan) {
                     // If the catalog does not have information about this
                     // collection, we create an new entry for it.
@@ -176,12 +176,14 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
                     if (statusWithNs.isOK()) {
                         wuow.commit();
                         auto orphanCollNs = statusWithNs.getValue();
-                        log() << "Successfully created an entry in the catalog for the orphaned "
-                                 "collection: "
-                              << orphanCollNs;
-                        warning() << orphanCollNs
-                                  << " does not have the _id index. Please manually "
-                                     "build the index.";
+                        LOGV2(22247,
+                              "Successfully created an entry in the catalog for orphaned "
+                              "collection",
+                              "namespace"_attr = orphanCollNs);
+                        LOGV2_WARNING(22265,
+                                      "Collection does not have an _id index. Please manually "
+                                      "build the index",
+                                      "namespace"_attr = orphanCollNs);
 
                         StorageRepairObserver::get(getGlobalServiceContext())
                             ->benignModification(str::stream() << "Orphan collection created: "
@@ -190,10 +192,15 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
                     } else {
                         // Log an error message if we cannot create the entry.
                         // reconcileCatalogAndIdents() will later drop this ident.
-                        error() << "Cannot create an entry in the catalog for the orphaned "
-                                   "collection ident: "
-                                << ident << " due to " << statusWithNs.getStatus().reason();
-                        error() << "Restarting the server will remove this ident.";
+                        LOGV2_ERROR(
+                            22268,
+                            "Cannot create an entry in the catalog for the orphaned "
+                            "collection ident: {ident} due to {statusWithNs_getStatus_reason}. "
+                            "Restarting the server will remove this ident",
+                            "Cannot create an entry in the catalog for orphaned ident. Restarting "
+                            "the server will remove this ident",
+                            "ident"_attr = ident,
+                            "error"_attr = statusWithNs.getStatus());
                     }
                 }
             }
@@ -201,15 +208,13 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
     }
 
     KVPrefix maxSeenPrefix = KVPrefix::kNotPrefixed;
-    for (const auto& nss : collectionsKnownToCatalog) {
-        std::string dbName = nss.db().toString();
-
+    for (DurableCatalog::Entry entry : catalogEntries) {
         if (loadingFromUncleanShutdownOrRepair) {
             // If we are loading the catalog after an unclean shutdown or during repair, it's
             // possible that there are collections in the catalog that are unknown to the storage
             // engine. If we can't find a table in the list of storage engine idents, either
             // attempt to recover the ident or drop it.
-            const auto collectionIdent = _catalog->getCollectionIdent(nss);
+            const auto collectionIdent = entry.ident;
             bool orphan = !std::binary_search(identsKnownToStorageEngine.begin(),
                                               identsKnownToStorageEngine.end(),
                                               collectionIdent);
@@ -217,17 +222,22 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
             // store, drop it from the catalog and skip initializing it by continuing past the
             // following logic.
             if (orphan) {
-                auto status = _recoverOrphanedCollection(opCtx, nss, collectionIdent);
+                auto status =
+                    _recoverOrphanedCollection(opCtx, entry.catalogId, entry.nss, collectionIdent);
                 if (!status.isOK()) {
-                    warning() << "Failed to recover orphaned data file for collection '" << nss
-                              << "': " << status;
+                    LOGV2_WARNING(22266,
+                                  "Failed to recover orphaned data file for collection "
+                                  "'{namespace}': {error}",
+                                  "Failed to recover orphaned data file for collection",
+                                  "namespace"_attr = entry.nss,
+                                  "error"_attr = status);
                     WriteUnitOfWork wuow(opCtx);
-                    fassert(50716, _catalog->_removeEntry(opCtx, nss));
+                    fassert(50716, _catalog->_removeEntry(opCtx, entry.catalogId));
 
                     if (_options.forRepair) {
                         StorageRepairObserver::get(getGlobalServiceContext())
                             ->invalidatingModification(str::stream()
-                                                       << "Collection " << nss
+                                                       << "Collection " << entry.nss
                                                        << " dropped: " << status.reason());
                     }
                     wuow.commit();
@@ -236,12 +246,15 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
             }
         }
 
-        _initCollection(opCtx, nss, _options.forRepair);
-        auto maxPrefixForCollection = _catalog->getMetaData(opCtx, nss).getMaxPrefix();
+        _initCollection(opCtx, entry.catalogId, entry.nss, _options.forRepair);
+        auto maxPrefixForCollection = _catalog->getMetaData(opCtx, entry.catalogId).getMaxPrefix();
         maxSeenPrefix = std::max(maxSeenPrefix, maxPrefixForCollection);
 
-        if (nss.isOrphanCollection()) {
-            log() << "Orphaned collection found: " << nss;
+        if (entry.nss.isOrphanCollection()) {
+            LOGV2(22248,
+                  "Orphaned collection found: {namespace}",
+                  "Orphaned collection found",
+                  "namespace"_attr = entry.nss);
         }
     }
 
@@ -254,14 +267,15 @@ void StorageEngineImpl::loadCatalog(OperationContext* opCtx) {
 }
 
 void StorageEngineImpl::_initCollection(OperationContext* opCtx,
+                                        RecordId catalogId,
                                         const NamespaceString& nss,
                                         bool forRepair) {
-    BSONCollectionCatalogEntry::MetaData md = _catalog->getMetaData(opCtx, nss);
+    BSONCollectionCatalogEntry::MetaData md = _catalog->getMetaData(opCtx, catalogId);
     uassert(ErrorCodes::MustDowngrade,
             str::stream() << "Collection does not have UUID in KVCatalog. Collection: " << nss,
             md.options.uuid);
 
-    auto ident = _catalog->getCollectionIdent(nss);
+    auto ident = _catalog->getEntry(catalogId).ident;
 
     std::unique_ptr<RecordStore> rs;
     if (forRepair) {
@@ -273,19 +287,19 @@ void StorageEngineImpl::_initCollection(OperationContext* opCtx,
         invariant(rs);
     }
 
-    auto uuid = _catalog->getCollectionOptions(opCtx, nss).uuid.get();
+    auto uuid = _catalog->getCollectionOptions(opCtx, catalogId).uuid.get();
 
     auto collectionFactory = Collection::Factory::get(getGlobalServiceContext());
-    auto collection = collectionFactory->make(opCtx, nss, uuid, std::move(rs));
+    auto collection = collectionFactory->make(opCtx, nss, catalogId, uuid, std::move(rs));
 
     auto& collectionCatalog = CollectionCatalog::get(getGlobalServiceContext());
-    collectionCatalog.registerCollection(uuid, std::move(collection));
+    collectionCatalog.registerCollection(uuid, &collection);
 }
 
 void StorageEngineImpl::closeCatalog(OperationContext* opCtx) {
     dassert(opCtx->lockState()->isLocked());
-    if (shouldLog(::mongo::logger::LogComponent::kStorageRecovery, kCatalogLogLevel)) {
-        LOG_FOR_RECOVERY(kCatalogLogLevel) << "loadCatalog:";
+    if (shouldLog(::mongo::logv2::LogComponent::kStorageRecovery, kCatalogLogLevel)) {
+        LOGV2_FOR_RECOVERY(4615632, kCatalogLogLevel.toInt(), "loadCatalog:");
         _dumpCatalog(opCtx);
     }
     CollectionCatalog::get(opCtx).deregisterAllCollections();
@@ -295,21 +309,25 @@ void StorageEngineImpl::closeCatalog(OperationContext* opCtx) {
 }
 
 Status StorageEngineImpl::_recoverOrphanedCollection(OperationContext* opCtx,
+                                                     RecordId catalogId,
                                                      const NamespaceString& collectionName,
                                                      StringData collectionIdent) {
     if (!_options.forRepair) {
         return {ErrorCodes::IllegalOperation, "Orphan recovery only supported in repair"};
     }
-    log() << "Storage engine is missing collection '" << collectionName
-          << "' from its metadata. Attempting to locate and recover the data for "
-          << collectionIdent;
+    LOGV2(22249,
+          "Storage engine is missing collection '{namespace}' from its metadata. Attempting "
+          "to locate and recover the data for {ident}",
+          "Storage engine is missing collection from its metadata. Attempting to locate and "
+          "recover the data",
+          "namespace"_attr = collectionName,
+          "ident"_attr = collectionIdent);
 
     WriteUnitOfWork wuow(opCtx);
-    const auto metadata = _catalog->getMetaData(opCtx, collectionName);
-    auto status =
+    const auto metadata = _catalog->getMetaData(opCtx, catalogId);
+    Status status =
         _engine->recoverOrphanedIdent(opCtx, collectionName, collectionIdent, metadata.options);
 
-
     bool dataModified = status.code() == ErrorCodes::DataModifiedByRepair;
     if (!status.isOK() && !dataModified) {
         return status;
@@ -336,8 +354,8 @@ Status StorageEngineImpl::_recoverOrphanedCollection(OperationContext* opCtx,
  * Third, a DurableCatalog may have an index ident that the KVEngine does not. This method will
  * rebuild the index.
  */
-StatusWith<std::vector<StorageEngine::CollectionIndexNamePair>>
-StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
+StatusWith<StorageEngine::ReconcileResult> StorageEngineImpl::reconcileCatalogAndIdents(
+    OperationContext* opCtx) {
     // Gather all tables known to the storage engine and drop those that aren't cross-referenced
     // in the _mdb_catalog. This can happen for two reasons.
     //
@@ -357,7 +375,7 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
         engineIdents.erase(catalogInfo);
     }
 
-    LOG_FOR_RECOVERY(2) << "Reconciling collection and index idents.";
+    LOGV2_FOR_RECOVERY(4615633, 2, "Reconciling collection and index idents.");
     std::set<std::string> catalogIdents;
     {
         std::vector<std::string> vec = _catalog->getAllIdents(opCtx);
@@ -393,14 +411,17 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
         // These idents have to be retained as long as the corresponding drops are not part of a
         // checkpoint.
         if (dropPendingIdents.find(it) != dropPendingIdents.cend()) {
-            log() << "Not removing ident for uncheckpointed collection or index drop: " << it;
+            LOGV2(22250,
+                  "Not removing ident for uncheckpointed collection or index drop: {ident}",
+                  "Not removing ident for uncheckpointed collection or index drop",
+                  "ident"_attr = it);
             continue;
         }
 
         const auto& toRemove = it;
-        log() << "Dropping unknown ident: " << toRemove;
+        LOGV2(22251, "Dropping unknown ident", "ident"_attr = toRemove);
         WriteUnitOfWork wuow(opCtx);
-        fassert(40591, _engine->dropIdent(opCtx, toRemove));
+        fassert(40591, _engine->dropIdent(opCtx, opCtx->recoveryUnit(), toRemove));
         wuow.commit();
     }
 
@@ -408,14 +429,13 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
     // engine. An omission here is fatal. A missing ident could mean a collection drop was rolled
     // back. Note that startup already attempts to open tables; this should only catch errors in
     // other contexts such as `recoverToStableTimestamp`.
-    auto collections = _catalog->getAllCollections();
+    std::vector<DurableCatalog::Entry> catalogEntries = _catalog->getAllCatalogEntries(opCtx);
     if (!_options.forRepair) {
-        for (const auto& coll : collections) {
-            const auto& identForColl = _catalog->getCollectionIdent(coll);
-            if (engineIdents.find(identForColl) == engineIdents.end()) {
+        for (DurableCatalog::Entry entry : catalogEntries) {
+            if (engineIdents.find(entry.ident) == engineIdents.end()) {
                 return {ErrorCodes::UnrecoverableRollbackError,
-                        str::stream() << "Expected collection does not exist. Collection: " << coll
-                                      << " Ident: " << identForColl};
+                        str::stream() << "Expected collection does not exist. Collection: "
+                                      << entry.nss << " Ident: " << entry.ident};
             }
         }
     }
@@ -425,15 +445,40 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
     //
     // Also, remove unfinished builds except those that were background index builds started on a
     // secondary.
-    std::vector<CollectionIndexNamePair> ret;
-    for (const auto& coll : collections) {
-        BSONCollectionCatalogEntry::MetaData metaData = _catalog->getMetaData(opCtx, coll);
+    StorageEngine::ReconcileResult ret;
+    for (DurableCatalog::Entry entry : catalogEntries) {
+        BSONCollectionCatalogEntry::MetaData metaData =
+            _catalog->getMetaData(opCtx, entry.catalogId);
+        NamespaceString coll(metaData.ns);
 
         // Batch up the indexes to remove them from `metaData` outside of the iterator.
         std::vector<std::string> indexesToDrop;
         for (const auto& indexMetaData : metaData.indexes) {
             const std::string& indexName = indexMetaData.name();
-            std::string indexIdent = _catalog->getIndexIdent(opCtx, coll, indexName);
+            std::string indexIdent = _catalog->getIndexIdent(opCtx, entry.catalogId, indexName);
+
+            // Warn in case of incorrect "multikeyPath" information in catalog documents. This is
+            // the result of a concurrency bug which has since been fixed, but may persist in
+            // certain catalog documents. See https://jira.mongodb.org/browse/SERVER-43074
+            const bool hasMultiKeyPaths =
+                std::any_of(indexMetaData.multikeyPaths.begin(),
+                            indexMetaData.multikeyPaths.end(),
+                            [](auto& pathSet) { return pathSet.size() > 0; });
+            if (!indexMetaData.multikey && hasMultiKeyPaths) {
+                LOGV2_WARNING(
+                    22267,
+                    "The 'multikey' field for index {index} on collection {namespace} was "
+                    "false with non-empty 'multikeyPaths'. This indicates corruption of "
+                    "the catalog. Consider either dropping and recreating the index, or "
+                    "rerunning with the --repair option. See "
+                    "http://dochub.mongodb.org/core/repair for more information.",
+                    "The 'multikey' field for index was false with non-empty 'multikeyPaths'. This "
+                    "indicates corruption of the catalog. Consider either dropping and recreating "
+                    "the index, or rerunning with the --repair option. See "
+                    "http://dochub.mongodb.org/core/repair for more information",
+                    "index"_attr = indexName,
+                    "namespace"_attr = coll);
+            }
 
             const bool foundIdent = engineIdents.find(indexIdent) != engineIdents.end();
             // An index drop will immediately remove the ident, but the `indexMetaData` catalog
@@ -441,35 +486,44 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
             // majority of nodes. The code will rebuild the index, despite potentially
             // encountering another `dropIndex` command.
             if (indexMetaData.ready && !foundIdent) {
-                log() << "Expected index data is missing, rebuilding. Collection: " << coll
-                      << " Index: " << indexName;
-                ret.emplace_back(coll.ns(), indexName);
+                LOGV2(22252,
+                      "Expected index data is missing, rebuilding. Collection: {namespace} Index: "
+                      "{index}",
+                      "Expected index data is missing, rebuilding",
+                      "index"_attr = indexName,
+                      "namespace"_attr = coll);
+                ret.indexesToRebuild.push_back({entry.catalogId, coll, indexName});
                 continue;
             }
 
-            // If this index was draining, do not delete any internal idents that it may have owned.
-            // Instead, the idents can be used later on to resume draining instead of a
-            // performing a full rebuild. This is only done for background secondary builds, because
-            // the index must be rebuilt, and it is dropped otherwise.
-            // TODO: SERVER-37952 Do not drop these idents for background index builds on
-            // primaries once index builds are resumable from draining.
-            if (!indexMetaData.ready && indexMetaData.isBackgroundSecondaryBuild &&
-                indexMetaData.buildPhase ==
-                    BSONCollectionCatalogEntry::kIndexBuildDraining.toString()) {
+            // Any index build with a UUID is an unfinished two-phase build and must be restarted.
+            // There are no special cases to handle on primaries or secondaries. An index build may
+            // be associated with multiple indexes.
+            if (indexMetaData.buildUUID) {
+                invariant(!indexMetaData.ready);
 
-                if (indexMetaData.constraintViolationsIdent) {
-                    auto it = internalIdentsToDrop.find(*indexMetaData.constraintViolationsIdent);
-                    if (it != internalIdentsToDrop.end()) {
-                        internalIdentsToDrop.erase(it);
-                    }
+                auto collUUID = metaData.options.uuid;
+                invariant(collUUID);
+                auto buildUUID = *indexMetaData.buildUUID;
+
+                LOGV2(22253,
+                      "Found index from unfinished build. Collection: {coll} ({uuid}), index: "
+                      "{indexName}, build UUID: {buildUUID}",
+                      "Found index from unfinished build",
+                      "namespace"_attr = coll,
+                      "uuid"_attr = *collUUID,
+                      "index"_attr = indexName,
+                      "buildUUID"_attr = buildUUID);
+
+                // Insert in the map if a build has not already been registered.
+                auto existingIt = ret.indexBuildsToRestart.find(buildUUID);
+                if (existingIt == ret.indexBuildsToRestart.end()) {
+                    ret.indexBuildsToRestart.insert({buildUUID, IndexBuildDetails(*collUUID)});
+                    existingIt = ret.indexBuildsToRestart.find(buildUUID);
                 }
 
-                if (indexMetaData.sideWritesIdent) {
-                    auto it = internalIdentsToDrop.find(*indexMetaData.sideWritesIdent);
-                    if (it != internalIdentsToDrop.end()) {
-                        internalIdentsToDrop.erase(it);
-                    }
-                }
+                existingIt->second.indexSpecs.emplace_back(indexMetaData.spec);
+                continue;
             }
 
             // If the index was kicked off as a background secondary index build, replication
@@ -477,19 +531,12 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
             // table is not found, or the index build did not successfully complete, this code
             // will return the index to be rebuilt.
             if (indexMetaData.isBackgroundSecondaryBuild && (!foundIdent || !indexMetaData.ready)) {
-                if (!serverGlobalParams.indexBuildRetry) {
-                    log() << "Dropping an unfinished index because --noIndexBuildRetry is set. "
-                             "Collection: "
-                          << coll << " Index: " << indexName;
-                    fassert(51197, _engine->dropIdent(opCtx, indexIdent));
-                    indexesToDrop.push_back(indexName);
-                    continue;
-                }
-
-                log() << "Expected background index build did not complete, rebuilding in  "
-                         "foreground - see SERVER-43097. Collection: "
-                      << coll << " Index: " << indexName;
-                ret.emplace_back(coll.ns(), indexName);
+                LOGV2(22255,
+                      "Expected background index build did not complete, rebuilding in foreground "
+                      "- see SERVER-43097",
+                      "namespace"_attr = coll,
+                      "index"_attr = indexName);
+                ret.indexesToRebuild.push_back({entry.catalogId, coll, indexName});
                 continue;
             }
 
@@ -500,10 +547,12 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
             // index when it replays the oplog. In these cases the index entry in the catalog
             // should be dropped.
             if (!indexMetaData.ready && !indexMetaData.isBackgroundSecondaryBuild) {
-                log() << "Dropping unfinished index. Collection: " << coll
-                      << " Index: " << indexName;
+                LOGV2(22256,
+                      "Dropping unfinished index",
+                      "namespace"_attr = coll,
+                      "index"_attr = indexName);
                 // Ensure the `ident` is dropped while we have the `indexIdent` value.
-                fassert(50713, _engine->dropIdent(opCtx, indexIdent));
+                fassert(50713, _engine->dropIdent(opCtx, opCtx->recoveryUnit(), indexIdent));
                 indexesToDrop.push_back(indexName);
                 continue;
             }
@@ -516,15 +565,15 @@ StorageEngineImpl::reconcileCatalogAndIdents(OperationContext* opCtx) {
         }
         if (indexesToDrop.size() > 0) {
             WriteUnitOfWork wuow(opCtx);
-            _catalog->putMetaData(opCtx, coll, metaData);
+            _catalog->putMetaData(opCtx, entry.catalogId, metaData);
             wuow.commit();
         }
     }
 
     for (auto&& temp : internalIdentsToDrop) {
-        log() << "Dropping internal ident: " << temp;
+        LOGV2(22257, "Dropping internal ident", "ident"_attr = temp);
         WriteUnitOfWork wuow(opCtx);
-        fassert(51067, _engine->dropIdent(opCtx, temp));
+        fassert(51067, _engine->dropIdent(opCtx, opCtx->recoveryUnit(), temp));
         wuow.commit();
     }
 
@@ -554,6 +603,11 @@ void StorageEngineImpl::cleanShutdown() {
 StorageEngineImpl::~StorageEngineImpl() {}
 
 void StorageEngineImpl::finishInit() {
+    // A storage engine may need to start threads that require OperationsContexts with real Lockers,
+    // as opposed to LockerNoops. Placing the start logic here, after the StorageEngine has been
+    // instantiated, causes makeOperationContext() to create LockerImpls instead of LockerNoops.
+    _engine->startAsyncThreads();
+
     if (_engine->supportsRecoveryTimestamp()) {
         _timestampMonitor = std::make_unique<TimestampMonitor>(
             _engine.get(), getGlobalServiceContext()->getPeriodicRunner());
@@ -562,10 +616,6 @@ void StorageEngineImpl::finishInit() {
     }
 }
 
-void StorageEngineImpl::notifyStartupComplete() {
-    _engine->notifyStartupComplete();
-}
-
 RecoveryUnit* StorageEngineImpl::newRecoveryUnit() {
     if (!_engine) {
         // shutdown
@@ -636,25 +686,25 @@ Status StorageEngineImpl::_dropCollectionsNoTimestamp(OperationContext* opCtx,
     WriteUnitOfWork untimestampedDropWuow(opCtx);
     for (auto& nss : toDrop) {
         invariant(getCatalog());
-        auto uuid = CollectionCatalog::get(opCtx).lookupUUIDByNSS(nss).get();
-        Status result = getCatalog()->dropCollection(opCtx, nss);
+        auto coll = CollectionCatalog::get(opCtx).lookupCollectionByNamespace(opCtx, nss);
+        Status result = getCatalog()->dropCollection(opCtx, coll->getCatalogId());
 
         if (!result.isOK() && firstError.isOK()) {
             firstError = result;
         }
 
-        auto removedColl = CollectionCatalog::get(opCtx).deregisterCollection(uuid);
+        auto removedColl = CollectionCatalog::get(opCtx).deregisterCollection(coll->uuid());
         opCtx->recoveryUnit()->registerChange(
             CollectionCatalog::get(opCtx).makeFinishDropCollectionChange(std::move(removedColl),
-                                                                         uuid));
+                                                                         coll->uuid()));
     }
 
     untimestampedDropWuow.commit();
     return firstError;
 }
 
-int StorageEngineImpl::flushAllFiles(OperationContext* opCtx, bool sync) {
-    return _engine->flushAllFiles(opCtx, sync);
+void StorageEngineImpl::flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) {
+    _engine->flushAllFiles(opCtx, callerHoldsReadLock);
 }
 
 Status StorageEngineImpl::beginBackup(OperationContext* opCtx) {
@@ -692,6 +742,10 @@ StatusWith<std::vector<std::string>> StorageEngineImpl::extendBackupCursor(
     return _engine->extendBackupCursor(opCtx);
 }
 
+bool StorageEngineImpl::supportsCheckpoints() const {
+    return _engine->supportsCheckpoints();
+}
+
 bool StorageEngineImpl::isDurable() const {
     return _engine->isDurable();
 }
@@ -704,11 +758,13 @@ SnapshotManager* StorageEngineImpl::getSnapshotManager() const {
     return _engine->getSnapshotManager();
 }
 
-Status StorageEngineImpl::repairRecordStore(OperationContext* opCtx, const NamespaceString& nss) {
+Status StorageEngineImpl::repairRecordStore(OperationContext* opCtx,
+                                            RecordId catalogId,
+                                            const NamespaceString& nss) {
     auto repairObserver = StorageRepairObserver::get(getGlobalServiceContext());
     invariant(repairObserver->isIncomplete());
 
-    Status status = _engine->repairIdent(opCtx, _catalog->getCollectionIdent(nss));
+    Status status = _engine->repairIdent(opCtx, _catalog->getEntry(catalogId).ident);
     bool dataModified = status.code() == ErrorCodes::DataModifiedByRepair;
     if (!status.isOK() && !dataModified) {
         return status;
@@ -721,17 +777,21 @@ Status StorageEngineImpl::repairRecordStore(OperationContext* opCtx, const Names
 
     // After repairing, re-initialize the collection with a valid RecordStore.
     auto& collectionCatalog = CollectionCatalog::get(getGlobalServiceContext());
-    auto uuid = collectionCatalog.lookupUUIDByNSS(nss).get();
+    auto uuid = collectionCatalog.lookupUUIDByNSS(opCtx, nss).get();
     collectionCatalog.deregisterCollection(uuid);
-    _initCollection(opCtx, nss, false);
-    return Status::OK();
+    _initCollection(opCtx, catalogId, nss, false);
+
+    return status;
 }
 
 std::unique_ptr<TemporaryRecordStore> StorageEngineImpl::makeTemporaryRecordStore(
     OperationContext* opCtx) {
     std::unique_ptr<RecordStore> rs =
         _engine->makeTemporaryRecordStore(opCtx, _catalog->newInternalIdent());
-    LOG(1) << "created temporary record store: " << rs->getIdent();
+    LOGV2_DEBUG(22258,
+                1,
+                "created temporary record store: {rs_getIdent}",
+                "rs_getIdent"_attr = rs->getIdent());
     return std::make_unique<TemporaryKVRecordStore>(getEngine(), std::move(rs));
 }
 
@@ -743,10 +803,6 @@ void StorageEngineImpl::setStableTimestamp(Timestamp stableTimestamp, bool force
     _engine->setStableTimestamp(stableTimestamp, force);
 }
 
-Timestamp StorageEngineImpl::getStableTimestamp() const {
-    return _engine->getStableTimestamp();
-}
-
 void StorageEngineImpl::setInitialDataTimestamp(Timestamp initialDataTimestamp) {
     _engine->setInitialDataTimestamp(initialDataTimestamp);
 }
@@ -811,7 +867,9 @@ StatusWith<Timestamp> StorageEngineImpl::recoverToStableTimestamp(OperationConte
 
     catalog::openCatalog(opCtx, state);
 
-    log() << "recoverToStableTimestamp successful. Stable Timestamp: " << swTimestamp.getValue();
+    LOGV2(22259,
+          "recoverToStableTimestamp successful",
+          "stableTimestamp"_attr = swTimestamp.getValue());
     return {swTimestamp.getValue()};
 }
 
@@ -831,6 +889,10 @@ bool StorageEngineImpl::supportsReadConcernMajority() const {
     return _engine->supportsReadConcernMajority();
 }
 
+bool StorageEngineImpl::supportsOplogStones() const {
+    return _engine->supportsOplogStones();
+}
+
 bool StorageEngineImpl::supportsPendingDrops() const {
     return supportsReadConcernMajority();
 }
@@ -839,8 +901,33 @@ void StorageEngineImpl::clearDropPendingState() {
     _dropPendingIdentReaper.clearDropPendingState();
 }
 
-void StorageEngineImpl::replicationBatchIsComplete() const {
-    return _engine->replicationBatchIsComplete();
+bool StorageEngineImpl::supportsTwoPhaseIndexBuild() const {
+    if (!enableTwoPhaseIndexBuild) {
+        return false;
+    }
+
+    if (!serverGlobalParams.featureCompatibility.isVersionInitialized()) {
+        return false;
+    }
+
+    if (serverGlobalParams.featureCompatibility.getVersion() !=
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44) {
+        return false;
+    }
+
+    return true;
+}
+
+void StorageEngineImpl::triggerJournalFlush() const {
+    return _engine->triggerJournalFlush();
+}
+
+void StorageEngineImpl::waitForJournalFlush(OperationContext* opCtx) const {
+    return _engine->waitForJournalFlush(opCtx);
+}
+
+void StorageEngineImpl::interruptJournalFlusherForReplStateChange() const {
+    return _engine->interruptJournalFlusherForReplStateChange();
 }
 
 Timestamp StorageEngineImpl::getAllDurableTimestamp() const {
@@ -863,8 +950,11 @@ void StorageEngineImpl::_dumpCatalog(OperationContext* opCtx) {
     while (rec) {
         // This should only be called by a parent that's done an appropriate `shouldLog` check. Do
         // not duplicate the log level policy.
-        LOG_FOR_RECOVERY(kCatalogLogLevel)
-            << "\tId: " << rec->id << " Value: " << rec->data.toBson();
+        LOGV2_FOR_RECOVERY(4615634,
+                           kCatalogLogLevel.toInt(),
+                           "Id: {rec_id} Value: {rec_data_toBson}",
+                           "rec_id"_attr = rec->id,
+                           "rec_data_toBson"_attr = rec->data.toBson());
         auto valueBson = rec->data.toBson();
         if (valueBson.hasField("md")) {
             std::string ns = valueBson.getField("md").Obj().getField("ns").String();
@@ -890,8 +980,9 @@ void StorageEngineImpl::_onMinOfCheckpointAndOldestTimestampChanged(const Timest
     // No drop-pending idents present if getEarliestDropTimestamp() returns boost::none.
     if (auto earliestDropTimestamp = _dropPendingIdentReaper.getEarliestDropTimestamp()) {
         if (timestamp > *earliestDropTimestamp) {
-            log() << "Removing drop-pending idents with drop timestamps before timestamp "
-                  << timestamp;
+            LOGV2(22260,
+                  "Removing drop-pending idents with drop timestamps before timestamp",
+                  "timestamp"_attr = timestamp);
             auto opCtx = cc().getOperationContext();
             invariant(opCtx);
 
@@ -913,7 +1004,7 @@ StorageEngineImpl::TimestampMonitor::TimestampMonitor(KVEngine* engine, Periodic
 }
 
 StorageEngineImpl::TimestampMonitor::~TimestampMonitor() {
-    log() << "Timestamp monitor shutting down";
+    LOGV2(22261, "Timestamp monitor shutting down");
     stdx::lock_guard<Latch> lock(_monitorMutex);
     invariant(_listeners.empty());
 }
@@ -921,7 +1012,7 @@ StorageEngineImpl::TimestampMonitor::~TimestampMonitor() {
 void StorageEngineImpl::TimestampMonitor::startup() {
     invariant(!_running);
 
-    log() << "Timestamp monitor starting";
+    LOGV2(22262, "Timestamp monitor starting");
     PeriodicRunner::PeriodicJob job(
         "TimestampMonitor",
         [&](Client* client) {
@@ -980,11 +1071,15 @@ void StorageEngineImpl::TimestampMonitor::startup() {
                     notifyAll(TimestampType::kMinOfCheckpointAndOldest, minOfCheckpointAndOldest);
                 }
             } catch (const ExceptionForCat<ErrorCategory::Interruption>& ex) {
-                if (!ErrorCodes::isCancelationError(ex.code()))
+                if (!ErrorCodes::isCancelationError(ex))
                     throw;
                 // If we're interrupted at shutdown or after PeriodicRunner's client has been
                 // killed, it's fine to give up on future notifications.
-                log() << "Timestamp monitor is stopping due to: " + ex.reason();
+                LOGV2(22263,
+                      "Timestamp monitor is stopping. {reason}",
+                      "Timestamp monitor is stopping",
+                      "error"_attr = ex.reason());
+                return;
             }
         },
         Seconds(1));
@@ -1028,10 +1123,11 @@ int64_t StorageEngineImpl::sizeOnDiskForDb(OperationContext* opCtx, StringData d
         size += collection->getRecordStore()->storageSize(opCtx);
 
         std::vector<std::string> indexNames;
-        _catalog->getAllIndexes(opCtx, collection->ns(), &indexNames);
+        _catalog->getAllIndexes(opCtx, collection->getCatalogId(), &indexNames);
 
         for (size_t i = 0; i < indexNames.size(); i++) {
-            std::string ident = _catalog->getIndexIdent(opCtx, collection->ns(), indexNames[i]);
+            std::string ident =
+                _catalog->getIndexIdent(opCtx, collection->getCatalogId(), indexNames[i]);
             size += _engine->getIdentSize(opCtx, ident);
         }
 
