@@ -29,7 +29,9 @@
 
 #pragma once
 
+#include <functional>
 #include <map>
+#include <memory>
 #include <string>
 
 #include "mongo/base/status_with.h"
@@ -45,8 +47,6 @@
 #include "mongo/db/storage/storage_engine_interface.h"
 #include "mongo/db/storage/temporary_record_store.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/stdx/functional.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/util/periodic_runner.h"
 
 namespace mongo {
@@ -69,41 +69,39 @@ class StorageEngineImpl final : public StorageEngineInterface, public StorageEng
 
 public:
     /**
-     * @param engine - ownership passes to me
+     * @param engine - ownership passes to me.
      */
     StorageEngineImpl(KVEngine* engine, StorageEngineOptions options = StorageEngineOptions());
 
     virtual ~StorageEngineImpl();
 
-    virtual void finishInit();
+    virtual void finishInit() override;
 
-    virtual RecoveryUnit* newRecoveryUnit();
+    virtual RecoveryUnit* newRecoveryUnit() override;
 
-    virtual void notifyStartupComplete() override;
+    virtual std::vector<std::string> listDatabases() const override;
 
-    virtual std::vector<std::string> listDatabases() const;
-
-    virtual bool supportsDocLocking() const {
+    virtual bool supportsDocLocking() const override {
         return _supportsDocLocking;
     }
 
-    virtual bool supportsDBLocking() const {
+    virtual bool supportsDBLocking() const override {
         return _supportsDBLocking;
     }
 
-    virtual bool supportsCappedCollections() const {
+    virtual bool supportsCappedCollections() const override {
         return _supportsCappedCollections;
     }
 
-    virtual Status closeDatabase(OperationContext* opCtx, StringData db);
+    virtual Status closeDatabase(OperationContext* opCtx, StringData db) override;
 
-    virtual Status dropDatabase(OperationContext* opCtx, StringData db);
+    virtual Status dropDatabase(OperationContext* opCtx, StringData db) override;
 
-    virtual int flushAllFiles(OperationContext* opCtx, bool sync);
+    virtual void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
-    virtual Status beginBackup(OperationContext* opCtx);
+    virtual Status beginBackup(OperationContext* opCtx) override;
 
-    virtual void endBackup(OperationContext* opCtx);
+    virtual void endBackup(OperationContext* opCtx) override;
 
     virtual Status disableIncrementalBackup(OperationContext* opCtx) override;
 
@@ -112,23 +110,26 @@ public:
 
     virtual void endNonBlockingBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::vector<std::string>> extendBackupCursor(OperationContext* opCtx);
+    virtual StatusWith<std::vector<std::string>> extendBackupCursor(
+        OperationContext* opCtx) override;
 
-    virtual bool isDurable() const;
+    virtual bool supportsCheckpoints() const override;
 
-    virtual bool isEphemeral() const;
+    virtual bool isDurable() const override;
 
-    virtual Status repairRecordStore(OperationContext* opCtx, const NamespaceString& nss);
+    virtual bool isEphemeral() const override;
+
+    virtual Status repairRecordStore(OperationContext* opCtx,
+                                     RecordId catalogId,
+                                     const NamespaceString& nss) override;
 
     virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(
         OperationContext* opCtx) override;
 
-    virtual void cleanShutdown();
+    virtual void cleanShutdown() override;
 
     virtual void setStableTimestamp(Timestamp stableTimestamp, bool force = false) override;
 
-    virtual Timestamp getStableTimestamp() const override;
-
     virtual void setInitialDataTimestamp(Timestamp initialDataTimestamp) override;
 
     virtual Timestamp getInitialDataTimestamp() override;
@@ -166,18 +167,24 @@ public:
 
     bool supportsReadConcernMajority() const final;
 
+    bool supportsOplogStones() const final;
+
     bool supportsPendingDrops() const final;
 
     void clearDropPendingState() final;
 
-    virtual void replicationBatchIsComplete() const override;
+    bool supportsTwoPhaseIndexBuild() const final;
+
+    void triggerJournalFlush() const final;
+
+    void waitForJournalFlush(OperationContext* opCtx) const final;
+
+    void interruptJournalFlusherForReplStateChange() const final;
 
     SnapshotManager* getSnapshotManager() const final;
 
     void setJournalListener(JournalListener* jl) final;
 
-    // ------ kv ------
-
     /**
      * A TimestampMonitor is used to listen for any changes in the timestamps implemented by the
      * storage engine and to notify any registered listeners upon changes to these timestamps.
@@ -205,7 +212,7 @@ public:
         class TimestampListener {
         public:
             // Caller must ensure that the lifetime of the variables used in the callback are valid.
-            using Callback = stdx::function<void(Timestamp timestamp)>;
+            using Callback = std::function<void(Timestamp timestamp)>;
 
             /**
              * A TimestampListener saves a 'callback' that will be executed whenever the specified
@@ -308,7 +315,6 @@ public:
         Mutex _monitorMutex = MONGO_MAKE_LATCH("TimestampMonitor::_monitorMutex");
         std::vector<TimestampListener*> _listeners;
 
-        // Periodic runner that the timestamp monitor schedules its job on.
         // This should remain as the last member variable so that its destructor gets executed first
         // when the class instance is being deconstructed. This causes the PeriodicJobAnchor to stop
         // the PeriodicJob, preventing us from accessing any destructed variables if this were to
@@ -320,10 +326,11 @@ public:
         return this;
     }
 
-    KVEngine* getEngine() {
+    KVEngine* getEngine() override {
         return _engine.get();
     }
-    const KVEngine* getEngine() const {
+
+    const KVEngine* getEngine() const override {
         return _engine.get();
     }
 
@@ -331,18 +338,15 @@ public:
                              const NamespaceString& nss,
                              StringData ident) override;
 
-    DurableCatalog* getCatalog() {
-        return _catalog.get();
-    }
-    const DurableCatalog* getCatalog() const {
+    DurableCatalog* getCatalog() override {
         return _catalog.get();
     }
 
-    /**
-     * Drop abandoned idents. Returns a parallel list of index name, index spec pairs to rebuild.
-     */
-    StatusWith<std::vector<StorageEngine::CollectionIndexNamePair>> reconcileCatalogAndIdents(
-        OperationContext* opCtx) override;
+    const DurableCatalog* getCatalog() const override {
+        return _catalog.get();
+    }
+
+    StatusWith<ReconcileResult> reconcileCatalogAndIdents(OperationContext* opCtx) override;
 
     std::string getFilesystemPathForDb(const std::string& dbName) const override;
 
@@ -358,7 +362,7 @@ public:
         return _timestampMonitor.get();
     }
 
-    std::set<std::string> getDropPendingIdents() const {
+    std::set<std::string> getDropPendingIdents() const override {
         return _dropPendingIdentReaper.getAllIdents();
     }
 
@@ -372,7 +376,10 @@ public:
 private:
     using CollIter = std::list<std::string>::iterator;
 
-    void _initCollection(OperationContext* opCtx, const NamespaceString& nss, bool forRepair);
+    void _initCollection(OperationContext* opCtx,
+                         RecordId catalogId,
+                         const NamespaceString& nss,
+                         bool forRepair);
 
     Status _dropCollectionsNoTimestamp(OperationContext* opCtx,
                                        std::vector<NamespaceString>& toDrop);
@@ -388,6 +395,7 @@ private:
      * collection.
      */
     Status _recoverOrphanedCollection(OperationContext* opCtx,
+                                      RecordId catalogId,
                                       const NamespaceString& collectionName,
                                       StringData collectionIdent);
 
