@@ -29,7 +29,7 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
     it in the license file.
 ======= */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include <sys/stat.h>
 
@@ -38,14 +38,13 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
 
 #include "mongo/db/encryption/encryption_options.h"
 #include "mongo/db/encryption/encryption_vault.h"
-#include "mongo/db/encryption/encryption_kmip.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/storage/wiredtiger/encryption_keydb.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/base64.h"
 #include "mongo/util/debug_util.h"
-#include "mongo/util/log.h"
 
 #include <third_party/wiredtiger/ext/encryptors/percona/encryption_keydb_c_api.h>
 
@@ -70,15 +69,15 @@ static void dump_key(unsigned char *key, const int _key_len, const char * msg) {
         ++key;
     }
     *p = 0;
-    log() << msg << ": " << buf;
+    LOGV2(29033, "{msg}: {buf}", "msg"_attr = msg, "buf"_attr = static_cast<const char*>(buf));
 }
 
 static void dump_table(WT_SESSION* _sess, const int _key_len, const char* msg) {
-    log() << msg;
+    LOGV2(29034, "{msg}", "msg"_attr = msg);
     WT_CURSOR *cursor;
     int res = _sess->open_cursor(_sess, "table:key", nullptr, nullptr, &cursor);
     if (res) {
-        log() << wiredtiger_strerror(res);
+        LOGV2(29035, "{e}", "e"_attr = wiredtiger_strerror(res));
         return;
     }
     while ((res = cursor->next(cursor)) == 0) {
@@ -93,28 +92,10 @@ static void dump_table(WT_SESSION* _sess, const int _key_len, const char* msg) {
     res = cursor->close(cursor);
 }
 
-std::unique_ptr<EncryptionKeyDB> EncryptionKeyDB::create(const std::string& path,
-                                                         const std::string& kmipMasterKeyId) {
-    std::unique_ptr<EncryptionKeyDB> db(new EncryptionKeyDB(false, path, kmipMasterKeyId, false));
-    db->init();
-    return db;
-}
-
-std::unique_ptr<EncryptionKeyDB> EncryptionKeyDB::open(const std::string& path,
-                                                       const std::string& kmipMasterKeyId) {
-    std::unique_ptr<EncryptionKeyDB> db(new EncryptionKeyDB(true, path, kmipMasterKeyId, false));
-    db->init();
-    return db;
-}
-
-EncryptionKeyDB::EncryptionKeyDB(const bool pre_existed,
-                                 const std::string& path,
-                                 const std::string& kmipMasterKeyId,
-                                 const bool rotation)
-    : _pre_existed(pre_existed),
-      _rotation(rotation),
-      _path(path),
-      _kmipMasterKeyId(kmipMasterKeyId) {
+EncryptionKeyDB::EncryptionKeyDB(const bool just_created, const std::string& path, const bool rotation)
+  : _just_created(just_created),
+    _rotation(rotation),
+    _path(path) {
     // single instance is allowed as main keydb
     // and another one for rotation
     if (!_rotation) {
@@ -136,10 +117,8 @@ EncryptionKeyDB::~EncryptionKeyDB() {
 }
 
 void EncryptionKeyDB::close_handles() {
-    DEV if (_sess)
+    if (kDebugBuild && _sess)
         dump_table(_sess, _key_len, "dump_table from destructor");
-    _backupSession.reset();
-    _backupCursor = nullptr;
     if (_sess) {
         _gcm_iv_reserved = _gcm_iv;
         store_gcm_iv_reserved();
@@ -155,36 +134,12 @@ void EncryptionKeyDB::close_handles() {
 // this function uses _srng without synchronization
 // caller must ensure it is safe
 void EncryptionKeyDB::generate_secure_key(char key[]) {
-    for (int i = 0; i < 4; ++i) {
-        ((int64_t*)key)[i] = _srng->nextInt64();
-    }
+    _srng->fill(key, _key_len);
 }
 
 void EncryptionKeyDB::init_masterkey() {
     std::string encoded_key;
-    if (!encryptionGlobalParams.kmipServerName.empty()) {
-        if (_kmipMasterKeyId.empty()) {
-            char newkey[_key_len];
-            generate_secure_key(newkey);
-            encoded_key = base64::encode(newkey, _key_len);
-            if (!_rotation) {
-                // @note: please see the `store_masterkey` function for
-                // the key writing during rotation
-                _kmipMasterKeyId = kmipWriteKey(encoded_key);
-                log() << "Master key has been created on the KMIP server: "
-                      << "kmipMasterKeyId = " << _kmipMasterKeyId;
-            }
-        } else {
-            encoded_key = kmipReadKey(_kmipMasterKeyId);
-            if (encoded_key.empty()) {
-                severe() << "Cannot start. Master encryption key is absent in KMIP. "
-                            "Check configuration options. kmipMasterKeyId = " << _kmipMasterKeyId;
-                fassertFailedNoTrace(29110);
-            }
-            log() << "Master key has been read from the KMIP server: "
-                  << "kmipMasterKeyId = " << _kmipMasterKeyId;
-        }
-    } else if (!encryptionGlobalParams.vaultServerName.empty()) {
+    if (!encryptionGlobalParams.vaultServerName.empty()) {
         if (encryptionGlobalParams.vaultToken.empty()) {
             struct stat stats;
 
@@ -215,7 +170,7 @@ void EncryptionKeyDB::init_masterkey() {
             // generate new key
             char newkey[_key_len];
             generate_secure_key(newkey);
-            encoded_key = base64::encode(newkey, _key_len);
+            encoded_key = base64::encode(StringData{newkey, _key_len});
         } else {
             // read key from the Vault
             encoded_key = vaultReadKey();
@@ -223,13 +178,13 @@ void EncryptionKeyDB::init_masterkey() {
             // if this happens on first run (with empty keydb) then
             // we can generate key here
             if (encoded_key.empty()) {
-                if (_pre_existed) {
+                if (!_just_created) {
                     throw std::runtime_error("Cannot start. Master encryption key is absent in the Vault. Check configuration options.");
                 }
-                log() << "Master key is absent in the Vault. Generating and writing one.";
+                LOGV2(29036, "Master key is absent in the Vault. Generating and writing one.");
                 char newkey[_key_len];
                 generate_secure_key(newkey);
-                encoded_key = base64::encode(newkey, _key_len);
+                encoded_key = base64::encode(StringData{newkey, _key_len});
                 vaultWriteKey(encoded_key);
             }
         }
@@ -275,68 +230,43 @@ int EncryptionKeyDB::_openWiredTiger(const std::string& path, const std::string&
     // For now we don't use event handler in EncryptionKeyDB
     WT_EVENT_HANDLER* wtEventHandler = nullptr;
 
-    // Default for new instances created by 4.2.x
-    std::string configStr =
-        wtOpenConfig + ",compatibility=(release=3.2,require_min=\"3.2\",require_max=\"3.2\")";
+    // MongoDB 4.4 will always run in compatibility version 10.0.
+    std::string configStr = wtOpenConfig + ",compatibility=(require_min=\"10.0.0\")";
     int ret = wiredtiger_open(path.c_str(), wtEventHandler, configStr.c_str(), &_conn);
     if (!ret) {
-        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_42_CLASSIC};
+        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_44_FCV_44};
         return ret;
     }
 
-    // Data files are in 4.4 -> 4.2 downgraded format
-    configStr =
-        wtOpenConfig + ",compatibility=(release=3.3,require_min=\"3.3\",require_max=\"3.3\")";
+    // MongoDB 4.4 doing clean shutdown in FCV 4.2 will use compatibility version 3.3.
+    configStr = wtOpenConfig + ",compatibility=(require_min=\"3.3.0\")";
     ret = wiredtiger_open(path.c_str(), wtEventHandler, configStr.c_str(), &_conn);
     if (!ret) {
-        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_42_UPGRADED};
+        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_44_FCV_42};
         return ret;
     }
 
-    // This shoud only succeed when the previous datafiles were in compatibility mode
-    // 3.1. `require_max=` is added for improved WT error messages. "3.3" is used because WT
-    // disallows using a release outside of the min/max range. These details also apply to the
-    // remaining `wiredtiger_open` calls.
-    configStr =
-        wtOpenConfig + ",compatibility=(release=3.2,require_min=\"3.1\",require_max=\"3.3\")";
+    // MongoDB 4.2 uses compatibility version 3.2.
+    configStr = wtOpenConfig + ",compatibility=(require_min=\"3.2.0\")";
     ret = wiredtiger_open(path.c_str(), wtEventHandler, configStr.c_str(), &_conn);
     if (!ret) {
-        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_40};
+        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_42};
         return ret;
     }
 
-    // Arbiters do not replicate the FCV document. Due to arbiter FCV semantics on 4.0, shutting
-    // down a 4.0 arbiter may either downgrade the data files to WT compatibility 2.9 or 3.0. Thus,
-    // 4.2 binaries must allow starting up on 2.9 and 3.0 files.
-    configStr =
-        wtOpenConfig + ",compatibility=(release=3.2,require_min=\"3.0\",require_max=\"3.3\")";
-    ret = wiredtiger_open(path.c_str(), wtEventHandler, configStr.c_str(), &_conn);
-    if (!ret) {
-        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_36};
-        return ret;
-    }
-
-    configStr =
-        wtOpenConfig + ",compatibility=(release=3.2,require_min=\"2.9\",require_max=\"3.3\")";
-    ret = wiredtiger_open(path.c_str(), wtEventHandler, configStr.c_str(), &_conn);
-    if (!ret) {
-        //_fileVersion = {WiredTigerFileVersion::StartupVersion::IS_34};
-        return ret;
-    }
-
-    warning() << "EncryptionKeyDB: Failed to start up WiredTiger under any compatibility version.";
+    LOGV2_WARNING(29054,
+        "EncryptionKeyDB: Failed to start up WiredTiger under any compatibility version. "
+        "This may be due to an unsupported upgrade or downgrade.");
     if (ret == WT_TRY_SALVAGE)
-        warning() << "EncryptionKeyDB: WiredTiger metadata corruption detected or "
-                  << "an invalid encryption key is used";
+        LOGV2_WARNING(29055, "EncryptionKeyDB: WiredTiger metadata corruption detected");
 
-    severe() << "Reason: " << wtRCToStatus(ret).reason();
-    fassertFailedNoTrace(29056);
+    LOGV2_FATAL_NOTRACE(29056, "Terminating.", "Reason"_attr = wtRCToStatus(ret).reason());
 
     return ret;
 }
 
 void EncryptionKeyDB::init() {
-    _srng = SecureRandom::create();
+    _srng = std::make_unique<SecureRandom>();
     _prng = std::make_unique<PseudoRandom>(_srng->nextInt64());
     try {
         init_masterkey();
@@ -354,11 +284,12 @@ void EncryptionKeyDB::init() {
         // https://source.wiredtiger.com/3.0.0/tune_durability.html
         ss << "log=(enabled,file_max=5MB),transaction_sync=(enabled=true,method=fsync),";
         std::string config = ss.str();
-        log() << "Initializing KeyDB with wiredtiger_open config: " << config;
+        LOGV2(29037, "Initializing KeyDB with wiredtiger_open config: {cfg}", "cfg"_attr = config);
         int res = _openWiredTiger(_path, config);
         if (res) {
             throw std::runtime_error(std::string("error opening keys DB at '") + _path + "': " + wiredtiger_strerror(res));
         }
+        _wtOpenConfig = config;
 
         // empty keyid means masterkey
         res = _conn->open_session(_conn, nullptr, nullptr, &_sess);
@@ -366,14 +297,14 @@ void EncryptionKeyDB::init() {
             throw std::runtime_error(std::string("error opening wiredTiger session: ") + wiredtiger_strerror(res));
         }
 
-        DEV dump_table(_sess, _key_len, "before create");
+        if (kDebugBuild) dump_table(_sess, _key_len, "before create");
         // try to create 'key' table
         // ignore error if table already exists
         res = _sess->create(_sess, "table:key", "key_format=S,value_format=u,access_pattern_hint=random");
         if (res) {
             throw std::runtime_error(std::string("error creating/opening key table: ") + wiredtiger_strerror(res));
         }
-        DEV dump_table(_sess, _key_len, "after create");
+        if (kDebugBuild) dump_table(_sess, _key_len, "after create");
 
         // try to create 'parameters' table
         // ignore error if table already exists
@@ -408,18 +339,18 @@ void EncryptionKeyDB::init() {
             }
         }
     } catch (std::exception& e) {
-        error() << e.what();
+        LOGV2_ERROR(29038, "Exception in EncryptionKeyDB::init: {e}", "e"_attr = e.what());
         throw;
     }
-    log() << "Encryption keys DB is initialized successfully";
+    LOGV2(29039, "Encryption keys DB is initialized successfully");
 }
 
-void EncryptionKeyDB::import_data_from(EncryptionKeyDB* proto) {
+void EncryptionKeyDB::clone(EncryptionKeyDB *old) {
     // not doing any synchronization here because key rotation process is single threaded
     try {
         // copy parameters table
         // clone is called right after init(). at this point _gcm_iv_reserved is equal to _gcm_iv
-        _gcm_iv_reserved = proto->_gcm_iv_reserved;
+        _gcm_iv_reserved = old->_gcm_iv_reserved;
         if (store_gcm_iv_reserved()) {
             throw std::runtime_error("failed to copy key db data during rotation");
         }
@@ -427,7 +358,7 @@ void EncryptionKeyDB::import_data_from(EncryptionKeyDB* proto) {
         int res;
         auto cursor_close = [](WT_CURSOR* c){c->close(c);};
         WT_CURSOR *srcc;
-        res = proto->_sess->open_cursor(proto->_sess, "table:key", nullptr, nullptr, &srcc);
+        res = old->_sess->open_cursor(old->_sess, "table:key", nullptr, nullptr, &srcc);
         if (res)
             throw std::runtime_error(std::string("clone: error opening cursor: ") + wiredtiger_strerror(res));
         std::unique_ptr<WT_CURSOR, std::function<void(WT_CURSOR*)>> srcc_guard(srcc, cursor_close);
@@ -451,44 +382,21 @@ void EncryptionKeyDB::import_data_from(EncryptionKeyDB* proto) {
         if (res != WT_NOTFOUND)
             throw std::runtime_error(std::string("clone: error reading key table: ") + wiredtiger_strerror(res));
     } catch (std::exception& e) {
-        error() << e.what();
+        LOGV2_ERROR(29049, "Exception in EncryptionKeyDB::clone: {e}", "e"_attr = e.what());
         throw;
     }
 }
 
 void EncryptionKeyDB::store_masterkey() {
-    auto encodedKey = base64::encode((const char*)_masterkey, _key_len);
-    if (!encryptionGlobalParams.kmipServerName.empty()) {
-        if (_kmipMasterKeyId.empty()) {
-            _kmipMasterKeyId = kmipWriteKey(encodedKey);
-            log() << "Master key has been created on the KMIP server: "
-                  << "kmipMasterKeyId = " << _kmipMasterKeyId;
-        }
-    } else if (!encryptionGlobalParams.vaultServerName.empty()) {
-        vaultWriteKey(encodedKey);
-    } else {
-        throw std::logic_error(
-            "Can't save master key because neither HashiCorp's Vault nor "
-            "KMIP server is configured");
-    }
-}
-
-std::unique_ptr<EncryptionKeyDB> EncryptionKeyDB::clone(const std::string& path,
-                                                        const std::string& kmipMasterKeyId) {
-    std::unique_ptr<EncryptionKeyDB> duplicate(
-        new EncryptionKeyDB(false, path, kmipMasterKeyId, true));
-    duplicate->init();
-    duplicate->import_data_from(this);
-    duplicate->store_masterkey();
-    return duplicate;
+    vaultWriteKey(base64::encode(StringData{(const char*)_masterkey, _key_len}));
 }
 
 int EncryptionKeyDB::get_key_by_id(const char *keyid, size_t len, unsigned char *key, void *pe) {
-    LOG(4) << "get_key_by_id for keyid: '" << std::string(keyid, len) << "'";
+    LOGV2_DEBUG(29050, 4, "get_key_by_id for keyid: '{id}'", "id"_attr = std::string{keyid, len});
     // return key from keyfile if len == 0
     if (len == 0) {
         memcpy(key, _masterkey, _key_len);
-        DEV dump_key(key, _key_len, "returning masterkey");
+        if (kDebugBuild) dump_key(key, _key_len, "returning masterkey");
         return 0;
     }
 
@@ -499,7 +407,7 @@ int EncryptionKeyDB::get_key_by_id(const char *keyid, size_t len, unsigned char
         stdx::lock_guard<Latch> lk(_lock_sess);
         res = _sess->open_cursor(_sess, "table:key", nullptr, nullptr, &cursor);
         if (res){
-            error() << "get_key_by_id: error opening cursor: " << wiredtiger_strerror(res);
+            LOGV2_ERROR(29040, "get_key_by_id: error opening cursor: {err}", "err"_attr = wiredtiger_strerror(res));
             return res;
         }
     }
@@ -514,7 +422,7 @@ int EncryptionKeyDB::get_key_by_id(const char *keyid, size_t len, unsigned char
     stdx::lock_guard<Latch> lk(_lock_key);
     // read key from DB
     std::string c_str(keyid, len);
-    LOG(4) << "trying to load encryption key for keyid: " << c_str;
+    LOGV2_DEBUG(29041, 4, "trying to load encryption key for keyid: {id}", "id"_attr = c_str);
     cursor->set_key(cursor, c_str.c_str());
     res = cursor->search(cursor);
     if (res == 0) {
@@ -522,20 +430,19 @@ int EncryptionKeyDB::get_key_by_id(const char *keyid, size_t len, unsigned char
         cursor->get_value(cursor, &v);
         invariant(v.size == _key_len);
         memcpy(key, v.data, _key_len);
-        DEV dump_key(key, _key_len, "loaded key from key DB");
+        if (kDebugBuild) dump_key(key, _key_len, "loaded key from key DB");
         _encryptors[c_str] = pe;
         return 0;
     }
     if (res != WT_NOTFOUND) {
-        error() << "cursor->search error " << res << " : " <<wiredtiger_strerror(res);
+        LOGV2_ERROR(29042, "cursor->search error {code}: {desc}",
+                    "code"_attr = res, "desc"_attr = wiredtiger_strerror(res));
         return res;
     }
 
     // create key if it does not exist
-    for (int i = 0; i < 4; ++i) {
-        // call to nextInt64() is protected by _lock_key above
-        ((int64_t*)key)[i] = _srng->nextInt64();
-    }
+    // call to fill() is protected by _lock_key above
+    _srng->fill(key, _key_len);
     WT_ITEM v;
     v.size = _key_len;
     v.data = key;
@@ -543,17 +450,18 @@ int EncryptionKeyDB::get_key_by_id(const char *keyid, size_t len, unsigned char
     cursor->set_value(cursor, &v);
     res = cursor->insert(cursor);
     if (res) {
-        error() << "cursor->insert error " << res << " : " <<wiredtiger_strerror(res);
+        LOGV2_ERROR(29043, "cursor->insert error {code}: {desc}",
+                    "code"_attr = res, "desc"_attr = wiredtiger_strerror(res));
         return res;
     }
 
-    DEV dump_key(key, _key_len, "generated and stored key");
+    if (kDebugBuild) dump_key(key, _key_len, "generated and stored key");
     _encryptors[c_str] = pe;
     return 0;
 }
 
 int EncryptionKeyDB::delete_key_by_id(const std::string&  keyid) {
-    LOG(4) << "delete_key_by_id for keyid: '" << keyid << "'";
+    LOGV2_DEBUG(29044, 4, "delete_key_by_id for keyid: '{id}'", "id"_attr = keyid);
 
     int res;
     // open cursor
@@ -562,7 +470,7 @@ int EncryptionKeyDB::delete_key_by_id(const std::string&  keyid) {
         stdx::lock_guard<Latch> lk(_lock_sess);
         res = _sess->open_cursor(_sess, "table:key", nullptr, nullptr, &cursor);
         if (res){
-            error() << "delete_key_by_id: error opening cursor: " << wiredtiger_strerror(res);
+            LOGV2_ERROR(29045, "delete_key_by_id: error opening cursor: {desc}", "desc"_attr = wiredtiger_strerror(res));
             return res;
         }
     }
@@ -577,7 +485,8 @@ int EncryptionKeyDB::delete_key_by_id(const std::string&  keyid) {
     cursor->set_key(cursor, keyid.c_str());
     res = cursor->remove(cursor);
     if (res) {
-        error() << "cursor->remove error " << res << " : " << wiredtiger_strerror(res);
+        LOGV2_ERROR(29046, "cursor->remove error {code}: {desc}",
+                    "code"_attr = res, "desc"_attr = wiredtiger_strerror(res));
     }
 
     // prepare encryptor for reuse in case DB with the same name will be recreated
@@ -604,7 +513,8 @@ int EncryptionKeyDB::store_gcm_iv_reserved() {
         stdx::lock_guard<Latch> lk(_lock_sess);
         res = _sess->open_cursor(_sess, "table:parameters", nullptr, nullptr, &cursor);
         if (res){
-            error() << "store_gcm_iv_reserved: error opening cursor: " << wiredtiger_strerror(res);
+            LOGV2_ERROR(29047, "store_gcm_iv_reserved: error opening cursor: {desc}",
+                       "desc"_attr = wiredtiger_strerror(res));
             return res;
         }
     }
@@ -623,7 +533,8 @@ int EncryptionKeyDB::store_gcm_iv_reserved() {
     cursor->set_value(cursor, &v);
     res = cursor->insert(cursor);
     if (res) {
-        error() << "cursor->insert error " << res << " : " <<wiredtiger_strerror(res);
+        LOGV2_ERROR(29048, "cursor->insert error {code}: {desc}",
+                    "code"_attr = res, "desc"_attr = wiredtiger_strerror(res));
         return res;
     }
     return 0;
@@ -657,6 +568,25 @@ void EncryptionKeyDB::store_pseudo_bytes(uint8_t *buf, int len) {
     }
 }
 
+void EncryptionKeyDB::reconfigure(const char *newCfg) {
+    // For now we don't use event handler in EncryptionKeyDB
+    WT_EVENT_HANDLER* wtEventHandler = nullptr;
+
+    auto startTime = Date_t::now();
+    LOGV2(29075, "Closing KeyDB in preparation for reconfiguring");
+    close_handles();
+    LOGV2(29076, "KeyDB closed", "duration"_attr = Date_t::now() - startTime);
+
+    startTime = Date_t::now();
+    invariantWTOK(wiredtiger_open(_path.c_str(), wtEventHandler, _wtOpenConfig.c_str(), &_conn));
+    LOGV2(29077, "KeyDB re-opened", "duration"_attr = Date_t::now() - startTime);
+
+    startTime = Date_t::now();
+    LOGV2(29078, "Reconfiguring KeyDB", "newConfig"_attr = newCfg);
+    invariantWTOK(_conn->reconfigure(_conn, newCfg));
+    LOGV2(29079, "KeyDB reconfigure complete", "duration"_attr = Date_t::now() - startTime);
+}
+
 extern "C" void store_pseudo_bytes(uint8_t *buf, int len) {
     invariant(encryptionKeyDB);
     encryptionKeyDB->store_pseudo_bytes(buf, len);
