@@ -49,7 +49,6 @@ Copyright (C) 2018-present Percona and/or its affiliates. All rights reserved.
 #include "mongo/db/storage/wiredtiger/wiredtiger_data_protector.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_encryption_hooks.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
 
@@ -83,17 +82,14 @@ public:
             encryptionGlobalParams.encryptionKeyFile,
             boost::filesystem::owner_read | boost::filesystem::owner_write);
 
-        _encryptionKeyDB = EncryptionKeyDB::create(_keydbpath.path(), "");
+        _encryptionKeyDB = std::make_unique<EncryptionKeyDB>(_keydbpath.path());
+        _encryptionKeyDB->init();
     }
 
     ~EncryptionHarness() {
         _encryptionKeyDB.reset(nullptr);
     }
 
-    EncryptionKeyDB* getKeyDB() const {
-        return _encryptionKeyDB.get();
-    }
-
 private:
     unittest::TempDir _keydbpath{"keydb"};
     std::unique_ptr<EncryptionKeyDB> _encryptionKeyDB;
@@ -175,13 +171,13 @@ void test_encryption_hooks(WiredTigerEncryptionHooks* hooks) {
 
 TEST(WiredTigerEncryptionTest, EncryptionCBC) {
     EncryptionHarness eKeyDB{"AES256-CBC"};
-    WiredTigerEncryptionHooksCBC hooks{eKeyDB.getKeyDB()};
+    WiredTigerEncryptionHooksCBC hooks{};
     test_encryption_hooks(&hooks);
 }
 
 TEST(WiredTigerEncryptionTest, EncryptionGCM) {
     EncryptionHarness eKeyDB{"AES256-GCM"};
-    WiredTigerEncryptionHooksGCM hooks{eKeyDB.getKeyDB()};
+    WiredTigerEncryptionHooksGCM hooks{};
     test_encryption_hooks(&hooks);
 }
 
