@@ -29,6 +29,7 @@
 
 #pragma once
 
+#include <functional>
 #include <list>
 #include <memory>
 #include <string>
@@ -38,6 +39,7 @@
 
 #include "mongo/bson/ordering.h"
 #include "mongo/bson/timestamp.h"
+#include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/storage/kv/kv_engine.h"
 #include "mongo/db/storage/storage_engine.h"
 #include "mongo/db/storage/wiredtiger/encryption_keydb.h"
@@ -45,7 +47,6 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/stdx/functional.h"
 #include "mongo/util/elapsed_tracker.h"
 
 namespace mongo {
@@ -56,13 +57,12 @@ class WiredTigerRecordStore;
 class WiredTigerSessionCache;
 class WiredTigerSizeStorer;
 class WiredTigerEngineRuntimeConfigParameter;
-class WiredTigerMaxCacheOverflowSizeGBParameter;
 
 struct WiredTigerFileVersion {
-    // "4.2 classic" is for data files never touched by a 4.4 binary. "4.2 upgraded" is the state of
-    // data files after being touched by a 4.4 binary. Only 4.2.6+ will work on "4.2 upgraded" data
-    // files.
-    enum class StartupVersion { IS_34, IS_36, IS_40, IS_42_CLASSIC, IS_42_UPGRADED };
+    // MongoDB 4.4+ will not open on datafiles left behind by 4.0 and earlier. MongoDB 4.4
+    // shutting down in FCV 4.2 will leave data files that 4.2.6+ will understand
+    // (IS_44_FCV_42). MongoDB 4.2.x always writes out IS_42.
+    enum class StartupVersion { IS_42, IS_44_FCV_42, IS_44_FCV_44 };
 
     StartupVersion _startupVersion;
     bool shouldDowngrade(bool readOnly, bool repairMode, bool hasRecoveryTimestamp);
@@ -78,7 +78,7 @@ public:
                        ClockSource* cs,
                        const std::string& extraOpenOptions,
                        size_t cacheSizeMB,
-                       size_t maxCacheOverflowFileSizeMB,
+                       size_t maxHistoryFileSizeMB,
                        bool durable,
                        bool ephemeral,
                        bool repair,
@@ -86,7 +86,7 @@ public:
 
     ~WiredTigerKVEngine();
 
-    void notifyStartupComplete() override;
+    void startAsyncThreads() override;
 
     void setRecordStoreExtraOptions(const std::string& options);
     void setSortedDataInterfaceExtraOptions(const std::string& options);
@@ -95,6 +95,13 @@ public:
 
     bool supportsDirectoryPerDB() const override;
 
+    /**
+     * WiredTiger supports checkpoints when it isn't running in memory.
+     */
+    bool supportsCheckpoints() const override {
+        return !isEphemeral();
+    }
+
     bool isDurable() const override {
         return _durable;
     }
@@ -133,9 +140,8 @@ public:
             opCtx, collOptions, ident, desc, KVPrefix::kNotPrefixed);
     }
 
-    SortedDataInterface* getSortedDataInterface(OperationContext* opCtx,
-                                                StringData ident,
-                                                const IndexDescriptor* desc) override {
+    std::unique_ptr<SortedDataInterface> getSortedDataInterface(
+        OperationContext* opCtx, StringData ident, const IndexDescriptor* desc) override {
         return getGroupedSortedDataInterface(opCtx, ident, desc, KVPrefix::kNotPrefixed);
     }
 
@@ -157,26 +163,26 @@ public:
                                             const IndexDescriptor* desc,
                                             KVPrefix prefix) override;
 
-    SortedDataInterface* getGroupedSortedDataInterface(OperationContext* opCtx,
-                                                       StringData ident,
-                                                       const IndexDescriptor* desc,
-                                                       KVPrefix prefix) override;
+    std::unique_ptr<SortedDataInterface> getGroupedSortedDataInterface(OperationContext* opCtx,
+                                                                       StringData ident,
+                                                                       const IndexDescriptor* desc,
+                                                                       KVPrefix prefix) override;
 
-    Status dropIdent(OperationContext* opCtx, StringData ident) override;
+    Status dropIdent(OperationContext* opCtx, RecoveryUnit* ru, StringData ident) override;
+
+    void keydbDropDatabase(const std::string& db) override;
 
     void alterIdentMetadata(OperationContext* opCtx,
                             StringData ident,
                             const IndexDescriptor* desc) override;
 
-    void keydbDropDatabase(const std::string& db) override;
-
     Status okToRename(OperationContext* opCtx,
                       StringData fromNS,
                       StringData toNS,
                       StringData ident,
                       const RecordStore* originalRecordStore) const override;
 
-    int flushAllFiles(OperationContext* opCtx, bool sync) override;
+    void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
     Status beginBackup(OperationContext* opCtx) override;
 
@@ -260,11 +266,13 @@ public:
 
     bool supportsReadConcernSnapshot() const final override;
 
-    /*
-     * This function is called when replication has completed a batch.  In this function, we
-     * refresh our oplog visiblity read-at-timestamp value.
-     */
-    void replicationBatchIsComplete() const override;
+    bool supportsOplogStones() const final override;
+
+    void triggerJournalFlush() const override;
+
+    void waitForJournalFlush(OperationContext* opCtx) const override;
+
+    void interruptJournalFlusherForReplStateChange() const override;
 
     bool isCacheUnderPressure(OperationContext* opCtx) const override;
 
@@ -294,17 +302,14 @@ public:
     }
 
     /*
-     * An oplog manager is always accessible, but this method will start the background thread to
+     * The oplog manager is always accessible, but this method will start the background thread to
      * control oplog entry visibility for reads.
      *
-     * On mongod, the background thread will be started when the first oplog record store is
-     * created, and stopped when the last oplog record store is destroyed, at shutdown time. For
-     * unit tests, the background thread may be started and stopped multiple times as tests create
-     * and destroy oplog record stores.
+     * On mongod, the background thread will be started when the oplog record store is created, and
+     * stopped when the oplog record store is destroyed. For unit tests, the background thread may
+     * be started and stopped multiple times as tests create and destroy the oplog record store.
      */
-    void startOplogManager(OperationContext* opCtx,
-                           const std::string& uri,
-                           WiredTigerRecordStore* oplogRecordStore);
+    void startOplogManager(OperationContext* opCtx, WiredTigerRecordStore* oplogRecordStore);
     void haltOplogManager();
 
     /*
@@ -321,21 +326,6 @@ public:
         return _oplogManager.get();
     }
 
-    /**
-     * Sets the implementation for `initRsOplogBackgroundThread` (allowing tests to skip the
-     * background job, for example). Intended to be called from a MONGO_INITIALIZER and therefore in
-     * a single threaded context.
-     */
-    static void setInitRsOplogBackgroundThreadCallback(stdx::function<bool(StringData)> cb);
-
-    /**
-     * Initializes a background job to remove excess documents in the oplog collections.
-     * This applies to the capped collections in the local.oplog.* namespaces (specifically
-     * local.oplog.rs for replica sets).
-     * Returns true if a background job is running for the namespace.
-     */
-    static bool initRsOplogBackgroundThread(StringData ns);
-
     static void appendGlobalStats(BSONObjBuilder& b);
 
     Timestamp getStableTimestamp() const override;
@@ -391,14 +381,7 @@ private:
     // srcPath, destPath, filename, size to copy
     typedef std::tuple<boost::filesystem::path, boost::filesystem::path, boost::uintmax_t, std::time_t> FileTuple;
 
-    Status _hotBackupPopulateLists(OperationContext* opCtx,
-                                   const std::string& path,
-                                   std::vector<DBTuple>& dbList,
-                                   std::vector<FileTuple>& filesList,
-                                   boost::uintmax_t& totalfsize);
-
-    // auxiliary function for beginNonBlockingBackup
-    StatusWith<StorageEngine::BackupInformation> _disableIncrementalBackup();
+    Status _hotBackupPopulateLists(OperationContext* opCtx, const std::string& path, std::vector<DBTuple>& dbList, std::vector<FileTuple>& filesList);
 
     /**
      * Opens a connection on the WiredTiger database 'path' with the configuration 'wtOpenConfig'.
@@ -516,10 +499,9 @@ private:
     AtomicWord<std::uint64_t> _initialDataTimestamp;
 
     std::unique_ptr<WiredTigerEngineRuntimeConfigParameter> _runTimeConfigParam;
-    std::unique_ptr<WiredTigerMaxCacheOverflowSizeGBParameter> _maxCacheOverflowParam;
 
     mutable Mutex _highestDurableTimestampMutex =
         MONGO_MAKE_LATCH("WiredTigerKVEngine::_highestDurableTimestampMutex");
-    mutable unsigned long long _highestSeenDurableTimestamp = 0ULL;
+    mutable unsigned long long _highestSeenDurableTimestamp = StorageEngine::kMinimumTimestamp;
 };
 }  // namespace mongo
