@@ -27,14 +27,15 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 
+#include <memory>
+
 #include "mongo/base/error_codes.h"
-#include "mongo/db/audit/audit.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/global_settings.h"
 #include "mongo/db/repl/repl_settings.h"
@@ -42,25 +43,20 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_parameters_gen.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_util.h"
-#include "mongo/stdx/memory.h"
+#include "mongo/logv2/log.h"
 #include "mongo/stdx/thread.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
 
-const std::string kWTRepairMsg =
-    "Please read the documentation for starting MongoDB with --repair here: "
-    "http://dochub.mongodb.org/core/repair";
-
 WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn, uint64_t epoch, uint64_t cursorEpoch)
     : _epoch(epoch),
       _cursorEpoch(cursorEpoch),
-      _session(NULL),
+      _session(nullptr),
       _cursorGen(0),
       _cursorsOut(0),
       _idleExpireTime(Date_t::min()) {
-    invariantWTOK(conn->open_session(conn, NULL, "isolation=snapshot", &_session));
+    invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session));
 }
 
 WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn,
@@ -70,16 +66,16 @@ WiredTigerSession::WiredTigerSession(WT_CONNECTION* conn,
     : _epoch(epoch),
       _cursorEpoch(cursorEpoch),
       _cache(cache),
-      _session(NULL),
+      _session(nullptr),
       _cursorGen(0),
       _cursorsOut(0),
       _idleExpireTime(Date_t::min()) {
-    invariantWTOK(conn->open_session(conn, NULL, "isolation=snapshot", &_session));
+    invariantWTOK(conn->open_session(conn, nullptr, "isolation=snapshot", &_session));
 }
 
 WiredTigerSession::~WiredTigerSession() {
     if (_session) {
-        invariantWTOK(_session->close(_session, NULL));
+        invariantWTOK(_session->close(_session, nullptr));
     }
 }
 
@@ -88,24 +84,36 @@ void _openCursor(WT_SESSION* session,
                  const std::string& uri,
                  const char* config,
                  WT_CURSOR** cursorOut) {
-    int ret = session->open_cursor(session, uri.c_str(), NULL, config, cursorOut);
+    int ret = session->open_cursor(session, uri.c_str(), nullptr, config, cursorOut);
     if (ret == EBUSY) {
         // This can only happen when trying to open a cursor on the oplog and it is currently locked
         // by a verify or salvage, because we don't employ database locks to protect the oplog.
         throw WriteConflictException();
     }
-    if (ret != 0) {
-        error() << "Failed to open a WiredTiger cursor. Reason: " << wtRCToStatus(ret)
-                << ", uri: " << uri << ", config: " << config;
-        error() << "This may be due to data corruption. " << kWTRepairMsg;
 
-        fassertFailedNoTrace(50882);
+    if (ret != 0) {
+        auto status = wtRCToStatus(ret);
+        std::string cursorErrMsg = str::stream()
+            << "Failed to open a WiredTiger cursor. Reason: " << status << ", uri: " << uri
+            << ", config: " << config;
+
+        if (ret == ENOENT) {
+            uasserted(ErrorCodes::CursorNotFound, cursorErrMsg);
+        }
+
+        LOGV2_FATAL_NOTRACE(50882,
+                            "Failed to open WiredTiger cursor. This may be due to data corruption",
+                            "uri"_attr = uri,
+                            "config"_attr = config,
+                            "error"_attr = status,
+                            "message"_attr = kWTRepairMsg);
     }
 }
 }  // namespace
 
-
-WT_CURSOR* WiredTigerSession::getCursor(const std::string& uri, uint64_t id, bool allowOverwrite) {
+WT_CURSOR* WiredTigerSession::getCachedCursor(const std::string& uri,
+                                              uint64_t id,
+                                              const char* config) {
     // Find the most recently used cursor
     for (CursorCache::iterator i = _cursors.begin(); i != _cursors.end(); ++i) {
         if (i->_id == id) {
@@ -116,31 +124,20 @@ WT_CURSOR* WiredTigerSession::getCursor(const std::string& uri, uint64_t id, boo
         }
     }
 
-    WT_CURSOR* cursor = NULL;
-    _openCursor(_session, uri, allowOverwrite ? "" : "overwrite=false", &cursor);
+    WT_CURSOR* cursor = nullptr;
+    _openCursor(_session, uri, config, &cursor);
     _cursorsOut++;
     return cursor;
 }
 
-WT_CURSOR* WiredTigerSession::getReadOnceCursor(const std::string& uri, bool allowOverwrite) {
-    const char* config = allowOverwrite ? "read_once=true" : "read_once=true,overwrite=false";
-
-    WT_CURSOR* cursor = NULL;
+WT_CURSOR* WiredTigerSession::getNewCursor(const std::string& uri, const char* config) {
+    WT_CURSOR* cursor = nullptr;
     _openCursor(_session, uri, config, &cursor);
     _cursorsOut++;
     return cursor;
 }
 
 void WiredTigerSession::releaseCursor(uint64_t id, WT_CURSOR* cursor) {
-    // When releasing the cursor, we would want to check if the session cache is already in shutdown
-    // and prevent the race condition that the shutdown starts after the check.
-    WiredTigerSessionCache::BlockShutdown blockShutdown(_cache);
-
-    // Avoids the cursor already being destroyed during the shutdown.
-    if (_cache->isShuttingDown()) {
-        return;
-    }
-
     invariant(_session);
     invariant(cursor);
     _cursorsOut--;
@@ -229,7 +226,7 @@ void WiredTigerSessionCache::shuttingDown() {
     if (_shuttingDown.fetchAndBitOr(kShuttingDownMask) & kShuttingDownMask)
         return;
 
-    // Spin as long as there are threads blocking shutdown.
+    // Spin as long as there are threads in releaseSession
     while (_shuttingDown.load() != kShuttingDownMask) {
         sleepmillis(1);
     }
@@ -241,57 +238,105 @@ bool WiredTigerSessionCache::isShuttingDown() {
     return _shuttingDown.load() & kShuttingDownMask;
 }
 
-void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableCheckpoint) {
+void WiredTigerSessionCache::waitUntilDurable(OperationContext* opCtx,
+                                              Fsync syncType,
+                                              UseJournalListener useListener) {
     // For inMemory storage engines, the data is "as durable as it's going to get".
     // That is, a restart is equivalent to a complete node failure.
     if (isEphemeral()) {
+        auto journalListener = [&]() -> JournalListener* {
+            // The JournalListener may not be set immediately, so we must check under a mutex so as
+            // not to access the variable while setting a JournalListener. A JournalListener is only
+            // allowed to be set once, so using the pointer outside of a mutex is safe.
+            stdx::unique_lock<Latch> lk(_journalListenerMutex);
+            return _journalListener;
+        }();
+        if (journalListener && useListener == UseJournalListener::kUpdate) {
+            // Update the JournalListener before we return. Does a write while fetching the
+            // timestamp if primary. As far as listeners are concerned, all writes are as 'durable'
+            // as they are ever going to get on an inMemory storage engine.
+            auto token = _journalListener->getToken(opCtx);
+            journalListener->onDurable(token);
+        }
         return;
     }
 
-    BlockShutdown blockShutdown(this);
+    const int shuttingDown = _shuttingDown.fetchAndAdd(1);
+    ON_BLOCK_EXIT([this] { _shuttingDown.fetchAndSubtract(1); });
 
     uassert(ErrorCodes::ShutdownInProgress,
             "Cannot wait for durability because a shutdown is in progress",
-            !isShuttingDown());
+            !(shuttingDown & kShuttingDownMask));
 
     // Stable checkpoints are only meaningful in a replica set. Replication sets the "stable
     // timestamp". If the stable timestamp is unset, WiredTiger takes a full checkpoint, which is
     // incidentally what we want. A "true" stable checkpoint (a stable timestamp was set on the
     // WT_CONNECTION, i.e: replication is on) requires `forceCheckpoint` to be true and journaling
     // to be enabled.
-    if (stableCheckpoint && getGlobalReplSettings().usingReplSets()) {
-        invariant(forceCheckpoint && _engine->isDurable());
+    if (syncType == Fsync::kCheckpointStableTimestamp && getGlobalReplSettings().usingReplSets()) {
+        invariant(_engine->isDurable());
     }
 
     // When forcing a checkpoint with journaling enabled, don't synchronize with other
     // waiters, as a log flush is much cheaper than a full checkpoint.
-    if (forceCheckpoint && _engine->isDurable()) {
+    if ((syncType == Fsync::kCheckpointStableTimestamp || syncType == Fsync::kCheckpointAll) &&
+        _engine->isDurable()) {
         UniqueWiredTigerSession session = getSession();
         WT_SESSION* s = session->getSession();
         auto encryptionKeyDB = _engine->getEncryptionKeyDB();
         std::unique_ptr<WiredTigerSession> session2;
         WT_SESSION* s2 = nullptr;
         if (encryptionKeyDB) {
-            session2 = stdx::make_unique<WiredTigerSession>(encryptionKeyDB->getConnection());
+            session2 = std::make_unique<WiredTigerSession>(encryptionKeyDB->getConnection());
             s2 = session2->getSession();
         }
         {
-            stdx::unique_lock<Latch> lk(_journalListenerMutex);
-            JournalListener::Token token = _journalListener->getToken();
-#ifdef PERCONA_AUDIT_ENABLED
-            // Make audit log durable
-            audit::fsyncAuditLog();
-#endif
-            auto config = stableCheckpoint ? "use_timestamp=true" : "use_timestamp=false";
+            auto journalListener = [&]() -> JournalListener* {
+                // The JournalListener may not be set immediately, so we must check under a mutex so
+                // as not to access the variable while setting a JournalListener. A JournalListener
+                // is only allowed to be set once, so using the pointer outside of a mutex is safe.
+                stdx::unique_lock<Latch> lk(_journalListenerMutex);
+                return _journalListener;
+            }();
+            boost::optional<JournalListener::Token> token;
+            if (journalListener && useListener == UseJournalListener::kUpdate) {
+                // Update a persisted value with the latest write timestamp that is safe across
+                // startup recovery in the repl layer. Then report that timestamp as durable to the
+                // repl layer below after we have flushed in-memory data to disk.
+                // Note: only does a write if primary, otherwise just fetches the timestamp.
+                token = journalListener->getToken(opCtx);
+            }
+
+            auto config = syncType == Fsync::kCheckpointStableTimestamp ? "use_timestamp=true"
+                                                                        : "use_timestamp=false";
             invariantWTOK(s->checkpoint(s, config));
             if (s2)
                 invariantWTOK(s2->checkpoint(s2, config));
-            _journalListener->onDurable(token);
+
+            if (token) {
+                journalListener->onDurable(token.get());
+            }
         }
-        LOG(4) << "created checkpoint (forced)";
+        LOGV2_DEBUG(22418, 4, "created checkpoint (forced)");
         return;
     }
 
+    auto journalListener = [&]() -> JournalListener* {
+        // The JournalListener may not be set immediately, so we must check under a mutex so as not
+        // to access the variable while setting a JournalListener. A JournalListener is only allowed
+        // to be set once, so using the pointer outside of a mutex is safe.
+        stdx::unique_lock<Latch> lk(_journalListenerMutex);
+        return _journalListener;
+    }();
+    boost::optional<JournalListener::Token> token;
+    if (journalListener && useListener == UseJournalListener::kUpdate) {
+        // Update a persisted value with the latest write timestamp that is safe across startup
+        // recovery in the repl layer. Then report that timestamp as durable to the repl layer below
+        // after we have flushed in-memory data to disk.
+        // Note: only does a write if primary, otherwise just fetches the timestamp.
+        token = journalListener->getToken(opCtx);
+    }
+
     uint32_t start = _lastSyncTime.load();
     // Do the remainder in a critical section that ensures only a single thread at a time
     // will attempt to synchronize.
@@ -305,15 +350,10 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
 
     // Nobody has synched yet, so we have to sync ourselves.
 
-    // This gets the token (OpTime) from the last write, before flushing (either the journal, or a
-    // checkpoint), and then reports that token (OpTime) as a durable write.
-    stdx::unique_lock<Latch> jlk(_journalListenerMutex);
-    JournalListener::Token token = _journalListener->getToken();
-
     // Initialize on first use.
     if (!_waitUntilDurableSession) {
         invariantWTOK(
-            _conn->open_session(_conn, NULL, "isolation=snapshot", &_waitUntilDurableSession));
+            _conn->open_session(_conn, nullptr, "isolation=snapshot", &_waitUntilDurableSession));
     }
     if (!_keyDBSession) {
         auto encryptionKeyDB = _engine->getEncryptionKeyDB();
@@ -324,18 +364,13 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
         }
     }
 
-#ifdef PERCONA_AUDIT_ENABLED
-    // Make audit log durable
-    audit::fsyncAuditLog();
-#endif
-
     // Use the journal when available, or a checkpoint otherwise.
     if (_engine && _engine->isDurable()) {
         invariantWTOK(_waitUntilDurableSession->log_flush(_waitUntilDurableSession, "sync=on"));
-        LOG(4) << "flushed journal";
+        LOGV2_DEBUG(22419, 4, "flushed journal");
     } else {
-        invariantWTOK(_waitUntilDurableSession->checkpoint(_waitUntilDurableSession, NULL));
-        LOG(4) << "created checkpoint";
+        invariantWTOK(_waitUntilDurableSession->checkpoint(_waitUntilDurableSession, nullptr));
+        LOGV2_DEBUG(22420, 4, "created checkpoint");
     }
 
     // keyDB is always durable (opened with journal enabled)
@@ -343,7 +378,9 @@ void WiredTigerSessionCache::waitUntilDurable(bool forceCheckpoint, bool stableC
         invariantWTOK(_keyDBSession->log_flush(_keyDBSession, "sync=on"));
     }
 
-    _journalListener->onDurable(token);
+    if (token) {
+        journalListener->onDurable(token.get());
+    }
 }
 
 void WiredTigerSessionCache::waitUntilPreparedUnitOfWorkCommitsOrAborts(OperationContext* opCtx,
@@ -393,8 +430,6 @@ void WiredTigerSessionCache::closeExpiredIdleSessions(int64_t idleTimeMillis) {
     }
 
     auto cutoffTime = _clockSource->now() - Milliseconds(idleTimeMillis);
-    SessionCache sessionsToClose;
-
     {
         stdx::lock_guard<Latch> lock(_cacheLock);
         // Discard all sessions that became idle before the cutoff time
@@ -403,18 +438,12 @@ void WiredTigerSessionCache::closeExpiredIdleSessions(int64_t idleTimeMillis) {
             invariant(session->getIdleExpireTime() != Date_t::min());
             if (session->getIdleExpireTime() < cutoffTime) {
                 it = _sessions.erase(it);
-                sessionsToClose.push_back(session);
+                delete (session);
             } else {
                 ++it;
             }
         }
     }
-
-    // Closing expired idle sessions is expensive, so do it outside of the cache mutex. This helps
-    // to avoid periodic operation latency spikes as seen in SERVER-52879.
-    for (auto session : sessionsToClose) {
-        delete session;
-    }
 }
 
 void WiredTigerSessionCache::closeAll() {
@@ -461,12 +490,12 @@ UniqueWiredTigerSession WiredTigerSessionCache::getSession() {
 
 void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
     invariant(session);
-    // We might have skipped releasing some cursors during the shutdown.
-    invariant(session->cursorsOut() == 0 || isShuttingDown());
+    invariant(session->cursorsOut() == 0);
 
-    BlockShutdown blockShutdown(this);
+    const int shuttingDown = _shuttingDown.fetchAndAdd(1);
+    ON_BLOCK_EXIT([this] { _shuttingDown.fetchAndSubtract(1); });
 
-    if (isShuttingDown()) {
+    if (shuttingDown & kShuttingDownMask) {
         // There is a race condition with clean shutdown, where the storage engine is ripped from
         // underneath OperationContexts, which are not "active" (i.e., do not have any locks), but
         // are just about to delete the recovery unit. See SERVER-16031 for more information. Since
@@ -527,6 +556,11 @@ void WiredTigerSessionCache::releaseSession(WiredTigerSession* session) {
 
 void WiredTigerSessionCache::setJournalListener(JournalListener* jl) {
     stdx::unique_lock<Latch> lk(_journalListenerMutex);
+
+    // A JournalListener can only be set once. Otherwise, accessing a copy of the _journalListener
+    // pointer without a mutex would be unsafe.
+    invariant(!_journalListener);
+
     _journalListener = jl;
 }
 
