@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kStorage
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kStorage
 
 #include "mongo/platform/basic.h"
 
@@ -35,54 +35,21 @@
 
 #include <limits>
 
-#include <boost/filesystem.hpp>
-#include <boost/filesystem/path.hpp>
-
 #include "mongo/base/simple_string_data_comparator.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/snapshot_window_options.h"
-#include "mongo/db/storage/storage_file_util.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
-#include "mongo/db/storage/wiredtiger/wiredtiger_parameters_gen.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/fail_point.h"
-#include "mongo/util/log.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
 
-namespace {
-
-const std::string kTableChecksFileName = "_wt_table_checks";
-
-/**
- * Removes the 'kTableChecksFileName' file in the dbpath, if it exists.
- */
-void removeTableChecksFile() {
-    auto path = boost::filesystem::path(storageGlobalParams.dbpath) /
-        boost::filesystem::path(kTableChecksFileName);
-
-    if (!boost::filesystem::exists(path)) {
-        return;
-    }
-
-    boost::system::error_code errorCode;
-    boost::filesystem::remove(path, errorCode);
-
-    if (errorCode) {
-        log() << "Failed to remove file " << path.generic_string() << " because "
-              << errorCode.message();
-        fassertFailedNoTrace(4366403);
-    }
-}
-
-}  // namespace
-
 using std::string;
 
 Status wtRCToStatus_slow(int retCode, const char* prefix) {
@@ -148,7 +115,7 @@ StatusWith<std::string> _getMetadata(WT_CURSOR* cursor, StringData uri) {
     } else if (ret != 0) {
         return StatusWith<std::string>(wtRCToStatus(ret));
     }
-    const char* metadata = NULL;
+    const char* metadata = nullptr;
     ret = cursor->get_value(cursor, &metadata);
     if (ret != 0) {
         return StatusWith<std::string>(wtRCToStatus(ret));
@@ -171,9 +138,15 @@ StatusWith<std::string> WiredTigerUtil::getMetadataCreate(OperationContext* opCt
     invariant(opCtx);
 
     auto session = WiredTigerRecoveryUnit::get(opCtx)->getSessionNoTxn();
-    WT_CURSOR* cursor =
-        session->getCursor("metadata:create", WiredTigerSession::kMetadataCreateTableId, false);
 
+    WT_CURSOR* cursor = nullptr;
+    try {
+        cursor = session->getCachedCursor(
+            "metadata:create", WiredTigerSession::kMetadataCreateTableId, NULL);
+    } catch (const ExceptionFor<ErrorCodes::CursorNotFound>& ex) {
+        LOGV2_FATAL_NOTRACE(51257, "Cursor not found", "error"_attr = ex);
+    }
+    invariant(cursor);
     auto releaser = makeGuard(
         [&] { session->releaseCursor(WiredTigerSession::kMetadataCreateTableId, cursor); });
 
@@ -193,7 +166,13 @@ StatusWith<std::string> WiredTigerUtil::getMetadata(OperationContext* opCtx, Str
     invariant(opCtx);
 
     auto session = WiredTigerRecoveryUnit::get(opCtx)->getSessionNoTxn();
-    WT_CURSOR* cursor = session->getCursor("metadata:", WiredTigerSession::kMetadataTableId, false);
+    WT_CURSOR* cursor = nullptr;
+    try {
+        cursor = session->getCachedCursor("metadata:", WiredTigerSession::kMetadataTableId, NULL);
+    } catch (const ExceptionFor<ErrorCodes::CursorNotFound>& ex) {
+        LOGV2_FATAL_NOTRACE(31293, "Cursor not found", "error"_attr = ex);
+    }
+    invariant(cursor);
     auto releaser =
         makeGuard([&] { session->releaseCursor(WiredTigerSession::kMetadataTableId, cursor); });
 
@@ -310,9 +289,14 @@ StatusWith<int64_t> WiredTigerUtil::checkApplicationMetadataFormatVersion(Operat
                                     << " has unsupported format version: " << version << ".");
     }
 
-    LOG(2) << "WiredTigerUtil::checkApplicationMetadataFormatVersion "
-           << " uri: " << uri << " ok range " << minimumVersion << " -> " << maximumVersion
-           << " current: " << version;
+    LOGV2_DEBUG(22428,
+                2,
+                "WiredTigerUtil::checkApplicationMetadataFormatVersion  uri: {uri} ok range "
+                "{minimumVersion} -> {maximumVersion} current: {version}",
+                "uri"_attr = uri,
+                "minimumVersion"_attr = minimumVersion,
+                "maximumVersion"_attr = maximumVersion,
+                "version"_attr = version);
 
     return version;
 }
@@ -349,18 +333,18 @@ Status WiredTigerUtil::checkTableCreationOptions(const BSONElement& configElem)
 }
 
 // static
-StatusWith<uint64_t> WiredTigerUtil::getStatisticsValue(WT_SESSION* session,
-                                                        const std::string& uri,
-                                                        const std::string& config,
-                                                        int statisticsKey) {
+StatusWith<int64_t> WiredTigerUtil::getStatisticsValue(WT_SESSION* session,
+                                                       const std::string& uri,
+                                                       const std::string& config,
+                                                       int statisticsKey) {
     invariant(session);
-    WT_CURSOR* cursor = NULL;
-    const char* cursorConfig = config.empty() ? NULL : config.c_str();
-    int ret = session->open_cursor(session, uri.c_str(), NULL, cursorConfig, &cursor);
+    WT_CURSOR* cursor = nullptr;
+    const char* cursorConfig = config.empty() ? nullptr : config.c_str();
+    int ret = session->open_cursor(session, uri.c_str(), nullptr, cursorConfig, &cursor);
     if (ret != 0) {
-        return StatusWith<uint64_t>(ErrorCodes::CursorNotFound,
-                                    str::stream() << "unable to open cursor at URI " << uri
-                                                  << ". reason: " << wiredtiger_strerror(ret));
+        return StatusWith<int64_t>(ErrorCodes::CursorNotFound,
+                                   str::stream() << "unable to open cursor at URI " << uri
+                                                 << ". reason: " << wiredtiger_strerror(ret));
     }
     invariant(cursor);
     ON_BLOCK_EXIT([&] { cursor->close(cursor); });
@@ -368,26 +352,26 @@ StatusWith<uint64_t> WiredTigerUtil::getStatisticsValue(WT_SESSION* session,
     cursor->set_key(cursor, statisticsKey);
     ret = cursor->search(cursor);
     if (ret != 0) {
-        return StatusWith<uint64_t>(ErrorCodes::NoSuchKey,
-                                    str::stream()
-                                        << "unable to find key " << statisticsKey << " at URI "
-                                        << uri << ". reason: " << wiredtiger_strerror(ret));
+        return StatusWith<int64_t>(ErrorCodes::NoSuchKey,
+                                   str::stream()
+                                       << "unable to find key " << statisticsKey << " at URI "
+                                       << uri << ". reason: " << wiredtiger_strerror(ret));
     }
 
-    uint64_t value;
-    ret = cursor->get_value(cursor, NULL, NULL, &value);
+    int64_t value;
+    ret = cursor->get_value(cursor, nullptr, nullptr, &value);
     if (ret != 0) {
-        return StatusWith<uint64_t>(ErrorCodes::BadValue,
-                                    str::stream() << "unable to get value for key " << statisticsKey
-                                                  << " at URI " << uri
-                                                  << ". reason: " << wiredtiger_strerror(ret));
+        return StatusWith<int64_t>(ErrorCodes::BadValue,
+                                   str::stream() << "unable to get value for key " << statisticsKey
+                                                 << " at URI " << uri
+                                                 << ". reason: " << wiredtiger_strerror(ret));
     }
 
-    return StatusWith<uint64_t>(value);
+    return StatusWith<int64_t>(value);
 }
 
 int64_t WiredTigerUtil::getIdentSize(WT_SESSION* s, const std::string& uri) {
-    StatusWith<int64_t> result = WiredTigerUtil::getStatisticsValueAs<int64_t>(
+    StatusWith<int64_t> result = WiredTigerUtil::getStatisticsValue(
         s, "statistics:" + uri, "statistics=(size)", WT_STAT_DSRC_BLOCK_SIZE);
     const Status& status = result.getStatus();
     if (!status.isOK()) {
@@ -413,8 +397,11 @@ size_t WiredTigerUtil::getCacheSizeMB(double requestedCacheSizeGB) {
         cacheSizeMB = 1024 * requestedCacheSizeGB;
     }
     if (cacheSizeMB > kMaxSizeCacheMB) {
-        log() << "Requested cache size: " << cacheSizeMB << "MB exceeds max; setting to "
-              << kMaxSizeCacheMB << "MB";
+        LOGV2(22429,
+              "Requested cache size: {requestedMB}MB exceeds max; setting to {maximumMB}MB",
+              "Requested cache size exceeds max, setting to maximum",
+              "requestedMB"_attr = cacheSizeMB,
+              "maximumMB"_attr = kMaxSizeCacheMB);
         cacheSizeMB = kMaxSizeCacheMB;
     }
     return static_cast<size_t>(cacheSizeMB);
@@ -436,9 +423,21 @@ int mdb_handle_error_with_startup_suppression(WT_EVENT_HANDLER* handler,
             if (sd.find("Version incompatibility detected:") != std::string::npos) {
                 return 0;
             }
+
+            // WT shipped with MongoDB 4.4 can read data left behind by 4.0, but cannot write 4.0
+            // compatible data. Instead of forcing an upgrade on the user, it refuses to start up
+            // with this error string.
+            if (sd.find("WiredTiger version incompatible with current binary") !=
+                std::string::npos) {
+                wtHandler->setWtIncompatible();
+                return 0;
+            }
         }
-        error() << "WiredTiger error (" << errorCode << ") " << redact(message)
-                << " Raw: " << message;
+        LOGV2_ERROR(22435,
+                    "WiredTiger error ({error}) {message}",
+                    "WiredTiger error",
+                    "error"_attr = errorCode,
+                    "message"_attr = message);
 
         // Don't abort on WT_PANIC when repairing, as the error will be handled at a higher layer.
         if (storageGlobalParams.repair) {
@@ -456,7 +455,11 @@ int mdb_handle_error(WT_EVENT_HANDLER* handler,
                      int errorCode,
                      const char* message) {
     try {
-        error() << "WiredTiger error (" << errorCode << ") " << redact(message);
+        LOGV2_ERROR(22436,
+                    "WiredTiger error ({errorCode}) {message}",
+                    "WiredTiger error",
+                    "error"_attr = errorCode,
+                    "message"_attr = redact(message));
 
         // Don't abort on WT_PANIC when repairing, as the error will be handled at a higher layer.
         if (storageGlobalParams.repair) {
@@ -471,7 +474,7 @@ int mdb_handle_error(WT_EVENT_HANDLER* handler,
 
 int mdb_handle_message(WT_EVENT_HANDLER* handler, WT_SESSION* session, const char* message) {
     try {
-        log() << "WiredTiger message " << redact(message);
+        LOGV2(22430, "WiredTiger message", "message"_attr = redact(message));
     } catch (...) {
         std::terminate();
     }
@@ -483,7 +486,10 @@ int mdb_handle_progress(WT_EVENT_HANDLER* handler,
                         const char* operation,
                         uint64_t progress) {
     try {
-        log() << "WiredTiger progress " << redact(operation) << " " << progress;
+        LOGV2(22431,
+              "WiredTiger progress",
+              "operation"_attr = redact(operation),
+              "progress"_attr = progress);
     } catch (...) {
         std::terminate();
     }
@@ -553,15 +559,11 @@ int WiredTigerUtil::verifyTable(OperationContext* opCtx,
     // Open a new session with custom error handlers.
     WT_CONNECTION* conn = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache()->conn();
     WT_SESSION* session;
-    invariantWTOK(conn->open_session(conn, &eventHandler, NULL, &session));
+    invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session));
     ON_BLOCK_EXIT([&] { session->close(session, ""); });
 
     // Do the verify. Weird parens prevent treating "verify" as a macro.
-    return (session->verify)(session, uri.c_str(), NULL);
-}
-
-void WiredTigerUtil::notifyStartupComplete() {
-    removeTableChecksFile();
+    return (session->verify)(session, uri.c_str(), nullptr);
 }
 
 bool WiredTigerUtil::useTableLogging(NamespaceString ns, bool replEnabled) {
@@ -598,15 +600,13 @@ Status WiredTigerUtil::setTableLogging(OperationContext* opCtx, const std::strin
 }
 
 Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& uri, bool on) {
-    invariant(!storageGlobalParams.readOnly);
-
-    if (gWiredTigerSkipTableLoggingChecksOnStartup) {
-        LOG(1) << "Skipping table logging check for " << uri;
-        return Status::OK();
+    std::string setting;
+    if (on) {
+        setting = "log=(enabled=true)";
+    } else {
+        setting = "log=(enabled=false)";
     }
 
-    const std::string setting = on ? "log=(enabled=true)" : "log=(enabled=false)";
-
     // This method does some "weak" parsing to see if the table is in the expected logging
     // state. Only attempt to alter the table when a change is needed. This avoids grabbing heavy
     // locks in WT when creating new tables for collections and indexes. Those tables are created
@@ -628,13 +628,22 @@ Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& u
         return Status::OK();
     }
 
-    LOG(1) << "Changing table logging settings. Uri: " << uri << " Enable? " << on;
+    LOGV2_DEBUG(22432,
+                1,
+                "Changing table logging settings. Uri: {uri} Enable? {on}",
+                "uri"_attr = uri,
+                "on"_attr = on);
     int ret = session->alter(session, uri.c_str(), setting.c_str());
     if (ret) {
-        severe() << "Failed to update log setting. Uri: " << uri << " Enable? " << on
-                 << " Ret: " << ret << " MD: " << redact(existingMetadata)
-                 << " Msg: " << session->strerror(session, ret);
-        fassertFailed(50756);
+        LOGV2_FATAL(50756,
+                    "Failed to update log setting. Uri: {uri} Enable? {enable} Ret: {error} MD: "
+                    "{metadata} Msg: {message}",
+                    "Failed to update log setting",
+                    "uri"_attr = uri,
+                    "enable"_attr = on,
+                    "error"_attr = ret,
+                    "metadata"_attr = redact(existingMetadata),
+                    "message"_attr = session->strerror(session, ret));
     }
 
     return Status::OK();
@@ -654,9 +663,9 @@ Status WiredTigerUtil::exportTableToBSON(WT_SESSION* session,
                                          const std::vector<std::string>& filter) {
     invariant(session);
     invariant(bob);
-    WT_CURSOR* c = NULL;
-    const char* cursorConfig = config.empty() ? NULL : config.c_str();
-    int ret = session->open_cursor(session, uri.c_str(), NULL, cursorConfig, &c);
+    WT_CURSOR* c = nullptr;
+    const char* cursorConfig = config.empty() ? nullptr : config.c_str();
+    int ret = session->open_cursor(session, uri.c_str(), nullptr, cursorConfig, &c);
     if (ret != 0) {
         return Status(ErrorCodes::CursorNotFound,
                       str::stream() << "unable to open cursor at URI " << uri
@@ -669,7 +678,7 @@ Status WiredTigerUtil::exportTableToBSON(WT_SESSION* session,
     std::map<string, BSONObjBuilder*> subs;
     const char* desc;
     uint64_t value;
-    while (c->next(c) == 0 && c->get_value(c, &desc, NULL, &value) == 0) {
+    while (c->next(c) == 0 && c->get_value(c, &desc, nullptr, &value) == 0) {
         StringData key(desc);
 
         StringData prefix;
@@ -730,7 +739,7 @@ void WiredTigerUtil::appendSnapshotWindowSettings(WiredTigerKVEngine* engine,
     const unsigned currentAvailableSnapshotWindow =
         stableTimestamp.getSecs() - oldestTimestamp.getSecs();
 
-    int64_t score = uassertStatusOK(WiredTigerUtil::getStatisticsValueAs<int64_t>(
+    int64_t score = uassertStatusOK(WiredTigerUtil::getStatisticsValue(
         session->getSession(), "statistics:", "", WT_STAT_CONN_CACHE_LOOKASIDE_SCORE));
 
     auto totalNumberOfSnapshotTooOldErrors = snapshotWindowParams.snapshotTooOldErrorCount.load();
@@ -745,7 +754,7 @@ void WiredTigerUtil::appendSnapshotWindowSettings(WiredTigerKVEngine* engine,
     settings.append("target available snapshots window size in seconds",
                     snapshotWindowParams.targetSnapshotHistoryWindowInSeconds.load());
     settings.append("current available snapshots window size in seconds",
-                    currentAvailableSnapshotWindow);
+                    static_cast<int>(currentAvailableSnapshotWindow));
     settings.append("latest majority snapshot timestamp available",
                     stableTimestamp.toStringPretty());
     settings.append("oldest majority snapshot timestamp available",
