@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kReplication
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplication
 
 #include "mongo/platform/basic.h"
 
@@ -47,12 +47,15 @@ ReplicationCoordinatorEmbedded::ReplicationCoordinatorEmbedded(ServiceContext* s
 
 ReplicationCoordinatorEmbedded::~ReplicationCoordinatorEmbedded() = default;
 
-void ReplicationCoordinatorEmbedded::startup(OperationContext* opCtx) {}
+void ReplicationCoordinatorEmbedded::startup(
+    OperationContext* opCtx, LastStorageEngineShutdownState lastStorageEngineShutdownState) {}
 
 void ReplicationCoordinatorEmbedded::enterTerminalShutdown() {}
 
 void ReplicationCoordinatorEmbedded::shutdown(OperationContext* opCtx) {}
 
+void ReplicationCoordinatorEmbedded::markAsCleanShutdownIfPossible(OperationContext* opCtx) {}
+
 const ReplSettings& ReplicationCoordinatorEmbedded::getSettings() const {
     static ReplSettings _settings;
     return _settings;
@@ -77,12 +80,12 @@ bool ReplicationCoordinatorEmbedded::canAcceptWritesForDatabase_UNSAFE(Operation
 }
 
 bool ReplicationCoordinatorEmbedded::canAcceptWritesFor(OperationContext* opCtx,
-                                                        const NamespaceString& ns) {
+                                                        const NamespaceStringOrUUID& nsOrUUID) {
     return true;
 }
 
-bool ReplicationCoordinatorEmbedded::canAcceptWritesFor_UNSAFE(OperationContext* opCtx,
-                                                               const NamespaceString& ns) {
+bool ReplicationCoordinatorEmbedded::canAcceptWritesFor_UNSAFE(
+    OperationContext* opCtx, const NamespaceStringOrUUID& nsOrUUID) {
     return true;
 }
 
@@ -152,7 +155,7 @@ void ReplicationCoordinatorEmbedded::appendConnectionStats(
 }
 
 MemberState ReplicationCoordinatorEmbedded::getMemberState() const {
-    UASSERT_NOT_IMPLEMENTED;
+    return MemberState::RS_PRIMARY;
 }
 
 std::vector<repl::MemberData> ReplicationCoordinatorEmbedded::getMemberData() const {
@@ -179,8 +182,7 @@ Status ReplicationCoordinatorEmbedded::setFollowerMode(const MemberState&) {
     UASSERT_NOT_IMPLEMENTED;
 }
 
-Status ReplicationCoordinatorEmbedded::setFollowerModeStrict(OperationContext* opCtx,
-                                                             const MemberState&) {
+Status ReplicationCoordinatorEmbedded::setFollowerModeRollback(OperationContext* opCtx) {
     UASSERT_NOT_IMPLEMENTED;
 }
 
@@ -304,11 +306,6 @@ Status ReplicationCoordinatorEmbedded::processReplSetGetStatus(BSONObjBuilder*,
     UASSERT_NOT_IMPLEMENTED;
 }
 
-void ReplicationCoordinatorEmbedded::fillIsMasterForReplSet(IsMasterResponse*,
-                                                            const SplitHorizon::Parameters&) {
-    UASSERT_NOT_IMPLEMENTED;
-}
-
 void ReplicationCoordinatorEmbedded::appendSlaveInfoData(BSONObjBuilder*) {
     UASSERT_NOT_IMPLEMENTED;
 }
@@ -317,7 +314,8 @@ ReplSetConfig ReplicationCoordinatorEmbedded::getConfig() const {
     UASSERT_NOT_IMPLEMENTED;
 }
 
-void ReplicationCoordinatorEmbedded::processReplSetGetConfig(BSONObjBuilder*) {
+void ReplicationCoordinatorEmbedded::processReplSetGetConfig(BSONObjBuilder*,
+                                                             bool commitmentStatus) {
     UASSERT_NOT_IMPLEMENTED;
 }
 
@@ -329,7 +327,7 @@ void ReplicationCoordinatorEmbedded::cancelAndRescheduleElectionTimeout() {
     UASSERT_NOT_IMPLEMENTED;
 }
 
-Status ReplicationCoordinatorEmbedded::setMaintenanceMode(OperationContext*, bool) {
+Status ReplicationCoordinatorEmbedded::setMaintenanceMode(bool) {
     UASSERT_NOT_IMPLEMENTED;
 }
 
@@ -349,6 +347,17 @@ Status ReplicationCoordinatorEmbedded::processReplSetReconfig(OperationContext*,
     UASSERT_NOT_IMPLEMENTED;
 }
 
+Status ReplicationCoordinatorEmbedded::doReplSetReconfig(OperationContext* opCtx,
+                                                         GetNewConfigFn getNewConfig,
+                                                         bool force) {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
+Status ReplicationCoordinatorEmbedded::awaitConfigCommitment(OperationContext* opCtx,
+                                                             bool waitForOplogCommitment) {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
 Status ReplicationCoordinatorEmbedded::processReplSetInitiate(OperationContext*,
                                                               const BSONObj&,
                                                               BSONObjBuilder*) {
@@ -372,10 +381,6 @@ std::vector<HostAndPort> ReplicationCoordinatorEmbedded::getHostsWrittenTo(const
     UASSERT_NOT_IMPLEMENTED;
 }
 
-std::vector<HostAndPort> ReplicationCoordinatorEmbedded::getOtherNodesInReplSet() const {
-    UASSERT_NOT_IMPLEMENTED;
-}
-
 Status ReplicationCoordinatorEmbedded::checkIfWriteConcernCanBeSatisfied(
     const WriteConcernOptions&) const {
     UASSERT_NOT_IMPLEMENTED;
@@ -386,9 +391,8 @@ Status ReplicationCoordinatorEmbedded::checkIfCommitQuorumCanBeSatisfied(
     UASSERT_NOT_IMPLEMENTED;
 }
 
-StatusWith<bool> ReplicationCoordinatorEmbedded::checkIfCommitQuorumIsSatisfied(
-    const CommitQuorumOptions& commitQuorum,
-    const std::vector<HostAndPort>& commitReadyMembers) const {
+bool ReplicationCoordinatorEmbedded::isCommitQuorumSatisfied(
+    const CommitQuorumOptions& commitQuorum, const std::vector<mongo::HostAndPort>& members) const {
     UASSERT_NOT_IMPLEMENTED;
 }
 
@@ -408,8 +412,9 @@ void ReplicationCoordinatorEmbedded::resetLastOpTimesFromOplog(OperationContext*
     UASSERT_NOT_IMPLEMENTED;
 }
 
-bool ReplicationCoordinatorEmbedded::shouldChangeSyncSource(
-    const HostAndPort&, const rpc::ReplSetMetadata&, boost::optional<rpc::OplogQueryMetadata>) {
+bool ReplicationCoordinatorEmbedded::shouldChangeSyncSource(const HostAndPort&,
+                                                            const rpc::ReplSetMetadata&,
+                                                            const rpc::OplogQueryMetadata&) {
     UASSERT_NOT_IMPLEMENTED;
 }
 
@@ -447,7 +452,7 @@ Status ReplicationCoordinatorEmbedded::processHeartbeatV1(const ReplSetHeartbeat
     UASSERT_NOT_IMPLEMENTED;
 }
 
-long long ReplicationCoordinatorEmbedded::getTerm() {
+long long ReplicationCoordinatorEmbedded::getTerm() const {
     UASSERT_NOT_IMPLEMENTED;
 }
 
@@ -460,10 +465,6 @@ void ReplicationCoordinatorEmbedded::waitUntilSnapshotCommitted(OperationContext
     UASSERT_NOT_IMPLEMENTED;
 }
 
-size_t ReplicationCoordinatorEmbedded::getNumUncommittedSnapshots() {
-    UASSERT_NOT_IMPLEMENTED;
-}
-
 void ReplicationCoordinatorEmbedded::createWMajorityWriteAvailabilityDateWaiter(OpTime opTime) {
     UASSERT_NOT_IMPLEMENTED;
 }
@@ -492,6 +493,10 @@ void ReplicationCoordinatorEmbedded::attemptToAdvanceStableTimestamp() {
     UASSERT_NOT_IMPLEMENTED;
 }
 
+void ReplicationCoordinatorEmbedded::finishRecoveryIfEligible(OperationContext* opCtx) {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
 void ReplicationCoordinatorEmbedded::updateAndLogStateTransitionMetrics(
     const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
     const size_t numOpsKilled,
@@ -499,5 +504,51 @@ void ReplicationCoordinatorEmbedded::updateAndLogStateTransitionMetrics(
     UASSERT_NOT_IMPLEMENTED;
 }
 
+TopologyVersion ReplicationCoordinatorEmbedded::getTopologyVersion() const {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
+void ReplicationCoordinatorEmbedded::incrementTopologyVersion() {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
+std::shared_ptr<const repl::IsMasterResponse> ReplicationCoordinatorEmbedded::awaitIsMasterResponse(
+    OperationContext* opCtx,
+    const repl::SplitHorizon::Parameters& horizonParams,
+    boost::optional<TopologyVersion> previous,
+    boost::optional<Date_t> deadline) {
+    UASSERT_NOT_IMPLEMENTED;
+};
+
+SharedSemiFuture<std::shared_ptr<const IsMasterResponse>>
+ReplicationCoordinatorEmbedded::getIsMasterResponseFuture(
+    const SplitHorizon::Parameters& horizonParams,
+    boost::optional<TopologyVersion> clientTopologyVersion) {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
+StatusWith<OpTime> ReplicationCoordinatorEmbedded::getLatestWriteOpTime(
+    OperationContext* opCtx) const noexcept {
+    return getMyLastAppliedOpTime();
+}
+
+HostAndPort ReplicationCoordinatorEmbedded::getCurrentPrimaryHostAndPort() const {
+    UASSERT_NOT_IMPLEMENTED;
+}
+
+void ReplicationCoordinatorEmbedded::cancelCbkHandle(
+    executor::TaskExecutor::CallbackHandle activeHandle) {
+    MONGO_UNREACHABLE;
+}
+
+BSONObj ReplicationCoordinatorEmbedded::runCmdOnPrimaryAndAwaitResponse(
+    OperationContext* opCtx,
+    const std::string& dbName,
+    const BSONObj& cmdObj,
+    OnRemoteCmdScheduledFn onRemoteCmdScheduled,
+    OnRemoteCmdCompleteFn onRemoteCmdComplete) {
+    MONGO_UNREACHABLE;
+}
+
 }  // namespace embedded
 }  // namespace mongo
