@@ -27,10 +27,11 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
+#include <memory>
 #include <set>
 #include <string>
 #include <vector>
@@ -56,8 +57,6 @@
 #include "mongo/s/sharding_router_test_fixture.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/stdx/future.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 namespace {
@@ -288,7 +287,7 @@ TEST_F(InsertRetryTest, DuplicateKeyErrorAfterWriteConcernFailureMatch) {
         response.setStatus(Status::OK());
         response.setN(1);
 
-        auto wcError = stdx::make_unique<WriteConcernErrorDetail>();
+        auto wcError = std::make_unique<WriteConcernErrorDetail>();
 
         WriteConcernResult wcRes;
         wcRes.err = "timeout";
@@ -353,7 +352,7 @@ TEST_F(UpdateRetryTest, Success) {
     future.default_timed_get();
 }
 
-TEST_F(UpdateRetryTest, NotWritablePrimaryErrorReturnedPersistently) {
+TEST_F(UpdateRetryTest, NotMasterErrorReturnedPersistently) {
     configTargeter()->setFindHostReturnValue(HostAndPort("TestHost1"));
 
     BSONObj objToUpdate = BSON("_id" << 1 << "Value"
@@ -369,14 +368,13 @@ TEST_F(UpdateRetryTest, NotWritablePrimaryErrorReturnedPersistently) {
                                                   updateExpr,
                                                   false,
                                                   ShardingCatalogClient::kMajorityWriteConcern);
-        ASSERT_EQUALS(ErrorCodes::NotWritablePrimary, status);
+        ASSERT_EQUALS(ErrorCodes::NotMaster, status);
     });
 
     for (int i = 0; i < 3; ++i) {
         onCommand([](const RemoteCommandRequest& request) {
             BSONObjBuilder bb;
-            CommandHelpers::appendCommandStatusNoThrow(
-                bb, {ErrorCodes::NotWritablePrimary, "not master"});
+            CommandHelpers::appendCommandStatusNoThrow(bb, {ErrorCodes::NotMaster, "not master"});
             return bb.obj();
         });
     }
@@ -384,8 +382,8 @@ TEST_F(UpdateRetryTest, NotWritablePrimaryErrorReturnedPersistently) {
     future.default_timed_get();
 }
 
-TEST_F(UpdateRetryTest, NotWritablePrimaryReturnedFromTargeter) {
-    configTargeter()->setFindHostReturnValue(Status(ErrorCodes::NotWritablePrimary, "not master"));
+TEST_F(UpdateRetryTest, NotMasterReturnedFromTargeter) {
+    configTargeter()->setFindHostReturnValue(Status(ErrorCodes::NotMaster, "not master"));
 
     BSONObj objToUpdate = BSON("_id" << 1 << "Value"
                                      << "TestValue");
@@ -400,13 +398,13 @@ TEST_F(UpdateRetryTest, NotWritablePrimaryReturnedFromTargeter) {
                                                   updateExpr,
                                                   false,
                                                   ShardingCatalogClient::kMajorityWriteConcern);
-        ASSERT_EQUALS(ErrorCodes::NotWritablePrimary, status);
+        ASSERT_EQUALS(ErrorCodes::NotMaster, status);
     });
 
     future.default_timed_get();
 }
 
-TEST_F(UpdateRetryTest, NotWritablePrimaryOnceSuccessAfterRetry) {
+TEST_F(UpdateRetryTest, NotMasterOnceSuccessAfterRetry) {
     HostAndPort host1("TestHost1");
     HostAndPort host2("TestHost2");
     configTargeter()->setFindHostReturnValue(host1);
@@ -437,12 +435,11 @@ TEST_F(UpdateRetryTest, NotWritablePrimaryOnceSuccessAfterRetry) {
         ASSERT_EQUALS(host1, request.target);
 
         // Ensure that when the catalog manager tries to retarget after getting the
-        // NotWritablePrimary response, it will get back a new target.
+        // NotMaster response, it will get back a new target.
         configTargeter()->setFindHostReturnValue(host2);
 
         BSONObjBuilder bb;
-        CommandHelpers::appendCommandStatusNoThrow(bb,
-                                                   {ErrorCodes::NotWritablePrimary, "not master"});
+        CommandHelpers::appendCommandStatusNoThrow(bb, {ErrorCodes::NotMaster, "not master"});
         return bb.obj();
     });
 
@@ -488,7 +485,7 @@ TEST_F(UpdateRetryTest, OperationInterruptedDueToPrimaryStepDown) {
         BatchedCommandResponse response;
         response.setStatus(Status::OK());
 
-        auto writeErrDetail = stdx::make_unique<WriteErrorDetail>();
+        auto writeErrDetail = std::make_unique<WriteErrorDetail>();
         writeErrDetail->setIndex(0);
         writeErrDetail->setStatus(
             {ErrorCodes::InterruptedDueToReplStateChange, "Operation interrupted"});
@@ -540,7 +537,7 @@ TEST_F(UpdateRetryTest, WriteConcernFailure) {
         response.setStatus(Status::OK());
         response.setNModified(1);
 
-        auto wcError = stdx::make_unique<WriteConcernErrorDetail>();
+        auto wcError = std::make_unique<WriteConcernErrorDetail>();
 
         WriteConcernResult wcRes;
         wcRes.err = "timeout";
