@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -28,16 +27,19 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
+#include "mongo/db/concurrency/locker_noop.h"
 #include "mongo/db/query/query_request.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_database.h"
 #include "mongo/s/catalog_cache.h"
 #include "mongo/s/catalog_cache_test_fixture.h"
+#include "mongo/s/database_version_helpers.h"
+#include "mongo/unittest/death_test.h"
 
 namespace mongo {
 namespace {
@@ -57,7 +59,7 @@ protected:
 
     void expectGetDatabase() {
         expectFindSendBSONObjVector(kConfigHostAndPort, [&]() {
-            DatabaseType db(kNss.db().toString(), {"0"}, true);
+            DatabaseType db(kNss.db().toString(), {"0"}, true, databaseVersion::makeNew());
             return std::vector<BSONObj>{db.toBSON()};
         }());
     }
@@ -79,7 +81,7 @@ TEST_F(CatalogCacheRefreshTest, FullLoad) {
     const OID epoch = OID::gen();
     const ShardKeyPattern shardKeyPattern(BSON("_id" << 1));
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     expectGetDatabase();
     expectGetCollection(epoch, shardKeyPattern);
@@ -92,18 +94,22 @@ TEST_F(CatalogCacheRefreshTest, FullLoad) {
                          {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << -100)},
                          version,
                          {"0"});
+        chunk1.setName(OID::gen());
         version.incMinor();
 
         ChunkType chunk2(kNss, {BSON("_id" << -100), BSON("_id" << 0)}, version, {"1"});
+        chunk2.setName(OID::gen());
         version.incMinor();
 
         ChunkType chunk3(kNss, {BSON("_id" << 0), BSON("_id" << 100)}, version, {"0"});
+        chunk3.setName(OID::gen());
         version.incMinor();
 
         ChunkType chunk4(kNss,
                          {BSON("_id" << 100), shardKeyPattern.getKeyPattern().globalMax()},
                          version,
                          {"1"});
+        chunk4.setName(OID::gen());
         version.incMinor();
 
         return std::vector<BSONObj>{chunk1.toConfigBSON(),
@@ -119,8 +125,35 @@ TEST_F(CatalogCacheRefreshTest, FullLoad) {
     ASSERT_EQ(4, cm->numChunks());
 }
 
+TEST_F(CatalogCacheRefreshTest, NoLoadIfShardNotMarkedStaleInOperationContext) {
+    const ShardKeyPattern shardKeyPattern(BSON("_id" << 1));
+    auto initialRoutingInfo(
+        makeChunkManager(kNss, shardKeyPattern, nullptr, true, {BSON("_id" << 0)}));
+    ASSERT_EQ(2, initialRoutingInfo->numChunks());
+
+    auto futureNoRefresh = scheduleRoutingInfoUnforcedRefresh(kNss);
+    auto routingInfo = futureNoRefresh.default_timed_get();
+    ASSERT(routingInfo->cm());
+    auto cm = routingInfo->cm();
+    ASSERT_EQ(2, cm->numChunks());
+}
+
+class MockLockerAlwaysReportsToBeLocked : public LockerNoop {
+public:
+    using LockerNoop::LockerNoop;
+
+    bool isLocked() const final {
+        return true;
+    }
+};
+
+DEATH_TEST_F(CatalogCacheRefreshTest, ShouldFailToRefreshWhenLocksAreHeld, "Invariant") {
+    operationContext()->setLockState(std::make_unique<MockLockerAlwaysReportsToBeLocked>());
+    scheduleRoutingInfoUnforcedRefresh(kNss);
+}
+
 TEST_F(CatalogCacheRefreshTest, DatabaseNotFound) {
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     // Return an empty database (need to return it twice because for missing databases, the
     // CatalogClient tries twice)
@@ -140,7 +173,7 @@ TEST_F(CatalogCacheRefreshTest, DatabaseNotFound) {
 }
 
 TEST_F(CatalogCacheRefreshTest, DatabaseBSONCorrupted) {
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     // Return a corrupted database entry
     expectFindSendBSONObjVector(kConfigHostAndPort,
@@ -161,7 +194,7 @@ TEST_F(CatalogCacheRefreshTest, DatabaseBSONCorrupted) {
 }
 
 TEST_F(CatalogCacheRefreshTest, CollectionNotFound) {
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     expectGetDatabase();
 
@@ -175,7 +208,7 @@ TEST_F(CatalogCacheRefreshTest, CollectionNotFound) {
 }
 
 TEST_F(CatalogCacheRefreshTest, CollectionBSONCorrupted) {
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     expectGetDatabase();
 
@@ -201,7 +234,7 @@ TEST_F(CatalogCacheRefreshTest, NoChunksFoundForCollection) {
     const OID epoch = OID::gen();
     const ShardKeyPattern shardKeyPattern(BSON("_id" << 1));
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     expectGetDatabase();
     expectGetCollection(epoch, shardKeyPattern);
@@ -232,7 +265,7 @@ TEST_F(CatalogCacheRefreshTest, ChunksBSONCorrupted) {
     const OID epoch = OID::gen();
     const ShardKeyPattern shardKeyPattern(BSON("_id" << 1));
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     expectGetDatabase();
     expectGetCollection(epoch, shardKeyPattern);
@@ -266,7 +299,7 @@ TEST_F(CatalogCacheRefreshTest, IncompleteChunksFoundForCollection) {
     const OID epoch = OID::gen();
     const ShardKeyPattern shardKeyPattern(BSON("_id" << 1));
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoUnforcedRefresh(kNss);
 
     expectGetDatabase();
     expectGetCollection(epoch, shardKeyPattern);
@@ -279,15 +312,18 @@ TEST_F(CatalogCacheRefreshTest, IncompleteChunksFoundForCollection) {
         version.incMinor();
 
         ChunkType chunk2(kNss, {BSON("_id" << -100), BSON("_id" << 0)}, version, {"1"});
+        chunk2.setName(OID::gen());
         version.incMinor();
 
         ChunkType chunk3(kNss, {BSON("_id" << 0), BSON("_id" << 100)}, version, {"0"});
+        chunk3.setName(OID::gen());
         version.incMinor();
 
         ChunkType chunk4(kNss,
                          {BSON("_id" << 100), shardKeyPattern.getKeyPattern().globalMax()},
                          version,
                          {"1"});
+        chunk4.setName(OID::gen());
         version.incMinor();
 
         return std::vector<BSONObj>{
@@ -324,7 +360,7 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoad) {
     auto initialRoutingInfo(makeChunkManager(kNss, shardKeyPattern, nullptr, true, {}));
     ASSERT_EQ(1, initialRoutingInfo->numChunks());
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoForcedRefresh(kNss);
 
     ChunkVersion version = initialRoutingInfo->getVersion();
 
@@ -332,11 +368,13 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoad) {
         version.incMajor();
         ChunkType chunk1(
             kNss, {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << 0)}, version, {"0"});
+        chunk1.setName(OID::gen());
 
         ChunkType chunk2(kNss,
                          {BSON("_id" << 0), shardKeyPattern.getKeyPattern().globalMax()},
                          ChunkVersion(1, 0, OID::gen()),
                          {"1"});
+        chunk2.setName(OID::gen());
 
         return std::vector<BSONObj>{chunk1.toConfigBSON(), chunk2.toConfigBSON()};
     }();
@@ -345,7 +383,6 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoad) {
     // frequently the catalog cache retries.
     expectGetCollection(initialRoutingInfo->getVersion().epoch(), shardKeyPattern);
     expectFindSendBSONObjVector(kConfigHostAndPort, inconsistentChunks);
-
     expectGetCollection(initialRoutingInfo->getVersion().epoch(), shardKeyPattern);
     expectFindSendBSONObjVector(kConfigHostAndPort, inconsistentChunks);
 
@@ -373,7 +410,7 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoadRecoveryAft
 
     setupNShards(2);
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoForcedRefresh(kNss);
 
     ChunkVersion oldVersion = initialRoutingInfo->getVersion();
     const OID newEpoch = OID::gen();
@@ -396,6 +433,7 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoadRecoveryAft
                          {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << 0)},
                          oldVersion,
                          {"0"});
+        chunk1.setName(OID::gen());
 
         // "Yield" happens here with drop and recreate in between. This is the "last" chunk from the
         // recreated collection.
@@ -403,6 +441,7 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoadRecoveryAft
                          {BSON("_id" << 100), shardKeyPattern.getKeyPattern().globalMax()},
                          ChunkVersion(5, 2, newEpoch),
                          {"1"});
+        chunk3.setName(OID::gen());
 
         return std::vector<BSONObj>{chunk1.toConfigBSON(), chunk3.toConfigBSON()};
     });
@@ -423,15 +462,18 @@ TEST_F(CatalogCacheRefreshTest, ChunkEpochChangeDuringIncrementalLoadRecoveryAft
                          {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << 0)},
                          newVersion,
                          {"0"});
+        chunk1.setName(OID::gen());
 
         newVersion.incMinor();
         ChunkType chunk2(kNss, {BSON("_id" << 0), BSON("_id" << 100)}, newVersion, {"0"});
+        chunk2.setName(OID::gen());
 
         newVersion.incMinor();
         ChunkType chunk3(kNss,
                          {BSON("_id" << 100), shardKeyPattern.getKeyPattern().globalMax()},
                          newVersion,
                          {"1"});
+        chunk3.setName(OID::gen());
 
         return std::vector<BSONObj>{
             chunk1.toConfigBSON(), chunk2.toConfigBSON(), chunk3.toConfigBSON()};
@@ -455,7 +497,7 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterCollectionEpochChange) {
 
     setupNShards(2);
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoForcedRefresh(kNss);
 
     ChunkVersion newVersion(1, 0, OID::gen());
 
@@ -474,12 +516,14 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterCollectionEpochChange) {
                          {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << 0)},
                          newVersion,
                          {"0"});
+        chunk1.setName(OID::gen());
         newVersion.incMinor();
 
         ChunkType chunk2(kNss,
                          {BSON("_id" << 0), shardKeyPattern.getKeyPattern().globalMax()},
                          newVersion,
                          {"1"});
+        chunk2.setName(OID::gen());
 
         return std::vector<BSONObj>{chunk1.toConfigBSON(), chunk2.toConfigBSON()};
     });
@@ -502,7 +546,7 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterSplit) {
 
     ChunkVersion version = initialRoutingInfo->getVersion();
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoForcedRefresh(kNss);
 
     expectGetCollection(version.epoch(), shardKeyPattern);
 
@@ -519,10 +563,12 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterSplit) {
         version.incMajor();
         ChunkType chunk1(
             kNss, {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << 0)}, version, {"0"});
+        chunk1.setName(OID::gen());
 
         version.incMinor();
         ChunkType chunk2(
             kNss, {BSON("_id" << 0), shardKeyPattern.getKeyPattern().globalMax()}, version, {"0"});
+        chunk2.setName(OID::gen());
 
         return std::vector<BSONObj>{chunk1.toConfigBSON(), chunk2.toConfigBSON()};
     });
@@ -546,7 +592,7 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterMove) {
 
     ChunkVersion version = initialRoutingInfo->getVersion();
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoForcedRefresh(kNss);
 
     ChunkVersion expectedDestShardVersion;
 
@@ -558,10 +604,12 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterMove) {
         expectedDestShardVersion = version;
         ChunkType chunk1(
             kNss, {shardKeyPattern.getKeyPattern().globalMin(), BSON("_id" << 0)}, version, {"1"});
+        chunk1.setName(OID::gen());
 
         version.incMinor();
         ChunkType chunk2(
             kNss, {BSON("_id" << 0), shardKeyPattern.getKeyPattern().globalMax()}, version, {"0"});
+        chunk2.setName(OID::gen());
 
         return std::vector<BSONObj>{chunk1.toConfigBSON(), chunk2.toConfigBSON()};
     }());
@@ -586,7 +634,7 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterMoveLastChunk) {
 
     ChunkVersion version = initialRoutingInfo->getVersion();
 
-    auto future = scheduleRoutingInfoRefresh(kNss);
+    auto future = scheduleRoutingInfoForcedRefresh(kNss);
 
     expectGetCollection(version.epoch(), shardKeyPattern);
 
@@ -598,6 +646,7 @@ TEST_F(CatalogCacheRefreshTest, IncrementalLoadAfterMoveLastChunk) {
                           shardKeyPattern.getKeyPattern().globalMax()},
                          version,
                          {"1"});
+        chunk1.setName(OID::gen());
 
         return std::vector<BSONObj>{chunk1.toConfigBSON()};
     }());
