@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kCommand
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kCommand
 
 #include "mongo/platform/basic.h"
 
@@ -38,9 +38,11 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/curop.h"
+#include "mongo/db/error_labels.h"
 #include "mongo/db/logical_clock.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/query/cursor_response.h"
+#include "mongo/db/repl/read_concern_args.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/rpc/write_concern_error_detail.h"
@@ -54,7 +56,6 @@
 #include "mongo/s/shard_id.h"
 #include "mongo/s/stale_exception.h"
 #include "mongo/s/transaction_router.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
@@ -96,23 +97,11 @@ std::unique_ptr<WriteConcernErrorDetail> getWriteConcernErrorDetailFromBSONObj(c
         }
     }
 
-    return stdx::make_unique<WriteConcernErrorDetail>(getWriteConcernErrorDetail(wcErrorElem));
+    return std::make_unique<WriteConcernErrorDetail>(getWriteConcernErrorDetail(wcErrorElem));
 }
 
 namespace {
 
-BSONObj appendDbVersionIfPresent(BSONObj cmdObj, const CachedDatabaseInfo& dbInfo) {
-    // Attach the databaseVersion if we have one cached for the database.
-    auto dbVersion = dbInfo.databaseVersion();
-    if (databaseVersion::isFixed(dbVersion)) {
-        return cmdObj;
-    }
-
-    BSONObjBuilder cmdWithVersionBob(std::move(cmdObj));
-    cmdWithVersionBob.append("databaseVersion", dbVersion.toBSON());
-    return cmdWithVersionBob.obj();
-}
-
 const auto kAllowImplicitCollectionCreation = "allowImplicitCollectionCreation"_sd;
 
 /**
@@ -122,9 +111,6 @@ const auto kAllowImplicitCollectionCreation = "allowImplicitCollectionCreation"_
 std::vector<AsyncRequestsSender::Request> buildUnversionedRequestsForShards(
     OperationContext* opCtx, std::vector<ShardId> shardIds, const BSONObj& cmdObj) {
     auto cmdToSend = cmdObj;
-    if (!cmdToSend.hasField(kAllowImplicitCollectionCreation)) {
-        cmdToSend = appendAllowImplicitCreate(cmdToSend, false);
-    }
 
     std::vector<AsyncRequestsSender::Request> requests;
     for (auto&& shardId : shardIds)
@@ -140,55 +126,13 @@ std::vector<AsyncRequestsSender::Request> buildUnversionedRequestsForAllShards(
     return buildUnversionedRequestsForShards(opCtx, std::move(shardIds), cmdObj);
 }
 
-std::vector<AsyncRequestsSender::Request> buildVersionedRequestsForTargetedShards(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const CachedCollectionRoutingInfo& routingInfo,
-    const BSONObj& cmdObj,
-    const BSONObj& query,
-    const BSONObj& collation) {
-    if (!routingInfo.cm()) {
-        // The collection is unsharded. Target only the primary shard for the database.
-
-        // Attach shardVersion "UNSHARDED", unless targeting the config server.
-        const auto cmdObjWithShardVersion = (routingInfo.db().primaryId() != "config")
-            ? appendShardVersion(cmdObj, ChunkVersion::UNSHARDED())
-            : cmdObj;
-
-        return buildUnversionedRequestsForShards(
-            opCtx,
-            {routingInfo.db().primaryId()},
-            appendDbVersionIfPresent(cmdObjWithShardVersion, routingInfo.db()));
-    }
-
-    auto cmdToSend = cmdObj;
-    if (!cmdToSend.hasField(kAllowImplicitCollectionCreation)) {
-        cmdToSend = appendAllowImplicitCreate(cmdToSend, false);
-    }
-
-    std::vector<AsyncRequestsSender::Request> requests;
-
-    // The collection is sharded. Target all shards that own chunks that match the query.
-    std::set<ShardId> shardIds;
-    routingInfo.cm()->getShardIdsForQuery(opCtx, query, collation, &shardIds);
-
-    for (const ShardId& shardId : shardIds) {
-        requests.emplace_back(shardId,
-                              appendShardVersion(cmdToSend, routingInfo.cm()->getVersion(shardId)));
-    }
-
-    return requests;
-}
-
-}  // namespace
-
-std::vector<AsyncRequestsSender::Response> gatherResponses(
+std::vector<AsyncRequestsSender::Response> gatherResponsesImpl(
     OperationContext* opCtx,
     StringData dbName,
     const ReadPreferenceSetting& readPref,
     Shard::RetryPolicy retryPolicy,
     const std::vector<AsyncRequestsSender::Request>& requests,
-    const std::set<ErrorCodes::Error>& ignorableErrors) {
+    bool throwOnStaleShardVersionErrors) {
 
     // Send the requests.
     MultiStatementTransactionRequestsSender ars(
@@ -214,8 +158,11 @@ std::vector<AsyncRequestsSender::Response> gatherResponses(
             auto& responseObj = response.swResponse.getValue().data;
             status = getStatusFromCommandResult(responseObj);
 
-            // Failing to establish a consistent shardVersion means no results should be examined.
-            if (ErrorCodes::isStaleShardVersionError(status.code())) {
+            // If we specify to throw on stale shard version errors, then we will early exit
+            // from examining results. Otherwise, we will allow stale shard version errors to
+            // accumulate in the list of results.
+            if (throwOnStaleShardVersionErrors &&
+                ErrorCodes::isStaleShardVersionError(status.code())) {
                 uassertStatusOK(status.withContext(str::stream()
                                                    << "got stale shardVersion response from shard "
                                                    << response.shardId << " at host "
@@ -236,21 +183,46 @@ std::vector<AsyncRequestsSender::Response> gatherResponses(
             if (ErrorCodes::CommandOnShardedViewNotSupportedOnMongod == status) {
                 uassertStatusOK(status);
             }
-
-            // TODO: This should not be needed once we get better targetting with SERVER-32723.
-            // Some commands are sent with allowImplicit: false to all shards and expect only some
-            // of them to succeed.
-            if (ignorableErrors.find(ErrorCodes::CannotImplicitlyCreateCollection) ==
-                    ignorableErrors.end() &&
-                ErrorCodes::CannotImplicitlyCreateCollection == status) {
-                uassertStatusOK(status);
-            }
         }
         responses.push_back(std::move(response));
     }
 
     return responses;
 }
+}  // namespace
+
+std::vector<AsyncRequestsSender::Response> gatherResponses(
+    OperationContext* opCtx,
+    StringData dbName,
+    const ReadPreferenceSetting& readPref,
+    Shard::RetryPolicy retryPolicy,
+    const std::vector<AsyncRequestsSender::Request>& requests) {
+    return gatherResponsesImpl(
+        opCtx, dbName, readPref, retryPolicy, requests, true /* throwOnStaleShardVersionErrors */);
+}
+
+std::vector<AsyncRequestsSender::Response> gatherResponsesNoThrowOnStaleShardVersionErrors(
+    OperationContext* opCtx,
+    StringData dbName,
+    const ReadPreferenceSetting& readPref,
+    Shard::RetryPolicy retryPolicy,
+    const std::vector<AsyncRequestsSender::Request>& requests) {
+    return gatherResponsesImpl(
+        opCtx, dbName, readPref, retryPolicy, requests, false /* throwOnStaleShardVersionErrors */);
+}
+
+BSONObj appendDbVersionIfPresent(BSONObj cmdObj, const CachedDatabaseInfo& dbInfo) {
+    return appendDbVersionIfPresent(std::move(cmdObj), dbInfo.databaseVersion());
+}
+
+BSONObj appendDbVersionIfPresent(BSONObj cmdObj, DatabaseVersion dbVersion) {
+    if (databaseVersion::isFixed(dbVersion)) {
+        return cmdObj;
+    }
+    BSONObjBuilder cmdWithDbVersion(std::move(cmdObj));
+    cmdWithDbVersion.append("databaseVersion", dbVersion.toBSON());
+    return cmdWithDbVersion.obj();
+}
 
 BSONObj appendShardVersion(BSONObj cmdObj, ChunkVersion version) {
     BSONObjBuilder cmdWithVersionBob(std::move(cmdObj));
@@ -264,6 +236,81 @@ BSONObj appendAllowImplicitCreate(BSONObj cmdObj, bool allow) {
     return newCmdBuilder.obj();
 }
 
+BSONObj applyReadWriteConcern(OperationContext* opCtx,
+                              bool appendRC,
+                              bool appendWC,
+                              const BSONObj& cmdObj) {
+    if (TransactionRouter::get(opCtx)) {
+        // When running in a transaction, the rules are:
+        // - Never apply writeConcern.  Applying writeConcern to terminal operations such as
+        //   abortTransaction and commitTransaction is done directly by the TransactionRouter.
+        // - Apply readConcern only if this is the first operation in the transaction.
+
+        if (!opCtx->isStartingMultiDocumentTransaction()) {
+            // Cannot apply either read or writeConcern, so short-circuit.
+            return cmdObj;
+        }
+
+        if (!appendRC) {
+            // First operation in transaction, but the caller has not requested readConcern be
+            // applied, so there's nothing to do.
+            return cmdObj;
+        }
+
+        // First operation in transaction, so ensure that writeConcern is not applied, then continue
+        // and apply the readConcern.
+        appendWC = false;
+    }
+
+    // Append all original fields except the readConcern/writeConcern field to the new command.
+    BSONObjBuilder output;
+    bool seenReadConcern = false;
+    bool seenWriteConcern = false;
+    for (const auto& elem : cmdObj) {
+        const auto name = elem.fieldNameStringData();
+        if (appendRC && name == repl::ReadConcernArgs::kReadConcernFieldName) {
+            seenReadConcern = true;
+        }
+        if (appendWC && name == WriteConcernOptions::kWriteConcernField) {
+            seenWriteConcern = true;
+        }
+        if (!output.hasField(name)) {
+            output.append(elem);
+        }
+    }
+
+    // Finally, add the new read/write concern.
+    if (appendRC && !seenReadConcern) {
+        const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
+        output.appendElements(readConcernArgs.toBSON());
+    }
+    if (appendWC && !seenWriteConcern) {
+        output.append(WriteConcernOptions::kWriteConcernField, opCtx->getWriteConcern().toBSON());
+    }
+
+    return output.obj();
+}
+
+BSONObj applyReadWriteConcern(OperationContext* opCtx,
+                              CommandInvocation* invocation,
+                              const BSONObj& cmdObj) {
+    const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
+    const auto readConcernSupport = invocation->supportsReadConcern(readConcernArgs.getLevel());
+    return applyReadWriteConcern(opCtx,
+                                 readConcernSupport.readConcernSupport.isOK(),
+                                 invocation->supportsWriteConcern(),
+                                 cmdObj);
+}
+
+BSONObj applyReadWriteConcern(OperationContext* opCtx, BasicCommand* cmd, const BSONObj& cmdObj) {
+    const auto& readConcernArgs = repl::ReadConcernArgs::get(opCtx);
+    const auto readConcernSupport = cmd->supportsReadConcern(cmdObj, readConcernArgs.getLevel());
+    return applyReadWriteConcern(opCtx,
+                                 readConcernSupport.readConcernSupport.isOK(),
+                                 cmd->supportsWriteConcern(cmdObj),
+                                 cmdObj);
+}
+
 BSONObj stripWriteConcern(const BSONObj& cmdObj) {
     BSONObjBuilder output;
     for (const auto& elem : cmdObj) {
@@ -276,6 +323,53 @@ BSONObj stripWriteConcern(const BSONObj& cmdObj) {
     return output.obj();
 }
 
+std::vector<AsyncRequestsSender::Request> buildVersionedRequestsForTargetedShards(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const CachedCollectionRoutingInfo& routingInfo,
+    const std::set<ShardId>& shardsToSkip,
+    const BSONObj& cmdObj,
+    const BSONObj& query,
+    const BSONObj& collation) {
+
+    auto cmdToSend = cmdObj;
+
+    if (!routingInfo.cm()) {
+        // The collection is unsharded. Target only the primary shard for the database.
+
+        const auto primaryShardId = routingInfo.db().primaryId();
+
+        if (shardsToSkip.find(primaryShardId) != shardsToSkip.end()) {
+            return {};
+        }
+
+        // Attach shardVersion "UNSHARDED", unless targeting the config server.
+        const auto cmdObjWithShardVersion = (primaryShardId != "config")
+            ? appendShardVersion(cmdToSend, ChunkVersion::UNSHARDED())
+            : cmdToSend;
+
+        return buildUnversionedRequestsForShards(
+            opCtx,
+            {primaryShardId},
+            appendDbVersionIfPresent(cmdObjWithShardVersion, routingInfo.db()));
+    }
+
+    std::vector<AsyncRequestsSender::Request> requests;
+
+    // The collection is sharded. Target all shards that own chunks that match the query.
+    std::set<ShardId> shardIds;
+    routingInfo.cm()->getShardIdsForQuery(opCtx, query, collation, &shardIds);
+
+    for (const ShardId& shardId : shardIds) {
+        if (shardsToSkip.find(shardId) == shardsToSkip.end()) {
+            requests.emplace_back(
+                shardId, appendShardVersion(cmdToSend, routingInfo.cm()->getVersion(shardId)));
+        }
+    }
+
+    return requests;
+}
+
 std::vector<AsyncRequestsSender::Response> scatterGatherUnversionedTargetAllShards(
     OperationContext* opCtx,
     StringData dbName,
@@ -296,12 +390,31 @@ std::vector<AsyncRequestsSender::Response> scatterGatherVersionedTargetByRouting
     Shard::RetryPolicy retryPolicy,
     const BSONObj& query,
     const BSONObj& collation) {
-    const auto requests =
-        buildVersionedRequestsForTargetedShards(opCtx, nss, routingInfo, cmdObj, query, collation);
+    const auto requests = buildVersionedRequestsForTargetedShards(
+        opCtx, nss, routingInfo, {} /* shardsToSkip */, cmdObj, query, collation);
 
     return gatherResponses(opCtx, dbName, readPref, retryPolicy, requests);
 }
 
+std::vector<AsyncRequestsSender::Response>
+scatterGatherVersionedTargetByRoutingTableNoThrowOnStaleShardVersionErrors(
+    OperationContext* opCtx,
+    StringData dbName,
+    const NamespaceString& nss,
+    const CachedCollectionRoutingInfo& routingInfo,
+    const std::set<ShardId>& shardsToSkip,
+    const BSONObj& cmdObj,
+    const ReadPreferenceSetting& readPref,
+    Shard::RetryPolicy retryPolicy,
+    const BSONObj& query,
+    const BSONObj& collation) {
+    const auto requests = buildVersionedRequestsForTargetedShards(
+        opCtx, nss, routingInfo, shardsToSkip, cmdObj, query, collation);
+
+    return gatherResponsesNoThrowOnStaleShardVersionErrors(
+        opCtx, dbName, readPref, retryPolicy, requests);
+}
+
 std::vector<AsyncRequestsSender::Response> scatterGatherOnlyVersionIfUnsharded(
     OperationContext* opCtx,
     const NamespaceString& nss,
@@ -320,10 +433,10 @@ std::vector<AsyncRequestsSender::Response> scatterGatherOnlyVersionIfUnsharded(
         requests = buildUnversionedRequestsForAllShards(opCtx, cmdObj);
     } else {
         requests = buildVersionedRequestsForTargetedShards(
-            opCtx, nss, routingInfo, cmdObj, BSONObj(), BSONObj());
+            opCtx, nss, routingInfo, {} /* shardsToSkip */, cmdObj, BSONObj(), BSONObj());
     }
 
-    return gatherResponses(opCtx, nss.db(), readPref, retryPolicy, requests, ignorableErrors);
+    return gatherResponses(opCtx, nss.db(), readPref, retryPolicy, requests);
 }
 
 AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
@@ -343,32 +456,75 @@ AsyncRequestsSender::Response executeCommandAgainstDatabasePrimary(
     return std::move(responses.front());
 }
 
-bool appendRawResponses(OperationContext* opCtx,
-                        std::string* errmsg,
-                        BSONObjBuilder* output,
-                        const std::vector<AsyncRequestsSender::Response>& shardResponses,
-                        std::set<ErrorCodes::Error> ignorableErrors) {
-    // Always include ShardNotFound as an ignorable error, since this node may not have realized a
-    // shard has been removed.
-    ignorableErrors.insert(ErrorCodes::ShardNotFound);
+AsyncRequestsSender::Response executeCommandAgainstShardWithMinKeyChunk(
+    OperationContext* opCtx,
+    const NamespaceString& nss,
+    const CachedCollectionRoutingInfo& routingInfo,
+    const BSONObj& cmdObj,
+    const ReadPreferenceSetting& readPref,
+    Shard::RetryPolicy retryPolicy) {
 
+    const auto query = routingInfo.cm()
+        ? routingInfo.cm()->getShardKeyPattern().getKeyPattern().globalMin()
+        : BSONObj();
+
+    auto responses =
+        gatherResponses(opCtx,
+                        nss.db(),
+                        readPref,
+                        retryPolicy,
+                        buildVersionedRequestsForTargetedShards(opCtx,
+                                                                nss,
+                                                                routingInfo,
+                                                                {} /* shardsToSkip */,
+                                                                cmdObj,
+                                                                query,
+                                                                BSONObj() /* collation */));
+    return std::move(responses.front());
+}
+
+RawResponsesResult appendRawResponses(
+    OperationContext* opCtx,
+    std::string* errmsg,
+    BSONObjBuilder* output,
+    const std::vector<AsyncRequestsSender::Response>& shardResponses) {
+
+    std::vector<AsyncRequestsSender::Response> successARSResponses;
     std::vector<std::pair<ShardId, BSONObj>> successResponsesReceived;
-    std::vector<std::pair<ShardId, Status>> ignorableErrorsReceived;
-    std::vector<std::pair<ShardId, Status>> nonIgnorableErrorsReceived;
+    std::vector<std::pair<ShardId, Status>> shardNotFoundErrorsReceived;
 
+    // "Generic errors" are all errors that are not shardNotFound errors.
+    std::vector<std::pair<ShardId, Status>> genericErrorsReceived;
+    std::set<ShardId> shardsWithSuccessResponses;
+
+    boost::optional<Status> firstStaleConfigErrorReceived;
     boost::optional<std::pair<ShardId, BSONElement>> firstWriteConcernErrorReceived;
 
     const auto processError = [&](const ShardId& shardId, const Status& status) {
         invariant(!status.isOK());
-        if (ignorableErrors.find(status.code()) != ignorableErrors.end()) {
-            ignorableErrorsReceived.emplace_back(std::move(shardId), std::move(status));
+        // It is safe to pass `hasWriteConcernError` as false in the below check because operations
+        // run inside transactions do not wait for write concern, except for commit and abort.
+        if (TransactionRouter::get(opCtx) &&
+            isTransientTransactionError(
+                status.code(), false /*hasWriteConcernError*/, false /*isCommitOrAbort*/)) {
+            // Re-throw on transient transaction errors to make sure appropriate error labels are
+            // appended to the result.
+            uassertStatusOK(status);
+        }
+        if (status.code() == ErrorCodes::ShardNotFound) {
+            shardNotFoundErrorsReceived.emplace_back(shardId, status);
             return;
         }
-        nonIgnorableErrorsReceived.emplace_back(shardId, status);
+
+        if (!firstStaleConfigErrorReceived && ErrorCodes::isStaleShardVersionError(status.code())) {
+            firstStaleConfigErrorReceived.emplace(status);
+        }
+
+        genericErrorsReceived.emplace_back(shardId, status);
     };
 
-    // Do a pass through all the received responses and group them into success, ignorable, and
-    // non-ignorable.
+    // Do a pass through all the received responses and group them into success, ShardNotFound, and
+    // error responses.
     for (const auto& shardResponse : shardResponses) {
         const auto& shardId = shardResponse.shardId;
 
@@ -390,15 +546,17 @@ bool appendRawResponses(OperationContext* opCtx,
         }
 
         successResponsesReceived.emplace_back(shardId, resObj);
+        successARSResponses.emplace_back(shardResponse);
+        shardsWithSuccessResponses.emplace(shardId);
     }
 
-    // If all shards reported ignorable errors, promote them all to non-ignorable errors.
-    if (ignorableErrorsReceived.size() == shardResponses.size()) {
-        invariant(nonIgnorableErrorsReceived.empty());
-        nonIgnorableErrorsReceived = std::move(ignorableErrorsReceived);
+    // If all shards reported ShardNotFound, promote them all to generic errors.
+    if (shardNotFoundErrorsReceived.size() == shardResponses.size()) {
+        invariant(genericErrorsReceived.empty());
+        genericErrorsReceived = std::move(shardNotFoundErrorsReceived);
     }
 
-    // Append a 'raw' field containing the success responses and non-ignorable error responses.
+    // Append a 'raw' field containing the success responses and error responses.
     BSONObjBuilder rawShardResponses;
     const auto appendRawResponse = [&](const ShardId& shardId, const BSONObj& response) {
         // Try to report the response by the shard's full connection string.
@@ -417,29 +575,30 @@ bool appendRawResponses(OperationContext* opCtx,
     for (const auto& success : successResponsesReceived) {
         appendRawResponse(success.first, success.second);
     }
-    for (const auto& error : nonIgnorableErrorsReceived) {
+    for (const auto& error : genericErrorsReceived) {
         BSONObjBuilder statusObjBob;
         CommandHelpers::appendCommandStatusNoThrow(statusObjBob, error.second);
         appendRawResponse(error.first, statusObjBob.obj());
     }
     output->append("raw", rawShardResponses.done());
 
-    // If there were no non-ignorable errors, report success (possibly with a writeConcern error).
-    if (nonIgnorableErrorsReceived.empty()) {
+    // If there were no errors, report success (possibly with a writeConcern error).
+    if (genericErrorsReceived.empty()) {
         if (firstWriteConcernErrorReceived) {
             appendWriteConcernErrorToCmdResponse(firstWriteConcernErrorReceived->first,
                                                  firstWriteConcernErrorReceived->second,
                                                  *output);
         }
-        return true;
+        return {
+            true, shardsWithSuccessResponses, successARSResponses, firstStaleConfigErrorReceived};
     }
 
-    // There was a non-ignorable error. Choose the first non-ignorable error as the top-level error.
-    const auto& firstNonIgnorableError = nonIgnorableErrorsReceived.front().second;
-    output->append("code", firstNonIgnorableError.code());
-    output->append("codeName", ErrorCodes::errorString(firstNonIgnorableError.code()));
-    *errmsg = firstNonIgnorableError.reason();
-    return false;
+    // There was an error. Choose the first error as the top-level error.
+    const auto& firstError = genericErrorsReceived.front().second;
+    output->append("code", firstError.code());
+    output->append("codeName", ErrorCodes::errorString(firstError.code()));
+    *errmsg = firstError.reason();
+    return {false, shardsWithSuccessResponses, successARSResponses, firstStaleConfigErrorReceived};
 }
 
 BSONObj extractQuery(const BSONObj& cmdObj) {
@@ -571,8 +730,8 @@ std::vector<std::pair<ShardId, BSONObj>> getVersionedRequestsForTargetedShards(
     const BSONObj& query,
     const BSONObj& collation) {
     std::vector<std::pair<ShardId, BSONObj>> requests;
-    auto ars_requests =
-        buildVersionedRequestsForTargetedShards(opCtx, nss, routingInfo, cmdObj, query, collation);
+    auto ars_requests = buildVersionedRequestsForTargetedShards(
+        opCtx, nss, routingInfo, {} /* shardsToSkip */, cmdObj, query, collation);
     std::transform(std::make_move_iterator(ars_requests.begin()),
                    std::make_move_iterator(ars_requests.end()),
                    std::back_inserter(requests),
@@ -599,30 +758,40 @@ StatusWith<CachedCollectionRoutingInfo> getCollectionRoutingInfoForTxnCmd(
     return catalogCache->getCollectionRoutingInfoAt(opCtx, nss, atClusterTime.asTimestamp());
 }
 
-std::vector<AsyncRequestsSender::Response> dispatchCommandAssertCollectionExistsOnAtLeastOneShard(
-    OperationContext* opCtx,
-    const NamespaceString& nss,
-    const ReadPreference readPreference,
-    const BSONObj& cmdObj) {
-    auto shardResponses = scatterGatherOnlyVersionIfUnsharded(
+StatusWith<Shard::QueryResponse> loadIndexesFromAuthoritativeShard(OperationContext* opCtx,
+                                                                   const NamespaceString& nss) {
+    const auto routingInfo =
+        uassertStatusOK(Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfo(opCtx, nss));
+
+    auto [indexShard, listIndexesCmd] = [&]() -> std::pair<std::shared_ptr<Shard>, BSONObj> {
+        auto cmdNoVersion = applyReadWriteConcern(
+            opCtx, true /* appendRC */, false /* appendWC */, BSON("listIndexes" << nss.coll()));
+        if (routingInfo.cm()) {
+            // For a sharded collection we must load indexes from a shard with chunks. For
+            // consistency with cluster listIndexes, load from the shard that owns the minKey chunk.
+            const auto minKeyShardId = routingInfo.cm()->getMinKeyShardIdWithSimpleCollation();
+            auto minKeyShard =
+                uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getShard(opCtx, minKeyShardId));
+            return {minKeyShard,
+                    appendShardVersion(cmdNoVersion, routingInfo.cm()->getVersion(minKeyShardId))};
+        } else {
+            // For an unsharded collection, the primary shard will have correct indexes. We attach
+            // unsharded shard version to detect if the collection has become sharded.
+            const auto cmdObjWithShardVersion =
+                (routingInfo.db().primaryId() != ShardRegistry::kConfigServerShardId)
+                ? appendShardVersion(cmdNoVersion, ChunkVersion::UNSHARDED())
+                : cmdNoVersion;
+            return {routingInfo.db().primary(),
+                    appendDbVersionIfPresent(cmdObjWithShardVersion, routingInfo.db())};
+        }
+    }();
+
+    return indexShard->runExhaustiveCursorCommand(
         opCtx,
-        nss,
-        CommandHelpers::filterCommandRequestForPassthrough(cmdObj),
-        ReadPreferenceSetting(readPreference),
-        Shard::RetryPolicy::kNoRetry,
-        {ErrorCodes::CannotImplicitlyCreateCollection});
-
-    if (std::all_of(shardResponses.begin(), shardResponses.end(), [](const auto& response) {
-            return response.swResponse.getStatus().isOK() &&
-                getStatusFromCommandResult(response.swResponse.getValue().data) ==
-                ErrorCodes::CannotImplicitlyCreateCollection;
-        })) {
-        // Propagate the ExtraErrorInfo from the first response.
-        uassertStatusOK(
-            getStatusFromCommandResult(shardResponses.front().swResponse.getValue().data));
-    }
-
-    return shardResponses;
+        ReadPreferenceSetting::get(opCtx),
+        nss.db().toString(),
+        listIndexesCmd,
+        opCtx->hasDeadline() ? opCtx->getRemainingMaxTimeMillis() : Milliseconds(-1));
 }
 
 }  // namespace mongo
