@@ -43,17 +43,22 @@ namespace mongo {
 class OperationContext;
 
 /**
- * Combines a shard and the version which that shard should be using
+ * Combines a shard, the shard version, and database version that the shard should be using
  */
 struct ShardEndpoint {
-    ShardEndpoint(const ShardId& shardName, const ChunkVersion& shardVersion)
-        : shardName(shardName), shardVersion(shardVersion) {}
+    ShardEndpoint(const ShardId& shardName,
+                  const ChunkVersion& shardVersion,
+                  const boost::optional<DatabaseVersion> dbVersion = boost::none)
+        : shardName(shardName), shardVersion(shardVersion), databaseVersion(dbVersion) {}
 
     ShardEndpoint(const ShardEndpoint& other)
-        : shardName(other.shardName), shardVersion(other.shardVersion) {}
+        : shardName(other.shardName), shardVersion(other.shardVersion) {
+        databaseVersion = other.databaseVersion;
+    }
 
     ShardId shardName;
     ChunkVersion shardVersion;
+    boost::optional<DatabaseVersion> databaseVersion;
 };
 
 /**
@@ -114,13 +119,6 @@ public:
     virtual StatusWith<std::vector<ShardEndpoint>> targetDelete(
         OperationContext* opCtx, const write_ops::DeleteOpEntry& deleteDoc) const = 0;
 
-    /**
-     * Returns a vector of ShardEndpoints for the entire collection.
-     *
-     * Returns !OK with message if the full collection could not be targeted.
-     */
-    virtual StatusWith<std::vector<ShardEndpoint>> targetCollection() const = 0;
-
     /**
      * Returns a vector of ShardEndpoints for all shards.
      *
@@ -143,8 +141,19 @@ public:
      *
      * If stale responses are is noted, we must not have noted that we cannot target.
      */
-    virtual void noteStaleResponse(const ShardEndpoint& endpoint,
-                                   const StaleConfigInfo& staleInfo) = 0;
+    virtual void noteStaleShardResponse(const ShardEndpoint& endpoint,
+                                        const StaleConfigInfo& staleInfo) = 0;
+
+    /**
+     * Informs the targeter of stale db routing version responses for this db from an endpoint,
+     * with further information available in the returned staleInfo.
+     *
+     * Any stale responses noted here will be taken into account on the next refresh.
+     *
+     * If stale responses are is noted, we must not have noted that we cannot target.
+     */
+    virtual void noteStaleDbResponse(const ShardEndpoint& endpoint,
+                                     const StaleDbRoutingVersion& staleInfo) = 0;
 
     /**
      * Refreshes the targeting metadata for the namespace if needed, based on previously-noted
@@ -160,6 +169,13 @@ public:
      */
     virtual Status refreshIfNeeded(OperationContext* opCtx, bool* wasChanged) = 0;
 
+    /**
+     * Returns whether this write targets the config server. Invariants if the write targets the
+     * config server AND there is more than one endpoint, since there should be no namespaces that
+     * target both config servers and shards.
+     */
+    virtual bool endpointIsConfigServer() const = 0;
+
     /**
      * Returns the number of shards that own one or more chunks for the targeted collection.
      */
