@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -41,8 +41,8 @@
 #include "mongo/db/query/killcursors_request.h"
 #include "mongo/executor/remote_command_request.h"
 #include "mongo/executor/remote_command_response.h"
+#include "mongo/s/catalog/type_shard.h"
 #include "mongo/util/assert_util.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 
@@ -51,7 +51,7 @@ const BSONObj AsyncResultsMerger::kWholeSortKeySortPattern = BSON(kSortKeyField
 
 namespace {
 
-// Maximum number of retries for network and replication NotPrimary errors (per host).
+// Maximum number of retries for network and replication notMaster errors (per host).
 const int kMaxNumFailedHostRetryAttempts = 3;
 
 /**
@@ -64,7 +64,11 @@ BSONObj extractSortKey(BSONObj obj, bool compareWholeSortKey) {
     if (compareWholeSortKey) {
         return key.wrap();
     }
-    invariant(key.type() == BSONType::Object);
+    // TODO (SERVER-43361): We expect the sort key to be an array, but if the sort key originated
+    // from a 4.2 mongod, it will be a document, instead. Either way, 'isABSONObj()' will return
+    // true, and 'compareSortKeys()' will behave the same way. After branching for 4.5, we can
+    // tighten this invariant to specifically require a 'BSONArray' type.
+    invariant(key.isABSONObj());
     return key.Obj();
 }
 
@@ -75,8 +79,8 @@ BSONObj extractSortKey(BSONObj obj, bool compareWholeSortKey) {
 int compareSortKeys(BSONObj leftSortKey, BSONObj rightSortKey, BSONObj sortKeyPattern) {
     // This does not need to sort with a collator, since mongod has already mapped strings to their
     // ICU comparison keys as part of the $sortKey meta projection.
-    const bool considerFieldName = false;
-    return leftSortKey.woCompare(rightSortKey, sortKeyPattern, considerFieldName);
+    const BSONObj::ComparisonRulesSet rules = 0;  // 'considerFieldNames' flag is not set.
+    return leftSortKey.woCompare(rightSortKey, sortKeyPattern, rules);
 }
 
 }  // namespace
@@ -102,7 +106,11 @@ AsyncResultsMerger::AsyncResultsMerger(OperationContext* opCtx,
     for (const auto& remote : _params.getRemotes()) {
         _remotes.emplace_back(remote.getHostAndPort(),
                               remote.getCursorResponse().getNSS(),
-                              remote.getCursorResponse().getCursorId());
+                              remote.getCursorResponse().getCursorId(),
+                              remote.getCursorResponse().getPartialResultsReturned());
+
+        // A remote cannot be flagged as 'partialResultsReturned' if 'allowPartialResults' is false.
+        invariant(!(_remotes.back().partialResultsReturned && !_params.getAllowPartialResults()));
 
         // We don't check the return value of _addBatchToBuffer here; if there was an error,
         // it will be stored in the remote and the first call to ready() will return true.
@@ -111,7 +119,23 @@ AsyncResultsMerger::AsyncResultsMerger(OperationContext* opCtx,
     }
     // If this is a change stream, then we expect to have already received PBRTs from every shard.
     invariant(_promisedMinSortKeys.empty() || _promisedMinSortKeys.size() == _remotes.size());
-    _highWaterMark = _promisedMinSortKeys.empty() ? BSONObj() : _promisedMinSortKeys.begin()->first;
+    _setInitialHighWaterMark();
+}
+
+void AsyncResultsMerger::_setInitialHighWaterMark() {
+    // If we do not have any minimum promised sort keys, this is not a change stream. Return early.
+    if (_promisedMinSortKeys.empty()) {
+        return;
+    }
+    // Find the minimum promised sort key whose remote is eligible to contribute a high water mark.
+    for (auto&& [minSortKey, remoteId] : _promisedMinSortKeys) {
+        if (_remotes[remoteId].eligibleForHighWaterMark) {
+            _highWaterMark = minSortKey;
+            break;
+        }
+    }
+    // We should always be guaranteed to find an eligible remote, if this is a change stream.
+    invariant(!_highWaterMark.isEmpty());
 }
 
 AsyncResultsMerger::~AsyncResultsMerger() {
@@ -183,24 +207,55 @@ void AsyncResultsMerger::addNewShardCursors(std::vector<RemoteCursor>&& newCurso
         const auto newIndex = _remotes.size();
         _remotes.emplace_back(remote.getHostAndPort(),
                               remote.getCursorResponse().getNSS(),
-                              remote.getCursorResponse().getCursorId());
+                              remote.getCursorResponse().getCursorId(),
+                              remote.getCursorResponse().getPartialResultsReturned());
         _addBatchToBuffer(lk, newIndex, remote.getCursorResponse());
     }
 }
 
-BSONObj AsyncResultsMerger::getHighWaterMark() {
+bool AsyncResultsMerger::partialResultsReturned() const {
     stdx::lock_guard<Latch> lk(_mutex);
-    auto minPromisedSortKey = _getMinPromisedSortKey(lk);
-    if (!minPromisedSortKey.isEmpty() && !_ready(lk)) {
-        _highWaterMark = minPromisedSortKey;
-    }
-    return _highWaterMark;
+    return std::any_of(_remotes.begin(), _remotes.end(), [](const auto& remote) {
+        return remote.partialResultsReturned;
+    });
 }
 
-BSONObj AsyncResultsMerger::_getMinPromisedSortKey(WithLock) {
+std::size_t AsyncResultsMerger::getNumRemotes() const {
+    // Take the lock to guard against shard additions or disconnections.
+    stdx::lock_guard<Latch> lk(_mutex);
+
+    // If 'allowPartialResults' is false, the number of participating remotes is constant.
+    if (!_params.getAllowPartialResults()) {
+        return _remotes.size();
+    }
+    // Otherwise, discount remotes which failed to connect or disconnected prematurely.
+    return std::count_if(_remotes.begin(), _remotes.end(), [](const auto& remote) {
+        return !remote.partialResultsReturned;
+    });
+}
+
+BSONObj AsyncResultsMerger::getHighWaterMark() {
+    stdx::lock_guard<Latch> lk(_mutex);
+    // At this point, the high water mark may be the resume token of the last document we returned.
+    // If no further results are eligible for return, we advance to the minimum promised sort key.
+    // If the remote associated with the minimum promised sort key is not currently eligible to
+    // provide a high water mark, then we do not advance even if no further results are ready.
+    if (auto minPromisedSortKey = _getMinPromisedSortKey(lk); minPromisedSortKey && !_ready(lk)) {
+        if (_remotes[minPromisedSortKey->second].eligibleForHighWaterMark) {
+            _highWaterMark = minPromisedSortKey->first;
+        }
+    }
+    // The high water mark is stored in sort-key format: {"": <high watermark>}. We only return
+    // the <high watermark> part of of the sort key, which looks like {_data: ..., _typeBits: ...}.
+    invariant(_highWaterMark.isEmpty() || _highWaterMark.firstElement().type() == BSONType::Object);
+    return _highWaterMark.isEmpty() ? BSONObj() : _highWaterMark.firstElement().Obj().getOwned();
+}
+
+boost::optional<AsyncResultsMerger::MinSortKeyRemoteIdPair>
+AsyncResultsMerger::_getMinPromisedSortKey(WithLock) {
     // We cannot return the minimum promised sort key unless all shards have reported one.
-    return _promisedMinSortKeys.size() < _remotes.size() ? BSONObj()
-                                                         : _promisedMinSortKeys.begin()->first;
+    return _promisedMinSortKeys.size() < _remotes.size() ? boost::optional<MinSortKeyRemoteIdPair>{}
+                                                         : *_promisedMinSortKeys.begin();
 }
 
 bool AsyncResultsMerger::_ready(WithLock lk) {
@@ -252,8 +307,8 @@ bool AsyncResultsMerger::_readySortedTailable(WithLock lk) {
         extractSortKey(*smallestResult.getResult(), _params.getCompareWholeSortKey());
     // We should always have a minPromisedSortKey from every shard in the sorted tailable case.
     auto minPromisedSortKey = _getMinPromisedSortKey(lk);
-    invariant(!minPromisedSortKey.isEmpty());
-    return compareSortKeys(keyWeWantToReturn, minPromisedSortKey, *_params.getSort()) <= 0;
+    invariant(minPromisedSortKey);
+    return compareSortKeys(keyWeWantToReturn, minPromisedSortKey->first, *_params.getSort()) <= 0;
 }
 
 bool AsyncResultsMerger::_readyUnsorted(WithLock) {
@@ -315,8 +370,10 @@ ClusterQueryResult AsyncResultsMerger::_nextReadySorted(WithLock) {
 
     // For sorted tailable awaitData cursors, update the high water mark to the document's sort key.
     if (_tailableMode == TailableModeEnum::kTailableAndAwaitData) {
-        _highWaterMark =
-            extractSortKey(*front.getResult(), _params.getCompareWholeSortKey()).getOwned();
+        if (_remotes[smallestRemote].eligibleForHighWaterMark) {
+            _highWaterMark =
+                extractSortKey(*front.getResult(), _params.getCompareWholeSortKey()).getOwned();
+        }
     }
 
     return front;
@@ -396,7 +453,7 @@ Status AsyncResultsMerger::_askForNextBatch(WithLock, size_t remoteIndex) {
     }
 
     executor::RemoteCommandRequest request(
-        remote.getTargetHost(), _params.getNss().db().toString(), cmdObj, _opCtx);
+        remote.getTargetHost(), remote.cursorNss.db().toString(), cmdObj, _opCtx);
 
     auto callbackStatus =
         _executor->scheduleRemoteCommand(request, [this, remoteIndex](auto const& cbData) {
@@ -503,7 +560,7 @@ StatusWith<CursorResponse> AsyncResultsMerger::_parseCursorResponse(
     return std::move(cursorResponse);
 }
 
-void AsyncResultsMerger::_updateRemoteMetadata(WithLock,
+void AsyncResultsMerger::_updateRemoteMetadata(WithLock lk,
                                                size_t remoteIndex,
                                                const CursorResponse& response) {
     // Update the cursorId; it is sent as '0' when the cursor has been exhausted on the shard.
@@ -518,9 +575,16 @@ void AsyncResultsMerger::_updateRemoteMetadata(WithLock,
         // The postBatchResumeToken should never be empty.
         invariant(!response.getPostBatchResumeToken()->isEmpty());
 
+        // Note that the PBRT is an object of format {_data: ..., _typeBits: ...} that we must wrap
+        // in a sort key so that it can compare correctly with sort keys from other streams.
+        auto newMinSortKey = BSON("" << *response.getPostBatchResumeToken());
+
+        // Determine whether the new batch is eligible to provide a high water mark resume token.
+        remote.eligibleForHighWaterMark =
+            _checkHighWaterMarkEligibility(lk, newMinSortKey, remote, response);
+
         // The most recent minimum sort key should never be smaller than the previous promised
-        // minimum sort key for this remote, if one exists.
-        auto newMinSortKey = *response.getPostBatchResumeToken();
+        // minimum sort key for this remote, if a previous promised minimum sort key exists.
         if (auto& oldMinSortKey = remote.promisedMinSortKey) {
             invariant(compareSortKeys(newMinSortKey, *oldMinSortKey, *_params.getSort()) >= 0);
             invariant(_promisedMinSortKeys.size() <= _remotes.size());
@@ -528,19 +592,62 @@ void AsyncResultsMerger::_updateRemoteMetadata(WithLock,
         }
         _promisedMinSortKeys.insert({newMinSortKey, remoteIndex});
         remote.promisedMinSortKey = newMinSortKey;
-    } else {
-        // If we don't have a postBatchResumeToken, then we should never have an oplog timestamp.
-        // TODO SERVER-38539: remove this validation when $internalLatestOplogTimestamp is removed.
-        if (response.getLastOplogTimestamp()) {
-            severe() << "Host " << remote.shardHostAndPort
-                     << " returned a cursor which has an oplog timestamp but does not have a "
-                        "postBatchResumeToken, suggesting that one or more shards are running an "
-                        "older version of MongoDB. This configuration is not supported.";
-            fassertFailedNoTrace(51062);
-        }
     }
 }
 
+bool AsyncResultsMerger::_checkHighWaterMarkEligibility(WithLock,
+                                                        BSONObj newMinSortKey,
+                                                        const RemoteCursorData& remote,
+                                                        const CursorResponse& response) {
+    // If the cursor is not on the "config.shards" namespace, then it is a normal shard cursor.
+    // These cursors are always eligible to provide a high water mark resume token.
+    if (remote.cursorNss != ShardType::ConfigNS) {
+        return true;
+    }
+
+    // If we are here, the cursor is on the "config.shards" namespace. This is an internal cursor
+    // which monitors for the addition of new shards. There are two special cases which we must
+    // handle for this cursor:
+    //
+    //   - The user specified a 'startAtOperationTime' in the future. This is a problem because the
+    //     config cursor must always be opened at the current clusterTime, to ensure that it detects
+    //     all shards that are added after the change stream is dispatched. We must make sure that
+    //     the high water mark ignores the config cursor's minimum promised sort keys, otherwise we
+    //     will end up returning a token that is earlier than the start time requested by the user.
+    //
+    //   - The cursor returns a "shard added" event. All events produced by the config cursor are
+    //     handled and swallowed internally by the stream. We therefore do not want to allow their
+    //     resume tokens to be exposed to the user via the postBatchResumeToken mechanism, since
+    //     these token are not actually resumable. See SERVER-47810 for further details.
+
+    // If the current high water mark is ahead of the config cursor, it implies that the client has
+    // opened a stream with a startAtOperationTime in the future. We should hold the high water mark
+    // at the user-specified start time until the config server catches up to it. The config cursor
+    // is therefore not eligible to provide a new high water mark.
+    if (compareSortKeys(newMinSortKey, _highWaterMark, *_params.getSort()) < 0) {
+        return false;
+    }
+    // If the config server returns an event which indicates a change in the cluster topology, it
+    // will be swallowed by the stream. It will not be returned to the user, and it should not be
+    // eligible to become the high water mark either.
+    if (!response.getBatch().empty()) {
+        return false;
+    }
+    // If we are here, then the only remaining reason not to mark this batch as eligible is if the
+    // current batch's sort key is the same as the last batch's, and the last batch was ineligible.
+    // Therefore, if the previous batch was eligible, this batch is as well.
+    if (remote.eligibleForHighWaterMark) {
+        return true;
+    }
+    // If we are here, then either the last batch we received was ineligible for one of the reasons
+    // outlined above, or this is the first batch we have ever received for this cursor. If this is
+    // the first batch, then we always mark the config cursor as ineligible so that the initial high
+    // water mark will be taken from one of the shards instead. If we received an ineligible batch
+    // last time, then the current batch is only eligible if its sort key is greater than the last.
+    return remote.promisedMinSortKey &&
+        compareSortKeys(newMinSortKey, *remote.promisedMinSortKey, *_params.getSort()) > 0;
+}
+
 void AsyncResultsMerger::_handleBatchResponse(WithLock lk,
                                               CbData const& cbData,
                                               size_t remoteIndex) {
@@ -584,14 +691,14 @@ void AsyncResultsMerger::_cleanUpFailedBatch(WithLock lk, Status status, size_t
     //
     // The ExchangePassthrough error code is an internal-only error code used specifically to
     // communicate that an error has occurred, but some other thread is responsible for returning
-    // the error to the user. In order to avoid polluting the user's error message, we ingore such
+    // the error to the user. In order to avoid polluting the user's error message, we ignore such
     // errors with the expectation that all outstanding cursors will be closed promptly.
     if (_params.getAllowPartialResults() || remote.status == ErrorCodes::ExchangePassthrough) {
-        remote.status = Status::OK();
-
-        // Clear the results buffer and cursor id.
+        // Clear the results buffer and cursor id, and set 'partialResultsReturned' if appropriate.
+        remote.partialResultsReturned = (remote.status != ErrorCodes::ExchangePassthrough);
         std::queue<ClusterQueryResult> emptyBuffer;
         std::swap(remote.docBuffer, emptyBuffer);
+        remote.status = Status::OK();
         remote.cursorId = 0;
     }
 }
@@ -656,7 +763,7 @@ bool AsyncResultsMerger::_addBatchToBuffer(WithLock lk,
                            str::stream() << "Missing field '" << AsyncResultsMerger::kSortKeyField
                                          << "' in document: " << obj);
                 return false;
-            } else if (!_params.getCompareWholeSortKey() && key.type() != BSONType::Object) {
+            } else if (!_params.getCompareWholeSortKey() && !key.isABSONObj()) {
                 remote.status =
                     Status(ErrorCodes::InternalError,
                            str::stream() << "Field '" << AsyncResultsMerger::kSortKeyField
@@ -763,10 +870,15 @@ executor::TaskExecutor::EventHandle AsyncResultsMerger::kill(OperationContext* o
 
 AsyncResultsMerger::RemoteCursorData::RemoteCursorData(HostAndPort hostAndPort,
                                                        NamespaceString cursorNss,
-                                                       CursorId establishedCursorId)
+                                                       CursorId establishedCursorId,
+                                                       bool partialResultsReturned)
     : cursorId(establishedCursorId),
       cursorNss(std::move(cursorNss)),
-      shardHostAndPort(std::move(hostAndPort)) {}
+      shardHostAndPort(std::move(hostAndPort)),
+      partialResultsReturned(partialResultsReturned) {
+    // If the 'partialResultsReturned' flag is set, the cursorId must be zero (closed).
+    invariant(!(partialResultsReturned && cursorId != 0));
+}
 
 const HostAndPort& AsyncResultsMerger::RemoteCursorData::getTargetHost() const {
     return shardHostAndPort;
