@@ -27,16 +27,12 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kConnectionPool
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kConnectionPool
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/client/replica_set_monitor.h"
-#include "mongo/executor/connection_pool_stats.h"
-#include "mongo/s/client/shard_registry.h"
-#include "mongo/s/is_mongos.h"
 #include "mongo/s/sharding_task_executor_pool_controller.h"
-#include "mongo/util/log.h"
 
 namespace mongo {
 
@@ -56,15 +52,6 @@ void emplaceOrInvariant(Map&& map, Args&&... args) noexcept {
     invariant(ret.second, "Element already existed in map/set");
 }
 
-bool isConfigServer(const ShardRegistry* sr, const HostAndPort& peer) {
-    if (!sr)
-        return false;
-    auto shard = sr->getShardForHostNoReload(peer);
-    if (!shard)
-        return false;
-    return shard->isConfig();
-}
-
 }  // namespace
 
 Status ShardingTaskExecutorPoolController::validateHostTimeout(const int& hostTimeoutMS) {
@@ -95,13 +82,7 @@ Status ShardingTaskExecutorPoolController::validatePendingTimeout(const int& pen
 
 Status ShardingTaskExecutorPoolController::onUpdateMatchingStrategy(const std::string& str) {
     // TODO Fix up after SERVER-40224
-    if (str == "automatic") {
-        if (isMongos()) {
-            gParameters.matchingStrategy.store(MatchingStrategy::kMatchPrimaryNode);
-        } else {
-            gParameters.matchingStrategy.store(MatchingStrategy::kDisabled);
-        }
-    } else if (str == "disabled") {
+    if (str == "disabled") {
         gParameters.matchingStrategy.store(MatchingStrategy::kDisabled);
     } else if (str == "matchPrimaryNode") {
         gParameters.matchingStrategy.store(MatchingStrategy::kMatchPrimaryNode);
@@ -173,22 +154,22 @@ public:
     explicit ReplicaSetChangeListener(ShardingTaskExecutorPoolController* controller)
         : _controller(controller) {}
 
-    void onFoundSet(const Key& key) override {
+    void onFoundSet(const Key& key) noexcept override {
         // Do nothing
     }
 
-    void onConfirmedSet(const State& state) override {
+    void onConfirmedSet(const State& state) noexcept override {
         stdx::lock_guard lk(_controller->_mutex);
 
         _controller->_removeGroup(lk, state.connStr.getSetName());
         _controller->_addGroup(lk, state);
     }
 
-    void onPossibleSet(const State& state) override {
+    void onPossibleSet(const State& state) noexcept override {
         // Do nothing
     }
 
-    void onDroppedSet(const Key& key) override {
+    void onDroppedSet(const Key& key) noexcept override {
         stdx::lock_guard lk(_controller->_mutex);
 
         _controller->_removeGroup(lk, key);
@@ -208,7 +189,6 @@ void ShardingTaskExecutorPoolController::addHost(PoolId id, const HostAndPort& h
 
     PoolData poolData;
     poolData.host = host;
-    poolData.isConfigServer = isConfigServer(_shardRegistry.lock().get(), host);
 
     // Set up the GroupAndId
     auto& groupAndId = _groupAndIds[host];
@@ -232,25 +212,8 @@ auto ShardingTaskExecutorPoolController::updateHost(PoolId id, const HostState&
 
     auto& poolData = getOrInvariant(_poolDatas, id);
 
-    size_t minConns, maxConns;
-    std::tie(minConns, maxConns) = [&] {
-        size_t lo = gParameters.minConnections.load();
-        size_t hi = gParameters.maxConnections.load();
-        if (poolData.isConfigServer) {
-            auto maybeOverride = [](size_t& t, int val) {
-                if (val >= 0)
-                    t = val;
-            };
-            maybeOverride(lo, gParameters.minConnectionsForConfigServers.load());
-            maybeOverride(hi, gParameters.maxConnectionsForConfigServers.load());
-        }
-        return std::tuple(lo, hi);
-    }();
-    // conn_pool_csrs.js looks for this message in the log.
-    MONGO_LOG(5) << "Update connection pool"    //
-                 << ": host=" << poolData.host  //
-                 << ", minConns=" << minConns   //
-                 << ", maxConns=" << maxConns;
+    const size_t minConns = gParameters.minConnections.load();
+    const size_t maxConns = gParameters.maxConnections.load();
 
     // Update the target for just the pool first
     poolData.target = stats.requests + stats.active;
@@ -348,9 +311,4 @@ Milliseconds ShardingTaskExecutorPoolController::toRefreshTimeout() const {
     return Milliseconds{gParameters.toRefreshTimeoutMS.load()};
 }
 
-void ShardingTaskExecutorPoolController::updateConnectionPoolStats(
-    executor::ConnectionPoolStats* cps) const {
-    cps->strategy = gParameters.matchingStrategy.load();
-}
-
 }  // namespace mongo
