@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
@@ -38,11 +38,12 @@
 #include "mongo/db/matcher/extensions_callback_noop.h"
 #include "mongo/db/query/canonical_query.h"
 #include "mongo/db/query/collation/collation_index_key.h"
+#include "mongo/logv2/log.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/cluster_commands_helpers.h"
+#include "mongo/s/database_version_helpers.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/shard_key_pattern.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -112,10 +113,11 @@ StatusWith<UpdateType> getUpdateExprType(const write_ops::UpdateOpEntry& updateD
  * generating the new document in the case of an upsert. It is therefore always correct to target
  * the operation on the basis of the combined updateExpr and query.
  */
-StatusWith<BSONObj> getUpdateExpr(OperationContext* opCtx,
-                                  const ShardKeyPattern& shardKeyPattern,
-                                  const UpdateType updateType,
-                                  const write_ops::UpdateOpEntry& updateDoc) {
+StatusWith<BSONObj> getUpdateExprForTargeting(OperationContext* opCtx,
+                                              const ShardKeyPattern& shardKeyPattern,
+                                              const NamespaceString& nss,
+                                              const UpdateType updateType,
+                                              const write_ops::UpdateOpEntry& updateDoc) {
     // We should never see an invalid update type here.
     invariant(updateType != UpdateType::kUnknown);
 
@@ -132,36 +134,24 @@ StatusWith<BSONObj> getUpdateExpr(OperationContext* opCtx,
     invariant(updateMod.type() == write_ops::UpdateModification::Type::kClassic);
     auto updateExpr = updateMod.getUpdateClassic();
 
-    // Find the set of all shard key fields that are missing from the update expression.
-    const auto missingFields = shardKeyPattern.findMissingShardKeyFieldsFromDoc(updateExpr);
+    // Replace any non-existent shard key values with a null value.
+    updateExpr = shardKeyPattern.emplaceMissingShardKeyValuesForDocument(updateExpr);
 
-    // If there are no missing fields, return the update expression as-is.
-    if (missingFields.empty()) {
+    // If we aren't missing _id, return the update expression as-is.
+    if (updateExpr.hasField(kIdFieldName)) {
         return updateExpr;
     }
-    // If there are any missing fields other than _id, then this update can never succeed.
-    if (missingFields.size() > 1 || *missingFields.begin() != kIdFieldName) {
-        return {ErrorCodes::ShardKeyNotFound,
-                str::stream() << "Expected replacement document to include all shard key fields, "
-                                 "but the following were omitted: "
-                              << BSON("missingShardKeyFields" << missingFields)};
-    }
-    // If the only missing field is _id, attempt to extract it from an exact match in the update's
-    // query spec. This will guarantee that we can target a single shard, but it is not necessarily
-    // fatal if no exact _id can be found.
-    invariant(missingFields.size() == 1 && *missingFields.begin() == kIdFieldName);
-    const auto idFromQuery = kVirtualIdShardKey.extractShardKeyFromQuery(opCtx, updateDoc.getQ());
+
+    // We are missing _id, so attempt to extract it from an exact match in the update's query spec.
+    // This will guarantee that we can target a single shard, but it is not necessarily fatal if no
+    // exact _id can be found.
+    const auto idFromQuery =
+        kVirtualIdShardKey.extractShardKeyFromQuery(opCtx, nss, updateDoc.getQ());
     if (!idFromQuery.isOK()) {
         return idFromQuery;
     } else if (auto idElt = idFromQuery.getValue()[kIdFieldName]) {
         updateExpr = updateExpr.addField(idElt);
     }
-    // Confirm that the finalized replacement shard key is valid.
-    auto skStatus =
-        ShardKeyPattern::checkShardKeySize(shardKeyPattern.extractShardKeyFromDoc(updateExpr));
-    if (!skStatus.isOK()) {
-        return skStatus;
-    }
 
     return updateExpr;
 }
@@ -202,7 +192,7 @@ bool isExactIdQuery(OperationContext* opCtx,
                     const BSONObj query,
                     const BSONObj collation,
                     ChunkManager* manager) {
-    auto qr = stdx::make_unique<QueryRequest>(nss);
+    auto qr = std::make_unique<QueryRequest>(nss);
     qr->setFilter(query);
     if (!collation.isEmpty()) {
         qr->setCollation(collation);
@@ -216,17 +206,17 @@ bool isExactIdQuery(OperationContext* opCtx,
     return cq.isOK() && isExactIdQuery(opCtx, *cq.getValue(), manager);
 }
 //
-// Utilities to compare shard versions
+// Utilities to compare shard and db versions
 //
 
 /**
  * Returns the relationship of two shard versions. Shard versions of a collection that has not
- * been dropped and recreated and where there is at least one chunk on a shard are comparable,
- * otherwise the result is ambiguous.
+ * been dropped and recreated or had its shard key refined and where there is at least one chunk on
+ * a shard are comparable, otherwise the result is ambiguous.
  */
 CompareResult compareShardVersions(const ChunkVersion& shardVersionA,
                                    const ChunkVersion& shardVersionB) {
-    // Collection may have been dropped
+    // Collection may have been dropped or had its shard key refined.
     if (shardVersionA.epoch() != shardVersionB.epoch()) {
         return CompareResult_Unknown;
     }
@@ -268,7 +258,7 @@ ChunkVersion getShardVersion(const CachedCollectionRoutingInfo& routingInfo,
  * ChunkManager or is implicit in the primary shard of the collection.
  */
 CompareResult compareAllShardVersions(const CachedCollectionRoutingInfo& routingInfo,
-                                      const ShardVersionMap& remoteShardVersions) {
+                                      const StaleShardVersionMap& remoteShardVersions) {
     CompareResult finalResult = CompareResult_GTE;
 
     for (const auto& shardVersionEntry : remoteShardVersions) {
@@ -281,9 +271,12 @@ CompareResult compareAllShardVersions(const CachedCollectionRoutingInfo& routing
             // Throws b/c shard constructor throws
             cachedShardVersion = getShardVersion(routingInfo, shardId);
         } catch (const DBException& ex) {
-            warning() << "could not lookup shard " << shardId
-                      << " in local cache, shard metadata may have changed"
-                      << " or be unavailable" << causedBy(ex);
+            LOGV2_WARNING(22915,
+                          "could not lookup shard {shardId} in local cache, shard metadata may "
+                          "have changed or be unavailable: {error}",
+                          "Could not lookup shard in local cache",
+                          "shardId"_attr = shardId,
+                          "error"_attr = ex);
 
             return CompareResult_Unknown;
         }
@@ -303,22 +296,38 @@ CompareResult compareAllShardVersions(const CachedCollectionRoutingInfo& routing
     return finalResult;
 }
 
+CompareResult compareDbVersions(const CachedCollectionRoutingInfo& routingInfo,
+                                const DatabaseVersion& remoteDbVersion) {
+    DatabaseVersion cachedDbVersion = routingInfo.db().databaseVersion();
+
+    // Db may have been dropped
+    if (cachedDbVersion.getUuid() != remoteDbVersion.getUuid()) {
+        return CompareResult_Unknown;
+    }
+
+    // Db may have been moved
+    if (cachedDbVersion.getLastMod() < remoteDbVersion.getLastMod()) {
+        return CompareResult_LT;
+    }
+
+    return CompareResult_GTE;
+}
+
 /**
  * Whether or not the manager/primary pair is different from the other manager/primary pair.
  */
 bool isMetadataDifferent(const std::shared_ptr<ChunkManager>& managerA,
-                         const std::shared_ptr<Shard>& primaryA,
+                         const DatabaseVersion dbVersionA,
                          const std::shared_ptr<ChunkManager>& managerB,
-                         const std::shared_ptr<Shard>& primaryB) {
-    if ((managerA && !managerB) || (!managerA && managerB) || (primaryA && !primaryB) ||
-        (!primaryA && primaryB))
+                         const DatabaseVersion dbVersionB) {
+    if ((managerA && !managerB) || (!managerA && managerB))
         return true;
 
     if (managerA) {
         return managerA->getVersion() != managerB->getVersion();
     }
 
-    return primaryA->getId() != primaryB->getId();
+    return !databaseVersion::equal(dbVersionA, dbVersionB);
 }
 
 /**
@@ -326,10 +335,10 @@ bool isMetadataDifferent(const std::shared_ptr<ChunkManager>& managerA,
  * of the metadata.
  */
 bool wasMetadataRefreshed(const std::shared_ptr<ChunkManager>& managerA,
-                          const std::shared_ptr<Shard>& primaryA,
+                          const DatabaseVersion dbVersionA,
                           const std::shared_ptr<ChunkManager>& managerB,
-                          const std::shared_ptr<Shard>& primaryB) {
-    if (isMetadataDifferent(managerA, primaryA, managerB, primaryB))
+                          const DatabaseVersion dbVersionB) {
+    if (isMetadataDifferent(managerA, dbVersionA, managerB, dbVersionB))
         return true;
 
     if (managerA) {
@@ -364,7 +373,7 @@ Status ChunkManagerTargeter::init(OperationContext* opCtx) {
     if (_targetEpoch) {
         uassert(ErrorCodes::StaleEpoch, "Collection has been dropped", _routingInfo->cm());
         uassert(ErrorCodes::StaleEpoch,
-                "Collection has been dropped and recreated",
+                "Collection epoch has changed",
                 _routingInfo->cm()->getVersion().epoch() == *_targetEpoch);
     }
 
@@ -380,26 +389,14 @@ StatusWith<ShardEndpoint> ChunkManagerTargeter::targetInsert(OperationContext* o
     BSONObj shardKey;
 
     if (_routingInfo->cm()) {
-        //
-        // Sharded collections have the following requirements for targeting:
-        //
-        // Inserts must contain the exact shard key.
-        //
-
         shardKey = _routingInfo->cm()->getShardKeyPattern().extractShardKeyFromDoc(doc);
-
-        // Check shard key exists
-        if (shardKey.isEmpty()) {
-            return {ErrorCodes::ShardKeyNotFound,
-                    str::stream() << "document " << doc
-                                  << " does not contain shard key for pattern "
-                                  << _routingInfo->cm()->getShardKeyPattern().toString()};
-        }
-
-        // Check shard key size on insert
-        Status status = ShardKeyPattern::checkShardKeySize(shardKey);
-        if (!status.isOK())
-            return status;
+        // The shard key would only be empty after extraction if we encountered an error case,
+        // such as the shard key possessing an array value or array descendants. If the shard key
+        // presented to the targeter was empty, we would emplace the missing fields, and the
+        // extracted key here would *not* be empty.
+        uassert(ErrorCodes::ShardKeyNotFound,
+                "Shard key cannot contain array values or array descendants.",
+                !shardKey.isEmpty());
     }
 
     // Target the shard key or database primary
@@ -412,7 +409,9 @@ StatusWith<ShardEndpoint> ChunkManagerTargeter::targetInsert(OperationContext* o
                                         << "; no metadata found");
         }
 
-        return ShardEndpoint(_routingInfo->db().primary()->getId(), ChunkVersion::UNSHARDED());
+        return ShardEndpoint(_routingInfo->db().primary()->getId(),
+                             ChunkVersion::UNSHARDED(),
+                             _routingInfo->db().databaseVersion());
     }
 
     return Status::OK();
@@ -420,15 +419,19 @@ StatusWith<ShardEndpoint> ChunkManagerTargeter::targetInsert(OperationContext* o
 
 StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetUpdate(
     OperationContext* opCtx, const write_ops::UpdateOpEntry& updateDoc) const {
+    // If the update is replacement-style:
+    // 1. Attempt to target using the query. If this fails, AND the query targets more than one
+    //    shard,
+    // 2. Fall back to targeting using the replacement document.
     //
-    // The rule is simple; we always attempt to target using the query first. If the update is
-    // replacement-style and the query targets more than one shard, we fall back to targeting using
-    // the replacement document, since it must always contain a full shard key. Upserts should have
-    // full shard key in the query and we always use query for targeting. Because mongoD will
-    // automatically propagate '_id' from an existing document, and will extract it from an
-    // exact-match in the query in the case of an upsert, we augment the replacement doc with the
-    // query's '_id' for targeting purposes, if it exists.
+    // If the update is an upsert:
+    // 1. Always attempt to target using the query. Upserts must have the full shard key in the
+    //    query.
     //
+    // NOTE: A replacement document is allowed to have missing shard key values, because we target
+    // as if the the shard key values are specified as NULL. A replacement document is also allowed
+    // to have a missing '_id', and if the '_id' exists in the query, it will be emplaced in the
+    // replacement document for targeting purposes.
     const auto updateType = getUpdateExprType(updateDoc);
     if (!updateType.isOK()) {
         return updateType.getStatus();
@@ -441,14 +444,16 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetUpdate(
                     str::stream() << "could not target update on " << getNS().ns()
                                   << "; no metadata found"};
         }
-        return std::vector<ShardEndpoint>{
-            {_routingInfo->db().primaryId(), ChunkVersion::UNSHARDED()}};
+        return std::vector<ShardEndpoint>{{_routingInfo->db().primaryId(),
+                                           ChunkVersion::UNSHARDED(),
+                                           _routingInfo->db().databaseVersion()}};
     }
 
     const auto& shardKeyPattern = _routingInfo->cm()->getShardKeyPattern();
     const auto collation = write_ops::collationOf(updateDoc);
 
-    const auto updateExpr = getUpdateExpr(opCtx, shardKeyPattern, updateType.getValue(), updateDoc);
+    const auto updateExpr = getUpdateExprForTargeting(
+        opCtx, shardKeyPattern, getNS(), updateType.getValue(), updateDoc);
     const bool isUpsert = updateDoc.getUpsert();
     const auto query = updateDoc.getQ();
     if (!updateExpr.isOK()) {
@@ -477,7 +482,7 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetUpdate(
     // to target based on the replacement doc, it could result in an insertion even if a document
     // matching the query exists on another shard.
     if (isUpsert) {
-        return targetByShardKey(shardKeyPattern.extractShardKeyFromQuery(opCtx, query),
+        return targetByShardKey(shardKeyPattern.extractShardKeyFromQuery(opCtx, getNS(), query),
                                 "Failed to target upsert by query");
     }
 
@@ -531,8 +536,8 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetDelete(
 
         // Get the shard key
         StatusWith<BSONObj> status =
-            _routingInfo->cm()->getShardKeyPattern().extractShardKeyFromQuery(opCtx,
-                                                                              deleteDoc.getQ());
+            _routingInfo->cm()->getShardKeyPattern().extractShardKeyFromQuery(
+                opCtx, getNS(), deleteDoc.getQ());
 
         // Bad query
         if (!status.isOK())
@@ -554,7 +559,7 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetDelete(
     // We failed to target a single shard.
 
     // Parse delete query.
-    auto qr = stdx::make_unique<QueryRequest>(getNS());
+    auto qr = std::make_unique<QueryRequest>(getNS());
     qr->setFilter(deleteDoc.getQ());
     if (!collation.isEmpty()) {
         qr->setCollation(collation);
@@ -586,13 +591,6 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetDelete(
     return _targetQuery(opCtx, deleteDoc.getQ(), collation);
 }
 
-StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::_targetDoc(
-    OperationContext* opCtx, const BSONObj& doc, const BSONObj& collation) const {
-    // NOTE: This is weird and fragile, but it's the way our language works right now - documents
-    // are either A) invalid or B) valid equality queries over themselves.
-    return _targetQuery(opCtx, doc, collation);
-}
-
 StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::_targetQuery(
     OperationContext* opCtx, const BSONObj& query, const BSONObj& collation) const {
     if (!_routingInfo->db().primary() && !_routingInfo->cm()) {
@@ -601,22 +599,22 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::_targetQuery(
                               << "; no metadata found"};
     }
 
+    if (!_routingInfo->cm()) {
+        return std::vector<ShardEndpoint>{{_routingInfo->db().primaryId(),
+                                           ChunkVersion::UNSHARDED(),
+                                           _routingInfo->db().databaseVersion()}};
+    }
+
     std::set<ShardId> shardIds;
-    if (_routingInfo->cm()) {
-        try {
-            _routingInfo->cm()->getShardIdsForQuery(opCtx, query, collation, &shardIds);
-        } catch (const DBException& ex) {
-            return ex.toStatus();
-        }
-    } else {
-        shardIds.insert(_routingInfo->db().primary()->getId());
+    try {
+        _routingInfo->cm()->getShardIdsForQuery(opCtx, query, collation, &shardIds);
+    } catch (const DBException& ex) {
+        return ex.toStatus();
     }
 
     std::vector<ShardEndpoint> endpoints;
     for (auto&& shardId : shardIds) {
-        const auto version = _routingInfo->cm() ? _routingInfo->cm()->getVersion(shardId)
-                                                : ChunkVersion::UNSHARDED();
-        endpoints.emplace_back(std::move(shardId), version);
+        endpoints.emplace_back(std::move(shardId), _routingInfo->cm()->getVersion(shardId));
     }
 
     return endpoints;
@@ -634,29 +632,6 @@ StatusWith<ShardEndpoint> ChunkManagerTargeter::_targetShardKey(const BSONObj& s
     MONGO_UNREACHABLE;
 }
 
-StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetCollection() const {
-    if (!_routingInfo->db().primary() && !_routingInfo->cm()) {
-        return {ErrorCodes::NamespaceNotFound,
-                str::stream() << "could not target full range of " << getNS().ns()
-                              << "; metadata not found"};
-    }
-
-    std::set<ShardId> shardIds;
-    if (_routingInfo->cm()) {
-        _routingInfo->cm()->getAllShardIds(&shardIds);
-    } else {
-        shardIds.insert(_routingInfo->db().primary()->getId());
-    }
-
-    std::vector<ShardEndpoint> endpoints;
-    for (auto&& shardId : shardIds) {
-        const auto version = _routingInfo->cm() ? _routingInfo->cm()->getVersion(shardId)
-                                                : ChunkVersion::UNSHARDED();
-        endpoints.emplace_back(std::move(shardId), version);
-    }
-    return endpoints;
-}
-
 StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetAllShards(
     OperationContext* opCtx) const {
     if (!_routingInfo->db().primary() && !_routingInfo->cm()) {
@@ -668,11 +643,13 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetAllShards(
     std::vector<ShardId> shardIds;
     Grid::get(opCtx)->shardRegistry()->getAllShardIdsNoReload(&shardIds);
 
+    // This function is only called if doing a multi write that targets more than one shard. This
+    // implies the collection is sharded, so we should always have a chunk manager.
+    invariant(_routingInfo->cm());
+
     std::vector<ShardEndpoint> endpoints;
     for (auto&& shardId : shardIds) {
-        const auto version = _routingInfo->cm() ? _routingInfo->cm()->getVersion(shardId)
-                                                : ChunkVersion::UNSHARDED();
-        endpoints.emplace_back(std::move(shardId), version);
+        endpoints.emplace_back(std::move(shardId), _routingInfo->cm()->getVersion(shardId));
     }
 
     return endpoints;
@@ -680,12 +657,14 @@ StatusWith<std::vector<ShardEndpoint>> ChunkManagerTargeter::targetAllShards(
 
 void ChunkManagerTargeter::noteCouldNotTarget() {
     dassert(_remoteShardVersions.empty());
+    dassert(!_remoteDbVersion);
     _needsTargetingRefresh = true;
 }
 
-void ChunkManagerTargeter::noteStaleResponse(const ShardEndpoint& endpoint,
-                                             const StaleConfigInfo& staleInfo) {
+void ChunkManagerTargeter::noteStaleShardResponse(const ShardEndpoint& endpoint,
+                                                  const StaleConfigInfo& staleInfo) {
     dassert(!_needsTargetingRefresh);
+    dassert(!_remoteDbVersion);
 
     ChunkVersion remoteShardVersion;
     if (!staleInfo.getVersionWanted()) {
@@ -696,7 +675,7 @@ void ChunkManagerTargeter::noteStaleResponse(const ShardEndpoint& endpoint,
         remoteShardVersion = *staleInfo.getVersionWanted();
     }
 
-    ShardVersionMap::iterator it = _remoteShardVersions.find(endpoint.shardName);
+    StaleShardVersionMap::iterator it = _remoteShardVersions.find(endpoint.shardName);
     if (it == _remoteShardVersions.end()) {
         _remoteShardVersions.insert(std::make_pair(endpoint.shardName, remoteShardVersion));
     } else {
@@ -714,6 +693,37 @@ void ChunkManagerTargeter::noteStaleResponse(const ShardEndpoint& endpoint,
     }
 }
 
+void ChunkManagerTargeter::noteStaleDbResponse(const ShardEndpoint& endpoint,
+                                               const StaleDbRoutingVersion& staleInfo) {
+    dassert(!_needsTargetingRefresh);
+    dassert(_remoteShardVersions.empty());
+
+    DatabaseVersion remoteDbVersion;
+    if (!staleInfo.getVersionWanted()) {
+        // If the vWanted is not set, assume the wanted version is higher than our current version.
+        remoteDbVersion = _routingInfo->db().databaseVersion();
+        remoteDbVersion = databaseVersion::makeIncremented(remoteDbVersion);
+    } else {
+        remoteDbVersion = *staleInfo.getVersionWanted();
+    }
+
+    // If databaseVersion was sent, only one shard should have been targeted. The shard should have
+    // stopped processing the batch after one write encountered StaleDbVersion, after which the
+    // shard should have simply copied that StaleDbVersion error as the error for the rest of the
+    // writes in the batch. So, all of the write errors that contain a StaleDbVersion error should
+    // contain the same vWanted version.
+    if (_remoteDbVersion) {
+        // Use uassert rather than invariant since this is asserting the contents of a network
+        // response.
+        uassert(
+            ErrorCodes::InternalError,
+            "Did not expect to get multiple StaleDbVersion errors with different vWanted versions",
+            databaseVersion::equal(*_remoteDbVersion, remoteDbVersion));
+        return;
+    }
+    _remoteDbVersion = remoteDbVersion;
+}
+
 Status ChunkManagerTargeter::refreshIfNeeded(OperationContext* opCtx, bool* wasChanged) {
     bool dummy;
     if (!wasChanged) {
@@ -722,15 +732,21 @@ Status ChunkManagerTargeter::refreshIfNeeded(OperationContext* opCtx, bool* wasC
 
     *wasChanged = false;
 
-    LOG(4) << "ChunkManagerTargeter checking if refresh is needed, needsTargetingRefresh("
-           << _needsTargetingRefresh << ") remoteShardVersions empty ("
-           << _remoteShardVersions.empty() << ")";
+    LOGV2_DEBUG(22912,
+                4,
+                "ChunkManagerTargeter checking if refresh is needed, "
+                "needsTargetingRefresh({needsTargetingRefresh}) has remoteShardVersion "
+                "({hasRemoteShardVersions})) has remoteDbVersion ({hasRemoteDbVersion})",
+                "ChunkManagerTargeter checking if refresh is needed",
+                "needsTargetingRefresh"_attr = _needsTargetingRefresh,
+                "hasRemoteShardVersions"_attr = !_remoteShardVersions.empty(),
+                "hasRemoteDbVersion"_attr = static_cast<bool>(_remoteDbVersion));
 
     //
     // Did we have any stale config or targeting errors at all?
     //
 
-    if (!_needsTargetingRefresh && _remoteShardVersions.empty()) {
+    if (!_needsTargetingRefresh && _remoteShardVersions.empty() && !_remoteDbVersion) {
         return Status::OK();
     }
 
@@ -739,7 +755,7 @@ Status ChunkManagerTargeter::refreshIfNeeded(OperationContext* opCtx, bool* wasC
     //
 
     auto lastManager = _routingInfo->cm();
-    auto lastPrimary = _routingInfo->db().primary();
+    auto lastDbVersion = _routingInfo->db().databaseVersion();
 
     auto initStatus = init(opCtx);
     if (!initStatus.isOK()) {
@@ -754,24 +770,23 @@ Status ChunkManagerTargeter::refreshIfNeeded(OperationContext* opCtx, bool* wasC
     //
 
     if (_needsTargetingRefresh) {
-        _remoteShardVersions.clear();
+        // Reset the field
         _needsTargetingRefresh = false;
 
         // If we couldn't target, we might need to refresh if we haven't remotely refreshed
-        // the
-        // metadata since we last got it from the cache.
+        // the metadata since we last got it from the cache.
 
         bool alreadyRefreshed = wasMetadataRefreshed(
-            lastManager, lastPrimary, _routingInfo->cm(), _routingInfo->db().primary());
+            lastManager, lastDbVersion, _routingInfo->cm(), _routingInfo->db().databaseVersion());
 
         // If didn't already refresh the targeting information, refresh it
         if (!alreadyRefreshed) {
             // To match previous behavior, we just need an incremental refresh here
-            return _refreshNow(opCtx);
+            return _refreshShardVersionNow(opCtx);
         }
 
         *wasChanged = isMetadataDifferent(
-            lastManager, lastPrimary, _routingInfo->cm(), _routingInfo->db().primary());
+            lastManager, lastDbVersion, _routingInfo->cm(), _routingInfo->db().databaseVersion());
         return Status::OK();
     } else if (!_remoteShardVersions.empty()) {
         // If we got stale shard versions from remote shards, we may need to refresh
@@ -779,24 +794,76 @@ Status ChunkManagerTargeter::refreshIfNeeded(OperationContext* opCtx, bool* wasC
 
         CompareResult result = compareAllShardVersions(*_routingInfo, _remoteShardVersions);
 
-        LOG(4) << "ChunkManagerTargeter shard versions comparison result: " << (int)result;
+        LOGV2_DEBUG(22913,
+                    4,
+                    "ChunkManagerTargeter shard versions comparison result: {result}",
+                    "ChunkManagerTargeter shard versions comparison",
+                    "result"_attr = static_cast<int>(result));
 
         // Reset the versions
         _remoteShardVersions.clear();
 
         if (result == CompareResult_Unknown || result == CompareResult_LT) {
             // Our current shard versions aren't all comparable to the old versions, maybe drop
-            return _refreshNow(opCtx);
+            return _refreshShardVersionNow(opCtx);
         }
 
         *wasChanged = isMetadataDifferent(
-            lastManager, lastPrimary, _routingInfo->cm(), _routingInfo->db().primary());
+            lastManager, lastDbVersion, _routingInfo->cm(), _routingInfo->db().databaseVersion());
+        return Status::OK();
+    } else if (_remoteDbVersion) {
+        // If we got stale database versions from the remote shard, we may need to refresh
+        // NOTE: Not sure yet if this can happen simultaneously with targeting issues
+
+        CompareResult result = compareDbVersions(*_routingInfo, *_remoteDbVersion);
+
+        LOGV2_DEBUG(22914,
+                    4,
+                    "ChunkManagerTargeter database versions comparison result: {result}",
+                    "ChunkManagerTargeter database versions comparison",
+                    "result"_attr = static_cast<int>(result));
+
+        // Reset the version
+        _remoteDbVersion = boost::none;
+
+        if (result == CompareResult_Unknown || result == CompareResult_LT) {
+            // Our current db version isn't always comparable to the old version, it may have been
+            // dropped
+            return _refreshDbVersionNow(opCtx);
+        }
+
+        *wasChanged = isMetadataDifferent(
+            lastManager, lastDbVersion, _routingInfo->cm(), _routingInfo->db().databaseVersion());
         return Status::OK();
     }
 
     MONGO_UNREACHABLE;
 }
 
+bool ChunkManagerTargeter::endpointIsConfigServer() const {
+    uassert(ErrorCodes::NamespaceNotFound,
+            str::stream() << "could not verify full range of " << getNS().ns()
+                          << "; metadata not found",
+            _routingInfo->db().primary() || _routingInfo->cm());
+
+    if (!_routingInfo->cm()) {
+        return _routingInfo->db().primaryId() == ShardRegistry::kConfigServerShardId;
+    }
+
+    std::set<ShardId> shardIds;
+    _routingInfo->cm()->getAllShardIds(&shardIds);
+
+    if (std::any_of(shardIds.begin(), shardIds.end(), [](const auto& shardId) {
+            return shardId == ShardRegistry::kConfigServerShardId;
+        })) {
+        // There should be no namespaces that target both config servers and shards.
+        invariant(shardIds.size() == 1);
+        return true;
+    }
+
+    return false;
+}
+
 int ChunkManagerTargeter::getNShardsOwningChunks() const {
     if (_routingInfo->cm()) {
         return _routingInfo->cm()->getNShardsOwningChunks();
@@ -805,8 +872,19 @@ int ChunkManagerTargeter::getNShardsOwningChunks() const {
     return 0;
 }
 
-Status ChunkManagerTargeter::_refreshNow(OperationContext* opCtx) {
-    Grid::get(opCtx)->catalogCache()->onStaleShardVersion(std::move(*_routingInfo));
+Status ChunkManagerTargeter::_refreshShardVersionNow(OperationContext* opCtx) {
+    auto routingInfoStatus =
+        Grid::get(opCtx)->catalogCache()->getCollectionRoutingInfoWithRefresh(opCtx, _nss, true);
+    if (!routingInfoStatus.isOK()) {
+        return routingInfoStatus.getStatus();
+    }
+
+    return init(opCtx);
+}
+
+Status ChunkManagerTargeter::_refreshDbVersionNow(OperationContext* opCtx) {
+    Grid::get(opCtx)->catalogCache()->onStaleDatabaseVersion(
+        _nss.db(), std::move(_routingInfo->db().databaseVersion()));
 
     return init(opCtx);
 }
