@@ -27,11 +27,15 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
+#include <boost/core/null_deleter.hpp>
 #include <boost/filesystem/operations.hpp>
+#include <boost/log/attributes/value_extraction.hpp>
+#include <boost/log/core.hpp>
+#include <boost/log/sinks.hpp>
 #include <cctype>
 #include <fstream>
 #include <iostream>
@@ -43,15 +47,24 @@
 #include "mongo/base/init.h"
 #include "mongo/base/initializer.h"
 #include "mongo/base/status.h"
+#include "mongo/client/authenticate.h"
 #include "mongo/client/mongo_uri.h"
+#include "mongo/client/sasl_aws_client_options.h"
+#include "mongo/config.h"
 #include "mongo/db/auth/sasl_command_constants.h"
 #include "mongo/db/client.h"
-#include "mongo/db/commands/test_commands_enabled.h"
 #include "mongo/db/log_process_details.h"
 #include "mongo/db/server_options.h"
 #include "mongo/logger/console_appender.h"
 #include "mongo/logger/logger.h"
 #include "mongo/logger/message_event_utf8_encoder.h"
+#include "mongo/logv2/attributes.h"
+#include "mongo/logv2/component_settings_filter.h"
+#include "mongo/logv2/console.h"
+#include "mongo/logv2/json_formatter.h"
+#include "mongo/logv2/log_domain_global.h"
+#include "mongo/logv2/log_manager.h"
+#include "mongo/logv2/text_formatter.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/scripting/engine.h"
 #include "mongo/shell/linenoise.h"
@@ -60,16 +73,16 @@
 #include "mongo/shell/shell_utils_launcher.h"
 #include "mongo/stdx/utility.h"
 #include "mongo/transport/transport_layer_asio.h"
+#include "mongo/util/errno_util.h"
 #include "mongo/util/exit.h"
 #include "mongo/util/file.h"
-#include "mongo/util/log.h"
+#include "mongo/util/net/ocsp/ocsp_manager.h"
 #include "mongo/util/net/ssl_options.h"
 #include "mongo/util/password.h"
 #include "mongo/util/quick_exit.h"
 #include "mongo/util/scopeguard.h"
 #include "mongo/util/signal_handlers.h"
 #include "mongo/util/stacktrace.h"
-#include "mongo/util/startup_test.h"
 #include "mongo/util/str.h"
 #include "mongo/util/text.h"
 #include "mongo/util/version.h"
@@ -100,21 +113,14 @@ const std::string kDefaultMongoURL = "mongodb://"s + kDefaultMongoHost + ":"s +
 // to the latest version because there is no feature gating that currently occurs at the mongo shell
 // level. The server is responsible for rejecting usages of new features if its
 // featureCompatibilityVersion is lower.
-MONGO_INITIALIZER_WITH_PREREQUISITES(SetFeatureCompatibilityVersion42, ("EndStartupOptionSetup"))
+MONGO_INITIALIZER_WITH_PREREQUISITES(SetFeatureCompatibilityVersionLatest,
+                                     ("EndStartupOptionSetup"))
 (InitializerContext* context) {
     mongo::serverGlobalParams.featureCompatibility.setVersion(
-        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo42);
+        ServerGlobalParams::FeatureCompatibility::Version::kFullyUpgradedTo44);
     return Status::OK();
 }
 
-// Initialize the testCommandsEnabled server parameter to true since the mongo shell does not have
-// any test-only commands that could cause harm to the server, and it may be necessary to enable
-// this to test certain features, for example through benchRun (see SERVER-40419).
-MONGO_INITIALIZER_WITH_PREREQUISITES(EnableShellTestCommands, ("EndStartupOptionSetup"))
-(InitializerContext* context) {
-    setTestCommandsEnabled(true);
-    return Status::OK();
-}
 const auto kAuthParam = "authSource"s;
 
 /**
@@ -122,6 +128,7 @@ const auto kAuthParam = "authSource"s;
  */
 class ShellConsoleAppender final : public logger::ConsoleAppender<logger::MessageEventEphemeral> {
     using Base = logger::ConsoleAppender<logger::MessageEventEphemeral>;
+    friend class ShellBackend;
 
 public:
     using Base::Base;
@@ -162,6 +169,41 @@ private:
     static inline bool loggingEnabled = true;
 };
 
+/**
+ * Logv2 equivalent of ShellConsoleAppender above. Sharing the lock and LoggingDisabledScope.
+ */
+class ShellBackend final : public boost::log::sinks::text_ostream_backend {
+public:
+    void consume(boost::log::record_view const& rec, string_type const& formatted_message) {
+        using boost::log::extract;
+
+        auto lk = stdx::lock_guard(ShellConsoleAppender::mx);
+        if (!ShellConsoleAppender::loggingEnabled &&
+            !extract<logv2::LogTag>(logv2::attributes::tags(), rec)
+                 .get()
+                 .has(logv2::LogTag::kAllowDuringPromptingShell))
+            return;
+        boost::log::sinks::text_ostream_backend::consume(rec, formatted_message);
+    }
+};
+
+/**
+ * Formatter to provide specialized formatting for logs from javascript engine
+ */
+class ShellFormatter final : private logv2::PlainFormatter, private logv2::JSONFormatter {
+public:
+    void operator()(boost::log::record_view const& rec, boost::log::formatting_ostream& strm) {
+        using namespace logv2;
+        using boost::log::extract;
+
+        if (extract<LogTag>(attributes::tags(), rec).get().has(LogTag::kPlainShell)) {
+            PlainFormatter::operator()(rec, strm);
+        } else {
+            JSONFormatter::operator()(rec, strm);
+        }
+    }
+};
+
 }  // namespace
 
 namespace mongo {
@@ -178,6 +220,19 @@ enum ShellExitCode : int {
 Scope* shellMainScope;
 }  // namespace mongo
 
+bool isSessionTimedOut() {
+    static Date_t previousCommandTime = Date_t::now();
+    if (shellGlobalParams.idleSessionTimeout > Seconds(0)) {
+        const Date_t now = Date_t::now();
+
+        if (now > (previousCommandTime + shellGlobalParams.idleSessionTimeout)) {
+            return true;
+        }
+        previousCommandTime = now;
+    }
+    return false;
+}
+
 void generateCompletions(const std::string& prefix, std::vector<std::string>& all) {
     if (prefix.find('"') != std::string::npos)
         return;
@@ -185,7 +240,7 @@ void generateCompletions(const std::string& prefix, std::vector<std::string>& al
     try {
         BSONObj args = BSON("0" << prefix);
         shellMainScope->invokeSafe(
-            "function callShellAutocomplete(x) {shellAutocomplete(x)}", &args, NULL);
+            "function callShellAutocomplete(x) {shellAutocomplete(x)}", &args, nullptr);
         BSONObjBuilder b;
         shellMainScope->append(b, "", "__autocomplete__");
         BSONObj res = b.obj();
@@ -211,7 +266,7 @@ void completionHook(const char* text, linenoiseCompletions* lc) {
 void shellHistoryInit() {
     Status res = linenoiseHistoryLoad(shell_utils::getHistoryFilePath().string().c_str());
     if (!res.isOK()) {
-        error() << "Error loading history file: " << res;
+        std::cout << "Error loading history file: " << res << std::endl;
     }
     linenoiseSetCompletionCallback(completionHook);
 }
@@ -219,7 +274,7 @@ void shellHistoryInit() {
 void shellHistoryDone() {
     Status res = linenoiseHistorySave(shell_utils::getHistoryFilePath().string().c_str());
     if (!res.isOK()) {
-        error() << "Error saving history file: " << res;
+        std::cout << "Error saving history file: " << res << std::endl;
     }
     linenoiseHistoryFree();
 }
@@ -446,164 +501,8 @@ std::string getURIFromArgs(const std::string& arg,
     return parseDbHost("", "127.0.0.1");
 }
 
-static std::string OpSymbols = "~!%^&*-+=|:,<>/?.";
-
-bool isOpSymbol(char c) {
-    for (size_t i = 0; i < OpSymbols.size(); i++)
-        if (OpSymbols[i] == c)
-            return true;
-    return false;
-}
-
-bool isUseCmd(const std::string& code) {
-    std::string cmd = code;
-    if (cmd.find(" ") > 0)
-        cmd = cmd.substr(0, cmd.find(" "));
-    return cmd == "use";
-}
-
-/**
- * Skip over a quoted string, including quotes escaped with backslash
- *
- * @param code      String
- * @param start     Starting position within string, always > 0
- * @param quote     Quote character (single or double quote)
- * @return          Position of ending quote, or code.size() if no quote found
- */
-size_t skipOverString(const std::string& code, size_t start, char quote) {
-    size_t pos = start;
-    while (pos < code.size()) {
-        pos = code.find(quote, pos);
-        if (pos == std::string::npos) {
-            return code.size();
-        }
-        // We want to break if the quote we found is not escaped, but we need to make sure
-        // that the escaping backslash is not itself escaped.  Comparisons of start and pos
-        // are to keep us from reading beyond the beginning of the quoted string.
-        //
-        if (start == pos || code[pos - 1] != '\\' ||   // previous char was backslash
-            start == pos - 1 || code[pos - 2] == '\\'  // char before backslash was not another
-        ) {
-            break;  // The quote we found was not preceded by an unescaped backslash; it is real
-        }
-        ++pos;  // The quote we found was escaped with backslash, so it doesn't count
-    }
-    return pos;
-}
-
-bool isBalanced(const std::string& code) {
-    if (isUseCmd(code))
-        return true;  // don't balance "use <dbname>" in case dbname contains special chars
-    int curlyBrackets = 0;
-    int squareBrackets = 0;
-    int parens = 0;
-    bool danglingOp = false;
-
-    for (size_t i = 0; i < code.size(); i++) {
-        switch (code[i]) {
-            case '/':
-                if (i + 1 < code.size() && code[i + 1] == '/') {
-                    while (i < code.size() && code[i] != '\n')
-                        i++;
-                }
-                continue;
-            case '{':
-                curlyBrackets++;
-                break;
-            case '}':
-                if (curlyBrackets <= 0)
-                    return true;
-                curlyBrackets--;
-                break;
-            case '[':
-                squareBrackets++;
-                break;
-            case ']':
-                if (squareBrackets <= 0)
-                    return true;
-                squareBrackets--;
-                break;
-            case '(':
-                parens++;
-                break;
-            case ')':
-                if (parens <= 0)
-                    return true;
-                parens--;
-                break;
-            case '"':
-            case '\'':
-                i = skipOverString(code, i + 1, code[i]);
-                if (i >= code.size()) {
-                    return true;  // Do not let unterminated strings enter multi-line mode
-                }
-                break;
-            case '\\':
-                if (i + 1 < code.size() && code[i + 1] == '/')
-                    i++;
-                break;
-            case '+':
-            case '-':
-                if (i + 1 < code.size() && code[i + 1] == code[i]) {
-                    i++;
-                    continue;  // postfix op (++/--) can't be a dangling op
-                }
-                break;
-        }
-        if (i >= code.size()) {
-            danglingOp = false;
-            break;
-        }
-        if (isOpSymbol(code[i]))
-            danglingOp = true;
-        else if (!std::isspace(static_cast<unsigned char>(code[i])))
-            danglingOp = false;
-    }
-
-    return curlyBrackets == 0 && squareBrackets == 0 && parens == 0 && !danglingOp;
-}
-
-struct BalancedTest : public mongo::StartupTest {
-public:
-    void run() {
-        verify(isBalanced("x = 5"));
-        verify(isBalanced("function(){}"));
-        verify(isBalanced("function(){\n}"));
-        verify(!isBalanced("function(){"));
-        verify(isBalanced("x = \"{\";"));
-        verify(isBalanced("// {"));
-        verify(!isBalanced("// \n {"));
-        verify(!isBalanced("\"//\" {"));
-        verify(isBalanced("{x:/x\\//}"));
-        verify(!isBalanced("{ \\/// }"));
-        verify(isBalanced("x = 5 + y "));
-        verify(!isBalanced("x = "));
-        verify(!isBalanced("x = // hello"));
-        verify(!isBalanced("x = 5 +"));
-        verify(isBalanced(" x ++"));
-        verify(isBalanced("-- x"));
-        verify(!isBalanced("a."));
-        verify(!isBalanced("a. "));
-        verify(isBalanced("a.b"));
-
-        // SERVER-5809 and related cases --
-        verify(isBalanced("a = {s:\"\\\"\"}"));            // a = {s:"\""}
-        verify(isBalanced("db.test.save({s:\"\\\"\"})"));  // db.test.save({s:"\""})
-        verify(isBalanced("printjson(\" \\\" \")"));       // printjson(" \" ") -- SERVER-8554
-        verify(isBalanced("var a = \"\\\\\";"));           // var a = "\\";
-        verify(isBalanced("var a = (\"\\\\\") //\""));     // var a = ("\\") //"
-        verify(isBalanced("var a = (\"\\\\\") //\\\""));   // var a = ("\\") //\"
-        verify(isBalanced("var a = (\"\\\\\") //"));       // var a = ("\\") //
-        verify(isBalanced("var a = (\"\\\\\")"));          // var a = ("\\")
-        verify(isBalanced("var a = (\"\\\\\\\"\")"));      // var a = ("\\\"")
-        verify(!isBalanced("var a = (\"\\\\\" //\""));     // var a = ("\\" //"
-        verify(!isBalanced("var a = (\"\\\\\" //"));       // var a = ("\\" //
-        verify(!isBalanced("var a = (\"\\\\\""));          // var a = ("\\"
-    }
-} balanced_test;
-
 std::string finishCode(std::string code) {
-    while (!isBalanced(code)) {
+    while (!shell_utils::isBalanced(code)) {
         inMultiLine = true;
         code += "\n";
         // cancel multiline if two blank lines are entered
@@ -704,7 +603,7 @@ static void edit(const std::string& whatToEdit) {
         GetTempPathA(sizeof tempFolder, tempFolder);
         sb << tempFolder << "mongo_edit" << time(0) + i << ".js";
 #else
-        sb << "/tmp/mongo_edit" << time(0) + i << ".js";
+        sb << "/tmp/mongo_edit" << time(nullptr) + i << ".js";
 #endif
         filename = sb.str();
         if (!::mongo::shell_utils::fileExists(filename))
@@ -796,7 +695,8 @@ static void edit(const std::string& whatToEdit) {
 namespace {
 bool mechanismRequiresPassword(const MongoURI& uri) {
     if (const auto authMechanisms = uri.getOption("authMechanism")) {
-        constexpr std::array<StringData, 2> passwordlessMechanisms{"GSSAPI"_sd, "MONGODB-X509"_sd};
+        constexpr std::array<StringData, 3> passwordlessMechanisms{
+            auth::kMechanismGSSAPI, auth::kMechanismMongoX509, auth::kMechanismMongoAWS};
         const std::string& authMechanism = authMechanisms.get();
         for (const auto& mechanism : passwordlessMechanisms) {
             if (mechanism.toString() == authMechanism) {
@@ -820,16 +720,20 @@ int _main(int argc, char* argv[], char** envp) {
     setupSignalHandlers();
     setupSignals();
 
-    logger::globalLogManager()->getGlobalDomain()->clearAppenders();
-    logger::globalLogManager()->getGlobalDomain()->attachAppender(
-        std::make_unique<ShellConsoleAppender>(
-            std::make_unique<logger::MessageEventDetailsEncoder>()));
+    // Log to stdout for any early logging before we re-configure the logger
+    auto& lv2Manager = logv2::LogManager::global();
+    logv2::LogDomainGlobal::ConfigurationOptions lv2Config;
+    uassertStatusOK(lv2Manager.getGlobalDomainInternal().configure(lv2Config));
+
     mongo::shell_utils::RecordMyLocation(argv[0]);
 
     mongo::runGlobalInitializersOrDie(argc, argv, envp);
     setGlobalServiceContext(ServiceContext::make());
     // TODO This should use a TransportLayerManager or TransportLayerFactory
     auto serviceContext = getGlobalServiceContext();
+
+    OCSPManager::get()->startThreadPool();
+
     transport::TransportLayerASIO::Options opts;
     opts.enableIPv6 = shellGlobalParams.enableIPv6;
     opts.mode = transport::TransportLayerASIO::Options::kEgress;
@@ -848,12 +752,22 @@ int _main(int argc, char* argv[], char** envp) {
     if (!mongo::serverGlobalParams.quiet.load())
         std::cout << mongoShellVersion(VersionInfoInterface::instance()) << std::endl;
 
-    mongo::StartupTest::runTests();
+    auto consoleSink = boost::make_shared<boost::log::sinks::synchronous_sink<ShellBackend>>();
+    consoleSink->set_filter(logv2::ComponentSettingsFilter(lv2Manager.getGlobalDomain(),
+                                                           lv2Manager.getGlobalSettings()));
+    consoleSink->set_formatter(ShellFormatter());
 
-    logger::globalLogManager()
-        ->getNamedDomain("javascriptOutput")
-        ->attachAppender(std::make_unique<ShellConsoleAppender>(
-            std::make_unique<logger::MessageEventUnadornedEncoder>()));
+    consoleSink->locked_backend()->add_stream(
+        boost::shared_ptr<std::ostream>(&logv2::Console::out(), boost::null_deleter()));
+
+    consoleSink->locked_backend()->auto_flush();
+
+    // Remove the initial config from above when setting this sink, otherwise we log everything
+    // twice.
+    lv2Config.makeDisabled();
+    uassertStatusOK(lv2Manager.getGlobalDomainInternal().configure(lv2Config));
+
+    boost::log::core::get()->add_sink(std::move(consoleSink));
 
     // Get the URL passed to the shell
     std::string& cmdlineURI = shellGlobalParams.url;
@@ -869,6 +783,13 @@ int _main(int argc, char* argv[], char** envp) {
     parsedURI.setOptionIfNecessary("authSource"s, shellGlobalParams.authenticationDatabase);
     parsedURI.setOptionIfNecessary("gssapiServiceName"s, shellGlobalParams.gssapiServiceName);
     parsedURI.setOptionIfNecessary("gssapiHostName"s, shellGlobalParams.gssapiHostName);
+#ifdef MONGO_CONFIG_SSL
+    if (!awsIam::saslAwsClientGlobalParams.awsSessionToken.empty()) {
+        parsedURI.setOptionIfNecessary("authmechanismproperties"s,
+                                       std::string("AWS_SESSION_TOKEN:") +
+                                           awsIam::saslAwsClientGlobalParams.awsSessionToken);
+    }
+#endif
 
     if (const auto authMechanisms = parsedURI.getOption("authMechanism")) {
         std::stringstream ss;
@@ -943,7 +864,7 @@ int _main(int argc, char* argv[], char** envp) {
     rcGlobalLocation = "/etc/mongorc.js";
 #else
     wchar_t programDataPath[MAX_PATH];
-    if (S_OK == SHGetFolderPathW(NULL, CSIDL_COMMON_APPDATA, NULL, 0, programDataPath)) {
+    if (S_OK == SHGetFolderPathW(nullptr, CSIDL_COMMON_APPDATA, nullptr, 0, programDataPath)) {
         rcGlobalLocation = str::stream()
             << toUtf8String(programDataPath) << "\\MongoDB\\mongorc.js";
     }
@@ -958,7 +879,7 @@ int _main(int argc, char* argv[], char** envp) {
     if (!shellGlobalParams.script.empty()) {
         mongo::shell_utils::MongoProgramScope s;
         if (!scope->exec(shellGlobalParams.script, "(shell eval)", false, true, false)) {
-            error() << "exiting with code " << static_cast<int>(kEvalError);
+            std::cout << "exiting with code " << static_cast<int>(kEvalError) << std::endl;
             return kEvalError;
         }
         scope->exec("shellPrintHelper( __lastres__ );", "(shell2 eval)", true, true, false);
@@ -971,8 +892,8 @@ int _main(int argc, char* argv[], char** envp) {
             std::cout << "loading file: " << shellGlobalParams.files[i] << std::endl;
 
         if (!scope->execFile(shellGlobalParams.files[i], false, true)) {
-            severe() << "failed to load: " << shellGlobalParams.files[i];
-            error() << "exiting with code " << static_cast<int>(kInputFileError);
+            std::cout << "failed to load: " << shellGlobalParams.files[i] << std::endl;
+            std::cout << "exiting with code " << static_cast<int>(kInputFileError) << std::endl;
             return kInputFileError;
         }
 
@@ -986,9 +907,10 @@ int _main(int argc, char* argv[], char** envp) {
             std::cout << std::endl;
 
             if (mongo::shell_utils::KillMongoProgramInstances() != EXIT_SUCCESS) {
-                severe() << "one more more child processes exited with an error during "
-                         << shellGlobalParams.files[i];
-                error() << "exiting with code " << static_cast<int>(kProcessTerminationError);
+                std::cout << "one more more child processes exited with an error during "
+                          << shellGlobalParams.files[i] << std::endl;
+                std::cout << "exiting with code " << static_cast<int>(kProcessTerminationError)
+                          << std::endl;
                 return kProcessTerminationError;
             }
 
@@ -997,14 +919,16 @@ int _main(int argc, char* argv[], char** envp) {
                 "function() { return typeof TestData === 'object' && TestData !== null && "
                 "TestData.hasOwnProperty('failIfUnterminatedProcesses') && "
                 "TestData.failIfUnterminatedProcesses; }"_sd;
-            shellMainScope->invokeSafe(code.rawData(), 0, 0);
+            shellMainScope->invokeSafe(code.rawData(), nullptr, nullptr);
             failIfUnterminatedProcesses = shellMainScope->getBoolean("__returnValue");
 
             if (failIfUnterminatedProcesses) {
-                severe() << "exiting with a failure due to unterminated processes, "
-                            "a call to MongoRunner.stopMongod(), ReplSetTest#stopSet(), or "
-                            "ShardingTest#stop() may be missing from the test";
-                error() << "exiting with code " << static_cast<int>(kUnterminatedProcess);
+                std::cout << "exiting with a failure due to unterminated processes, "
+                             "a call to MongoRunner.stopMongod(), ReplSetTest#stopSet(), or "
+                             "ShardingTest#stop() may be missing from the test"
+                          << std::endl;
+                std::cout << "exiting with code " << static_cast<int>(kUnterminatedProcess)
+                          << std::endl;
                 return kUnterminatedProcess;
             }
         }
@@ -1021,10 +945,10 @@ int _main(int argc, char* argv[], char** envp) {
         std::string rcLocation;
         if (!shellGlobalParams.norc) {
 #ifndef _WIN32
-            if (getenv("HOME") != NULL)
+            if (getenv("HOME") != nullptr)
                 rcLocation = str::stream() << getenv("HOME") << "/.mongorc.js";
 #else
-            if (getenv("HOMEDRIVE") != NULL && getenv("HOMEPATH") != NULL)
+            if (getenv("HOMEDRIVE") != nullptr && getenv("HOMEPATH") != nullptr)
                 rcLocation = str::stream()
                     << toUtf8String(_wgetenv(L"HOMEDRIVE")) << toUtf8String(_wgetenv(L"HOMEPATH"))
                     << "\\.mongorc.js";
@@ -1032,9 +956,12 @@ int _main(int argc, char* argv[], char** envp) {
             if (!rcLocation.empty() && ::mongo::shell_utils::fileExists(rcLocation)) {
                 hasMongoRC = true;
                 if (!scope->execFile(rcLocation, false, true)) {
-                    severe() << "The \".mongorc.js\" file located in your home folder could not be "
-                                "executed";
-                    error() << "exiting with code " << static_cast<int>(kMongorcError);
+                    std::cout
+                        << "The \".mongorc.js\" file located in your home folder could not be "
+                           "executed"
+                        << std::endl;
+                    std::cout << "exiting with code " << static_cast<int>(kMongorcError)
+                              << std::endl;
                     return kMongorcError;
                 }
             }
@@ -1115,6 +1042,15 @@ int _main(int argc, char* argv[], char** envp) {
                 free(line);
                 break;
             }
+
+            // Support idle session lifetime limits
+            if (isSessionTimedOut()) {
+                std::cout << "Idle Connection Timeout: Shell session has expired" << std::endl;
+                if (line)
+                    free(line);
+                break;
+            }
+
             if (code == "cls") {
                 free(line);
                 linenoiseClearScreen();
@@ -1214,8 +1150,8 @@ int wmain(int argc, wchar_t* argvW[], wchar_t* envpW[]) {
         WindowsCommandLine wcl(argc, argvW, envpW);
         returnCode = _main(argc, wcl.argv(), wcl.envp());
     } catch (mongo::DBException& e) {
-        severe() << "exception: " << e.what();
-        error() << "exiting with code " << static_cast<int>(kDBException);
+        std::cout << "exception: " << e.what() << std::endl;
+        std::cout << "exiting with code " << static_cast<int>(kDBException) << std::endl;
         returnCode = kDBException;
     }
     quickExit(returnCode);
@@ -1226,8 +1162,8 @@ int main(int argc, char* argv[], char** envp) {
     try {
         returnCode = _main(argc, argv, envp);
     } catch (mongo::DBException& e) {
-        severe() << "exception: " << e.what();
-        error() << "exiting with code " << static_cast<int>(kDBException);
+        std::cout << "exception: " << e.what() << std::endl;
+        std::cout << "exiting with code " << static_cast<int>(kDBException) << std::endl;
         returnCode = kDBException;
     }
     quickExit(returnCode);
