@@ -84,6 +84,89 @@ MongoRunner.VersionSub = function(pattern, version) {
     this.version = version;
 };
 
+(function() {
+// Hang Analyzer integration.
+
+function getPids() {
+    let pids = [];
+    if (typeof TestData !== 'undefined' && typeof TestData.peerPids !== 'undefined') {
+        pids = pids.concat(TestData.peerPids);
+    }
+    pids = pids.concat(MongoRunner.runningChildPids());
+    return pids;
+}
+
+// A path.join-like thing for paths that must work
+// on Windows (\-separated) and *nix (/-separated).
+function pathJoin(...parts) {
+    const separator = _isWindows() ? '\\' : '/';
+    return parts.join(separator);
+}
+
+// Internal state to determine if the hang analyzer should be enabled or not.
+// Accessible via global setter/getter defined below.
+let _hangAnalyzerEnabled = true;
+
+/**
+ * Run `./buildscripts/resmoke.py hang-analyzer`.
+ *
+ * @param {Number[]} pids
+ *     optional pids of processes to pass to the hang analyzer.
+ *     If not specified will use `TestData.peerPids` (pids of
+ *     "fixture" processes started and passed in by resmoke)
+ *     plus `MongoRunner.runningChildPids()` which includes all
+ *     child processes started by `MongoRunner.runMongo*()` etc.
+ */
+function runHangAnalyzer(pids) {
+    if (typeof TestData === 'undefined') {
+        print("Skipping runHangAnalyzer: no TestData (not running from resmoke)");
+        return;
+    }
+
+    if (!TestData.inEvergreen) {
+        print('Skipping runHangAnalyzer: not running in Evergreen');
+        return;
+    }
+
+    if (!_hangAnalyzerEnabled) {
+        print('Skipping runHangAnalyzer: manually disabled');
+        return;
+    }
+
+    if (typeof pids === 'undefined') {
+        pids = getPids();
+    }
+    if (pids.length <= 0) {
+        print("Skipping runHangAnalyzer: no running child or peer mongo processes.");
+        return;
+    }
+    // Result of runningChildPids may be NumberLong(), so
+    // add 0 to convert to Number.
+    pids = pids.map(p => p + 0).join(',');
+    print(`Running hang analyzer for pids [${pids}]`);
+
+    const scriptPath = pathJoin('.', 'buildscripts', 'resmoke.py');
+    const args = ['python', scriptPath, 'hang-analyzer', '-d', pids];
+
+    // Enable core dumps if not an ASAN build.
+    if (!_isAddressSanitizerActive()) {
+        args.push('-c');
+    }
+
+    return runProgram(...args);
+}
+
+MongoRunner.runHangAnalyzer = runHangAnalyzer;
+
+MongoRunner.runHangAnalyzer.enable = function() {
+    _hangAnalyzerEnabled = true;
+};
+
+MongoRunner.runHangAnalyzer.disable = function() {
+    _hangAnalyzerEnabled = false;
+};
+})();
+
 /**
  * Returns an array of version elements from a version string.
  *
@@ -124,7 +207,7 @@ MongoRunner.binVersionSubs = [
     new MongoRunner.VersionSub(extractMajorVersionFromVersionString(shellVersion()),
                                shellVersion()),
     // To-be-updated when we branch for the next release.
-    new MongoRunner.VersionSub("last-stable", "4.0")
+    new MongoRunner.VersionSub("last-stable", "4.2")
 ];
 
 MongoRunner.getBinVersionFor = function(version) {
@@ -241,7 +324,6 @@ MongoRunner.logicalOptions = {
     waitForConnect: true,
     bridgeOptions: true,
     skipValidation: true,
-    allowedExitCode: true,
 };
 
 MongoRunner.toRealPath = function(path, pathOpts) {
@@ -363,7 +445,7 @@ MongoRunner.arrOptions = function(binaryName, args) {
 
             fullArgs.push("--" + k);
 
-            if (v != "") {
+            if (v !== "") {
                 fullArgs.push("" + v);
             }
         };
@@ -503,6 +585,8 @@ MongoRunner.mongoOptions = function(opts) {
         (!opts.restart && !opts.noRemember) || (opts.restart && opts.appendOptions);
     if (shouldRemember) {
         MongoRunner.savedOptions[opts.runId] = Object.merge(opts, {});
+        // We don't want to persist 'waitForConnect' across node restarts.
+        delete MongoRunner.savedOptions[opts.runId].waitForConnect;
     }
 
     if (jsTestOptions().networkMessageCompressors) {
@@ -571,9 +655,22 @@ var _removeSetParameterIfBeforeVersion = function(opts, parameterName, requiredV
  *     oplogSize
  *   }
  */
-MongoRunner.mongodOptions = function(opts) {
+MongoRunner.mongodOptions = function(opts = {}) {
     opts = MongoRunner.mongoOptions(opts);
 
+    if (jsTestOptions().alwaysUseLogFiles) {
+        if (opts.cleanData || opts.startClean || opts.noCleanData === false ||
+            opts.useLogFiles === false) {
+            throw new Error("Always using log files, but received conflicting option.");
+        }
+
+        opts.cleanData = false;
+        opts.startClean = false;
+        opts.noCleanData = true;
+        opts.useLogFiles = true;
+        opts.logappend = "";
+    }
+
     opts.dbpath = MongoRunner.toRealDir(opts.dbpath || "$dataDir/mongod-$port", opts.pathOpts);
 
     opts.pathOpts = Object.merge(opts.pathOpts, {dbpath: opts.dbpath});
@@ -668,6 +765,15 @@ MongoRunner.mongosOptions = function(opts) {
         opts.configdb = opts.configdb.host;
     }
 
+    if (jsTestOptions().alwaysUseLogFiles) {
+        if (opts.useLogFiles === false) {
+            throw new Error("Always using log files, but received conflicting option.");
+        }
+
+        opts.useLogFiles = true;
+        opts.logappend = "";
+    }
+
     opts.pathOpts = Object.merge(opts.pathOpts, {configdb: opts.configdb.replace(/:|\/|,/g, "-")});
 
     if (!opts.logFile && opts.useLogFiles) {
@@ -710,6 +816,13 @@ MongoRunner.mongosOptions = function(opts) {
     return opts;
 };
 
+/**
+ * @return {NumberLong[]} Running pids e.g. those started by `MongoRunner.runMongod`.
+ */
+MongoRunner.runningChildPids = function() {
+    return _runningMongoChildProcessIds();
+};
+
 /**
  * Starts a mongod instance.
  *
@@ -830,8 +943,7 @@ MongoRunner.StopError.prototype = Object.create(Error.prototype);
 MongoRunner.StopError.prototype.constructor = MongoRunner.StopError;
 
 // Constants for exit codes of MongoDB processes
-// On Windows, std::abort causes the process to exit with return code 14.
-MongoRunner.EXIT_ABORT = _isWindows() ? 14 : -6;
+MongoRunner.EXIT_ABORT = -6;
 MongoRunner.EXIT_CLEAN = 0;
 MongoRunner.EXIT_BADOPTIONS = 2;
 MongoRunner.EXIT_REPLICATION_ERROR = 3;
@@ -871,11 +983,12 @@ MongoRunner.validateCollectionsCallback = function(port) {};
  *      skipValidation: <bool>,
  *      allowedExitCode: <int>
  *    }
+ * @param {boolean} waitpid should we wait for the process to terminate after stopping it.
  *
  * Note: The auth option is required in a authenticated mongod running in Windows since
  *  it uses the shutdown command, which requires admin credentials.
  */
-MongoRunner.stopMongod = function(conn, signal, opts) {
+MongoRunner.stopMongod = function(conn, signal, opts, waitpid) {
     if (!conn.pid) {
         throw new Error("first arg must have a `pid` property; " +
                         "it is usually the object returned from MongoRunner.runMongod/s");
@@ -888,6 +1001,7 @@ MongoRunner.stopMongod = function(conn, signal, opts) {
 
     signal = parseInt(signal) || 15;
     opts = opts || {};
+    waitpid = (waitpid === undefined) ? true : waitpid;
 
     var allowedExitCode = MongoRunner.EXIT_CLEAN;
 
@@ -918,7 +1032,12 @@ MongoRunner.stopMongod = function(conn, signal, opts) {
             MongoRunner.validateCollectionsCallback(port);
         }
 
-        returnCode = _stopMongoProgram(port, signal, opts);
+        returnCode = _stopMongoProgram(port, signal, opts, waitpid);
+    }
+
+    // If we are not waiting for shutdown, then there is no exit code to check.
+    if (!waitpid) {
+        return 0;
     }
     if (allowedExitCode !== returnCode) {
         throw new MongoRunner.StopError(returnCode);
@@ -932,83 +1051,6 @@ MongoRunner.stopMongod = function(conn, signal, opts) {
 
 MongoRunner.stopMongos = MongoRunner.stopMongod;
 
-/**
- * Starts an instance of the specified mongo tool
- *
- * @param {String} binaryName - The name of the tool to run.
- * @param {Object} [opts={}] - Options of the form --flag or --key=value to pass to the tool.
- * @param {string} [opts.binVersion] - The version of the tool to run.
- *
- * @param {...string} positionalArgs - Positional arguments to pass to the tool after all
- * options have been specified. For example,
- * MongoRunner.runMongoTool("executable", {key: value}, arg1, arg2) would invoke
- * ./executable --key value arg1 arg2.
- *
- * @see MongoRunner.arrOptions
- */
-MongoRunner.runMongoTool = function(binaryName, opts, ...positionalArgs) {
-    var opts = opts || {};
-
-    // Normalize and get the binary version to use
-    if (opts.binVersion instanceof MongoRunner.versionIterator.iterator) {
-        // Advance the version iterator so that subsequent calls to MongoRunner.runMongoTool()
-        // use the next version in the list.
-        const iterator = opts.binVersion;
-        opts.binVersion = iterator.current();
-        iterator.advance();
-    }
-    opts.binVersion = MongoRunner.getBinVersionFor(opts.binVersion);
-
-    // Recent versions of the mongo tools support a --dialTimeout flag to set for how
-    // long they retry connecting to a mongod or mongos process. We have them retry
-    // connecting for up to 30 seconds to handle when the tests are run on a
-    // resource-constrained host machine.
-    //
-    // The bsondump tool doesn't accept the --dialTimeout flag because it doesn't connect to a
-    // mongod or mongos process.
-    if (!opts.hasOwnProperty('dialTimeout') && binaryName !== 'bsondump' &&
-        _toolVersionSupportsDialTimeout(opts.binVersion)) {
-        opts['dialTimeout'] = '30';
-    }
-
-    // Convert 'opts' into an array of arguments.
-    var argsArray = MongoRunner.arrOptions(binaryName, opts);
-
-    // Append any positional arguments that were specified.
-    argsArray.push(...positionalArgs);
-
-    return runMongoProgram.apply(null, argsArray);
-};
-
-var _toolVersionSupportsDialTimeout = function(version) {
-    if (version === "latest" || version === "") {
-        return true;
-    }
-    var versionParts =
-        convertVersionStringToArray(version).slice(0, 3).map(part => parseInt(part, 10));
-    if (versionParts.length === 2) {
-        versionParts.push(Infinity);
-    }
-
-    if (versionParts[0] > 3 || (versionParts[0] === 3 && versionParts[1] > 3)) {
-        // The --dialTimeout command line option is supported by the tools
-        // with a major version newer than 3.3.
-        return true;
-    }
-
-    for (var supportedVersion of ["3.3.4", "3.2.5", "3.0.12"]) {
-        var supportedVersionParts = convertVersionStringToArray(supportedVersion)
-                                        .slice(0, 3)
-                                        .map(part => parseInt(part, 10));
-        if (versionParts[0] === supportedVersionParts[0] &&
-            versionParts[1] === supportedVersionParts[1] &&
-            versionParts[2] >= supportedVersionParts[2]) {
-            return true;
-        }
-    }
-    return false;
-};
-
 // Given a test name figures out a directory for that test to use for dump files and makes sure
 // that directory exists and is empty.
 MongoRunner.getAndPrepareDumpDirectory = function(testName) {
@@ -1057,6 +1099,11 @@ function appendSetParameterArgs(argArray) {
     // programName includes the version, e.g., mongod-3.2.
     // baseProgramName is the program name without any version information, e.g., mongod.
     let programName = argArray[0];
+    const separator = _isWindows() ? '\\' : '/';
+    if (programName.indexOf(separator) !== -1) {
+        let pathElements = programName.split(separator);
+        programName = pathElements[pathElements.length - 1];
+    }
 
     let [baseProgramName, programVersion] = programName.split("-");
     let programMajorMinorVersion = 0;
@@ -1069,6 +1116,13 @@ function appendSetParameterArgs(argArray) {
         if (jsTest.options().enableTestCommands) {
             argArray.push(...['--setParameter', "enableTestCommands=1"]);
         }
+
+        if (!programMajorMinorVersion || programMajorMinorVersion >= 404) {
+            if (jsTest.options().testingDiagnosticsEnabled) {
+                argArray.push(...['--setParameter', "testingDiagnosticsEnabled=1"]);
+            }
+        }
+
         if (jsTest.options().authMechanism && jsTest.options().authMechanism != "SCRAM-SHA-1") {
             if (!argArrayContainsSetParameterValue('authenticationMechanisms=')) {
                 argArray.push(...['--setParameter',
@@ -1094,9 +1148,7 @@ function appendSetParameterArgs(argArray) {
             }
 
             // Disable background cache refreshing to avoid races in tests
-            if (!argArrayContainsSetParameterValue('disableLogicalSessionCacheRefresh=')) {
-                argArray.push(...['--setParameter', "disableLogicalSessionCacheRefresh=true"]);
-            }
+            argArray.push(...['--setParameter', "disableLogicalSessionCacheRefresh=true"]);
         }
 
         // Since options may not be backward compatible, mongos options are not
@@ -1139,8 +1191,8 @@ function appendSetParameterArgs(argArray) {
                 }
             }
 
-            // New mongod-specific option in 4.2.x.
-            if (!programMajorMinorVersion || programMajorMinorVersion >= 420) {
+            // New mongod-specific option in 4.3.x.
+            if (!programMajorMinorVersion || programMajorMinorVersion >= 430) {
                 // Allow the parameter to be overridden if set explicitly via TestData.
                 if ((jsTest.options().setParameters === undefined ||
                      jsTest.options().setParameters['minNumChunksForSessionsCollection'] ===
@@ -1168,9 +1220,7 @@ function appendSetParameterArgs(argArray) {
                 }
             }
 
-            // Since options may not be backward compatible, mongod options are not
-            // set on older versions, e.g., mongod-3.0.
-            if (programName.endsWith('mongod')) {
+            if (!programMajorMinorVersion || programMajorMinorVersion >= 306) {
                 if (jsTest.options().storageEngine === "wiredTiger" ||
                     !jsTest.options().storageEngine) {
                     if (jsTest.options().enableMajorityReadConcern !== undefined &&
@@ -1178,6 +1228,14 @@ function appendSetParameterArgs(argArray) {
                         argArray.push(...['--enableMajorityReadConcern',
                                           jsTest.options().enableMajorityReadConcern.toString()]);
                     }
+                }
+            }
+
+            // Since options may not be backward compatible, mongod options are not
+            // set on older versions, e.g., mongod-3.0.
+            if (programName.endsWith('mongod')) {
+                if (jsTest.options().storageEngine === "wiredTiger" ||
+                    !jsTest.options().storageEngine) {
                     if (jsTest.options().storageEngineCacheSizeGB &&
                         !argArrayContains('--wiredTigerCacheSizeGB')) {
                         argArray.push(...['--wiredTigerCacheSizeGB',
@@ -1226,8 +1284,11 @@ function appendSetParameterArgs(argArray) {
                             continue;
                         }
 
-                        if (paramName === "storeFindAndModifyImagesInSideCollection" &&
-                            argArrayContains("storeFindAndModifyImagesInSideCollection")) {
+                        // Do not set this if test commands are not enabled or we are not on a new
+                        // enough version.
+                        if (paramName === "assertStableTimestampEqualsAppliedThroughOnRecovery" &&
+                            (!jsTest.options().enableTestCommands ||
+                             programMajorMinorVersion < 404)) {
                             continue;
                         }
 
@@ -1249,6 +1310,38 @@ function appendSetParameterArgs(argArray) {
     return argArray;
 }
 
+/**
+ * Continuously tries to establish a connection to the server on the specified port.
+ *
+ * If a connection cannot be established within a time limit, an exception will be thrown. If the
+ * process for the given 'pid' is found to no longer be running, this function will terminate and
+ * return null.
+ *
+ * @param {int} [pid] the process id of the node to connect to.
+ * @param {int} [port] the port of the node to connect to.
+ * @returns a new Mongo connection object, or null if the process is not running.
+ */
+MongoRunner.awaitConnection = function(pid, port) {
+    var conn = null;
+    assert.soon(function() {
+        try {
+            conn = new Mongo("127.0.0.1:" + port);
+            conn.pid = pid;
+            return true;
+        } catch (e) {
+            var res = checkProgram(pid);
+            if (!res.alive) {
+                print("mongo program was not running at " + port +
+                      ", process ended with exit code: " + res.exitCode);
+                serverExitCodeMap[port] = res.exitCode;
+                return true;
+            }
+        }
+        return false;
+    }, "unable to connect to mongo program on port " + port, 600 * 1000);
+    return conn;
+};
+
 /**
  * Start a mongo process with a particular argument array.
  * If we aren't waiting for connect, return {pid: <pid>}.
@@ -1270,31 +1363,14 @@ MongoRunner._startWithArgs = function(argArray, env, waitForConnect) {
 
     delete serverExitCodeMap[port];
     if (!waitForConnect) {
+        print("Skip waiting to connect to node with pid=" + pid + ", port=" + port);
         return {
             pid: pid,
             port: port,
         };
     }
 
-    var conn = null;
-    assert.soon(function() {
-        try {
-            conn = new Mongo("127.0.0.1:" + port);
-            conn.pid = pid;
-            return true;
-        } catch (e) {
-            var res = checkProgram(pid);
-            if (!res.alive) {
-                print("Could not start mongo program at " + port +
-                      ", process ended with exit code: " + res.exitCode);
-                serverExitCodeMap[port] = res.exitCode;
-                return true;
-            }
-        }
-        return false;
-    }, "unable to connect to mongo program on port " + port, 600 * 1000);
-
-    return conn;
+    return MongoRunner.awaitConnection(pid, port);
 };
 
 /**
@@ -1342,8 +1418,13 @@ runMongoProgram = function() {
     args = appendSetParameterArgs(args);
     var progName = args[0];
 
-    // The bsondump tool doesn't support these auth related command line flags.
-    if (jsTestOptions().auth && progName != 'mongod' && progName != 'bsondump') {
+    const separator = _isWindows() ? '\\' : '/';
+    progName = progName.split(separator).pop();
+    const [baseProgramName, programVersion] = progName.split("-");
+
+    // Non-shell binaries (which are in fact instantiated via `runMongoProgram`) may not support
+    // these command line flags.
+    if (jsTestOptions().auth && baseProgramName != 'mongod') {
         args = args.slice(1);
         args.unshift(progName,
                      '-u',
@@ -1389,10 +1470,18 @@ startMongoProgramNoConnect = function() {
 };
 
 myPort = function() {
-    var m = db.getMongo();
-    if (m.host.match(/:/))
-        return m.host.match(/:(.*)/)[1];
-    else
-        return 27017;
+    const hosts = db.getMongo().host.split(',');
+
+    const ip6Numeric = hosts[0].match(/^\[[0-9A-Fa-f:]+\]:(\d+)$/);
+    if (ip6Numeric) {
+        return ip6Numeric[1];
+    }
+
+    const hasPort = hosts[0].match(/:(\d+)/);
+    if (hasPort) {
+        return hasPort[1];
+    }
+
+    return 27017;
 };
 }());
