@@ -66,6 +66,7 @@ void installShellUtilsLauncher(Scope& scope);
 class ProgramOutputMultiplexer {
 public:
     void appendLine(int port, ProcessId pid, const std::string& name, const std::string& line);
+    /** @return up to 100000 characters of the most recent log output. */
     std::string str() const;
     void clear();
 
@@ -107,9 +108,9 @@ private:
     std::map<ProcessId, HANDLE> _handles;
 
 public:
-    HANDLE getHandleForPid(ProcessId pid);
+    /** Will uassert with ErrorCodes::BadValue if the pid is unregistered. */
+    HANDLE getHandleForPid(ProcessId pid) const;
     void eraseHandleForPid(ProcessId pid);
-    std::size_t countHandleForPid(ProcessId pid);
     void insertHandleForPid(ProcessId pid, HANDLE handle);
 
 #endif
