@@ -95,9 +95,9 @@ function isRetryableError(error) {
         "HostNotFound",
         "NetworkTimeout",
         "SocketException",
-        "NotWritablePrimary",
-        "NotPrimaryNoSecondaryOk",
-        "NotPrimaryOrSecondary",
+        "NotMaster",
+        "NotMasterNoSlaveOk",
+        "NotMasterOrSecondary",
         "PrimarySteppedDown",
         "WriteConcernFailed",
         "WriteConcernLegacyOK",
@@ -109,11 +109,6 @@ function isRetryableError(error) {
     return retryableErrors.some(err => error.message.includes(err));
 }
 
-// Check if node is PSMDB or enterprise
-function isPSMDBOrEnterprise(buildInfo) {
-    return "psmdbVersion" in buildInfo || buildInfo.modules.includes("enterprise");
-}
-
 // Please consider using bsonWoCompare instead of this as much as possible.
 friendlyEqual = function(a, b) {
     if (a == b)
@@ -264,11 +259,20 @@ jsTestName = function() {
     return "__unknown_name__";
 };
 
-var _jsTestOptions = {enableTestCommands: true};  // Test commands should be enabled by default
+var _jsTestOptions = {};
 
 jsTestOptions = function() {
     if (TestData) {
         return Object.merge(_jsTestOptions, {
+            // Test commands should be enabled by default if no enableTestCommands were present in
+            // TestData
+            enableTestCommands:
+                TestData.hasOwnProperty('enableTestCommands') ? TestData.enableTestCommands : true,
+            // Testing diagnostics should be enabled by default if no testingDiagnosticsEnabled was
+            // present in TestData
+            testingDiagnosticsEnabled: TestData.hasOwnProperty('testingDiagnosticsEnabled')
+                ? TestData.testingDiagnosticsEnabled
+                : true,
             serviceExecutor: TestData.serviceExecutor,
             setParameters: TestData.setParameters,
             setParametersMongos: TestData.setParametersMongos,
@@ -303,6 +307,7 @@ jsTestOptions = function() {
             // Note: does not support the array version
             mongosBinVersion: TestData.mongosBinVersion || "",
             shardMixedBinVersions: TestData.shardMixedBinVersions || false,
+            mixedBinVersions: TestData.mixedBinVersions || false,
             networkMessageCompressors: TestData.networkMessageCompressors,
             replSetFeatureCompatibilityVersion: TestData.replSetFeatureCompatibilityVersion,
             skipRetryOnNetworkError: TestData.skipRetryOnNetworkError,
@@ -320,6 +325,8 @@ jsTestOptions = function() {
             skipValidationNamespaces: TestData.skipValidationNamespaces || [],
             skipCheckingUUIDsConsistentAcrossCluster:
                 TestData.skipCheckingUUIDsConsistentAcrossCluster || false,
+            skipCheckingIndexesConsistentAcrossCluster:
+                TestData.skipCheckingIndexesConsistentAcrossCluster || false,
             skipCheckingCatalogCacheConsistencyWithShardingCatalog:
                 TestData.skipCheckingCatalogCacheConsistencyWithShardingCatalog || false,
             skipAwaitingReplicationOnShardsBeforeCheckingUUIDs:
@@ -343,15 +350,31 @@ jsTestOptions = function() {
                 TestData.setSkipShardingPartsOfPrepareTransactionFailpoint || false,
             roleGraphInvalidationIsFatal: TestData.roleGraphInvalidationIsFatal || false,
             networkErrorAndTxnOverrideConfig: TestData.networkErrorAndTxnOverrideConfig || {},
+            // When useRandomBinVersionsWithinReplicaSet is true, randomly assign the binary
+            // versions of each node in the replica set to 'latest' or 'last-stable'.
+            // This flag is currently a placeholder and only sets the replica set to last-stable
+            // FCV.
+            useRandomBinVersionsWithinReplicaSet:
+                TestData.useRandomBinVersionsWithinReplicaSet || false,
+            // Set a specific random seed to be used when useRandomBinVersionsWithinReplicaSet is
+            // true.
+            seed: TestData.seed || undefined,
+            // Override the logging options for mongod and mongos so they always log to a file
+            // in dbpath; additionally, prevent the dbpath from being cleared after a node
+            // is shut down.
+            alwaysUseLogFiles: TestData.alwaysUseLogFiles || false,
+            skipCheckOrphans: TestData.skipCheckOrphans || false,
+            inEvergreen: TestData.inEvergreen || false,
+            // The ReplSetTest sets the slaveOk bit to true on its connections to the nodes in
+            // the replset, which will result in running all commands with a readPreference of
+            // 'secondaryPreferred'. However, some tests might not want to implicitly allow
+            // secondary reads. Such tests can set this option to turn off this implicit behavior.
+            shouldSkipSettingSlaveOk: TestData.shouldSkipSettingSlaveOk || false,
         });
     }
     return _jsTestOptions;
 };
 
-setJsTestOption = function(name, value) {
-    _jsTestOptions[name] = value;
-};
-
 jsTestLog = function(msg) {
     if (typeof msg === "object") {
         msg = tojson(msg);
@@ -365,7 +388,6 @@ jsTest = {};
 
 jsTest.name = jsTestName;
 jsTest.options = jsTestOptions;
-jsTest.setOption = setJsTestOption;
 jsTest.log = jsTestLog;
 jsTest.readOnlyUserRoles = ["read"];
 jsTest.basicUserRoles = ["dbOwner"];
@@ -389,7 +411,9 @@ jsTest.authenticate = function(conn) {
                 pwd: jsTestOptions().authPassword,
             });
             return conn.authenticated;
-        }, "Authenticating connection: " + conn, 5000, 1000);
+            // Dont' run the hang analyzer because we expect that this might fail in the normal
+            // course of events.
+        }, "Authenticating connection: " + conn, 5000, 1000, {runHangAnalyzer: false});
     } catch (e) {
         print("Caught exception while authenticating connection: " + tojson(e));
         conn.authenticated = false;
@@ -848,6 +872,8 @@ shellHelper.show = function(what) {
     what = args[0];
     args = args.splice(1);
 
+    var messageIndent = "        ";
+
     if (what == "profile") {
         if (db.system.profile.count() == 0) {
             print("db.system.profile is empty");
@@ -999,10 +1025,27 @@ shellHelper.show = function(what) {
                 if (res.log.length == 0) {
                     return "";
                 }
-                print("Server has startup warnings: ");
+                print("---");
+                print("The server generated these startup warnings when booting: ");
                 for (var i = 0; i < res.log.length; i++) {
-                    print(res.log[i]);
+                    var logOut;
+                    try {
+                        var parsedLog = JSON.parse(res.log[i]);
+                        var linePrefix = messageIndent + parsedLog.t["$date"] + ": ";
+                        logOut = linePrefix + parsedLog.msg + "\n";
+                        if (parsedLog.attr) {
+                            for (var attr in parsedLog.attr) {
+                                logOut += linePrefix + messageIndent + attr + ": " +
+                                    parsedLog.attr[attr] + "\n";
+                            }
+                        }
+                    } catch (err) {
+                        // err is intentionally unused here
+                        logOut = res.log[i];
+                    }
+                    print(logOut);
                 }
+                print("---");
                 return "";
             } else if (res.errmsg == "no such cmd: getLog") {
                 // Don't print if the command is not available
@@ -1075,15 +1118,19 @@ shellHelper.show = function(what) {
                     print("---");
                 } else if (freemonStatus.state === 'undecided') {
                     //print(
-                    //    "---\n" +
+                    //    "---\n" + messageIndent +
                     //    "Enable MongoDB's free cloud-based monitoring service, which will then receive and display\n" +
+                    //    messageIndent +
                     //    "metrics about your deployment (disk utilization, CPU, operation statistics, etc).\n" +
-                    //    "\n" +
+                    //    "\n" + messageIndent +
                     //    "The monitoring data will be available on a MongoDB website with a unique URL accessible to you\n" +
+                    //    messageIndent +
                     //    "and anyone you share the URL with. MongoDB may use this information to make product\n" +
+                    //    messageIndent +
                     //    "improvements and to suggest MongoDB products and deployment options to you.\n" +
-                    //    "\n" +
+                    //    "\n" + messageIndent +
                     //    "To enable free monitoring, run the following command: db.enableFreeMonitoring()\n" +
+                    //    messageIndent +
                     //    "To permanently disable this reminder, run the following command: db.disableFreeMonitoring()\n" +
                     //    "---\n");
                 }
@@ -1427,7 +1474,6 @@ rs.help = function() {
     print(
         "\trs.printSecondaryReplicationInfo()             check replica set members and replication lag");
     print("\tdb.isMaster()                              check who is primary");
-    print("\tdb.hello()                              check who is primary");
     print();
     print("\treconfiguration helpers disconnect from the database so the shell will display");
     print("\tan error, even if the command succeeds.");
@@ -1448,9 +1494,6 @@ rs.status = function() {
 rs.isMaster = function() {
     return db.isMaster();
 };
-rs.hello = function() {
-    return db.hello();
-};
 rs.initiate = function(c) {
     return db._adminCommand({replSetInitiate: c});
 };
@@ -1757,9 +1800,9 @@ help = shellHelper.help = function(x) {
         print("\t" +
               "use <db_name>                set current database");
         print("\t" +
-              "db.foo.find()                list objects in collection foo");
+              "db.mycoll.find()             list objects in collection mycoll");
         print("\t" +
-              "db.foo.find( { a : 1 } )     list objects in foo where a == 1");
+              "db.mycoll.find( { a : 1 } )  list objects in mycoll where a == 1");
         print(
             "\t" +
             "it                           result of the last line evaluated; use to further iterate");
