@@ -33,12 +33,8 @@ Copyright (C) 2019-present Percona and/or its affiliates. All rights reserved.
 
 #include <iostream>
 
-#define MONGO_UTIL_VERSION_CONSTANTS_H_WHITELISTED
-
 #include "mongo/db/encryption/encryption_options.h"
 #include "mongo/util/options_parser/startup_options.h"
-#include "mongo/util/version.h"
-#include "mongo/util/version_constants.h"
 
 namespace mongo {
 
@@ -48,34 +44,23 @@ void printPerconaDecryptHelp(std::ostream* out) {
     *out << "Usage:" << std::endl
          << "    perconadecrypt [options] --inputPath <src> --outputPath <dest> --encryptionKeyFile <key path>" << std::endl
          << "    perconadecrypt [options] --inputPath <src> --outputPath <dest> --vaultServerName <server name> [other Vault options]" << std::endl
-         << "    perconadecrypt [options] --inputPath <src> --outputPath <dest> --kmipServerName <server name> [other KMIP options]" << std::endl
          << "    perconadecrypt --help" << std::endl;
     *out << moe::startupOptions.helpString();
     *out << std::flush;
 }
 
-void printPerconaDecryptVersion(std::ostream* out) {
-    *out << "perconadecrypt version v" << version::kVersion << std::endl;
-    *out << "git version: " << version::kGitVersion << std::endl;
-    *out << std::flush;
-}
-
 bool handlePreValidationPerconaDecryptOptions(const moe::Environment& params) {
     if (params.count("help")) {
         printPerconaDecryptHelp(&std::cout);
         return false;
     }
-    if (params.count("version")) {
-        printPerconaDecryptVersion(&std::cout);
-        return false;
-    }
     return true;
 }
 
 Status storePerconaDecryptOptions(const moe::Environment& params,
                                const std::vector<std::string>& args) {
-    if (!params.count("encryptionKeyFile") && !params.count("vaultServerName") && !params.count("kmipServerName")) {
-        return {ErrorCodes::BadValue, "Missing required option: one of --encryptionKeyFile, --vaultServerName or --kmipServerName must be specified"};
+    if (!params.count("encryptionKeyFile") && !params.count("vaultServerName")) {
+        return {ErrorCodes::BadValue, "Missing required option: one of --encryptionKeyFile and --vaultServerName must be specified"};
     }
 
     if (params.count("encryptionKeyFile")) {
@@ -106,30 +91,6 @@ Status storePerconaDecryptOptions(const moe::Environment& params,
         encryptionGlobalParams.vaultDisableTLS = params["vaultDisableTLSForTesting"].as<bool>();
     }
 
-    if (params.count("kmipServerName")) {
-        encryptionGlobalParams.kmipServerName = params["kmipServerName"].as<std::string>();
-    }
-
-    if (params.count("kmipPort")) {
-        encryptionGlobalParams.kmipPort = params["kmipPort"].as<int>();
-    }
-
-    if (params.count("kmipServerCAFile")) {
-        encryptionGlobalParams.kmipServerCAFile = params["kmipServerCAFile"].as<std::string>();
-    }
-
-    if (params.count("kmipClientCertificateFile")) {
-        encryptionGlobalParams.kmipClientCertificateFile = params["kmipClientCertificateFile"].as<std::string>();
-    }
-
-    if (params.count("kmipClientCertificatePassword")) {
-        encryptionGlobalParams.kmipClientCertificatePassword =
-            params["kmipClientCertificatePassword"].as<std::string>();
-    }
-
-    if (params.count("kmipKeyIdentifier")) {
-        encryptionGlobalParams.kmipKeyIdentifier = params["kmipKeyIdentifier"].as<std::string>();
-    }
 
     if (params.count("encryptionCipherMode")) {
         encryptionGlobalParams.encryptionCipherMode = params["encryptionCipherMode"].as<std::string>();
