@@ -27,29 +27,35 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kNetwork
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kNetwork
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/transport/transport_layer_asio.h"
 
+#include <fstream>
+
 #include <asio.hpp>
 #include <asio/system_timer.hpp>
 #include <boost/algorithm/string.hpp>
+#include <boost/filesystem/operations.hpp>
 
 #include "mongo/config.h"
 
 #include "mongo/base/system_error.h"
 #include "mongo/db/server_options.h"
 #include "mongo/db/service_context.h"
+#include "mongo/db/stats/counters.h"
+#include "mongo/logv2/log.h"
 #include "mongo/transport/asio_utils.h"
 #include "mongo/transport/service_entry_point.h"
+#include "mongo/transport/transport_options_gen.h"
 #include "mongo/util/hierarchical_acquisition.h"
-#include "mongo/util/log.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/net/sockaddr.h"
 #include "mongo/util/net/ssl_manager.h"
 #include "mongo/util/net/ssl_options.h"
+#include "mongo/util/options_parser/startup_options.h"
 
 #ifdef MONGO_CONFIG_SSL
 #include "mongo/util/net/ssl.hpp"
@@ -66,6 +72,31 @@
 namespace mongo {
 namespace transport {
 
+namespace {
+#ifdef TCP_FASTOPEN
+using TCPFastOpen = asio::detail::socket_option::integer<IPPROTO_TCP, TCP_FASTOPEN>;
+#endif
+/**
+ * On systems with TCP_FASTOPEN_CONNECT (linux >= 4.11),
+ * we can get TFO "for free" by letting the kernel handle
+ * postponing connect() until the first send() call.
+ *
+ * https://github.com/torvalds/linux/commit/19f6d3f3c8422d65b5e3d2162e30ef07c6e21ea2
+ */
+#ifdef TCP_FASTOPEN_CONNECT
+using TCPFastOpenConnect = asio::detail::socket_option::boolean<IPPROTO_TCP, TCP_FASTOPEN_CONNECT>;
+#endif
+
+/**
+ * Set to `true` if any of the following set parameters were explicitly configured.
+ * - tcpFastOpenServer
+ * - tcpFastOpenClient
+ * - tcpFastOpenQueueSize
+ */
+bool tcpFastOpenIsConfigured = false;
+boost::optional<Status> maybeTcpFastOpenStatus;
+}  // namespace
+
 MONGO_FAIL_POINT_DEFINE(transportLayerASIOasyncConnectTimesOut);
 
 class ASIOReactorTimer final : public ReactorTimer {
@@ -82,7 +113,7 @@ public:
     void cancel(const BatonHandle& baton = nullptr) override {
         // If we have a baton try to cancel that.
         if (baton && baton->networking() && baton->networking()->cancelTimer(*this)) {
-            LOG(2) << "Canceled via baton, skipping asio cancel.";
+            LOGV2_DEBUG(23010, 2, "Canceled via baton, skipping asio cancel.");
             return;
         }
 
@@ -109,7 +140,11 @@ private:
             armTimer();
             return _timer->async_wait(UseFuture{}).tapError([timer = _timer](const Status& status) {
                 if (status != ErrorCodes::CallbackCanceled) {
-                    LOG(2) << "Timer received error: " << status;
+                    LOGV2_DEBUG(23011,
+                                2,
+                                "Timer received error: {error}",
+                                "Timer received error",
+                                "error"_attr = status);
                 }
             });
 
@@ -158,9 +193,10 @@ public:
     }
 
     void drain() override {
+        ThreadIdGuard threadIdGuard(this);
         _ioContext.restart();
         while (_ioContext.poll()) {
-            LOG(2) << "Draining remaining work in reactor.";
+            LOGV2_DEBUG(23012, 2, "Draining remaining work in reactor.");
         }
         _ioContext.stop();
     }
@@ -193,10 +229,12 @@ private:
     class ThreadIdGuard {
     public:
         ThreadIdGuard(TransportLayerASIO::ASIOReactor* reactor) {
+            invariant(!_reactorForThread);
             _reactorForThread = reactor;
         }
 
         ~ThreadIdGuard() {
+            invariant(_reactorForThread);
             _reactorForThread = nullptr;
         }
     };
@@ -421,7 +459,13 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
     GenericSocket sock(*_egressReactor);
     WrappedResolver resolver(*_egressReactor);
 
+    Date_t timeBefore = Date_t::now();
     auto swEndpoints = resolver.resolve(peer, _listenerOptions.enableIPv6);
+    Date_t timeAfter = Date_t::now();
+    if (timeAfter - timeBefore > kSlowOperationThreshold) {
+        networkCounter.incrementNumSlowDNSOperations();
+    }
+
     if (!swEndpoints.isOK()) {
         return swEndpoints.getStatus();
     }
@@ -451,7 +495,13 @@ StatusWith<SessionHandle> TransportLayerASIO::connect(HostAndPort peer,
         (sslMode == kGlobalSSLMode &&
          ((globalSSLMode == SSLParams::SSLMode_preferSSL) ||
           (globalSSLMode == SSLParams::SSLMode_requireSSL)))) {
+        Date_t timeBefore = Date_t::now();
         auto sslStatus = session->handshakeSSLForEgress(peer).getNoThrow();
+        Date_t timeAfter = Date_t::now();
+        if (timeAfter - timeBefore > kSlowOperationThreshold) {
+            networkCounter.incrementNumSlowSSLOperations();
+        }
+
         if (!sslStatus.isOK()) {
             return sslStatus;
         }
@@ -466,7 +516,21 @@ StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncCon
     Endpoint endpoint, const HostAndPort& peer, const Milliseconds& timeout) {
     GenericSocket sock(*_egressReactor);
     std::error_code ec;
-    sock.open(endpoint->protocol());
+
+    const auto protocol = endpoint->protocol();
+    sock.open(protocol);
+
+#ifdef TCP_FASTOPEN_CONNECT
+    const auto family = protocol.family();
+    if ((family == AF_INET) || (family == AF_INET6)) {
+        sock.set_option(TCPFastOpenConnect(gTCPFastOpenClient), ec);
+        if (tcpFastOpenIsConfigured) {
+            return errorCodeToStatus(ec);
+        }
+        ec = std::error_code();
+    }
+#endif
+
     sock.non_blocking(true);
 
     auto now = Date_t::now();
@@ -497,7 +561,7 @@ StatusWith<TransportLayerASIO::ASIOSessionHandle> TransportLayerASIO::_doSyncCon
 
     sock.non_blocking(false);
     try {
-        return std::make_shared<ASIOSession>(this, std::move(sock), false);
+        return std::make_shared<ASIOSession>(this, std::move(sock), false, *endpoint);
     } catch (const DBException& e) {
         return e.toStatus();
     }
@@ -511,12 +575,14 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
     struct AsyncConnectState {
         AsyncConnectState(HostAndPort peer,
                           asio::io_context& context,
-                          Promise<SessionHandle> promise_)
+                          Promise<SessionHandle> promise_,
+                          const ReactorHandle& reactor)
             : promise(std::move(promise_)),
               socket(context),
               timeoutTimer(context),
               resolver(context),
-              peer(std::move(peer)) {}
+              peer(std::move(peer)),
+              reactor(reactor) {}
 
         AtomicWord<bool> done{false};
         Promise<SessionHandle> promise;
@@ -528,12 +594,13 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
         WrappedEndpoint resolvedEndpoint;
         const HostAndPort peer;
         TransportLayerASIO::ASIOSessionHandle session;
+        ReactorHandle reactor;
     };
 
     auto reactorImpl = checked_cast<ASIOReactor*>(reactor.get());
     auto pf = makePromiseFuture<SessionHandle>();
-    auto connector =
-        std::make_shared<AsyncConnectState>(std::move(peer), *reactorImpl, std::move(pf.promise));
+    auto connector = std::make_shared<AsyncConnectState>(
+        std::move(peer), *reactorImpl, std::move(pf.promise), reactor);
     Future<SessionHandle> mergedFuture = std::move(pf.future);
 
     if (connector->peer.host().empty()) {
@@ -569,9 +636,13 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
         .then([connector, timeBefore](WrappedResolver::EndpointVector results) {
             try {
                 Date_t timeAfter = Date_t::now();
-                if (timeAfter - timeBefore > Seconds(1)) {
-                    warning() << "DNS resolution while connecting to " << connector->peer
-                              << " took " << timeAfter - timeBefore;
+                if (timeAfter - timeBefore > kSlowOperationThreshold) {
+                    LOGV2_WARNING(23019,
+                                  "DNS resolution while connecting to {peer} took {duration}",
+                                  "DNS resolution while connecting to peer was slow",
+                                  "peer"_attr = connector->peer,
+                                  "duration"_attr = timeAfter - timeBefore);
+                    networkCounter.incrementNumSlowDNSOperations();
                 }
 
                 stdx::lock_guard<Latch> lk(connector->mutex);
@@ -583,12 +654,19 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
                 return futurize(ex.code());
             }
 
+#ifdef TCP_FASTOPEN_CONNECT
+            std::error_code ec;
+            connector->socket.set_option(TCPFastOpenConnect(gTCPFastOpenClient), ec);
+            if (tcpFastOpenIsConfigured) {
+                return futurize(ec);
+            }
+#endif
             return connector->socket.async_connect(*connector->resolvedEndpoint, UseFuture{});
         })
         .then([this, connector, sslMode]() -> Future<void> {
             stdx::unique_lock<Latch> lk(connector->mutex);
-            connector->session =
-                std::make_shared<ASIOSession>(this, std::move(connector->socket), false);
+            connector->session = std::make_shared<ASIOSession>(
+                this, std::move(connector->socket), false, *connector->resolvedEndpoint);
             connector->session->ensureAsync();
 
 #ifndef MONGO_CONFIG_SSL
@@ -601,9 +679,17 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
                 (sslMode == kGlobalSSLMode &&
                  ((globalSSLMode == SSLParams::SSLMode_preferSSL) ||
                   (globalSSLMode == SSLParams::SSLMode_requireSSL)))) {
+                Date_t timeBefore = Date_t::now();
                 return connector->session
-                    ->handshakeSSLForEgressWithLock(std::move(lk), connector->peer)
-                    .then([connector] { return Status::OK(); });
+                    ->handshakeSSLForEgressWithLock(
+                        std::move(lk), connector->peer, connector->reactor)
+                    .then([connector, timeBefore] {
+                        Date_t timeAfter = Date_t::now();
+                        if (timeAfter - timeBefore > kSlowOperationThreshold) {
+                            networkCounter.incrementNumSlowSSLOperations();
+                        }
+                        return Status::OK();
+                    });
             }
 #endif
             return Status::OK();
@@ -612,8 +698,8 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
             return makeConnectError(status, connector->peer, connector->resolvedEndpoint);
         })
         .getAsync([connector](Status connectResult) {
-            if (MONGO_FAIL_POINT(transportLayerASIOasyncConnectTimesOut)) {
-                log() << "asyncConnectTimesOut fail point is active. simulating timeout.";
+            if (MONGO_unlikely(transportLayerASIOasyncConnectTimesOut.shouldFail())) {
+                LOGV2(23013, "asyncConnectTimesOut fail point is active. simulating timeout.");
                 return;
             }
 
@@ -632,6 +718,145 @@ Future<SessionHandle> TransportLayerASIO::asyncConnect(HostAndPort peer,
     return mergedFuture;
 }
 
+namespace {
+#if defined(TCP_FASTOPEN) || defined(TCP_FASTOPEN_CONNECT)
+/**
+ * Attempt to set an option on a dummy SOCK_STREAM/AF_INET socket
+ * and report success/failure.
+ */
+bool trySetSockOpt(int level, int opt, int val) {
+    auto sock = ::socket(AF_INET, SOCK_STREAM, 0);
+
+#ifdef _WIN32
+    char* pval = reinterpret_cast<char*>(&val);
+#else
+    void* pval = &val;
+#endif
+
+    const auto ret = ::setsockopt(sock, level, opt, pval, sizeof(val));
+
+#ifdef _WIN32
+    closesocket(sock);
+#else
+    close(sock);
+#endif
+
+    return ret == 0;
+}
+#endif
+
+Status validateFastOpen() noexcept {
+    namespace moe = optionenvironment;
+    if (moe::startupOptionsParsed.count("setParameter")) {
+        const auto params =
+            moe::startupOptionsParsed["setParameter"].as<std::map<std::string, std::string>>();
+        tcpFastOpenIsConfigured = (params.find("tcpFastOpenServer") != params.end()) ||
+            (params.find("tcpFastOpenClient") != params.end()) ||
+            (params.find("tcpFastOpenQueueSize") != params.end());
+    }
+
+#ifndef TCP_FASTOPEN
+    if (tcpFastOpenIsConfigured && gTCPFastOpenServer) {
+        return {ErrorCodes::BadValue,
+                "TCP FastOpen server support unavailable in this build of MongoDB"};
+    }
+#else
+    networkCounter.setTFOServerSupport(trySetSockOpt(IPPROTO_TCP, TCP_FASTOPEN, 1));
+#endif
+
+#ifndef TCP_FASTOPEN_CONNECT
+    if (tcpFastOpenIsConfigured && gTCPFastOpenClient) {
+        return {ErrorCodes::BadValue,
+                "TCP FastOpen client support unavailable in this build of MongoDB"};
+    }
+#else
+    networkCounter.setTFOClientSupport(trySetSockOpt(IPPROTO_TCP, TCP_FASTOPEN_CONNECT, 1));
+#endif
+
+#if defined(TCP_FASTOPEN) && defined(__linux)
+    if (!gTCPFastOpenServer && !gTCPFastOpenClient) {
+        return Status::OK();
+    }
+
+    std::string procfile("/proc/sys/net/ipv4/tcp_fastopen");
+    boost::system::error_code ec;
+    if (!boost::filesystem::exists(procfile, ec)) {
+        return {ErrorCodes::BadValue,
+                str::stream() << "Unable to locate " << procfile << ": " << errorCodeToStatus(ec)};
+    }
+
+    std::fstream f(procfile, std::ifstream::in);
+    if (!f.is_open()) {
+        return {ErrorCodes::BadValue, str::stream() << "Unable to read " << procfile};
+    }
+
+    std::int64_t val;
+    f >> val;
+    networkCounter.setTFOKernelSetting(val);
+
+    constexpr std::int64_t kTFOClientBit = (1 << 0);
+    constexpr std::int64_t kTFOServerBit = (1 << 1);
+
+    // Future proof this setting by allowing extra bits to stay set in help output.
+    std::int64_t wantval = val;
+    if (gTCPFastOpenClient) {
+        wantval |= kTFOClientBit;
+    }
+    if (gTCPFastOpenServer) {
+        wantval |= kTFOServerBit;
+    }
+
+    if (val != wantval) {
+        return {ErrorCodes::BadValue,
+                str::stream() << "TCP FastOpen disabled in kernel. "
+                              << "Set " << procfile << " to " << std::to_string(wantval)};
+    }
+#endif
+
+    return Status::OK();
+}
+
+Status validateFastOpenOnce() noexcept {
+    if (!maybeTcpFastOpenStatus) {
+        // If we haven't validated the TCP FastOpen situation yet, do so.
+        maybeTcpFastOpenStatus = validateFastOpen();
+
+        if (!maybeTcpFastOpenStatus->isOK()) {
+            // This has to be a char[] because that's what logv2 understands
+            static constexpr char kPrefixString[] = "Unable to enable TCP FastOpen";
+
+            if (tcpFastOpenIsConfigured) {
+                // If the user asked for TCP FastOpen and we couldn't provide it, log a startup
+                // warning in addition to the hard failure.
+                LOGV2_WARNING_OPTIONS(23014,
+                                      {logv2::LogTag::kStartupWarnings},
+                                      kPrefixString,
+                                      "reason"_attr = maybeTcpFastOpenStatus->reason());
+            } else {
+                LOGV2(4648601,
+                      "Implicit TCP FastOpen unavailable. "
+                      "If TCP FastOpen is required, set tcpFastOpenServer, tcpFastOpenClient, "
+                      "and tcpFastOpenQueueSize.");
+            }
+
+            maybeTcpFastOpenStatus->addContext(kPrefixString);
+        } else {
+            if (!tcpFastOpenIsConfigured) {
+                LOGV2(4648602, "Implicit TCP FastOpen in use.");
+            }
+        }
+    }
+
+    if (!tcpFastOpenIsConfigured) {
+        // If nobody asked for TCP FastOpen, no one will miss it.
+        return Status::OK();
+    }
+
+    // TCP FastOpen was requested. It's either there or it's not.
+    return *maybeTcpFastOpenStatus;
+}
+}  // namespace
+
 Status TransportLayerASIO::setup() {
     std::vector<std::string> listenAddrs;
     if (_listenerOptions.ipList.empty() && _listenerOptions.isIngress()) {
@@ -649,6 +874,10 @@ Status TransportLayerASIO::setup() {
     }
 #endif
 
+    if (auto foStatus = validateFastOpenOnce(); !foStatus.isOK()) {
+        return foStatus;
+    }
+
     if (!(_listenerOptions.isIngress()) && !listenAddrs.empty()) {
         return {ErrorCodes::BadValue,
                 "Cannot bind to listening sockets with ingress networking is disabled"};
@@ -661,14 +890,17 @@ Status TransportLayerASIO::setup() {
     std::set<WrappedEndpoint> endpoints;
     for (auto& ip : listenAddrs) {
         if (ip.empty()) {
-            warning() << "Skipping empty bind address";
+            LOGV2_WARNING(23020, "Skipping empty bind address");
             continue;
         }
 
         auto swAddrs =
             resolver.resolve(HostAndPort(ip, _listenerPort), _listenerOptions.enableIPv6);
         if (!swAddrs.isOK()) {
-            warning() << "Found no addresses for " << swAddrs.getStatus();
+            LOGV2_WARNING(23021,
+                          "Found no addresses for {peer}",
+                          "Found no addresses for peer",
+                          "peer"_attr = swAddrs.getStatus());
             continue;
         }
         auto& addrs = swAddrs.getValue();
@@ -679,25 +911,38 @@ Status TransportLayerASIO::setup() {
 #ifndef _WIN32
         if (addr.family() == AF_UNIX) {
             if (::unlink(addr.toString().c_str()) == -1 && errno != ENOENT) {
-                error() << "Failed to unlink socket file " << addr.toString().c_str() << " "
-                        << errnoWithDescription(errno);
+                LOGV2_ERROR(23024,
+                            "Failed to unlink socket file {path} {error}",
+                            "Failed to unlink socket file",
+                            "path"_attr = addr.toString().c_str(),
+                            "error"_attr = errnoWithDescription(errno));
                 fassertFailedNoTrace(40486);
             }
         }
 #endif
         if (addr.family() == AF_INET6 && !_listenerOptions.enableIPv6) {
-            error() << "Specified ipv6 bind address, but ipv6 is disabled";
+            LOGV2_ERROR(23025, "Specified ipv6 bind address, but ipv6 is disabled");
             fassertFailedNoTrace(40488);
         }
 
         GenericAcceptor acceptor(*_acceptorReactor);
         acceptor.open(addr->protocol());
         acceptor.set_option(GenericAcceptor::reuse_address(true));
+
+        std::error_code ec;
+#ifdef TCP_FASTOPEN
+        if (gTCPFastOpenServer && ((addr.family() == AF_INET) || (addr.family() == AF_INET6))) {
+            acceptor.set_option(TCPFastOpen(gTCPFastOpenQueueSize), ec);
+            if (tcpFastOpenIsConfigured) {
+                return errorCodeToStatus(ec);
+            }
+            ec = std::error_code();
+        }
+#endif
         if (addr.family() == AF_INET6) {
             acceptor.set_option(asio::ip::v6_only(true));
         }
 
-        std::error_code ec;
         acceptor.non_blocking(true, ec);
         if (ec) {
             return errorCodeToStatus(ec);
@@ -711,8 +956,11 @@ Status TransportLayerASIO::setup() {
 #ifndef _WIN32
         if (addr.family() == AF_UNIX) {
             if (::chmod(addr.toString().c_str(), serverGlobalParams.unixSocketPermissions) == -1) {
-                error() << "Failed to chmod socket file " << addr.toString().c_str() << " "
-                        << errnoWithDescription(errno);
+                LOGV2_ERROR(23026,
+                            "Failed to chmod socket file {path} {error}",
+                            "Failed to chmod socket file",
+                            "path"_attr = addr.toString().c_str(),
+                            "error"_attr = errnoWithDescription(errno));
                 fassertFailedNoTrace(40487);
             }
         }
@@ -742,7 +990,7 @@ Status TransportLayerASIO::setup() {
     const auto& sslParams = getSSLGlobalParams();
 
     if (_sslMode() != SSLParams::SSLMode_disabled && _listenerOptions.isIngress()) {
-        _ingressSSLContext = stdx::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+        _ingressSSLContext = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
 
         Status status =
             getSSLManager()->initSSLContext(_ingressSSLContext->native_handle(),
@@ -751,10 +999,17 @@ Status TransportLayerASIO::setup() {
         if (!status.isOK()) {
             return status;
         }
+
+        auto resp = getSSLManager()->stapleOCSPResponse(_ingressSSLContext->native_handle());
+        if (!resp.isOK()) {
+            return Status(ErrorCodes::InvalidSSLConfiguration,
+                          str::stream()
+                              << "Can not staple OCSP Response. Reason: " << resp.reason());
+        }
     }
 
     if (_listenerOptions.isEgress() && getSSLManager()) {
-        _egressSSLContext = stdx::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
+        _egressSSLContext = std::make_unique<asio::ssl::context>(asio::ssl::context::sslv23);
         Status status =
             getSSLManager()->initSSLContext(_egressSSLContext->native_handle(),
                                             sslParams,
@@ -780,22 +1035,24 @@ void TransportLayerASIO::_runListener() noexcept {
         asio::error_code ec;
         acceptor.second.listen(serverGlobalParams.listenBacklog, ec);
         if (ec) {
-            severe() << "Error listening for new connections on " << acceptor.first << ": "
-                     << ec.message();
-            fassertFailed(31339);
+            LOGV2_FATAL(31339,
+                        "Error listening for new connections on {listenAddress}: {error}",
+                        "Error listening for new connections on listen address",
+                        "listenAddrs"_attr = acceptor.first,
+                        "error"_attr = ec.message());
         }
 
         _acceptConnection(acceptor.second);
-        log() << "Listening on " << acceptor.first.getAddr();
+        LOGV2(23015, "Listening on", "address"_attr = acceptor.first.getAddr());
     }
 
-    const char* ssl = "";
+    const char* ssl = "off";
 #ifdef MONGO_CONFIG_SSL
     if (_sslMode() != SSLParams::SSLMode_disabled) {
-        ssl = " ssl";
+        ssl = "on";
     }
 #endif
-    log() << "waiting for connections on port " << _listenerPort << ssl;
+    LOGV2(23016, "Waiting for connections", "port"_attr = _listenerPort, "ssl"_attr = ssl);
 
     _listener.active = true;
     _listener.cv.notify_all();
@@ -817,10 +1074,15 @@ void TransportLayerASIO::_runListener() noexcept {
         auto& addr = acceptor.first;
         if (addr.getType() == AF_UNIX && !addr.isAnonymousUNIXSocket()) {
             auto path = addr.getAddr();
-            log() << "removing socket file: " << path;
+            LOGV2(
+                23017, "removing socket file: {path}", "removing socket file", "path"_attr = path);
             if (::unlink(path.c_str()) != 0) {
                 const auto ewd = errnoWithDescription();
-                warning() << "Unable to remove UNIX socket " << path << ": " << ewd;
+                LOGV2_WARNING(23022,
+                              "Unable to remove UNIX socket {path}: {error}",
+                              "Unable to remove UNIX socket",
+                              "path"_attr = path,
+                              "error"_attr = ewd);
             }
         }
     }
@@ -893,18 +1155,33 @@ void TransportLayerASIO::_acceptConnection(GenericAcceptor& acceptor) {
         }
 
         if (ec) {
-            log() << "Error accepting new connection on "
-                  << endpointToHostAndPort(acceptor.local_endpoint()) << ": " << ec.message();
+            LOGV2(23018,
+                  "Error accepting new connection on {localEndpoint}: {error}",
+                  "Error accepting new connection on local endpoint",
+                  "localEndpoint"_attr = endpointToHostAndPort(acceptor.local_endpoint()),
+                  "error"_attr = ec.message());
             _acceptConnection(acceptor);
             return;
         }
 
+#ifdef TCPI_OPT_SYN_DATA
+        struct tcp_info info;
+        socklen_t info_len = sizeof(info);
+        if (!getsockopt(peerSocket.native_handle(), IPPROTO_TCP, TCP_INFO, &info, &info_len) &&
+            (info.tcpi_options & TCPI_OPT_SYN_DATA)) {
+            networkCounter.acceptedTFOIngress();
+        }
+#endif
+
         try {
             std::shared_ptr<ASIOSession> session(
                 new ASIOSession(this, std::move(peerSocket), true));
             _sep->startSession(std::move(session));
         } catch (const DBException& e) {
-            warning() << "Error accepting new connection " << e;
+            LOGV2_WARNING(23023,
+                          "Error accepting new connection: {error}",
+                          "Error accepting new connection",
+                          "error"_attr = e);
         }
 
         _acceptConnection(acceptor);
